function doscar_read, file_in, first_atom, last_atom, TotalSOC=TSOC, InvertSpin=InvSpin
;+
;;******* **********
;**
;** The call is e.g. w1=doscar_read('filename', atom)
;
;   DOSCAR file from VASP will be read
;   total DOS block returned if first_atom and last_atom <0
;   Newer version of read_doscar, reads Fermi energy
;   Revisited - Aug 2013 - in the case of spin-orbit coupling (SOC)
;   See VASP manual for possible file formats
;   SOC calculations look like non spin polarised calcs since first block does not have spin-up and down columns
;   If keyword /TotalSOC set in call only the total (1st of 4 columns) will be output
;   first_atom and last_atom used used to define a block of atoms which will be summed
;   If keyword /InvertSpin set DOWN spins will be multiplied by -1
;    
; initialise arrays and variables
;-

atom_info = ' ' & separator = ' ' & line_info = ' '
junk1 = ' ' & junk4 = strarr(4)     ; character variables for skipping lines
n_atoms_selected=last_atom-first_atom+1

; start reading file to get variables
openr,f_in,file_in,/get_lun
readf,f_in,atom_info           ; num atoms on first line, first entry
atoms = strsplit(atom_info,/extract) & n_atoms=atoms[0]
print, 'number of atoms is  ', n_atoms
readf,f_in,junk4               ; skipping
readf,f_in,separator           ; num lines third entry
lines = strsplit(separator,/extract) & n_lines=lines[2] & Efermi=lines[3]
print, 'numer of energy points is  ', n_lines
print, 'Fermi energy is  ', Efermi

; start to read first block, total DOS
readf,f_in,line_info
; find num of columns, depends on spin pol.
lines  = strsplit(line_info,/extract)
n_cols_t = n_elements(lines) & print, 'number of columns in first block is  ', n_cols_t
ispin=0
if (n_cols_t gt 3) then begin
  print, 'spin_polarised file'
  ispin=1
endif

; create array for total dos
dos_tot=fltarr(n_cols_t, n_lines-1)
; read remaining n-1 lines
readf,f_in,dos_tot

; separate energy from dos
dos_tot_x=dos_tot[0,*]
; dump cumulants
if (ispin eq 1) then dos_tot_w=dos_tot[1:2,*] else dos_tot_w=dos_tot[1,*]
; invert spin down
if keyword_set(InvSpin) then dos_tot_w[1,*]=-1.0*dos_tot_w[1,*]

; jump OUT if only reading total DOS
if (first_atom lt 0) then begin
  datp={x:dos_tot_x-Efermi,x_tit: 'energy[eV]'} & give_datp, datp
  return,transpose(dos_tot_w)
endif

; read first atom block
readf,f_in,separator
readf,f_in,line_info
lines  = strsplit(line_info,/extract)
n_cols = n_elements(lines) & print, 'number of columns in first ATOM block is  ', n_cols
; These lines removed 'Aug 2013'
; ispin can be used instead of iproj - I think
; iproj=0
; if (n_cols eq 19) then begin
;   print, 'spd projections'
;   iproj=1
; endif


; create array for 1 atom dos: contains energy axis and dos columns
dos_atom=fltarr(n_cols, n_lines-1)
; and all atoms: without energy axis
dos_all_atoms=fltarr(n_atoms, n_cols-1, n_lines-1)

; read remaining n-1 lines and copy to all_atoms
readf,f_in,dos_atom & dos_all_atoms[0,*,*]=dos_atom[1:n_cols-1,*]

; loop thro remaining n_atoms-1
; skip 2 lines each time: one is an info line, the other because only reading nlines-1
for i=1,n_atoms-1 do begin
  readf,f_in,separator
  readf,f_in,line_info
  readf,f_in,dos_atom & dos_all_atoms[i,*,*]=dos_atom[1:n_cols-1,*]
end

; invert DOS in spin polarised case
if keyword_set(InvSpin) then for ip=1,n_cols-1,2 do dos_all_atoms[*,ip,*]= -1.0*dos_all_atoms[*,ip,*]

; close file
free_lun, f_in
print, 'file read'

; prepare output
datp={x:dos_tot_x-Efermi,x_tit: 'energy[eV]'}
give_datp, datp
if keyword_set(TSOC) then begin
  dos_w=fltarr(n_atoms_selected,(n_cols-1)/4,n_lines-1)
  for i=0,n_cols-2,4 do begin
;   print, i, i/4
    dos_w[*,i/4,*]=dos_all_atoms[first_atom:last_atom,i,*]
  endfor
; print text file of integrated DOS
; columns s,p,p,p,d,d,d,d,d
; rows atoms/sites
file_out=file_in+'_Integrated_DOS_'+strtrim(first_atom,2)+'-'+strtrim(last_atom,2)+'.dat'
openw,f_out,file_out,/get_lun
printf,f_out,'sites: ',first_atom,' to ', last_atom
printf,f_out, '      s        p        p        p        d        d        d        d        d'
; take total up to Fermi level
idx=where(dos_tot_x-Efermi lt 0., EfermiChannels) & print, 'channels below Fermi level: ',EfermiChannels
; normalise with EnergyStep
EnergyStep=(dos_tot_x[EfermiChannels]-dos_tot_x[0])/EfermiChannels & print,'EStep ',EnergyStep
dos2d_w = total(dos_w[*,*,0:EfermiChannels],3)*EnergyStep 
printf,f_out, transpose(dos2d_w), format='(9F9.2)'
free_lun, f_out
endif else begin
  dos_w=dos_all_atoms[first_atom:last_atom,*,*]
endelse

;print, size(dos_w)
dos_out=total(dos_w,1) ;*1.0/n_atoms_selected
return,transpose(dos_out)

if (atom gt n_atoms) then begin
  print, 'atom number too big'
endif

end
