;+
;*******************************************************************************
;
;     FUNCTION normalise, w_in, /raw, monitor=monitor, time=time,  verbose
;
;For BRISP data.
;
;
; ---------------------
; Normalises raw data to monitor or counting time,
;
;KEYWORDS:
;
; /raw          : no normalisation (error bars calculated)
; /time         : normalise data to counting time (will not work for summed data)
;
;
;OPTIONS
; monitor        : normalise data to 10^9 monitor counts (DEFAULT)
;    possibility to choose among the four monitors
;    monitor='M1'....'M4': DEFAULT: all monitors
; hours         : normalise data to total counting time in hours (will work for summed data)
;
;INPUT: 2D regrouped data (i.e. after brisp_rad)
;  it will work also for 3D data, but memory quickly becomes insufficient because of 3D errors
;
;
; COMMAND SYNTAX
;  w2=brisp_norm(w1[,monitor][,hours][,/raw][,/time][,verbose])
;  w2=brisp_norm(w1) normalise to the four monitors
;
;  FF, 03-2009 (modification in May 2010)
; -------------------------------------------------------------------------------
;*******************************************************************************
;-

FUNCTION brisp_norm, w_in, RAW=raw, MONITOR=monitor, HOURS=hours, TIME=time, verbose=verbose
;+
;******* **********
;** User callable
;-

  forward_function str_fit
  
  COMMON c_lamp_access, inst
  
  IF keyword_set(verbose) THEN PRINT,'NORMALISE: WARNING: Development version 2009-13-03.'
  IF keyword_set(verbose) THEN PRINT,'NORMALISE: Instrument is: ',strupcase(inst)
  
  take_datp, datp
  ; give_w,0 dangerous: with w1=brisp_norm(w1)--> error!!!
  
  ; -------------------------------------------------------------------------------
  ;Set up start parameters
  
  mon_8  = datp.n;     4 monitors on the sample + 4 on the BC
  mon_4=mon_8[*,0:3];    Only the four monitors on the sample are taken into account
  smon=size(mon_4) & mon_ch=smon[1] & mon_num=smon[2]
  
  para  = datp.p
  x_in = datp.x
  y_in = datp.y
  z_in = datp.z
  e_in = datp.e
  
  sw=size(w_in) &  sw0=sw[0];
  
  normf=0;
  
  if keyword_set(raw)+keyword_set(hours)+n_elements(time)+n_elements(monitor) GE 2 then begin
    print,'Choose only one way to normalise!'
    return,w_in
  endif
  
  IF keyword_set(raw) THEN normf=1.0 & inorm=0;--------------------->  case RAW
  ;
  IF keyword_set(time)  THEN normf=para[0]/60 & inorm=1;  -------->  case TIME
  IF n_elements(hours)  THEN normf=hours/1. & inorm=1;  -------->  case Total TIME for summed data
  ;
  ;
  ; -----------------------------  case MONITOR   ------------------------
  
  ; ------------------------------
  keys=keyword_set(time)+keyword_set(raw)+ n_elements(hours)
  IF (n_elements(monitor) GE 0) AND (keys eq 0) THEN BEGIN
  
    ;  Treat the case where the data are already normalised --> set wout to win
    if mean(mon_4) eq 1 then begin
      print,'NORMALISE: WARNING:  Data already normalised!'
      return,w_in
    endif
    
    inorm=2
    normf=mon_4*0.;
    ;
    ; ------------------------------------------------
    ;;    gaussian FIT of the 4 m onitors intensities
    ;
    mpar = str_fit(mon_4,Bkgd=0,Xin=findgen(n_elements(mon_ch)),/gauss) ;Gaussian with a flat background
    mon_height=mpar[*,1]
    mon_center=mpar[*,2]
    mon_width=mpar[*,3]
    mon_bckg=mpar[*,4]
    mon_dataint=mpar[*,7];  integral of data-bckg
    mon_fitint=mpar[*,8];  integral of fit-bckg
    
    ; ------------------------------------------------
    ;
    normf=mon_fitint/1.e9;
    
    IF (n_elements(monitor) NE 0) THEN BEGIN
      IF monitor EQ 'M1' THEN normf=normf[0]
      IF monitor EQ 'M2' THEN normf=normf[1]
      IF monitor EQ 'M3' THEN normf=normf[2]
      IF monitor EQ 'M4' THEN normf=normf[3]
    ENDIF ELSE normf=total(normf)
    
    w_out=w_in/normf & e_out=e_in/normf
    ; -------------------------------------------------------------------------------
    ;   If  normf is 0 but data not previouosly normalised
    ; -------------------------------------------------------------------------------
    
    if normf le 0.0 then begin
      print,' NORMALISE: ERROR:  Monitor is <= 0!, Abort...'
      return, w_in
    endif
    
    datp.n=1;
    
    IF  keyword_set(verbose) THEN BEGIN
      PRINT, 'NORMALISE: Monitor 1 2 3 4 '
      PRINT, 'NORMALISE: Monitor elastic peak position: ',strtrim( string(mon_center,format='(F6.2)'), 2 )
      PRINT, 'NORMALISE: Monitor FWHM                 : ',strtrim(string(mon_width,format='(F6.3)'),2),' channels'
      PRINT, 'NORMALISE: Monitor background           : ',strtrim( string(mon_bckg,format='(F12.1)'),2)
      PRINT, 'NORMALISE: Monitor integral             : ',strtrim( string(mon_fitint,format='(F12.1)'),2)
      PRINT, 'NORMALISE: Total Monitor integral       : ',strtrim( string(normf * 1.0e6,format='(F12.1)'),2)
      
    ENDIF
    
  ENDIF
  
  
  s = STRTRIM(STRING(normf),2) & i = STRPOS(s,'.') & normf = STRMID(s,0,i[0]+3)
  
  CASE inorm OF
    0: BEGIN
    
      nors = '/raw'
      nstring = '(not normalised.) '
      datp.z_tit= 'Counts/pixel'
    END
    1: BEGIN
      nors = '/m'
      nstring = '(normalised to 1 hour.) '
      datp.z_tit= 'Counts/hour'
    END
    2: BEGIN
      nors = '/t'
      nstring='(normalised to 10^9 Monitor counts.) '
      datp.z_tit= 'Counts/10^9 Monitor counts'
    END
  ENDCASE
  normalisation = ' no('+nors +')'
  outstring=nstring
  datp.p=para
  ;
  s = datp.other_tit & i =  strpos(/reverse_search,s,' ') & n = STRLEN(s) & numor = STRMID(s,i+1,n-1)
  IF (STRPOS(numor,'>') EQ -1) THEN $
    numor = STRTRIM(STRING(FIX(para[10])),2)
  datp.other_tit = inst + ' #' + numor +' '+ nstring
  
  mod_datp,  datp, "e",e_out
  give_datp, datp
  
  
  ; -------------------------------------------------------------------------------
  ;   Return parameters and exit
  ; -------------------------------------------------------------------------------
  RETURN, w_out
END
