;******* ************
function brisp_rad, W_in , XY0=xy0, INCR=incr, MASK=masq, Pix=xypsiz, $
    Nrun_hr=N_run_hr,eff_dat=file_eff,bck_ch=bck_ch, $
    bck_slope=bck_slope, $
    el_ch=fileout,th_min=th_min,th_max=th_max
;+
;******* *********
;** User callable
;**      radial integration with solid angle and efficiency correction (FF, Apr 2007)
;**
;**      calculate elastic channel position as a function of theta by fitting
;**      a quadratic law: ch=A0+A1*theta+A2*theta^2             (FF, Apr 2010)
;**
;**     Call:   w3=brisp_rad(W2,XY0=C,INCR=6,MASK=w40,bck_ch=[1500,1600],el_ch='elas.txt',th_min=1.,th_max=12.)
;**     ****
;** Input:  M      =2D or 3D data matrix.
;** *****   xy0    =[x0,y0] center determined by "brisp_center.pro"
;**     incr   =Ray increment (width) in pixels units   (default  1.0)
;**     masq   =same as M(n,m), contains 1's for masked values otherwise 0's(default)
;**     xypsiz =size of the pixel in Cm. Default is [.428,1.27]
;**     dst    =Distance sample-detector in Cm.
;**
;**** OPTIONS
;**     N_run_hr  = [Num of run over which M is averaged, Number of hour/run]: due to absence of
;**                  moinitors, is used to propagate errors after integration
;**     file_eff  = file with efficiency values (size =3 X 160, eff in the 2nd column)
;**     bck_ch    = subtraction of a constant bckg evaluated as average between the
;**                 channels given in bck_ch=[ch_min, ch_max]
;**    \bck_slope = alternative to bck_ch: if given, it subtracts a linear bckg a+b*ch
;**                 where a e b are evaluated by the mean value at the extreme of channels
;**                 ONLY FOR 512 CHANNELS
;*****
;*****  el_ch =      namefile where the coefficient of the fit ch=A0+A1*theta+A2*theta^2 are stored
;*****  th_min, th_max = allows to limit the theta fitting range :
;*****                   lowest and/or highest angles elastic position can be poorly defined
;**
;** Output: Wout  =result of Radial integration.
;** ******  X     =diagram abscissa: angles (deg).
;**     error     =diagram errors
;-
    
    
  Wout=0
  M=W_in
  ;*****************************CHECK FOR PARAMETERS******************************
  s=size(M)   & s12= s[1]* s[2]
  
  IF (s[0] lt 2) or (s[1] lt 2) or (s[2] lt 2) THEN RETURN, Wout
  TAKE_DATP, datp                                    ;workspace variables
  Z_in=datp.z
  if n_elements(xypsiz) ne 2 then xypsiz=[.428*2,1.27] ;[vertical,horizontal] default.
  if n_elements(N_run_hr) ne 2 then N_run_hr=[1,3.]              ; number of averaged runs
  
  dst=datp.p[27]    ; Sample-detector distance in m
  chw=datp.p[18]    ; channel width (microsec)
  
  
  ;***************************************************************************************
  ;***************************** CORRECTIONS *********************************************
  ;***************************************************************************************
  ;
  ;***** Solid angle *****
  dst=dst*100.; ;    Sample-detector distance in cm
  
  r_pix=brisp_dist(M,CENTER=xy0); pixel-det_centre distance in cm
  L2_pix=sqrt((r_pix)^2+(dst)^2); pixel-sample distance in cm
  teta=ATAN(r_pix/dst)       ; angle seen from the pixel
  
  dOmega=cos(teta)^3;
  
  IF (s[0] EQ 2) THEN M=M/dOmega
  
  IF (s[0] EQ 3) THEN BEGIN
    FOR ich=0,s[3]-1 DO M[*,*,ich]=M[*,*,ich]/dOmega
  ENDIF
  
  ;***** Efficiency *****
  
  IF (N_ELEMENTS(file_eff) GT 0) THEN BEGIN
    ON_IOERROR, giveup
    
    OPENR, u, filein, /GET_LUN
    ON_IOERROR, readup
    
    eff=fltarr(3,s[2])
    READF, u, eff;  efficiency data are in the 2nd column
    
    IF (s[0] EQ 2) THEN BEGIN
      FOR j=0,s[2]-1 DO BEGIN
        IF (eff[1,j] NE 0) THEN M[*,j]=M[*,j]/eff[1,j] ELSE M[*,j]=M[*,j]
      ENDFOR
    ENDIF
    FREE_LUN, u
    IF (s[0] EQ 3) THEN BEGIN
      FOR j=0,s[2]-1 DO BEGIN
        IF (eff[1,j] NE 0) THEN M[*,j,*]=M[*,j,*]/eff[1,j] ELSE M[*,j,*]=M[*,j,*]
      ENDFOR
    ENDIF
    
    readup:FREE_LUN, u
    giveup:
  ENDIF
  
  
  ;************************
  ;***** The input Weights due to previous corrections *****
  s_e =size( datp.e)
  if n_elements(datp.e) ge s12 then begin & in_e=fltarr(s[1],s[2])
  if s_e[0] eq 3 then begin & M2=total(M,3) & E2=total(datp.e,3)   & ide=where(E2 gt 0.) ;average weights 3D->2D
  in_e[ide]=(M2[ide]/E2[ide]^2)>0.    & endif $
  else if s_e[s_e[0]+2] eq n_elements(M) then begin & ide=where(datp.e gt 0.)   ;M=2D,E=2D ->  calculate weights
  in_e[ide]=(M[ ide]/datp.e[ide]^2)>0. & endif $
  else if s_e[s_e[0]+2] eq s12 then in_e=datp.e>0.               ;M=3D,E=2D -> we already have weights f(M/e^2)
endif

if n_elements(datp.e) ge s12 then in_e=in_e*dOmega else in_e=dOmega

;***** Call radial integrations *****
radial_AT, M, CENTER=xy0, INCREMENT=incr, MASK=masq, PixSize=xypsiz, $
  input_weight=in_e, DIAGRAM=Wout, X=xx, ERROR=error
  
coeff_err=sqrt(N_run_hr[0])*N_run_hr[1]
X=atan(xx/dst)*!RADEG ;Make unit angles
ERROR=ERROR/coeff_err

;***** Return All *****
sw=size(Wout)
if sw[0] eq 2 then begin
  Wout =transpose(temporary(Wout))
  error=transpose(temporary(error))
  
  ;***** Subtraction of a time-independent background  *****
  
  IF (N_ELEMENTS(bck_ch) GT 0) THEN BEGIN
    bck=fltarr(sw[1])
    chmin=bck_ch[0] & chmax=bck_ch[1]
    FOR i=0,sw[1]-1  DO BEGIN
      bck[i]=mean(Wout[chmin:chmax,i])
      Wout[*,i]=Wout[*,i]-bck[i]
      
    ENDFOR
  ;    Wout=Wout>0
  ENDIF
  
  IF KEYWORD_SET(bck_slope) THEN BEGIN  ; WORK ONLY FOR 512 CHANNELS
  
    a0=fltarr(sw[1]) & b0=fltarr(sw[1])
    x_tmp=findgen(sw[2])
    slope=fltarr(sw[2],sw[1])
    FOR i=0,sw[1]-1  DO BEGIN
      a0[i]=mean(Wout[0:20,i])
      b0[i]=(mean(Wout[490:511,i])-a0[i])/(sw[2]-1)
      slope[*,i]=a0[i]+b0[i]*x_tmp[*]
      Wout[*,i]=Wout[*,i]-slope[*,i]
    ENDFOR
  ENDIF
  ;
  
  ;***** fit channel(theta[degree]): ch=A0+A1*theta+A2*theta^2  *****
  ;***** and store all in output file                           *****
  
  IF (N_ELEMENTS(fileout) GT 0) THEN BEGIN
    ON_IOERROR, getout
    i0=0 & i1=sw[1]-1;
    IF (N_ELEMENTS(th_min) NE 0) THEN i0=min(where (X GE th_min))
    IF (N_ELEMENTS(th_max) NE 0) THEN i1=max(where (X LE th_max))
    
    chel=fltarr(sw[1])
    FOR i=0, sw[1]-1 DO BEGIN
      gauss = str_fit(Wout[*,i],Bkgd=0,Xin=Z_in)
      chel[i]   = gauss[2]
    ENDFOR
    OPENW, u, fileout, /GET_LUN
    ON_IOERROR, readout
    fitch = POLY_FIT(X[i0:i1], chel[i0:i1], 2)
    
    PRINTF,u, 'Coef  of the fit: elastic channel=A0+A1*theta+A2*theta^2'
    PRINTF,u, 'between ',X[i0], ' and ',x[i1], ' degree'
    PRINTF, u, fitch[0]
    PRINTF, u, fitch[1]
    PRINTF, u, fitch[2]
    PRINTF,u, ''
    PRINTF, u,'  ', 'theta(degree)', '   ','exp el ch', '  ','el ch from fit'
    PRINTF,u, ''
    for j=0,sw[1]-1. do PRINTF, u, X[j], chel[j],fitch[0]+fitch[1]*X[j]+fitch[2]*X[j]^2
    PRINTF,u, ''
    PRINTF, u, 'total nr of angles =',sw[1]
    PRINTF, u, 'nr. of fitted angles =',i1-i0+1
    
    FREE_LUN, u
    readout: FREE_LUN, u
  ENDIF
  getout:
  
  ; *******
  datp.x_tit="Channels"
  datp.y_tit="Angles"
  datp.z_tit="Integrated counts"
  Y=X & X=Z_in & Z=0.
endif else begin

  datp.x_tit="Angles"
  datp.y_tit="Integrated counts"
  datp.z_tit=""
  Y=0. & Z=0.
endelse

MOD_DATP ,datp,"X",X
MOD_DATP ,datp,"Y",Y
MOD_DATP ,datp,"Z",Z
MOD_DATP ,datp,"E",error
GIVE_DATP,datp
return,Wout
end
