; ----------------------------------------------------------------------------
;*****************************************************************************
FUNCTION reb_brisp, w_in, dE=dE, force=forcebin, keep_elastic=keep_elastic, lambda=lambda, eb, fb
;+
;******* *********
;** User callable
;** For IN4, IN5, IN6, D7 and BRISP data.
;** 
;**  rebins output workspace from t2e to constant energy bin width.
;** 
;** ARGUMENTS:
;**  dE  : required energy bin width
;**  lambda : incident wavelength - required for BRISP, as value in parameter block is not reliable
;**  (eb and fb are obsolete and kept for backwards compatibility)
;** 
;** KEYWORDS:
;**  /force  : rebin over entire energy range
;**       otherwise: rebin only where dE is > point spacing (default)
;**  /keepelastic  : do not rebin over the elastic peak
;** 
;** DIMENSIONS
;**  w_in=(nE,nspectra) -> w_out(dE, nspectra)
;** 
;** COMMAND SYNTAX:
;**  w10=reb(w9,dE=<dE>[,/force])
;** 
;**  (optional keywords shown in square brackets)
;** 
;**               KHA,JRS 17/5/07
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-

  iprint=0  ; if iprint>0, show debugging messages
  
  COMMON c_lamp_access, inst
  COMMON constants, const1, const2, const3, const4
  
  IF (iprint GT 0) THEN PRINT,'Start reb:'
  
  take_datp, datp
  IF(N_ELEMENTS(eb) GT 0) THEN dE=eb
  IF(N_ELEMENTS(fb) GT 0) THEN forcebin=fb
  
  ; ------------------------------------------------------------------------------
  ;Set up input workspace
  
  par  =datp.p
  w_out=0
  
  sw=SIZE(w_in)
  IF (sw[0] EQ 1) THEN nspectra=1     ELSE $
    IF (sw[0] EQ 2) THEN nspectra=sw[2] ELSE return,w_in
  nchannels=sw[1]
  IF (iprint GT 0) THEN PRINT,'SIZE(w_in)=',sw
  IF (iprint GT 0) THEN PRINT,'nchannels=',nchannels,' nspectra=',nspectra
  
  npts=nchannels
  
  IF (N_ELEMENTS(dE) EQ 0) THEN BEGIN
    PRINT,'reb: error - dE not specified'
    RETURN, w_in
  ENDIF
  
  IF (N_ELEMENTS(forcebin) EQ 0) THEN forcebin=0
  
  x_in=datp.x
  e_in=datp.e
  
  se=size(e_in)
  sw=size(w_in)
  
  errors=1
  IF (se[0] NE sw[0] OR se[1] NE sw[1] OR se[2] NE sw[2]) THEN errors=0
  
  sx=size(x_in)  & BRISP=0
  IF (sx[0] GT 1) THEN BRISP=1
  IF (BRISP AND N_ELEMENTS(lambda) EQ 0 AND forcebin EQ 0) THEN BEGIN
    PRINT,'reb: error - lambda must be specified for BRISP (unles you use forcebin)'
    RETURN, w_in
  ENDIF
  
  IF (iprint GT 0) THEN PRINT,'npts=',npts,' nspectra=',nspectra,' forcebin=',forcebin,' BRISP=',BRISP
  IF (iprint GT 0) THEN PRINT,'End of input setup section'
  
  ; ------------------------------------------------------------------------------
  ;Determine elastic peak region
  
  IF KEYWORD_SET(keep_elastic) THEN BEGIN
    IF (BRISP) THEN BEGIN
      PRINT,'Warning - cannot keep elastic-peak binning for BRISP data'
    ENDIF ELSE BEGIN
      lambda=par(21)
      IF (inst EQ 'D7') THEN lambda=par(4)
      Ei=const4/lambda^2
      x_buf=x_in
      w_buf=TOTAL(w_in,2)
      e_buf=SQRT(TOTAL(e_in^2,2))
      gauss  = FLTARR(4)
      dgauss = FLTARR(4)
      FITGAUSS, x_buf, w_buf, e_buf, -Ei/10., Ei/10., gauss, dgauss
      IF iprint THEN PRINT,' Gauss pars =',gauss
      xElmin=gauss[2]-gauss[3]*2.355
      xElmax=gauss[2]+gauss[3]*2.355
      iEl=WHERE(x_buf GT xElmin AND x_buf LT xElmax)
      x_El=x_in[iEl]
      w_El=w_in[iEl,*]
      e_El=e_in[iEl,*]
    ENDELSE
  ENDIF
  
  ; ------------------------------------------------------------------------------
  ;Set up rebin arrays
  
  x_buf=x_in
  IF (BRISP) THEN BEGIN
    x_buf=REFORM(x_buf[*,nspectra/2])
    Ei=const4/lambda^2
    IF (forcebin EQ 0) THEN xmin=MIN(x_in[0,*])
    xmax=MAX(x_in[npts-1,*])
  ENDIF ELSE BEGIN
    xmin=x_buf[0]
    xmax=x_buf[npts-1]
  ENDELSE
  IF (forcebin) THEN xmin=MAX([x_buf[0],-50.])
  
  dx_buf=x_buf
  dx_buf[1:npts-2]=(x_buf[2:npts-1]-x_buf[0:npts-3])/2.
  dx_buf[0]=x_buf[1]-x_buf[0]  & dx_buf[npts-1]=x_buf[npts-1]-x_buf[npts-2]
  
  IF (BRISP) THEN BEGIN
    x_in0=FLTARR(npts+1,nspectra)  ; channel boundaries
    x_in0[1:npts-1,*]=(x_in[0:npts-2,*]+x_in[1:npts-1,*])/2.
    x_in0[0,*]=x_in0[1,*]-(x_in[1,*]-x_in[0,*])
    x_in0[npts,*]=x_in0[npts-1,*]+(x_in[npts-1,*]-x_in[npts-2,*])
  ENDIF ELSE BEGIN
    x_in0=FLTARR(npts+1)    ; channel boundaries
    x_in0[1:npts-1]=(x_in[0:npts-2]+x_in[1:npts-1])/2.
    x_in0[0]=x_in0[1]-(x_in[1]-x_in[0])
    x_in0[npts]=x_in0[npts-1]+(x_in[npts-1]-x_in[npts-2])
  ENDELSE
  
  IF (iprint GT 0) THEN BEGIN
    PRINT,'xmin=',xmin,' xmax=',xmax
    PRINT,'   channel      x_buf        dx_buf        x01         x02'
    IF (BRISP) THEN $
      FOR i=0,npts-1 DO PRINT, i, x_buf[i], dx_buf[i], x_in0[i,nspectra/2], x_in0[i+1,nspectra/2] $
    ELSE $
      FOR i=0,npts-1 DO PRINT, i, x_buf[i], dx_buf[i], x_in0[i], x_in0[i+1]
  ENDIF
  
  nbin=LONG((xmax-xmin)/dE)+2
  
  IF (forcebin EQ 1) THEN BEGIN
    IF (iprint GT 0) THEN PRINT,'forcebin=1'
    neverwider:
    i0=0
    IF (iprint GT 0) THEN PRINT,'rebin everything'
    xstart=dE*FLOAT(FIX(xmin/dE)-1)
    x_bin=FLOAT(INDGEN(nbin))*dE+xstart
  ENDIF ELSE BEGIN
    IF (iprint GT 0) THEN PRINT,'forcebin=0'
    wider=WHERE(dx_buf GT dE, nw)
    IF (nw LE 0) THEN GOTO, neverwider
    x_bin=FLTARR(nbin)
    i0=wider[nw-1]  & x0=x_buf[i0]
    IF (iprint GT 0) THEN PRINT,'rebin from x=',x0
    x_bin[0:i0]=x_buf[0:i0]  ; channel centres
    n=FIX((xmax-x0)/dE)
    IF (x0 LT 0.) THEN xstart=dE*FLOAT(FIX(x0/dE)+1) $
    ELSE xstart=dE*FLOAT(FIX(x0/dE+2))
    x_bin[i0+1:i0+n]=FLOAT(INDGEN(n))*dE+xstart
    x_bin[i0+1]=(x_bin[i0]+x_bin[i0+2])/2.
    nbin=i0+n+1
    x_bin=x_bin[0:nbin-1]
    IF (BRISP) THEN BEGIN
      Eps=x_bin[1]  & dEps=x_bin[1]-x_bin[0]
      k=dEps/(Ei-Eps)^1.5
      WHILE (Eps GT xmin) DO BEGIN
        Eps=x_bin[0]
        dEps=k*(Ei-Eps)^1.5
        x_bin=[Eps-dEps,x_bin]
      ENDWHILE
    ENDIF
  ENDELSE
  
  IF (iprint GT 0) THEN PRINT,'Number of points added at low E =',N_ELEMENTS(x_bin)-nbin
  nbin=N_ELEMENTS(x_bin)
  
  dx_bin=x_bin
  dx_bin[1:nbin-2]=(x_bin[2:nbin-1]-x_bin[0:nbin-3])/2.
  dx_bin[0]=x_bin[1]-x_bin[0]  & dx_bin[nbin-1]=x_bin[nbin-1]-x_bin[nbin-2]
  
  x_bin0=FLTARR(nbin+1)  ; channel boundaries
  x_bin0[1:nbin-1]=(x_bin[0:nbin-2]+x_bin[1:nbin-1])/2.
  x_bin0[0]=x_bin0[1]-(x_bin[1]-x_bin[0])
  x_bin0[nbin]=x_bin0[nbin-1]+(x_bin[nbin-1]-x_bin[nbin-2])
  
  IF (iprint GT 0) THEN BEGIN
    PRINT,'   bin      x_bin     dx_bin      x01       x02'
    FOR i=0,nbin-1 DO PRINT, i, x_bin[i], dx_bin[i], x_bin0[i], x_bin0[i+1]
  ENDIF
  
  IF (iprint GT 0) THEN PRINT,'End of rebin array setup section'
  
  ; ------------------------------------------------------------------------------
  ;Perform rebinning
  
  w_out=FLTARR(nbin,nspectra)
  IF (errors) THEN e_out=FLTARR(nbin,nspectra)
  
  IF (BRISP) THEN BEGIN
  
    iptmin=LONARR(nspectra)  & iptmin[*]=0
    ysum=FLTARR(nspectra)  & esum2=ysum
    
    FOR ibin=0,nbin-1 DO BEGIN
      xmin=x_bin0[ibin]  & xmax=x_bin0[ibin+1]
      FOR ispec=0,nspectra-1 DO BEGIN
        xsum=0.  & ysum=0. & esum2=0.
        FOR ipt=iptmin[ispec],npts-1 DO BEGIN
          x2=x_in0[ipt+1,ispec]
          IF (x2 GT xmin) THEN BEGIN
            x1=x_in0[ipt,ispec]
            IF (x1 GT xmax) THEN GOTO, nomore1
            IF (x2 LT xmax) THEN xhi=x2 ELSE xhi=xmax
            xhi=MIN([x2,xmax])
            xlo=MAX([x1,xmin])
            deltax=xhi-xlo
            IF (deltax LT 0.0) THEN PRINT,'Error in rebinning'
            xsum=xsum+deltax
            ysum=ysum+deltax*w_in[ipt,ispec]
            IF (errors EQ 1) THEN esum2=esum2+(deltax*e_in[ipt,ispec])^2
          ENDIF
        ENDFOR
        nomore1:      IF (xsum EQ 0.) THEN BEGIN
          w_out[ibin,ispec]=0.
          IF (errors EQ 1) THEN e_out[ibin,ispec]=-1.
        ENDIF ELSE BEGIN
          w_out[ibin,ispec]=ysum/xsum
          IF (errors EQ 1) THEN e_out[ibin,ispec]=SQRT(esum2)/xsum
        ENDELSE
        iptmin[ispec]=(ipt-1)>0
      ENDFOR  ; loop over spectra
    ENDFOR
    
  ENDIF ELSE BEGIN
  
    IF (i0 GT 0) THEN BEGIN  ; do not rebin first points
      w_out[0:i0-1,*]=w_in[0:i0-1,*]
      IF (errors EQ 1) THEN e_out[0:i0-1,*]=e_in[0:i0-1,*]
    ENDIF
    
    iptmin=0
    ysum=FLTARR(nspectra)  & esum2=ysum
    
    FOR ibin=i0,nbin-1 DO BEGIN
      xmin=x_bin0[ibin]  & xmax=x_bin0[ibin+1]
      xsum=0.    & ysum[*]=0.  & esum2[*]=0.
      FOR ipt=iptmin,npts-1 DO BEGIN
        x2=x_in0[ipt+1]
        IF (x2 GT xmin) THEN BEGIN
          x1=x_in0[ipt]
          IF (x1 GT xmax) THEN GOTO, nomore2
          IF (x2 LT xmax) THEN xhi=x2 ELSE xhi=xmax
          xhi=MIN([x2,xmax])
          xlo=MAX([x1,xmin])
          deltax=xhi-xlo
          IF (deltax LT 0.0) THEN PRINT,'Error in rebinning'
          xsum=xsum+deltax
          ysum=ysum+deltax*w_in[ipt,*]
          IF (errors EQ 1) THEN esum2=esum2+(deltax*e_in[ipt,*])^2
        ENDIF
      ENDFOR
      nomore2:    IF (xsum EQ 0.) THEN BEGIN
        w_out[ibin,*]=0.
        IF (errors EQ 1) THEN e_out[ibin,*]=-1.
      ENDIF ELSE BEGIN
        w_out[ibin,*]=ysum/xsum
        IF (errors EQ 1) THEN e_out[ibin,*]=SQRT(esum2)/xsum
      ENDELSE
      iptmin=(ipt-1)>0
    ENDFOR
    
  ENDELSE
  
  x_out=x_bin
  
  IF (iprint GT 0) THEN PRINT,'End of rebinning section'
  
  ; ------------------------------------------------------------------------------
  ;Remove zeroed points
  
  IF (errors EQ 1) THEN BEGIN
    WHILE (TOTAL(e_out[0,*]) LT -FLOAT(nspectra)+0.01) DO BEGIN
      x_out=x_out[1:nbin-1]
      w_out=w_out[1:nbin-1,*]
      e_out=e_out[1:nbin-1,*]
      nbin=nbin-1
    ENDWHILE
    WHILE (TOTAL(e_out[nbin-1,*]) LT -FLOAT(nspectra)+0.01) DO BEGIN
      x_out=x_out[0:nbin-2]
      w_out=w_out[0:nbin-2,*]
      e_out=e_out[0:nbin-2,*]
      nbin=nbin-1
    ENDWHILE
  ENDIF
  
  FOR ispec=0,nspectra-1 DO BEGIN
    IF (TOTAL(w_in[*,ispec],1) EQ 0.) THEN BEGIN
      w_out[*,ispec]=0.
      IF (errors EQ 1) THEN e_out[*,ispec]=-1.
    ENDIF
  ENDFOR
  
  IF (iprint GT 0) THEN PRINT,'End of remove zeros section'
  
  ; ------------------------------------------------------------------------------
  ;Splice unbinned data back in
  
  IF KEYWORD_SET(keep_elastic) THEN BEGIN
    i1=WHERE(x_out LT xElmin)
    i2=WHERE(x_out GT xElmax)
    x_out=[x_out[i1],x_El,x_out[i2]]
    w_out=[w_out[i1,*],w_El,w_out[i2,*]]
    e_out=[e_out[i1,*],e_El,e_out[i2,*]]
    IF iprint THEN HELP, x_out, w_out, e_out
  ENDIF
  
  ; ------------------------------------------------------------------------------
  ;Return parameters and exit
  
  mod_datp, datp, "x", x_out
  IF (errors EQ 1) THEN mod_datp, datp, "e", e_out
  
  s=STRTRIM(STRING(dE),2) & dE = string_round(s)
  IF forcebin THEN fs = ',/force' ELSE fs = ''
  datp.other_tit=datp.other_tit+' -rb(dE='+dE+fs+')'
  
  IF (forcebin EQ 0) THEN BEGIN
    s=STRTRIM(STRING(x_out[i0]),2)  & Elim = string_round(s)
    range='where E > '+Elim+' meV'
    IF (BRISP) THEN datp.p[21]=lambda
  ENDIF ELSE range='everywhere'
  
  PRINT, 'Reb: rebinned '+range+' to constant bin width of '+dE+'meV'
  
  give_datp, datp
  
  finished:
  RETURN, w_out
  
END
