; ----------------------------------------------------------------------------
;*****************************************************************************
FUNCTION rebrisp, w_in, dE=dE, force=forcebin, keep_elastic=keep_elastic, lam=lam, common_e=common_e, eb, fb
;+
;******* *******
;** User callable
;** For IN4, IN5, IN6 and D7 data.
;** 
;**  rebins output workspace from t2e to constant energy bin width.
;** 
;** ARGUMENTS:
;**  dE    : required energy bin width
;**  (eb and fb are obsolete and kept for backwards compatability)
;** 
;** KEYWORDS:
;**  /force    : rebin over entire energy range
;**        otherwise: rebin only where dE is > point spacing (default)
;**  /keepelastic  : do not rebin over the elastic peak
;** 
;** DIMENSIONS
;**  w_in=(nE,nspectra) -> w_out(dE, nspectra)
;** 
;** COMMAND SYNTAX:
;**  w10=reb(w9,dE=<dE>[,/force])
;** 
;**  (optional keywords shown in square brackets)
;** 
;**                 KHA,JRS 6/10/06
;** 
;** ************* adapted to BRISP (FF 2007), but to be simpified......
;**     lam=wavelength
;**     keyword /common_e allows to create an unique energy scale
;**     /keepelastic not implemented
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-

  iprint=0   ; if iprint>0, show debugging messages
  
  COMMON c_lamp_access, inst
  
  IF (iprint GT 0) THEN PRINT,'Start reb:'
  
  take_datp, datp
  IF(N_ELEMENTS(eb) GT 0) THEN dE=eb
  IF(N_ELEMENTS(fb) GT 0) THEN forcebin=fb
  
  ; ------------------------------------------------------------------------------
  ;Set up input workspace
  
  par  =datp.p
  w_out=0
  
  sw=SIZE(w_in)
  IF (sw[0] EQ 1) THEN nspectra=1     ELSE $
    IF (sw[0] EQ 2) THEN nspectra=sw[2] ELSE return,w_in
  nchannels=sw[1]
  IF (iprint GT 0) THEN PRINT,'SIZE(w_in)=',sw
  IF (iprint GT 0) THEN PRINT,'nchannels=',nchannels,' nspectra=',nspectra
  
  npts=nchannels
  
  IF (N_ELEMENTS(dE) EQ 0) THEN BEGIN
    PRINT,'reb: error - dE not specified'
    return,w_in
  ENDIF
  
  IF (N_ELEMENTS(forcebin) EQ 0) THEN forcebin=0
  
  ; ******* modif
  x_in=datp.x & sx=size(x_in)
  y_in=datp.y & sy=size(y_in)
  e_in=datp.e & se=size(e_in)
  ; ******* end modif
  
  errors=1
  IF (se[0] NE sw[0] OR se[1] NE sw[1] OR se[2] NE sw[2]) THEN errors=0
  
  
  IF (iprint GT 0) THEN PRINT,'npts=',npts,' nspectra=',nspectra,' forcebin=',forcebin
  IF (iprint GT 0) THEN PRINT,'End of input setup section'
  
  ; ------------------------------------------------------------------------------
  ;Determine elastic peak region
  
  IF KEYWORD_SET(keep_elastic) THEN BEGIN
    Ei=81.81/datp.p[21]^2
    IF inst EQ 'D7' THEN Ei=81.81/datp.p[4]^2
    x_buf=x_in
    w_buf=TOTAL(w_in,2)
    e_buf=SQRT(TOTAL(e_in^2,2))
    gauss  = FLTARR(4)
    dgauss = FLTARR(4)
    FITGAUSS, x_buf, w_buf, e_buf, -Ei/10., Ei/10., gauss, dgauss
    IF iprint THEN PRINT,' Gauss pars =',gauss
    
    xElmin=gauss[2]-gauss[3]*2.355
    xElmax=gauss[2]+gauss[3]*2.355
    iEl=WHERE(x_buf GT xElmin AND x_buf LT xElmax)
    x_buf=x_in[iEl]
    w_buf=w_in[iEl,*]
    e_buf=e_in[iEl,*]
  ENDIF
  
  ; ------------------------------------------------------------------------------
  ;Set up rebin arrays
  ;
  ;************ modif ********************************************
  xmin=FLTARR(nspectra) & xmax=FLTARR(nspectra)
  
  IF (forcebin EQ 0) THEN xmin=x_in[0,*] & xmax=x_in[npts-1,*]
  
  IF (forcebin EQ 1)  THEN xmin=MAX(x_in[0,*]) & xmax=MIN(x_in[npts-1,*])
  
  dx_in=x_in
  
  dx_in[1:npts-2,*]=(x_in[2:npts-1,*]-x_in[0:npts-3,*])/2.
  dx_in[0,*]=x_in[1,*]-x_in[0,*]   & dx_in[npts-1,*]=x_in[npts-1,*]-x_in[npts-2,*]
  
  x_in0=FLTARR(npts+1,nspectra)   ; histogram
  x_in0[1:npts-1,*]=(x_in[0:npts-2,*]+x_in[1:npts-1,*])/2.
  x_in0[0,*]=x_in0[1,*]-(x_in[1,*]-x_in[0,*])
  x_in0[npts,*]=x_in0[npts-1,*]+(x_in[npts-1,*]-x_in[npts-2,*])
  
  iv=10
  IF (iprint GT 0) THEN BEGIN
    PRINT,'   channel      x_in        dx_in        x01         x02'
    FOR i=0,npts-1 DO PRINT, i, x_in[i,iv], dx_in[i,iv], x_in0[i,iv], x_in0[i+1,iv]
  ENDIF
  
  nbin=LONG((min(xmax)-max(xmin))/dE)+2
  xx_bin=FLTARR(nbin,nspectra)
  xstart=FLTARR(nspectra)
  
  IF (forcebin EQ 1) THEN BEGIN
    IF (iprint GT 0) THEN PRINT,'forcebin=1'
    neverwider:
    i0=0
    i00=fltarr(nspectra)
    IF (iprint GT 0) THEN PRINT,'rebin everything'
    ;       xstart=dE*FLOAT(FIX(xmin/dE)-1)
    ;       x_bin=FLOAT(INDGEN(nbin))*dE+xstart
    ;
    ;******* and now I'm just building up a matrix made of a constant energy scale
    FOR j=0,nspectra-1 DO BEGIN
      xstart[j]=dE*FLOAT(FIX(xmin/dE)-1)
      xx_bin[*,j]=FLOAT(INDGEN(nbin))*dE+xstart[j]
    ENDFOR
    
    x_bin=xx_bin
    
  ENDIF ELSE BEGIN
    IF (iprint GT 0) THEN PRINT,'forcebin=0'
    
    
    ;************ modif ********************************************
    
    nw=INTARR(nspectra)
    
    FOR j=0,nspectra-1 DO BEGIN
      dx_in_buf=dx_in[*,j]
      w_wider=WHERE(dx_in_buf GT dE, nnw)
      nw[j]=nnw
    ENDFOR
    
    ;       IF (nw LE 0) THEN GOTO, neverwider
    
    ;      xx_bin=FLTARR(nbin,nspectra)
    i00=nw-1
    imax=npts-max(i00)
    
    
    x0sp=FLTARR(nspectra)
    ;        xstart=FLTARR(nspectra)
    n=INTARR(nspectra)
    
    FOR j=0,nspectra-1 DO BEGIN
      x0sp[j]=x_in[i00[j],j]
      
      ;
      IF (iprint GT 0) THEN PRINT,'rebin from x=',x0sp
      xx_bin[0:i00[j],j]=x_in[0:i00[j],j]
      
      n[j]=FIX((x_in[i00[j]+imax-1,j]-x0sp[j])/dE)
      IF (x0sp[j] LT 0.) THEN xstart[j]=dE*FLOAT(FIX(x0sp[j]/dE)+1) $
      ELSE xstart[j]=dE*FLOAT(FIX(x0sp[j]/dE)+2)
      xx_bin[i00[j]+1:i00[j]+n[j],j]=FLOAT(INDGEN(n[j]))*dE+xstart[j]
      xx_bin[i00[j]+1,j]=(xx_bin[i00[j],j]+xx_bin[i00[j]+2,j])/2.
    ENDFOR
    
    
    nbin=min(i00+n+1)
    
    x_bin=FLTARR(nbin,nspectra)
    
    x_bin=xx_bin[0:nbin-1,*]
  ENDELSE
  
  dx_bin=x_bin
  dx_bin[1:nbin-2,*]=(x_bin[2:nbin-1,*]-x_bin[0:nbin-3,*])/2.
  dx_bin[0,*]=x_bin[1,*]-x_bin[0,*]    & dx_bin[nbin-1,*]=x_bin[nbin-1,*]-x_bin[nbin-2,*]
  
  x_bin0=FLTARR(nbin+1,nspectra)  ; histogram
  x_bin0[1:nbin-1,*]=(x_bin[0:nbin-2,*]+x_bin[1:nbin-1,*])/2.
  x_bin0[0,*]=x_bin0[1,*]-(x_bin[1,*]-x_bin[0,*])
  x_bin0[nbin,*]=x_bin0[nbin-1,*]+(x_bin[nbin-1,*]-x_bin[nbin-2,*])
  
  ;    IF (iprint GT 0) THEN BEGIN
  ;       PRINT,'   bin      x_bin     dx_bin      x01       x02'
  ;       FOR i=0,nbin-1 DO PRINT, i, x_bin(i), dx_bin(i), x_bin0(i), x_bin0(i+1)
  ;    ENDIF
  ;
  ;    IF (iprint GT 0) THEN PRINT,'End of rebin array setup section'
  ;
  ;;;-------------------------------------------------------------------------------
  ;;Perform rebinning
  
  w_out=FLTARR(nbin,nspectra)
  
  IF (errors EQ 1) THEN e_out=FLTARR(nbin,nspectra)
  
  
  FOR jspec=0,nspectra-1 DO BEGIN
    IF (i00[jspec] GT 0) THEN BEGIN    ; do not rebin first points
      w_out[0:i00[jspec]-1,jspec]=w_in[0:i00[jspec]-1,jspec]
      IF (errors EQ 1) THEN e_out[0:i00[jspec]-1,jspec]=e_in[0:i00[jspec]-1,jspec]
    ENDIF
  ENDFOR
  
  iptmin=0
  ;    xsum=FLTARR(nspectra)
  ysum=FLTARR(nspectra)  & esum2=ysum
  ;
  FOR jspec=0,nspectra-1 DO BEGIN
    ibin=0 & iptmin=0 & ipt=0
    i0=i00[jspec]
    FOR ibin=i0,nbin-1 DO BEGIN
      xx_bin0=x_bin0[*,jspec]
      xx_in0=x_in0[*,jspec]
      xmin=xx_bin0[ibin] & xmax=xx_bin0[ibin+1]
      xsum=0.     & ysum[*]=0. & esum2[*]=0.
      FOR ipt=iptmin,npts-1 DO BEGIN
        x2=xx_in0[ipt+1]
        IF (x2 GT xmin) THEN BEGIN
          x1=xx_in0[ipt]
          IF (x1 GT xmax) THEN GOTO, nomore
          IF (x2 LT xmax) THEN xhi=x2 ELSE xhi=xmax
          xhi=MIN([x2,xmax])
          xlo=MAX([x1,xmin])
          deltax=xhi-xlo
          IF (deltax LT 0.0) THEN PRINT,'Error in rebinning'
          xsum=xsum+deltax
          ysum[jspec]=ysum[jspec]+deltax*w_in[ipt,(jspec)]
          ;          stop
          IF (errors EQ 1) THEN esum2[jspec]=esum2[jspec]+(deltax*e_in[ipt,(jspec)])^2
        ENDIF
      ENDFOR
      
      nomore:     IF (xsum EQ 0.) THEN BEGIN
      
        w_out[ibin,(jspec)]=0.
        IF (errors EQ 1) THEN e_out[ibin,(jspec)]=-1.
      ENDIF ELSE BEGIN
      
        w_out[ibin,(jspec)]=ysum[jspec]/xsum
        
        IF (errors EQ 1) THEN e_out[ibin,jspec]=SQRT(esum2[jspec])/xsum
      ENDELSE
      iptmin=(ipt-1)>0
    ENDFOR
    
  ENDFOR
  ;stop
  x_out=x_bin
  
  IF (iprint GT 0) THEN PRINT,'End of rebinning section'
  
  ; ------------------------------------------------------------------------------
  ;Remove zeroed points
  
  IF (errors EQ 1) THEN BEGIN
    WHILE (TOTAL(e_out[0,*]) LT -FLOAT(nspectra)+0.01) DO BEGIN
      x_out=x_out[1:nbin-1,*]
      w_out=w_out[1:nbin-1,*]
      e_out=e_out[1:nbin-1,*]
      nbin=nbin-1
    ENDWHILE
    WHILE (TOTAL(e_out[nbin-1,*]) LT -FLOAT(nspectra)+0.01) DO BEGIN
      x_out=x_out[0:nbin-2,*]
      w_out=w_out[0:nbin-2,*]
      e_out=e_out[0:nbin-2,*]
      nbin=nbin-1
    ENDWHILE
  ENDIF
  
  FOR ispec=0,nspectra-1 DO BEGIN
    IF (TOTAL(w_in[*,ispec],1) EQ 0.) THEN BEGIN
      w_out[*,ispec]=0.
      IF (errors EQ 1) THEN e_out[*,ispec]=-1.
    ENDIF
  ENDFOR
  
  IF (iprint GT 0) THEN PRINT,'End of remove zeros section'
  ;
  ;;-------------------------------------------------------------------------------
  IF KEYWORD_SET(common_e) THEN BEGIN
  
    min_e=max(x_out[i00,*])
    max_e=min(x_out[nbin-1,*])
    n1=INTARR(nspectra)
    n2=INTARR(nspectra)
    
    
    FOR jspec=0,nspectra-1 DO BEGIN
    
      ;    n1(jspec)=WHERE(x_out(*,jspec) EQ min_e) ;!! sometimes does find a value with EQ!!
      ;    n2(jspec)=WHERE(x_out(*,jspec) EQ max_e) ; and then I use the following
    
      n1[jspec]=min(WHERE(abs(x_out[*,jspec]-min_e) LT 0.0001)) ;!!
      n2[jspec]=min(WHERE(abs(x_out[*,jspec]-max_e) LT 0.0001))
      
    ENDFOR
    dn1=min(n2-n1) & dn2=max(n2-n1)
    ;.    stop
    IF (dn1 NE dn2) THEN BEGIN
      PRINT,'reb: error in evaluation of unique enrgy axis'
      return,w_in
    ENDIF
    ;      x1_out=FLTARR(dn1+1,nspectra)
    x1_out=FLTARR(dn1+1)
    x_out=x1_out
    w1_out=FLTARR(dn1+1,nspectra)
    e1_out=FLTARR(dn1+1,nspectra)
    
    
    x1_out=x_bin[n1[0]:n2[0],0]
    
    FOR jspec=0,nspectra-1 DO BEGIN
      w1_out[*,jspec]=w_out[n1[jspec]:n2[jspec],jspec]
      e1_out[*,jspec]=e_out[n1[jspec]:n2[jspec],jspec]
    ENDFOR
    
    x_out=x1_out &  w_out=w1_out & e_out=e1_out
    
  ENDIF ELSE x_out=x_bin[*,0] & min_e=x_out[0]
  
  
  
  ;;-------------------------------------------------------------------------------
  ;;Splice unbinned data back in
  ;
  ;    IF KEYWORD_SET(keep_elastic) THEN BEGIN
  ;       i1=WHERE(x_out LT xElmin)
  ;       i2=WHERE(x_out GT xElmax)
  ;       x_out=[x_out(i1),x_buf,x_out(i2)]
  ;       w_out=[w_out(i1,*),w_buf,w_out(i2,*)]
  ;       e_out=[e_out(i1,*),e_buf,e_out(i2,*)]
  ;       IF iprint THEN HELP, x_out, w_out, e_out
  ;    ENDIF
  
  ; ------------------------------------------------------------------------------
  ;Return parameters and exit
  
  mod_datp, datp, "x", x_out
  
  IF (errors EQ 1) THEN mod_datp, datp, "e", e_out
  
  s=STRTRIM(STRING(dE),2) & dE = string_round(s)
  IF forcebin THEN fs = ',/force' ELSE fs = ''
  datp.other_tit=datp.other_tit+' -rb(dE='+dE+fs+')'
  
  s=STRARR(nspectra)
  Elim=STRARR(nspectra)
  range=STRARR(nspectra)
  
  ;    FOR jspec=0,nspectra-1 DO BEGIN
  ;    IF (forcebin EQ 0) THEN BEGIN
  ;       s(jspec)=STRTRIM(STRING(x_out(i00(jspec))),2)    & Elim(jspec) = string_round(s(jspec))
  ;       range(jspec)='where E > '+Elim(jspec)+' meV'
  ;    ENDIF ELSE range(jspec)='everywhere'
  ;
  s=STRTRIM(STRING(min_e))    & Elim = string_round(s)
  e_min='where E > '+Elim+' meV'
  PRINT, 'Reb: rebinned '+e_min+' to constant energy bin width of '+dE+'meV'
  ;    ENDFOR
  ;stop
  give_datp, datp
  
  finished:
  RETURN, w_out
  
END
