;******* ************
function sqw_pix_old, W_in , XY0=xy0, MASK=msk, file_el=file_el, dE=dEps, $
                    dQ=dQ,Qmax=Qmax,Ib=Wbck, fileout=fileout
;+
;******* ***********
;** OUTPUT: Sqw from raw data, converting I(theta, channels) of each pixels, without rebinning.
;**         If Ib used, ambient background is subtracted. ki/kf factor is included.         
;**            
;** *****
;** INPUT:  W_in    = 3D raw data matrix
;** *****   xy0     = [x0,y0] center determined by "brisp_center.pro"
;**         MASK    = mask determined manually and by "brisp_center.pro"
;**                   contains 1's for masked values otherwise 0's(default)
;**         file_el = file determined by "brisp_rad.pro" containing the elastic channel position
;**
;**** OPTIONAL INPUTS
;**	     Ib      = 3D matrix containing background data randomnly generated by raw_bck.pro   
;**	     dE      = energies step (default 0.5 meV)   
;**	     dQ      = Q step (default 0.1 Ang^-1)   
;**	     Qmax    = maximum Q at which Sqw is determined (default 2.0 Ang^-1)   
;**          fileout = possibility to export ASCII files with Sqw data, one for each Q
;**                     (but it is often convenient to normalise and then export data with out_Sqw) 
;**     
;**** DIMENSIONS: Wout(dE,dQ)
;**
;** Call: w20=sqw_pix(w2,XY0=c,MASK=w40,file_el='van_el.dat',Ib=w13,dE=0.5,dQ=0.1,Qmax=2.0,fileout='test')
;**
;**	(FF, April 2010)
;**
;-

; ********************* USEFUL CONVERSION FACTORS ******************************

    c1 = 5.22697          ; E(meV) =c1*V(m/ms)^2 for neutron
    c2 = 2.07193571       ; E(meV) =c2*k(A^-1)^2 for neutron
    c3 = 3.956076         ; V(m/ms)=c3/lambda(A) for neutron
    c4 = 81.8066          ; E(meV) =c4/lambda(A)^2 for neutron

; ************** INPUT WORKSPACE - CHECK FOR PARAMETERS  *********************
Wout=0 & M=W_in
TAKE_DATP, datp                 ; workspace variables
par = datp.p
s=size(M)   & s12= s(1)* s(2)

IF (s(0) lt 2) or (s(1) lt 2) or (s(2) lt 2) THEN RETURN, Wout
	IF (s(0) eq 3) then begin 
	nch=s(3) & chan=datp.z 
	endif else nch=1
 if n_elements(dEps)  eq 0 then dEps = 0.5
 if n_elements(dQ)    eq 0 then dQ   = 0.1
 if n_elements(Qmax)  eq 0 then Qmax = 2.0
 if n_elements(Wbck)  ne 0 then ib = 1 else ib=0

;******************** INSTRUMENT SETUP *****************************
chw=	par(18)		; channel width (musec)
L2= 	par(27)*1000.   ; sample-detector distance (mm)
lambda=	par(21)		; incident wavelength (Ang)

;;;***** Read coeff 2degree of ch_el vs teta *****
ON_IOERROR, giveup
OPENR, u, file_el, /GET_LUN
ON_IOERROR, readup
pol2=fltarr(1,3)
line=''
READF, u, line;	
READF, u, line;	
READF, u, pol2;	
FREE_LUN, u
readup:FREE_LUN, u
giveup:
;
; ****** DEFINE MATRIX WITH DISTANCES AND ANGLES *********************************
;
r_pix=brisp_dist(M,CENTER=xy0)	; pixel-det_centre distance in cm
L2_th=sqrt((10*r_pix)^2+L2^2)	; pixel-sample distance in mm
ang_r=ATAN(10*r_pix/L2)      	; angle seen from the pixel [rad]
ang_d=ang_r*!RADEG		; angle seen from the pixel [deg]
dOm=cos(ang_r)^3		; Solid Angle seen form the pixels
;
; ****** SET ENERGY-TOF PARAMETERS  *********************************
;
M_chel=pol2(0)+pol2(1)*ang_d+pol2(2)*ang_d^2	; 2D matrix of ch_el for each pixel
Ei = c4/lambda^2				; incident energy [meV]
Vi = SQRT(Ei/c1)				; incident velocity [m/ms]
T2   = L2/Vi 	          			; Tel [musec] at theta=0
T2_th= L2_th/Vi           			; Tel(theta) [musec] (ie at each pixel)
;
;********** THE MASK: imsk=maked pixels, ok=no masked pixels *******************
;
if n_elements(msk) eq s12 then begin
imsk=where(msk, COMPLEMENT=ok) 
     if imsk(0) ge 0 then th_r=ang_r(ok); angles with mask/nomask id [rad]
endif
n_ok=n_elements(ok)			; number of nomasked pixels
jM = ARRAY_INDICES(msk, ok)		; recover the index of nomasked pixels

;************* build the matrix w,Q,Int with index (teta,channel)*****************
M_hw=fltarr(n_ok,nch)		; hw(theta,ch)
M_q=fltarr(n_ok,nch)		; Q(theta,ch)
I_hw=fltarr(n_ok,nch)		; I(theta,ch)
Ib_hw=fltarr(n_ok,nch)		; I(theta,ch) for background
fact=fltarr(n_ok,nch)		; (dt/dE)(ki/kf) (theta,ch)

	FOR i=0L,n_ok-1 DO BEGIN	; use only the nomasked pixels		 
k=ok(i)				
t_th=T2_th(k)+chw*(chan-M_chel(k))	; travel time for each theta
Ef=c1*(L2_th(k)/t_th)^2			; Ef [meV]
M_hw(i,*)=Ei-Ef				; hw(theta,ch)
ca=2*cos(ang_r(k))
M_q(i,*)=SQRT((Ei+Ef-ca*SQRT(Ei*Ef))/c2); Q(theta,ch)
fact(i,*)= SQRT(c1*Ei)/(2.*Ef^2)	; dt/dE*ki/kf = SQRT(c1/(4.*Ef^3))*SQRT(Ei/Ef)
I_hw(i,*)=M(jM(0,i),jM(1,i),*)/dOm(k)	; Int(theta,ch)/dOmega
if ib then Ib_hw(i,*)=Wbck(jM(0,i),jM(1,i),*);/dOm(k)
	ENDFOR

;************* hw-Q grids ***********************
Esup=max(M_hw,min=Einf)
Emin=dEps*round(Einf/dEps) & Emax=dEps*round(Esup/dEps);
nEps = FIX((Emax - Emin) / dEps) + 1;
 Eps = Emin + dEps*FINDGEN(nEps);
dEmin= Eps  - dEps/2.;

; *****************
IF (lambda GT 1.5) then Qmax=1.0 ; to avoid numerical bug at 2 Ang
Qsup=max(M_q,min=Qinf)
Qmin=dQ*round(Qinf/dQ)
nQ = FIX((Qmax - Qmin) / dQ) +1
Qx =  Qmin + dQ*FINDGEN(nQ)
dQmin=Qx-dQ/2.;

;************* fill Q-hw grids of sqo ***********************
sqo=fltarr(nQ,nEps) & esqo=sqo  ; prepare output matrices

 FOR iQ=0,nQ-1 DO BEGIN

indQ = where((M_q ge dQmin(iQ)) AND (M_q lt dQmin(iQ)+dQ)); select Q: Q-dQ < Q < Q+dQ
MQ_hw=M_hw(indQ)  	; select hw, Int, Ibckg fact inside Q+/-dQ Q
IQ_hw=I_hw(indQ) & factQ=fact(indQ)  
if ib then IbQ_hw=Ib_hw(indQ) 

   FOR iE = 0,nEps-1  DO BEGIN
indE = where((MQ_hw ge dEmin(iE)-dEps/2.) AND (MQ_hw lt dEmin(iE)+dEps/2)); select E: E-dE/2 < E < E+dE/2
;indE = where((MQ_hw ge dEmin(iE)) AND (MQ_hw lt dEmin(iE)+dEps)); select E: E-dE/2 < E < E+dE/2

	if indE(0) ge 0 then begin
yes=n_elements(indE)		; to normalise to effective nr of (theta,channels)
av_fact=mean(factQ(indE)) 	; variation of factor in E+/-dE is negligible: use of average values
			 	; is preferable to speed evaluation of errors
sumS= total(IQ_hw(indE))*av_fact; 
sqo(iQ,iE)=sumS/yes
esqo(iQ,iE)=SQRT(av_fact)*SQRT(sumS)/yes;
	   if ib then begin 
	   sumSb=total(IbQ_hw(indE))*av_fact
	   Sbqw=sumSb/yes		; sqo for bck
	    sqo(iQ,iE)=sqo(iQ,iE)-Sbqw > 0; 
	   endif
	endif
    ENDFOR
 ENDFOR


;; ********** OUTPUT: sqo *************
Wout=transpose(sqo) & error=transpose(esqo) & X=Eps & Y=Qx & Z=0
datp.y_tit="Q [Ang-1]" & datp.x_tit="E [meV]" & datp.z_tit=""

MOD_DATP ,datp,"X",X
MOD_DATP ,datp,"Y",Y
MOD_DATP ,datp,"Z",Z
MOD_DATP ,datp,"E",error
GIVE_DATP,datp

;********** Open and write to output files ***********
if n_elements(fileout) NE 0 then begin
    ON_IOERROR, getout

    FOR iQ=0,nQ-1 DO BEGIN
	QQ=Qx(iq)
       Qstring=STRTRIM(STRING(FIX(100.*QQ)),2)
       IF (QQ LT 0.1) THEN Qstring='q00'+Qstring $
       ELSE IF (QQ LT 1.0) THEN Qstring='q0'+Qstring $
       ELSE Qstring='q'+Qstring

       PRINT,'Saving to ',STRTRIM(fileout,2),'.',Qstring
       S=FLTARR(nEps) & S(*)=sqo(iQ,*)
       dS=FLTARR(nEps)    & dS(*)=esqo(iQ,*)
       ii=WHERE(dS GT 0, n)

       IF (n GT 0) THEN BEGIN
         E=Eps(ii)    & S=S(ii)   & dS=dS(ii)
         OPENW, u, fileout+'.'+Qstring, /GET_LUN
         ON_IOERROR, readout
         ch=STRTRIM(STRING(QQ),2)  & jj=STRPOS(ch,'.') & Q=STRMID(ch,0,jj(0)+4)
         PRINTF, u, datp.w_tit+' | S(Q,E) at Q='+Q+' Ang-1'
         PRINTF, u, 'numor: ', datp.other_tit
         PRINTF, u, datp.x_tit, ', S(Q,E), err_S(Q,E)'
         PRINTF, u, 'N points = '
         PRINTF, u, n
         FOR ii=0,n-1 DO PRINTF, u, E(ii), ' ', S(ii),' ', dS(ii)
         FREE_LUN, u 
         readout: FREE_LUN, u
      ENDIF

    ENDFOR
getout:
endif

return,Wout
end
