;+
;*******************************************************************************
;
;     FUNCTION t2e_bri, w_in, in5multi = idet
;
;For BRISP: based on t2e, t2e_elp     FF (2007)
;
;Converts tof data in w_in to energy transfer in w_out.
; Includes the factor of ki/kf.
; teta dependence of time flight, EPS
;
;
;**     FITGAUSS replaced by str_fit.
;
;*** option: save elastic channels/fit or use input data fit (eg Van) to determine elastic peak
;
;*** an_elmin: if set, the intensity around the elastic peak are put =0 fot teta< an_elmin
;
;***  w13=t2e_bri(w3,phi_min=0.1,phi_max=17.,Lsd=4.,lam=1.,ch_w=3.73,elch_out='van_el.dat')
;        OR
;***  w14=t2e_bri(w4,phi_min=0.1,phi_max=17.,Lsd=4.,lam=1.,ch_w=3.73,elch_in=[94.39,0.14,0.031])
;
;DIMENSIONS:
; w_in(nchannels,nspectra) -> w_out(nE,spectra)
;
;
;*******************************************************************************
;-

FUNCTION t2e_tof_elp, w_in, idet, $
    phi_min=phi_min, phi_max=phi_max, $
    elch_out=fileout,elch_in=poly2,an_elmin=an_elmin
;+
;******* ***********
;** t2e_bri stuff
;-
    
  forward_function str_fit    ;
    
  COMMON c_lamp_access, inst
  COMMON printing, iprint, outstring
  COMMON constants, const1, const2, const3, const4
  
  iprint=0
  
  IF iprint THEN PRINT,'Start t2e_tof:'
  
  take_datp, datp
  
  ; ------------------------------------------------------------------------------
  ;Set up input workspace and energy parameters
  
  para = datp.p
  
  chw    =para[18]         ; channel width (microsec)
  L2     = para[27]*1000.       ; sample-detector distance (mm)
  lambda=para[21]         ; incident wavelength (Ang)
  
  IF iprint THEN begin
    PRINT,'lambda=',lambda,'A'
    print,' chw=',chw,'micro-sec'
    ;    print,' chel=',chel
    PRINT,' L2=',L2,'mm'
  endif
  
  sw = SIZE(w_in)
  nchannels = sw[1]
  IF sw[0] EQ 1 THEN nspectra = 1 ELSE nspectra = sw[2]
  
  IF iprint THEN PRINT,'nchannels=',nchannels,' nspectra=',nspectra
  x_in = datp.x
  y_in = datp.y
  e_in = datp.e
  
  se = SIZE(e_in)
  IF (se[0] NE sw[0]) OR (se[1] NE sw[1]) THEN e_in = SQRT(w_in)
  
  IF iprint THEN PRINT, 't2e: End of check dimensions, etc. section'
  
  ; ------------------------------------------------------------------------------------
  
  IF ( (inst EQ 'IN5') or (inst EQ 'BRISP') ) AND (nspectra GT 1) THEN BEGIN
    ; select the working angular range with keywords or not
    if keyword_set(phi_min) then ind = where( y_in ge float(phi_min) ) else ind = -1
    if ind[0] lt 0 then ind_phi_min = 0 else ind_phi_min = ind[0]
    
    if keyword_set(phi_max) then ind = where( y_in le float(phi_max) ) else ind = -1
    if ind[0] lt 0 then ind_phi_max = n_elements(y_in)-1 else ind_phi_max = ind[n_elements(ind)-1]
    nspectra = ind_phi_max - ind_phi_min + 1
    
    print, 'Transformation range = [',y_in[ind_phi_min],'/ ',y_in[ind_phi_max],'] degrees'
    
    w_buf = w_in[*,ind_phi_min:ind_phi_max]
    e_buf = e_in[*,ind_phi_min:ind_phi_max]
    x_buf = w_in[*,ind_phi_min:ind_phi_max]
    y_out=y_in[ind_phi_min:ind_phi_max]
    x_out = x_in
    w_out = w_buf
    e_out = e_buf
    
    Ei   = const4/lambda^2
    Vi   = SQRT(Ei/const1)
    L2teta=L2/(cos(y_out*!DTOR))
    Tteta= L2teta/Vi  ;          Tel  = L2/Vi            ; Tel en ms
    ; -------------------------------------------------------
    ; modify to find exact elp position for each spectrum
    ; -------------------------------------------------------
    ytot = TOTAL(w_buf,2)
    y0     = MAX(ytot,i0)
    chel=i0
    xmin   = x_in[(i0 - 30) > 0]
    xmax   = x_in[(i0 + 30) < (nchannels - 1)]
    
    
    ind   = where(x_in gt xmin and x_in lt xmax)
    chel_new   = fltarr(nspectra)
    gauss = chel_new
    E_min   = fltarr(nspectra)
    E_max   = fltarr(nspectra)
    
    FOR i=0, nspectra-1 DO BEGIN
    
    
      IF (N_ELEMENTS(poly2) EQ 3) THEN BEGIN
        chel_new[i]=poly2[0]+poly2[1]*y_out[i]+poly2[2]*y_out[i]^2; elastic channel for each teta (i.e. pixel)
      ENDIF ELSE BEGIN
        gauss = str_fit(w_buf[ind,i],Bkgd=0,Xin=x_in[ind])
        chel_new[i]   = gauss[2]
      ENDELSE
      
      print, 'Spectrum ',i+1,', found elastic channel at: ',chel_new[i], ' at' , y_out[i], ' deg'
      timeteta = chw*FLOAT(x_in - chel_new[i]) + Tteta[i]
      Efteta   = const1*(L2teta[i]/timeteta)^2
      Eps  = Ei - Efteta
      ; ------------------------------------------------------------------------------------
      ;     Transform to energy transfer
      ; ------------------------------------------------------------------------------------
      dtdE = SQRT(const1/(4.*Efteta^3))
      kikf = SQRT(Ei/Efteta)
      IF iprint THEN BEGIN
        if i eq 1 then begin
          ;                PRINT,'channel    x_in   elp    time    Eps       dtdE      kikf'
          FOR ii=0,nchannels-1 DO $
            PRINT, FORMAT='(I4,2I5,2F10.3,6F10.6)', $
            ii+1, x_in[ii], chel_new, timeteta[ii], Eps[ii], dtdE[ii], kikf[ii]
        endif
      ENDIF
      
      fact=(dtdE*kikf) # (FLTARR(1,nspectra) + 1.)
      x_buf[*,i] = Eps
      
      IF KEYWORD_SET(an_elmin) THEN BEGIN
        IF (y_out[i] LT an_elmin) THEN BEGIN
          jchel=FIX(chel_new)
          w_buf[jchel[i]-20:jchel[i]+20,i]=0.
          e_buf[jchel[i]-20:jchel[i]+20,i]=0.
        ENDIF
      ENDIF
      w_out[*,i] = w_buf[*,i]*fact
      e_out[*,i] = e_buf[*,i]*fact
    ENDFOR
    
    sx=size(x_buf)
    x_out=x_buf
    
  ENDIF ELSE BEGIN
    print,'Wrong instrument or just one spectrum??'
  ENDELSE
  
  ; ------------------------------------------------------------------------------------
  ; Save :
  ;elastic channels-teta degree to be used in case one wants to use
  ; other data (eg Van) for energy conversion
  ; their 2nd poly degree fit in order to rebuild the elastic channel at each teta
  
  
  IF (N_ELEMENTS(fileout) GT 0) THEN BEGIN
  
    OPENW, 1, fileout
    PRINTF, 1, '    ',datp.x_tit,'    ', datp.y_tit
    PRINTF, 1, '    ',nspectra
    for j=0,sx[2]-1 do begin
      PRINTF, 1, chel_new[j], y_out[j]
    endfor
    PRINTF,1, ''
    PRINTF,1, 'Coef  of the 2nd degree pol fitting elas_ch=f(teta/degree)'
    
    fitch = POLY_FIT(y_out, chel_new, 2)
    PRINTF, 1, fitch[0]
    PRINTF, 1, fitch[1]
    PRINTF, 1, fitch[2]
    CLOSE, 1
  ENDIF
  
  
  ; ------------------------------------------------------------------------------------
  ; Prepare data and return
  
  s = STRTRIM(STRING(Ei),2) & i =  strpos(/reverse_search,s,'.') & Ei = STRMID(s,0,i + 3)
  
  outstring = 'Ei=' + Ei + 'meV'
  
  bit=' -t2e(Ei='+Ei
  IF inst EQ 'IN5' THEN bit = bit + ',' + STRTRIM(STRING(det),2) + ')' $
  ELSE bit = bit + ')'
  
  datp.other_tit = datp.other_tit + bit
  datp.x_tit = 'Energy Transfer (meV)'
  
  mod_datp, datp, "x", x_out
  mod_datp, datp, "e", e_out
  IF inst EQ 'IN5' THEN mod_datp, datp, "y", y_out
  
  IF inst EQ 'BRISP' THEN mod_datp, datp, "y", y_out
  
  give_datp, datp
  
  RETURN, w_out
END

; ------------------------------------------------------------------------------
;*******************************************************************************

FUNCTION t2e_bri, w_in, in5multi = idet, $
                  phi_min=phi_min, phi_max=phi_max, $
                  elch_out=fileout,elch_in=filein,an_elmin=an_elmin
;+
;******* *******
;** User callable
;** 
;-

  forward_function str_fit

  COMMON c_lamp_access, inst
  COMMON printing, iprint, outstring
  COMMON constants, const1, const2, const3, const4
  
  
  iprint = 0    ; if iprint>0, show debugging messages
  
  const1 = 5.22697          ; E(meV)=const1*V(m/ms)^2 for neutron
  const2 = 2.07193571       ; E(meV)=const2*k(A^-1)^2 for neutron
  const3 = 3.956076         ; V(m/ms)=const3/lambda(A) for neutron
  const4 = 81.8066          ; E(meV)=const4/lambda(A)^2 for neutron
  
  IF iprint THEN PRINT,'Start t2e:'
  
  IF KEYWORD_SET(in5multi) THEN idet=0
  
  ; ------------------------------------------------------------------------------
  ;Check instrument name and call appropriate function
  
  IF (inst EQ 'IN5') OR (inst EQ 'BRISP') THEN $
    w_out = t2e_tof_elp(w_in, idet, phi_min=phi_min, phi_max=phi_max,elch_out=fileout,elch_in=filein,an_elmin=an_elmin)
    
  ; ------------------------------------------------------------------------------
  ;Return parameters and exit
    
  IF N_ELEMENTS(outstring) GT 0 THEN PRINT,'t2e: '+outstring
  
  RETURN, w_out
END
