Pro axis_conversion_init
;+
; :Description:
;    Set some commons.
;    
; :History:
;    14 Oct 2010 Created, Eric Pellegrini
;    
; :Author: 
;    pellegrini
;-

    common axis_conversion_global, instr_struct
     
    ; Structure that stores the 'datp' indexes for the wavelength property.      
    wavelength = {d1b : 4, d4 : 4, d7 : 4, d16 : 8, d20 : 33, in4 : 21, in5 : 21, in6 : 21}
    
    ; Structure that stores the 'datp' indexes for the offset property.      
    offset = {d20 : 37}
    
    ; A super structure made out of the property structures is build and will be used as a global structure.
    instr_struct = create_struct('wavelength', wavelength, 'offset', offset)

End 

;+
; :Description:
;    Retrieves in the |datp| structure of a given numor a list of input parameters.
;
; :Params:
;    w : in, required
;        the workspace the internal structure of which will be searched for the wavelegnth.
; :Keyword:
;    parameters : in, optional
;        a list of string specifying the parameters to retrieve from the selected numor.
;        
; :Returns:
;    A structure whose tag names are the upperized values of the input parameters.
;    Two special cases for the value associated to a given tag
;        - -7777 -> the parameters was not found in the |instr_struct| global structure
;        - -8888 -> the instrument was not found for the selected parameter
;    
; :History:
;    06 Oct 2010 Created, Eric Pellegrini
;    
; :Author: 
;    pellegrini
;-
Function get_numor_parameters, w, parameters = parameters

    ; Import some common.
    common axis_conversion_global, instr_struct
    common c_lamp_access, inst_value

    ; Store the internal structure associated to the input workspace in |datp| structure.
    take_datp, datp

    ; If the 'instr_struct' structure is not defined, initialize it.    
    if n_elements(instr_struct) eq 0 then axis_conversion_init 
    
    ; If no parameters is provided, set the wavelength as the default property.    
    if not keyword_set(parameters) then parameters = ['wavelength']
    
    result = {}    

    ; Loop over the parameters array.         
    for i = 0, n_elements(parameters) - 1 do begin
    
        ; The current parameter is upperized.
        p = strupcase(parameters[i])
    
        ; And search in the |instr_struct| tag names.
        idx = where(tag_names(instr_struct) eq p)
        
        ; If the parameter is not found in the |instr_struct| tag names set its results value to -7777.
        if idx eq -1 then begin
        
            result = create_struct(result, create_struct(p, -7777))
            
        ; Otherwise, search if the instrument is present in the structure corresponding to the current parameter.
        endif else begin
                        
            idx1 = where(tag_names(instr_struct.(idx)) eq inst_value)

            ; If not, set the results to -8888.            
            if idx1 eq -1 then begin
            
                result = create_struct(result, create_struct(p, -8888))
                
            ; Otherwise set the results to the corresponding |datp.p| value.
            endif else begin
            
                result = create_struct(result, create_struct(p, datp.p[instr_struct.(idx).(idx1)]))
            
            endelse
            
        endelse
                            
    endfor
    
    return, result
    
End

;+
; :Description:
;    Perform the conversion of x axis from q to d.
;
; :Params:
;    w : in, required
;
; :Returns:
;    A new workspace with x-axis converted from q to d.
;    
; :History:
;    07 Oct 2010 Created, Eric Pellegrini
;    
; :Author: pellegrini
;-
Function q_to_d, w, datp = datp

    ; Store the internal structure associated to the input workspace in |datp| structure.
    if not keyword_set(datp) then take_datp, datp
    
    ; A local copy of the x-axis values.
    ; This can be a multi-dimensional array.
    d = datp.x
            
    ; This is the conversion formula between q and d.
    d = 2.0*!pi/d
    
    ; The number of dimensions of d matrix (1 or 2).
    n_dims = size(d, /N_DIMENSIONS)

    ; If q is a matrix store its second dimension in |dim_max|.    
    If n_dims ne 1 then begin
        
        dim_max = (size(d, /DIMENSIONS))[1]
                
    ; Else if q is a vector store sets its second dimension to 0 and store it in |dim_max|.    
    Endif else begin
    
        dim_max = 0
    
    Endelse
    
    ; This array will store the subscripts of the q matrix sorted in ascending order row by row.     
    idx = intarr(size(d, /DIMENSIONS))
    
    ; Sort the q matrix row by row.
    For i = 0, dim_max do begin
    
        idx[*,i] = sort(d[*,i])        
                
    Endfor
    
    ; Replace the current q matrix by its sorted version.
    d = d[idx]
    
    ; The 'x' field of the internal structure is modifed with d sorted values instead of the q. 
    mod_datp, datp, 'x', d
        
    ; The 'x_tit' field of the internal structure is set to the new x axis title. 
    mod_datp, datp, 'x_tit', 'd/Angstrom'
    
    ; If error are available modify also them according to the sorted indexes.
    If n_elements(datp.e) eq n_elements(w) then begin
    
        ; Local copy of the error.
        e = datp.e
        
        mod_datp, datp, 'E', e[idx]
        
    Endif

    ; Modify the datas according to the sorted indexes.
    ww = w[idx]
    
    ; Apply the changes to the workspace that will be returned.
    give_datp, datp
    
    ; The output workspace.
    return, ww
    
End

;+
; :Description:
;    Performs the conversion of x axis from 2Theta to q in the elastic case.
;
; :Params:
;    w : in, required
;        the workspace used to perform the conversion.
;
; :Keywords:
;    wavelength : in, optional
;        the wavelength used for the conversion.
;        
;    offset : in, optional
;        the offset in 2Theta.
;        
; :Returns:
;    A new workspace with x-axis converted from 2Theta to q.
;
; :History:
;    06 Oct 2010 Created, Eric Pellegrini
;
; :Author: 
;    pellegrini
;-
Function two_theta_to_q, w, wavelength = wavelength, offset = offset, datp = datp

    common c_lamp_access, inst_value
    
    ; A copy of the input workspace.
    ww = w 

    ; Store the internal structure associated to the input workspace in |datp| structure.
    take_datp, datp
        
    ; Case where the user did not provide the 'wavelength' keyword. Guess it for him.     
    if not keyword_set(wavelength) then begin
        res = get_numor_parameters(w, parameters = ['wavelength'])
        help, res, /STRUCTURE
        wavelength = res.wavelength
    endif
            
    ; If no wavelength is defined, do nothing and return.
    If wavelength < 0 then return, 0
    
    ; Set the offset to a null default value if none provided.         
    if not keyword_set(offset) then begin
        res = get_numor_parameters(w, parameters = ['offset'])
        offset = res.offset
    endif
    
    ; If no offset is defined, set it to 0.
    If offset < 0 then offset = 0.0
        
    ; The x-axis values are corrected from the offset.
    ; This can be a multi-dimensional array.
    x = abs(datp.x) - offset
            
    ; This is the conversion formula between 2Theta and q.
    q = 4.0*!pi*sin((!pi/360.0)*x)/wavelength
            
    ; The number of dimensions of q matrix (1 or 2).
    n_dims = size(q, /N_DIMENSIONS)

    ; If q is a matrix store its second dimension in |dim_max|.    
    If n_dims ne 1 then begin
        
        dim_max = (size(q, /DIMENSIONS))[1]
                
    ; Else if q is a vector store sets its second dimension to 0 and store it in |dim_max|.    
    Endif else begin
    
        dim_max = 0
    
    Endelse
    
    ; This array will store the subscripts of the q matrix sorted in ascending order row by row.     
    idx = intarr(size(q, /DIMENSIONS))
    
    ; Sort the q matrix row by row.
    For i = 0, dim_max do begin
    
        idx[*,i] = sort(q[*,i])        
                
    Endfor
    
    ; Replace the current q matrix by its sorted version.
    q = q[idx]
    
    ; The 'x' field of the internal structure is modifed with q sorted values instead of the 2Theta. 
    mod_datp, datp, 'x', q
    
    ; The 'x_tit' field of the internal structure is set to the new x axis title. 
    mod_datp, datp, 'x_tit', 'Q/Ang^-1'
    
    ; If error are available modify also them according to the sorted indexes.
    If n_elements(datp.e) eq n_elements(w) then begin
    
        ; Local copy of the error.
        e = datp.e
        
        mod_datp, datp, 'E', e[idx]
        
    Endif

    ; Modify the datas according to the sorted indexes.
    ww = ww[idx]
    
    ; Apply the changes to the workspace that will be retunred.
    If not arg_present(datp) then give_datp, datp
    
    ; The output workspace.
    return, ww
    
End

;+
; :Description:
;    Performs the conversion of x axis from 2Theta to d.
;
; :Params:
;    w_in : in, required
;
; :Keywords:
;    _EXTRA : in, optional
;        the extra keyword. Can be 'wavelength' and/or 'offset'.
;        
; :Returns:
;    A new workspace with x-axis converted from 2Theta to d.
;    
; :History:
;    07 Oct 2010 Created, Eric Pellegrini
;    
; :Author: pellegrini
;-
Function two_theta_to_d, w_in, _EXTRA = extra
            
    w_out = two_theta_to_q(w_in, datp=datp, _EXTRA = extra)
    
    w_out = q_to_d(w_out, datp=datp)

    return, w_out
        
End

pro axis_conversion
;just to compile
end

