  FUNCTION sort_d16, win, tol=tol
  ;+
  ;******* ***********
  ;**
  ;** User callable
  ;** The call is w6=sort_d16(w1)
  ;**
  ;** win is a 3D diffractogram measured on D16 as a combination of
  ;** several acquisitions, e.g. w1 = rdopr(1:7) and the routine
  ;** checks the number of different gamma positions measured and
  ;** combines and orders them into an output workspace containing
  ;** only the N different gammas measured, i.e. a [320,320,N] array.
  ;**
  ;** The keyword tol can be used to give a certain tolerance to accept
  ;** gamma positions that differ by less than tol (in degrees).
  ;** If not given, the default tolerance is set to 0.1 degrees.
  ;** For a given number of gamma positions within the tolerance, 
  ;** the value retained is the smaller one.
  ;-
  
  s = size(win)
  if s[0] ne 3 then return, win
  size_x = s[1]
  size_y = s[2]
  nscans = s[3]
  
  if n_elements(tol) eq 0 then tol = 0.1
  
  ;get variables associated with win
  take_datp, datp
  
  gamma_pos = datp.pv[0,*]
  
  ;sort scans by increasing gamma
  order_index = sort(gamma_pos)
  
  gamma_pos = gamma_pos[order_index]
  wtmp = win[*,*,order_index]
  params = datp.pv[*,order_index]  
  
  ;make equal gamma values that are within tolerance
  for i = 0, n_elements(gamma_pos)-2 do begin
    reference_value = gamma_pos[i]
    in_tolerance = where(abs(gamma_pos - reference_value) le tol, count)
    if count gt 1 then begin
      average_gamma = total(gamma_pos[in_tolerance]) / count
      gamma_pos[in_tolerance] = average_gamma
      i += (count-1)
    endif
  endfor
  
  gamma_uniq = gamma_pos[uniq(gamma_pos)]
    
  ;define output array
  wout = fltarr(size_x, size_y, n_elements(gamma_uniq))
  pout = fltarr(n_elements(datp.p), n_elements(gamma_uniq))
  xout = findgen(size_x)
  yout = findgen(size_y) 
  counting_time = fltarr(n_elements(gamma_uniq))
  monitor1 = fltarr(n_elements(gamma_uniq))
  monitor2 = fltarr(n_elements(gamma_uniq))
  
  for i = 0, n_elements(gamma_uniq)-1 do begin
    index = where(abs(gamma_pos - gamma_uniq[i]) lt tol)
    if n_elements(index) gt 1 then wout[*,*,i] = total(wtmp[*,*,index],3) else wout[*,*,i] = wtmp[*,*,index[0]]
    pout[*,i] = params[*, index[0]]
    counting_time[i] = total(params[37,index],2)
    monitor1[i] = total(params[38,index],2)
    monitor2[i] = total(params[39,index],2)
  endfor
  eout = sqrt(wout)
  pout[37,*] = counting_time
  pout[38,*] = monitor1
  pout[39,*] = monitor2

  ;return sorted data + errors and parameters
  zout = indgen(n_elements(gamma_uniq))
  hist = datp.history + 'gamma sorted'
  outdatp = {x:xout, y:yout, z: zout, e:eout, n:monitor1, p:pout[*,0], pv:pout, w_tit:datp.w_tit, $
             x_tit:'X pixel', y_tit:'Y pixel', z_tit:'Gamma scan', other_tit:datp.other_tit,      $
             time:datp.time, history:hist, par_txt:datp.par_txt}

  give_datp, outdatp
  return, wout

end