  FUNCTION straight_d16, win, center=xy0, norm=norm
  ;+
  ;******* ***********
  ;**
  ;** User callable
  ;** The call is w6=straight_d16(w1)
  ;**
  ;** win is a 3D diffractogram measured on D16 as a combination of 
  ;** several acquisitions, e.g. w1 = rdopr(1:7) 
  ;** If the gamma scan is not ordered or there is one or several
  ;** gamma positions measured more than once, win needs to be
  ;** previously sorted by the sort_d16 routine.
  ;-

  ;set normalization method
  if n_elements(norm) eq 0 then norm = 'monitor'
  norm = strlowcase(strtrim(norm,2))
  if ((norm eq 't') or (norm eq 'time')) then norm = 'time' else norm = 'monitor'

  ;get variables associated with win
  take_datp, datp 
  
  det_dist = datp.p[12]  ;sample-detector distance (cm)
  xpixel_size = 0.1      ;cm
  ypixel_size = 0.1      ;cm
  wavelength = datp.p[8] ;angstroms
  
  s = size(win)
  size_x = s[1]
  size_y = s[2]
  if s[0] eq 2 then nscans = 1 else nscans = s[3]
  
  ;center (xy_firstpixel = [0.5,0.5], xy_lastpixel = [319.5,319.5], xy_center = [160,160])
  if n_elements(xy0) eq 0 then begin
    cx = float(s[1]/2)
    cy = float(s[2]/2) 
  endif else begin
    cx = float(xy0[0])
    cy = float(xy0[1])
  endelse
  print, 'Straight_d16 using center = ', cx, cy, ' & norm = ', norm
  
  ;horizontal and vertical distances to center
  dx = xpixel_size * (findgen(size_x)-cx+0.5) 
  dy = ypixel_size * (findgen(size_y)-cy+0.5) 
  theta_x = atan(dx/det_dist) * !radeg
  
  ;errors (need to be provided correctly, otherwise set to zero)
  ;if n_elements(datp.e) ne n_elements(win) then datp.e *= 0.0
  ;!!!!!!!!!!!! NEED TO DECIDE WHAT TO DO WITH ERRORS
  
  ;call to straight_1d --> straight_2d
  wout = [] & xout = [] & yout = [] & eout = []
  
  for i = 0, nscans-1 do begin
  
    if nscans gt 1 then begin
       gamma          = datp.pv[0,i]
       monitor_counts = datp.pv[38,i]
       counting_time  = datp.pv[37,i]  
    endif else begin
       gamma          = datp.p[0]
       monitor_counts = datp.p[38]
       counting_time  = datp.p[37]
    endelse
  
    ;normalize to monitor/time
    wtmp = win[*,*,i]
    if (norm eq 'time') then begin
      if counting_time gt 0.0 then begin
        wtmp /= counting_time 
        if n_elements(datp.e) eq n_elements(win) then errors = datp.e[*,*,i] / counting_time else errors = wtmp*0.0
      endif else begin
        print, 'Counting time = 0. No normalization applied!'
        if n_elements(datp.e) eq n_elements(win) then errors = datp.e[*,*,i] else errors = wtmp*0.0
      endelse
    endif else begin
      if monitor_counts gt 0.0 then begin
        wtmp /= monitor_counts
        if n_elements(datp.e) eq n_elements(win) then errors = datp.e[*,*,i] / monitor_counts else errors = wtmp*0.0
      endif else begin
        print, 'Monitor = 0!'
        if counting_time gt 0.0 then begin
          print, 'Normalizing by counting time.'
          wtmp /= counting_time
          if n_elements(datp.e) eq n_elements(win) then errors = datp.e[*,*,i] / counting_time else errors = wtmp*0.0
        endif else begin
          print, 'Counting time = 0. No normalization applied!'
          if n_elements(datp.e) eq n_elements(win) then errors = datp.e[*,*,i] else errors = wtmp*0.0
        endelse
      endelse  
    endelse
  
    x_axis = theta_x + gamma ;horizontal angle (degrees)
    y_axis = dy              ;vertical distance to center (cm)
    dattmp ={x:x_axis,y:y_axis, e:errors} 

    m = straight_1d(wtmp, distance = det_dist, datp = dattmp)
    wout = [wout, m]
    xout = [xout, dattmp.x]
    eout = [eout, dattmp.e]
    if i eq 0 then nbins = n_elements(m) else begin
      if n_elements(m) ne nbins then begin
        print, 'Problem! Scan ', i+1, ' binned to ', n_elements(m), ' x-points vs ', nbins, ' points for previous scans'
        print, 'Nothing returned.'
        return, 0
      endif
    endelse
  endfor
  
  wout = reform(wout, nbins, nscans)
  eout = reform(eout, nbins, nscans)
  xout = reform(xout, nbins, nscans)  
  
  ;Remove first and last points of output because some times have much lower intensity
  mod_datp, datp, 'e', eout[1:-2,*]
  mod_datp, datp, 'x', xout[1:-2,*]
  mod_datp, datp, 'y', 0
  give_datp, datp
  return, wout[1:-2,*]

end
