;+
; PRO d17_he3cal_gui
;
; LAMP GUI to calculate the polarization efficiency of a 3He cell on D17
; Based on code by Rebeen Salehbag and Hassan Sheidani
; ARW 16.03.07
;
; The program contains the following procedures:
;	d17_he3cal_gui_event:	Handles all events when changing things in the GUI
;	aw_filexist:		Checks whether a file exists
;	aw_rwd:			Locates the base widget
;	aw_heiopars:		Reads and writes the contents of the gui to a file
;	aw_mainupd		Collects relevant information on the main beam
;	aw_waterdef:		Collects relevant information and runs d17_waterdef
;	aw_setmask:		Manipulates the mask window and creates the mask text
;	aw_calibrate:		Collects all relevant information and passes it to d17_he3calibrate
;	aw_dateset:		Collects the date information and update the appropriate windows
;	aw_fitbuttons:		Sets fit and limits on clicking the 'fix' buttons in the fit window
;	aw_chfitpars:		Updates the uvalue field of the top widget on changing a fit parameter
;       aw_fit:			Carries out the fit
;       aw_plotwinhide:         Hides the plot window
;       aw_plotwinshow:         Creates the plot window
;       aw_plotupd:             Updates the data in the plot window
;	d17_he3cal_gui:		Sets up the GUI
; Calls to:  d17_waterdef, d17_he3calibrate,rdrun (external)
; Last modified:   ARW 3.1.11
;-
; *** Specify external subroutines that will get called from this program
forward_function rdrun
forward_function d17_he3calibrate
forward_function d17_waternorm

; ----------------------------------------------------------------------------------------------------------------------------
PRO d17_he3cal_gui_event,event
;+
;****************************************************
;******* This is the D17_HE3CAL event manager *******
;****************************************************
;*** This procedure sorts the possible actions and calls the relevant subroutine
;-

   WIDGET_CONTROL,event.id,get_uvalue=gv
   if(gv eq 'open')then aw_heiopars,event.top,'load'
   if(gv eq 'save')then aw_heiopars,event.top,'save'
   if(gv eq 'help')then print,'No help yet, sunshine'
   if(gv eq 'about')then begin
      print,'D17_he3cal.prox'
      print,'LAMP and IDL widget for calibrating the 3He filter on D17'
      print,'v.1.02, modified 22.06.08 by ARW'
      print,'Based on code by Rebeen Salehbag and Hassan Sheidani'
   endif
   if(gv eq 'setmask') or (gv eq 'changemask') then aw_setmask,event.top,gv
   if(gv eq 'watfil') then aw_waterdef,event.top
   if(gv eq 'mainfil') then aw_mainupd,event.top
   if(gv eq 'd17_calibrate') then aw_calibrate,event.top
   if(gv eq 'f1update') then begin
      dropindex=WIDGET_INFO(event.id,/DROPLIST_SELECT)
      widat=WIDGET_INFO(event.top,find_by_uname='f1val')
      WIDGET_CONTROL,widat,SENSITIVE=dropindex
   endif
    if(gv eq 'f2update') then begin
      dropindex=WIDGET_INFO(event.id,/DROPLIST_SELECT)
      widat=WIDGET_INFO(event.top,find_by_uname='f2val')
      WIDGET_CONTROL,widat,SENSITIVE=dropindex
   endif
   if(gv eq 'setime') then aw_dateset,event.id,gv
   if(gv eq 'setdate') then aw_dateset,event.id,gv
   if strmatch(gv,'parfx*') eq 1 or strmatch(gv,'limfx*') eq 1 then aw_fitbuttons,event.id,gv
   if(gv eq 'chfpar') eq 1 then aw_chfitpars,event.top
   if(gv eq 'he3fit') eq 1 then aw_fit,event.id
   if(gv eq 'plotshow') eq 1 then begin
      aw_rwd,event.top,topid
      WIDGET_CONTROL,topid,get_uvalue=topdat
      plotwin=topdat.plotwin
      isplot=WIDGET_INFO(plotwin,/VALID_ID)
      if isplot ne 1 then aw_plotwinshow,topid,plotwin
      WIDGET_CONTROL,plotwin,MAP=1
      aw_plotupd,plotwin,topdat
   endif
   if(gv eq 'quit')then begin
      aw_rwd,event.id,topid
      WIDGET_CONTROL,topid,get_uvalue=check
      widget_control,topid,/destroy
   endif
END

; ----------------------------------------------------------------------------------------------------------------------------
PRO aw_filexist,path,runno,exist
;+
;*******************************************************
;*** Checks whether a file exists ***
;*** 	'exist' = 0 if the file does not exist, =1 if it does
;*******************************************************
;-
	exist = 0
	rsz=size(runno)
	if rsz[1] ne 7 then runno=strcompress(string(runno),/REMOVE_ALL)
	while strlen(runno) lt 6 do runno='0'+runno
	result = file_which(path,runno)
	if strlen(result) gt 1 then exist = 1
end

; ----------------------------------------------------------------------------------------------------------------------------
PRO aw_rwd,widin,widout
;+
;*******************************************************
;******* This returns the ID for the base window *******
;*******************************************************
;*** Rewind to top widget ID
;-
   twid=widin
   widout=widin
   while twid ne 0 do begin
	twid=WIDGET_INFO(twid,/PARENT)
	if twid ne 0 then widout=twid
   endwhile
end

; ----------------------------------------------------------------------------------------------------------------------------
PRO aw_heiopars,widin,flag
;+
;***********************************************************************
;******* This reads and writes the contents of the gui to a file *******
;***********************************************************************
;-

; *** Find a file unit number
   get_lun,unit
;*** Find the relevant widgets
   aw_rwd,widin,topid					    ; Top base widget
   widat=intarr(4)
   widat[0]=WIDGET_INFO(topid,find_by_uname='umsg')	    ; Message window
;*** First addresses will contain text
   widat[1]=WIDGET_INFO(topid,find_by_uname='fdirtxt')	    ; File directory
   widat[2]=WIDGET_INFO(topid,find_by_uname='filetxt')	    ; File name
;*** Next addresses will be for the dropboxes
   ndp=5
   dp=strarr(ndp)
   dp[0]=WIDGET_INFO(widin,find_by_uname='watdim')      ; Dropbox for water dimensions
   dp[1]=WIDGET_INFO(topid,find_by_uname='mskf')        ; Dropbox for Mask choice
   dp[2]=WIDGET_INFO(topid,find_by_uname='f1inp')       ; Dropbox for Flipper 1 calc/fix
   dp[3]=WIDGET_INFO(topid,find_by_uname='f2inp')       ; Dropbox for Flipper 2 calc/fix
   dp[4]=WIDGET_INFO(topid,find_by_uname='fitcrss')     ; Dropbox for crossed polarizer/3He
;*** Next addresses will be for the buttons
   nb=12
   b=strarr(nb)
   b[0]=WIDGET_INFO(topid,find_by_uname='parfx1')     ; Button for fix/free date
   b[1]=WIDGET_INFO(topid,find_by_uname='parfx2')      ; Button for fix/free initial polarization
   b[2]=WIDGET_INFO(topid,find_by_uname='parfx3')       ; Button for fix/free t1
   b[3]=WIDGET_INFO(topid,find_by_uname='parfx4')     ; Button for fix/free pressure
   b[4]=WIDGET_INFO(topid,find_by_uname='parfx5')       ; Button for fix/free polarizer
   b[5]=WIDGET_INFO(topid,find_by_uname='parfx6')      ; Button for fix/free wavelength
   b[6]=WIDGET_INFO(topid,find_by_uname='limfx1')      ; Button for limits Date
   b[7]=WIDGET_INFO(topid,find_by_uname='limfx2')      ; Button for limits Initial Polarization
   b[8]=WIDGET_INFO(topid,find_by_uname='limfx3')       ; Button for limits t1
   b[9]=WIDGET_INFO(topid,find_by_uname='limfx4')     ; Button for limits Pressure
   b[10]=WIDGET_INFO(topid,find_by_uname='limfx5')       ; Button for limits Polarizer
   b[11]=WIDGET_INFO(topid,find_by_uname='limfx6')      ; Button for limits wavelength
;*** Next addresses will be for the parameters
   np=39
   p=strarr(np)
   p[0]=WIDGET_INFO(widin,find_by_uname='watfil')      ; Water file box
   p[1]=WIDGET_INFO(widin,find_by_uname='mainfil')     ; Main beam file box
   p[2]=WIDGET_INFO(topid,find_by_uname='mskpar1val')  ; Mask parameter 1
   p[3]=WIDGET_INFO(topid,find_by_uname='mskpar2val')  ; Mask parameter 2
   p[4]=WIDGET_INFO(topid,find_by_uname='mskpar3val')  ; Mask parameter 3
   p[5]=WIDGET_INFO(topid,find_by_uname='mskpar4val')  ; Mask parameter 4
   p[6]=WIDGET_INFO(topid,find_by_uname='calfils')     ; Calibration filenames
   p[7]=WIDGET_INFO(topid,find_by_uname='f1val')       ; Flipper 1 value
   p[8]=WIDGET_INFO(topid,find_by_uname='f1valpm')     ; Flipper 1 error
   p[9]=WIDGET_INFO(topid,find_by_uname='f2val')       ; Flipper 2 value
   p[10]=WIDGET_INFO(topid,find_by_uname='f2valpm')     ; Flipper 2 error
   p[11]=WIDGET_INFO(topid,find_by_uname='hrv')         ; Hour window
   p[12]=WIDGET_INFO(topid,find_by_uname='minv')        ; Minute window
   p[13]=WIDGET_INFO(topid,find_by_uname='dayv')        ; Day window
   p[14]=WIDGET_INFO(topid,find_by_uname='monv')        ; Month window
   p[15]=WIDGET_INFO(topid,find_by_uname='yrv')         ; Year window
   p[16]=WIDGET_INFO(topid,find_by_uname='parvals2')    ; Initial polarization window
   p[17]=WIDGET_INFO(topid,find_by_uname='parvals3')    ; T1 window
   p[18]=WIDGET_INFO(topid,find_by_uname='parvals4')    ; Pressure window
   p[19]=WIDGET_INFO(topid,find_by_uname='parvals5')    ; Polarizer window
   p[20]=WIDGET_INFO(topid,find_by_uname='parvals6')    ; Wavelength window
   p[21]=WIDGET_INFO(topid,find_by_uname='parerr1')     ; Date error
   p[22]=WIDGET_INFO(topid,find_by_uname='parerr2')     ; Initial polarization error
   p[23]=WIDGET_INFO(topid,find_by_uname='parerr3')     ; T1 error
   p[24]=WIDGET_INFO(topid,find_by_uname='parerr4')     ; Pressure error
   p[25]=WIDGET_INFO(topid,find_by_uname='parerr5')     ; Polarizer error
   p[26]=WIDGET_INFO(topid,find_by_uname='parerr6')     ; Wavelength error
   p[27]=WIDGET_INFO(topid,find_by_uname='min1')       ; Minimum Date
   p[28]=WIDGET_INFO(topid,find_by_uname='min2')       ; Minimum Initial Polarization
   p[29]=WIDGET_INFO(topid,find_by_uname='min3')       ; Minimum t1
   p[30]=WIDGET_INFO(topid,find_by_uname='min4')       ; Minimum Pressure
   p[31]=WIDGET_INFO(topid,find_by_uname='min5')       ; Minimum Polarizer
   p[32]=WIDGET_INFO(topid,find_by_uname='min6')       ; Minimum Wavelength
   p[33]=WIDGET_INFO(topid,find_by_uname='max1')        ; Maxmum Date
   p[34]=WIDGET_INFO(topid,find_by_uname='max2')        ; Maxmum Initial Polarization
   p[35]=WIDGET_INFO(topid,find_by_uname='max3')        ; Maxmum t1
   p[36]=WIDGET_INFO(topid,find_by_uname='max4')        ; Maxmum Pressure
   p[37]=WIDGET_INFO(topid,find_by_uname='max5')        ; Maxmum Polarizer
   p[38]=WIDGET_INFO(topid,find_by_uname='max6')        ; Maxmum Wavelength
;*** Next addresses will be for the base widgets
   nbasepars=6
   basepars=strarr(nbasepars)
   basepars[0]=WIDGET_INFO(widin,find_by_uname='watinfo')      ; Base for water information
   basepars[1]=WIDGET_INFO(widin,find_by_uname='maininfo')     ; Base for main beam information
   basepars[2]=WIDGET_INFO(topid,find_by_uname='mskinfo')      ; Base for mask information
   basepars[3]=WIDGET_INFO(topid,find_by_uname='calinfo')      ; Base for Calibration frame
;*** Last addresses will contain structured arrays
;   widat(66)=WIDGET_INFO(widin,find_by_uname='watlab')       ; Water label
;   widat(67)=WIDGET_INFO(topid,find_by_uname='calfils')     ; Calibration filenames
; *** Get directory and file names
   WIDGET_CONTROL,widat[1],get_value=fpath   ; Directory
   if !version.os eq 'Win32' then separator='\' else separator='/'
   slashlook= strpos(/reverse_search,fpath,separator)
   pathlen=strlen(fpath)
   if slashlook[0] ne pathlen[0] then fpath=fpath+separator
   WIDGET_CONTROL,widat[2],get_value=ffile  ; Saved parameter filename
   ffile_all=fpath+ffile
; *** If loading:
   if strpos(flag,'load') ne -1 then begin
      if !version.os eq 'Win32' then begin
         ffile_all='"'+ffile_all+'"'
         spawn,'dir '+ffile_all,filecheck
      endif else spawn,'ls '+ffile_all,filecheck
      dud=strmatch(filecheck,ffile)
      if dud[0] ne 0 then begin
         WIDGET_CONTROL,widat[0],set_value='ERROR:  Parameter file does not exist'
         free_lun,unit
         return
      endif
      openr,unit,ffile
; *** Read in and set dropbox status
      readf,unit,newndp         ; Number of dropboxes is saved in the file
      for i=0,newndp[0]-1 do begin
         if ~ EOF(unit) then begin
            readf,unit,newline,FORMAT='(I)'
            WIDGET_CONTROL,long(dp[i]),set_droplist_select=newline
         endif
      endfor
; *** Read in and set button status
      readf,unit,newnb         ; Number of buttons is saved in the file
      for i=0,newnb[0]-1 do begin
         if ~ EOF(unit) then begin
            readf,unit,newline,FORMAT='(I)'
            WIDGET_CONTROL,long(b[i]),set_button=newline
            flag=WIDGET_INFO(long(b[i]),/UNAME)
            aw_fitbuttons,long(b[i]),flag
         endif
      endfor
; *** Read in and write parameters to windows
      readf,unit,newnp         ; Number of parameters
      newline=''
      for i=0,newnp[0]-1 do begin
         if ~ EOF(unit) then begin
            readf,unit,newline,FORMAT='(A)'
            WIDGET_CONTROL,long(p[i]),set_value=newline
         endif
      endfor
; *** Update the date
      aw_dateset,topid,'setime'
; *** Update main beam and mask
      aw_mainupd,long(basepars[1])
      aw_setmask,long(basepars[2]),'setmask'
      aw_chfitpars,topid
; *** Update the water
      aw_waterdef,long(basepars[0])
      WIDGET_CONTROL,widat[0],set_value='Window parameters correctly loaded'
   endif
; *** If saving
   if strpos(flag,'save') ne -1 then begin
      openw,unit,ffile
      printf,unit,ndp                              ; Number of dropboxes
      for i=0,ndp-1 do begin
         dpstate=WIDGET_INFO(long(dp[i]),/droplist_select)
         printf,unit,strtrim(string(dpstate),2)
      endfor
      printf,unit,nb                               ; Number of buttons
      for i=0,nb-1 do begin
         bstate=WIDGET_INFO(long(b[i]),/button_set)
         printf,unit,strtrim(string(bstate),2)
      endfor
      printf,unit,np                               ; Number of parameters
      for i=0,np-1 do begin
         WIDGET_CONTROL,long(p[i]),get_value=pstate
         printf,unit,pstate
      endfor
      WIDGET_CONTROL,widat[0],set_value='Window parameters correctly saved'
   endif
; *** Close files and clear units
   close,unit
   free_lun,unit
end

; ----------------------------------------------------------------------------------------------------------------------------
PRO aw_waterdef,widin
;+
;**************************************************************
;******* This updates the water information on a change *******
;**************************************************************
;-

; ** Find the relevant widgets
   aw_rwd,widin,topid					    ; Top base widget
   widat=intarr(6)
   widat[0]=WIDGET_INFO(topid,find_by_uname='umsg')	    ; Message window
   widat[1]=WIDGET_INFO(widin,find_by_uname='watinfo')      ; Base for water information
   widat[2]=WIDGET_INFO(widin,find_by_uname='watlab')       ; Box for water label
   widat[3]=WIDGET_INFO(widin,find_by_uname='watfil')       ; water file box
   widat[4]=WIDGET_INFO(widin,find_by_uname='watdim')       ; Dropbox for water dimensions
   widat[5]=WIDGET_INFO(topid,find_by_uname='mskinfo')      ; Base for mask information
;*** Get relevant information
   WIDGET_CONTROL,widat[3],get_value=newatfil               ; Name of water file
   WIDGET_CONTROL,widat[4],get_value=dims		    ; Labels for dimensions
   dropindex=WIDGET_INFO(long(widat[4]),/DROPLIST_SELECT)   ; No. of dimensions
   WIDGET_CONTROL,widat[5],get_uvalue=maskstr               ; The current mask string
;*** Run d17_waterdef
   if strcmp(newatfil[0],'File number') ne 1 then begin
      newater=call_function('d17_waterdef',newatfil[0],mask=maskstr[0],dims=dropindex+1)
      TAKE_DATP,nwdatp
;*** Write information to the watinfo base widget
      WIDGET_CONTROL,widat[1],set_uvalue=newater	       ; Store in the Base widget
      WIDGET_CONTROL,widat[2],set_uvalue=nwdatp                ; Store in the Label widget
;*** Write information to the User Information window
      WIDGET_CONTROL,widat[0],set_value='Water successfully loaded'
   endif
end

; ----------------------------------------------------------------------------------------------------------------------------
PRO aw_mainupd,widin
;+
;******************************************************************
;******* This updates the main beam information on a change *******
;******************************************************************
;-

; ** Find the relevant widgets
   aw_rwd,widin,topid					    ; Top base widget
   widat=intarr(5)
   widat[0]=WIDGET_INFO(topid,find_by_uname='umsg')	    ; Message window
   widat[1]=WIDGET_INFO(widin,find_by_uname='maininfo')     ; Base for main beam information
   widat[2]=WIDGET_INFO(widin,find_by_uname='mainlab')      ; Box for main beam label
   widat[3]=WIDGET_INFO(widin,find_by_uname='mainfil')      ; Box for main beam file name
;*** Get relevant information
   WIDGET_CONTROL,topid,get_uvalue=topvars
   path=topvars.path
   WIDGET_CONTROL,widat[3],get_value=newmbfil
;*** Check whether the file exists
   aw_filexist,path,newmbfil,exist
   if exist eq 1 then begin
;*** Set the new file name
   	WIDGET_CONTROL,widat[1],set_uvalue=newmbfil
;*** Send message to user window
   	WIDGET_CONTROL,widat[0],set_value='New main beam file successfully loaded'
   endif else begin
   	WIDGET_CONTROL,widat[1],set_uvalue=0
   	WIDGET_CONTROL,widat[0],set_value='Waiting for new main beam file'
   endelse
end

; ----------------------------------------------------------------------------------------------------------------------------
PRO aw_setmask,widin,flag
;+
;******************************************************
;******* This sets the MASK windows on a change *******
;******************************************************
;*** The information will be stored in the Base widget relevant to the masks
;-

; ** Find the relevant widgets
   aw_rwd,widin,topid					    ; Top base widget
   widat=intarr(13)
   widat[0]=WIDGET_INFO(topid,find_by_uname='umsg')	    ; Message window
   widat[1]=WIDGET_INFO(widin,find_by_uname='mskinfo')      ; Base for mask information
   widat[2]=WIDGET_INFO(widin,find_by_uname='mskf')	    ; Mask droplist
   widat[3]=WIDGET_INFO(widin,find_by_uname='mskpars')      ; Base for mask parameters
   widat[4]=WIDGET_INFO(widin,find_by_uname='mskpar1lab')   ; Par 1 label
   widat[5]=WIDGET_INFO(widin,find_by_uname='mskpar2lab')   ; Par 2 label
   widat[6]=WIDGET_INFO(widin,find_by_uname='mskpar3lab')   ; Par 3 label
   widat[7]=WIDGET_INFO(widin,find_by_uname='mskpar4lab')   ; Par 4 label
   widat[8]=WIDGET_INFO(widin,find_by_uname='mskpar1val')   ; Par 1 value
   widat[9]=WIDGET_INFO(widin,find_by_uname='mskpar2val')   ; Par 2 value
   widat[10]=WIDGET_INFO(widin,find_by_uname='mskpar3val')  ; Par 3 value
   widat[11]=WIDGET_INFO(widin,find_by_uname='mskpar4val')  ; Par 4 value
   widat[12]=WIDGET_INFO(widin,find_by_uname='mskpar4')     ; Base for Pars 4
;*** Determine which mask is to be applied
   dropindex=WIDGET_INFO(long(widat[2]),/DROPLIST_SELECT)
   msk=['none','box','circle','ellipse']
   msk=msk[dropindex]
;*** In case of changing the drop menu
   if strcmp(flag,'setmask') eq 1 then begin
;*** Set the labels for the parameter boxes
      mlab=[['','x-min','x-cen','x-loc'], $
 	    ['','x-max','y-cen','y-loc'], $
            ['','y-min','rad','x-cen'],   $
            ['','y-max','','y-cen']]
;*** Set map for whether the objects are visible or not
      mskmap=[0,1,1,1]
      dmap=[0,1,0,1]
;*** Change labels
      for i=0,3 do WIDGET_CONTROL,widat[i+4],set_value=mlab[dropindex,i]
;*** Set visible
      WIDGET_CONTROL,widat[3],map=mskmap[dropindex]
      WIDGET_CONTROL,widat[12],map=dmap[dropindex]
;*** Send text to Information Window
      WIDGET_CONTROL,widat[0],set_value='Mask defined as '+msk
   endif
;*** Collect information from the parameter windows
   WIDGET_CONTROL,widat[8],get_value=mskp1
   WIDGET_CONTROL,widat[9],get_value=mskp2
   WIDGET_CONTROL,widat[10],get_value=mskp3
   WIDGET_CONTROL,widat[11],get_value=mskp4
;*** Create text to be stored in the base widget
   msktxt='none'
   mskprs=[long(mskp1),long(mskp2),long(mskp3),long(mskp4)]
   case dropindex of
      0: msktxt='none'
      1: begin
	     if total(mskprs) ne 0 then msktxt=msk+',['+mskp1+','+mskp2+','+mskp3+','+mskp4+']'
	     if total(mskprs[2:3]) ne 0 then msktxt=msk+',['+mskp1+','+mskp2+']'

;            if (mskprs(0) ne 0)and(mskprs(1) ne 0)and(mskprs(2) ne 0) and (mskprs(3) ne 0) then msktxt=msk+',['+mskp1+','+mskp2+','+mskp3+','+mskp4+']'
;            if (mskprs(0) ne 0)and(mskprs(1) ne 0)and(mskprs(2) eq 0) and (mskprs(3) eq 0) then msktxt=msk+',['+mskp1+','+mskp2+']'
         end
      2: if (mskprs[0] ne 0)and(mskprs[1] ne 0)and(mskprs[2] ne 0) then msktxt=msk+',['+mskp1+','+mskp2+','+mskp3+']'
      3: if (mskprs[0] ne 0)and(mskprs[1] ne 0)and(mskprs[2] ne 0) and (mskprs[3] ne 0) then msktxt=msk+',['+mskp1+','+mskp2+','+mskp3+','+mskp4+']'
   endcase

;*** Now write mask information to the base widget
   WIDGET_CONTROL,widat[1],get_uvalue=oldmsktxt
   if strcmp(oldmsktxt,msktxt) ne 1 then begin
      WIDGET_CONTROL,widat[1],set_uvalue=msktxt
      mgtxt=string('New mask will be '+msktxt)
      WIDGET_CONTROL,widat[0],set_value=mgtxt[0]
   endif
end

; ----------------------------------------------------------------------------------------------------------------------------
PRO aw_calibrate,widin
;+
;****************************************************************************************************************************
;******* On clicking 'Load files' in the 'Calibration window', collects information and passes it to d17_he3calibrate *******
;****************************************************************************************************************************
;-

; ** Find the address for the top widget
   aw_rwd,widin,topid
;*** Get the parameters corresponding to the last time calibration was run
   WIDGET_CONTROL,topid,get_uvalue=lastcal
;*** Find all the relevant information in the new windows
   widat=intarr(21)
   widat[0]=WIDGET_INFO(topid,find_by_uname='umsg')	 ; Message window
   widat[1]=WIDGET_INFO(topid,find_by_uname='watinfo')   ; Base for water information
   widat[2]=WIDGET_INFO(topid,find_by_uname='maininfo')  ; Base for mainbeam information
   widat[3]=WIDGET_INFO(topid,find_by_uname='mskinfo')   ; Base for mask information
   widat[4]=WIDGET_INFO(topid,find_by_uname='calfils')   ; POLCHECK run numbers
   widat[5]=WIDGET_INFO(topid,find_by_uname='f1inp')     ; Include F1 in he3calibrate?
   widat[6]=WIDGET_INFO(topid,find_by_uname='f2inp')     ; Include F2 in he3calibrate?
   widat[7]=WIDGET_INFO(topid,find_by_uname='calinfo')   ; Base for calibration information
   widat[8]=WIDGET_INFO(topid,find_by_uname='caldat')    ; Base for POLCHECK files
   widat[9]=WIDGET_INFO(topid,find_by_uname='f1val')     ; F1 value
   widat[10]=WIDGET_INFO(topid,find_by_uname='f1valpm')  ; F1 error
   widat[11]=WIDGET_INFO(topid,find_by_uname='f2val')    ; F2 value
   widat[12]=WIDGET_INFO(topid,find_by_uname='f2valpm')  ; F2 error
   widat[13]=WIDGET_INFO(topid,find_by_uname='parvals6')   ; Wavelength
   widat[14]=WIDGET_INFO(topid,find_by_uname='fitinfo')  ; Base for fit data
   widat[15]=WIDGET_INFO(topid,find_by_uname='hrv')      ; Hour for start time
   widat[16]=WIDGET_INFO(topid,find_by_uname='minv')     ; Minute for start time
   widat[17]=WIDGET_INFO(topid,find_by_uname='dayv')     ; Day for start time
   widat[18]=WIDGET_INFO(topid,find_by_uname='monv')     ; Month for start time
   widat[19]=WIDGET_INFO(topid,find_by_uname='yrv')      ; Year for start time
   widat[20]=WIDGET_INFO(topid,find_by_uname='parfx1')   ; 'Fix' button for date
;*** Get relevant information
   WIDGET_CONTROL,widat[1],get_uvalue=watinfo            ; Water workspace, if required
   WIDGET_CONTROL,widat[2],get_uvalue=maininfo           ; Main beam file, if required
   WIDGET_CONTROL,widat[3],get_uvalue=mskinfo            ; Mask string, if required
   WIDGET_CONTROL,widat[4],get_value=calfils             ; New POLCHECK files
   f1drpind=WIDGET_INFO(long(widat[5]),/DROPLIST_SELECT) ; Fix F1?
   f2drpind=WIDGET_INFO(long(widat[6]),/DROPLIST_SELECT) ; Fix F2?
   WIDGET_CONTROL,widat[9],get_value=ff1                 ; F1 value
   WIDGET_CONTROL,widat[10],get_value=eff1               ; F1 error
   WIDGET_CONTROL,widat[11],get_value=ff2                ; F2 value
   WIDGET_CONTROL,widat[12],get_value=eff2               ; F2 error
;*** Write information to the User Information window
   WIDGET_CONTROL,widat[0],set_value='He3 calibration en route.  Refer to IDL window for more information'
;*** Make sure that 'calfils' is a one-element string array
   if n_elements(calfils) gt 1 then begin
      ncalfils=calfils[0]
      for i=1,n_elements(calfils)-1 do begin
         ncalfils=ncalfils+','+calfils[i]
      endfor
;*** Check that first and last characters are not commas
      lastcomma=strpos(ncalfils,',',/REVERSE_SEARCH)
      if lastcomma eq strlen(ncalfils)-1 then ncalfils=strmid(ncalfils,0,lastcomma)
      firstcomma=strpos(ncalfils,',')
      if firstcomma eq 0 then ncalfils=strmid(ncalfils,1,n_elements(ncalfils)-1)
      calfils=ncalfils[0]
   endif
;*** Check whether F1 and F2 are to be forced
   if f1drpind eq 1 then ff1=[double(ff1),double(eff1)]
   if f2drpind eq 1 then ff2=[double(ff2),double(eff2)]
;*** Determine whether new calibration is needed
   newcal=0
   if array_equal(watinfo,lastcal.waterfile) ne 1 then newcal=1
   if array_equal(maininfo,lastcal.mainfile) ne 1 then newcal=1
   if array_equal(f1drpind,lastcal.f1inp) ne 1 then newcal=1
   if array_equal(f2drpind,lastcal.f2inp) ne 1 then newcal=1
   if strcmp(calfils,lastcal.calfils) ne 1 then newcal=1
   if strcmp(mskinfo,lastcal.mask) ne 1 then newcal=1
;*** Run d17_he3calibrate
   if newcal eq 1 then begin
;*** Update the wavelength in the Fit Control boxes
      mdp=1
      if maininfo ne 0 then begin
         main=rdrun(long(maininfo),datp=mdp)
         WIDGET_CONTROL,widat[13],set_value=strtrim(string(mdp.p[24]),1)
      endif
      new3he=call_function('d17_he3calibrate',calfils,main=maininfo,mask=mskinfo,water=watinfo,F1=ff1,F2=ff2)
      TAKE_DATP,nw3hdatp
;*** Write information to the calinfo and caldat base widget
      WIDGET_CONTROL,widat[7],set_uvalue=new3he		    ; Store in the Base widget
      WIDGET_CONTROL,widat[8],set_uvalue=nw3hdatp           ; Store in the Label widget
;*** Write the F1 and F2 values to the appropriate locations
      WIDGET_CONTROL,widat[9],set_value=strtrim(string(new3he[0,5]),1)
      WIDGET_CONTROL,widat[10],set_value=strtrim(string(nw3hdatp.e[0,5]),1)
      WIDGET_CONTROL,widat[11],set_value=strtrim(string(new3he[0,6]),1)
      WIDGET_CONTROL,widat[12],set_value=strtrim(string(nw3hdatp.e[0,6]),1)
;*** Estimate the start time, if the 'fix' button is not selected
      datesel=WIDGET_INFO(widat[20],/button_set)
      if datesel eq 0 then begin
         starttime=nw3hdatp.x[0]
         caldat,starttime,stmonth,stday,styear,sthour,stmin
         WIDGET_CONTROL,widat[15],set_value=strtrim(string(sthour),1)
         WIDGET_CONTROL,widat[16],set_value=strtrim(string(stmin),1)
         WIDGET_CONTROL,widat[17],set_value=strtrim(string(stday),1)
         WIDGET_CONTROL,widat[18],set_value=strtrim(string(stmonth),1)
         WIDGET_CONTROL,widat[19],set_value=strtrim(string(styear),1)
      endif
;*** Write information to top base macro
      caldata=[[nw3hdatp.x],[new3he[*,3:4]],[nw3hdatp.e[*,3:4]]]
      newcal={waterfile:watinfo, $
           mainfile:maininfo, $
           mask:mskinfo, $
           calfils:calfils, $
           f1inp:f1drpind, $
           f2inp:f2drpind, $
           caldata:caldata, $
           plotwin:lastcal.plotwin, $
           fitpars:lastcal.fitpars, $
           fitdata:lastcal.fitdata}
      WIDGET_CONTROL,topid,set_uvalue=newcal
;*** Activate the fit window
      WIDGET_CONTROL,widat[14],SENSITIVE=1
;*** Write information to the User Information window
      WIDGET_CONTROL,widat[0],set_value='He3 calibration successfully concluded'
   endif else begin
       WIDGET_CONTROL,widat[0],set_value='No changes since last calibration.  Old 3He values kept'
   endelse
end

; ----------------------------------------------------------------------------------------------------------------------------
PRO aw_dateset,widin,flag
;+
;*********************************************************************************
;******* Collects the date information and updates the appropriate windows *******
;*********************************************************************************
;-

; ** Find the address for the top widget
   aw_rwd,widin,topid
;*** Find the relevant information
   widat=intarr(6)
   widat[0]=WIDGET_INFO(topid,find_by_uname='parvals1')
   widat[1]=WIDGET_INFO(topid,find_by_uname='hrv')
   widat[2]=WIDGET_INFO(topid,find_by_uname='minv')
   widat[3]=WIDGET_INFO(topid,find_by_uname='dayv')
   widat[4]=WIDGET_INFO(topid,find_by_uname='monv')
   widat[5]=WIDGET_INFO(topid,find_by_uname='yrv')
   if strcmp(flag,'setime') eq 1 then begin
      WIDGET_CONTROL,widat[1],get_value=hour
      WIDGET_CONTROL,widat[2],get_value=minute
      WIDGET_CONTROL,widat[3],get_value=day
      WIDGET_CONTROL,widat[4],get_value=month
      WIDGET_CONTROL,widat[5],get_value=year
      WIDGET_CONTROL,widat[0],set_uvalue=julday(double(month),double(day),double(year),double(hour),double(minute),0.0)
   endif else begin
      WIDGET_CONTROL,widat[0],get_uvalue=datenum
      caldat,datenum,month,day,year,hour,minute,second
      WIDGET_CONTROL,widat[1],set_value=strtrim(string(hour),1)
      WIDGET_CONTROL,widat[2],set_value=strtrim(string(minute),1)
      WIDGET_CONTROL,widat[3],set_value=strtrim(string(day),1)
      WIDGET_CONTROL,widat[4],set_value=strtrim(string(month),1)
      WIDGET_CONTROL,widat[5],set_value=strtrim(string(year),1)
   endelse
end

; ----------------------------------------------------------------------------------------------------------------------------
PRO aw_fitbuttons,widin,flag
;+
;**************************************************
;******* Fits the 3He data using d17_he3fit *******
;**************************************************
;-

; ** Find the address for the top widget
   aw_rwd,widin,topid
;*** Find the unames for the relevant fields
   parnum=strmid(flag,5,6)
   pfxid='parfx'+parnum
   pid='parvals'+parnum
   lfxid='limfx'+parnum
   minid='min'+parnum
   maxid='max'+parnum
;*** The flag contains all the information necessary to set the sensitivity
   widat=intarr(5)
   widat[0]=WIDGET_INFO(topid,find_by_uname=pfxid)
   widat[1]=WIDGET_INFO(topid,find_by_uname=pid)
   widat[2]=WIDGET_INFO(topid,find_by_uname=lfxid)
   widat[3]=WIDGET_INFO(topid,find_by_uname=minid)
   widat[4]=WIDGET_INFO(topid,find_by_uname=maxid)
;*** Determine what the state of the clicked button is
   buttonset=WIDGET_INFO(widin,/BUTTON_SET)
;*** Determine which fields to sensitize
   if widat[0] eq widin then begin
      if buttonset eq 1 then buttonset = 0 else buttonset = 1
      for i=1,4 do WIDGET_CONTROL,widat[i],SENSITIVE=buttonset
   endif
   if widat[2] eq widin then begin
      for i=3,4 do WIDGET_CONTROL,widat[i],SENSITIVE=buttonset
   endif
end

; ----------------------------------------------------------------------------------------------------------------------------
PRO aw_chfitpars,widin
;+
;******************************************************
;******* Update the uvalue field for top widget *******
;******************************************************
;-

; ** Find the address for the top widget
   aw_rwd,widin,topid
;*** Initialize fitparameters
   fpars=dblarr(2,8)
;*** Find the date
   widat=WIDGET_INFO(topid,find_by_uname='parvals1')
   WIDGET_CONTROL,widat,get_uvalue=tval
   widat=WIDGET_INFO(topid,find_by_uname='parerr1')
   WIDGET_CONTROL,widat,get_value=teval
   fpars[*,0]=[double(tval),double(teval)]
;*** Find the 3He parameters
   for i=2,6 do begin
      widat=WIDGET_INFO(topid,find_by_uname='parvals'+strtrim(string(i),2))     ; Parameter values
      WIDGET_CONTROL,widat,get_value=tval
      widat=WIDGET_INFO(topid,find_by_uname='parerr'+strtrim(string(i),2))      ; Error values
      WIDGET_CONTROL,widat,get_value=teval
      fpars[*,i-1]=[double(tval),double(teval)]
   endfor
   widat=WIDGET_INFO(topid,find_by_uname='f1val')   ; F1 value
   WIDGET_CONTROL,widat,get_value=tval
   widat=WIDGET_INFO(topid,find_by_uname='f1valpm') ; F1 error
   WIDGET_CONTROL,widat,get_value=teval
   fpars[*,6]=[double(tval),double(teval)]
   widat=WIDGET_INFO(topid,find_by_uname='f2val')   ; F2 value
   WIDGET_CONTROL,widat,get_value=tval
   widat=WIDGET_INFO(topid,find_by_uname='f2valpm') ; F2 error
   WIDGET_CONTROL,widat,get_value=teval
   fpars[*,7]=[double(tval),double(teval)]
; *** Find and set the field of the structured array in the base widget
   WIDGET_CONTROL,topid,get_uvalue=tlbuval
   tlbuval.fitpars=fpars
   WIDGET_CONTROL,topid,set_uvalue=tlbuval
end


; ----------------------------------------------------------------------------------------------------------------------------
PRO aw_fit,widin
;+
;**************************************************
;******* Fits the 3He data using d17_he3fit *******
;**************************************************
;-

; ** Find the address for the top widget
   aw_rwd,widin,topid
;*** Find the windows that contain the required input:
   widat=lonarr(40)
   widat[0]=WIDGET_INFO(topid,find_by_uname='umsg')	 ; Message window
   widat[1]=WIDGET_INFO(topid,find_by_uname='calinfo')   ; Base for calibration information: contains calibration data
   widat[2]=WIDGET_INFO(topid,find_by_uname='caldat')    ; Base for POLCHECK files: contains calibration datp
   for i=3,8 do widat[i]=WIDGET_INFO(topid,find_by_uname='parvals'+strtrim(string(i-2),2))     ; Parameter values
   for i=9,14 do widat[i]=WIDGET_INFO(topid,find_by_uname='parfx'+strtrim(string(i-8),2))      ; Tells what parameters are fixed
   for i=15,20 do widat[i]=WIDGET_INFO(topid,find_by_uname='limfx'+strtrim(string(i-14),2))      ; Tells what limits are fixed
   for i=21,26 do widat[i]=WIDGET_INFO(topid,find_by_uname='min'+strtrim(string(i-20),2))       ; Minimum for limits
   for i=27,32 do widat[i]=WIDGET_INFO(topid,find_by_uname='max'+strtrim(string(i-26),2))       ; Maximum for limits
   for i=33,38 do widat[i]=WIDGET_INFO(topid,find_by_uname='parerr'+strtrim(string(i-32),2))    ; Error values
   widat[39]=WIDGET_INFO(topid,find_by_uname='fitcrss')	 ; Check for crossed polarizer-analyser
;*** Write information to the User Information window
      WIDGET_CONTROL,widat[0],set_value='Fit en route.  Refer to IDL window for more information'
;*** Collect the relevant information
   WIDGET_CONTROL,widat[1],get_uvalue=caldata            ; calibration data
   WIDGET_CONTROL,widat[2],get_uvalue=caldatp            ; calibration datp
;*** Collect the input parameters
   pin=dblarr(6)
   WIDGET_CONTROL,widat[3],get_uvalue=ptemp
   pin[0]=ptemp
   for i=4,8 do begin
      WIDGET_CONTROL,widat[i],get_value=ptemp
      pin[i-3]=ptemp
   endfor
;*** Collect the fix information
   pfx=intarr(6)
   for i=9,14 do pfx[i-9]=WIDGET_INFO(widat[i],/BUTTON_SET)
;*** Collect the limit fix information
   lfx=intarr(6)
   for i=15,20 do lfx[i-15]=WIDGET_INFO(widat[i],/BUTTON_SET)
;*** Collect the limits
   lim=dblarr(2,6)
   for i=21,26 do begin
      WIDGET_CONTROL,widat[i],get_value=ptemp
      lim[0,i-21]=ptemp
   endfor
   for i=27,32 do begin
      WIDGET_CONTROL,widat[i],get_value=ptemp
      lim[1,i-27]=ptemp
   endfor
;*** Collect whether the polarizers are crossed
   cross_sel=WIDGET_INFO(widat[39],/DROPLIST_SELECT)
;*** Find where limits are selected, ignore the rest
   limin=-1.0+0.0*lim
   lfx=where(lfx gt 0,count)
   if count gt 0 then limin[*,lfx]=lim[*,lfx]
;*** Do the fit
   d17_he3fit,caldata,pin,pout,yout,HDATP=caldatp,PFIX=pfx,LIMITS=limin,CROSS=cross_sel
;*** Write the output to the base widget
   WIDGET_CONTROL,topid,get_uvalue=lastfit
;*** Write information to top base macro
   youtsize=size(yout,/DIMENSIONS)
   newfit={waterfile:lastfit.waterfile, $
           mainfile:lastfit.mainfile, $
           mask:lastfit.mask, $
           calfils:lastfit.calfils, $
           f1inp:lastfit.f1inp, $
           f2inp:lastfit.f2inp, $
           caldata:lastfit.caldata, $
           plotwin:lastfit.plotwin, $
           fitpars:pout, $
           fitdata:yout}
   WIDGET_CONTROL,topid,set_uvalue=newfit
;*** Write the new parameters back in to the input fields
   WIDGET_CONTROL,widat[3],set_uvalue=pout[0,0]
   aw_dateset,widat[3],'setdate'
   WIDGET_CONTROL,widat[33],set_value=strtrim(string(pout[1,0]),1)
   for i=4,8 do begin
      WIDGET_CONTROL,widat[i],set_value=strtrim(string(pout[0,i-3]),1)
      WIDGET_CONTROL,widat[30+i],set_value=strtrim(string(pout[1,i-3]),1)
   endfor
   print,'Copy the following line to your batch macro for the 3He parameters'
   pstring='p=['
   for i=0,7 do begin
      pline='['+strtrim(string(pout[0,i]),1)+','+strtrim(string(pout[1,i]),1)+']'
      pstring=pstring+pline
      if (i lt 7) then pstring=pstring+','
   endfor
   pstring=pstring+']'
   print,pstring
;*** Send completion message
   WIDGET_CONTROL,widat[0],set_value='Fit completed. Refer to IDL window for polarization parameter line'
end


; ----------------------------------------------------------------------------------------------------------------------------
PRO aw_plotwinhide,event
;+
;****************************
;*** Hide the plot window ***
;****************************
;-
   WIDGET_CONTROL,event.top,MAP=0
end

; ----------------------------------------------------------------------------------------------------------------------------
PRO aw_plotwinshow,tlb,plotwin
;+
;******************************
;*** Create the plot window ***
;******************************
;-
   plotwin=WIDGET_BASE(/base_align_center,/COLUMN,GROUP_LEADER=tlb,uvalue=tlb)
   plotpol=WIDGET_DRAW(plotwin,xsize=500,ysize=300,uname='plotpol')
   plottrans=WIDGET_DRAW(plotwin,xsize=500,ysize=300,uname='plottrans')
   plotclose=WIDGET_BUTTON(plotwin,value='Close',EVENT_PRO='aw_plotwinhide')
   WIDGET_CONTROL,plotwin,/REALIZE,MAP=0
end

; ----------------------------------------------------------------------------------------------------------------------------
PRO aw_plotupd,plotwin,caldata
;+
;******************************
;*** Update the plot window ***
;******************************
;-

; ** Get the relevant addresses for the DRAW widgets

   plotpol=WIDGET_INFO(plotwin,find_by_uname='plotpol')
   WIDGET_CONTROL,plotpol,get_value=polid
   plottrans=WIDGET_INFO(plotwin,find_by_uname='plottrans')
   WIDGET_CONTROL,plottrans,get_value=transid
;*** Get the data from the structured array
   data=caldata.caldata
   fit=caldata.fitdata
   fpars=caldata.fitpars
;*** Make the x-axis in hours, assume that we haven't measured for more than a month!
   if fpars[0,0] ne 0 then time=data[*,0]-fpars[0,0] else time=data[*,0]-data[0,0]
   caldat,time,mm1,dd1,yy1,hh1,mn1,ss1
   caldat,0,mm0,dd0,yy0,hh0,mn0,ss0
   time=(dd1-dd0)*24.0+(hh1-hh0)+(mn1-mn0)/60.0+(ss1-ss0)/3600.0
;*** Plot the polarization

   wset,polid
   plot,time,data[*,1],title='Transmission (good neutrons)',ytitle='Transmission',xtitle='Hours',psym=2,$
      xrange=[0,max(time)+1],yrange=[0.99*min(data[*,1]),1.01*max(data[*,1])]
   errplot,time,data[*,1]-data[*,3],data[*,1]+data[*,3]
   oplot,time,fit[*,1],color=200
;*** Plot the transmission
   wset,transid
   plot,time,data[*,2],title='Polarizer-analyser inefficiency',ytitle='Polarization',xtitle='Hours',psym=2,$
      xrange=[0,max(time)+1],yrange=[0.99*min(data[*,2]),1.01*max(data[*,2])]
   errplot,time,data[*,2]-data[*,4],data[*,2]+data[*,4]
   oplot,time,fit[*,0],color=200  
end

; ----------------------------------------------------------------------------------------------------------------------------
PRO d17_he3cal_gui
;+
;*********************************************
;** User callable
;******* This is the D17_HE3CAL widget *******
;*********************************************
;-

; *** Find the path that LAMP is currently using, as defined by the current
;	path for lamp
   @lamp.cbk
   path=path_for_online
;   if !version.os eq 'Win32' then spawn,'cd',workdir else spawn,'pwd',workdir

;***    Specifying the top level base widget and its properties
   tlb=WIDGET_BASE(TITLE="D17 3He CALIBRATION",/FRAME,SPACE=5,/COLUMN,$
         /ALIGN_CENTER,XOFFSET=50,MBAR=menubar,$
         uvalue={waterfile:0L,$
                 mainfile:0L,$
                 mask:'none',$
                 calfils:'e.g. P1_file1:P1:file2,P2_file1:P2_file2',$
                 f1inp:0D,f2inp:0D, $
                 caldata:dblarr(1,4), $
                 plotwin:0L, $
                 fitpars:dblarr(2,8),fitdata:dblarr(1,2),$
		 path:path})
   WIDGET_CONTROL,tlb,get_uvalue=tlbuval

;******* MENU BAR *******
; *** Name of the menu (File)
   fileID=WIDGET_BUTTON(menubar,VALUE="File",/MENU)
; *** Submenu of File
   openID=WIDGET_BUTTON(fileID,VALUE="Open",uvalue='open')
   saveID=WIDGET_BUTTON(fileID,VALUE="Save",uvalue='save')
   exitID=WIDGET_BUTTON(fileID,VALUE="Exit",uvalue='quit')
; *** Name of the menu (Plot)
   plotID=WIDGET_BUTTON(menubar,VALUE="Plot",/MENU)
; *** Submenu of Plot
   plotmeID=WIDGET_BUTTON(plotID,VALUE="Plot Results",uvalue='plotshow')
; *** Name of the menu (Help)
   helpID=WIDGET_BUTTON(menubar,VALUE="Help",/MENU)
; *** Submenu of Help
   contentID=WIDGET_BUTTON(helpID,VALUE="Help",uvalue='help')
   aboutID=WIDGET_BUTTON(helpID,VALUE="About",uvalue='about')

;******* FILE BOX *******
   finfo=WIDGET_BASE(tlb,/ROW,/FRAME)
; *** Text box showing the current file
      fdirlab=WIDGET_LABEL(finfo,VALUE='Work directory:')
      fdirtxt=WIDGET_TEXT(finfo,XSIZE=30,/EDITABLE,VALUE=workdir,UNAME='fdirtxt')
      filelab=WIDGET_LABEL(finfo,VALUE='File:')
      fiload=WIDGET_BUTTON(finfo,VALUE='Load',uname='fiload',uvalue='open')
      filetxt=WIDGET_TEXT(finfo,/EDITABLE,VALUE='File Name',UNAME='filetxt')
      filsav=WIDGET_BUTTON(finfo,VALUE='Save',uname='filsav',uvalue='save')

;******* USER INFORMATION BOX *******
; *** Creating a label box to display messages to the user
   userinfo=WIDGET_BASE(tlb,SPACE=20,/ROW,/FRAME)
      umsg=WIDGET_LABEL(userinfo,uname='umsg',/DYNAMIC_RESIZE,/ALIGN_CENTER,VALUE='Welcome to the D17 3He calibration routine')

;******* OPTIONAL INPUT BOX *******
   ainfo=WIDGET_BASE(tlb,/COLUMN)
      atxt=WIDGET_LABEL(ainfo,value='Optional input')
; *** Create the Water box
      watinfo=WIDGET_BASE(ainfo,uname='watinfo',uvalue=0D,/ROW)
         watlab=WIDGET_LABEL(watinfo,uname='watlab',value='Water run:')
         watfil=WIDGET_TEXT(watinfo,/EDITABLE,value='File number',uname='watfil',uvalue='watfil')
         watdim=WIDGET_DROPLIST(watinfo,uname='watdim',title='Final Dimensions',value=['1','2'],uvalue='watfil')
; *** Create the Main Beam box
      maininfo=WIDGET_BASE(ainfo,uname='maininfo',uvalue=0D,/ROW)
         mainlab=WIDGET_LABEL(maininfo,value='Main beam:')
         mainfil=WIDGET_TEXT(maininfo,/EDITABLE,value='File number',uname='mainfil',uvalue='mainfil',/ALL_EVENTS)
; *** Create the Mask box
      mskinfo=WIDGET_BASE(ainfo,uname='mskinfo',uvalue='none',/ROW)
         mskf=WIDGET_DROPLIST(mskinfo, $
		uname='mskf', $
		TITLE='Mask option', $
		value=['None','Box','Circle','Ellipse'], $
		uvalue='setmask')
         mskpars=WIDGET_BASE(mskinfo,uname='mskpars',/ROW)
            mskpar1lab=WIDGET_LABEL(mskpars,uname='mskpar1lab')
            mskpar1val=WIDGET_TEXT(mskpars,uname='mskpar1val',uvalue='changemask',XSIZE=7,/EDITABLE,/ALL_EVENTS)
            mskpar2lab=WIDGET_LABEL(mskpars,uname='mskpar2lab')
            mskpar2val=WIDGET_TEXT(mskpars,uname='mskpar2val',uvalue='changemask',XSIZE=7,/EDITABLE,/ALL_EVENTS)
            mskpar3lab=WIDGET_LABEL(mskpars,uname='mskpar3lab')
            mskpar3val=WIDGET_TEXT(mskpars,uname='mskpar3val',uvalue='changemask',XSIZE=7,/EDITABLE,/ALL_EVENTS)
	    mskpar4=WIDGET_BASE(mskpars,uname='mskpar4',/ROW)
               mskpar4lab=WIDGET_LABEL(mskpar4,uname='mskpar4lab')
               mskpar4val=WIDGET_TEXT(mskpar4,uname='mskpar4val',uvalue='changemask',XSIZE=7,/EDITABLE,/ALL_EVENTS)

;******* CALIBRATE FILES BOX *******
   calinfo=WIDGET_BASE(tlb,uname='calinfo',uvalue=0D,/COLUMN,/FRAME)
; *** Create POLCHECK box
      caltxt=WIDGET_LABEL(calinfo,value='Enter POLCHECKS (separators ":" if consecutive runs, "," if non-consecutive )',uvalue='None')
      caldat=WIDGET_BASE(calinfo,uname='caldat',uvalue=0D,/ROW)
         calfils=WIDGET_TEXT(caldat,uname='calfils',value='e.g. P1_file1:P1:file2,P2_file1:P2_file2',XSIZE=100,YSIZE=2,/EDITABLE,uvalue='none')
; *** Create LOAD button
         calgo=WIDGET_BUTTON(caldat,value='Load files',uvalue='d17_calibrate')
; *** Create buttons to fix F1, F2 inputs
      flpdat=WIDGET_BASE(calinfo,/COLUMN,/FRAME)
         f1info=WIDGET_BASE(flpdat,uvalue=[0,0],/ROW)
            f1lab=WIDGET_LABEL(f1info,value='F1=')
            f1val=WIDGET_TEXT(f1info,uname='f1val',value='0',/EDITABLE,/SENSITIVE)
            f1labpm=WIDGET_LABEL(f1info,value='+/-')
            f1valpm=WIDGET_TEXT(f1info,uname='f1valpm',/EDITABLE,/SENSITIVE,value='0',uvalue='none')
            f1inp=WIDGET_DROPLIST(f1info,uname='f1inp',value=['Calculate','Fix'],uvalue='f1update')
         f2info=WIDGET_BASE(flpdat,uvalue=[0,0],/ROW)
            f2lab=WIDGET_LABEL(f2info,value='F2=')
            f2val=WIDGET_TEXT(f2info,uname='f2val',value='0',/EDITABLE,/SENSITIVE)
            f2labpm=WIDGET_LABEL(f2info,value='+/-')
            f2valpm=WIDGET_TEXT(f2info,uname='f2valpm',/EDITABLE,/SENSITIVE,value='0',uvalue='none')
            f2inp=WIDGET_DROPLIST(f2info,uname='f2inp',value=['Calculate','Fix'],uvalue='f2update')

;******* FIT PARAMETERS BOX *********
   fitinfo=WIDGET_BASE(tlb,uname='fitinfo',/COLUMN,/FRAME,/SENSITIVE)
;*** Create 'cross' option and 'Launch fit' button
      fithd=WIDGET_BASE(fitinfo,/ROW)
         fititle=WIDGET_LABEL(fithd,value='Fit Control')
         fitcrss=WIDGET_DROPLIST(fithd,uname='fitcrss',value=['3He & polarizer parallel','3He & polarizer antiparallel'],uvalue='none')
         fitgo=WIDGET_BUTTON(fithd,value='Start fit',uvalue='he3fit')
;*** The following table consists of 7 bases in column formats, each with 5 bases inside
;*** First base has labels
      btop=lonarr(7)
      bsub=lonarr(5,7)
      btop[0]=WIDGET_BASE(fitinfo,/ROW)
         bsub[0,0]=WIDGET_BASE(btop[0],/ROW)
            phead=WIDGET_LABEL(bsub[0,0],value='Fit parameters (click to fix)',/DYNAMIC_RESIZE)
         bsub[1,0]=WIDGET_BASE(btop[0],/ROW)
         bsub[2,0]=WIDGET_BASE(btop[0],/ROW)
            ehead=WIDGET_LABEL(bsub[2,0],value='Errors:')
         bsub[3,0]=WIDGET_BASE(btop[0],/ROW)
            lhead=WIDGET_LABEL(bsub[3,0],value='Lims on/off')
         bsub[4,0]=WIDGET_BASE(btop[0],/ROW)
            mnhd=WIDGET_LABEL(bsub[4,0],value='Min:')
            mxhd=WIDGET_LABEL(bsub[4,0],value='Max:')
;*** Second base has the Date information
      btop[1]=WIDGET_BASE(fitinfo,/ROW)
         bsub[0,1]=WIDGET_BASE(btop[1],/ROW,/NONEXCLUSIVE)
            datelab=WIDGET_BUTTON(bsub[0,1],uname='parfx1',value='Start time:',uvalue='parfx1')
         bsub[1,1]=WIDGET_BASE(btop[1],/ROW,/SENSITIVE)
            datevals=WIDGET_BASE(bsub[1,1],uname='parvals1',uvalue=0D,/ROW,/SENSITIVE)
               hrv=WIDGET_TEXT(datevals,uname='hrv',value='hh',uvalue='setime',XSIZE=2,/EDITABLE,/ALL_EVENTS)
               sep=WIDGET_LABEL(datevals,value=':')
               minv=WIDGET_TEXT(datevals,uname='minv',value='mm',uvalue='setime',XSIZE=3,/EDITABLE,/ALL_EVENTS)
               sep=WIDGET_LABEL(datevals,value='on')
               dayv=WIDGET_TEXT(datevals,uname='dayv',value='dd',uvalue='setime',XSIZE=2,/EDITABLE,/ALL_EVENTS)
               sep=WIDGET_LABEL(datevals,value='/')
               monv=WIDGET_TEXT(datevals,uname='monv',value='mm',uvalue='setime',XSIZE=3,/EDITABLE,/ALL_EVENTS)
               sep=WIDGET_LABEL(datevals,value='/')
               yrv=WIDGET_TEXT(datevals,uname='yrv',value='yyyy',uvalue='setime',XSIZE=4,/EDITABLE,/ALL_EVENTS)
         bsub[2,1]=WIDGET_BASE(btop[1],/ROW)
            daterr=WIDGET_TEXT(bsub[2,1],value='0',uname='parerr1')
         bsub[3,1]=WIDGET_BASE(btop[1],uvalue=0L,uname='lfx1',/COLUMN,/NONEXCLUSIVE,/SENSITIVE)
            ldatfx=WIDGET_BUTTON(bsub[3,1],value=' ',uname='limfx1',uvalue='limfx1')
         bsub[4,1]=WIDGET_BASE(btop[1],uname='lims1',/ROW,/SENSITIVE)
            mindat=WIDGET_TEXT(bsub[4,1],XSIZE=5,uname='min1',uvalue='none',/EDITABLE,/ALL_EVENTS)
            maxdat=WIDGET_TEXT(bsub[4,1],XSIZE=5,uname='max1',uvalue='none',/EDITABLE,/ALL_EVENTS)
;*** Third base has the Initial Polarization information
      btop[2]=WIDGET_BASE(fitinfo,/ROW)
         bsub[0,2]=WIDGET_BASE(btop[2],/ROW,/NONEXCLUSIVE)
             pollab=WIDGET_BUTTON(bsub[0,2],uname='parfx2',value='Initial 3He pol (%):',uvalue='parfx2')
         bsub[1,2]=WIDGET_BASE(btop[2],/ROW,/SENSITIVE)
             polvals=WIDGET_TEXT(bsub[1,2],uname='parvals2',value='0',uvalue='none',/EDITABLE,/SENSITIVE,/ALL_EVENTS)
         bsub[2,2]=WIDGET_BASE(btop[2],/ROW)
             polerr=WIDGET_TEXT(bsub[2,2],value='0',uname='parerr2')
         bsub[3,2]=WIDGET_BASE(btop[2],uvalue=0L,uname='lfx2',/COLUMN,/NONEXCLUSIVE,/SENSITIVE)
             ldatfx=WIDGET_BUTTON(bsub[3,2],value=' ',uname='limfx2',uvalue='limfx2')
         bsub[4,2]=WIDGET_BASE(btop[2],uname='lims2',/ROW,/SENSITIVE)
             minpol=WIDGET_TEXT(bsub[4,2],XSIZE=5,uname='min2',uvalue='none',/EDITABLE,/ALL_EVENTS)
             maxpol=WIDGET_TEXT(bsub[4,2],XSIZE=5,uname='max2',uvalue='none',/EDITABLE,/ALL_EVENTS)
;*** Fourth base has the t1 information
      btop[3]=WIDGET_BASE(fitinfo,/ROW)
         bsub[0,3]=WIDGET_BASE(btop[3],/ROW,/NONEXCLUSIVE)
             t1lab=WIDGET_BUTTON(bsub[0,3],uname='parfx3',value='Decay time (hrs):',uvalue='parfx3')
         bsub[1,3]=WIDGET_BASE(btop[3],/ROW,/SENSITIVE)
             t1vals=WIDGET_TEXT(bsub[1,3],uname='parvals3',value='0',uvalue='none',/EDITABLE,/SENSITIVE,/ALL_EVENTS)
         bsub[2,3]=WIDGET_BASE(btop[3],/ROW)
             t1err=WIDGET_TEXT(bsub[2,3],value='0',uname='parerr3')
         bsub[3,3]=WIDGET_BASE(btop[3],uvalue=0L,uname='lfx3',/COLUMN,/NONEXCLUSIVE,/SENSITIVE)
             t1fx=WIDGET_BUTTON(bsub[3,3],value=' ',uname='limfx3',uvalue='limfx3')
         bsub[4,3]=WIDGET_BASE(btop[3],uname='lims3',/ROW,/SENSITIVE)
             mint1=WIDGET_TEXT(bsub[4,3],XSIZE=5,uname='min3',uvalue='none',/EDITABLE,/ALL_EVENTS)
             maxt1=WIDGET_TEXT(bsub[4,3],XSIZE=5,uname='max3',uvalue='none',/EDITABLE,/ALL_EVENTS)
;*** Fifth base has the Polarizer Efficiency information
      btop[4]=WIDGET_BASE(fitinfo,/ROW)
         bsub[0,4]=WIDGET_BASE(btop[4],/ROW,/NONEXCLUSIVE)
             preslab=WIDGET_BUTTON(bsub[0,4],uname='parfx4',value='Polarizer:',uvalue='parfx4')
         bsub[1,4]=WIDGET_BASE(btop[4],/ROW,/SENSITIVE)
             presvals=WIDGET_TEXT(bsub[1,4],uname='parvals4',value='0',uvalue='none',/EDITABLE,/SENSITIVE,/ALL_EVENTS)
         bsub[2,4]=WIDGET_BASE(btop[4],/ROW)
             preserr=WIDGET_TEXT(bsub[2,4],value='0',uname='parerr4')
         bsub[3,4]=WIDGET_BASE(btop[4],uvalue=0L,uname='lfx4',/COLUMN,/NONEXCLUSIVE,/SENSITIVE)
             presfx=WIDGET_BUTTON(bsub[3,4],value=' ',uname='limfx4',uvalue='limfx4')
         bsub[4,4]=WIDGET_BASE(btop[4],uname='lims4',/ROW,/SENSITIVE)
             minpres=WIDGET_TEXT(bsub[4,4],XSIZE=5,uname='min4',uvalue='none',/EDITABLE,/ALL_EVENTS)
             maxpres=WIDGET_TEXT(bsub[4,4],XSIZE=5,uname='max4',uvalue='none',/EDITABLE,/ALL_EVENTS)
;*** Sixth base has the Pressure information
      btop[5]=WIDGET_BASE(fitinfo,/ROW)
         bsub[0,5]=WIDGET_BASE(btop[5],/ROW,/NONEXCLUSIVE)
             p1lab=WIDGET_BUTTON(bsub[0,5],uname='parfx5',value='Pressure (bar):',uvalue='parfx5')
         bsub[1,5]=WIDGET_BASE(btop[5],/ROW,/SENSITIVE)
             p1vals=WIDGET_TEXT(bsub[1,5],uname='parvals5',value='0',uvalue='none',/EDITABLE,/SENSITIVE,/ALL_EVENTS)
         bsub[2,5]=WIDGET_BASE(btop[5],/ROW)
             p1err=WIDGET_TEXT(bsub[2,5],value='0',uname='parerr5')
         bsub[3,5]=WIDGET_BASE(btop[5],uvalue=0L,uname='lfx5',/COLUMN,/NONEXCLUSIVE,/SENSITIVE)
             p1fx=WIDGET_BUTTON(bsub[3,5],value=' ',uname='limfx5',uvalue='limfx5')
         bsub[4,5]=WIDGET_BASE(btop[5],uname='lims5',/ROW,/SENSITIVE)
             minp1=WIDGET_TEXT(bsub[4,5],XSIZE=5,uname='min5',uvalue='none',/EDITABLE,/ALL_EVENTS)
             maxp1=WIDGET_TEXT(bsub[4,5],XSIZE=5,uname='max5',uvalue='none',/EDITABLE,/ALL_EVENTS)
;*** Sixth base has the Wavelength information
      btop[6]=WIDGET_BASE(fitinfo,/ROW)
         bsub[0,6]=WIDGET_BASE(btop[6],/ROW,/NONEXCLUSIVE)
             wavlab=WIDGET_BUTTON(bsub[0,6],uname='parfx6',value='Wavelength (A):',uvalue='parfx6')
         bsub[1,6]=WIDGET_BASE(btop[6],/ROW,/SENSITIVE)
             wavvals=WIDGET_TEXT(bsub[1,6],uname='parvals6',value='0',uvalue='none',/EDITABLE,/SENSITIVE,/ALL_EVENTS)
         bsub[2,6]=WIDGET_BASE(btop[6],/ROW)
             waverr=WIDGET_TEXT(bsub[2,6],value='0',uname='parerr6')
         bsub[3,6]=WIDGET_BASE(btop[6],uvalue=0L,uname='lfx6',/COLUMN,/NONEXCLUSIVE,/SENSITIVE)
             wavfx=WIDGET_BUTTON(bsub[3,6],value=' ',uname='limfx6',uvalue='limfx6')
         bsub[4,6]=WIDGET_BASE(btop[6],uname='lims6',/ROW,/SENSITIVE)
             minwav=WIDGET_TEXT(bsub[4,6],XSIZE=5,uname='min6',uvalue='none',/EDITABLE,/ALL_EVENTS)
             maxwav=WIDGET_TEXT(bsub[4,6],XSIZE=5,uname='max6',uvalue='none',/EDITABLE,/ALL_EVENTS)
;*** Set base widths to line everything up
   widthbtop=intarr(5)
   bgeom=WIDGET_INFO(bsub[0,2],/GEOMETRY)
   widthbtop[0]=bgeom.scr_xsize
   bgeom=WIDGET_INFO(bsub[1,1],/GEOMETRY)
   widthbtop[1]=bgeom.scr_xsize
   bgeom=WIDGET_INFO(bsub[2,1],/GEOMETRY)
   widthbtop[2]=bgeom.scr_xsize
   bgeom=WIDGET_INFO(bsub[3,0],/GEOMETRY)
   widthbtop[3]=bgeom.scr_xsize
   bgeom=WIDGET_INFO(bsub[4,1],/GEOMETRY)
   widthbtop[4]=bgeom.scr_xsize
   for j=0,6 do begin
      for i=0,4 do begin
         WIDGET_CONTROL,bsub[i,j],scr_xsize=widthbtop[i]
      endfor
   endfor

;******* QUIT BOX *******
; *** Create the quit button
   quitbutton=WIDGET_BUTTON(tlb,UNAME='qbut',value='Close 3He Cal',uvalue='quit')

; *** Create WIDGET window
   WIDGET_CONTROL,tlb,/REALIZE

;******* Create the plot window ********
   aw_plotwinshow,tlb,plotwin

;******* Modify uvalue in top widget to include address for plot window ********
   tlbuval.plotwin=plotwin
   WIDGET_CONTROL,tlb,set_uvalue=tlbuval

; *** Run some of the subroutines to update the windows
   aw_setmask,mskinfo,'setmask'
   WIDGET_CONTROL,umsg,set_value='Welcome to the D17 3He calibration routine'
   WIDGET_CONTROL,f1val,SENSITIVE=0
   WIDGET_CONTROL,f2val,SENSITIVE=0
   WIDGET_CONTROL,f1valpm,SENSITIVE=0
   WIDGET_CONTROL,f2valpm,SENSITIVE=0
   WIDGET_CONTROL,fitinfo,SENSITIVE=0

   XMANAGER,'d17_he3cal_gui',tlb,/just_reg,/no_block
end
