;+
;pro D17_he3calc,refl,main=main,water=water,mask=maskstring,time,phi,f1,f2,trans
;
;	LAMP procedure to take a standard 7-run measurement of the four flipper states
;	for a He3 calibration and return the calculated phi, f1 and f2 for the mean time
;	of the measurements.  The calculation is from the equations in:
;	Wildes, Rev. Sci. Instrum., 70 (1999) 4241
;	ARW 31.08.04
;
;	NECESSARY VARIABLES:
;	refl=a string with the first and last of the 7 files, separated by a colon
;
;	OPTIONAL VARIABLES:
;	main=a string with the numbers for a measurement of the main beam, with an empty cell
;	water=the number of the workspace with the (masked) water file in it.
;		The data will be divided by this workspace if it exists
;	maskstring=your choice of masking the detector, written as a string
;			The default is no mask
;
;	OUTPUT VARIABLES:
;	time=the mean time for the measurement.  The values for phi,f1,f2 and trans correspond
;		to this time
;	phi=the combined polarizer/analyzer inefficiency
;	f1=the first flipper inefficiency
;	f2=the second flipper inefficiency
;	trans=the transmission of the cell.  This is only output if 'main' is specified
;
;	e.g.
;	 d17_he3calc,'90230:90236',main='90201',water=1,mask='box,[190,200,63,222]',time,phi,f1,f2,trans
;
;
;	CALLS TO:	aw_peaksearch,aw_sumup,aw_gaussfun (internal), d17_errmsg
;			D17_polcalc,mload,mpfitfun,d17_waternorm(external)
;	Last modified:	ARW 4.2.11
;-
forward_function d17_polcalc
forward_function mload
forward_function mpfitfun
forward_function d17_waternorm


;********************************************************************************
;************  First define the peaksearch **************************************
;********************************************************************************
pro aw_peaksearch,win,cen,sigma,x=x,range=range
;+
;** *************
;**	LAMP procedure that takes a 1D array and searches for a peak
;**	within the limits of range.
;**	Returns the peak centre (calculated by weighted mean)
;**	and the standard deviation.
;**	ARW 03.03.04
;**
;**	NECESSARY INPUT
;**	win=input data, a 1D array
;**
;**	OPTIONAL INPUT
;**	x=x-coordinates for win	(defaults is pixels)
;**	range=2 element array, limits the search for the peak
;**			(Give units in x, default is pixels)
;**
;**	RETURNS
;**	cen=weighted mean for the peak
;**	sigma=standard deviation
;**
;**	e.g.  aw_peaksearch,array1D,cen,sigma,x=findgen(51),range=[45,51]
;**
;**	CALLS TO:
;**	LAST MODIFIED:	ARW 19.03.04
;-

;***	Check for x and range, set to integers if non-existant
if (n_elements(x) eq 0) then x=findgen(n_elements(win))
if (n_elements(range) ne 2) then range=float([min(x),max(x)])

;***	Check that elements in win and x correspond
if (n_elements(x) ne n_elements(win)) then begin
	d17_errmsg,1,'ERROR: data and x-axis coords have different numbers of elements'
	cen=0
	sigma=0
	return
endif

;***	Check that the range is within the x-coordinates
if (max(x) lt max(range)) or (min(x) gt min(range)) then begin
	d17_errmsg,1,'ERROR: The range is larger than the x-coordinates'
	cen=0
	sigma=0
	return
endif

;***	Extract the data that falls within the limits
els=where(x ge range[0] and x le range[1])
x_sub=x[els]
win_sub=win[els]

;***	Find the weighted mean and variance of the data
cen=total(x_sub*win_sub)/total(win_sub)
sigma=sqrt(total(win_sub*(x_sub-cen)^2)/total(win_sub))

d17_errmsg,1,'Centre ='+string(cen)
d17_errmsg,1,'Sigma = '+string(sigma)
end

;*********************************************************************
;************  A trapezoidal summation routine ***********************
;*********************************************************************
pro aw_sumup,xin,yin,ein,bkg,yout,eout
;+
;** ********
;**	LAMP procedure to integrate the intensity over a 1D peak
;**	Uses a trapezoidal summation, subtracts a FLAT background(!!!)
;**	ARW 4.2.11
;**
;**	NECESSARY INPUT
;**	xin=x-values for input data
;**	yin=y-values for input data
;**	ein=corresponding error for the data
;**	bkgd=a 2 element array 
;**	    = [value for the constant background, error in background]
;**
;**	RETURNS
;**	yout=summed data, minus the background
;**	eout=corresponding error
;**
;**	CALLS TO:
;**	last modified:	ARW 4.2.11
;-

npts =n_elements(xin)
xstep=xin[1:*]-xin[0:npts-2]
ystep=0.5*(yin[1:*]+yin[0:npts-2])
estep=0.5*sqrt(ein[1:*]^2+ein[0:npts-2])
bktot=bkg[0]*(xin[npts-1]-xin[0])
ebkg =sqrt((bkg[1]*(xin[npts-1]-xin[0]))^2)

yout=total(xstep*ystep)-bktot
eout=sqrt(total((xstep*estep)^2)-ebkg^2)

end

;*********************************************************************
;************  A Gauss function **************************************
;*********************************************************************
function aw_gaussfun,x,p
;+
;******* ***********
;**	LAMP function that will write a Gaussian to a workspace.
;**	Can be used in combination with 'mnfitfun'
;**	Can also be used in combination with aw_gauss (a procedure)
;**		This option is rather useful if you want to fit many peaks
;**
;**	NECESSARY INPUTS:
;**	x = the range of x-values in whatever units you want
;**	p = the parameters for the fit
;**		p(0) = background
;**		p(1) = amplitude (not normalized)
;**		p(2) = centre point
;**		p(3) = standard deviation
;**	If you wish to edit the program to fit N Gaussians, p will need to
;**		have 3N+1 elements:
;**		p(0) is always background
;**		p(3*i-2) is the amplitude of the (1 <= i <= N)th peak
;**		p(3*i-1) is the centre point of the ith peak
;**		p(3*i)   is the standard deviation of the ith peak
;**	ARW 22.07.05
;**
;**	e.g.  w1=aw_gauss(indgen(286),[5.0,100.0,150.0,2.0])
;**	Last modified:
;-
   bkg=p[0]
   amp=p[1]
   cen=p[2]
   sigma=p[3]
   y=amp*exp(-0.5*((x-cen)/sigma)^2)
   y=y+bkg
   return,y
end

;********************************************************************************
;************  And finally the routine ******************************************
;********************************************************************************
pro D17_he3calc,refl,main=main,water=water,mask=maskstring,time,phi,f1,f2,trans
;+
;** ***********
;** User callable
;-

; ***	Read the data files in to an array
I=mload(refl,norm='time',mask=maskstring)
TAKE_DATP,Idatp
tot=total(total(I,2),2)
totdatp=Idatp
totdatp.e=sqrt(total(Idatp.e^2,2))

; ***	Find the mean time of the measurement
time=mean(Idatp.pv[2,*])
caldat,time,Month, Day, Year, Hour, Minute, Second

; ***	Read the water if included
wflag=n_elements(water)
if wflag ne 0 then begin
	if water ne 0 then begin
		TAKE_W,ww,w=water
		TAKE_DATP,waterdatp,w=water
		if n_elements(ww) ne n_elements(tot) then begin
			d17_errmsg,1,'ERROR: The number of elements in the data and the water files must correspond'
			return
		endif
		I=d17_waternorm(I,ww)
		TAKE_DATP,Idatp
	endif else print,'No water run used'
endif else print,'No water run used'

; ***	Determine peak positions in the data and estimate the widths
; ***	Here's the method:
aw_peaksearch,tot,cen,sigma				; *** Gives an estimate of the peak centre and width
rng = floor(cen-7.0*sigma)+indgen(ceil(cen+7.0*sigma)+1-floor(cen-7.0*sigma))	; *** Creates a data range, +/- 7 standard deviations

parinfo=replicate({value:0.D,fixed:0,limited:[0,0],limits:[0.D,0.D]},4)  ; *** create parameter info
parinfo.value=[min(tot[where(tot ne 0)]),max(tot)/4.0,cen,sigma]	 ; *** initial fit esimate

; *** Now fit the data to find the background
; *** F1=0, F2=0
pout=mpfitfun('aw_gaussfun',Idatp.x[rng],I[rng,*,0],Idatp.e[rng,*,0],PARINFO=parinfo,PERROR=peout,COVAR=covout,BESTNORM=chi2)
catchem=pout
ecatchem=peout
aw_sumup,Idatp.x[rng],I[rng,*,0],Idatp.e[rng,*,0],[pout[0],peout[0]],dI,deI
w_out=dI
e_out=deI
Itot=[dI,deI]
; *** F1=1, F2=0
pout=mpfitfun('aw_gaussfun',Idatp.x[rng],I[rng,*,2],Idatp.e[rng,*,2],PARINFO=parinfo,PERROR=peout,COVAR=covout,BESTNORM=chi2)
catchem=[[catchem],[pout]]
ecatchem=[[ecatchem],[peout]]
aw_sumup,Idatp.x[rng],I[rng,*,2],Idatp.e[rng,*,2],[pout[0],peout[0]],dI,deI
Itot=[[Itot],[dI,deI]]
; *** F1=1, F2=1
pout=mpfitfun('aw_gaussfun',Idatp.x[rng],I[rng,*,3],Idatp.e[rng,*,3],PARINFO=parinfo,PERROR=peout,COVAR=covout,BESTNORM=chi2)
catchem=[[catchem],[pout]]
ecatchem=[[ecatchem],[peout]]
aw_sumup,Idatp.x[rng],I[rng,*,3],Idatp.e[rng,*,3],[pout[0],peout[0]],dI,deI
Itot=[[Itot],[dI,deI]]
; *** F1=0, F2=1
pout=mpfitfun('aw_gaussfun',Idatp.x[rng],I[rng,*,1],Idatp.e[rng,*,0],PARINFO=parinfo,PERROR=peout,COVAR=covout,BESTNORM=chi2)
catchem=[[catchem],[pout]]
ecatchem=[[ecatchem],[peout]]
aw_sumup,Idatp.x[rng],I[rng,*,1],Idatp.e[rng,*,1],[pout[0],peout[0]],dI,deI
Itot=[[Itot],[dI,deI]]


; ***	Create a parameter set for the fits
;parinfo=replicate({value:0.D,fixed:0,limited:[0,0],limits:[0.D,0.D]},4)
;aw_peaksearch,tot,cen,sigma
;parinfo.value=[min(tot(where(tot ne 0))),max(tot)/4.0,cen,sigma]

; ***	Now find the individual integrated intensities
;pout=mpfitfun('aw_gaussfun',Idatp.x,I(*,*,1),Idatp.e(*,*,1),PARINFO=parinfo,PERROR=peout,COVAR=covout,BESTNORM=chi2)
;Itot=[abs(pout(1)*pout(3)),sqrt((peout(1)*pout(3))^2+(peout(3)*pout(1))^2)]
;catchem=pout
;pout=mpfitfun('aw_gaussfun',Idatp.x,I(*,*,3),Idatp.e(*,*,3),PARINFO=parinfo,PERROR=peout,COVAR=covout,BESTNORM=chi2)
;Itot=[abs(pout(1)*pout(3)),sqrt((peout(1)*pout(3))^2+(peout(3)*pout(1))^2),Itot]
;catchem=[[pout],[catchem]]
;pout=mpfitfun('aw_gaussfun',Idatp.x,I(*,*,2),Idatp.e(*,*,2),PARINFO=parinfo,PERROR=peout,COVAR=covout,BESTNORM=chi2)
;Itot=[abs(pout(1)*pout(3)),sqrt((peout(1)*pout(3))^2+(peout(3)*pout(1))^2),Itot]
;catchem=[[pout],[catchem]]
;pout=mpfitfun('aw_gaussfun',Idatp.x,I(*,*,0),Idatp.e(*,*,0),PARINFO=parinfo,PERROR=peout,COVAR=covout,BESTNORM=chi2)
;Itot=[abs(pout(1)*pout(3)),sqrt((peout(1)*pout(3))^2+(peout(3)*pout(1))^2),Itot]
;catchem=[[pout],[catchem]]
;w_out=abs(pout(1)*pout(3))
;e_out=sqrt((peout(1)*pout(3))^2+(peout(3)*pout(1))^2)

; ***	Read the main beam if it's included
mflag=n_elements(main)
if (mflag ne 0) then begin
	if (main ne 0) then begin
		mbeam=mload(main,norm='time',mask=maskstring)
		TAKE_DATP,mdp
		aw_peaksearch,mbeam,cen,sigma
		rng = floor(cen-7.0*sigma)+indgen(ceil(cen+7.0*sigma)+1-floor(cen-7.0*sigma))
		parinfo.value=[min(tot[where(mbeam ne 0)]),max(mbeam),cen,sigma]
		pout=mpfitfun('aw_gaussfun',mdp.x[rng],mbeam[rng],mdp.e[rng],PARINFO=parinfo,PERROR=peout,COVAR=covout,BESTNORM=chi2)
		aw_sumup,mdp.x[rng],mbeam[rng],mdp.e[rng],[pout[0],peout[0]],mout,meout
;		pout=mpfitfun('aw_gaussfun',mdp.x,mbeam,mdp.e,PARINFO=parinfo,PERROR=peout,COVAR=covout,BESTNORM=chi2)
;		mout=abs(pout(1)*pout(3))
;		meout=sqrt((peout(1)*pout(3))^2+(peout(3)*pout(1))^2)
; Correct for slit sizes, take S2W*S3W
		Dsz=(mdp.p[7]*mdp.p[9])/(Idatp.p[7]*Idatp.p[9])
		trans=Dsz*[w_out/mout]
		trans=[trans,Dsz*sqrt((e_out/mout)^2+(meout*w_out/mout^2)^2)]
		d17_errmsg,1,'Transmission = '+string(trans[0])+' +/- '+string(trans[1])
	endif
endif else trans = [0,0]

d17_errmsg,1,'I00/I10 = '+string(Itot[0]/Itot[2])
d17_errmsg,1,'I00/I01 = '+string(Itot[0]/Itot[6])
d17_polcalc,Itot,phi,f1,f2
d17_errmsg,1,'at time = '+strtrim(string(hour),1)+':'+strtrim(string(minute),1)+':'+strtrim(string(second),1)+', '+strtrim(string(day),1)+'/'+strtrim(string(month),1)+'/'+strtrim(string(year),1)
;print,Itot
;print,catchem
;print,ecatchem
;print,pout
;print,mout
end
