function D17_he3calibrate,calfiles,main=main,water=water,mask=maskstring,F1=FF1,F2=FF2
;+
;	LAMP function to create a workspace 
;	with the values of phi,F1,F2 and transmission as a function of time
;	The function uses aw_he3calc to calculate phi, F1 and F2,
;	It then estimates the mean F1 and F2 (which are time-independent)
;	It then recalculates phi (the only time-dependent quantity)
;	The output has 7 rows:
;	Row 1 = phi, calculated for each He3 calibration
;	Row 2 = F1, calculated for each He3 calibration
;	Row 3 = F2, calculated for each He3 calibration
;	Row 4 = transmission, calculated for each He3 calibration
;	Row 5 = phi calculated from the mean value of F1 below
;			OR, using the optional F1 and/or F2 if specified
;	Row 6 = the mean F1 for all the He3 calibration files
;	Row 7 = the mean F2 for all the He3 calibration files
;	The errors have also been estimated assuming Poissonian statistics
;	ARW 02.09.04
;
;	NECESSARY VARIABLES:
;	calfiles=a string with ALL the 3He calibration runs included
;		The first and last file for each calibration scan must be
;		separated by a colon (:)
;		Separate calibration scans must be separated by a comma (,)
;
;	OPTIONAL VARIABLES:
;	main=a string with the numbers for a measurement of the main beam, with an empty cell
;	water=the number of the workspace with the (masked) water file in it.  
;		The data will be divided by this workspace if it exists
;	maskstring=your choice of masking the detector, written as a string
;			The default is no mask
;	FF1 = a two element array with the first flipper efficiency and its error
;		Specify this if you want to force F1 to a value
;	FF2 = a two element array with the second flipper efficiency and its error
;		Specify this if you want to force F2 to a value
;
;	e.g. 
;	w1=d17_he3calibrate('97087:97093,97339:97345,97591:97597',main='90201',water=1,mask='aw_box,[190,200,63,222]')
;
;	Calls to:	D17_he3calc,rdopr,d17_errmsg (external)
;	Last modified:	ARW 17.10.10
;-
forward_function rdopr
forward_function d17_he3calc

; ***   Get rid of this jolly error!
calfiles=calfiles[0]

; ***	Find the first comma
eohe=strpos(calfiles,',')
commapos=eohe
if commapos eq -1 then commapos = strlen(calfiles)

; ***	Use the first file to initialize the datp
datp=0
strun1=strmid(calfiles,0,strpos(calfiles,':'))
run1=rdopr(strun1,datp=datp)

; ***	Loop through the He3 calibration runs and fill in the array
hefiles=strmid(calfiles,0,commapos)
calfiles=strmid(calfiles,commapos+1,strlen(calfiles))
D17_he3calc,hefiles,main=main,water=water,mask=maskstring,time,phi,f1,f2,trans
wout=transpose([phi[0],f1[0],f2[0],trans[0]])
eout=transpose([phi[1],f1[1],f2[1],trans[1]])
tout=time
while eohe ne -1 do begin
	eohe=strpos(calfiles,',')
	commapos=eohe
	if commapos eq -1 then commapos = strlen(calfiles)
	hefiles=strmid(calfiles,0,commapos)
	calfiles=strmid(calfiles,commapos+1,strlen(calfiles))
	D17_he3calc,hefiles,main=main,water=water,mask=maskstring,time,phi,f1,f2,trans
	wout=[wout,transpose([phi[0],f1[0],f2[0],trans[0]])]
	eout=[eout,transpose([phi[1],f1[1],f2[1],trans[1]])]
	tout=[tout,time]
endwhile

; ***	Subtract off a random time, say, 1st Jan, 2004
;tout=tout-julday(1,1,2004,0,0,0)

; ***	Find the mean F1 and F2 values
psize=size(wout)
mf1=total(wout[*,1])/psize[1]
emf1=sqrt(total(eout[*,1]^2))/psize[1]
mf2=total(wout[*,2])/psize[1]
emf2=sqrt(total(eout[*,2]^2))/psize[1]

d17_errmsg,1,['Mean F1 = '+string(100*(1-mf1))+'% +/- '+string(100*emf1)]
d17_errmsg,1,['Mean F2 = '+string(100*(1-mf2))+'% +/- '+string(100*emf2)]

; ***	Determine whether F1 is to be forced and use the input values
if n_elements(FF1) eq 2 then begin
	d17_errmsg,1,['First flipper efficiency is forced to '+string(100*(1-FF1[0]))+'% +/- '+string(100*FF1[1])]
	mf1=FF1[0]
	emf1=FF1[1]
endif

; ***	Determine whether F2 is to be forced and use the input values
if n_elements(FF2) eq 2 then begin
	d17_errmsg,1,['Second flipper efficiency is forced to '+string(100*(1-FF2[0]))+'% +/- '+string(100*FF2[1])]
	mf2=FF2[0]
	emf2=FF2[1]
endif


; ***	Calculate the values of phi assuming F1 and F2 are their mean values
; ***	First, calculate the flipping ratios given by the data
fr1denom=(1-wout[*,1])*wout[*,0]+wout[*,1]*(1-wout[*,0])
fliprat1=(1-wout[*,0])/fr1denom
efr1=sqrt( (eout[*,0]*((1-2*wout[*,1])*(1-wout[*,0])/fr1denom^2+1/fr1denom))^2+(eout[*,1]*(1-2*wout[*,0])/fr1denom^2)^2)
fr2denom=(1-wout[*,2])*wout[*,0]+wout[*,2]*(1-wout[*,0])
fliprat2=(1-wout[*,0])/fr2denom
efr2=sqrt( (eout[*,0]*((1-2*wout[*,2])*(1-wout[*,0])/fr2denom^2+1/fr2denom))^2+(eout[*,2]*(1-2*wout[*,0])/fr2denom^2)^2)

; ***	Now, calculate the phi for those flipping ratios with the mean F1, F2
mf1denom=1+fliprat1*(1-2*mf1)
mphi1=(1-fliprat1*mf1)/mf1denom
ephi1=(efr1*(mf1/mf1denom+(1-2*mf1)*(1-fliprat1*mf1)/mf1denom^2))^2
ephi1=ephi1+(emf1*fliprat1*(2*(1-fliprat1*mf1)/mf1denom^2-1/mf1denom))^2
ephi1=sqrt(ephi1)
mf2denom=1+fliprat2*(1-2*mf2)
mphi2=(1-fliprat2*mf2)/mf2denom
ephi2=(efr2*(mf2/mf2denom+(1-2*mf2)*(1-fliprat2*mf2)/mf2denom^2))^2
ephi2=ephi2+(emf2*fliprat2*(2*(1-fliprat2*mf2)/mf2denom^2-1/mf2denom))^2
ephi2=sqrt(ephi2)

wout=[[wout],[mphi1],[mf1*(fltarr(psize[1])+1.0)],[mf2*(fltarr(psize[1])+1.0)]]
eout=[[eout],[ephi1],[emf1*(fltarr(psize[1])+1.0)],[emf2*(fltarr(psize[1])+1.0)]]

; ***	Modify datp
MOD_DATP,datp,"e",eout
MOD_DATP,datp,"x",tout
MOD_DATP,datp,"w_tit",'Polarization analysis inefficiency corrections'
MOD_DATP,datp,"x_tit",'Time of measurement'
MOD_DATP,datp,"y_tit",'phi F1 F2 trans mean(F1) mean(F2) phi_m'
MOD_DATP,datp,"other_tit",' '
MOD_DATP,datp,"n",0

GIVE_DATP,datp
return,wout
end
