function d17_he3corr,win,params,tol=tol
;+
;	LAMP function to correct polarization analysis data taken with
;	the He3 filter
;	ARW 06.09.04
;
;	NECESSARY VARIABLES:
;	win=workspace with the data, must have the four polarization states as 
;		[[I00],[I01],[I10],[I11]]
;	params=the fitted parameters to phi and trans
;		This can be a 1 or 2D array, 
;			the second row in a 2D array is the error on the parameter		
;		params[0]=initial time for filter (t0)
;		params[1]=initial polarization of 3He gas (P0He)
;		params[2]=decay time of cell (t1)
;		params[3]=polarizer inefficiency (p1)
;		params[4]=gas pressure ([3He])
;		params[5]=wavelength   (lambda)
;		params[6]=first flipper efficiency (f1)
;		params[7]=second flipper efficiency (f2)
;		Cell thickness is fixed at 10 cm
;
;	OPTIONAL VARIABLES:
;	tol=the tolerance in the time for the measurement in seconds.
;		The correction requires a time of measurement for the 4 flipper states
;		This is taken as the average time of measurement.
;		There will be some variation on the calculated average, 
;		normally +/- a second or two
;		tol=2 should cover this and allow a proper correction to be made
;		If the data falls outside this tolerance, an error message will be sent
;
;	e.g. w5=d17_he3corr(w4,p,tol=2)
;
;	CALLS TO:	he3tptm, d17_errmsg
;	Last modified:	ARW 17.12.10
;-
forward_function he3tptm

; ***	Check for optional variables
if n_elements(tol) eq 0 then tol = 1e-7

; ***	Collect the necessary parameters from the workspaces
TAKE_DATP,w1datp		;workspace information
woutdatp=w1datp
; ***	Extract the errors in the data
ein=w1datp.e

; *** 	If only one data point has been measured, the workspace needs to be reshaped
wsize=size(win,/DIMENSIONS)
if wsize[1] eq 1 then mod_datp,w1datp,"y",0
if n_elements(wsize) eq 2 then begin
	win=reform(win,wsize[0],1,wsize[1],/OVERWRITE)
	ein=reform(w1datp.e,wsize[0],1,wsize[1],/OVERWRITE)
	dpv=reform(w1datp.pv,6,1,wsize[1],/OVERWRITE)
	mod_datp,w1datp,"pv",dpv
	mod_datp,w1datp,"y",0
endif

; ***	Determine that the parameters have the right number of elements
perrmess=0
psize=size(params)
if psize[0] eq 1 then begin
	if psize[1] ne 8 then perrmess=1
	params=transpose([[params],[fltarr(6)]])
endif
if psize[0] eq 2 and psize[1] eq 8 and psize[2] eq 2 then params=transpose(params)
if psize[0] eq 2 and psize[1]+psize[2] ne 10 then perrmess=1
if perrmess eq 1 then begin
	d17_errmsg,1,'ERROR: Input parameters to d17_he3corr have the wrong dimensions'
	return,0
endif


; ***	Extract polarization correction data
P1=params[*,3]
F1=params[*,6]	; Mean F1 with error
F2=params[*,7]	; Mean F2 with error

; ***	Calculate the time independent matrices
F1mat=1.0/(1-F1[0])*[[1-F1[0],0,0,0],[0,1-F1[0],0,0],[-F1[0],0,1,0],[0,-F1[0],0,1]]
F2mat=1.0/(1-F2[0])*[[1-F2[0],0,0,0],[-F2[0],1,0,0],[0,0,1-F2[0],0],[0,0,-F2[0],1]]
P1mat=1.0/(1-2*P1[0])*[[1-P1[0],0,-P1[0],0],[0,1-P1[0],0,-P1[0]], $
		[-P1[0],0,1-P1[0],0],[0,-P1[0],0,1-P1[0]]]
F1F2P1mat=P1mat##F2mat##F1mat

eF1mat=F1[1]/(1-F1[0])^2*[[0,0,0,0],[0,0,0,0],[-1.0,0,1.0,0],[0,-1.0,0,1.0]]
eF2mat=F2[1]/(1-F2[0])^2*[[0,0,0,0],[-1.0,1.0,0,0],[0,0,0,0],[0,0,-1.0,1.0]]
eP1mat=P1[1]/(1-2*P1[0])^2*[[1.0,0,-1.0,0],[0,1.0,0,-1.0],[-1.0,0,1.0,0],[0,-1.0,0,1.0]]

eF1F2P1= P1mat ## F2mat ##eF1mat
F1eF2P1= P1mat ##eF2mat ## F1mat
F1F2eP1=eP1mat ## F2mat ## F1mat

; ***	Extract the times for the flipper dependent measurements
; ***	and check that they are within tolerance
ftimes=w1datp.pv[2,*,*]
; ***	If only one point has been measured, ftimes needs to be reformed
if n_elements(ftimes) eq 4 then ftimes=reform(ftimes,1,1,4,/OVERWRITE)
pvsize=size(ftimes,/DIMENSIONS)
if pvsize[2] ne 4 or pvsize[1] ne n_elements(w1datp.y) then begin
	d17_errmsg,1,'ERROR: Each flipper state needs its own measurement time'
	return,0
endif

ftimes=[ftimes[*,*,0],ftimes[*,*,1],ftimes[*,*,2],ftimes[*,*,3]]
if n_elements(ftimes) eq 4 then ftimes=reform(ftimes,1,4,/OVERWRITE)
hetime=dblarr(n_elements(w1datp.y))
onesec=julday(1,1,2004,0,0,1)-julday(1,1,2004,0,0,0)
tol=tol*onesec
for i=0,n_elements(w1datp.y)-1 do begin
;	caldat,ftimes(i,*),Month, Day, Year, Hour, Minute, Sec
	timecheck=abs(ftimes[*,i]-mean(ftimes[*,i]))
	timecheck=where(timecheck gt tol)
	if n_elements(timecheck) gt 1 or timecheck[0] ne -1  then begin
		d17_errmsg,1,'ERROR: The listed times for the measurements differ outside the tolerance'
		return,0
	endif
	hetime[i]=mean(ftimes[*,i])
endfor
;MOD_DATP,woutdatp,"pv",hetime

; ***	Calculate PHI and TRANS for the given times
pin=transpose(params)
he3tptm,hetime,pin,tp,tm
;P2=tm/(tp+tm)

; ***	Loop through each time for the measurements 
; ***	and correct for polarization and transmission
wout=dblarr(n_elements(w1datp.x),n_elements(w1datp.y),4)
eout=wout
for i=0,n_elements(hetime)-1 do begin
	dinp=reform(win[*,i,*])
	einp=reform(ein[*,i,*])
	Hemat=1.0/(tp[i,0]^2-tm[i,0]^2)* $
		[[tp[i,0],-tm[i,0],0,0],[-tm[i,0],tp[i,0],0,0], $
		[0,0,tp[i,0],-tm[i,0]],[0,0,-tm[i,0],tp[i,0]]]
	M=Hemat##F1F2P1mat
	tp_tm=-1.0*(tp[i,0]^2+tm[i,0]^2)
	tptm=2*tp[i,0]*tm[i,0]
	eHetp=tp[i,1]/(tp[i,0]^2-tm[i,0]^2)^2 * $
			[[tp_tm,tptm,0,0],[tptm,tp_tm,0,0], $
			[0,0,tp_tm,tptm],[0,0,tptm,tp_tm]]
	eHetm=tm[i,1]/(tp[i,0]^2-tm[i,0]^2)^2 * $
			[[tptm,tp_tm,0,0],[tp_tm,tptm,0,0], $
			[0,0,tptm,tp_tm],[0,0,tp_tm,tptm]]
	for j=0,n_elements(w1datp.x)-1 do begin
		wout[j,i,*]=M##dinp[j,*]
		eout[j,i,*]= (eHetp##F1F2P1mat##dinp[j,*])^2     + $
				(eHetm##F1F2P1mat##dinp[j,*])^2  + $
				(Hemat##eF1F2P1##dinp[j,*])^2    + $
				(Hemat##F1eF2P1##dinp[j,*])^2    + $
				(Hemat##F1F2eP1##dinp[j,*])^2    + $
				M^2##einp[j,*]^2
	endfor
endfor

woutdatp.e=sqrt(eout)

MOD_DATP,woutdatp,"pv",w1datp.pv[*,*,0]
GIVE_DATP,woutdatp
return,wout

end
