;+
;pro d17_he3fit,win,pin,pout,yout,HDAT=datp,PFIX=pfix,LIMITS=lims,DSEL=dsel,CROSS=cross
;
;	LAMP procedure to fit the 3He data to calculate the
;	polarization decay constants.
;	ARW 09.09.04
;
;	NECESSARY INPUT:
;	win=the workspace with the values of phi and the transmission.
;		calculated using d17_he3calibrate
;	pin=the input parameters:
;		pin(0) = definition of time t=0 (by 'julday')
;		pin(1) = initial polarization of the He3 gas	(%)
;		pin(2) = the time constant for decay 		(hours)
;		pin(3) = the inefficiency of the polarizer
;		pin(4) = the pressure of the gas 		(bar)
;		pin(5) = the wavelength 			(Angstrom)
;
;	OPTIONAL INPUT
;	pfix=a 6-element array showing which of pin are fixed and which are free
;		A value of zero is free, a value of one is fixed
;	limits=a 2 column by 6 row array with the upper and lower limits
;		for each parameter.  Set values to -1 for no limit
;	DSEL= a 2 row array with the same number of columns as polchecks
;		This parameter can be set to deselect data points
;		A value of 0 for the corresponding data point means that it is DESELECTED
;		Any other value will select the data point
;	cross=0 if the 3He is polarized in the SAME direction as the monochromator
;	     =1 if the 3He is polarized OPPOSITE to the monochromator
;		Default is cross=0
;
;	OUTPUT:
;	pout=the output parameters with errors as a 2 column array:
;		pout(0) = definition of time t=0 (by 'julday')
;		pout(1) = initial polarization of the He3 gas	(%)
;		pout(2) = the time constant for decay 		(hours)
;		pout(3) = the inefficiency of the polarizer
;		pout(4) = the pressure of the gas 		(bar)
;		pout(5) = the wavelength 			(Angstrom)
;		pout(6) = the first flipper efficiency
;		pout(7) = the second flipper efficiency
;	yout=the values of phi and trans as a function of time
;
;	Example:
;	d17_he3fit,w2,[julday(8,25,2004,14,15,00),0.6,150,0.013,0.6,5.3],pout,w11, $
;		PFIX=[0,0,0,1,0,1], $
;		LIMITS=[[-1,min(x2)],[0.5,0.65],[100,200],[0,0.02],[0.5,0.7],[-1,-1]]
;
;	CALLS TO:	he3tptm, mpfunfit, d17_errmsg
;	Last modified:	ARW 7.2.11
;-
forward_function he3tptm

;********************************************************************************
;*** Two fit rountines, whether the 'cross' if on or not.  Ugly, but it works ***
;********************************************************************************
function he3polfunc,time,params,sim=sim
;+
;******* **********
;**	LAMP function to calculate the polarization and the transmission of a 3He filter
;**	as a function of time
;**	The function returns phi (c.f. Wildes, Rev. Sci. Instrum., 70, 1999, 4241)
;**	and the transmission as either a 1 row array (for fits)
;**	or a 2 row array (for simulations)
;**	CAUTION: This routine cannot be used blindly!  If you want to simulate the data,
;**	   the output array must have the same number of time elements as the entire data array
;**	   (including deselected points).  This means that you'll need to be careful to 
;**	   feed the right time array as input to this routine if you want to use it to simulate
;**	ARW 08.09.04
;**
;**	NECESSARY VARIABLES:
;**	time:	The date and time for the polarization of the instrument.
;**		This is a single number for the date, made using the 'julday' procedure
;**	params:	The parameter array.  The parameters are:
;**		params(0) = definition of time t=0 (by 'julday')(t0)
;**		params(1) = initial polarization of the He3 gas (P0He)
;**		params(2) = the time constant for decay (hours)	(t1)
;**		params(3) = the inefficiency of the polarizer	(p1)
;**		params(4) = the pressure of the gas (bar)	(conc)
;**		params(5) = the wavelength (Angstrom)		(sa)
;**	NOTE: The thickness of the D17 cell is currently fixed to 10 cm
;**
;**	OPTIONAL VARIABLES:
;**	sim:	Set this flag to simulate the transmission and polarization for one set of times 
;**			as a 2 row * n column array
;**		The flag can contain anything
;**
;**	e.g. phiout=aw_he3polfunc(time,[julday(8,25,2004,12,0,0),0.57,150,0.05,0.5,5.40])
;**
;**	CALLS TO:	he3tptm
;**	Last modified:	ARW 4.2.11
;-

; *** The polarizer's inefficiency
p1=params[3]
; *** Calculate polarization and transmission for all times
t=abs(time)
he3tptm,t,params,tplus,tminus
p2=tminus/(tplus+tminus)
PHI=p1+p2-2*p1*p2
TRANS=2*((1-p1)*tplus+p1*tminus)
; *** Determine if this is a simulation
if n_elements(sim) gt 0 then return,[[PHI],[TRANS]]
; *** In a fit, negative times are for transmission, positive for polarization
t_trans=where(time lt 0)
t_pol  =where(time gt 0)
if (t_trans[0] gt -1) and (t_pol[0] gt -1) then return,[PHI[t_pol],TRANS[t_trans]]
if (t_pol[0] gt -1) then return,PHI[t_pol]
if (t_trans[0] gt -1) then return,TRANS[t_trans]
end

;********************************************************************************
function he3polfuncx,time,params,sim=sim
;+
;******* ***********
;**	CALLS TO:	he3tptm
;**	Last modified:	ARW 7.02.11
;-

; *** The polarizer's inefficiency
p1=params[3]
; *** Calculate polarization and transmission for all times
t=abs(time)
he3tptm,t,params,tminus,tplus
p2=tminus/(tplus+tminus)
PHI=p1+p2-2*p1*p2
TRANS=2*((1-p1)*tplus+p1*tminus)
; *** Determine if this is a simulation
if n_elements(sim) gt 0 then return,[[PHI],[TRANS]]
; *** In a fit, negative times are for transmission, positive for polarization
t_trans=where(time lt 0)
t_pol  =where(time gt 0)
if (t_trans[0] gt -1) and (t_pol[0] gt -1) then return,[PHI[t_pol],TRANS[t_trans]]
if (t_pol[0] gt -1) then return,PHI[t_pol]
if (t_trans[0] gt -1) then return,TRANS[t_trans]
end
;********************************
;*** And finally the rountine ***
;********************************
pro d17_he3fit,win,pin,pout,yout,HDATP=datp,PFIX=pfix,LIMITS=lims,DSEL=dsel,CROSS=cross
;+
;** **********
;** User callable
;-

   if n_elements(datp) eq 0 then take_datp,datp
; ***	Collect all the variables.  The 'phi' used is calculated using the mean F1 and F2
   nx=n_elements(datp.x)
   x=[datp.x,-datp.x]
   y=[win[*,4],win[*,3]]	; *** This is a 1 row array, with pol 1st and trans 2nd
   err=[datp.e[*,4],datp.e[*,3]]
;   w=1/err^2

; ***	Check for selected data
   if n_elements(dsel) gt 0 then begin
      dsel=fix([dsel[*,0],dsel[*,1]])
      if n_elements(dsel) ne n_elements(x) then begin
         d17_errmsg,1,'WARNING: The number of elements in DSEL differ from the number of data ponts'
         d17_errmsg,1,'All data points will be selected for the fit'
         dsel=1+0*indgen(n_elements(x))
      endif
      selpts=where(dsel ne 0)
      x=x[selpts]
      y=y[selpts]
      err=err[selpts]
   endif

; ***	Create PARINFO values
   parinfo=replicate({value:0.D,fixed:0,limited:[0,0],limits:[0.D,0]},6)
   parinfo.value=pin
; ***	Check for fixed values
   if n_elements(pfix) ne 6 then begin
      d17_errmsg,1,'WARNING:  All parameters are free'
      pfix = 1+0*indgen(6)
   endif
   parinfo.fixed=pfix
; ***	Check for limits
   if n_elements(lims) gt 0 then begin
      slims=size(lims)
      if slims[1] ne 2 or slims[2] ne 6 then begin
         d17_errmsg,1,'WARNING: Limit array has bad dimensions.  No limits set for the fit'
         lims=fltarr(2,6)-1
      endif
      nolim=intarr(2,6)+1
      nolim[where(lims lt 0)]=0
      lims[where(lims lt 0)]=0
      parinfo.limited=nolim
      parinfo.limits=lims
   endif
; ***	Check for cross
   if n_elements(cross) ne 1 then cross=0
; ***	Do the fit
   print,'initial parameters'
   print,pin
   if cross eq 0 then begin
      pout=mpfitfun('he3polfunc',x,y,err,PARINFO=parinfo, $
        PERROR=peout,COVAR=covout,BESTNORM=chi2)
      yout=he3polfunc(datp.x,pout,SIM=1)
   endif else begin
      pout=mpfitfun('he3polfuncx',x,y,err,PARINFO=parinfo, $
        PERROR=peout,COVAR=covout,BESTNORM=chi2)
      yout=he3polfuncx(datp.x,pout,SIM=1)
   endelse
   pout=[pout,transpose(win[0,5:6])]
   peout=[peout,transpose(datp.e[0,5:6])]
   pout=[transpose(pout),transpose(peout)]
   d17_errmsg,1,'FINAL PARAMETERS:'
   d17_errmsg,1,['Initial time          '+string(pout[0,0])+' +/- '+string(pout[1,0])]
   d17_errmsg,1,['Initial polarization  '+string(pout[0,1])+' +/- '+string(pout[1,1])]
   d17_errmsg,1,['Relaxation time       '+string(pout[0,2])+' +/- '+string(pout[1,2])]
   d17_errmsg,1,['Polarizer effficiency '+string(pout[0,3])+' +/- '+string(pout[1,3])]
   d17_errmsg,1,['Pressure              '+string(pout[0,4])+' +/- '+string(pout[1,4])]
   d17_errmsg,1,['Wavelength            '+string(pout[0,5])+' +/- '+string(pout[1,5])]
   d17_errmsg,1,['Flipper 1 efficiency  '+string(pout[0,6])+' +/- '+string(pout[1,6])]
   d17_errmsg,1,['Flipper 2 efficiency  '+string(pout[0,7])+' +/- '+string(pout[1,7])]
   d17_errmsg,1,['Chi^2 = '+string(chi2)]
   d17_errmsg,1,' '
   print,'Covariance matrix:'
   print,covout
end
