pro d17_polcalc,main,maskm=maskm,ferro=ferro,maskf=maskf,phi,F1,F2,p1,p2
;+
;	LAMP procedure to calculate the inefficiencies of the polarizing elements
;	based on a series of scans with different flipper states
;	The corrections are based on the equations in:
;	Wildes, Rev. Sci. Instrum., 70 (1999) 4241
;	Code based on d174flip.pro by RC
;	ARW 12.04.03
;
;	NECESSARY VARIABLES:
;	main= Can be either:
;	i)	an array of eight numbers corresponding to the integrated intensities 
;		measured for the main beam OR a reflection from a non-polarizing
;		sample (e.g. silicon), with their errors.
;		The format of the numbers is as follows:
;		[I00,e(00),I10,e(10),I11,e(11),I01,e(01)]
;	ii)	a string with files containing the same information
;		Syntax for file access is the same as for MLOAD
;
;	OPTIONAL VARIABLES:
;	ferro=Can be either:
;	i)	an array of four numbers corresponding to the integrated intensities
;		measured for the reflection from a ferromagnetic sample, with their errors
;		The format of the numbers is as follows:
;		[I00,e(00),I10,e(10),I11,e(11),I01,e(01)]
;	ii)	a string with files containing the same information
;		Syntax for file access is the same as for MLOAD
;	maskm=mask string for "main" files - useful if files are input
;	maskf=mask string for "ferro" files - useful if files are input
;
;	OUTPUTS:
;	All the outputs are two component arrays, the first element is the value and the second is the error
;	phi=a combination of the efficiencies of the polarizer and the analyser
;	F1=the efficiency of the 1st flipper
;	F2=the efficiency of the 2nd flipper
;	p1=the efficiency of the polarizer	(can only be determined if ferro is included)
;	p2=the efficiency of the analyser	(can only be determined if ferro is included)
;
;	e.g. type at the LAMP interface
;		d17_polcalc,[2206,10.2,20,2.1,2110,9.5,40,3.1],ferro=[1104,5.2,20,2.1,100,10.0,10,3.1],phi,F1,F2,p1,p2
;		d17_polcalc,'81624:81627',mainm='box,[63,73,11,51]'
;
;	CALLS TO: mload, d17_errmsg
;	last modified:	ARW 2.6.13
;-

forward_function mload
forward_function d17_errmsg

; Determine whether main has been entered as a string or as numbers
ntype=size(main,/TYPE)
if ntype eq 7 then begin
	if n_elements(maskm) ne 1 then maskm='none'
	win=mload(main,norm='time',mask=maskm)
	take_datp,datp
	ein=datp.e
	main=[total(win[*,*,0]),sqrt(total(ein[*,*,0]^2)),total(win[*,*,2]),sqrt(total(ein[*,*,2]^2)),total(win[*,*,3]),sqrt(total(ein[*,*,3]^2)),total(win[*,*,1]),sqrt(total(ein[*,*,1]^2))]
	d17_errmsg,1,'Type 1 intensities, normalized to time:'
	d17_errmsg,1,['I00 = '+string(main[0])+' +/- '+string(main[1])]
	d17_errmsg,1,['I10 = '+string(main[2])+' +/- '+string(main[3])]
	d17_errmsg,1,['I11 = '+string(main[4])+' +/- '+string(main[5])]
	d17_errmsg,1,['I01 = '+string(main[6])+' +/- '+string(main[7])]
endif

if n_elements(main) ne 8 then begin
	d17_errmsg,1,'ERROR: There must be eight numbers in the input array,'
	d17_errmsg,1,'format: [I00,e(00),I10,e(10),I11,e(11),I01,e(01)]'
	return
endif

mI00=main[0]
mI10=main[2]
mI11=main[4]
mI01=main[6]

me00=main[1]
me10=main[3]
me11=main[5]
me01=main[7]

;print,mI00,mI10,mI11,mI01

; ***	Calculate phi
numer=((mI11+mI01-mI00)*mI00+(mI00-2*mI01)*mI10)
denom=2*(mI11*mI00-mI10*mI01)
enumer=sqrt(((mI11+mI01-2*mI00+mI10)*me00)^2+(mI00*me11)^2+((mI00-2*mI10)*me01)^2+((mI00-2*mI01)*me10)^2)
edenom=sqrt((mI11*me00)^2+(mI00*me11)^2+(mI10*me01)^2+(mI01*me10)^2)
phi=numer/denom
ephi=0.5*sqrt((enumer*phi/numer)^2+(edenom*phi/denom)^2)

; ***	Calculate F1
numer=mI00+mI10-mI01-mI11
denom=2*(mI00-mI01)
enumer=sqrt(me00^2+me10^2+me01^2+me11^2)
edenom=sqrt(me00^2+me01^2)
F1=numer/denom
eF1=0.5*sqrt((enumer*F1/numer)^2+(edenom*F1/denom)^2)

; ***	Calculate F2 (NOTE: enumer remains unchanged)
numer=mI00+mI01-mI10-mI11
denom=2*(mI00-mI10)
edenom=sqrt(me00^2+me10^2)
F2=numer/denom
eF2=0.5*sqrt((enumer*F2/numer)^2+(edenom*F2/denom)^2)

; ***	Now determine the individual polarizer and analyser efficiencies
; ***	The code assumes that the phi, F1 and F2 remain constant between the two scans.

if n_elements(ferro) ne 0 then begin
; ***	First, determine that all the data are present
	; Determine whether main has been entered as a string or as numbers
	ntype=size(main,/TYPE)
	if ntype eq 7 then begin
		if n_elements(maskf) ne 1 then maskf='none'
		fwin=mload(main,norm='time',mask=maskf)
		take_datp,fdatp
		fein=fdatp.e
		ferro=[total(fwin[*,*,0]),sqrt(total(fein[*,*,0]^2)),total(fwin[*,*,2]),sqrt(total(fein[*,*,2]^2)),total(fwin[*,*,3]),sqrt(total(fein[*,*,3]^2)),total(fwin[*,*,1]),sqrt(total(fein[*,*,1]^2))]
		d17_errmsg,1,'Type 2 intensities, normalized to time:'
		d17_errmsg,1,['I00 = '+string(ferro[0])+' +/- '+string(ferro[1])]
		d17_errmsg,1,['I10 = '+string(ferro[2])+' +/- '+string(ferro[3])]
		d17_errmsg,1,['I11 = '+string(ferro[4])+' +/- '+string(ferro[5])]
		d17_errmsg,1,['I01 = '+string(ferro[6])+' +/- '+string(ferro[7])]
	endif
	if n_elements(ferro) ne 8 then begin
		d17_errmsg,1,'ERROR: There must be eight numbers in the ferro array,'
		d17_errmsg,1,'format: [I00,e(00),I10,e(10),I11,e(11),I01,e(01)]'
		return
	endif

	fI00=ferro[0]
	fI10=ferro[2]
	fI11=ferro[4]
	fI01=ferro[6]

	fe00=ferro[1]
	fe10=ferro[3]
	fe11=ferro[5]
	fe01=ferro[7]
		
	fa=2*((2*F1-1)*fI00+(1-2*F1)*fI01-fI10+fI11)
	ea=2*sqrt(((2*F1-1)*fe00)^2 + ((1-2*F1)*fe01)^2 + fe10^2 + fe11^2 + (2*(fI00-fI01)*eF1)^2 )
	fb=-fa
	fc=(F1-F2+phi*(2*F2-1))*fI00+((1-phi)*(1-F1)-phi*F1)*fI01-(phi*F2+(1-phi)*(1-F2))*fI10+phi*fI11
	ec=((F1-F2+phi*(2*F2-1))*fe00)^2
	ec=ec+(((1-phi)*(1-F1)-phi*F1)*fe01)^2
	ec=ec+((phi*F2+(1-phi)*(1-F2))*fe10)^2 + (phi*fe11)^2
	ec=ec+((fI00-fI01)*eF1)^2
	ec=ec+(((2*phi-1)*fI00+fI10)*eF2)^2
	ec=ec+(((2*F2-1)*fI00-fI01+fI10+fI11)*ephi)^2
	ec=sqrt(ec)
		
;	print,fa,ea,fc,ec
;	print,(-fb+sqrt(fb^2-4*fa*fc))/(2*fa),(-fb-sqrt(fb^2-4*fa*fc))/(2*fa)
	p1=(-fb+sqrt(fb^2-4*fa*fc))/(2*fa)
	p2=(phi-p1)/(1-2*p1)
	
	ep1=( ((fa-2*fc)/(2*fa*sqrt(fa^2-4*fa*fc)) - sqrt(fa^2-4*fa*fc)/(2*fa^2))*ea)^2
	ep1=sqrt(ep1+(ec/sqrt(fa^2-4*fa*fc))^2)
	ep2=sqrt((ephi/(1-2*p1))^2+((2*p2-1)*ep1/(1-2*p1))^2)

endif

d17_errmsg,1,['Flipper 1 efficiency = '+string(100*(1-F1))+'% +/- '+string(100*eF1)]
d17_errmsg,1,['Flipper 2 efficiency = '+string(100*(1-F2))+'% +/- '+string(100*eF2)]
d17_errmsg,1,['Combined Polarizer-Analyser efficiency = '+string(100*(1-phi))+'% +/-'+string(100*ephi)]

phi=[phi,ephi]
F1=[F1,eF1]
F2=[F2,eF2]

if n_elements(ferro) ne 0 then begin
	d17_errmsg,1,['Polarizer efficiency ='+string(100*(1-p1))+'% +/- '+string(100*ep1)]
	d17_errmsg,1,['Analyzer efficiency  ='+string(100*(1-p2))+'% +/- '+string(100*ep2)]
	p1=[p1,ep1]
	p2=[p2,ep2]
endif

;	print,mI00,mI10,mI11,mI01
;	print,me00,me10,me11,me01
;	print,fI00,fI10,fI11,fI01
;	print,fe00,fe10,fe11,fe01

end
