function d17_polcorr,win,phi=phi,F1=F1,F2=F2,p1=p1,p2=p2
;+
;	LAMP function to take up to four workspaces, one for each flipper state,
;	and to correct for the polarizing inefficiencies.
;	The corrections are based on the equations in:
;	Wildes, Rev. Sci. Instrum., 70 (1999) 4241
;	ARW 31.03.04
;
;	NECESSARY INPUT:
;	win=input workspace.  This can be a 4 rows x N columns matrix, 
;		where N is the number of 2 theta values.  The order of the
;		rows *must* be (from the top) I00, I01, I10, I11
;		It is assumed that each column gives the values of I measured
;		at *the same* position in theta
;		'mload' will automatically create an array of the right dimensions
;		You can create such a matrix from four 1d datasets using the 
;		aw_four1d2one2d function
;				OR, it can be a 2 rows x N columns matrix for
;		those measurements without an analyser (i.e. for 'asymmetry')
;		The order of the rows *must* be (from the top) I0, I1 
;		
;	SEMI-NECESSARY INPUT:
;	All this input is in the form of a two-element array, 
;		the first number is the value of the parameter
;		the second number is its error
;	The program aw_polcalc returns the values in the right format
;	Depending on the type of measurement, you must include some combination
;	of the following:
;	p1=inefficiency of the polarizer (between 0 and 1)
;	p2=inefficiency of the analyzer (between 0 and 1)
;	F1=inefficiency of the first flipper (between 0 and 1)
;	F2=inefficiency of the second flipper (between 0 and 1)
;	phi=p1+p2-2*p1*p2 (between 0 and 1)
;	NOTE:  If you enter p1 and/or p2, the program will IGNORE phi! 
;
;	OUTPUT:
;	The workspace will be either a 4 rows x N columns matrix, 
;		the order of the rows will be (from the top) SIGMA(++),SIGMA(+-),SIGMA(-+),SIGMA(--)
;	Or the workspace will be a 2 rows x N columns matrix, 
;		the order of the rows will be (from the top) SIGMA(+),SIGMA(-)
;
;	e.g. type at LAMP interface
;		w6=d17_polcorr(w5,F1=[0.007,0.0008],F2=[0.005,0.0008],p1=[0.05,0.002],p2=[0.011,0.002])
;			(good for ferro and ferrimagnets)
;		w6=d17_polcorr(w5,F1=[0.007,0.0008],F2=[0.005,0.0008],phi=[0.05,0.002])
;			(good for antiferro and paramagnets)
;		w6=d17_polcorr(w5,F1=[0.007,0.0008],p1=[0.05,0.002])
;			(good for spin asymmetry)
;
;	CALLS TO: d17_errmsg
;	last modified:	ARW 17.12.10
;-

;***	Get important information from the workspace:
TAKE_DATP,datp
ein=datp.e

;***	Check out the number of rows in the workspace.  Expand to 4 rows if it only contains two
winsize=size(win)

if winsize[2] ne 4 then begin
	if winsize[2] ne 2 then begin
		d17_errmsg,1,'ERROR: Wrong number of rows in the input workspace.  It must have either 2 or 4 rows'
		return,0
	endif
endif
if winsize[2] eq 2 then begin
	windat=[[win[*,0]],[win[*,0]],[win[*,1]],[win[*,1]]]
	ein=[[ein[*,0]],[fltarr(n_elements(ein[*,1]))],[ein[*,1]],[fltarr(n_elements(ein[*,1]))]]
endif else windat=win

;***	Check the existence of the flipper efficiencies
if n_elements(F1) ne 2 then begin
	d17_errmsg,1,'WARNING: F1 not correctly entered.  Assuming first flipper is 100% efficient'
	F1=[0.0,0.0]
endif
if n_elements(F2) ne 2 then begin
	d17_errmsg,1,'WARNING: F2 not correctly entered.  Assuming second flipper is 100% efficient'
	F2=[0.0,0.0]
endif

;***	Check for polarizer efficiencies
p1flag=0
p2flag=0
phiflag=0
if n_elements(p1) eq 2 then p1flag=1
if n_elements(p2) eq 2 then p2flag=1
if n_elements(phi) eq 2 then phiflag=1

if (p1flag eq 0) and (p2flag eq 0) and (phiflag eq 0) then begin
	d17_errmsg,1,'WARNING: No polarizer or analyser inefficiencies have been entered.  Assuming 100% efficiency'
	p1=[0.0,0.0]
	p2=[0.0,0.0]
	phi=[0.0,0.0]
endif
if (p1flag eq 1) and (p2flag eq 0) and (phiflag eq 0) then begin
	d17_errmsg,1,'WARNING: p2 not correctly entered.  Assuming 100% analyser efficiency'
	p2=[0.0,0.0]
	phi=[0.0,0.0]
endif
if (p1flag eq 1) and (p2flag eq 0) and (phiflag eq 1) then begin
	d17_errmsg,1,'Calculating analyser inefficiency from p1 and phi'
	p2=(phi[0]-p1[0])/(1-2*p1[0])
	p2=[p2,sqrt((phi[1]/(1-2*p1[0]))^2+(p1[1]*(2*phi[0]-1)/(1-2*p1[0])^2)^2)]
endif
if (p1flag eq 0) and (p2flag eq 1) and (phiflag eq 1) then begin
	d17_errmsg,1,'Calculating polarizer inefficiency from p2 and phi'
	p1=(phi[0]-p2[0])/(1-2*p2[0])
	p1=[p1,sqrt((phi[1]/(1-2*p2[0]))^2+(p2[1]*(2*phi[0]-1)/(1-2*p2[0])^2)^2)]
endif
if (p1flag eq 0) and (p2flag eq 0) and (phiflag eq 1) then begin
	d17_errmsg,1,'WARNING: Only phi is defined.  Correction is only good for certain experiments'
	p1=0.5*(1-sqrt(1-2*phi[0]))
	p1=[p1,phi[1]/(sqrt(2.0)*(1-2*p1))]
	p2=p1
endif

;print,[[p1],[p2],[F1],[F2]]

;***	Calculate the matrices
F1mat=1/(1-F1[0])*[[1-F1[0],0,0,0],[0,1-F1[0],0,0],[-F1[0],0,1.0,0],[0,-F1[0],0,1.0]]
F2mat=1/(1-F2[0])*[[1-F2[0],0,0,0],[-F2[0],1.0,0,0],[0,0,1-F2[0],0],[0,0,-F2[0],1.0]]
P1mat=1/(1-2*p1[0])*[[1-p1[0],0,-p1[0],0],[0,1-p1[0],0,-p1[0]],[-p1[0],0,1-p1[0],0],[0,-p1[0],0,1-p1[0]]]
P2mat=1/(1-2*p2[0])*[[1-p2[0],-p2[0],0,0],[-p2[0],1-p2[0],0,0],[0,0,1-p2[0],-p2[0]],[0,0,-p2[0],1-p2[0]]]

eF1mat=F1[1]/(1-F1[0])^2*[[0,0,0,0],[0,0,0,0],[-1.0,0,1.0,0],[0,-1.0,0,1.0]]
eF2mat=F2[1]/(1-F2[0])^2*[[0,0,0,0],[-1.0,1.0,0,0],[0,0,0,0],[0,0,-1.0,1.0]]
eP1mat=p1[1]/(1-2*p1[0])^2*[[1.0,0,-1.0,0],[0,1.0,0,-1.0],[-1.0,0,1.0,0],[0,-1.0,0,1.0]]
eP2mat=p2[1]/(1-2*p2[0])^2*[[1.0,-1.0,0,0],[-1.0,1.0,0,0],[0,0,1.0,-1.0],[0,0,-1.0,1.0]]

MAT=P2mat ## P1mat ## F2mat ## F1mat

;***	Calculate the corrected counts
wout=fltarr(winsize[1],4)
eout=wout

for i=0,winsize[1]-1 do begin
	dI=windat[i,*]
	eI=ein[i,*]
	wout[i,*]=MAT ## dI
	eout[i,*]=(eP2mat ## P1mat ## F2mat ## F1mat ## dI)^2
	eout[i,*]=eout[i,*]+(P2mat ## eP1mat ## F2mat ## F1mat ## dI)^2
	eout[i,*]=eout[i,*]+(P2mat ## P1mat ## eF2mat ## F1mat ## dI)^2
	eout[i,*]=eout[i,*]+(P2mat ## P1mat ## F2mat ## eF1mat ## dI)^2
	eout[i,*]=eout[i,*]+MAT^2 ## eI^2
endfor

;***	If the workspace started with 2 rows, reduce it back to 2 rows
if winsize[2] eq 2 then begin
	wout=[[wout[*,0]],[wout[*,2]]]
	eout=[[eout[*,0]],[eout[*,2]]]
endif

eout=sqrt(eout)
MOD_DATP,datp,"e",eout

GIVE_DATP,datp
RETURN,wout
end
