;+
;function d17_qxspec,win,mbeam,spix,bkg=bkg,lambda=lambda,sanoff=sanoff
;
;	LAMP function to extract the specular intensity from a workspace.
;	This function is better when the substrate is bent, 
;		or there is significant off-specular scattering at constant Qz
;	This function converts the raw data space from
;		x-pixel vs. SAN/DAN
;	to:
;		pi-pf vs pi+pf
;	It then rebins the data so that cuts will be taken at constant pi+pf
;	ARW 08.09.09
;	
;	NECESSARY VARIABLES:
;	win = the input workspace.  Monochromatic data, loaded through 'mload'
;	mbeam = either a file number or a workspace with the main beam
;	spix = the number of pixels that will be integrated over to calculate 
;		the specular intensity.  Ideally, this should be an odd number
;
;	OPTIONAL VARIABLES:
;	bkg = two column times n row element array that define the background
;		The first row gives the number of pixels between the specular pixel range and the background range
;			A NEGATIVE value is to the LEFT of the specular range, a POSITIVE to the right
;		The second row gives the number of pixels in the background range
;	lambda=wavelength.  This is stored with the data, but you have the
;		option here of forcing it to another value
;	sanoff=an offset in SAN, which can exist if the sample angle calibration
;		is out.  This value is ADDED to the instrument-stored value
;		of SAN to find the correct value for theta_in
;
;	OUTPUT VARIABLES
;	wout is the workspace containing the specular intensity, background subtracted,
;		with the x-axis = the y axis for win
;
;	e.g. type at LAMP interface:
;		w2=d17_qxspec(w1,00935,11,,sanoff='auto',bkg=[[-5,5],[5,5]])
;
;	CALLS TO:
;	Last modified:  ARW 12.10.09
;-

;************************************************************
;*** Define the procedures for interpolating and integrating ***
;************************************************************
pro sinterp,yin,win,ein,yout,wout,eout
;+
;** *******
;*** This one for the specular scattering
;*** This makes heavy use of the IDL interpolate function
;*** The input data is a parallelogram - need to interpolate on to the central pixel
;-

; ** Taking the Qz values from the central pixel 
arrsize=size(win,/DIMENSIONS)
yinter=yin[floor(arrsize[0]/2),*]

;*** Can only apply the interpolation where the parallelogram has the full width
ylims=[min(yin[0,*]),max(yin[arrsize[0]-1,*])]
yout=[[ylims[0]],[transpose(yinter[where(yinter gt ylims[0] and yinter lt ylims[1])])],[ylims[1]]]
;*** This range is smaller than the data, so there should be no NaNs

;*** Now interpolate the data and errors on to yout, column by column
wtemp=fltarr(arrsize[0],n_elements(yout),arrsize[2])
etemp=wtemp
for i=0,arrsize[0]-1 do begin
	for j=0,arrsize[2]-1 do begin
		wtemp[i,*,j]=interpol(win[i,*,j],yin[i,*],yout)
		etemp[i,*,j]=interpol(ein[i,*,j],yin[i,*],yout)
	endfor
endfor

;*** Now have a rectangular array and can simply sum the data
wout=total(wtemp,1)
eout=sqrt(total(etemp^2,1))

end

pro binterp,yin,win,ein,yout,spix,wout,eout,n
;+
;** *******
;*** This one for the background
;*** This makes heavy use of the IDL interpolate function
;*** The data will be a number of 'parallelograms'
;*** yout is fed in to the dataset and comes from sinterp
;*** Data will be normalized to number of pixels, taken from spix
;-

; ** Define array sizes
arrsize=size(win,/DIMENSIONS)
wtemp=fltarr(arrsize[0],n_elements(yout),arrsize[2])
etemp=wtemp
n=0*indgen(n_elements(yout))

;*** Run through the data column by column and interpolate on to yout (where possible)
for i=0,arrsize[0]-1 do begin
	ytempin=where(yin[i,*] ge min(yout) and yin[i,*] le max(yout))
	ytempout=where(yout ge min(yin[i,*]) and yout le max(yin[i,*]))
	for j=0,arrsize[2]-1 do begin
		wtemp[i,ytempout,j]=interpol(win[i,ytempin,j],yin[i,ytempin],yout[ytempout])
		etemp[i,ytempout,j]=interpol(ein[i,ytempin,j],yin[i,ytempin],yout[ytempout])
	endfor
endfor


;*** Now have a rectangular array and can simply sum the data
wout=total(wtemp,1)
eout=sqrt(total(etemp^2,1))

;*** Still need to normalise to the number of non-zero pixels.
spix=float(spix)
for i=0,n_elements(yout)-1 do begin
	n[i]=float(n_elements(where(etemp[*,i,0] ne 0.0)))
	wout[i,*]=spix[0]/n[i]*wout[i,*]
	eout[i,*]=spix[0]/n[i]*eout[i,*]
endfor

end

;********************************
;*** And finally the function ***
;********************************
function d17_qxspec,win,mbeam,spix,bkg=bkg,lambda=lambda,sanoff=sanoff
;+
;******* **********
;** User callable
;-

forward_function d17_xyconvert

;*** Get important information from the input workspace
TAKE_DATP,datp

;*** Check that the input parameters have the right number of variables
;*** First SANOFF
if n_elements(sanoff) eq 0 then begin
	print,'No SANOFF entered.  SANOFF will be automatically determined'
	print,'(set sanoff = 0.0 if you wish to turn this off)'
	sanoff='auto'
endif else begin
	if n_elements(sanoff) ne 1 then begin
		print,'WARNING: sanoff must be a single number.  Setting sanoff = 0.0'
		sanoff=0.0
	endif
endelse
;*** Now LAMBDA
if n_elements(lambda) eq 0 then begin
	lambda=datp.p[24]
endif else begin
	if n_elements(lambda) ne 1 then begin
		print,'WARNING: lambda must be a single number.  Setting lambda from the parameter set in MAD'
		lambda=datp.p[24]
	endif
endelse
;*** Now BKG
if n_elements(bkg) ne 0 then begin
	nbkg=size(bkg,/DIMENSIONS)
	if nbkg[0] ne 2 then begin
		print,'WARNING: BKG is incorrectly entered.  It must have 2 columns.'
		print,'No background will be subtracted'
		nbkg=0
	endif
endif else begin
	print,'No background defined.  Background is not subtracted'
	nbkg=0
endelse


;*** Convert workspace in to 'pipf'
wc=d17_xyconvert(win,'pipf',mbeam,lambda=lambda,sanoff=sanoff,datp=datp)

;*** Get important information from the (converted) input workspace
ein=datp.e
xin=datp.x
yin=datp.y
zin=datp.z

nels=n_elements(yin[0,*])

;*** Initialize arrays
;*** These are for specular
ycr=fltarr(spix,nels)
xcr=ycr
wcr=fltarr(spix,nels,n_elements(zin))
ecr=wcr
;*** These are for background
if nbkg[0] ne 0 then begin
	hspix=floor(spix/2)
	if bkg[0,0] gt 0 then bst=bkg[0,0]+1+hspix else bst=bkg[0,0]-bkg[1,0]-hspix
	bels=bst+indgen(bkg[1,0])
	for j=1,n_elements(bkg[0,*])-1 do begin
	      	if bkg[0,j] gt 0 then bst=bkg[0,j]+1+hspix else bst=bkg[0,j]-bkg[1,j]-hspix
		bels=[bels,bst+indgen(bkg[1,j])]
	endfor
	bycr=fltarr(n_elements(bels),nels)
	bxcr=bycr
	bwcr=fltarr(n_elements(bels),nels,n_elements(zin))
	becr=bwcr		
endif

;*** Now extract the data.  
;*** Run thru the data row by row and pick out the correct range of pixels 
for i=0,nels-1 do begin
;*** First, find the pixel in each row with the pi-pf closest to 0
	specpix=where(abs(xin[*,i]) eq min(abs(xin[*,i])))
;*** Then pick out the x-pixels centred on this point.  This is the specular data
	srng=specpix[0]-floor((spix-1)/2)+indgen(spix)
;	yout(i)=yin(specpix,i)
	xcr[*,i]=xin[srng,i]
	ycr[*,i]=yin[srng,i]
	wcr[*,i,*]=wc[srng,i,*]
	ecr[*,i,*]=ein[srng,i,*]
;*** Now pick out background pixels
	if nbkg[0] ne 0 then begin
		brng=specpix[0]+bels
		bycr[*,i]=yin[brng,i]
		bwcr[*,i,*]=wc[brng,i,*]
		becr[*,i,*]=ein[brng,i,*]
	endif
endfor

;*** Now carry out interpolations for the specular
sinterp,ycr,wcr,ecr,yspec,wspec,espec

;*** Now carry out the interpolations for the background
wback=0*wspec
eback=wback
if nbkg[0] ne 0 then binterp,bycr,bwcr,becr,yspec,spix,wback,eback,nback
;print,nback

;*** Now subtract the background
yout=yspec
;wout=[[wspec],[wback]]
;eout=[[espec],[eback]]
wout=wspec-wback
eout=sqrt(espec^2+eback^2)

;*** Write the new parameters
MOD_DATP,datp,"e",eout
MOD_DATP,datp,"x",yout
MOD_DATP,datp,"y",1
MOD_DATP,datp,"z",1
MOD_DATP,datp,"x_tit",datp.y_tit
MOD_DATP,datp,"y_tit",'Intensity (arb. units)'

GIVE_DATP,datp
RETURN,wout

end
