pro d17_save,win,outfile,header=header,outformat=outformat
;+
;	LAMP procedure to save a dataset with arbitrary X and Y parameters
;	The dataset can have one, two or three dimensions
;	The default output file will contain:
;		an array with the Z values (the number of arrays stacked on top of each other)
;		an array with the X values
;		an array with the Y values
;		an array with the data
;		an array with the errors on the data
;	Note that the array sizes for X, Y, and the data can all be different
;		but that the data and errors will have the same array size
;	It also assumes that matrices, if stacked, all have the same X- and Y-values
;	ARW 04.10.04
;
;	NECESSARY INPUT:
;	win=the input workspace
;	outfile=the output file name (as a string)
;
;	OPTIONAL VARIABLES:
;	header=a string that will be printed in to the file
;       outformat=a string that permits the output format to be changed
;		The default output format is noted above, and is best for 2D datasets
;		Other supported formats are:
;			'xye'	requires 1D datasets with one set of x-values, e.g. specular data
;				Will transpose the arrays, and will save with columns:
;						x  y1 e(y1) y2 e(y2) etc.
;				Will not save Z or Y
;
;	e.g. type at LAMP interface:
;		d17_save,w10,'delme',header='Throw me away'
;
;	Calls to: d17_errmsg
;	Last modified:  ARW 17.12.10
;-

; ***	Format for output.  Change this if desired
fmatdef='(g20.10," "))'


; *** Check for output format.  Uses a variable called 'outflag'  Values for outflag are:
;		outflag=0			Default
;		outflag=1			xye
outflag=0
if n_elements(outformat) gt 0 then begin
	nformats=1
	outflag=0*indgen(1+nformats)
	outflag[1]=strcmp(outformat,'xye',/FOLD_CASE)
	if total(outflag) eq 0 then begin
		d17_errmsg,1,'WARNING:  Output format does not correspond.  Default output format will be used'
		outflag=0
	endif else outflag=where(outflag ne 0)
endif

; ***	Open the output file for writing
if strlen(outfile) le 0 then begin
	d17_errmsg,1,'ERROR:  The output filename must be given as a string'
	return
endif
get_lun,unit
openw,unit,outfile
if n_elements(header) gt 0 then printf,unit,header
printf,unit,' '

; ***	Define sizes and check for consistency
emess=0
wsize=size(win)
TAKE_DATP,datp
err=datp.e
esize=size(err)
; ***	Check error size against workspace size.  Error arrays can be non-existant, or the same size as the workspace
if (esize[0] ne 0) then begin
	if total(wsize[0:n_elements(wsize)-3]-esize[0:n_elements(esize)-3]) ne 0 then begin
		d17_errmsg,1,'ERROR: workspace array and error array have different sizes'
		help,win,err
		emess=1
	endif
endif else print,'WARNING: workspace error not defined.  Errors will not be saved'
; ***	Check x-axis against workspace size.
; *** 	X can be non-existant, a 1D array of the same size as the workspace or a 2D array with the in-plane dimensions (not including z)
xdata=datp.x
xsize=size(xdata)
if xsize[0] ne 0 then begin
	if (xsize[0] ne 1) and (total(xsize[1:2]) ne total(wsize[1:2])) then begin
		print,xsize,wsize
		d17_errmsg,1,'ERROR: workspace array size and x-axis array size do not correspond'
		help,win,xdata
		emess=2
	endif else begin
		if (xsize[0] eq 1) and (xsize[1] ne wsize[1]) then begin
			d17_errmsg,'ERROR: Discrepancy between the number of columns in workspace and the number of values on x-axis'
			help,win,xdata
			emess=3
		endif
		if (xsize[0] eq wsize[0]) and (total(xsize-wsize) ne 0) then begin
			d17_errmsg,1,'ERROR: workspace array and x-axis array have different sizes'
			help,win,xdata
			emess=4
		endif
	endelse
endif else d17_errmsg,1,'WARNING: X-axis not defined.  Values will not be saved'
; ***	Check y-axis against workspace size.
; ***	Y can be non-existant, a 1D array of the same size as the workspace or a 2D array with the in-plane dimensions (not including z)
ydata=datp.y
ysize=size(ydata)
if ysize[0] ne 0 then begin
	if (ysize[0] ne 1) and (total(ysize[1:2]) ne total(wsize[1:2])) then begin
		d17_errmsg,1,'ERROR: workspace array size and y-axis array size do not correspond'
		help,win,ydata
		emess=5
	endif else begin
		if (ysize[0] eq 1) and (ysize[1] ne wsize[2]) then begin
			d17_errmsg,1,'ERROR: Discrepancy between the number of columns in workspace and the number of values on y-axis'
			help,win,ydata
			emess=6
		endif
		if (ysize[0] eq wsize[0]) and (total(ysize-wsize) ne 0) then begin
			d17_errmsg,1,'ERROR: workspace array and y-axis array have different sizes'
			help,win,ydata
			emess=7
		endif
	endelse
endif else print,'WARNING: Y-axis not defined.  Values will not be saved'
; ***	Check z-axis against workspace size.
; ***	Z can be non-existant, or a 1D array of the same size as the workspace dimension
zdata=datp.z
zsize=size(zdata)
if zsize[0] ne 0 then begin
	if (zsize[0] eq 1) and (zsize[1] ne wsize[3]) then begin
		d17_errmsg,1,'ERROR: Discrepancy between the number of stacked arrays in workspace and the number of values on z-axis'
		help,win,zdata
		emess=8
	endif
endif else d17_errmsg,1,'WARNING: Z-axis not defined.  Values will not be saved'

; ***	And tidy up if there has been one or more errors
if emess ne 0 then begin
	close,unit
	free_lun,unit
	return
endif

; ***	Turn zdata in to a number if it is a NaN
NaNcheck=where(finite(zdata,/NAN) eq 1,count)
if count ne 0 then zdata=1

; ***	Format for output.
fmat='('+string(wsize[1])+fmatdef
yfmat=fmat

; *** Now establish which output format to use
case outflag[0] of

0:	begin		; *** Default output
; ***	Write the z-values to the output file
	if zsize[0] ne 0 then begin
		zsize=zsize[1]
		printf,unit,'The Z-axis is given by ',datp.z_tit
		printf,unit,'Z-axis: ',zdata
	endif else zsize=1

; ***	Write the x-values to the output file
	if xsize[0] ne 0 then begin
		xsave=[1,1,1]
		xsave[0:xsize[0]-1]=xsize[1:xsize[0]]
		printf,unit,'The X-axis is equivalent to ',datp.x_tit
		printf,unit,'X-axis: (',xsave[0],' columns by ',xsave[1],' rows)'
		for i=0,xsave[1]-1 do begin
			printf,unit,xdata[*,i],FORMAT=fmat
		endfor
	endif

; ***	Write the y-values to the output file
	if ysize[0] ne 0 then begin
		ysave=[1,1,1]
		ysave[0:ysize[0]-1]=ysize[1:ysize[0]]
		printf,unit,'The Y-axis is equivalent to ',datp.y_tit
		printf,unit,'Y-axis: (',ysave[0],' columns by ',ysave[1],' rows)'
		if ysave[0] eq 1 then ysave[1] = 1
		if ysave[0] gt wsize[1] then yfmat='('+string(ysave[0])+fmatdef
		for i=0,ysave[1]-1 do printf,unit,ydata[*,i],FORMAT=yfmat
	endif


; ***	Write the data to the output file
	wsave=[1,1,1]
	wsave[0:wsize[0]-1]=wsize[1:wsize[0]]
	printf,unit,'Data: (',wsave[2],' arrays of ',wsave[0],' columns by ',wsave[1],' rows)'
	for j=0,zsize[0]-1 do begin
		printf,unit,'Data in Array ',j+1,', z=',zdata[j]
		for i=0,wsave[1]-1 do printf,unit,win[*,i,j],FORMAT=fmat
	endfor

; ***	Write the errors to the output file
	if esize[0] ne 0 then begin
		printf,unit,'Error: (',wsave[2],' arrays of ',wsave[0],' columns by ',wsave[1],' rows)'
		for j=0,zsize[0]-1 do begin
			printf,unit,'Error in Array ',j+1,', z=',zdata[j]
			for i=0,wsave[1]-1 do printf,unit,err[*,i,j],FORMAT=fmat
		endfor
	endif
	end

1:	begin			; *** xye
; *** Collect the x-values, y-values, data and error and transpose them
	ox=transpose(xdata)		; Assumes that x and y are row arrays
;	ow=rotate(win,1)		; Data and errors can be a 2D array, but must not have a z component
;	oe=rotate(win,1)
	ow=transpose(win)		; Data and errors can be a 2D array, but must not have a z component
	oe=transpose(err)
; *** Write a column header
	colhead=' x '
	for i=0,n_elements(ydata)-1 do colhead=[colhead,string(ydata[i]),'e'+strtrim(string(ydata[i]),1)] 
	printf,unit,colhead
; *** Create the output array
	wsave=ox
	for i=0,n_elements(ydata)-1 do wsave=[wsave,ow[i,*],oe[i,*]]
; *** And write it to the file
	for i=0,n_elements(ox)-1 do printf,unit,wsave[*,i],FORMAT=fmat
	end

endcase

close,unit
free_lun,unit
d17_errmsg,1,'Data correctly saved in to '+outfile

end
