;+
;function d17_xspec,win,spix,bkg=bkg,method=method,mbeam=mbeam,snorm=snorm,xaxis=xaxis,lambda=lambda,sanoff=sanoff
;
;       LAMP function to extract the specular intensity from a workspace.
;       It also (correctly) calculates the errors and weights by the number of pixels
;               in each range
;       ARW 6.11.09
;
;       NECESSARY VARIABLES:
;       win = the input workspace, loaded through mload
;       spix = two element array with the left and right pixels that limit the
;               width of the specular ridge
;
;       OPTIONAL VARIABLES:
;       bkg = two column times n row element array that define the background pixels
;       method = a string stating the type of background subtraction you would like to do.
;               May also involve rebinning the data.
;               The choices are:
;                       'smpl' = keeps the data in (x-pixel, san) space,
;                                       integrates the counts within a certain range of x-pixels for each row,
;                                       and subtracts backgrounds calculated by row from the data .
;                                This is useful for narrow, well-defined specular reflectivity with no significant off-specular.
;                       'tth' = Converts the data in (twotheta, san) space,
;                                       assumes that theta_in = twotheta/2 and calculates Qz based on this,
;                                       but subtracts backgrounds calculated by row from the data.
;                                This is useful for messy reflectivity peaks with no significant off-specular
;                                       (e.g. bent and/or faceted substrates)
;                                This option will rebin the data in Qz.
;                       'q' = converts the data in to (pi-pf, pi+pf) space,
;                                       interpolates the data on to the central pixel,
;                                       and subtracts backgounds interpolated on to the same Qz.
;                               This is useful for samples with large off-specular
;                                       (e.g. rough multilayers)
;                               This option will rebin the data in Qz.
;       mbeam = the number with a main beam measurement.
;               This is a NECESSARY VARIABLE if 'tth' or 'q' are chosen for the method
;       snorm = normalization to slit sizes.  A string input is required.
;               Two options are possible:
;                       'none'= no normalization (default choice)
;                       'q'   = divides the intensities by Q (equivalent to dividing by sin(theta))
;                       's2'  = divides the intensities by S2W
;                       's2q' = divides the intensities by S2W*Q (equivalent to dividing by sin(theta))
;                       's2s3'= divides by S2W*S3W
;       xaxis = a choice of the final x-axis values
;               You may choose the x-axis in Qz (xaxis = 'q')   (default choice)
;                                         or theta_in (xaxis='san')
;       lambda = the wavelength.  If unspecified, the wavelength will be taken from the data parameters
;       sanoff = an offset in the sample angle in degrees.
;               This will be ADDED to the recorded sample angle
;               You may specify 'auto' here for the program to guess for you
;               Default is sanoff=0 for 'smpl', and is 'auto' for 'tth' and 'q'
;
;       OUTPUT VARIABLES
;       wout is the workspace containing the specular intensity, background subtracted,
;               with the x-axis = the y axis for win
;
;       e.g. type at LAMP interface:
;               w2=d17_xspec(w1,[60,85],bkg=[11,59],method='tth',mbeam=22189,xaxis='q',lambda=5.513,sanoff='auto')
;
;       CALLS TO:  d17_xyconvert, d17_errmsg
;       Last modified:  AW 1.4.13
;-

;+
; ****************************************************************
; ***  Define procedure for returning an error ***
; ****************************************************************
;-
pro aw_errmsg,errin,errout
        case errin of
                0: print,'ERROR: You must enter 2 values for the specular range, e.g. [60,85]'
                1: begin
                        d17_errmsg,1,'ERROR: You must enter a 2 column x n row array for the background'
                        d17_errmsg,1,'  e.g. bkg=[11,59]'
                        d17_errmsg,1,'          or   bkg=[[11,59],[200,210]]'
                end
                3: d17_errmsg,1,'ERROR: If you choose a method other than smpl, you must include a main beam'
                4: d17_errmsg,1,'ERROR: spix values are outside the range of your input workspace'
                5: begin
                        d17_errmsg,1,'The input workspace was created with an old version of mload'
                        d17_errmsg,1,'Please update the D17 LAMP library and reload the data using mload'
                end
                11: begin
                        d17_errmsg,1,'WARNING: Incorrect method entered'
                        d17_errmsg,1,'          Valid options are:'
                        d17_errmsg,1,'          smpl = keeps the data in (x-pixel, san) space'
                        d17_errmsg,1,'          tth = converts the data to (twotheta,san) space'
                        d17_errmsg,1,'          q = converts the data to (pi-pf, pi+pf) space'
                        d17_errmsg,1,'Using default method =smpl'
                        errout='smpl'
                end
                12: begin
                        d17_errmsg,1,'WARNING: Final x-axis choice is not correctly entered'
                        d17_errmsg,1,'          Setting final x-axis to Qz'
                        errout='q'
                end
                13: begin
                        d17_errmsg,1,'WARNING: sanoff must be a single number.  Setting sanoff = 0.0'
                        errout=0.0
                end
                14: begin
                        d17_errmsg,1,'WARNING: Incorrect slit normalization entered'
                        d17_errmsg,1,'          Valid options are:'
                        d17_errmsg,1,'          none =  no normalization'
                        d17_errmsg,1,'          q    =  divides data by Q'
                        d17_errmsg,1,'          s2   =  divides data by S2W'
                        d17_errmsg,1,'          s2q  =  divides data by S2W * Q'
                        d17_errmsg,1,'          s2s3 =  divides data by S2W*S3W'
                        d17_errmsg,1,'Using default method =none'
                        errout='none'
                end
                15: begin
                        d17_errmsg,1,'SANOFF set to 0'
                        errout=0.0
                end
                16: begin
                        d17_errmsg,1,'SANOFF set to auto'
                        d17_errmsg,1,'(set sanoff = 0.0 if you wish to turn this off)'
                        errout='auto'
                end
                17: begin
                        d17_errmsg,1,'WARNING: SANOFF has been set to auto, but the mean beam has not been set'
                        d17_errmsg,1,'Setting sanoff = 0.0'
                        d17_errmsg,1,'Set the mbeam parameter if you want to use the automatic search'
                        errout=0.0
                end
                18: begin
                        d17_errmsg,1,'WARNING: The SAN-range is too small to effectively use the TTH or Q methods'
                        d17_errmsg,1,'d17_xspec is returning the specular intensity using the SMPL method'
                end
                
        endcase
        
        if errin le 10 then errout='quit'
        
end


;********************************************************************
;*** Define a procedure for interpolating and integrating specular **
;********************************************************************
pro sinterp,yin,win,ein,yout,wout,eout
;+
;*** This one for the specular scattering
;*** This makes heavy use of the IDL interpolate function
;*** The input data is a parallelogram - need to interpolate on to the central pixel
;-
        
        ;*** Taking the y values from the central pixel
        arrsize=size(win,/DIMENSIONS)
        yinter=yin[floor(arrsize[0]/2),*]
        
        ;*** Can only apply the interpolation where the parallelogram has the full width
        ylims=[min(yin[0,*]),max(yin[arrsize[0]-1,*])]
        if n_elements(where(yinter gt ylims[0] and yinter lt ylims[1])) gt 1 then begin
                yout=[[ylims[0]],[transpose(yinter[where(yinter gt ylims[0] and yinter lt ylims[1])])],[ylims[1]]]
                ;*** This range is smaller than the data, so there should be no NaNs
                
                ;*** Now interpolate the data and errors on to yout, column by column
                wtemp=fltarr(arrsize[0],n_elements(yout),arrsize[2])
                etemp=wtemp
                for i=0,arrsize[0]-1 do begin
                        for j=0,arrsize[2]-1 do begin
                                wtemp[i,*,j]=interpol(win[i,*,j],yin[i,*],yout)
                                etemp[i,*,j]=interpol(ein[i,*,j],yin[i,*],yout)
                        endfor
                endfor
                
                ;*** Now have a rectangular array and can simply sum the data
                wout=total(wtemp,1)
                eout=sqrt(total(etemp^2,1))
        endif else begin
                aw_errmsg,18,errout
                yout=yinter
                wout=total(win,1)
                eout=sqrt(total(ein^2,1))
        endelse
        
end

;**********************************************************************
;*** Define a procedure for interpolating and integrating background **
;**********************************************************************
pro binterp,yin,win,ein,yout,spix,wout,eout,n
;+
;*** This one for the background
;*** This makes heavy use of the IDL interpolate function
;*** The data will be a number of 'parallelograms'
;*** yout is fed in to the dataset and comes from sinterp
;*** Data will be normalized to number of pixels, taken from spix
;-
        
        ;*** Define array sizes
        arrsize=size(win,/DIMENSIONS)
        wtemp=fltarr(arrsize[0],n_elements(yout),arrsize[2])
        etemp=wtemp
        n=0*indgen(n_elements(yout))
        
        ;*** Run through the data column by column and interpolate on to yout (where possible)
        for i=0,arrsize[0]-1 do begin
                ytempin=where(yin[i,*] ge min(yout) and yin[i,*] le max(yout))
                ytempout=where(yout ge min(yin[i,*]) and yout le max(yin[i,*]))
                for j=0,arrsize[2]-1 do begin
                        wtemp[i,ytempout,j]=interpol(win[i,ytempin,j],yin[i,ytempin],yout[ytempout])
                        etemp[i,ytempout,j]=interpol(ein[i,ytempin,j],yin[i,ytempin],yout[ytempout])
                endfor
        endfor
        
        
        ;*** Now have a rectangular array and can simply sum the data
        wout=total(wtemp,1)
        eout=sqrt(total(etemp^2,1))
        
        ;*** Still need to normalise to the number of non-zero pixels.
        spix=float(spix)
        for i=0,n_elements(yout)-1 do begin
                n[i]=float(n_elements(where(etemp[*,i,0] ne 0.0)))
                wout[i,*]=spix[0]/n[i]*wout[i,*]
                eout[i,*]=spix[0]/n[i]*eout[i,*]
        endfor
        
end

;+
; ****************************************************************
; ***  Finally, the function ***
; ****************************************************************
;-
function d17_xspec,win,spix,bkg=bkg,method=method,mbeam=mbeam,snorm=snorm,xaxis=xaxis,lambda=lambda,sanoff=sanoff

        forward_function d17_xyconvert
        
        ;*** Get important information from the input workspace
        TAKE_DATP,datp
        ein=datp.e
        yin=datp.y
        zin=datp.z
        pvin=datp.pv
        
        ;*** If options are not chosen, define the defaults
        if n_elements(method) eq 0 then method = 'smpl'
        if n_elements(snorm) eq 0 then snorm = 'none'
        if n_elements(xaxis) eq 0 then begin
                d17_errmsg,1,'Setting final x-axis to Qz'
                xaxis='q'
        endif
        if n_elements(lambda) eq 0 then lambda = datp.p[24]
        if n_elements(lambda) ne 1 then begin
                d17_errmsg,1,'WARNING: lambda must be a single number.  Setting lambda from the parameter set in MAD'
                lambda=datp.p[24]
        endif
        if n_elements(bkg) eq 0 then bkg=[-2,-1]
        
        ;*** Check that the input parameters have the right number of variables
        errout='continue'
        if n_elements(spix) ne 2 then d17_errmsg,0
        if n_elements(bkg[*,0]) gt 0 and n_elements(bkg[*,0]) ne 2 then aw_errmsg,1,errout
        if strcmp(method,'smpl') ne 1 and n_elements(mbeam) eq 0 then aw_errmsg,3,errout
        methodmark=[strcmp(method,'smpl'),strcmp(method,'tth'),strcmp(method,'q')]
        if max(methodmark) ne 1 then aw_errmsg,11,method
        if max([strcmp(snorm,'none'),strcmp(snorm,'q'),strcmp(snorm,'s2'),strcmp(snorm,'s2q'),strcmp(snorm,'s2s3')]) ne 1 then aw_errmsg,14,method
        if strcmp(xaxis,'san') eq 0 and strcmp(xaxis,'q') eq 0 then aw_errmsg,12,xaxis
        if strcmp(errout,'quit') eq 1 then return, 0
        
        ;*** Sort out defaults for sanoff
        if n_elements(sanoff) eq 0 then begin
                d17_errmsg,1,'No SANOFF entered'
                case where(methodmark eq 1) of
                        0: aw_errmsg,15,sanoff
                        1: aw_errmsg,16,sanoff
                        2: aw_errmsg,16,sanoff
                endcase
        endif
        if (strcmp(method,'smpl') eq 1) and (strcmp(sanoff,'auto') eq 1) and (n_elements(mbeam) eq 0) then aw_errmsg,17,sanoff
        
        if n_elements(sanoff) eq 0 then aw_errmsg,13,sanoff
        
        ;*** Sort out foreground range
        if spix[0] gt spix[1] then begin
                d17_errmsg,1,'WARNING: spix.  The left value is larger than the right.  Swapping values.'
                spix=[spix[1],spix[0]]
        endif
        spix=spix[0]+indgen(spix[1]+1-spix[0])
        
        ;*** Check that the foreground range is within the limits of the input workspace
        if max(spix) gt n_elements(win[*,0])-1 or min(spix) le 0 then aw_errmsg,4
        
        if total(bkg) gt 0 then begin
                ;*** Sort the background ranges and look for overlap
                bpix=-1
                for i=0,n_elements(bkg[0,*])-1 do begin
                        if bkg[0,i] gt bkg[1,i] then begin
                                d17_errmsg,1,'WARNING: bkg Row ',i,'. The left value is larger than the right.  Swapping values.'
                                bkg[*,i]=[bkg[1,i],bkg[0,i]]
                        endif
                        bpix=[bpix,bkg[0,i]+indgen(bkg[1,i]+1-bkg[0,i])]
                endfor
                bpix=bpix[1:n_elements(bpix)-1]
                tbpix=bpix[uniq(bpix,sort(bpix))]
                if n_elements(bpix) ne n_elements(tbpix) then begin
                        d17_errmsg,1,'WARNING: Some of the background ranges overlap.'
                        d17_errmsg,1,'         Have sorted and discarded overlapped background pixels'
                endif
                bpix=tbpix[where(tbpix ge 0)]
                
                ;*** Check that the background range is within the limits of the input workspace
                if max(bpix) gt n_elements(win[*,0])-1 then begin
                        d17_errmsg,1,'WARNING: The maximum background range is outside the range of the input array'
                        d17_errmsg,1,'          Setting the maximum background pixel to the maximum pixel of the array'
                        bpix=bpix[where(bpix le n_elements(win[*,0]-1))]
                endif
                
                ;*** Check that the background range does not overlap with the specular range
                totpix=[spix,bpix]
                tbpix=totpix[uniq(totpix,sort(totpix))]
                if n_elements(totpix) ne n_elements(tbpix) then begin
                        d17_errmsg,1,'WARNING: The background range overlaps with the specular range'
                        d17_errmsg,1,'          Truncating the background range to fit'
                        bpixlims=[where(bpix lt min(spix)),where(bpix gt max(spix))]
                        bpix=bpix[bpixlims[where(bpixlims ne -1)]]
                endif
        endif
        
        ;*******************
        ;*** METHOD 1: smpl
        ;*******************
        if strcmp(method,'smpl') eq 1 then begin
        
                ;*** Extract the specular range
                ns=float(n_elements(spix))
                wspec=total(win[spix,*,*],1)
                espec=sqrt(total(ein[spix,*,*]^2,1))
                nw=float(n_elements(spix))
                
                ;*** Extract the backgrounds
                if n_elements(bpix) gt 0 then begin
                        nb=float(n_elements(bpix))
                        wbkg=total(win[bpix,*,*],1)
                        ebkg=sqrt(total(ein[bpix,*,*]^2,1))
                        if total(where(wbkg eq 0)) ge 0 then ebkg[where(wbkg eq 0)]=0.0
                endif else begin
                        nb=1.0
                        wbkg=0.0*wspec
                        ebkg=wbkg
                endelse
                
                ;*** Now make the subtraction
                wout=wspec-ns/nb*wbkg
                eout=sqrt((espec^2)+(ns/nb*ebkg)^2)
                
                ;*** Determine final x-axis values
                smpldatp=datp
                ;*** Use sanoff=auto if requested
                xout=pvin[0,*]
                if strcmp(sanoff,'auto') eq 1 then begin
                        wsmpl=d17_xyconvert(win,'tito',mbeam,sanoff=sanoff,datp=smpldatp)
                        xout=smpldatp.y[0,*]
                endif else xout=xout+sanoff
                qz=4*!pi*sin(!pi*xout/180.0)/lambda
                if strcmp(xaxis,'q') eq 1 then begin
                        xout=qz
                        xtit='Qz'
                endif
                
        endif
        
        ;*******************
        ;*** METHOD 2: tth
        ;*******************
        if strcmp(method,'tth') eq 1 then begin
        
                tthdatp=datp
                
                ;*** Convert to (san,tth) space
                wtth=d17_xyconvert(win,'stth',mbeam,sanoff=sanoff,datp=tthdatp)
                etth=tthdatp.e
                xtth=tthdatp.x
                ;*** Extract the specular pixels
                ns=n_elements(spix)
                wspec=wtth[spix,*,*]
                espec=etth[spix,*,*]
                xspec=xtth[spix,*]
                
                ;*** Extract the backgrounds for each row
                if n_elements(bpix) gt 0 then begin
                        nb=float(n_elements(bpix))
                        wbkg=total(wtth[bpix,*,*],1)
                        ebkg=sqrt(total(etth[bpix,*,*]^2,1))
                        if total(where(wbkg eq 0)) ge 0 then ebkg[where(wbkg eq 0)]=0.0
                endif else begin
                        nb=1.0
                        wbkg=0.0*wspec[0,*,*]
                        ebkg=wbkg
                endelse
                
                ;*** Now make the subtraction
                wspec_b=0.0*wspec
                espec_b=wspec
                for i=0,ns-1 do begin
                        wspec_b[i,*,*]=wspec[i,*,*]-wbkg/nb
                        espec_b[i,*,*]=sqrt(espec[i,*,*]^2+ebkg^2/nb)
                endfor
                
                ;*** Now interpolate the data on to the central pixel and integrate it
                sinterp,xspec,wspec_b,espec_b,xout,wout,eout
                
                ;*** Determine final x-axis values
                xout=xout/2.0
                qz=4*!pi*sin(!pi*xout/180.0)/lambda
                if strcmp(xaxis,'q') eq 1 then begin
                        xout=qz
                        xtit='Qz'
                endif
                
        endif
        
        ;*******************
        ;*** METHOD 3: q
        ;*******************
        if strcmp(method,'q') eq 1 then begin
        
                ;*** Convert workspace in to 'pipf'
                qdatp=datp
                wiq=d17_xyconvert(win,'pipf',mbeam,lambda=lambda,sanoff=sanoff,datp=qdatp)
                xtit='Qz'
                
                ;*** Determine number of pixels in specular range
                spix=n_elements(spix)
                
                ;*** Get important information from the (converted) input workspace
                eiq=qdatp.e
                xiq=qdatp.x
                yiq=qdatp.y
                ziq=qdatp.z
                
                nels=n_elements(yiq[0,*])
                
                ;*** Initialize arrays for specular
                ycr=fltarr(spix,nels)
                xcr=ycr
                wcr=fltarr(spix,nels,n_elements(ziq))
                ecr=wcr
                xout=fltarr(nels)
                
                ;*** Now extract the data.
                ;*** Run thru the data row by row and pick out the correct range of pixels
                for i=0,nels-1 do begin
                        ;*** First, find the pixel in each row with the pi-pf closest to 0
                        specpix=where(abs(xiq[*,i]) eq min(abs(xiq[*,i])))
                        ;*** Then pick out the x-pixels centred on this point.  This is the specular data
                        srng=specpix[0]-floor((spix-1)/2)+indgen(spix)
                        xcr[*,i]=xiq[srng,i]
                        ycr[*,i]=yiq[srng,i]
                        wcr[*,i,*]=wiq[srng,i,*]
                        ecr[*,i,*]=eiq[srng,i,*]
                        ;*** Now pick out background pixels
                        if bkg[0] ne -2 then begin
                                bycr=yiq[bpix,*]
                                bxcr=xiq[bpix,*]
                                bwcr=wiq[bpix,*,*]
                                becr=eiq[bpix,*,*]
                        endif
                endfor
                
                ;*** Now carry out interpolations for the specular
                sinterp,ycr,wcr,ecr,yspec,wspec,espec
                
                
                ;*** Now carry out the interpolations for the background
                wback=0*wspec
                eback=wback
                if bkg[0] ne -2 then binterp,bycr,bwcr,becr,yspec,spix,wback,eback,nback
                
                ;*** Now subtract the background
                yout=yspec
                wout=wspec-wback
                eout=sqrt(espec^2+eback^2)
                
                qz=yout
                xout=yout
        endif
        
        ;*** Determine slit normalizations
        ;*** First, work out for backwards compatibility (S2W and S3W are columns 6 and 7 in newer mload)
        ;stop
        pvsz=size(pvin,/DIMENSIONS)
        wosz=[1,1]
        twosz=size(wout,/DIMENSIONS)
        wosz[0:n_elements(twosz)-1]=twosz
        slitrows=1.0+fltarr(wosz[0],1)
        slitcols=1.0+fltarr(1,wosz[1])
        if pvsz[0] eq 8 then begin
                case where([strcmp(snorm,'q'),strcmp(snorm,'s2'),strcmp(snorm,'s2q'),strcmp(snorm,'s2s3')] eq 1) of
                        -1:
                        0: begin
                                slitrows=transpose(qz)
                        end
                        1: begin
                                slitrows=transpose(pvin[6,*,0])
                        end
                        2: begin
                                slitrows=transpose(pvin[6,*,0]*qz)
                        end
                        3: begin
                                slitrows=transpose(pvin[6,*,0]*pvin[7,*,0])
                        end
                endcase
        endif else begin
                aw_errmsg,5,errout
                return,0
                ;       d17_errmsg,1,'The input workspace was created with an old version of mload'
                ;       d17_errmsg,1,'Please update the D17 LAMP library and reload the data using mload'
        endelse
        
        slitnorm=slitcols##slitrows
        
        wout=wout/slitnorm
        eout=eout/slitnorm
        
        
        ;*** Write the new parameters
        MOD_DATP,datp,"e",eout
        MOD_DATP,datp,"x",xout
        MOD_DATP,datp,"y",zin
        MOD_DATP,datp,"z",1
        MOD_DATP,datp,"x_tit",xtit
        MOD_DATP,datp,"y_tit",'Intensity (arb. units)'
        
        GIVE_DATP,datp
        RETURN,wout
        
end
