;+
;FUNCTION d17_xyconvert,win,oaxes,mainbeam,lambda=lambda,sanoff=sanoff,datp=datp
;
;	LAMP function to take a workspace and convert its axes
;		Works with monochromatic or TOF data
;		For monochromatic data, the input workspace must have: 
;			x-pixel on the x-axis
;			SAN or DAN on the y-axis
;		Monochromatic data may be 3D
;		For TOF data, the input workspace must have:
;			x-pixel on the x-axis
;			wavelength on the y-axis
;		TOF data must be 2D
;	then to re-bin the data for equally spaced points with
;		x and y axes with physical meaning
;	ARW 17.02.05
;
;	NECESSARY INPUT:
;	win=input workspace
;	oaxes=the output axes, put as a string
;		Allowable options are:
;			'qxqz' = qx (x-axis) vs. qz (y-axis)
;			'pipf' = pi-pf (x-axis) vs. pi+pf (y-axis)
;		   the above options are available for TOF and monochromatic data sets
;			'tito' = theta out (x-axis) vs. theta in (y-axis)
;			'stth' = two theta (x-axis) vs san (y-axis)
;		   the above options are only available for monochromatic data sets
;			'ltth' = two theta (x-axis) vs wavelength (y-axis)
;		   the above options are only available for TOF datasets
;	mainbeam=data file for the main beam, used to calibrate twotheta
;
;	OPTIONAL INPUT:
;	lambda=wavelength settings.  This has two possibilities:
;		Monochromator data: this is the actual wavelength, a single number.
;		  This should be stored with the data, but you have the
;		  option here of forcing it to another value
;			e.g. lambda = 5.325
;		TOF data: this is a two element array with the minimum and maximum
;		  wavelengths for the calculation
;			e,g. lambda = [2,20] (this happens to be the default)
;	sanoff=an offset in SAN, which can exist if the sample angle calibration
;		is out.  This value is SUBTRACTED from the instrument-stored value
;		of SAN to find the correct value for theta_in
;	        For an attempt to automatically find sanoff, use sanoff='auto'
;		Otherwise, equate sanoff to a number
;		Default is sanoff = 0
;	bin=option whether to rebin the data as equally spaced points
;		bin=0 (default)for no rebinning - X and Y coordinates will be
;			arrays the same size as win and can be viewed in the
;			desired space using the 'Regular Grid' option in LAMP
;		bin=1 for rebinning.  The data will be interpolated on to a
;			regular grid
;	datp = the data structure for the workspace
;		If unspecified, the program will take the data structure
;		from the workspace
;		(This is only useful if calling d17_xyconvert from inside another function, 
;			e.g. aw_qxspec)
;
;	e.g. type at LAMP interface:
;		w2=d17_xyconvert(w1,'qxqz',97936,sanoff='auto')
;
;	Calls to:	rdrun, d17_errmsg
;	Last modified:	PG 26.07.12
;
;Iformation on how to load the maps into common plotting programs by saving the LAMP file in column format:
;
;Igor Pro:
;1.Data:LoadWaves:LoadWaves
;2.Filetype:DelimitedText 
;3.set Tweaks to cosider space as the delimiting character
;4.Line containing column labels: 1
;5.First line containing data: 2
;6.Set number of lines containing data to the number of y-points
;7.First column containing data: 2
;8.Tick the 'Load columns into matrix' box
;9.Tick the 'read column positions' box
;10. Format the y-axis manually
;
;Origin:
;1.File:Import:SingleAscii
;2.Check the 'Partial import' box and go to 'Options'
;3.Ti9ck the 'Delimited' tick box and chose space as the delimiter
;4.Skip main header, number of lines: 1
;5.Specify known subheader lines: 1
;6.Max # of lines stored in the coumn header: 1
;7.In the partial import dialog chose Row from 1 to the number of y-data points you have
;-

; ****************************************************************
; ***  Define procedure for finding a peak ***
; ****************************************************************
pro aw_peaksearch,win,cen,sigma
;+
;** *************
;***  Define procedure for finding a peak ***
;-

x=findgen(n_elements(win))
range=float([min(x),max(x)])
;***	Extract the data that falls within the limits
els=where(x ge range[0] and x le range[1])
x_sub=x[els]
win_sub=win[els]
;***	Find the weighted mean and variance of the data
cen=total(x_sub*win_sub)/total(win_sub)
sigma=sqrt(total(win_sub*(x_sub-cen)^2)/total(win_sub))
end

; ****************************************************************
; ***  Finally, the function ***
; ****************************************************************
FUNCTION d17_xyconvert,win,oaxes,mainbeam,lambda=lambda,sanoff=sanoff,datp=datp
;+
;******* *************
;** User callable
;-

; **	Read the desired output axes format
eflag=[strcmp(oaxes,'tito'),strcmp(oaxes,'qxqz'),strcmp(oaxes,'pipf'),strcmp(oaxes,'stth'),strcmp(oaxes,'ltth')]
if (strlen(oaxes) eq 0) or (max(eflag) ne 1) then begin
	d17_errmsg,1,'ERROR: A string with the desired output axes format must be specified'
	d17_errmsg,1,'Allowable options are:'
	d17_errmsg,1,'	tito = theta out (x-axis) vs. theta in (y-axis)'
	d17_errmsg,1,'	qxqz = qx (x-axis) vs. qz (y-axis)'
	d17_errmsg,1,'	pipf = pi-pf (x-axis) vs. pi+pf (y-axis)'
	d17_errmsg,1,' stth = two theta (x-axis) vs san (y-axis)'
	d17_errmsg,1,' ltth = two theta (x-axis) vs wavelength (y-axis)'
	return,0
endif 

; ***  Get the important information from the workspace
if n_elements(datp) eq 0 then TAKE_DATP,datp else datp=datp

; *** Check whether the measurement is in TOF or mono
if strmatch(datp.par_txt[24],'*Time*') eq 1 then mode='TOF' else mode='mono'

; *** Some axis definitions don't make sense in some modes
eflag=[strcmp(oaxes,'qxqz'),strcmp(oaxes,'pipf'),strcmp(oaxes,'ltth')]
if (strcmp(mode,'TOF') eq 1) and (max(eflag) ne 1) then begin
	d17_errmsg,1,'ERROR: The chosen output axes are not compatible with TOF mode'
	d17_errmsg,1,'Allowable options are:'
	d17_errmsg,1,'	qxqz = qx (x-axis) vs. qz (y-axis)'
	d17_errmsg,1,'	pipf = pi-pf (x-axis) vs. pi+pf (y-axis)'
	d17_errmsg,1,' ltth = two theta (x-axis) vs wavelength (y-axis)'
	return,0
endif
eflag=[strcmp(oaxes,'qxqz'),strcmp(oaxes,'pipf'),strcmp(oaxes,'tito'),strcmp(oaxes,'stth')]
if (strcmp(mode,'mono') eq 1) and (max(eflag) ne 1) then begin
	d17_errmsg,1,'ERROR: The chosen output axes are not compatible with TOF mode'
	d17_errmsg,1,'Allowable options are:'
	d17_errmsg,1,'	qxqz = qx (x-axis) vs. qz (y-axis)'
	d17_errmsg,1,'	pipf = pi-pf (x-axis) vs. pi+pf (y-axis)'
	d17_errmsg,1,'	tito = theta out (x-axis) vs. theta in (y-axis)'
	d17_errmsg,1,' stth = two theta (x-axis) vs san (y-axis)'
	return,0
endif

wcalc=win

wout=win
win_size=size(win,/DIMENSIONS)
if n_elements(win_size) eq 3 then win_size=win_size[0:1]
eout=datp.e
;***	Collect the input x and y
;xin=indgen(datp.p(13)+1);***	A 1D array with pixel number
xin=datp.x
;***	Determine what to take as SAN and DAN
sanin=datp.p[3]
danin=datp.p[4]
pvsize=size(datp.pv,/DIMENSIONS)
if pvsize[0] eq 8 or pvsize[0] eq 3 then begin
	sanin=datp.pv[0,*]	;***	A 1D array with SAN
	danin=datp.pv[1,*]	;***	A 1D array with DAN
endif
det=datp.p[21]
if det - 10.0 gt 0 then det=det/1000.0	; Convert to metres if necessary
mmppx=datp.p[19]/1000.0			; Convert to metres
dcen=datp.p[18]
;***	Read main beam file
mdp=1
mbeam=rdrun(mainbeam,datp=mdp)
dan_0=mdp.p[4]
cen_main=mdp.p[17]
mbeam_size=size(mbeam,/DIMENSIONS)
;***	If it is a 2D workspace, sum vertically
if n_elements(mbeam_size) eq 2 then mbeam=total(mbeam,2)
;***	Find the peak
aw_peaksearch,mbeam,xpix_0,sigma_main
mainb=(dcen-xpix_0)*mmppx

; *** Now need to have separate routines for monochromatic or TOF

; *** monochromatic data
if strcmp(mode,'mono') eq 1 then begin
;***	Calculate the values of twotheta from x-pixel and SAN
	san=make_array(win_size)
	twotheta=make_array(win_size)
	if n_elements(win_size) eq 1 then win_size=[win_size,1]
	for i=0,win_size[1]-1 do begin
		san[*,i]=fltarr(win_size[0])+sanin[i]
		refl =(dcen-xin)*mmppx
		tth=danin[i] + (180/!pi)*atan(refl/det) - dan_0 - (180/!pi)*atan(mainb/det)
		twotheta[*,i]=tth
	endfor

	newx=twotheta
	newy=san
	newx_tit='Two Theta'

; ***	If 'stth' is chosen, stop here, otherwise continue
	if strcmp(oaxes,'stth') ne 1 then begin

; ***	Set the value for the wavelength
		if n_elements(lambda) ne 1 then lambda=datp.p[24]

; ***	Set the default value for the SAN offset if it has not been passed
		if n_elements(sanoff) eq 0 then begin
			sanoff=0.0
		endif else begin
;*** If 'auto' is chosen, the program will have a stab at estimating sanoff
			if strmatch(string(sanoff),'auto') eq 1 then begin
				w1d=total(win[*,*,0],2) ;*** Take the first data array, project on to x-axis
				sw1d=max(w1d)		;*** Find the maximum.  This is taken to be the central pixel for the specular beam
				scen=where(w1d eq sw1d)	;*** Find the position for the maximum 
				swid=scen
				i=1
				while sw1d ge max(w1d)/2.0 do begin ;*** Set up a loop to look for the 1/2 width
					sw1d=w1d[scen+i]
					i=i+1
				endwhile
				srange=scen[0]-2*i+indgen(4*i+1)	    ;*** Estimate for the integration range
				theta_in=transpose(newy[0,*])
				twotheta_out=0*theta_in
		;*** Extract the data that falls within the limits
				w1d=win[srange,*,0]		
				for i=0,n_elements(theta_in)-1 do begin
					x1d=newx[srange,i]
					w1d=win[srange,i,0]
			;***	Find the weighted mean and variance of the data
					cen=total(x1d*w1d)/total(w1d)
;			cen=mean(x1d(where(w1d eq max(w1d))))
;			dummy=gaussfit(x1d,w1d,cen) & cen=cen(1)
;			sigma=sqrt(total(win_sub*(x_sub-cen)^2)/total(win_sub))
					twotheta_out[i]=cen	
				endfor
		;*** Fit a straight line to the data
				sangrad=regress(twotheta_out,theta_in,CONST=sanoff)
				d17_errmsg,1,'Fit to line gives:'
				d17_errmsg,1,[' theta_in = '+string(sangrad)+' * 2theta_out + '+string(sanoff)]
			endif
		endelse
		newy=newy-sanoff

; ***	Set the default value for the bin if it has not been passed
		if n_elements(bin) eq 0 then bin=0

; ***	Convert the workspace X and Y arrays to the desired axes format
		newx=newx-newy
		newx_tit='Theta Out'
		newy_tit='Theta In'
		if strcmp(oaxes,'tito') ne 1 then begin
			kmod=2.0*!pi/lambda
			newx=!pi*newx/180.0
			newy=!pi*newy/180.0
			qx=kmod*(cos(newx)-cos(newy))
			qz=kmod*(sin(newy)+sin(newx))
			padd=qz
			psub=kmod*(sin(newy)-sin(newx))
			if strcmp(oaxes,'qxqz') eq 1 then begin
				newx=qx
				newy=qz
				newx_tit='Qx (Angstroms^-1)'
				newy_tit='Qz (Angstroms^-1)'
			endif else begin
				newx=psub
				newy=padd
				newx_tit='pi-pf (Angstroms^-1)'
				newy_tit='pi+pf (Angstroms^-1)'
			endelse
		endif
	endif
endif

; *** Time-of-flight data
if strcmp(mode,'TOF') eq 1 then begin
	; *** Calculate twotheta
	refl =(dcen-xin)*mmppx
	tth=danin + (180/!pi)*atan(refl/det) - dan_0 - (180/!pi)*atan(mainb/det)
		
; ***	Set the default value for the SAN offset if it has not been passed
	if n_elements(sanoff) eq 0 then begin
		sanoff=0.0
	endif else begin
;*** If 'auto' is chosen, the program will have a stab at estimating sanoff
		if strmatch(string(sanoff),'auto') eq 1 then begin
			w1d=total(win,2) ;*** Take the first data array, project on to x-axis
			sw1d=max(w1d)		;*** Find the maximum.  This is taken to be the central pixel for the specular beam
			scen=where(w1d eq sw1d)	;*** Find the position for the maximum 
			swid=scen
			i=1
			while sw1d ge max(w1d)/2.0 do begin ;*** Set up a loop to look for the 1/2 width
				sw1d=w1d[scen+i]
				i=i+1
			endwhile
			srange=scen[0]-2*i+indgen(4*i+1)	    ;*** Estimate for the integration range
		;*** Extract the data that falls within the limits
			w1d=w1d[srange]		
			x1d=tth[srange]
		;***	Find the weighted mean and variance of the data
			cen=total(x1d*w1d)/total(w1d)
			sanoff=sanin-cen/2.0
			d17_errmsg,1,['Two theta for specular found at: '+string(cen)]
			d17_errmsg,1,['SAN recorded as :'+string(sanin)]
			d17_errmsg,1,['Estimated SAN ='+string(cen/2.0)]
			d17_errmsg,1,['Recorded SAN - estimated SAN ='+string(sanin - cen/2.0)]
		endif
	endelse

	sanin=(sanin-sanoff)

; ***	Set the range for the wavelength
	if n_elements(lambda) ne 2 then lambda=[2,20]
	lels=where((datp.y ge min(lambda)) and (datp.y le max(lambda)))
	wcalc=wcalc[*,lels]
	eout=eout[*,lels]
	lambda=datp.y[lels] 	

; ***	Convert the workspace X and Y arrays to the desired axes format
	newx=tth
	newy=lambda
	if strcmp(oaxes,'ltth') ne 1 then begin
		kmod=transpose(2.0*!pi/lambda)
		thi =sanin*!pi/180.0
		tho =(tth-sanin)*!pi/180.0
		wcalc=wcalc[where(tho gt 0),*]
		eout=eout[where(tho gt 0),*]
		tho = tho[where(tho gt 0)] 	; *** Remove those angles that are less than zero if necessary
		onestth=1.0+fltarr(n_elements(tho),1)
		kix=(kmod##onestth)*cos(thi)
		kiz=(kmod##onestth)*sin(thi)
		kfx=kmod##cos(tho)
		kfz=kmod##sin(tho)
		if strcmp(oaxes,'qxqz') eq 1 then begin
			newx=kix-kfx
			newy=kiz+kfz
			newx_tit='Qx (Angstroms^-1)'
			newy_tit='Qz (Angstroms^-1)'
		endif else begin
			newx=kiz-kfz
			newy=kiz+kfz
			newx_tit='pi-pf (Angstroms^-1)'
			newy_tit='pi+pf (Angstroms^-1)'
		endelse		
	endif
	MOD_DATP,datp,"z",0	; Get rid of Z axis
endif

; ***	Rebin if requested
;if bin eq 1 then wcalc=aw_rebin(wcalc,datp=datp)

Wout=wcalc
MOD_DATP,datp,"x",newx
MOD_DATP,datp,"y",newy
MOD_DATP,datp,"e",eout
MOD_DATP,datp,"x_tit",newx_tit
MOD_DATP,datp,"y_tit",newy_tit
GIVE_DATP,datp
return, Wout
end
