pro he3tptm,time,params,tplus,tminus
;+
;	LAMP procedure to calculate the transmission of parallel and antiparallel
;	neutrons at a given time with a given set of input parameters
;	ARW 15.09.04
;
;	NECESSARY INPUT:
;	time=the time for the calculation, derived using the 'julday' IDL function
;	params=the parameter array.  It can be 2-dimensional with the parameters
;		in a column and their errors in the other column
;		params(0) = definition of time t=0 (by 'julday')(t0)
;		params(1) = initial polarization of the He3 gas (P0He)
;		params(2) = the time constant for decay (hours)	(t1)
;		params(3) = the inefficiency of the polarizer	(p1)
;		params(4) = the pressure of the gas (bar)	(conc)
;		params(5) = the wavelength (Angstrom)		(sa)
;	NOTE: The thickness of the D17 cell is currently fixed to 10 cm
;
;	OUTPUT:
;	tplus=the transmission with polarization parallel to the 3He nucleii
;	tminus=the transmission with the polarization antiparallel
;
;	Calls to:
;	Last modified:	2.05.07 ARW
;-

psize=size(params)

; ***	Define the input parameters
t0 = params[0]
P0He=params[1]
t1=params[2]
conc=params[4]
sa=params[5]
l=10		; thickness of D17 cell.  Currently fixed to 10 cm!
O=7.33e-2	; Conversion constant to go from bar.cm.Angstrom to unitless


; ***	Convert t1 to IDL units using julday
t1=t1*(julday(1,1,1980,2,0,0)-julday(1,1,1980,1,0,0))

; ***	Define the time from the initial time for the 3He cell
t=time-t0
if min(t) lt 0 then begin
	print,'WARNING: Attempt to calculate the 3He polarization at negative times'
	print,'Setting ellapsed time to zero'
	t=0
	params[0]=min(time)
endif

; ***	Calculate the 3He polarization
	PHe=P0He*exp(-t/t1)
; ***	Calculate the transmission
tplus=0.5*exp(-O*conc*l*sa*(1-PHe))
tminus=0.5*exp(-O*conc*l*sa*(1+PHe))

if psize[0] eq 2 then begin
	eparams=params[*,1]
	et0=eparams[0]
	eP0He=eparams[1]
	et1=eparams[2]
	et1=et1*(julday(1,1,1980,2,0,0)-julday(1,1,1980,1,0,0))
	econc=eparams[4]
	esa=eparams[5]
	ePHe=sqrt((eP0He*PHe/P0He)^2+(et0*PHe/t1)^2+(et1*t*PHe/t1^2)^2)
	etplus=tplus*O*sqrt( $
		(econc*l*sa*(1-PHe))^2 $
		+(esa*conc*l*(1-PHe))^2  $
		+(ePHe*conc*l*sa)^2)
	etminus=tminus*O*sqrt( $
		(econc*l*sa*(1+PHe))^2 $
		+(esa*conc*l*(1+PHe))^2  $
		+(ePHe*conc*l*sa)^2)
	tplus=[[tplus],[etplus]]
	tminus=[[tminus],[etminus]]
endif

end
