;+
;	function maskme,maskproc,win,datp=datp
;
;	LAMP function to mask the unwanted points from a workspace
;	ARW 05.08.04
;
;	NECESSARY VARIABLES
;	maskproc=a string with the mask procedure and relevant parameters
;		The available mask procedures are:
;		box:	needs a 2- or 4-element array to define the box
;		circle:	needs a 3-element array with centreX, centreY, radius
;		ellipse:needs a 4-element array with Xlocus, Ylocus, centreX, centreY
;		none:	no mask (for calling within functions) 
;	win=input workspace
;
;	OPTIONAL VARIABLES
;	datp = the data structure for the workspace
;		If unspecified, the program will take the data structure
;		from the workspace
;
;	e.g. type at lamp interface:
;		w2=maskme('box,[190,200,63,222]',w1)
;
;	CALLS TO: appropriate mask procedure, d17_errmsg
;	last modified:	ARW 17.12.10
;-

; ************************************************************************************
; *************** First define the mask functions ************************************
; ************************************************************************************
pro box,limits,win,wout
;+
;	LAMP procedure to mask all the pixels from a workspace
;	except those within the box bounded by the specified limits
;	All pixels outside the mask will have a value of -1
;	ARW 05.08.04
;
;	NECESSARY VARIABLES:
;	limits=a two- or four-element array with the pixel limits for the mask
;		if limits is a two element array, the limits are [xmin,xmax]
;		if it's a 4 element array, the limits are [xmin,xmax,ymin,ymax]
;	win=the input workspace
;	
;	OUTPUT VARIABLES:
;	wout=the output workspace
;
;	e.g. box,[190,200,63,222],win,wout
;
;	CALLS TO:
;	last modified:	ARW 13.08.04
;-

wout=win
wsize=size(win)

;***	Check that the right number of elements have been entered
if (n_elements(limits) ne 2) and (n_elements(limits) ne 4) then begin
	d17_errmsg,1,'ERROR: You must enter 2 or 4 limits in to box! No mask applied'
	return
endif

;***	Define X and Y arrays with the row and column numbers for win
X=(0*indgen(wsize[2])+1)##indgen(wsize[1])
Y=indgen(wsize[2])##(0*indgen(wsize[1])+1)

;***	Mask the y-axis
if n_elements(limits) eq 4 then begin 
	if limits[3] lt limits[2] then begin
		d17_errmsg,1,'ERROR: ymax is LESS that ymin! No mask applied'
		return
	endif
	if limits[3] gt wsize[2]-1 then begin
		d17_errmsg,1,'WARNING: ymax is GREATER than the dimension of y. No mask applied'
		limits[2]=0
		limits[3]=wsize[2]-1
	endif
	wy1=where(Y lt limits[2],count)
	if count ne 0 then wout[where(Y lt limits[2])]=0
	wy2=where(Y gt limits[3],count)
	if count ne 0 then wout[where(Y gt limits[3])]=0
endif

;***	Mask the x-axis
if limits[1] lt limits[0] then begin
	d17_errmsg,1,'ERROR: xmax is LESS that xmin! No mask applied'
	return
endif
if limits[1] gt wsize[1]-1 then begin
	d17_errmsg,1,'WARNING: xmax is GREATER than the dimension of x. No mask applied'
	limits[0]=0
	limits[1]=wsize[1]-1
endif 
wx1=where(X lt limits[0],count)
if count ne 0 then wout[where(X lt limits[0])]=0
wx2=where(X gt limits[1],count)
if count ne 0 then wout[where(X gt limits[1])]=0

end

pro circle,limits,win,wout
;+
;	LAMP procedure to mask all the pixels from a workspace
;	except those within the circle defined by the specified limits
;	All pixels outside the mask will have a value of -1
;	ARW 05.08.04
;
;	NECESSARY VARIABLES:
;	limits=a four element array with the radius (in pixels) 
;		and centre (x and y) pixels for the ellipse
;		The equation is (x-xc)^2+(y-yc)^2 < radius^2
;	win=the input workspace
;	
;	OUTPUT VARIABLES:
;	wout=the output workspace
;
;	e.g. circle,[100,50,110],win,wout
;
;	CALLS TO:
;	last modified:  13.08.04  ARW
;-

wout=win
wsize=size(win)

;***	Check that the right number of elements have been entered
if (n_elements(limits) ne 3) then begin
	d17_errmsg,'ERROR: You must enter 3 limits in to circle!  No mask applied'
	return
endif

;***	Define X and Y arrays with the row and column numbers for win
X=(0*indgen(wsize[2])+1)##indgen(wsize[1])
Y=indgen(wsize[2])##(0*indgen(wsize[1])+1)

;***	Mask the circle
wy=where((X-limits[1])^2+(Y-limits[2])^2 gt limits[0]^2, count)
if count ne 0 then wout[wy]=0

end

pro ellipse,limits,win,wout
;+
;	LAMP procedure to mask all the pixels from a workspace
;	except those within the ellipse defined by the specified limits
;	All pixels outside the mask will have a value of -1
;	ARW 05.08.04
;
;	NECESSARY VARIABLES:
;	limits=a four element array with the x- and y-locii (in pixels) 
;		and centre (x and y) pixels for the ellipse
;		The equation is ((x-xc)/xlocus)^2+((y-yc)/ylocus)^2 < 1
;	win=the input workspace
;	
;	OUTPUT VARIABLES:
;	wout=the output workspace
;
;	e.g. ellipse,[120,40,135,135],win,wout
;		HINT:  The aspect ratio of y- to x- pixels is ~3!
;
;	CALLS TO:
;	last modified:	13.08.04  ARW
;-

wout=win
wsize=size(win)
;***	Check that the right number of elements have been entered
if (n_elements(limits) ne 4) then begin
	d17_errmsg,1,'ERROR: You must enter 4 limits in to ellipse!  No mask applied'
	return
endif

;***	Define X and Y arrays with the row and column numbers for win
X=(0*indgen(wsize[2])+1)##indgen(wsize[1])
Y=indgen(wsize[2])##(0*indgen(wsize[1])+1)

;***	Mask the ellipse
ellipse=(float(X-limits[2])/float(limits[0]))^2+(float(Y-limits[3])/float(limits[1]))^2
wy=where(fix(ellipse) gt 1,count)
if count ne 0 then wout[wy] = 0
end

; ************************************************************************************
; *************** And finally the 'maskme' function **********************************
; ************************************************************************************
function maskme,maskproc,win,datp=datp
;+
;******* ******
;** User callable
;-

; *** Get rid of this lovely error!
maskproc=maskproc[0]

; ***  Get the important information from the workspace
if n_elements(datp) eq 0 then begin
	TAKE_DATP,wdatp
endif else wdatp=datp

; *** Check the presence of a mask string
if strlen(maskproc) eq 0 then begin
	d17_errmsg,1,'WARNING: Mask procedure not defined.  No mask applied'
	GIVE_DATP,wdatp
	return,win
endif

; *** Circumvent all if 'none' is chosen
if strcmp(maskproc,'none') eq 0 then begin

; *** Check that mask file exists
	maskexist=['box','circle','ellipse']
	maskfin=strmid(maskproc,0,strpos(maskproc,','))
	if total(strcmp(maskexist,maskfin)) ne 1.0 then begin
      		d17_errmsg,1,'Mask procedure not defined.  Allowed mask procedures are box, circle and ellipse'
      		GIVE_DATP,wdatp
      	return,win
	endif

; Extract the limits from the maskproc string
	bra=strpos(maskproc,'[')
	ket=strpos(maskproc,']')
	limstr=strmid(maskproc,bra+1,ket-bra)
	maskname=strmid(maskproc,0,bra-1)
;help,maskname

	lims=0
	comma=0
	while comma ne -1 do begin
		comma=strpos(limstr,',')
		if comma ne -1 then begin 
			lims=[lims,fix(strmid(limstr,0,comma))]
		endif else lims=[lims,fix(strmid(limstr,0,strlen(limstr)-1))]
		limstr=strmid(limstr,comma+1,strlen(limstr)-comma+1)
	endwhile
	lims=lims[1:n_elements(lims)-1]

	call_procedure,maskname,lims,win,wout
	call_procedure,maskname,lims,wdatp.e,wdatp.e
endif else wout=win

GIVE_DATP,wdatp
return,wout

end
