;+
;	function mload,nums,F12=F12,yaxis=yaxe,norm=norm,mask=maskstring,multiy=multiy,tol=tol
;
;	LAMP function to create a workspace from a series of datasets
;	with different flipper states
;	Supercedes POLOAD and MONOLOAD
;	ARW 30.11.06
;
;	NECESSARY VARIABLES
;	nums=EITHER a string with the filenumbers containing the data. 
;		Consecutive files are separated by a colon (:)
;		Adjacent files are separated by a comma (,)
;		The string must be enclosed in quotation marks (')
;		e.g. '90230:90240,90250:90260'
;	     OR an array of numbers
;
;	OPTIONAL VARIABLES:
;	F12 = two element array with flipper states (=0 for off, =1 for on)
;		e.g. F12=[0,1] is F1=off, F2=on
;		At the moment, this is ONLY necessary if 'nums' is a STRING
;		NOTE:  If you HAVEN'T defined these and 'nums' is a STRING,
;			the program will NOT differentiate between flipper states.
;			ALL the data at the same 'yaxis' value will be added up! 
;	norm=your choice of normalization, 'time', 'mon', or 'none'
;			The default (if no scantime is entered) is 'time'
;	yaxe=your choice for the yaxis, 'san','dan','tra', or 'ran'
;			The default (if no yaxe is entered) is 'san'
;	maskstring=your choice of masking the detector, written as a string
;			The default is no mask
;		NOTE:  The data is NOT normalized to the number of y-pixels 
;			in the mask, i.e. circular and elliptical masks will have
;			less weighting away from the centre.  Divide by a
;			masked water run to perform this action
;	multiy=your choice to combine y data with the same SAN/DAN value or not
;		A string variable, ='av' to combine, 'noav' to leave uncombined
;		Default is 'av'
;	tol=a tolerance in y for the y-combination  NOTE:  This is a FUDGE!  SAN often misses a bit, so I have put in a TOLERANCE
; 		Franck has to FIX THIS!  ARW 13.05.05
;
;	OUTPUT VARIABLES
;	wout is a workspace with the data of the desired flipper states.  If the data is 2D, it will be
;	integrated vertically  
;		The X-axis is given in pixel number
;		The Y-axis is given as SAN or DAN, user's choice
;		The monitor is given as time, monitor, or as unity (for 'none'), user's choice
;		There is a new array of variables in the 'pv' field.  This is a M rows * 6 columns array,
;			where M is the number of steps in the scans.  The columns are:
;		   [SAN, DAN, mean date, counting time, monitor, normalization factor,S2W,S3W]
;			The normalization factor should be multiplied with the data for any further operations
;			(e.g. addition of workspaces) and the errors for the correct propagation of errors
;			
;	e.g.. type at the LAMP interface:
;		w1=mload('68263:68300',F12=[0,0],yaxis='san',norm='time',mask='box,[190,200,63,222]',multiy='av')
;	     or:
;		w1=mload(68263+indgen(37),F12=[0,0])
;
;	CALLS TO:  rdrun, maskme (external), arraygen, datecon, d17_errmsg (external)
;	last modified	ARW 22.4.11
;-

forward_function rdrun
forward_function maskme

; ************************************************************************************
; *************** First define the arraygen function *********************************
; ************************************************************************************
pro arraygen,arraytext,outarray
;+
;	LAMP procedure to create an array of file numbers from a string
;	ARW 30.08.04
;
;	NECESSARY VARIABLES:
;	arraytext = the text containing the file numbers 
;		and separation parameters
;	The separation parameters are:
;		first:last = takes a sequence of numbers from first to last
;		first,last = takes the first and the last numbers only
;
;	OPTIONAL VARIABLES
;
;	OUTPUT VARIABLES:
;	outarray = an array of filenumbers
;
;	e.g. arraygen,'90201:90220,90230',outarray
;
;	CALLS TO:
;	Last modified:	ARW 10.09.04
;-

; **	Find the separators
outarray=0
colon=-10
dsep=0
while dsep ne -1 do begin
	dsep=strpos(arraytext,':',dsep+1)
	if dsep ne -1 then colon=[colon,dsep]
endwhile
if n_elements(colon) gt 1 then colon=colon[1:n_elements(colon)-1]

dsep=0
comma=-10
while dsep ne -1 do begin
	dsep=strpos(arraytext,',',dsep+1)
	if dsep ne -1 then comma=[comma,dsep]
endwhile
if n_elements(comma) gt 1 then comma=comma[1:n_elements(comma)-1]

;***	Define the number and position of separators entered
sep=-1
dcol=where(colon gt 0,count)
if count gt 0 then sep=[sep,colon[dcol]] 
dcom=where(comma gt 0,count)
if count gt 0 then sep=[sep,comma[dcom]] 
sep=[sep,strlen(arraytext)]
sep=sep[sort(sep)]

;***	Extract file numbers from the string
fnums=0
for i=0,n_elements(sep)-2 do begin
	fnums=[fnums,long(strmid(arraytext,sep[i]+1,sep[i+1]))]
endfor
fnums=fnums[1:n_elements(fnums)-1]

;***	Now create the output array
dout=fnums[0]
for i=1,n_elements(sep)-2 do begin
	oper=where(colon eq sep[i],cntcol)
	oper=where(comma eq sep[i],cntcom)
	if cntcol ne 0 then begin
		dout=[dout,fnums[i-1]+indgen(fnums[i]-fnums[i-1]+1)]
	endif
	if cntcom ne 0 then dout=[dout,fnums[i-1],fnums[i]]
endfor

;***	Check for doubles
outarray=0
for i=0,n_elements(dout)-1 do begin
	if dout[i] gt 0 then begin
		outarray=[outarray,dout[i]]
		dbls=where(dout eq dout[i],count)
		if count gt 1 then dout[dbls]=-1
	endif
endfor

outarray=outarray[1:n_elements(outarray)-1]
;print,outarray

end

; ************************************************************************************
; *************** Now define the datecon function ************************************
; ************************************************************************************
pro datecon,input,start,stop
;+
;	LAMP procedure to take the date line from a D17 data file
;	and return the start and finish times as a binary string.
;	Example of the date line:
;		06-AUG-04 09:41:33  06-AUG-04 10:11:29 
;	ARW 31.08.04
;
;	NECESSARY VARIABLES:
;	input=the ascii string that represents the date line from the file
;
;	OUTPUT VARIABLES:
;	start=the start time as a binary string
;	stop=the finish time as a binary string
;
;	e.g.
;		datecon,dates,start,end	
;
;	CALLS TO:
;	Last modified:	ARW 02.09.04
;-

; ***	Split the input string in to the four components
bpos=0
dtext=strarr(4)
for i=0,3 do begin
	bpos=strpos(input,' ')
	dtext[i]=strtrim(strmid(input,0,bpos))
	input=strtrim(strmid(input,bpos,strlen(input)-1),1)
endfor

; ***	dtext(1) and dtext(3) have the dates in them.
; ***	They need to be converted in to MON,DD and YYYY format

startcal=dtext[0]
startday=strmid(startcal,0,2)
startmon=strmid(startcal,3,3)
startyr='20'+strmid(startcal,7,2)

start=bin_date('XXX '+startmon+' '+startday+' '+dtext[1]+' '+startyr)
start=julday(start[1],start[2],start[0],start[3],start[4],start[5])

endcal=dtext[2]
endday=strmid(endcal,0,2)
endmon=strmid(endcal,3,3)
endyr='20'+strmid(endcal,7,2)

stop=bin_date('XXX '+endmon+' '+endday+' '+dtext[3]+' '+endyr)
stop=julday(stop[1],stop[2],stop[0],stop[3],stop[4],stop[5])

;print,start
;print,stop

end


; ************************************************************************************
; *************** And finally the 'mload' function ***********************************
; ************************************************************************************
function mload,nums,F12=F12,yaxis=yaxe,norm=norm,mask=maskstring,multiy=multiy,tol=tol
;+
;******* *****
;**
;-

; ***	Determine whether 'nums' is a numeric array or a string
ntype=size(nums,/TYPE)
if ntype eq 7 then begin	; turn it in to a numeric array if it is a string
	arraygen,nums,numsout
	nums=numsout
endif
nscans=n_elements(nums)

; ***	Define the optional variables if not defined, and check typing
; ***   Determine whether a tolerance is needed
if n_elements(tol) ne 1 then tol=0.004
; ***	If F12 is defined, determine whether it is correctly defined
Fsts=[[0,0],[0,1],[1,0],[1,1]]
if n_elements(F12) ne 0 then begin
	if n_elements(F12) ne 2 then begin
		d17_errmsg,1,'ERROR: F12 must have two elements (e.g. F12=[0,0])'
		return,0
	endif
	print,'Flipper state '+strtrim(string(F12[0]),1)+' '+strtrim(string(F12[1]),1)+' selected'
	F12=where(Fsts[0,*] eq F12[0] and Fsts[1,*] eq F12[1])
	if F12 eq -1 then begin
		d17_errmsg,1,'ERROR: The values of F12 must be either 0 or 1'
		return,0
	endif
	
endif 
; *** 	Determine what to use as the y-axis
if n_elements(yaxe) ne 1 then yaxe='san'
yaxeopts=where(strmatch(['san','dan','tra','ran'],yaxe,/FOLD_CASE))
if yaxeopts eq -1 then begin
	d17_errmsg,1,'ERROR: You must enter san, dan, tra, ran, or nothing for the y-axis'
	return,0
endif
; ***	Determine whether to normalise the data and to what
if n_elements(norm) ne 1 then norm='time'
if (norm ne 'time') then begin
	if (norm ne 'mon') then begin
		if (norm ne 'none') then begin
			d17_errmsg,1,'ERROR: You must enter time, mon or none for normalization'
			return,0
		endif
	endif
endif
; ***	Define what gets saved in the 'monitor' field of the output workspace
; ***	Put time for monitor if there is no normalization
preset=norm
if preset eq 'none' then preset='time'
mflag = 1
if n_elements(maskstring) ne 1 then begin
	d17_errmsg,1,'No data mask is applied'
	maskstring='none'
endif
; *** Determine whether averaging data with the same Y is required
if n_elements(multiy) ne 1 then multiy='av'
if (multiy ne 'av') then begin
	if (multiy ne 'noav') then begin
		d17_errmsg,1,'WARNING: MULTIY is badly specified.  Data at equivalent ',yaxe, ' WILL be averaged'
		multiy='av'
	endif
endif else print,'All data at equivalent ',yaxe,' will be added up'

; ***	Read in the first scan to workspace
dp=1
w_in_all=rdrun(nums[0],datp=dp)
time_all=dp.p[1]
mon_all=dp.p[2]
sanarr_all=dp.p[3]
danarr_all=dp.p[4]
traarr_all=dp.p[30]
ranarr_all=dp.p[31]
s2warr_all=dp.p[7]
s3warr_all=dp.p[9]
F1state=dp.p[22]
F2state=dp.p[23]
w_size=size(w_in_all)

; ***	Mask the workspace if required
mod_datp,dp,"e",sqrt(w_in_all)
w_in_all=maskme(maskstring,w_in_all,datp=dp)
; ***	Check dimensions of the workspace. If it is 2D, sum vertically
if w_size[0] eq 2 then w_in_all=total(w_in_all,2)

; ***	Read start and finish times for the scan
datecon,dp.time,sstart,send
sdates_all=mean([sstart,send])

;***	now loop and fill the workspace with the scans in the range
tdatp=1
d17_errmsg,1,'Loading data.  Please wait...'
for i=1,nscans-1 do begin
	w_temp=rdrun(nums[i],datp=tdatp)
	mod_datp,tdatp,"e",sqrt(w_temp)
	w_temp=maskme(maskstring,w_temp,datp=tdatp)
	w_size=size(w_temp)
	if w_size[0] eq 2 then w_temp=total(w_temp,2)
	datecon,tdatp.time,sstart,ssend
	w_in_all=[[w_in_all],[w_temp]]
	sanarr_all=[sanarr_all,tdatp.p[3]]
	danarr_all=[danarr_all,tdatp.p[4]]
	traarr_all=[traarr_all,tdatp.p[30]]
	ranarr_all=[ranarr_all,tdatp.p[31]]
	s2warr_all=[s2warr_all,tdatp.p[7]]
	s3warr_all=[s3warr_all,tdatp.p[9]]
	sdates_all=[sdates_all,mean([sstart,ssend])]
	time_all=[time_all,tdatp.p[1]]
	mon_all=[mon_all,tdatp.p[2]]
	F1state=[F1state,tdatp.p[22]]
	F2state=[F2state,tdatp.p[23]]
endfor

; ***	Define choice for y-variables
case yaxeopts[0] of
	0: yvalues_all=sanarr_all
	1: yvalues_all=danarr_all
	2: yvalues_all=traarr_all
	3: yvalues_all=ranarr_all
endcase

; *** 	Now sort the data in to the approrpiate flipper states
d17_errmsg,1,'There are a total of '+strtrim(string(nscans))+' scans in the selected range'
count=nscans  
; ***	If one flipper state was asked for, choose the appropriate data
if n_elements(F12) ne 0 then Fsts=Fsts[*,F12]
; ***	Start the loop
for j=0,n_elements(Fsts[0,*])-1 do begin
	suffix=strtrim(string(Fsts[0,j]),1)+strtrim(string(Fsts[1,j]),1)
	F=where((F1state eq Fsts[0,j]) and (F2state eq Fsts[1,j]),tcount)	
	d17_errmsg,1,'Flip state '+strtrim(string(Fsts[0,j]),1)+strtrim(string(Fsts[1,j]),1)+' starts with '+strtrim(string(tcount),1)+' pts'
	if tcount gt 0 then begin
		w_in=w_in_all[*,F]
		yvalues=yvalues_all[F]
		sanarr=sanarr_all[F]
		danarr=danarr_all[F]
		traarr=traarr_all[F]
		ranarr=ranarr_all[F]
		s2warr=s2warr_all[F]
		s3warr=s3warr_all[F]
		sdates=sdates_all[F]
		time=time_all[F]
		mon=mon_all[F]
	 	if multiy eq 'av' then begin
; ***	Check to see if there are multiple values for y, average them if there are
; ***	Start by defining the output variables
			wout=fltarr(n_elements(dp.x))
			eout=wout
			pvout=fltarr(8)
			yout=0.0
; ***	Now start a loop.
; ***	NOTE:  There is a FUDGE here!  SAN often misses a bit, so I have put in a TOLERANCE
; ***	Franck has to FIX THIS!  ARW 13.05.05
;	tol=0.004
			ny=n_elements(yvalues)
			for i=0,ny-1 do begin
				if finite(yvalues[i]) then begin
					dbls=where(abs(yvalues-yvalues[i]) lt tol)
					if n_elements(dbls) gt 1 then begin
						wout=[[wout],[total(w_in[*,dbls],2)]]
						yout=[yout,mean(yvalues[dbls])]
						pvout=[[pvout],[ $
							mean(sanarr[dbls]),$
							mean(danarr[dbls]),$
							mean(sdates[dbls]),$
							total(time[dbls]),$
							total(mon[dbls]),$
							1.0,$
							mean(s2warr[dbls]),$
							mean(s3warr[dbls])]]
						yvalues[dbls]=!VALUES.D_NAN
					endif else begin
						wout=[[wout],[w_in[*,i]]]
						yout=[yout,yvalues[i]]
						pvout=[[pvout],[sanarr[i],danarr[i],sdates[i],time[i],mon[i],1.0,s2warr[i],s3warr[i]]]			
					endelse 
				endif
			endfor
; ***	Tidy up the output variables
			wout=wout[*,1:*]
			yout=yout[1:*]
			pvout=pvout[*,1:*]
		endif else begin
			print,'WARNING: no ',yaxe,' combination chosen.  Data at equivalent ',yaxe,' will *not* be averaged'
			wout=w_in
			yout=yvalues
			pvout=[transpose(sanarr),transpose(danarr), $
				transpose(sdates),transpose(time), $
				transpose(mon),0.0*transpose(sanarr)+1.0,$
				transpose(s2warr),transpose(s3warr)]
		endelse	

; ***	Sort all the data in ascending order relative to 'y'
		yindex=sort(yout)
		yout=yout[yindex]
		wout=wout[*,yindex]
		pvout=pvout[*,yindex]

; ***	Define the errors as the square root of the input data
		eout=sqrt(wout)
		eout[where(eout eq 0.0)] = 1.0

; ***	Determine whether to normalise to time or monitor
		ny=n_elements(yout)
		if strcmp(norm,'time') gt 0 then nfac=3
		if strcmp(norm,'mon') gt 0 then nfac=4 ; ***	NOTE TO SELF:  check whether we have a monitor divider!
		if strcmp(norm,'none') gt 0 then nfac=5
		for i=0,ny-1 do begin
			wout[*,i]=wout[*,i]/pvout[nfac,i]
			eout[*,i]=eout[*,i]/pvout[nfac,i]
			pvout[5,i]=pvout[nfac,i]
		endfor
		nout=pvout[nfac,*]
;		dp.p(1)=mean(pvout(3,*))
;		dp.par_txt(1)=' 1 Mean run time     '
;		dp.p(2)=mean(pvout(4,*))
;		dp.par_txt(2)=' 2 Mean monitor      '			 

	d17_errmsg,1,'		and finishes with '+strtrim(string(n_elements(yout)),1)+' pts'
	endif else begin
		wout=0
		yout=0
	endelse
; ***	Create an array with the output data
	if n_elements(wout) gt 1 then begin
;		if j eq 0 then begin		; ***	First, define the output workspace
		if n_elements(wout_all) eq 0 then begin		; ***	First, define the output workspace
			wout_all=wout
			eout_all=eout
			yout_all=yout
			pvout_all=pvout
			nout_all=nout
			zout_all=10*Fsts[0,j]+1*Fsts[1,j]
		endif else begin		; ***	Now, stack the other workspaces on top
; ***	First, check that X and Y dimensions correspond
			wallsz=size(wout_all)
			woutsz=size(wout)
			if wallsz[1] ne woutsz[1] then begin
				print,'ERROR: X-dimensions do not correspond!'
				print,'You have a problem with the detector binning, prod ARW for a software fix'
				return,0
			end
; ***	If the arrays are the same size, they will directly stack
			if array_equal(wallsz,woutsz) or n_elements(yout_all[*,0]) eq n_elements(yout) then begin
				wout_all=[[[wout_all]],[[wout]]]
				eout_all=[[[eout_all]],[[eout]]]
				yout_all=[[yout_all],[yout]]
				pvout_all=[[[pvout_all]],[[pvout]]]
				nout_all=[[nout_all],[nout]]
				zout_all=[zout_all,10*Fsts[0,j]+1*Fsts[1,j]]
			endif else begin
; ***	If the arrays have a different number of y values, give an error for the moment
; ***	A potential change for the future?  ARW 30.11.06				
				d17_errmsg,1,'ERROR: Y values do not correspond!'
				print,yout_all[*,0]
				print,yout
				d17_errmsg,2,'First set of y values'
				dtxt1=string(yout_all[*,0])
				dtxt2=dtxt1[0]
				for i=1,n_elements(dtxt1) do dtxt2=[dtxt2+' '+dtxt1[1]]
				d17_errmsg,2,string(dtxt2)
				d17_errmsg,2,'Second set of y values'
				dtxt1=string(yout)
				dtxt2=dtxt1[0]
				for i=1,n_elements(dtxt1) do dtxt2=[dtxt2+' '+dtxt1[1]]
				d17_errmsg,2,string(dtxt2)
				return,0
			endelse
		endelse
	endif	
endfor

; ***	Tidy up the y-axis
yout_all=yout_all[*,0]

; ***	Tidy up the y-axis if there is only one Q point
wallsz=size(wout_all)
if wallsz[0] eq 3 and wallsz[2] eq 1 then yout_all=zout_all

;***	Write the new values to the new workspace
MOD_DATP, dp, "y", yout_all
MOD_DATP, dp, "z", zout_all
MOD_DATP, dp, "n",nout_all
MOD_DATP,dp,"e",eout_all
MOD_DATP,dp,"y_tit",yaxe
MOD_DATP,dp,"pv",pvout_all

GIVE_DATP, dp
RETURN, wout_all
end






