function xspec,win,spix,lbkg=lbkg,rbkg=rbkg,xaxis=xaxis,lambda=lambda,sanoff=sanoff
;+
;	LAMP function to extract the specular intensity from a workspace.
;	It simply integrates the counts within a certain range of x-pixels
;	and subtracts the backgrounds (if specified)
;	It also (correctly) calculates the errors and weights by the number of pixels
;		in each range
;	ARW 30.08.06
;
;	NECESSARY VARIABLES:
;	win = the input workspace.  This has usually been passed through
;		aw_monoload, but will also work for TOF data loaded directly
;	spix = two element array with the left and right pixels that limit the
;		width of the specular ridge
;
;	OPTIONAL VARIABLES:
;	lbkg = two element array with the left and right pixels that limit the
;		background to the LEFT of the specular ridge
;	rbkg = a two element array with the left and right pixles that limit the
;		background to the RIGHT of the specular ridge
;	In reality, the two optional variables are interchangeable 
;		- i.e. left and right don't matter.  What matters is that
;		two separate pixel ranges can be used for background
;	xaxis = a choice of the final x-axis values
;		You may choose the x-axis in Qz (xaxis = 'q')	(default choice)
;					  or theta_in (xaxis='san')
;	lambda = the wavelength.  If unspecified, the wavelength will be taken from the data parameters
;	sanoff = an offset in the sample angle in degrees.  
;		This will be ADDED to the recorded sample angle 
;
;	OUTPUT VARIABLES
;	wout is the workspace containing the specular intensity, background subtracted,
;		with the x-axis = the y axis for win
;
;	e.g. type at LAMP interface:
;		w2=xspec(w1,[60,85],lbkg=[11,59],rbkg=[120,140])
;
;	CALLS TO:
;	Last modified:  ARW 7.11.09
;-

; ** Check that the input parameters have the right number of variables
emsg=0
if n_elements(spix) ne 2 then emsg = 1
if n_elements(lbkg) gt 0 and n_elements(lbkg) ne 2 then emsg = 2
if n_elements(rbkg) gt 0 and n_elements(rbkg) ne 2 then emsg = 3
if emsg gt 0 then begin
	case emsg of
		1: print,'ERROR: You must enter 2 values for the specular range, e.g. [60,85]'
		2: print,'ERROR: You must enter 2 values for the left background, e.g. lbkg=[11,59]'
		3: print,'ERROR: You must enter 2 values for the right background, e.g. rbkg=[86,100]'
	endcase
	return,0
endif

if n_elements(lbkg) eq 0 then lbkg=[-1,-1]
if n_elements(rbkg) eq 0 then rbkg=[-1,-1]

if spix[0] gt spix[1] then emsg = 1
if lbkg[0] gt lbkg[1] then emsg = 2
if rbkg[0] gt rbkg[1] then emsg = 3
if emsg gt 0 then begin
	case emsg of
	1: begin
		print,'WARNING: The left pixel number is larger than the right for the specular range'
		spix=[spix[1],spix[0]]
	   end
	2: begin
		print,'WARNING: The left pixel number is larger than the right for the left background'
		lbkg=[lbkg[1],lbkg[0]]
	   end
	3: begin
		print,'WARNING: The left pixel number is larger than the right for the right background'
		rbkg=[rbkg[1],rbkg[0]]
	   end
	endcase
	print,'The pixel numbers will be swapped'
endif
if n_elements(xaxis) ne 0 then begin
	if strcmp(xaxis,'q') eq 0 then begin
		if strcmp(xaxis,'san') eq 0 then begin
			print,'WARNING: Final x-axis choice is not correctly entered'
			print,'		Setting final x-axis to Qz'
			xaxis='q'
		endif
	endif
endif else begin
	print,'Setting final x-axis to Qz'
	xaxis='q'
endelse

;*** Get important information from the input workspace
TAKE_DATP,datp
ein=datp.e
yin=datp.y
zin=datp.z
pvin=datp.pv

;*** Extract the specular range
wspec=total(win[spix[0]:spix[1],*,*],1)
espec=sqrt(total(ein[spix[0]:spix[1],*,*]^2,1))
nw=float(spix[1]+1-spix[0])

;*** Extract the backgrounds
if total(lbkg) ge 0 then begin
	wlbkg=total(win[lbkg[0]:lbkg[1],*,*],1)
	elbkg=sqrt(total(ein[lbkg[0]:lbkg[1],*,*]^2,1))
	if total(where(wlbkg eq 0)) ge 0 then elbkg[where(wlbkg eq 0)] = 0.0
endif else begin
	wlbkg=0.0*wspec
	elbkg=0.0*wspec
	lbkg=[0,-1]
endelse
if total(rbkg) ge 0 then begin
	wrbkg=total(win[rbkg[0]:rbkg[1],*,*],1)
	erbkg=sqrt(total(ein[rbkg[0]:rbkg[1],*,*]^2,1))
	if total(where(wrbkg eq 0)) ge 0 then erbkg[where(wrbkg eq 0)] = 0.0
endif else begin
	wrbkg=0.0*wspec
	erbkg=0.0*wspec
	rbkg=[0,-1]
endelse

;*** Now make the subtraction
nw=float(spix[1]+1-spix[0])
nb=float(lbkg[1]+1-lbkg[0])+float(rbkg[1]+1-rbkg[0])
if nb eq 0.0 then nb = 1.0

wout=wspec-nw/nb*(wlbkg+wrbkg)
eout=sqrt(espec^2+(nw/nb)^2*(elbkg^2+erbkg^2))

;*** Determine final x-axis values
xout=pvin[0,*]
xtit=datp.y_tit
if n_elements(lambda) eq 0 then lambda = datp.p[24]
if n_elements(sanoff) ne 0 then xout=xout+sanoff
if strcmp(xaxis,'q') eq 1 then begin
	xout=4*!pi*sin(!pi*xout/180.0)/lambda
	xtit='Qz'
endif

;*** Write the new parameters
MOD_DATP,datp,"e",eout
MOD_DATP,datp,"x",xout
MOD_DATP,datp,"y",zin
MOD_DATP,datp,"z",1
MOD_DATP,datp,"x_tit",xtit
MOD_DATP,datp,"y_tit",'Intensity (arb. units)'

GIVE_DATP,datp
RETURN,wout

end
