PRO calib_d1a, numor, n, cal_fil_in, cal_fil_out
;+
;** *********
;**
;** User callable
;** The call is e.g. calib_d1a, 30008, 9,  'd1acal.raw', 'd1acal.eff'
;**
;** 30008 is the first scan, 9 is the number of scans,
;** d1acal.raw is the file with efficiencies=1, d1acal.eff is the new one
;**
;** *** efficiencies are reversed for output since this routine reads data
;** that has been ordered in angle, whereas the raw data  (when the
;** efficiency correction is applied) is in reverse order
;**
;** divide by efficiencies for d1a (multiply for d2b)
;-

print, 'calib_d1a on pcmrjohnson'

;read the first numor and get array dimensions
datp=1 & wi=40 ; use workspace 40 for reading data
win=rdrun(numor,DATP=datp,w=wi)
print, n_elements(datp)
npts=(size(win))[1] & print, 'npoints   ',npts
nt  =25            ;number of detector tubes
nsteps=npts/nt     ;number of scan steps e.g.60 for d1a
tolerance=1e-4     ;used to stop refinment cycle for efficiencies

;define matrix of efficiencies per scan (eff) and global (effg & effgold) (new and old)
eff    =fltarr(n,nt)      & eff[*,*]   =1.0
effg   =fltarr(nt)        & effg[*]    =1.0
effgold=fltarr(nt)        & effgold[*] =2.0
icount_max=30 & effgall=fltarr(nt,icount_max+1)

;make array to store all calibration workspaces and enter the first one, just read
w_all=fltarr(n,npts) & w_diff=fltarr(icount_max,npts)
w_all[0,*]=win
print, 'reading data'
for i=1,n-1 do begin
  newnumor=numor+i
  win=rdrun(newnumor)
  w_all[i,*]=win
endfor

;find average intensity and scale it, used to find beamstop
ave_int=0.5 * total(win)/npts & print, 'scaled ave intensity for beamstop correction', ave_int

;define arrays for storing summed/total counts
n_totpts=npts+(n-1)*nsteps & print, 'total no. of points in pattern', n_totpts
w_tot=fltarr(n_totpts) & w_test=fltarr(n_totpts,icount_max)
i_tot=intarr(n_totpts)

;while loop to refine efficiencies, stop when they are no longer changing according to the tolerance
icount=0 & deviation=tolerance*2
while (deviation gt tolerance) and (icount lt icount_max) do begin
	icount=icount+1 & print, 'doing efficiencies   ', icount
	;calculate the total including efficiencies, initialise total and counter arrays
	w_tot[*]=0.0 & i_tot[*]=0
	print, 'accumulating scans'
	for i=0,n-1 do begin     ;loop over different scans
	  for j=0,nt-1 do begin  ;loop over detector tubes
	    alllowlim=nsteps*j     & allhilim=alllowlim+nsteps-1
	    totlowlim=nsteps*(i+j) & tothilim=totlowlim+nsteps-1
	    if (total(w_all[i,alllowlim:allhilim]/nsteps) gt ave_int) then begin
	      ; include detector if intensity is reasonable
	      i_tot[totlowlim:tothilim]=i_tot[totlowlim:tothilim]+1
	      ; calculate total from weighted sum of individual scans
	      w_tot[totlowlim:tothilim]=w_tot[totlowlim:tothilim]+w_all[i,alllowlim:allhilim]/effg[j]
	    endif else begin
	      print,'scan/tube omitted',i,j
        endelse
	  endfor
	endfor

    ;calculate the average
    index=where(i_tot gt 0,count) & print,'gt zero in total', count
	w_tot[index]=w_tot[index]/i_tot[index]
	w_test[*,icount-1]=w_tot

    ;calculate differences between individual scans and the average i.e. efficiencies per scan
	;averaged over the number of scan steps
	for i=0,n-1 do begin    ;loop over different scans
 	 for j=0,nt-1 do begin  ;loop over detector tubes
 	   alllowlim=nsteps*j     & allhilim=alllowlim+nsteps-1
 	   totlowlim=nsteps*(i+j) & tothilim=totlowlim+nsteps-1
 	   ;sum over nsteps, invert compared to d2b
 	   ;what to do with eff(i,j) that is not set
 	   if (total(w_tot[totlowlim:tothilim]) gt 0) then begin
 	     eff[i,j]=total(w_all[i,alllowlim:allhilim])/total(w_tot[totlowlim:tothilim])
 	   endif else begin
 	     eff[i,j]=-1.0
 	   endelse
 	 endfor
	endfor

	;calulate average/global efficiency for detectors not omitted
	print,'average efficiencies and deviation from previous values'
    for j=0,nt-1 do begin
      index=where(eff[*,j] gt 0.,count)
      effg[j]=total(eff[index,j],1)/count
    endfor

    ;copy efficiencies to effgall to check convergence
    effgall[*,icount]=effg

    ;calculate deviation from previous efficiencies
	deviation=total(total(abs(effg-effgold),1))/nt
	print,'deviation ',deviation & print, 'average efficiency ',mean(effg)

	;update previous efficiencies
	print,'updating old efficiencies'
	effgold=effg

endwhile ;*** efficiencies refined in effg

;normalise efficiencies
ave=mean(effg) & print, ave & effg=effg/ave

print, 'max and min efficiencies ', max(effg), ' & ', min(effg)

; ****************** update calibration file ******************
;write new calibration file
;4 lines; title,angles,efficiencies, integers
print, 'writing calibration file'
openr,f_in,cal_fil_in,/get_lun
;read from old calibration file
j_tit='' & j_angles=' ' & j_eff=' ' & j_int=' '
readf,f_in,j_tit    ;skip title
readf,f_in,j_angles ;get angles
readf,f_in,j_eff    ;get efficiencies
readf,f_in,j_int    ;get integers
free_lun, f_in
;write new calib file
openw,f_out,cal_fil_out,/get_lun
title='vana:'+string(numor)+', '+string(n)+' files'
printf,f_out,title    ;write title
printf,f_out,j_angles ;write angles
printf,f_out,format='(17f8.5,8f9.5)', reverse(effg)     ;write efficiencies
printf,f_out,j_int    ;write integers
free_lun, f_out

end
