PRO calib_ang_tubes_gauss, numor, n, wi, cal_fil_in, cal_fil_out, title
;+
;** *********************
;**
;** User callable
;** The call is e.g. calib_ang, 51577, 5, 20,  'd2bcal.zeff', 'd2bcal.ang', 'title'
;**
;** 51577 is the first scan, 5 is the number of scans, 20 is the number of a working workspace
;** d2bcal.raw is the old file, d2bcal.eff is the new one
;** data must be read in with a calibration file that defines the active zones and efficiencies,
;** no detectors should be excluded (set cutoff>3.0)
;** angles should be standard values, separation 1.25 degrees, step 0.05
;** Assume data is CeO2 and define angular ranges in which peak positions can be found.
;** Read-in n scans and calculate the total
;** Get angular positions for each scan and for total
;** Also detector numbers for peaks in each scan
;** Output useful info in calib.info
;-

; define list of peak position ranges i.e. angular ranges in which to search for maxima (fit?)
; this is specifically for CeO2
FORWARD_FUNCTION rdrun, rdsum, gaussfit
n_peaks=14
peak_limits=[28,32,32,36,48,52,56,60,60,64,70,74,77,81,81,85,91,95,98,102,111,115,119,123,123,127,136,140]
;n_peaks=1 & peak_limits=[32,36]
; and arrays in which to store peak angles and detectors and for total spectrum
peak_pos=fltarr(n,n_peaks) & peak_det=intarr(n,n_peaks) & peak_pos_tot=fltarr(n_peaks)

; will convert 2D patterns to 1D, define height limits
h_min=44 & h_max=84
; tolerance
tolerance=1e-3     ;used to stop refinment cycle for efficiencies

;read the first numor and get array dimensions
print, 'reading ...'
datp=1
win=rdrun(numor,DATP=datp,w=wi)
print, n_elements(datp)
npts=(size(win))[1] ;& print, 'npoints   ',npts
nh  =(size(win))[2] ;& print, 'nh        ',nh
nt  =128           ;number of detector tubes
nsteps=npts/nt     ;number of scan steps e.g.25, n is number of different scans
win_min=fltarr(n+1) & win_max=fltarr(n+1) & win_min[0]=datp.x[0] & win_max[0]=datp.x[npts-1]

;make array to store all workspaces and angles and enter the first one, just read
w_all=fltarr(n,npts,nh) & win_ang=fltarr(n+1,npts)
w_all[0,*,*]=win        & win_ang[0,*]=datp.x
print, 'reading data'
for i=1,n-1 do begin
  newnumor=numor+i
  win=rdrun(newnumor,DATP=datp,w=wi) & w_all[i,*,*]=win ; get data
  win_ang[i,*]=datp.x & win_min[i]=win_ang[i,0] & win_max[i]=win_ang[i,npts-1] ; get angles & min & max
endfor

;make 2D patterns into 1D patterns
print,'2D to 1D'
w_all_1D=total(w_all[*,*,h_min:h_max],3)

;find peak positions
print,'extracting angles ...'
for i=0,n-1 do begin
  for j=0,n_peaks-1 do begin
    amin=peak_limits[j*2] & amax=peak_limits[j*2+1]                    ;get limiting angles from list
    idx_amin=round((npts-1)*(amin-win_min[i])/(win_max[i]-win_min[i])) ;convert from angle to index - min
    idx_amax=round((npts-1)*(amax-win_min[i])/(win_max[i]-win_min[i])) ;convert from angle to index - max
    ;*** use gaussfit to find the peak position
    ;*** define angular range x, intensities y
      x=win_ang[i,idx_amin:idx_amax]
      y=w_all_1D[i,idx_amin:idx_amax]
      peak_max=max(w_all_1D[i,idx_amin:idx_amax])                      ; max value for Gaussfit
      peak_idx=where(w_all_1D[i,*] eq peak_max)                        ; index
      peak=win_ang[i,peak_idx]                                         ; angle
      peak_fit=gaussfit(x,y,a,estimates=[1000.0,peak,0.2,10.0],nterms=4); fit
    peak_pos[i,j]=a[1]                                                 ; get angle from fit parameters
    peak_det[i,j]=floor(peak_idx/nsteps)                               ; corresponding detector
  endfor
endfor

;get total pattern and angles, watch out for parameters for 1 scan rather than sum
print,'reading total ...'
numor_end=numor+n-1
win=rdsum(numor,numor_end,DATP=datp,w=wi)
win_min[n]=win_min[0] & win_max[n]=win_max[n-1] ; get limits from individual scans
ang_step=0.05 ; ang_step=abs(win_ang(0,0)-win_ang(0,1))         ; get step size (should be 0.05)
npts=round((win_max[n]-win_min[n])/ang_step+1)  ; to calculate number of points in total scan
win_ang_tot=win_min[n]+ang_step*findgen(npts)
	print, ang_step, win_min[n], win_max[n], npts
;convert to 1D
win_1D=total(win[*,h_min:h_max],2)

;get peak positions in total pattern
print,'finding peaks ...'
for j=0,n_peaks-1 do begin
  amin=peak_limits[j*2] & amax=peak_limits[j*2+1]                    ;get limiting angles from list
  idx_amin=round((npts-1)*(amin-win_min[n])/(win_max[n]-win_min[n])) ;convert from angle to index - min
  idx_amax=round((npts-1)*(amax-win_min[n])/(win_max[n]-win_min[n])) ;convert from angle to index - max
  ;*** use gaussfit to find the peak position
  ;*** define angular range x, intensities y
    x=win_ang_tot[idx_amin:idx_amax]
    y=win_1D[idx_amin:idx_amax]
    peak_max=max(win_1D[idx_amin:idx_amax])                          ; max value for Gaussfit
    peak_idx=where(win_1D eq peak_max)                               ; index
    peak=win_ang_tot[peak_idx]                                       ; angle
    peak_fit=gaussfit(x,y,a,estimates=[1000.0,peak,0.2,10.0],nterms=4); fit
  peak_pos_tot[j]=a[1]                                               ; angle
endfor

;read calibration file to get angles used in reading in all of this data
;copy zones and efficiencies into new file, put in new title
print,'reading and writing calibration file   '
openr,f_in,cal_fil_in,/get_lun
openw,f_out,cal_fil_out,/get_lun
junk=''
icount=0
while junk ne 'angles' do begin
  icount=icount+1
  readf,f_in,junk
  if icount eq 1 then printf,f_out,title else printf,f_out,junk ; print title on first line
endwhile
cal_angles=fltarr(nt)
readf,f_in,cal_angles ;& print, cal_angles

;calculate a new set of detector angles for each scan
shift_all=fltarr(n,nt)
count_all=intarr(n,nt)
for i=0,n-1 do begin
  shift_all[i,*]=0.0 & count_all[i,*]=0
  for j=0,n_peaks-1 do begin
    idx_det=peak_det[i,j]
    shift_all[i,idx_det]=peak_pos_tot[j]-peak_pos[i,j] & count_all[i,idx_det]=1
  endfor
endfor
; calculate average shifts and add to angles
shift_ave=fltarr(nt) & shift_ave[*]=0.0
for j=0,nt-1 do begin
  if abs(total(count_all[*,j],1)) gt 0 then begin
    shift_ave[j]=total(shift_all[*,j],1)/total(count_all[*,j],1)
    if abs(shift_ave[j])lt 0.0001 then shift_ave[j]=0.0 ; clean up some numerical errors
  endif
endfor
cal_angles=cal_angles+shift_ave & print, shift_ave

;write calibration file with new angles
print, 'writing angles to calibration file'
printf,f_out,cal_angles
;for j=0,nt-1 do printf,f_out,j,' ',cal_angles(j)
printf,f_out,'calib_end'

free_lun, f_in & free_lun, f_out
print, 'finished calibration file'

;write useful info
;make list of detectors which measure peaks
dets_in=reform(peak_det,n*n_peaks)
dets_uniq=dets_in[uniq(dets_in,sort(dets_in))]
n_dets=n_elements(dets_uniq)
dets_num=dets_uniq
for i=0,n_dets-1 do begin
  idx=where(dets_in eq dets_uniq[i]) & dets_num[i]=n_elements(idx)
endfor

openw,f_out,'calib_g.info',/get_lun
printf,f_out,'peak positions - individual scans'
printf,f_out,peak_pos
printf,f_out,'peak positions/detectors - individual scans'
printf,f_out,peak_det
printf,f_out,'peak positions - total scan'
printf,f_out,peak_pos_tot
printf,f_out,'detectors used, occurences and shifts'
for j=0,n_dets-1 do printf,f_out,dets_uniq[j],dets_num[j],shift_all[*,dets_uniq[j]],shift_ave[dets_uniq[j]]
printf,f_out,'max shift= ',max(abs(shift_ave))
free_lun, f_out

end
