FUNCTION calib_eff_1d, numor, n, wi, cal_fil_in, cal_fil_out, title
;+
;******* ************
;**
;** User callable
;** The call is e.g. w2=calib_eff_1d(51577, 5, 20,  'd2bcal.raw', 'd2bcal.eff', 'title')
;**
;** 51577 is the first scan, 5 is the number of scans, 20 is the number of a working workspace
;** d2bcal.raw is the old file, d2bcal.eff is the new one
;** data must be read in with a calibration file that defines the active zones
;** since efficiencies are to be determined for these zones,
;** *** but the efficiencies table in                      ***
;** *** the input calibration file should contain only 1's ***
;** this macro as for calib_eff, but 1 efficiency per tube rather than height resolution
;** in "1d" version integrate over height at the start
;** wout (=w2), returned workspace, contains 1d efficiencies
;-

print, 'calib_eff_1d on pcmrjohnson'

;read the first numor and get array dimensions
datp=1
win=rdrun(numor,DATP=datp,w=wi)
print, n_elements(datp)
npts=(size(win))[1] & print, 'npoints   ',npts
nh  =(size(win))[2] & print, 'nh        ',nh
nt=128             ;number of detector tubes
nsteps=npts/nt     ;number of scan steps e.g.25, n is number of different scans
tolerance=1e-4     ;used to stop refinment cycle for efficiencies

;define matrix of efficiencies per scan (eff) and global (effg & effgold) (new and old)
eff    =fltarr(n,nt)      & eff[*,*] =1.0
effg   =fltarr(nt)        & effg[*]=1.0
effgold=fltarr(nt)        & effgold[*]=2.0
effout =fltarr(nh,nt)     & effout[*,*]=1.0
efflimit=1.5 ; limit to exclude bad detectors from sum
tube_score=intarr(nt) & tube_score[*]=0 ; array to describe good or bad tubes
;tube_score(41)=1 & tube_score(45)=1 & tube_score(101)=1 & tube_score(102)=1 & tube_score(121)=1 ; bad tubes to be omitted from total
;effg(41)=2.1     & effg(45)=2.1     & effg(101)=2.1     & effg(102)=2.1     & effg(121)=2.1     ; fix efficiency

;make array to store all calibration workspaces and enter the first one, just read
w_all=fltarr(n,npts)
w_all[0,*]=total(win,2)
print, 'reading data'
for i=1,n-1 do begin
  newnumor=numor+i
  win=rdrun(newnumor)
  w_all[i,*]=total(win,2)
endfor

;define arrays for storing summed/total counts
n_totpts=npts+(n-1)*nsteps & print, n_totpts
w_tot=fltarr(n_totpts)
i_tot=intarr(n_totpts)

;while loop to refine efficiencies, stop when they are no longer changing according to the tolerance
icount=0 & deviation=tolerance*2
while (deviation gt tolerance) and (icount lt 100) do begin
	icount=icount+1 & print, 'doing efficiencies   ', icount
	;calculate the total including efficiencies, initialise total and counter arrays
	w_tot[*]=0.0 & i_tot[*]=0
	print, 'accumulating scans'
	for i=0,n-1 do begin
	  for j=0,nt-1 do begin
	    if (tube_score[j] lt 1) then begin
	      alllowlim=nsteps*j     & allhilim=alllowlim+nsteps-1
	      totlowlim=nsteps*(i+j) & tothilim=totlowlim+nsteps-1
	      i_tot[totlowlim:tothilim]=i_tot[totlowlim:tothilim]+1
	      w_tot[totlowlim:tothilim]=w_tot[totlowlim:tothilim]+w_all[i,alllowlim:allhilim]*effg[j]
	    endif else begin
	      ; if (i eq 0) then print,'detector missing', j
	    endelse
	  endfor
	endfor
	;calculate the average
	print, 'average of scans'
	w_tot[*]=w_tot[*]/i_tot[*]

	;calculate differences between individual scans and the average i.e. efficiencies per scan
	;averaged over the number of scan steps
	for i=0,n-1 do begin    ;loop over different scans
 	 for j=0,nt-1 do begin  ;loop over detector tubes
 	   if (tube_score[j] lt 1) then begin
 	     alllowlim=nsteps*j     & allhilim=alllowlim+nsteps-1 ;& print, alllowlim, allhilim  ;limits to define a single tube
 	     totlowlim=nsteps*(i+j) & tothilim=totlowlim+nsteps-1 ;& print, totlowlim, tothilim  ;limits to define a single tube, including scan offset
 	     eff[i,j]=total(w_tot[totlowlim:tothilim])/total(w_all[i,alllowlim:allhilim]) ;sum over nsteps
 	   endif
 	 endfor
	endfor

	;calulate average/global efficiency for detectors not omitted
	print,'average efficiencies and deviation from previous values'
    for j=0,nt-1 do begin
      if (tube_score[j] lt 1) then effg[j]=total(eff[*,j],1)/n
    endfor
    ;print, 'efficiencies',effg

	;calculate deviation from previous efficiencies
	deviation=total(total(abs(effg-effgold),1))/nt & print,'deviation ',deviation & print, 'average efficiency ',mean(effg)

	;update previous efficiencies
	print,'updating old efficiencies'
	effgold=effg

endwhile ;*** efficiencies refined in effg

;normalise efficiencies
ave=mean(effg) & print, ave & effg=effg/ave

print, 'max and min efficiencies ', max(effg), ' & ', min(effg)

; ****************** update calibration file ******************
;convert effg into effout for the calibration file
for j=0,nt-1 do effout[*,j]=effg[j] ;effout in format (nh,nt)
;and write new calibration file
print, 'writing calibration file'
openr,f_in,cal_fil_in,/get_lun
openw,f_out,cal_fil_out,/get_lun
;write new zones info
printf,f_out,title
;copy zones from old calibration file
junk=''
readf,f_in,junk ;skip title
while junk ne 'efficiencies' do begin
   readf,f_in,junk
   printf,f_out,junk
endwhile
;write efficiencies
printf,f_out,effout
;read down to angles in old file
while junk ne 'angles' do readf,f_in,junk
;then write angles from old file
printf,f_out,'angles'
while junk ne 'calib_end' do begin
   readf,f_in,junk
   printf,f_out,junk
endwhile
;printf,f_out,'calib_end'
free_lun, f_in & free_lun, f_out

return, effg

end
