FUNCTION calib_eff_2d_A, numor, n, wi, bms_threshold, cal_fil_in, cal_fil_out, title
;+
;******* ****************
;**
;** User callable
;** The call is e.g. w2=calib_eff_2d_A(51577, 5, 20, 0.5, 'd2bcal.raw', 'd2bcal.eff', 'title')
;**
;** 51577 is the first scan, 5 is the number of scans, 20 is the number of a working workspace,
;** 0.5 is the fraction of the average intensity used as threshold to determine tubes masked by beamstop,
;** w2 is an output workspace containing the table of efficiencies
;** d2bcal.raw is the old file, d2bcal.eff is the new one
;** data must be read in with a calibration file that defines the active zones
;** since efficiencies are to be determined for these zones,
;** *** but the efficiencies table in                      ***
;** *** the input calibration file should contain only 1's ***
;** this macro as for calib_eff_tubes_1d, but efficiencies per vertical block (height hb) rather than per tube
;** in "1d" version integrate over height at the start
;** ---------------------------------------------------
;** METHOD
;** ------
;** Sum data using initial efficiencies to get average diffraction pattern
;** Difference between each scan and the average gives efficiency per tube, height group and scan
;** Average efficiency over different scans gives new effciency per tube, height group
;** Repeat
;** Currently the beamstop is not omitted.
;** For this, identify tubes that scan the beam stop
;** Use tube_score(scan,tube)
;** ---------------------------------------------------
;-

forward_function rdrun 

; debug = 0 --> minimal output
; debug = 1 --> print main info
; debug = 2 --> print all details + plots
; debug = 3 --> print list of blocks omitted and strange eff values
debug = 1

;standard: fullTube = 0
;fullTube = 1 --> Calculate eff for the full tube, not only active zone
;Requires that d2bcal.raw is in use before calling the calibration routine
;All eff values < 0.3 or > 3.0 are replaced by 1 
fullTube = 0

;read the first numor and get array dimensions
datp=1
win=rdrun(numor,DATP=datp,w=wi)
print, ''
npts=(size(win))[1] 
nh  =(size(win))[2] 
nt=128             ;number of detector tubes
nsteps=npts/nt     ;number of scan steps e.g.25, n is number of different scans
tolerance=1e-4     ;used to stop refinment cycle for efficiencies
max_cycle=30       ;max no. of cycles in efficiency calculation
min_cycle=3        ;min no. of cycles in efficiency calculation
hb=16              ;height of blocks for efficiency calculation
nb=nh/hb           ;number of blocks in height

if debug ge 1 then begin
  print, 'CALCULATING EFFICIENCY: '
  print, '  Number of points in scan = ', npts
  print, '  Number of detector tubes (hardcoded) = ', nt
  print, '  Number of pixels per tube = ', nh
  print, '  Number of pixels per block (hardcoded) = ', hb
  print, '  Number of blocks per tube = ', nb
  print, ''
endif

;define matrix of efficiencies per scan (eff) and global (effg & effgold) (new and old)
eff=fltarr(n,nt,nb)          & effg=fltarr(nt,nb)           & effgold=fltarr(nt,nb)
eff[*,*,*]=1.0               & effg[*,*]=1.0                & effgold[*,*]=2.0
effout =fltarr(nh,nt)        & effout[*,*]=1.0

;make array to store all calibration workspaces and enter the first scan, read the rest
w_all_3d=fltarr(n,npts,nh)
w_all_3d[0,*,*]=win
print, '1. Reading data'
for i=1,n-1 do begin
  newnumor=numor+i
  win=rdrun(newnumor)
  w_all_3d[i,*,*]=win
endfor
print, ''

;find average intensity and scale it, used to find beamstop
if (bms_threshold < 0) or (bms_threshold > 1) then bms_threshold = 0.5
ave_block = total(win)/npts/nb
ave_int = bms_threshold * ave_block
if debug ge 1 then begin
  print, 'Average intensity in vertical block = ', ave_block, ' (', ave_block/hb, ' per pixel)'
  print, 'Threshold used to discard blocks = ', ave_int
  print, ''
endif

;define arrays for storing summed/total counts
n_totpts=npts+(n-1)*nsteps ;& print, n_totpts
w_tot=fltarr(n_totpts)
i_tot=intarr(n_totpts)

;while loop to refine efficiencies, stop when they are no longer changing according to the tolerance
icount=0 & deviation=tolerance*2
dev_old=0. & dev_new=0. & del_dev=1.; variables to follow gradient of deviation, stop on sign change
while (icount lt max_cycle) and (abs(del_dev) gt 1.0e-5) do begin
  excluded = 0
  included = 0
	icount=icount+1 
	print, '2. Doing efficiencies cycle ', icount
	print, ''
	;calculate the total including efficiencies, initialise total and counter arrays
	w_tot[*]=0.0 & i_tot[*]=0
	for i=0,n-1 do begin       ; loop thro scans
	  for j=0,nt-1 do begin    ; loop thro tubes
	    alllowlim=nsteps*j     & allhilim=alllowlim+nsteps-1 ;limits to define a single tube
	    totlowlim=nsteps*(i+j) & tothilim=totlowlim+nsteps-1 ;limits to define a single tube, including scan offset
	    for jh=0,nb-1 do begin ; loop thro height blocks
	      hmin=jh*hb & hmax=(jh+1)*hb-1
	      tot_temp=total(w_all_3d[i,alllowlim:allhilim,hmin:hmax],3) ; a vector nsteps long
	      if (mean(tot_temp) gt ave_int) then begin
	        i_tot[totlowlim:tothilim]=i_tot[totlowlim:tothilim]+hb ; count how many times adding into w_tot
	        w_tot[totlowlim:tothilim]=w_tot[totlowlim:tothilim]+tot_temp*effg[j,jh]
	        included += 1
	      endif else begin
	        if debug ge 3 then print, ' --> Omitting scan,tube,height: ', i, j, jh
	        excluded += 1
	      endelse
	    endfor
	  endfor
	endfor
  
	;calculate the average
	index = where(i_tot gt 0,count) & w_tot[index]=w_tot[index]/i_tot[index]
	if debug ge 1 then begin
	  print, 'Number of blocks accepted = ', included, ', number of blocks excluded = ', excluded
	  print, ''
	  if debug ge 2 then begin
	    print, 'w_tot = Vector containing counts * eff at each measured position. Size = ', n_elements(w_tot)
	    print, 'w_tot contains the AVERAGE number of counts PER PIXEL. Plot:'
	    plot = plot(w_tot)
	  endif
	endif
  
	;calculate differences between individual scans and the average i.e. efficiencies per scan i
	;averaged over the number of scan steps
	for i=0,n-1 do begin      ; loop over different scans
 	 for j=0,nt-1 do begin    ; loop over detector tubes
 	   alllowlim=nsteps*j     & allhilim=alllowlim+nsteps-1 ;limits to define a single tube
 	   totlowlim=nsteps*(i+j) & tothilim=totlowlim+nsteps-1 ;limits to define a single tube, including scan offset
 	   for jh=0,nb-1 do begin ; loop thro height blocks, nb is number of blocks height hb
 	     hmin=jh*hb & hmax=(jh+1)*hb-1
 	     totot_temp = total(w_all_3d[i,alllowlim:allhilim,hmin:hmax])/hb
 	     if (totot_temp gt 0 and total(w_tot[totlowlim:tothilim]) gt 0) then begin
 	       eff[i,j,jh]=total(w_tot[totlowlim:tothilim])/totot_temp
 	       if (debug ge 3 and eff[i,j,jh] gt 5.0) then print, ' --> Efficiency > 5 for scan, tube, block = ', i, j, jh
 	     endif else begin
 	       eff[i,j,jh]=-1.0
 	       if debug ge 3 then begin
 	         print, ' --> Cannot compute efficiency for scan, tube, block = ', i, j, jh
 	         print, ' --> totot_temp = total(w_all_3d[i,alllowlim:allhilim,hmin:hmax])/hb = ', totot_temp
 	         print, ' --> total(w_tot[totlowlim:tothilim]) = ', total(w_tot[totlowlim:tothilim])
 	       endif
 	     endelse
 	   endfor
 	 endfor
	endfor
  
	;calculate average/global efficiency for detectors not omitted
  for j=0,nt-1 do begin
     for jh=0,nb-1 do begin
        index=where(eff[*,j] gt 0.,count)
        if count gt 0 then effg[j,jh]=total(eff[index,j,jh],1)/count
     endfor
     if debug ge 2 then print, 'Tube ', j, ' effg[j,*] = ', transpose(effg[j,*])
  endfor
  
  ;calculate deviation from previous efficiencies
  idx = where(effg gt 0.3 and effg lt 3.0) ;use only 'reasonable' efficiencies to check convergence
	dev_new=total(abs(effg[idx]-effgold[idx]))/nt/nb  
	del_dev=dev_old-dev_new & dev_old=dev_new 
	if debug ge 1 then begin
	  print, 'Deviation = ', dev_new, ', change from last dev = ', del_dev 
	  print, 'Eff(min, max, mean, median) = ', min(effg), max(effg), mean(effg), median(effg)
    print, ''
	endif
  
	;update previous efficiencies
	effgold=effg
	
endwhile ;*** efficiencies refined in effg

ave = total(effg)/n_elements(effg)
print, '3. Efficiency calculation completed. Average eff before normalizing = ', ave
print, ''
if fullTube then begin 
  idx = where(effg lt 0.3, count)
  if count gt 0 then effg[idx] = 1.0
  idx = where(effg gt 3.0, count)
  if count gt 0 then effg[idx] = 1.0
endif else effg=effg/ave

;write transpose of effg into effout
for j=0,nt-1 do begin
  for jh=0,nb-1 do begin
    hmin=jh*hb & hmax=(jh+1)*hb-1
    effout[hmin:hmax,j]=effg[j,jh]
  endfor
endfor

; ****************** update calibration file ******************
;effout in format (nh,nt), write new calibration file
print, '4. Writing calibration file'
print, ''
openr,f_in,cal_fil_in,/get_lun
openw,f_out,cal_fil_out,/get_lun
;write new zones info
printf,f_out,title + ' (method A)'
;copy zones from old calibration file
junk=''
readf,f_in,junk ;skip title
while junk ne 'efficiencies' do begin
   readf,f_in,junk
   printf,f_out,junk
endwhile
;write efficiencies
printf,f_out,effout
;read down to angles in old file
while junk ne 'angles' do readf,f_in,junk
;then write angles from old file
printf,f_out,'angles'
while junk ne 'calib_end' do begin
   readf,f_in,junk
   printf,f_out,junk
endwhile
;printf,f_out,'calib_end'
free_lun, f_in & free_lun, f_out
return, effout

end
