FUNCTION calib_eff_2d_D, numor, n, wi, cal_fil_in, cal_fil_out, title
;+
;******* ****************
;**
;** User callable
;** The call is e.g. w2=calib_eff_2d(51577, 5, 20,  'd2bcal.raw', 'd2bcal.eff', 'title')
;**
;** 51577 is the first scan, 5 is the number of scans, 20 is the number of a working workspace
;** w2 is an output workspace containing the table of efficiencies
;** d2bcal.raw is the old file, d2bcal.eff is the new one
;** data must be read in with a calibration file that defines the active zones
;** since efficiencies are to be determined for these zones,
;** *** but the efficiencies table in                      ***
;** *** the input calibration file should contain only 1's ***
;** this macro as for calib_eff_tubes_1d, but efficiencies per vertical block (height hb) rather than per tube
;** in "1d" version integrate over height at the start
;** ---------------------------------------------------
;** METHOD
;** ------
;** Sum data using initial efficiencies to get average diffraction pattern
;** Difference between each scan and the average gives efficiency per tube, height group and scan
;** Average efficiency over different scans gives new effciency per tube, height group
;** Repeat
;** Currently the beamstop is not omitted.
;** For this, identify tubes that scan the beam stop
;** Use tube_score(scan,tube)
;** ---------------------------------------------------
;-
forward_function rdrun

;read the first numor and get array dimensions
datp=1
win=rdrun(numor,DATP=datp,w=wi)
print, n_elements(datp)
npts=(size(win))[1] & print, 'npoints   ',npts
nh  =(size(win))[2] & print, 'nh        ',nh
nt=128             ;number of detector tubes
nsteps=npts/nt     ;number of scan steps e.g.25, n is number of different scans
tolerance=1e-4     ;used to stop refinment cycle for efficiencies
max_cycle=30       ;max no. of cycles in efficiency calculation
min_cycle=3        ;min no. of cycles in efficiency calculation
hb=1               ;height of blocks for efficiency calculation
nb=nh/hb           ;number of blocks in height

;define matrix of efficiencies per scan (eff) and global (effg & effgold) (new and old)
eff=fltarr(n,nt,nb)          & effg=fltarr(nt,nb)           & effgold=fltarr(nt,nb)
eff[*,*,*]=1.0               & effg[*,*]=1.0                & effgold[*,*]=2.0
effout =fltarr(nh,nt)        & effout[*,*]=1.0

;make array to store all calibration workspaces and enter the first scan, read the rest
w_all_3d=fltarr(n,npts,nh)
w_all_3d[0,*,*]=win
print, 'reading data'
for i=1,n-1 do begin
  newnumor=numor+i
  win=rdrun(newnumor)
  w_all_3d[i,*,*]=win
endfor
print, 'data read!'

w_all_3d_blocks = fltarr(n,npts,nb)
if hb gt 1 then begin
   for jh=0,nb-1 do begin ; loop thro height blocks
      hmin=jh*hb & hmax=(jh+1)*hb-1
      w_all_3d_blocks[*,*,jh] = total(w_all_3d[*,*,hmin:hmax],3)
   endfor
endif else w_all_3d_blocks = w_all_3d

median_pixel = median(w_all_3d)
median_block = median(w_all_3d_blocks)
median_tube  = median(w_all_3d) * 128
counts_in_blocks = fltarr(nt,nb)
counter_in_blocks = fltarr(nt,nb)
tolerance = 0.5 * median_tube * nsteps
median_block_nsteps = median_block * nsteps
tubes_done = fltarr(nt)
adding = []

for i=0,n-1 do begin        ; loop thro scans
   for j=0,nt-1 do begin    ; loop thro tubes
     tube_lowlim=nsteps*j  & tube_hilim=tube_lowlim+nsteps-1 ;limits to define a single tube
     ;only consider full tubes not masked by beamstop
     if (total(w_all_3d_blocks[i, tube_lowlim:tube_hilim, *]) gt tolerance) then begin
       tubes_done[j] = 1.0
       for k=0,nb-1 do begin ; loop thro height blocks
         ; counts in tube j, pixel k sum over nsteps
         tmp_counts = total(w_all_3d_blocks[i, tube_lowlim:tube_hilim, k])
         if tmp_counts gt 0.2 * median_pixel * nsteps then begin
            ;adding = [adding,  total(w_all_3d_blocks[i, tube_lowlim:tube_hilim, k])]
            counts_in_blocks[j,k]  += total(w_all_3d_blocks[i, tube_lowlim:tube_hilim, k])
            counter_in_blocks[j,k] += 1
         endif   
       endfor
     endif
   endfor
endfor

eff_blocks = fltarr(nt,nb)*0.0 + 1.0
check = where(counter_in_blocks le 0.0, missing_blocks, complement=good_blocks)
if missing_blocks gt 0 then print, 'Problem: Efficiency could not be calculated from some detector blocks. Set to 1'
eff_blocks[good_blocks] = median_block_nsteps/(counts_in_blocks[good_blocks]/counter_in_blocks[good_blocks])

;write effg into effout, the transpose
for j=0,nt-1 do begin
  for jh=0,nb-1 do begin
    hmin=jh*hb & hmax=(jh+1)*hb-1
    effout[hmin:hmax,j]=eff_blocks[j,jh]
  endfor
endfor


; ****************** update calibration file ******************
;effout in format (nh,nt), write new calibration file
print, 'writing calibration file'
openr,f_in,cal_fil_in,/get_lun
openw,f_out,cal_fil_out,/get_lun
;write new zones info
printf,f_out,title + ' (method D)'
;copy zones from old calibration file
junk=''
readf,f_in,junk ;skip title
while junk ne 'efficiencies' do begin
   readf,f_in,junk
   printf,f_out,junk
endwhile
;write efficiencies
printf,f_out,effout
;read down to angles in old file
while junk ne 'angles' do readf,f_in,junk
;then write angles from old file
printf,f_out,'angles'
while junk ne 'calib_end' do begin
   readf,f_in,junk
   printf,f_out,junk
endwhile
;printf,f_out,'calib_end'
free_lun, f_in & free_lun, f_out

return, effout

end
