; -----------------------------------------------------------------------------
;******************************************************************************
;
FUNCTION binq, w_in, dQ=dQ, qb
;+
;******* ****
;** User callable
;**Interpolates data along the x-axis to regular steps with point spacing dQ. 
;**
;**ARGUMENTS:
;** dQ  :desired constant Q-bin width
;** (qb is obsolete, kept for backwards compatability)
;**
;**DIMENSIONS:
;** w_in(nspectra) -> w_out(nQ)
;**
;**COMMAND SYNTAX:
;** w2=binq(w1,dQ=<dQ>)
;**            KHA,JRS 4/12/02
;**------------------------------------------------------------------------------
;********************************************************************************
;-

  iprint = 0  ; if iprint>0, show debugging messages

  IF iprint  THEN PRINT,'Start binq:'

  take_datp, datp

  IF N_ELEMENTS(qb) GT 0 THEN dQ = qb
  
; ------------------------------------------------------------------------------
;Check dimensions of input workspace and setup output workspace

  IF ((SIZE(w_in))[0] NE 1) AND ((SIZE(w_in))[0] NE 2) THEN RETURN, w_in
  par = datp.p

  nx = (SIZE(w_in))[1]
  IF (SIZE(w_in))[0] EQ 2 THEN ny = (SIZE(w_in))[2] ELSE ny = 1

  e_in = datp.e
  x_in = datp.x

  IF iprint THEN PRINT,'nx = ', nx, 'ny = ', ny
  IF iprint THEN PRINT,'End of "w_in dimensions check etc." section'
; ------------------------------------------------------------------------------
;Rebin to constant Q

  Qmax   = datp.x[nx - 1]
  nx_out = FIX(Qmax/dQ) + 1
  x_out  = INDGEN(nx_out)*dQ

  w_out  = FLTARR(nx_out,ny)
  e_out  = FLTARR(nx_out,ny) - 1
  wsum   = FLTARR(ny)
  e2sum  = FLTARR(ny)

  i0 = 0
  FOR iQ = 0, nx_out - 1 DO BEGIN
    Q0 = x_out[iQ]
    wsum[*]  = 0.
    e2sum[*] = 0.
    n = 0
    FOR i = i0, nx - 1 DO BEGIN
      Q = x_in[i]
      IF (ABS(Q - Q0) LT dQ/2.) THEN BEGIN
        wsum[*]  = wsum[*] + w_in[i,*]
        e2sum[*] = e2sum[*] + e_in[i,*]^2
        n = n + 1
      ENDIF ELSE IF Q GE Q0 + dQ/2. THEN GOTO, endloop
    ENDFOR
endloop:  i0 = i
    IF n GT 0 THEN BEGIN
      w_out[iQ,*] = wsum/n
      e_out[iQ,*] = sqrt(e2sum)/n
    ENDIF
  ENDFOR

  IF iprint THEN PRINT,'End of "rebin to constant Q" section'

; ------------------------------------------------------------------------------
;Remove undefined points

  i     = WHERE(e_out[*,0] GE 0.,n)
  x_out = x_out[i]
  w_buf = FLTARR(n,ny)
  e_buf = FLTARR(n,ny)
  FOR iy = 0, ny - 1 DO BEGIN
    w_buf[*,iy] = w_out[i,iy]
    e_buf[*,iy] = e_out[i,iy]
  ENDFOR
  w_out = w_buf
  e_out = e_buf

  IF iprint GT 0 THEN PRINT,'End of "remove undefined points" section'

; ------------------------------------------------------------------------------
;Return parameters and exit

  dQs=STRTRIM(STRING(dQ),2) & np= strpos(/reverse_search,dQs,'.')+3 & dQs=STRMID(dQs,0,np)
  PRINT,'binq: data binned to dQ='+dQs
finished:

  mod_datp, datp, "e", e_out
  mod_datp, datp, "x", x_out

  datp.other_tit=datp.other_tit+' -bq(dQ='+dQs+')'

  IF (iprint GT 0) THEN PRINT,'End binq:'

  give_datp, datp

  RETURN, w_out
  END
