; ------------------------------------------------------------------------------
;*******************************************************************************
;
FUNCTION components, w_in0
;+
;******* **********
;** User callable
;** For D7 data only
;** 
;** Separates data taken with Z or XYZ polarisation analysis into
;** the constituent components:
;** 
;** index
;**  0: total
;**  1: incoherent
;**  2: coherent + isotope-incoherent
;**  3: paramagnetic (average of spin-flip and non-spin-flip paramagnetic)
;**  4: non-spin-flip paramagnetic
;**  5: spin-flip paramagnetic
;** 
;** DIMENSIONS:
;**  No-TOF: w_in(nspectra,nphases,nruns) -> w_out(nspectra,ncomponents,nruns)
;**  TOF:    w_in(nchannels,nspectra*nphases)   -> w_out(nchannels,nspectra*ncomponents)
;** 
;** COMMAND SYNTAX:
;**  w3=components(w2)
;** 
;**  (optional keywords shown in square brackets)
;**               KHA,JRS 6/10/06
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-
  
  
  iprint = 0  ; if iprint>0, show debugging messages
  
  IF iprint THEN PRINT,'Start components:'
  
  TAKE_DATP, datp
  
  ; ------------------------------------------------------------------------------
  ;Check dimensions of input workspaces
  
  CHECK_D7, w_in0, datp, 'Components', stf, nspectra, nphases, nruns, TOF
  
  IF TOF THEN nchannels = FIX(datp.p[6])
  IF TOF THEN chw       = datp.p[7]
  
  IF nphases EQ 2 THEN ncomps = 3 ELSE ncomps = nphases
  
  IF nruns EQ 1 THEN x_in = datp.x ELSE x_in = datp.z
  y_in  = datp.y
  e_in0 = datp.e
  
  IF NOT TOF THEN BEGIN
    IF nphases EQ 1 THEN BEGIN
      w_in = FLTARR(nspectra,nphases,nruns)
      e_in = FLTARR(nspectra,nphases,nruns)
      w_in[*,0,*] = w_in0[*,*]
      e_in[*,0,*] = e_in0[*,*]
    ENDIF ELSE BEGIN
      w_in = w_in0
      e_in = e_in0
    ENDELSE
  ENDIF ELSE BEGIN
    w_in = REFORM(w_in0,nchannels,nspectra,nphases)
    e_in = REFORM(e_in0,nchannels,nspectra,nphases)
  ENDELSE
  
  IF iprint THEN BEGIN
    PRINT,'TOF       = ', TOF
    PRINT,'nspectra  = ', nspectra
    PRINT,'nphases   = ', nphases
    IF TOF THEN PRINT,'nchannels = ', nchannels
    PRINT,'nruns     = ', nruns
    PRINT,'ncomps    = ', ncomps
  ENDIF
  
  IF iprint THEN PRINT,'End of "w_in dimensions check etc." section'
  
  ; ------------------------------------------------------------------------------
  ;Separate components
  
  ; No-TOF: w_in(nspectra,nphases,nruns)     -> w_out(nspectra,ncomponents,nruns)
  ; TOF:    w_in(nchannels,nspectra*nphases) -> w_out(nchannels,nspectra*ncomps)
  ;
  ; 0: total
  ; 1: incoherent
  ; 2: coherent + isotope-incoherent
  ; 3: paramagnetic (average of spin-flip and non-spin-flip paramagnetic)
  ; 4: non-spin-flip paramagnetic
  ; 5: spin-flip paramagnetic
  ;
  ; Expressions used here:
  ;   Mnf, Mfl and M the same
  ;   Tnf=Znf+Xnf+Ynf
  ;   Tfl=Zfl+Xfl+Yfl
  ;    SI=Tfl/2-M
  ;  C+II=(2*Tnf-Tfl)/6
  
  IF NOT TOF THEN BEGIN
    w_out = FLTARR(nspectra,ncomps,nruns)
    e_out = FLTARR(nspectra,ncomps,nruns)
    IF nphases EQ 1 THEN w_out = w_in $
    ELSE IF nphases EQ 2 THEN BEGIN
      w_out[*,0,*] = TOTAL(w_in,2)
      e_out[*,0,*] = SQRT(TOTAL(e_in^2,2))
      w_out[*,1,*] = 1.5*w_in[*,1,*]
      e_out[*,1,*] = 1.5*e_in[*,1,*]
      w_out[*,2,*] = w_in[*,0,*] - w_in[*,1,*] / 2.
      e_out[*,2,*] = SQRT(e_in[*,0,*]^2 + (e_in[*,1,*] / 2.)^2)
    ENDIF ELSE BEGIN
      magsf  = 2.* (w_in[*,3,*] + w_in[*,5,*] - 2.*w_in[*,1,*])
      dmagsf = 2.* SQRT(e_in[*,3,*]^2 + e_in[*,5,*]^2 + (2.*e_in[*,1,*])^2)
      magnf  = 2.* (2.*w_in[*,0,*] - w_in[*,2,*] - w_in[*,4,*])
      dmagnf = 2.* SQRT((2.*e_in[*,0,*])^2 + e_in[*,2,*]^2 + e_in[*,4,*]^2)
      zsf = WHERE(dmagsf EQ 0.,nzsf)
      IF nzsf NE 0 THEN dmagsf[zsf] = 1.
      znf = WHERE(dmagnf EQ 0.,nznf)
      IF nznf NE 0 THEN dmagnf[znf] = 1.
      mag  = (magsf/dmagsf^2 + magnf/dmagnf^2) / (1./dmagsf^2 + 1./dmagnf^2)
      dmag = 1./SQRT(1./dmagsf^2 + 1./dmagnf^2)
      IF nzsf NE 0 THEN dmagsf[zsf] = 0.
      IF nznf NE 0 THEN dmagnf[znf] = 0.
      IF nzsf NE 0 THEN BEGIN
        mag[zsf]  = magnf[zsf]
        dmag[zsf] = dmagnf[zsf]
      ENDIF
      IF nznf NE 0 THEN BEGIN
        mag[znf]  = magsf[znf]
        dmag[znf] = dmagsf[znf]
      ENDIF
      
      Tnf   = w_in[*,0,*] + w_in[*,2,*] + w_in[*,4,*]
      dTnf  = SQRT(e_in[*,0,*]^2 + e_in[*,2,*]^2 + e_in[*,4,*]^2)
      Tfl   = w_in[*,1,*] + w_in[*,3,*] + w_in[*,5,*]
      dTfl  = SQRT(e_in[*,1,*]^2+e_in[*,3,*]^2+e_in[*,5,*]^2)
      inc   = Tfl/2. - mag
      dinc  = SQRT((dTfl/2.)^2 + dmag^2)
      coh   = (2.*Tnf - Tfl)/6.
      dcoh  = SQRT((2.*dTnf)^2 + dTfl^2)/6.
      
      w_out[*,0,*] = TOTAL(w_in,2) / 3.
      e_out[*,0,*] = SQRT(total(e_in^2,2))/3.
      w_out[*,1,*] = inc
      e_out[*,1,*] = dinc
      w_out[*,2,*] = coh
      e_out[*,2,*] = dcoh
      w_out[*,3,*] = mag  & e_out[*,3,*] = dmag
      w_out[*,4,*] = magnf  & e_out[*,4,*] = dmagnf
      w_out[*,5,*] = magsf  & e_out[*,5,*] = dmagsf
    ENDELSE
  ENDIF ELSE BEGIN
    w_out = FLTARR(nchannels,nspectra,ncomps)
    e_out = FLTARR(nchannels,nspectra,ncomps)
    IF nphases EQ 1 THEN w_out = w_in $
    ELSE IF nphases EQ 2 THEN BEGIN
      w_out[*,*,0] = TOTAL(w_in,3)
      e_out[*,*,0] = SQRT(TOTAL(e_in^2,3))
      w_out[*,*,1] = 1.5*w_in[*,*,1]
      e_out[*,*,1] = 1.5*e_in[*,*,1]
      w_out[*,*,2] = w_in[*,*,0] - w_in[*,*,1]/2.
      e_out[*,*,2] = sqrt(e_in[*,*,0]^2 + (e_in[*,*,1]/2.)^2)
    ENDIF ELSE BEGIN
      magsf  = 2.*(w_in[*,*,3] + w_in[*,*,5] - 2.*w_in[*,*,1])
      dmagsf = 2.*SQRT(e_in[*,*,3]^2 + e_in[*,*,5]^2 + (2.*e_in[*,*,1])^2)
      magnf  = 2.*(2.*w_in[*,*,0]-w_in[*,*,2]-w_in[*,*,4])
      dmagnf = 2.*SQRT((2.*e_in[*,*,0])^2 + e_in[*,*,2]^2 + e_in[*,*,4]^2)
      zsf    = WHERE(dmagsf EQ 0.,nzsf)
      IF nzsf NE 0 THEN dmagsf[zsf] = 1.
      znf   = WHERE(dmagnf EQ 0.,nznf)
      IF nznf NE 0 THEN dmagnf[znf] = 1.
      mag   = (magsf/dmagsf^2 + magnf/dmagnf^2)/(1./dmagsf^2 + 1./dmagnf^2)
      dmag  = 1./SQRT(1./dmagsf^2 + 1./dmagnf^2)
      IF nzsf NE 0 THEN dmagsf[zsf] = 0.
      IF nznf NE 0 THEN dmagnf[znf] = 0.
      IF nzsf NE 0 THEN BEGIN
        mag[zsf]  = magnf[zsf]
        dmag[zsf] = dmagnf[zsf]
      ENDIF
      IF nznf NE 0 THEN BEGIN
        mag[znf]  = magsf[znf]
        dmag[znf] = dmagsf[znf]
      ENDIF
      Tnf  = w_in[*,*,0] + w_in[*,*,2] + w_in[*,*,4]
      dTnf = SQRT(e_in[*,*,0]^2 + e_in[*,*,2]^2 + e_in[*,*,4]^2)
      Tfl  = w_in[*,*,1] + w_in[*,*,3] + w_in[*,*,5]
      dTfl = SQRT(e_in[*,*,1]^2 + e_in[*,*,3]^2 + e_in[*,*,5]^2)
      w_out[*,*,0] = TOTAL(w_in,3)/3.
      e_out[*,*,0] = SQRT(TOTAL(e_in^2,3))/3.
      w_out[*,*,1] = Tfl/2. - mag
      e_out[*,*,1] = SQRT((dTfl/2.)^2 + dmag^2)
      w_out[*,*,2] = (2.*Tnf - Tfl)/6.
      e_out[*,*,2] = SQRT((2.*dTnf)^2 + dTfl^2)/6.
      w_out[*,*,3] = mag  & e_out[*,*,3] = dmag
      w_out[*,*,4] = magnf  & e_out[*,*,4] = dmagnf
      w_out[*,*,5] = magsf  & e_out[*,*,5] = dmagsf
    ENDELSE
  ENDELSE
  
  IF iprint THEN PRINT,'End of Component Separation section'
  
  ; ------------------------------------------------------------------------------
  ;Rezero detectors and check dimensions of output data
  
  IF NOT TOF THEN BEGIN
    FOR irun = 0, nruns - 1 DO BEGIN
      zeroed = WHERE(e_in[*,0,irun] LT -0.9, nz)
      IF nz NE 0 THEN BEGIN
        w_out[zeroed,*,irun] =  0.
        e_out[zeroed,*,irun] = -1.
      ENDIF
    ENDFOR
  ENDIF ELSE BEGIN
    w_buf = INTARR(nchannels,nspectra)*0
    FOR i = 0, nphases - 1 DO BEGIN
      zeroed = WHERE(e_in[*,*,i] LE -1.,n)
      IF n GT 0 THEN w_buf[zeroed] = 1
    ENDFOR
    zeroed = WHERE(w_buf EQ 1,nz)
    IF iprint THEN PRINT,nz/nchannels,' zeroed spectra'
    IF nz GT 0 THEN FOR i = 0, nphases - 1 DO BEGIN
      w_buf = w_out[*,*,i]  & e_buf = e_out[*,*,i]
      w_buf[zeroed] = 0.  & e_buf[zeroed] = -1.
      w_out[*,*,i]  = w_buf  & e_out[*,*,i]  = e_buf
    ENDFOR
  ENDELSE
  
  IF TOF THEN BEGIN
    w_out = REFORM(w_out,nchannels,nspectra*ncomps)
    e_out = REFORM(e_out,nchannels,nspectra*ncomps)
    y_buf = y_in[0:nspectra - 1]
    IF ncomps EQ 3 THEN y_out = [y_buf,y_buf,y_buf]
    IF ncomps EQ 6 THEN y_out = [y_buf,y_buf,y_buf,y_buf,y_buf,y_buf]
  ENDIF
  
  IF iprint THEN PRINT,'End of "Rezero detectors" section'
  
  ; ------------------------------------------------------------------------------
  ;Return parameters and exit
  
  datp.p[2] = FLOAT(ncomps)
  IF TOF THEN mod_datp, datp, "y", y_out
  IF nphases EQ 2 THEN MOD_DATP, datp, "e", e_out $
  ELSE datp.e = e_out
  IF TOF AND (nphases EQ 6) THEN MOD_DATP, datp, "y", y_in[0:ncomps*nspectra - 1]
  
  datp.w_tit     = 'component of '+STRTRIM(datp.w_tit,2)
  datp.other_tit = datp.other_tit+' -co'
  PRINT, 'Components: components separated'
  
  finished:
  IF iprint THEN PRINT,'End Components:'
  
  GIVE_DATP, datp
  
  RETURN, w_out
END
