; ------------------------------------------------------------------------------
;*******************************************************************************

FUNCTION corr_xyz, w_in0, filenum = iFR, fudge = fudge, frn
;+
;******* ********
;** User calable
;** For D7 polarisation analysis data only
;** 
;** Corrects for finite polarization using a previously measured quartz run
;** 
;** ARGUMENTS:
;**  filenum	: numor(s) of flipping ratio file(s) to use:
;** 		  e.g. filenum=27134 to use the file "quartz_27134.dat'
;** 		       filenum=[27134,27135,27136] to use multiple files
;**  fudge		: Specifies an array of fudgefactors (for depolarization
;**       corrections) for each spectrum
;** 
;** DIMENSIONS:
;**  TOF   -> w_in=w_out(nchannels,nspectra*nphases)
;**  NOTOF -> w_in=w_out(nspectra,nphases,nruns)
;** 
;** COMMAND SYNTAX
;**   w2=corr_xyz(w1,filenum=<filenum>[,temp=<temp>])
;** 
;**               KHA,JRS 29/5/05
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-
  
  iprint = 0  ; if iprint>0, show debugging messages
  
  IF iprint THEN PRINT,'Start corr_xyz:'
  ON_IOERROR, finished
  
  TAKE_DATP, datp
  
  IF N_ELEMENTS(frn) GT 0 THEN iFR  = frn
  ; ------------------------------------------------------------------------------
  ;Check dimensions of input workspaces
  
  CHECK_D7, w_in0, datp, 'Corr_xyz', stf, nspectra, nphases, nruns, TOF
  
  IF TOF THEN nchannels = FIX(datp.p[6])
  
  IF iprint THEN BEGIN
    PRINT,'TOF      =',TOF
    PRINT,'nspectra =',nspectra
    PRINT,'nphases  =',nphases
    PRINT,'nruns    =',nruns
  ENDIF
  
  IF nruns EQ 1 THEN x_in = datp.x ELSE x_in = datp.z
  y_in  = datp.y
  e_in0 = datp.e
  
  IF NOT TOF AND (nphases EQ 1) THEN BEGIN
    w_in = FLTARR(nspectra,nphases,nruns)
    e_in = FLTARR(nspectra,nphases,nruns)
    w_in[*,0,*] = w_in0[*,*]
    e_in[*,0,*] = e_in0[*,*]
  ENDIF ELSE BEGIN
    w_in = w_in0
    e_in = e_in0
  ENDELSE
  
  nFR     = N_ELEMENTS(iFR)
  FR_file = STRARR(nFR)
  corr_FR = 1
  
  CASE nFR OF
    0: BEGIN
      PRINT, 'Corr_xyz: no flipping ratio correction'
      iFR     = 0
      corr_FR = 0
    END
    1: BEGIN
      FR_file[0] = 'quartz_'+STRTRIM(STRING(iFR),2)+'.dat'
      ii = file_search(FR_file[0], COUNT = co)
      IF co EQ 0 THEN BEGIN
        FR_file[0] = '/home/vis/d7/lambda/QUARTZFILES/' + FR_file[0]
        ii = file_search(FR_file[0], COUNT = co)
        IF co EQ 0 THEN BEGIN
          s = 'Corr_XYZ: Cannot find quartz file'
          i = dialog_message(s, /ERROR)
          RETURN, w_in
        ENDIF
      ENDIF
      PRINT,'Corr_xyz: performing FR correction with quartz file: ' + FR_file[0]
    END
    ELSE: BEGIN
      PRINT,'Corr_xyz: performing FR correction with quartz files:'
      FOR i = 0, nFR - 1 DO BEGIN
        FR_file[i] = 'quartz_' + STRTRIM(STRING(iFR[i]),2) + '.dat'
        ii = file_search(FR_file[i],COUNT = co)
        IF co EQ 0 THEN BEGIN
          FR_file[i] = '/home/vis/d7/lambda/QUARTZFILES/' + FR_file[i]
          junk = file_search(FR_file[i], COUNT = co)
          IF co EQ 0 THEN BEGIN
            s = 'Corr_XYZ: Cannot find quartz file'
            i = dialog_message(s, /ERROR)
            RETURN, w_in
          ENDIF
        ENDIF
        PRINT,'Corr_xyz: performing FR correction with quartz files:' + FR_file[i]
      ENDFOR
    END
  ENDCASE
  
  IF iprint THEN BEGIN
    PRINT,'nruns   = ',nruns
    PRINT,'corr_FR = ', corr_FR, ' nFR = ', nFR
  ENDIF
  
  IF (nruns EQ 1) AND (nFR GT 1) THEN BEGIN
    s = STRARR(2)
    s[0] = 'Corr_XYZ: Workspace contains data from only one run,'
    s[1] = '          but more than one quartz file is given'
    i = dialog_message(s, /ERROR)
    RETURN, w_in
  ENDIF
  
  IF iprint THEN PRINT,'End of "w_in dimensions check etc." section'
  
  ; ------------------------------------------------------------------------------
  ;Open and read from input files
  
  line=''
  
  IF corr_FR THEN BEGIN
    num   = FLTARR(5)
    Flip  = FLTARR(nspectra,nphases/2,nruns)
    dFlip = FLTARR(nspectra,nphases/2,nruns)
    
    FOR irun =0, nFR - 1 DO BEGIN
      IF iprint THEN PRINT,'Opening FR file: ' + FR_file[irun]
      OPENR, lun, FR_file[irun], ERROR = err, /GET_LUN
      IF err NE 0 THEN BEGIN
        i = dialog_message(!error_state.msg, /ERROR)
        RETURN, w_in
      ENDIF
      FOR i = 0, 2 DO READF, lun, line
      line = line + ' -1'
      READS, line, num
      IF (num[4] GT -1.1) AND (num[4] LT -0.9) THEN BEGIN
        F = FLTARR(4,nspectra)
        IF nphases EQ 6 THEN BEGIN
          s = 'Corr_XYZ: Error - only 2 quartz phases in file'
          i = dialog_message(s, /ERROR)
        ENDIF
      ENDIF ELSE BEGIN
        F = FLTARR(8,nspectra)
        IF (nphases EQ 2) AND iprint THEN $
          PRINT,'Corr_xyz: Taking Z quartz phase for FR correction'
      ENDELSE
      POINT_LUN, lun, 0
      FOR i = 0, 1 DO READF, lun, line
      READF, lun, F
      FREE_LUN, lun
      iphase = INDGEN(nphases/2)*2
      Flip[*,iphase/2,irun] = TRANSPOSE(F[2+iphase,*])
      dFlip[*,iphase/2,irun] = TRANSPOSE(F[3+iphase,*])
    ENDFOR
    
    IF nruns GT 1 THEN BEGIN
      inrun = INTARR(nruns)
      IF nFR EQ 1 THEN BEGIN
        Flip[*,*,0:nruns-1] =  Flip[*,*,inrun]
        dFlip[*,*,0:nruns-1] = dFlip[*,*,inrun]
      ENDIF ELSE IF (nruns GT 2) AND (nFR EQ 2) THEN BEGIN
        FOR irun=nruns-1,1,-1 DO BEGIN
          Flip[*,*,irun] =  Flip[*,*,0] +  (Flip[*,*,1] -  Flip[*,*,0])*FLOAT(irun)/FLOAT(nruns-1)
          dFlip[*,*,irun]= dFlip[*,*,0] + (dFlip[*,*,1] - dFlip[*,*,0])*FLOAT(irun)/FLOAT(nruns-1)
        ENDFOR
      ENDIF
    ENDIF
  ENDIF
  
  IF (iprint GT 0) THEN PRINT,'End of "read from input files" section'
  
  ; ------------------------------------------------------------------------------
  ;Perform correction
  
  IF (N_ELEMENTS(fudge) EQ 1) AND (corr_FR EQ 1) THEN BEGIN
  
    Flip = fudge*Flip
    dFlip = fudge*dFlip
    
  ENDIF ELSE IF  (N_ELEMENTS(fudge) GT 1) AND (corr_FR EQ 1) THEN BEGIN
    FOR i = 0, nphases/2 - 1 DO BEGIN
      Flip[*,i,*]  = fudge[i]*Flip[*,i,*]
      dFlip[*,i,*] = fudge[i]*dFlip[*,i,*]
    ENDFOR
  ENDIF
  
  w_out = w_in
  e_out = e_in
  
  IF NOT TOF THEN BEGIN
    w_out  = FLTARR(nspectra,nphases,nruns)
    e_out  = FLTARR(nspectra,nphases,nruns)
    iphase = INDGEN(nphases/2)*2
    R0     = w_in[*,iphase,*]
    R1     = w_in[*,iphase + 1,*]
    F      = Flip[*,iphase/2,*]
    dR0    = e_in[*,iphase,*]
    dR1    = e_in[*,iphase + 1,*]
    dF     = dFlip[*,iphase/2,*]
    
    w_out[*,iphase,*]     = R0 + (R0 - R1)/(F - 1.)
    e_out[*,iphase,*]     = SQRT((dR0*F/(F - 1))^2 + (dF*(R0 - R1)/(F - 1.)^2)^2)
    w_out[*,iphase + 1,*] = R1 - (R0 - R1)/(F - 1.)
    e_out[*,iphase + 1,*] = SQRT((dR1*F/(F-1))^2 + (dF*(R0 - R1)/(F - 1.)^2)^2)
    
    ispec = INDGEN(nspectra)
    i     = WHERE(w_in EQ 0. AND e_in LT 0., nz)
    IF nz NE 0 THEN w_out[i] = 0.
    IF nz NE 0 THEN e_out[i] = -1.
  ENDIF ELSE BEGIN
    w_out  = FLTARR(nchannels,nphases*nspectra)
    e_out  = FLTARR(nchannels,nphases*nspectra)
    iphase = INDGEN(nphases / 2) * 2
    ispec  = INDGEN(nspectra,nphases)
    R0     = w_in[*,REFORM(ispec[*,iphase])]
    R1     = w_in[*,REFORM(ispec[*,iphase + 1])]
    dR0    = e_in[*,REFORM(ispec[*,iphase])]
    dR1    = e_in[*,REFORM(ispec[*,iphase + 1])]
    ones   = FLTARR(nchannels) + 1.0
    F  = FLTARR(nchannels,nspectra*nphases/2)
    dF = FLTARR(nchannels,nspectra*nphases/2)
    FOR iph = 0, nphases/2 - 1 DO BEGIN
      F[*,INDGEN(nspectra) + iph*nspectra] = ones #  Flip[*,iph]
      dF[*,INDGEN(nspectra) + iph*nspectra] = ones # dFlip[*,iph]
    ENDFOR
    
    w_out[*,REFORM(ispec[*,iphase])]   = R0 + (R0 - R1)/(F - 1.)
    e_out[*,REFORM(ispec[*,iphase])]   = SQRT((dR0*F/(F - 1.))^2 + (dF*(R0 - R1)/(F - 1.)^2)^2)
    w_out[*,REFORM(ispec[*,iphase+1])] = R1 - (R0 - R1)/(F - 1.)
    e_out[*,REFORM(ispec[*,iphase+1])] = SQRT((dR1*F/(F - 1))^2 + (dF*(R0 - R1)/(F - 1.)^2)^2)
    
    ispec = INDGEN(nspectra*nphases)
    is    = WHERE(TOTAL(w_in[*,ispec],1) EQ 0., nz)
    IF nz NE 0 THEN w_out[*,is] = 0.
    IF nz NE 0 THEN e_out[*,is] = -1.
  ENDELSE
  
  IF iprint THEN PRINT,'End of Correction section'
  
  ; ------------------------------------------------------------------------------
  ;Return parameters and exit
  
  IF NOT TOF AND (nphases EQ 1) THEN BEGIN
    w_out0 = FLTARR(nspectra,nruns)
    e_out0 = FLTARR(nspectra,nruns)
    w_out0[*,*] = w_out[*,0,*]
    e_out0[*,*] = e_out[*,0,*]
  ENDIF ELSE BEGIN
    w_out0 = w_out
    e_out0 = e_out
  ENDELSE
  datp.e     = e_out0
  datp.z_tit = 'Intensity (arb. units)'
  
  IF NOT corr_FR THEN FRs = '0' ELSE FRs = STRTRIM(STRING(iFR[0]),2)
  IF nFR GT 1 THEN FOR i = 1, nFR - 1 DO FRs = FRs + '+' + STRTRIM(STRING(iFR[i]),2)
  
  datp.other_tit = datp.other_tit+' -cx('+FRs+')'
  
  IF iprint THEN PRINT,'End corr_xyz:'
  
  give_datp, datp
  
  RETURN, w_out0
  
  finished:
END
