; ----------------------------------------------------------------------------
;*****************************************************************************

FUNCTION norm_scan, w_in
;+
;******* *********
;** User callable
;**  Rebins data to uniform bins defined by data points (i.e. not constant)
;**  Designed to perform inter-normalisation of data in new detector bank
;** 
;**  For diffraction data:   Should be run after theta_scan
;**  For TOF data:           Should be run on one spin-phase/component
;** 
;**               JRS 29/11/06
;** 
;** -----------------------------------------------------------------------------
;** *****************************************************************************
;-

  iprint = 0      ;turns on debugging messages
  
  TAKE_DATP, datp
  
  x_in = datp.x
  e_in = datp.e
  TOF = FIX(datp.p[8])
  IF NOT(TOF) THEN BEGIN
    npts    = (SIZE(x_in))[1]
    IF (SIZE(w_in))[0] EQ 2 THEN nphases = (SIZE(w_in))[2] ELSE nphases = 1
    IF (SIZE(w_in))[0] EQ 3 THEN BEGIN
      s  = 'Norm_scan:  Run theta_scan on data first'
      ii = dialog_message(s, /ERROR)
      RETURN, w_in
    ENDIF
    IF iprint THEN PRINT,'Non-TOF: nspectra=', npts, ' nphases=', nphases
  ENDIF ELSE BEGIN
    y_in = datp.y
    npts    = (SIZE(y_in))[1]
    nspectra = (SIZE(x_in))[1]
    IF (SIZE(w_in))[0] GT 2 THEN BEGIN
      s = 'Norm_scan: Only one phase allowed for TOF data'
      ii = dialog_message(s,/ERROR)
      RETURN, w_in
    ENDIF
    IF iprint THEN PRINT,'TOF: nchannels=', npts, ' nspectra=', nspectra
  ENDELSE
  
  IF iprint THEN PRINT, 'norm_scan: End of check dimensions, etc. section'
  ; -----------------------------------------------------------------------------
  ;Perform binning
  
  IF NOT(TOF) THEN BEGIN
    w_out = FLTARR(1e4,nphases)
    e_out = FLTARR(1e4,nphases)
    x_out = FLTARR(1e4)
    
    last = 0
    FOR n = 0,1e4 DO BEGIN
      i = WHERE(ABS(x_in - x_in[last]) LT 0.5)
      last = MAX(i) + 1
      num  = N_ELEMENTS(i)
      w_out[n,*] = TOTAL(w_in[i,*],1) / num
      e_out[n,*] = SQRT(TOTAL(e_in[i,*]^2,1)) / num
      x_out[n]   = TOTAL(x_in[i]) / num
      IF last EQ npts THEN GOTO, finished
    ENDFOR
  ENDIF ELSE BEGIN
    w_out = FLTARR(nspectra,1e4)
    e_out = FLTARR(nspectra,1e4)
    y_out = FLTARR(1e4)
    
    last = 0
    FOR n = 0,1e4 DO BEGIN
      i = WHERE(ABS(y_in - y_in[last]) LT 0.5)
      last = MAX(i) + 1
      num  = N_ELEMENTS(i)
      IF num EQ 1 THEN w_out[*,n] = w_in[*,i] ELSE w_out[*,n] = TOTAL(w_in[*,i],2) / num
      IF num EQ 1 THEN e_out[*,n] = e_in[*,i] ELSE e_out[*,n] = SQRT(TOTAL(e_in[*,i]^2,2)) / num
      y_out[n]   = TOTAL(y_in[i]) / num
      IF last EQ npts THEN GOTO, finished
    ENDFOR
  ENDELSE
  finished:
  IF iprint THEN PRINT, 'norm_scan: End of binning section'
  
  ; -----------------------------------------------------------------------------
  ;Return params and data and exit
  
  IF NOT(TOF) THEN BEGIN
    MOD_DATP, datp, "x", x_out[0:n]
    MOD_DATP, datp, "e", e_out[0:n,*]
  ENDIF ELSE BEGIN
    MOD_DATP, datp, "y", y_out[0:n]
    MOD_DATP, datp, "e", e_out[*,0:n]
  ENDELSE
  
  GIVE_DATP, datp
  
  IF NOT(TOF) THEN RETURN, w_out[0:n,*] ELSE RETURN, w_out[*,0:n]
END
