; ------------------------------------------------------------------------------
;*******************************************************************************
;
FUNCTION qrebin, w_in, omegashift=omegashift, fold=fold, $
                 nQ=nQ, dE=dE, Qmax=Qmax, oms
;+
;******* ******
;** User callable
;** For D7 and TOF data
;** 
;** takes output workspace of omega_scan.pro containing omega-scan, converts to
;** Q-space and rebins to a constant Qx-Qy grid in a single quadrant, giving
;** sensible error bars! Deals with only one spin phase.
;** 
;** ARGUMENTS:
;**  omegashift  : offset in degrees of omega=0 position
;**  nQ    : number of Qx/Qy bins per quadrant
;**  dE    : energy transfer of data
;** 
;**  (oms is obsolete, kept for backwards compatability)
;** 
;** KEYWORDS:
;**  fold    : folds quadrants into +,+
;** 
;** DIMENSIONS:
;**  w_in(nspectra,nruns)  -> w_out(nQ,nQ) (folded), w_out(2*nQ,2*nQ) (unfolded)
;** 
;** COMMAND SYNTAX:
;**  w10=qrebin(w9,[nQ=<nQ>][,omegashift=<omegashift>][,/fold])
;** 
;**             KHA,JRS,PPD 30/05/08
;** 
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-
    
  COMMON c_lamp_access, inst
  
  TAKE_DATP, datp
  
  iprint = 0  ; if iprint > 0, show debugging messages
  
  fld = 1    ; don't fold data (default)
  IF N_ELEMENTS(omegashift) LE 0 THEN omegashift = 0.
  IF N_ELEMENTS(oms) GT 0        THEN omegashift = oms
  IF N_ELEMENTS(dE) LE 0         THEN dE         = 0.
  IF N_ELEMENTS(nQ) LE 0         THEN nQ         = 26
  IF KEYWORD_SET(fold)           THEN fld        = 0
  
  ; ------------------------------------------------------------------------------
  ;Set constants
  nspectra = (SIZE(w_in))[1]
  nruns    = (SIZE(w_in))[2]
  
  IF inst EQ 'IN4' OR inst EQ 'IN6' OR inst EQ 'IN5' THEN $
    lambda   = datp.p[21] ELSE lambda   = datp.p[4]
    
  const2 = 2.07194  ; hbar^2/2m
  
  ki = 2.*!pi/lambda
  kf = SQRT(ki^2 - (dE/const2))
  
  IF iprint THEN BEGIN
    PRINT, 'nspectra = ', nspectra
    PRINT, 'nruns    = ', nruns
    PRINT, 'lambda   = ', lambda
    PRINT, 'ki       = ', ki
    PRINT, 'kf   = ', kf
  ENDIF
  
  IF iprint THEN PRINT,'Qrebin: End of "set constants" section'
  
  ; ------------------------------------------------------------------------------
  ;Set up arrays
  
  ntheta = nspectra
  nomega = nruns
  
  e_in     = datp.e
  twotheta = datp.x*!pi/180.    ; detector angle (rad)
  omega    = datp.y - omegashift    ; sample rotation angle (degrees)
  
  Qmag = SQRT(ki^2 + kf^2 - 2*ki*kf*COS(twotheta))
  
  beta  = (twotheta/(ABS(twotheta)))*(180./!pi)*ACOS((ki^2 - kf^2 + Qmag^2)/(2*ki*Qmag))
  
  ; beta is angle between ki and Q (degrees)
  
  alpha = 180. + ((FLTARR(ntheta) + 1.) # TRANSPOSE(omega)) + (beta # TRANSPOSE(FLTARR(nomega) + 1.))
  
  Qx = (Qmag # TRANSPOSE(FLTARR(nomega) + 1.))* COS(alpha * !pi / 180.)
  Qy = (Qmag # TRANSPOSE(FLTARR(nomega) + 1.))* SIN(alpha * !pi / 180.)
  
  IF N_ELEMENTS(Qmax) LE 0 THEN Qmax = 1.1* MAX(Qmag)
  dQ   = Qmax / FLOAT(nQ)
  
  IF iprint THEN PRINT,'Qrebin: End of "Set up arrays" section'
  
  ; ------------------------------------------------------------------------------
  ;Rebin data
  
  w_out=FLTARR((fld+1)*nQ,(fld+1)*nQ)  & w_out[*,*]=0.
  e_out=FLTARR((fld+1)*nQ,(fld+1)*nQ)  & e_out[*,*]=0.
  n_out=FLTARR((fld+1)*nQ,(fld+1)*nQ)  & n_out[*,*]=0.
  
  FOR itheta=0,ntheta-1 DO BEGIN
    FOR iomega=0,nomega-1 DO BEGIN
      IF (fld EQ 1) THEN BEGIN
        IF (ABS(Qx[itheta,iomega]) GT 0.99*Qmax) OR (ABS(Qy[itheta,iomega]) GT 0.99*Qmax) THEN GOTO, next
        ix=FIX(((Qx[itheta,iomega]+dQ/2.)/dQ)+nQ)
        iy=FIX(((Qy[itheta,iomega]+dQ/2.)/dQ)+nQ)
        w_out[ix,iy]=w_out[ix,iy]+w_in[itheta,iomega]
        e_out[ix,iy]=e_out[ix,iy]+e_in[itheta,iomega]^2
        n_out[ix,iy]=n_out[ix,iy]+1.
      ENDIF ELSE BEGIN
        ix=FIX(ABS((Qx[itheta,iomega])+dQ/2.)/dQ)
        iy=FIX(ABS((Qy[itheta,iomega])+dQ/2.)/dQ)
        w_out[ix,iy]=w_out[ix,iy]+w_in[itheta,iomega]
        e_out[ix,iy]=e_out[ix,iy]+e_in[itheta,iomega]^2
        n_out[ix,iy]=n_out[ix,iy]+1.
      ENDELSE
      next:
    ENDFOR
  ENDFOR
  FOR ix=0,(fld+1)*nQ-1 DO BEGIN
    FOR iy=0,(fld+1)*nQ-1 DO BEGIN
      IF (n_out[ix,iy] GT 0.) THEN BEGIN
        w_out[ix,iy]=w_out[ix,iy]/n_out[ix,iy]
        e_out[ix,iy]=SQRT(e_out[ix,iy])/n_out[ix,iy]
      ENDIF ELSE BEGIN
        w_out[ix,iy]=0.
        e_out[ix,iy]=-1.
      ENDELSE
    ENDFOR
  ENDFOR
  
  x_out=FLOAT((INDGEN((fld+1)*nQ)-(fld*nQ))*dQ)  & y_out=x_out
  
  IF (iprint GT 0) THEN PRINT,'Qrebin: End of "rebin data" section'
  
  ; ------------------------------------------------------------------------------
  ;Return parameters and exit
  
  mod_datp, datp, "x", x_out
  mod_datp, datp, "y", y_out
  mod_datp, datp, "e", e_out
  
  datp.x_tit='Qx'
  datp.y_tit='Qy'
  
  d=STRING_ROUND(STRING(dQ))
  s=STRING_ROUND(STRING(omegashift))
  r=STRING_ROUND(STRING(dE))
  
  PRINT,'Qrebin: rebinned to Qx-Qy grid with dQ='+d+'A-1 and omega-shift = '+s+' degrees'
  datp.other_tit=datp.other_tit+' -qr(dQ='+d+',o='+s+',dE='+r
  IF(fld=0) THEN datp.other_tit=datp.other_tit+',/fold)' ELSE $
    datp.other_tit=datp.other_tit+')'
    
  finished:
  give_datp, datp
  
  RETURN, w_out
END
