; -----------------------------------------------------------------------------
;******************************************************************************
;
FUNCTION quartz, w_in, NOSAVE = nosave, FR_depol = FRR, $
                 temp = temp, sk, fr1, tmp
;+
;******* ******
;** User callable 
;** For D7 data only
;** 
;** takes 2-D or 3-D workspace containing a normalised non-tof z-PA
;** or xyz-PA background-subtracted quartz run (or runs). Puts the flipping ratios
;** into w_out and also into the file 'quartz_"numor".dat'.  Will also correct
;** for depolarisation due to the sample (assuming cylindrical geometry) if desired
;** 
;** 
;** ARGUMENTS:
;**  FR_depol  :if depolarisation is present in the sample -
;**      FR_depol is flipping ratio (IN M2)in the presence of
;**      depolarisation
;**  temp    :if depolarisation is present in the sample -
;**      temp is the temperature (used to name the output file
;**      "quartz_<numor>_<temp>.dat") to identify the depolarisation
;**      correction at that temperature
;**  (sk, fr1 and tmp are obsolete, kept for backwards compatability)
;** 
;** KEYWORDS:
;**  /nosave  :don't save flipping ratios to file
;** 
;** DIMENSIONS:
;**  w_in(nspectra, nphases) -> w_out(nspectra, nphases/2.)
;** 
;** COMMAND SYNTAX:
;**  w5=quartz(w4[,/nosave][,FR_depol=<FR_depol>,temp=<temp>]
;** 
;**  (optional keywords/arguments shown in square brackets)
;** 
;**               KHA/JRS, 4/10/06
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-

  COMMON c_lamp
  
  iprint = 0   ; if iprint > 0, show debugging messages
  
  IF iprint THEN PRINT,'Start quartz:'
  TAKE_DATP, datp
  
  IF N_ELEMENTS(sk)  GT 0  THEN skip = sk ELSE skip = 0
  IF N_ELEMENTS(fr1) GT 0  THEN FRR  = fr1
  IF N_ELEMENTS(tmp) GT 0  THEN temp = tmp
  IF N_ELEMENTS(FRR) EQ 0  THEN FRR  = 0
  IF KEYWORD_SET(nosave)   THEN skip = 1
  
  ; ------------------------------------------------------------------------------
  ;Check dimensions of input workspaces
  
  CHECK_D7, w_in, datp, 'quartz', stf, nspectra, nphases, nruns, TOF, nvers
  IF N_ELEMENTS(stf) THEN RETURN, w_in
  
  mon      = datp.n
  e_in     = datp.e
  nphases  = nphases / 2.
  IF nruns EQ 1 THEN x_in = datp.x ELSE x_in = datp.z
  
  IF iprint THEN PRINT,'End of "w_in dimensions check etc." section'
  
  ; ------------------------------------------------------------------------------
  ;Calculate flipping ratios
  
  fr    = FLTARR(nspectra)
  dfr   = FLTARR(nspectra)
  x_out = x_in
  y_out = INDGEN(nphases)
  w_out = FLTARR(nspectra,nphases,nruns)
  e_out = w_out
  IF nruns GT 1 THEN z_out = INDGEN(nruns)
  
  iphase = INDGEN(nphases)
  ip0    = 2*iphase
  ip1    = ip0 + 1
  w_out[*,iphase,*] = w_in[*,ip0,*]/w_in[*,ip1,*]
  e_out[*,iphase,*] = SQRT( (e_in[*,ip0,*]/w_in[*,ip1,*])^2 $
    + (e_in[*,ip1,*]*w_in[*,ip0,*]/w_in[*,ip1,*]^2)^2 )
    
  IF iprint THEN PRINT,'End of "calculate flip. etc." section'
  ; ------------------------------------------------------------------------------
  ;Blech-Averbach correction for angular dependence (drops out if FRR=1)
  
  IF FRR GT 0 THEN BEGIN
  
    IF nphases EQ 3 THEN BEGIN
      x = [datp.p[28], datp.p[29], datp.p[30]]
      FRRT1 = MOMENT(x)
      FRRT  = FRRT1[0]
    ENDIF ELSE FRRT  = datp.p[28]
    
    p1 = (FRR  - 1.)/(FRR + 1.)
    p0 = (FRRT - 1.)/(FRRT + 1.)
    
    a1 = 1.7133  & a2 = -0.0927
    b1 = -0.0368 & b2 = -0.3750
    
    muR = (-a1 + SQRT(a1^2 - 4.*a2*ALOG(p1/p0)))/(2*a2)
    muR = ABS(muR)
    
    IF iprint THEN PRINT, 'p1 = ', p1, 'p0 = ', p0, 'muR = ', muR
    
    x_buf  = (ABS(x_in)*!pi/180.)/2.
    ptheta =  EXP(-(a1 + b1*SIN(x_buf)^2)*muR - $
      (a2 + b2*SIN(x_buf)^2)*muR^2)
      
    w_buf = ptheta*(w_out - 1.)/(w_out + 1.)
    e_buf = ptheta*e_out*(2./((w_out + 1.)^2))
    
    w1 = 1. + w_buf  & e1 = e_buf
    w2 = 1. - w_buf  & e2 = e_buf
    w_out = w1/w2
    e_out = e_buf*(2./((1.-w_buf)^2))
    
    IF iprint THEN PRINT,'End of Blech-Averbach correction section.'
  ENDIF
  
  ; -------------------------------------------------------------------------------
  ;Print out results in Debug mode
  
  IF iprint THEN PRINT,'Corrected Flipping ratios, and errors are:'
  IF iprint AND nphases EQ 1 THEN BEGIN
    PRINT, 'Spect:  ZFr        dZfr'
    PRINT, FORMAT = '(I3,"   ",F6.2," +/- ",F6.2)', TRANSPOSE([[INDGEN(nspectra) + 1],[w_out],[e_out]])
  ENDIF ELSE IF iprint THEN BEGIN
    PRINT, 'Spect:  ZFr        dZfr     XFr        dXfr     YFr          dYfr'
    FOR ispec = 0, nspectra - 1 DO PRINT, $
      FORMAT = '(I3,3("   ",F6.2," +/- ",F6.2))', $
      ispec + 1, TRANSPOSE([[[w_out[ispec,*]]],[[e_out[ispec,*]]]])
      
  ENDIF
  ; ------------------------------------------------------------------------------
  ;Write flipping ratios to quartz file
  
  numor = LONARR(nruns)
  FOR irun = 0, nruns - 1 DO BEGIN
    IF nruns EQ 1 THEN numor[irun] = LONG(datp.p[0]) ELSE $
      numor[irun] = LONG(datp.pv[0,irun])
    mon = LONG(mon)
    
    title = 'Flipping Ratios from ' + STRTRIM(datp.w_tit,2)
    
    IF NOT skip THEN BEGIN
      ispec = INDGEN(nspectra) + 1
      IF nspectra EQ 32 THEN ispec = INDGEN(32)*2 + 2
      IF nspectra EQ 42 THEN ispec = [(INDGEN(32)*2 + 2),(INDGEN(10) + 65)]
      IF nspectra EQ 68 THEN ispec = [INDGEN(44),INDGEN(24)*2 + 46]
      IF NOT FRR THEN out_file = 'quartz_' + STRTRIM(STRING(numor[irun]),2) + '.dat' $
      ELSE            out_file = 'quartz_' + STRTRIM(STRING(numor[irun]),2) + '_' + STRTRIM(STRING(temp),2) + '.dat'
      OPENW, lun, out_file, /GET_LUN
      PRINTF, lun, STRTRIM(title,2)
      IF nphases EQ 1 THEN BEGIN
        PRINTF, lun, 'Spectrum    Angle      Flip     dFlip'
        PRINTF, lun, FORMAT='(I7,2F11.3,F7.3)', $
          TRANSPOSE([[ispec],[x_out[*,irun]],[w_out[*,0,irun]],[e_out[*,0,irun]]])
        PRINTF, lun, FORMAT='("   Time 0  ",I18)', mon[0,1,irun]
        PRINTF, lun, FORMAT='("   Time 1  ",I18)', mon[0,2,irun]
        PRINTF, lun, FORMAT='("Monitor 0  ",I18)', mon[1,1,irun]
        PRINTF, lun, FORMAT='("Monitor 1  ",I18)', mon[1,2,irun]
      ENDIF ELSE BEGIN
        PRINTF, lun, 'Spectrum    Angle      FlipZ  dFlipZ     FlipX  dFlipX     FlipY  dFlipY'
        PRINTF, lun, FORMAT='(I7,2F11.3,F7.3,F11.3,F7.3,F11.3,F7.3)', $
          TRANSPOSE([[ispec],[x_out[*,irun]],[w_out[*,0,irun]],[e_out[*,0,irun]], $
          [w_out[*,1,irun]],[e_out[*,1,irun]],[w_out[*,2,irun]],[e_out[*,2,irun]]])
        PRINTF, lun, FORMAT='("   Time 0  ",3I18)', mon[0,1,irun], mon[0,3,irun], mon[0,5,irun]
        PRINTF, lun, FORMAT='("   Time 1  ",3I18)', mon[0,2,irun], mon[0,4,irun], mon[0,6,irun]
        PRINTF, lun, FORMAT='("Monitor 0  ",3I18)', mon[1,1,irun], mon[1,3,irun], mon[1,5,irun]
        PRINTF, lun, FORMAT='("Monitor 1  ",3I18)', mon[1,2,irun], mon[1,4,irun], mon[1,6,irun]
      ENDELSE
      FREE_LUN, lun
      IF STRMID(lamp_host,0,2) EQ 'd7' THEN $
        SPAWN, 'cp ' + out_file + ' /home/tofhr/lambda/QUARTZFILES/'
      PRINT, 'Quartz:  Flipping Ratios written to ' + out_file
    ENDIF
  ENDFOR
  
  IF iprint THEN PRINT,'End of "Write to quartz_file" section'
  
  ; ------------------------------------------------------------------------------
  ;Return parameters and exit
  
  MOD_DATP, datp, "y", y_out
  MOD_DATP, datp, "e", e_out
  
  datp.w_tit     = title
  datp.other_tit = datp.other_tit + ' -qu'
  
  IF nphases EQ 1 THEN datp.y_tit = 'Flipping Ratio' ELSE BEGIN
    datp.y_tit = 'Phase'
    datp.z_tit = 'Flipping Ratio'
  ENDELSE
  
  IF iprint THEN PRINT,'End quartz:'
  
  GIVE_DATP, datp
  
  RETURN, w_out
END
