; ------------------------------------------------------------------------------
;*******************************************************************************
;
FUNCTION QxQy, w_in, omegashift=omegashift, dQ=dQ, dE=dE, Qmax=Qmax
;+
;******* ****
;** User callable
;** Takes output workspace of omega_scan.pro containing omega-scan, converts to
;** Q-space and interpolates to a constant Qx-Qy grid (rough and ready version of
;** qrebin.pro)  Deals with only one spin phase at a time
;** 
;** ARGUMENTS:
;**  omegashift  : angle (in degrees) of omega=0 position (default = 0)
;**  dQ    : Q interpolation bin
;**  dE    : energy transfer of data (default = 0)
;**  Qmax    : maximum Q in grid
;** 
;** DIMENSIONS
;**  w_in(nspectra,nruns) -> w_out(nQx,nQy)
;** 
;** COMMAND SYNTAX:
;**  w6=QxQy(w5[,omegashift=<omega>][,dQ=<q-binning>][,dE=<energy transfer>][Qmax=<max mod Q>])
;** 
;**  (optional arguments shown in square brackets)
;** 
;**             KHA,JRS 30/5/08
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-

  COMMON c_lamp_access, inst
  
  TAKE_DATP, datp
  
  iprint = 0  ; if iprint>0, show debugging messages
  
  IF N_ELEMENTS(omegashift) EQ 0 THEN omegashift = 0.
  IF N_ELEMENTS(dE) LE 0         THEN dE         = 0.
  ; ------------------------------------------------------------------------------
  ;Set constants
  
  nspectra = FIX(datp.p[1])
  nruns    = FIX(datp.p[3])
  lambda   = datp.p[4]
  
  IF inst NE 'D7' THEN BEGIN
    nspectra = (SIZE(w_in))[1]
    nruns    = (SIZE(w_in))[2]
    lambda   = datp.p[21]
  ENDIF
  
  const2 = 2.07194  ; hbar^2/2m
  
  ki = 2.*!pi/lambda
  kf = SQRT(ki^2 - (dE/const2))
  
  IF iprint THEN BEGIN
    PRINT, 'nspectra = ', nspectra
    PRINT, 'nruns    = ', nruns
    PRINT, 'lambda   = ', lambda
    PRINT, 'ki       = ', ki
    PRINT, 'kf   = ', kf
  ENDIF
  
  
  IF iprint THEN PRINT,'End of "set constants" section'
  
  ; ------------------------------------------------------------------------------
  ;Set up arrays
  
  ntheta = nspectra
  nomega = nruns
  
  e_in     = datp.e
  twotheta = datp.x*!pi/180.    ; detector angle (rad)
  omega    = datp.y - omegashift    ; sample rotation angle (degrees)
  
  Qmag = SQRT(ki^2 + kf^2 - 2*ki*kf*COS(twotheta))
  
  beta  = (180./!pi)*ACOS((ki^2 - kf^2 + Qmag^2)/(2*ki*Qmag))
  
  ; beta is angle between ki and Q (degrees)
  
  alpha = 180. + ((FLTARR(ntheta) + 1.) # TRANSPOSE(omega)) + (beta # TRANSPOSE(FLTARR(nomega) + 1.))
  
  Qx = (Qmag # TRANSPOSE(FLTARR(nomega) + 1.))* COS(alpha * !pi / 180.)
  Qy = (Qmag # TRANSPOSE(FLTARR(nomega) + 1.))* SIN(alpha * !pi / 180.)
  
  IF iprint THEN PRINT,'End of "Set up arrays" section'
  
  ; ------------------------------------------------------------------------------
  ; Interpolate to regular Qx-Qy grid
  
  TRIANGULATE, Qx, Qy, triangles, b
  
  IF N_ELEMENTS(dQ) EQ 0 THEN dQ = 0.04
  GS = [dQ,dQ]
  IF N_ELEMENTS(Qmax) LE 0 THEN Qmax = 1.1*MAX(Qmag)
  Limits = [-Qmax,-Qmax,Qmax,Qmax]
  
  w_out = TRIGRID(Qx,Qy,w_in,  triangles,GS,limits, MISSING = 0.0)
  e_out = TRIGRID(Qx,Qy,datp.e,triangles,GS,limits, MISSING = -1.0)
  
  x_out = dQ*(FINDGEN((SIZE(w_out))[1]) - ((SIZE(w_out))[1] - 1)/2.)
  y_out = dQ*(FINDGEN((SIZE(w_out))[2]) - ((SIZE(w_out))[2] - 1)/2.)
  
  nx = (SIZE(x_out))[1]
  ny = (SIZE(y_out))[1]
  
  IF iprint THEN PRINT,'End of "Interpolate to regular grid" section'
  
  ; ------------------------------------------------------------------------------
  ;Zero Points outside grid
  
  min_q = MIN(Qmag)
  max_q = MAX(Qmag)
  FOR i = 0, (SIZE(w_out))[1]-2 DO BEGIN
    FOR j = 0, (SIZE(w_out))[2]-2 DO BEGIN
      Qmag_out = SQRT(x_out[i]^2 + y_out[j]^2)
      IF Qmag_out LE min_q OR Qmag_out GE max_q THEN BEGIN
        w_out[i,j] = 0
        e_out[i,j] = -1
      ENDIF
      beta_out  = (180./!pi)*ACOS((ki^2 - kf^2 + Qmag_out^2)/(2*ki*Qmag_out))
      alpha_out = (180./!pi)*ACOS(x_out[i]/Qmag_out)
      IF y_out[j] LT 0 THEN alpha_out = 360. - alpha_out
      omega_out =  alpha_out - beta_out -180
      
      IF Qmag_out EQ 0 OR Qmag_out GT max_q THEN omega_out = -10000
      
      IF (omega_out LT MIN(omega) OR omega_out GT MAX(omega)) AND $
        omega_out-360 LT MIN(omega) THEN BEGIN
        w_out[i,j] = 0
        e_out[i,j] = -1
      ENDIF
      
    ENDFOR
  ENDFOR
  
  IF iprint THEN PRINT,'End of "Zero points" section'
  
  
  ; ------------------------------------------------------------------------------
  ;Return parameters and exit
  
  mod_datp, datp, "x", x_out
  mod_datp, datp, "y", y_out
  mod_datp, datp, "e", e_out
  
  datp.x_tit='Qx'
  datp.y_tit='Qy'
  
  s = STRING_ROUND(STRING(omegashift))
  t = STRING_ROUND(STRING(dE))
  PRINT,'QxQy: converted to Qx-Qy grid for dE='+t+'meV, with omega-shift='+s+'degrees'
  datp.other_tit=datp.other_tit+' -qx[o='+s+',dE='+t+']'
  
  give_datp, datp
  
  RETURN, w_out
END
