; ----------------------------------------------------------------------------
;*****************************************************************************

FUNCTION theta_scan, w_in, POSITIVE = positive, NEGATIVE = negative, $
                     ALL = all, CODER = coder, detector = idet
;+
;******* **********
;** User callable
;** For non-TOF D7 data only
;** 
;** Puts all detector angles onto a common x-axis.
;** Removes zeroed data points.
;** 
;** ARGUMENTS:
;**  detector  :optional argument specifying an individual detector to use
;** 
;** KEYWORDS:
;**  /negative  :only use data with negative angles
;**  /positive  :only use data with positive angles
;**  /all    :use all detector banks (default)
;**  /not_new      :ignore new analysers
;**  /coder  :only used if an individual detector is specified
;**      If set:
;**        use coder angle as x-axis,
;**      otherwise:
;**       use calculated detector angle as x-axis
;** 
;** 
;** DIMENSIONS:
;** input format: w(nspectra,nphases,nruns)  - unless nphases or nruns is 1
;** output:       w(nruns*nspectra,nphases)  - unless nphases is 1
;** 
;** COMMAND SYNTAX:
;**  w10=theta_scan(w9[,/all_angles][,/pos_angles][,/neg_angles][detector=<det>][,/coder])
;** 
;**               JRS 5/6/07
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-
    
  iprint = 0
  
  IF iprint THEN PRINT, 'Theta_scan:  Starting ..'
  
  TAKE_DATP, datp
  
  ; ----------------------------------------------------------------------------
  ; Data check section
  
  CHECK_D7, w_in, datp, 'Theta_scan', stf, nspectra, nphases, nruns, TOF, nvers
  
  IF N_ELEMENTS(stf) GT 0 THEN RETURN, 0
  IF TOF THEN BEGIN
    ii = dialog_message('Theta_scan:  TOF data not implemented', /ERROR)
    RETURN, 0
  ENDIF
  
  e_in = datp.e
  
  IF nruns    GT 1  THEN x_in0 = datp.z ELSE x_in0 = datp.x
  IF nphases  EQ 1  THEN BEGIN
    w_in0 = REFORM(w_in, nspectra, 1, nruns)
    e_in0 = REFORM(e_in, nspectra, 1, nruns)
  ENDIF ELSE BEGIN
    w_in0 = w_in
    e_in0 = e_in
  ENDELSE
  
  IF iprint THEN PRINT, 'Theta_scan:  Finished checking data...'
  
  ; ----------------------------------------------------------------------------
  ; Choose spectrum numbers based on keyword options
  
  CASE 1 OF
    KEYWORD_SET(positive) : BEGIN
      ispec = WHERE(x_in0 GE 0.0)
      s = 'positive angle detectors'
      ot = '/pos'
    END
    KEYWORD_SET(negative) : BEGIN
      ispec = WHERE(x_in0 LE 0.0)
      s  = 'negative angle detectors'
      ot = '/neg'
    END
    N_ELEMENTS(idet) NE 0 : BEGIN
      det      = idet - 1
      ispec    = INDGEN(nruns) * nspectra + det
      bank_num = 4 - FIX(det/44)
      x_in0    = datp.pv[15+bank_num,*]
      s        = 'detector number ' + STRTRIM(STRING(idet),2)
      b        = 'bank number ' + STRTRIM(STRING(bank_num),2)
      ot       = 'det=' + STRTRIM(STRING(idet),2)
      IF NOT KEYWORD_SET(coder) THEN $
        s = s + ', ' + b + ' with detector angle as x-axis' ELSE BEGIN  
        s = s + ', ' + b + ' with coder angle as x-axis'
      ot = ot + ',/coder'
    ENDELSE
  END
  KEYWORD_SET(coder)   : BEGIN
    s = STRARR(2)
    s[0] = 'Theta_scan: /CODER keyword only used'
    s[1] = '            for individual detector scans'
    ii = dialog_message(s, /ERROR)
    RETURN, w_in0
  END
ELSE         : BEGIN
  ispec = WHERE(x_in0 NE 0.0)
  s = 'all detectors'
  ot = '/all'
END

ENDCASE

IF iprint THEN PRINT, 'Theta_scan: finished choosing detectors'

; -----------------------------------------------------------------------------
; Rearrange spectra

IF NOT KEYWORD_SET(coder) THEN x_in0 = x_in0[ispec]

w_buf = FLTARR((SIZE(ispec))[1], nphases)
e_buf = FLTARR((SIZE(ispec))[1], nphases)

FOR iphase = 0, nphases - 1 DO BEGIN
  w_in1 = REFORM(w_in0[*,iphase,*],nspectra*nruns)
  e_in1 = REFORM(e_in0[*,iphase,*],nspectra*nruns)
  w_buf[*,iphase] = w_in1[ispec]
  e_buf[*,iphase] = e_in1[ispec]
ENDFOR

;Sort spectra
IF NOT KEYWORD_SET(coder) THEN x_in0  = ABS(x_in0)
sortx = SORT(x_in0)
x_out = x_in0[sortx]  & w_out = w_buf[sortx,*] & e_out = e_buf[sortx,*]

;Remove zeroed spectra
igood = WHERE(e_out[*,0] GT 0)
x_out = x_out[igood] & w_out = w_out[igood,*] & e_out = e_out[igood,*]

IF iprint THEN PRINT, 'Theta_scan: finished rearranging angles'
; -----------------------------------------------------------------------------
; Return data and parameters

MOD_DATP, datp, "x", x_out
MOD_DATP, datp, "e", e_out

datp.p[1]  = nspectra
datp.p[3]  = 1
datp.x_tit = 'Scattering Angle'
IF KEYWORD_SET(coder) THEN datp.x_tit = b + ' Angle'
datp.y_tit = datp.z_tit

PRINT,'Theta_scan: keep ' + s
datp.other_tit = datp.other_tit + ' -ts(' + ot + ')'
IF iprint THEN PRINT,'End theta_scan:'

GIVE_DATP, datp
RETURN, w_out

END
