; ------------------------------------------------------------------------------
;*******************************************************************************
;
FUNCTION vanadium, w_in, nosave=nosave, sk
;+
;******* ********
;** User callable
;** For D7 data only
;** 
;** Takes 2-D or 3-D workspace containing one or more normalised non-tof z-PA
;** or xyz-PA background-subtracted vanadium runs.  For quartz and multiple
;** scattering corrections, data should be run through corr_xyz first. Vanadium.pro
;** adds the flip and non-flip intensities and puts the sums into w_out and also
;** into the file 'vanadium_<numor>.dat'
;** 
;** KEYWORDS:
;**  /nosave  :Don't save in file format
;**  (sk is obsolete and should not be used)
;** 
;** DIMENSIONS:
;**  input format: w_in(nspectra,nphases,nruns)
;**  output format: w_out(nspectra,nruns) (saved in <nrun> individual files)
;** 
;** COMMAND SYNTAX:
;**  w5=vanadium(w4[,/nosave])
;**  (optional keyword shown in square brackets)
;** 
;**               KHA,JRS 4/10/06
;**     Modified 1.8.13 ARW to stop program crashing at end
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-

  COMMON c_lamp
  
  iprint = 0  ; if iprint>0, show debugging messages
  
  IF iprint THEN PRINT,'Start vanadium:'
  
  ON_IOERROR, finished
  
  take_datp, datp
  
  IF N_ELEMENTS(sk) GT 0    THEN skip = sk
  IF KEYWORD_SET(nosave)    THEN skip = 1
  IF N_ELEMENTS(skip) EQ 0  THEN skip = 0
  
  ; ------------------------------------------------------------------------------
  ;Check dimensions of input workspaces
  
  x_in = datp.x
  e_in = datp.e
  
  TOF      = FIX(datp.p[8])
  nspectra = FIX(datp.p[1])
  nphases  = FIX(datp.p[2])
  nruns    = FIX(datp.p[3])
  sw       = SIZE(w_in)
  
  IF TOF AND (sw[1] EQ 128) THEN BEGIN
    s = 'Vanadium: TOF data must be integrated before calling vanadium.pro'
    i = dialog_message(s, /ERROR)
    RETURN, w_in
  ENDIF
  
  IF (iprint GT 0) THEN PRINT,'End of "w_in dimensions check etc." section'
  
  ; ------------------------------------------------------------------------------
  ;Add up vanadium counts
  
  IF (nruns EQ 1) THEN x_out = x_in ELSE x_out = datp.z
  IF iprint THEN PRINT,'nphases=',nphases
  
  ispec = INDGEN(nspectra) + 1
  IF (nspectra EQ 32) THEN ispec = INDGEN(32)*2 + 2
  IF (nspectra EQ 64) THEN ispec = INDGEN(64) + 1
  IF (nspectra EQ 42) THEN ispec = [(INDGEN(32)*2 + 2),(INDGEN(10) + 65)]
  
  IF nphases EQ 1 THEN BEGIN
    w_out = w_in
    e_out = e_in
  ENDIF ELSE BEGIN
    rnorm = FLOAT(nphases/2)
    w_out = TOTAL(w_in,2)/rnorm
    e_out = SQRT(TOTAL(e_in^2,2))/rnorm
  ENDELSE
  
  IF iprint THEN PRINT,'End of "Sum V counts" section'
  
  ; ------------------------------------------------------------------------------
  ;Write vanadium intensities to 'vanadium_file'
  
  numor = STRTRIM(STRING(LONG(datp.p[0])),2)
  title = 'Vanadium Intensities from #'+numor
  
  IF NOT skip THEN BEGIN
    FOR irun = 0,nruns - 1 DO BEGIN
      IF (nruns NE 1) THEN BEGIN
        numor = STRTRIM(STRING(LONG(datp.pv[0,irun])),2)
        title = 'Vanadium Intensities from #'+numor
      ENDIF
      out_file = 'vanadium_'+numor+'.dat'
      IF iprint THEN PRINT,'out_file = ' + out_file
      OPENW,  lun, out_file, /GET_LUN
      PRINTF, lun, title
      PRINTF, lun, 'Spectrum     Angle     Vana       dVana'
      PRINTF, lun, FORMAT='(I7,F11.3,2F11.6)', $
        TRANSPOSE([[ispec],[x_out[*,irun]],[w_out[*,irun]],[e_out[*,irun]]])
      ;      CLOSE, lun, /ALL
      ; *** Following two lines added 1.8.13 ARW
      CLOSE, lun
      FREE_LUN,lun
      
      IF (lamp_host EQ 'd7') OR (lamp_host EQ 'd7sgi') OR (lamp_host EQ 'd7lin') THEN $
        spawn, 'cp '+out_file+' /home/tofhr/lambda/VANFILES/'
    ENDFOR
  ENDIF
  
  IF iprint THEN PRINT,'End of "Write to vanadium_file" section'
  
  ; ------------------------------------------------------------------------------
  ;Return parameters and exit
  
  datp.p[2] = 1.
  
  mod_datp, datp, "e", e_out
  
  datp.w_tit     = title
  datp.other_tit = datp.other_tit+' -va'
  datp.y_tit     = 'Intensity'
  
  IF iprint THEN PRINT,'End vanadium:'
  
  give_datp, datp
  
  RETURN, w_out
  
  finished:
END
