; -------------------------------------------------------------------------
FUNCTION shoelace, a
;+
;******* ********
;**  Shoelace formula for calculating area of arbitrary convex polygon
;**  a is an array of points for general shoelace formula.
;**  Polygon must be convex
;-

  debug = 1
  
  numpoints = (SIZE(a))[2]
  IF numpoints EQ 1 THEN RETURN, 0
  
  ;first order polygon points
  
  ;choose startpoint
  ii       = WHERE(a[1,*] EQ MIN(a[1,*]))
  unit_vec = FLTARR(numpoints) + 1.
  astart   = a[*,ii] # unit_vec
  a_rel    = a - astart
  
  ;find angles from startpoint and order
  angles = ATAN(a_rel[1,*]/a_rel[0,*])*180./!pi
  ii     = WHERE(angles LT 0, count)
  IF count GT 0 THEN angles[ii] = angles[ii] + 180.
  ii     = SORT(angles)
  a      = a[*,ii]
  
  IF debug THEN FOR ip = 0, numpoints-1 DO XYOUTS, a[0,ip], a[1,ip], STRTRIM(STRING(ip),2)
  
  ;shoelace formula
  sum = 0
  FOR ipoint = 0, numpoints - 2 DO BEGIN
    sum = sum + a[0,ipoint]*a[1,ipoint+1] - a[1,ipoint]*a[0,ipoint+1]
  ENDFOR
  
  sum = sum + a[0,numpoints - 1]*a[1,0] - a[1,numpoints - 1]*a[0,0]
  area = 0.5 * ABS(sum)
  
  RETURN, area
  
END

; -------------------------------------------------------------------------

FUNCTION triangle_area, a, b, c
;+
;******* *************
;**  shoelace formula for a triangle
;-

  area = 0.5 * ABS(a[0]*b[1] + b[0]*c[1] + c[0]*a[1] - a[1]*b[0] - b[1]*c[0] - c[1]*a[0])
  
  RETURN, area
END

; -------------------------------------------------------------------------

FUNCTION inside_triangle, p, a, b, c
;+
;******* ***************
;**  p is coordinates of test point
;**  a, b and c are triangle vertexes
;-

  ans = 0
  
  a_tri  = triangle_area(a,b,c)
  aa     = triangle_area(p,a,b)
  ab     = triangle_area(p,b,c)
  ac     = triangle_area(p,a,c)
  
  IF ABS(aa + ab + ac - a_tri) LT 1.e-5 THEN ans = 1
  
  RETURN, ans
END
; -------------------------------------------------------------------------

FUNCTION inside_quad, p, a, b, c, d
;+
;******* ***********
;**  p is coordinates of test point
;**  a, b ,c and d are ordered quadrilateral vertexes
;-

  ans=0
  
  a_quad = triangle_area(a,b,c) + triangle_area(c,d,a)
  aa     = triangle_area(p,a,b)
  ab     = triangle_area(p,b,c)
  ac     = triangle_area(p,c,d)
  ad     = triangle_area(p,d,a)
  
  IF ABS(aa + ab + ac + ad - a_quad) LT 1.e-5 THEN ans = 1
  
  RETURN, ans
END

; -------------------------------------------------------------------------

FUNCTION between, a, b, p
;+
;******* *******
;** checks that point p is between a and b
;-

  ba = [b[0] - a[0], b[1] - a[1]]
  pa = [p[0] - a[0], p[1] - a[1]]
  
  ba_dot_pa = ba[0]*pa[0] + ba[1]*pa[1]
  ba_dot_ba = ba[0]^2 + ba[1]^2
  IF ba_dot_pa LT 0 THEN RETURN, 0
  IF ba_dot_pa LT ba_dot_ba THEN RETURN, 1 ELSE RETURN, 0
  
END
; -------------------------------------------------------------------------

FUNCTION crossing_lines, a, b, c, d
;+
;******* **************
;**  finds crossing points of 2 lines a-b and c-d
;-

  aa = b[0] - a[0]
  bb = d[0] - c[0]
  cc = b[1] - a[1]
  dd = d[1] - c[1]
  
  det = aa*dd - bb*cc
  
  IF ABS(det) LT 1.e-10 THEN BEGIN
    RETURN, 0
  ENDIF ELSE BEGIN
    cross_x = (a[1] - c[1])*aa*bb - a[0]*bb*cc + c[0]*aa*dd
    cross_y = (a[1]*aa - a[0]*cc)*dd - (c[1]*bb - c[0]*dd)*cc
    ps = [cross_x,cross_y] / det
    IF between(b,a,ps) AND between(d,c,ps) THEN RETURN, ps ELSE RETURN, 0
  ENDELSE
  
END

; ------------------------------------------------------------------------

FUNCTION intersection_points, a1,b1,c1,d1,a2,b2,c2,d2
;+
;******* *******************
;**  finds intersection points between 2 quadrilaterals
;-

  debug = 1
  
  intpoints = [0,0]
  
  IF inside_quad(a1,a2,b2,c2,d2) THEN intpoints= [[intpoints],[a1]]
  IF inside_quad(b1,a2,b2,c2,d2) THEN intpoints= [[intpoints],[b1]]
  IF inside_quad(c1,a2,b2,c2,d2) THEN intpoints= [[intpoints],[c1]]
  IF inside_quad(d1,a2,b2,c2,d2) THEN intpoints= [[intpoints],[d1]]
  IF inside_quad(a2,a1,b1,c1,d1) THEN intpoints= [[intpoints],[a2]]
  IF inside_quad(b2,a1,b1,c1,d1) THEN intpoints= [[intpoints],[b2]]
  IF inside_quad(c2,a1,b1,c1,d1) THEN intpoints= [[intpoints],[c2]]
  IF inside_quad(d2,a1,b1,c1,d1) THEN intpoints= [[intpoints],[d2]]
  
  aa = crossing_lines(a1,b1,a2,b2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(a1,b1,b2,c2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(a1,b1,c2,d2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(a1,b1,d2,a2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(b1,c1,a2,b2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(b1,c1,b2,c2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(b1,c1,c2,d2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(b1,c1,d2,a2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(c1,d1,a2,b2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(c1,d1,b2,c2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(c1,d1,c2,d2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(c1,d1,d2,a2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(d1,a1,a2,b2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(d1,a1,b2,c2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(d1,a1,c2,d2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  aa = crossing_lines(d1,a1,d2,a2)
  IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
  
  IF (SIZE(intpoints))[0] EQ 1 THEN BEGIN
    intpoints = 0
    numpoints = 0
  ENDIF ELSE BEGIN
    numpoints = (SIZE(intpoints))[2]
    intpoints = intpoints[*,1:numpoints-1]
  ENDELSE
  
  IF debug THEN BEGIN
    PLOT,  [a1[0],b1[0],c1[0],d1[0],a1[0]], [a1[1],b1[1],c1[1],d1[1],a1[1]],XRANGE=[0,1],YRANGE=[0,1]
    OPLOT, [a2[0],b2[0],c2[0],d2[0],a2[0]], [a2[1],b2[1],c2[1],d2[1],a2[1]]
    IF numpoints GT 0 THEN BEGIN
      FOR ipoint = 0, numpoints - 2 DO BEGIN
        OPLOT, [intpoints[0,ipoint]],[intpoints[1,ipoint]],psym=4
      ENDFOR
    ENDIF
  ENDIF
  
  RETURN, intpoints
END

; -----------------------------------------------------------------------

FUNCTION order_vertices, a, b, c, d
;+
;******* **************
;** stuff for inside_quad
;-

  IF N_ELEMENTS(crossing_lines(a,b,c,d)) EQ 2 THEN BEGIN
    af = a & bf = c & cf = b & df = d
  ENDIF
  IF N_ELEMENTS(crossing_lines(a,c,b,d)) EQ 2 THEN BEGIN
    af = a & bf = b & cf = c & df = d
  ENDIF
  IF N_ELEMENTS(crossing_lines(a,d,b,c)) EQ 2 THEN BEGIN
    af = a & bf = b & cf = d & df = c
  ENDIF
  
  RETURN,[[af], [bf], [cf], [df]]
  
END

; -------------------------------------------------------------------------

FUNCTION quad_generator
;+
;******* **************
;**  generates convex quadrilateral vertexes in cylic order
;-

  seed = LONG(SYSTIME(/seconds))
  
  generate:
  a = RANDOMU(seed,2)
  b = RANDOMU(seed,2)
  c = RANDOMU(seed,2)
  d = RANDOMU(seed,2)
  
  ; check for convex quadrilateral
  aa = inside_triangle(a,b,c,d)
  bb = inside_triangle(b,a,c,d)
  cc = inside_triangle(c,b,a,d)
  dd = inside_triangle(d,b,c,a)
  
  IF aa + bb + cc + dd GT 0 THEN GOTO, generate
  
  ;order the vertexes
  ordered_quad = order_vertices(a,b,c,d)
  
  RETURN,ordered_quad
  
END

; -----------------------------------------------------------------------
FUNCTION test
;+
;******* ****
;** stuff for inside_quad
;-

  aa = quad_generator()
  wait, 1
  bb = quad_generator()
  
  cc = intersection_points(aa[*,0],aa[*,1],aa[*,2],aa[*,3],bb[*,0],bb[*,1],bb[*,2],bb[*,3])
  
  RETURN, cc
END

