; ------------------------------------------------------------------------------
;*******************************************************************************
;
FUNCTION qrebin_v2, w_in0,omegashift=omegashift,dQ=dQ, dE=dE, Qmax=Qmax,$
                    fold=fold, unfold=unfold
;+
;******* *********
;** User callable
;** For D7 and TOF data
;** 
;** takes output workspace of omega_scan.pro containing omega-scan, converts to
;** Q-space and rebins to a constant Qx-Qy grid in a single quadrant, giving
;** sensible error bars! Deals with only one spin phase.
;** 
;** ARGUMENTS:
;**  omegashift  : offset in degrees of omega=0 position
;**  Qmax    : maximum mod Q to be output
;**  dQ    : Qbin width
;**  dE    : energy transfer of data
;** 
;**  (oms is obsolete, kept for backwards compatability)
;** 
;** KEYWORDS:
;**  fold    : folds quadrants into +,+
;** 
;** DIMENSIONS:
;**  w_in(nspectra,nruns)  -> w_out(nQ,nQ) (folded), w_out(2*nQ,2*nQ) (unfolded)
;** 
;** COMMAND SYNTAX:
;**  w10=qrebin(w9,[nQ=<nQ>][,omegashift=<omegashift>][,/fold])
;** 
;**             KHA,JRS,PPD 27/10/09
;** 
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-
    
  COMMON c_lamp_access, inst
  
  time = SYSTIME(1)
  TAKE_DATP, datp
  
  iprint = 0  ; if iprint > 0, show debugging messages
  debug  = 0  ; if debug > 0,  show debugging plots
  
  IF N_ELEMENTS(omegashift) LE 0 THEN omegashift = 0.
  IF N_ELEMENTS(oms) GT 0        THEN omegashift = oms
  IF N_ELEMENTS(dE) LE 0         THEN dE         = 0.
  IF N_ELEMENTS(dQ) LE 0         THEN dQ         = 0.1
  
  ; ------------------------------------------------------------------------------
  ;Set constants
  nspectra = (SIZE(w_in0))[1]
  nruns    = (SIZE(w_in0))[2]
  
  IF inst EQ 'IN4' OR inst EQ 'IN6' OR inst EQ 'IN5' THEN $
    lambda   = datp.p[21] ELSE lambda   = datp.p[4]
    
  const2 = 2.07194  ; hbar^2/2m
  ki = 2.*!pi/lambda
  kf = SQRT(ki^2 - (dE/const2))
  
  IF iprint THEN BEGIN
    PRINT, 'nspectra = ', nspectra
    PRINT, 'nruns    = ', nruns
    PRINT, 'lambda   = ', lambda
    PRINT, 'ki       = ', ki
    PRINT, 'kf   = ', kf
  ENDIF
  
  IF iprint THEN PRINT,'Qrebin: End of "set constants" section'
  
  ; ------------------------------------------------------------------------------
  ;Set up arrays
  
  ntheta = nspectra
  nomega = nruns
  
  ; sort input workspace into ascending 2theta/omega
  
  w_in  = w_in0
  e_in  = datp.e
  x_in  = datp.x
  y_in  = datp.y
  ii    = SORT(x_in)
  jj    = SORT(y_in)
  x_in  = x_in[ii]
  y_in  = y_in[jj]
  w_in  = w_in[ii,*]
  w_in  = w_in[*,jj]
  e_in  = e_in[ii,*]
  e_in  = e_in[*,jj]
  
  ; calculate input Qx-Qy grid
  
  twotheta = x_in*!pi/180.    ; detector angle (rad)
  omega    = y_in - omegashift    ; sample rotation angle (degrees)
  Qmag     = SQRT(ki^2 + kf^2 - 2*ki*kf*COS(twotheta))
  beta     = (twotheta/(ABS(twotheta)))*(180./!pi)*ACOS((ki^2 - kf^2 + Qmag^2)/(2*ki*Qmag))
  ; beta is angle between ki and Q (degrees)
  alpha = 180. + ((FLTARR(ntheta) + 1.) # TRANSPOSE(omega)) + (beta # TRANSPOSE(FLTARR(nomega) + 1.))
  Qx = (Qmag # TRANSPOSE(FLTARR(nomega) + 1.))* COS(alpha * !pi / 180.)
  Qy = (Qmag # TRANSPOSE(FLTARR(nomega) + 1.))* SIN(alpha * !pi / 180.)
  
  ; calculate output Qx-Qy grid
  IF N_ELEMENTS(Qmax) LE 0 THEN Qmax = 1.1* MAX(Qmag)
  a_nQ = FIX(Qmax / dQ)
  nQ    = 2*a_nQ + 1
  x_out = (INDGEN(nQ) - a_nQ) * dQ
  y_out = x_out
  w_out = FLTARR(nQ,nQ)
  e_out = w_out
  n_out = w_out
  
  IF iprint THEN PRINT,'Qrebin_v2: End of "Set up arrays" section'
  
  ; ------------------------------------------------------------------------------
  ;Rebin data
  
  FOR itheta=0,ntheta-1 DO BEGIN
    FOR iomega=0,nomega-1 DO BEGIN
      IF debug THEN PLOT, Qx, Qy, PSYM=4
      
      modQ = SQRT(Qx[itheta,iomega]^2 + Qy[itheta,iomega]^2)
      IF (ABS(Qx[itheta,iomega]) GT 0.99*Qmax) OR $
        (ABS(Qy[itheta,iomega]) GT 0.99*Qmax) OR $
        (modQ GT 0.99*Qmax) THEN GOTO, next
        
      ;compute Voronoi cell for Qx,Qy point
        
      IF itheta NE 0 THEN BEGIN
        dQx_theta_minus = 0.5 * (Qx[itheta-1,iomega] - Qx[itheta,iomega])
        dQy_theta_minus = 0.5 * (Qy[itheta-1,iomega] - Qy[itheta,iomega])
      ENDIF
      IF itheta NE ntheta - 1 THEN BEGIN
        dQx_theta_plus  = 0.5 * (Qx[itheta,iomega]   - Qx[itheta+1,iomega])
        dQy_theta_plus  = 0.5 * (Qy[itheta,iomega]   - Qy[itheta+1,iomega])
      ENDIF
      IF iomega NE 0 THEN BEGIN
        dQx_omega_minus = 0.5 * (Qx[itheta,iomega-1] - Qx[itheta,iomega])
        dQy_omega_minus = 0.5 * (Qy[itheta,iomega-1] - Qy[itheta,iomega])
      ENDIF
      IF iomega NE nomega - 1 THEN BEGIN
        dQx_omega_plus  = 0.5 * (Qx[itheta,iomega]   - Qx[itheta,iomega+1])
        dQy_omega_plus  = 0.5 * (Qy[itheta,iomega]   - Qy[itheta,iomega+1])
      ENDIF
      IF itheta EQ 0 THEN BEGIN
        dQx_theta_minus = dQx_theta_plus
        dQy_theta_minus = dQy_theta_plus
      ENDIF
      IF itheta EQ ntheta-1 THEN BEGIN
        dQx_theta_plus = dQx_theta_minus
        dQy_theta_plus = dQy_theta_minus
      ENDIF
      IF iomega EQ 0 THEN BEGIN
        dQx_omega_minus = dQx_omega_plus
        dQy_omega_minus = dQy_omega_plus
      ENDIF
      IF iomega EQ nomega-1 THEN BEGIN
        dQx_omega_plus = dQx_omega_minus
        dQy_omega_plus = dQy_omega_minus
      ENDIF
      centre = [Qx[itheta,iomega],Qy[itheta,iomega]]
      
      ;a-b-c-d is the input quadrilateral
      a = centre - [ dQx_theta_plus  + dQx_omega_plus,   dQy_theta_plus  + dQy_omega_plus]
      b = centre - [ dQx_theta_plus  - dQx_omega_minus,  dQy_theta_plus  - dQy_omega_minus]
      c = centre - [-dQx_theta_minus - dQx_omega_minus, -dQy_theta_minus - dQy_omega_minus]
      d = centre - [-dQx_theta_minus + dQx_omega_plus,  -dQy_theta_minus + dQy_omega_plus]
      point_area = shoelace([[a],[b],[c],[d]], 0)
      
      iix = WHERE(x_out GT centre[0])
      iiy = WHERE(y_out GT centre[1])
      
      ; test output bins for intersection, and assign weighted intensities
      num = 0
      frac_attributed = 0
      offx = [0,0,1,1, 1, 0,-1,-1,-1,0,1,2,2,2, 2, 2, 1, 0,-1,-2,-2,-2,-2,-2,-1]
      offy = [0,1,1,0,-1,-1,-1, 0, 1,2,2,2,1,0,-1,-2,-2,-2,-2,-2,-1, 0, 1, 2, 2]
      find_area:
      xbin =   iix[0]-offx[num]
      ybin =  iiy[0]-offy[num]
      b_centre = [x_out[xbin],y_out[ybin]]
      
      ;e-f-g-h is the bin box
      e = b_centre - [ 0.5*dQ, 0.5*dQ]
      f = b_centre - [ 0.5*dQ,-0.5*dQ]
      g = b_centre - [-0.5*dQ,-0.5*dQ]
      h = b_centre - [-0.5*dQ, 0.5*dQ]
      
      intersect_poly = intersection_points(a,b,c,d,e,f,g,h)
      intersect_area = shoelace(intersect_poly, debug)
      
      frac = intersect_area / point_area
      frac_attributed = frac_attributed + frac
      
      IF intersect_area GT 0 THEN BEGIN
        w_out[xbin,ybin] = w_out[xbin,ybin] +  frac*w_in[itheta,iomega]
        e_out[xbin,ybin] = e_out[xbin,ybin] + (frac*e_in[itheta,iomega])^2
        n_out[xbin,ybin] = n_out[xbin,ybin] + frac
      ENDIF
      
      IF debug THEN BEGIN
        OPLOT, [a[0],b[0],c[0],d[0],a[0]], [a[1],b[1],c[1],d[1],a[1]]
        OPLOT, [e[0],f[0],g[0],h[0],e[0]], [e[1],f[1],g[1],h[1],e[1]]
        PRINT, 'Box centre          = ', b_centre
        int_dims = (SIZE(intersect_poly))[0]
        IF int_dims GT 1 THEN BEGIN
          int_num = (SIZE(intersect_poly))[2]
          PRINT, 'Intersection points = ', intersect_poly[*,0]
          FOR ipoint = 1, int_num-1 DO BEGIN
            PRINT, '                      ', intersect_poly[*,ipoint]
          ENDFOR
        ENDIF
        PRINT, 'Intersection area   = ', intersect_area
        PRINT, 'Fractional area     = ', frac
        PRINT, 'Fraction of bin so far...', frac_attributed
        READ,s
      ENDIF
      
      ; test for total bin allocation
      IF frac_attributed LT 0.95 THEN BEGIN
        num = num + 1
        IF (num EQ 25) THEN BEGIN
          PRINT,'WARNING...  full weight not acheived'
          PRINT,'            weight attributed = ', frac_attributed
          GOTO, next
        ENDIF
        GOTO, find_area
      ENDIF
      next:
    ENDFOR
  ENDFOR
  
  FOR ix=0, nQ-1 DO BEGIN
    FOR iy=0, nQ-1 DO BEGIN
      IF (n_out[ix,iy] GT 0.) THEN BEGIN
        w_out[ix,iy] = w_out[ix,iy]/n_out[ix,iy]
        e_out[ix,iy] = SQRT(e_out[ix,iy])/n_out[ix,iy]
      ENDIF ELSE BEGIN
        w_out[ix,iy] =  0.
        e_out[ix,iy] = -1.
      ENDELSE
    ENDFOR
  ENDFOR
  
  IF (iprint GT 0) THEN PRINT,'Qrebin_v2: End of "rebin data" section'
  
  ; -----------------------------------------------------------------------------
  ;fold data for output if required
  
  IF KEYWORD_SET(fold) OR KEYWORD_SET(unfold) THEN BEGIN
    n_buf = 0.0*w_out
    w_buf = w_out
    e_buf = e_out^2
    ii = WHERE(e_out GE 0., n, COMPLEMENT=jj, NCOMPLEMENT=nn)
    IF n  GT 0 THEN n_buf[ii] = 1.
    IF nn GT 0 THEN e_buf[jj] = 0.0
    w_buf1 = REVERSE(w_buf,1) & w_buf = w_buf + w_buf1
    e_buf1 = REVERSE(e_buf,1) & e_buf = e_buf + e_buf1
    n_buf1 = REVERSE(n_buf,1) & n_buf = n_buf + n_buf1
    w_buf1 = REVERSE(w_buf,2) & w_buf = w_buf + w_buf1
    e_buf1 = REVERSE(e_buf,2) & e_buf = e_buf + e_buf1
    n_buf1 = REVERSE(n_buf,2) & n_buf = n_buf + n_buf1
    IF KEYWORD_SET(fold) THEN BEGIN
      w_out = w_buf[(nQ-1)/2:nQ-1,(nQ-1)/2:nQ-1]
      e_out = SQRT(e_buf[(nQ-1)/2:nQ-1,(nQ-1)/2:nQ-1])
      n_out = n_buf[(nQ-1)/2:nQ-1,(nQ-1)/2:nQ-1]
      x_out = x_out[(nQ-1)/2:nQ-1]
      y_out = y_out[(nQ-1)/2:nQ-1]
    ENDIF ELSE BEGIN
      w_out = w_buf
      e_out = SQRT(e_buf)
      n_out = n_buf
    ENDELSE
    ii = WHERE(n_out GT 0., n, COMPLEMENT=jj, NCOMPLEMENT=nn)
    IF n  GT 0 THEN BEGIN
      w_out[ii] = w_out[ii]/n_out[ii]
      e_out[ii] = e_out[ii]/n_out[ii]
    ENDIF
    IF nn GT 0 THEN BEGIN
      w_out[jj] = 0
      e_out[jj] = -1
    ENDIF
  ENDIF
  
  IF (iprint GT 0) THEN PRINT,'Qrebin_v2: End of "folding" section'
  
  ; ------------------------------------------------------------------------------
  ;Return parameters and exit
  
  MOD_DATP, datp, "x", x_out
  MOD_DATP, datp, "y", y_out
  MOD_DATP, datp, "e", e_out
  
  datp.x_tit='Qx'
  datp.y_tit='Qy'
  
  d=STRING_ROUND(STRING(dQ))
  s=STRING_ROUND(STRING(omegashift))
  r=STRING_ROUND(STRING(dE))
  
  PRINT,'Qrebin_v2: rebinned to Qx-Qy grid with dQ='+d+'A-1 and omega-shift = '+s+' degrees'
  datp.other_tit=datp.other_tit+' -qr2(dQ='+d+',o='+s+',dE='+r+')'
  IF iprint THEN PRINT,'Qrebin_v2: rebinning time..', SYSTIME(1) - time,'secs'
  
  finished:
  GIVE_DATP, datp
  
  RETURN, w_out
END

; ------------------------------------------------------------------------------
