;***********************************************************************
; ----------------------------------------------------------------------
FUNCTION sqw_rebin_v2, w_in, dQ=dQ, emin=emin, swap_QE=swap_QE
;+
;******* ************
;** User callable
;** Rebins S(E,2theta) to S(E,modQ) (does same job as previous sqw_rebin)
;** Doesn't rely on detectors being contiguous (as does sqw_rebin).
;** 
;**  INPUT:   w_in(nEn,nspectra)
;**  OUTPUT:  w_out(nEn,nQ)     where nQ is specified by the user
;** 
;**  Arguments:
;** -----------
;**     dQ:    modQ bin width
;**     emin:    minimum energy transfer for output
;**     swap_QE:   make output, w_out(nQ,nEn)
;** 
;**                JRS 27/10/09
;** 
;** ***********************************************************************
;** -----------------------------------------------------------------------
;-


  ; Check input data

  COMMON c_lamp_access, inst
  
  tic    = SYSTIME(1)    ; time marker
  iprint = 0      ; print out debugging messages
  debug  = 0      ; show intersection plots
  
  TAKE_DATP, datp
  
  IF inst EQ 'D7' THEN lambda = datp.p[4] ELSE lambda = datp.p[21]
  
  Ei   = 81.8066/lambda^2
  phi  = datp.y*!pi/180.
  Eps  = datp.x
  e_in = datp.e
  
  IF N_ELEMENTS(emin) EQ 0 THEN emin = Eps[0]
  IF N_ELEMENTS(dQ)   EQ 0 THEN dQ = 0.1
  
  IF iprint THEN PRINT, 'Ei = ',Ei
  nphi = (SIZE(phi))[1]
  nEps = (SIZE(Eps))[1]
  
  IF iprint THEN PRINT, 'sqw_rebin_v2: Finished "check arrays" section'
  
  ; ----------------------------------------------------------------------
  ; Find gaps between detectors
  
  iphi = INDGEN(nphi-1) + 1
  diff = phi[iphi] - phi[iphi-1]
  mean_diff = MEAN(diff)
  i = WHERE(diff GT 2.*mean_diff)
  IF i[0] EQ -1 THEN BEGIN
    w_in1 = w_in
    e_in1 = e_in
    GOTO, nogaps
  ENDIF
  
  ngaps = (SIZE(i))[1]
  
  FOR igap = 0,ngaps - 1 DO BEGIN
    gapdiff = phi[i[igap]+1] - phi[i[igap]]
    ndets   = FIX(gapdiff/mean_diff)
    IF iprint THEN PRINT, 'gap between ',phi[i[igap]],' and',phi[i[igap]+1],", ", ndets,' detectors to fill in'
    filldiff = gapdiff/FLOAT(ndets+1)
    gapphi = (INDGEN(ndets)+1.)*filldiff + phi[i[igap]]
    phi = [phi,gapphi]
    extraw = FLTARR(nEps,ndets)
    extrae = FLTARR(nEps,ndets) - 1
    w_in1 = [[w_in],[extraw]]
    e_in1 = [[e_in],[extrae]]
  ENDFOR
  
  isort = SORT(phi)
  phi = phi[isort]
  w_in1 = w_in1[*,isort]
  e_in1 = e_in1[*,isort]
  nphi = (SIZE(phi))[1]
  
  IF iprint THEN PRINT, 'sqw_rebin_v2: Finished "check for gaps" section'
  
  nogaps:
  ; ----------------------------------------------------------------------
  ; Form E and Q 2d-arrays, and setup output S(Q,w) grid
  
  ii = WHERE(Eps GE emin)
  Eps = Eps[ii]
  nEps = N_ELEMENTS(Eps)
  w_in1 = w_in1[ii,*]
  e_in1 = e_in1[ii,*]
  
  Eco    = Eps # (FLTARR(1,nphi)+1.)
  COSphi = (FLTARR(nEps)+1.) # COS(phi)
  Qco    = SQRT((2.*Ei-Eco-2.*SQRT(Ei*(Ei-Eco))*COSphi)/2.07193571)
  
  Qmax  = 1.1*MAX(Qco)
  Qmin  = 0.9*MIN(Qco)
  numQ  = FIX((Qmax-Qmin) / dQ)
  numz  = FIX(Qmin / dQ)
  lowQ  = dQ * numz
  y_out = INDGEN(numQ)*dQ + lowQ
  nQ    = N_ELEMENTS(y_out)
  
  w_out = FLTARR(nEps,nQ)
  e_out = FLTARR(nEps,nQ)
  n_out = FLTARR(nEps,nQ)
  x_out = Eps
  
  IF iprint THEN PRINT, 'sqw_rebin_v2: Finished "forming Q and E arrays" section'
  
  ; ----------------------------------------------------------------------
  ;Rebinning section
  
  FOR ien = 0, nEps - 1 DO BEGIN
    FOR iphi = 0, nphi - 1 DO BEGIN
      IF debug THEN PLOT, Eco, Qco, PSYM=4
      
      ; -----------------------------------------------------------------------------
      ;compute Voronoi cell for dE,Q point
      
      IF ien NE 0 THEN BEGIN
        dE_minus    = 0.5 * (Eps[ien-1] - Eps[ien])
        dQ_en_minus = 0.5 * (Qco[ien-1,iphi] - Qco[ien,iphi])
      ENDIF
      IF ien NE nEps - 1 THEN BEGIN
        dE_plus     = 0.5 * (Eps[ien] - Eco[ien+1])
        dQ_en_plus  = 0.5 * (Qco[ien,iphi]   - Qco[ien+1,iphi])
      ENDIF
      IF iphi NE 0 THEN BEGIN
        dQ_phi_minus = 0.5 * (Qco[ien,iphi-1] - Qco[ien,iphi])
      ENDIF
      IF iphi NE nphi - 1 THEN BEGIN
        dQ_phi_plus  = 0.5 * (Qco[ien,iphi]   - Qco[ien,iphi+1])
      ENDIF
      IF ien EQ 0       THEN BEGIN
        dE_minus     = dE_plus
        dQ_en_minus  = dQ_en_plus
      ENDIF
      IF ien EQ nEps-1  THEN BEGIN
        dE_plus      = dE_minus
        dQ_en_plus   = dQ_en_minus
      ENDIF
      IF iphi EQ 0      THEN dQ_phi_minus = dQ_phi_plus
      IF iphi EQ nphi-1 THEN dQ_phi_plus  = dQ_phi_minus
      
      centre = [Eps[ien],Qco[ien,iphi]]
      
      ;a-b-c-d is the Voronoi cell
      a = centre - [ dE_plus,   dQ_en_plus  + dQ_phi_plus]
      b = centre - [ dE_plus,   dQ_en_plus  - dQ_phi_minus]
      c = centre - [-dE_minus, -dQ_en_minus - dQ_phi_minus]
      d = centre - [-dE_minus, -dQ_en_minus + dQ_phi_plus]
      point_area = shoelace([[a],[b],[c],[d]], 0)
      
      iix = WHERE(x_out EQ centre[0])
      iiy = WHERE(y_out GT centre[1])
      
      ; ---------------------------------------------------------------------------
      ; test output bins for intersection, and assign weighted intensities
      
      num = 0
      frac_attributed = 0
      off = [0,1,-1,2,-2,3,-3]
      find_area:
      xbin =   iix
      ybin =  iiy[0]-off[num]
      b_centre = [x_out[xbin],y_out[ybin]]
      
      ;e-f-g-h is the bin box
      e = b_centre - [ dE_plus,  0.5*dQ]
      f = b_centre - [ dE_plus, -0.5*dQ]
      g = b_centre - [-dE_minus,-0.5*dQ]
      h = b_centre - [-dE_minus, 0.5*dQ]
      
      intersect_poly = intersection_points(a,b,c,d,e,f,g,h)
      intersect_area = shoelace(intersect_poly, debug)
      
      frac = intersect_area / point_area
      frac_attributed = frac_attributed + frac
      
      IF intersect_area GT 0 AND e_in1[ien,iphi] NE -1 THEN BEGIN
        w_out[xbin,ybin] = w_out[xbin,ybin] +  frac*w_in1[ien,iphi]
        e_out[xbin,ybin] = e_out[xbin,ybin] + (frac*e_in1[ien,iphi])^2
        n_out[xbin,ybin] = n_out[xbin,ybin] + frac
      ENDIF
      
      IF debug THEN BEGIN
        OPLOT, [a[0],b[0],c[0],d[0],a[0]], [a[1],b[1],c[1],d[1],a[1]]
        OPLOT, [e[0],f[0],g[0],h[0],e[0]], [e[1],f[1],g[1],h[1],e[1]]
        PRINT, 'Box centre          = ', b_centre
        int_dims = (SIZE(intersect_poly))[0]
        IF int_dims GT 1 THEN BEGIN
          int_num = (SIZE(intersect_poly))[2]
          PRINT, 'Intersection points = ', intersect_poly[*,0]
          FOR ipoint = 1, int_num-1 DO BEGIN
            PRINT, '                      ', intersect_poly[*,ipoint]
          ENDFOR
        ENDIF
        PRINT, 'Intersection area   = ', intersect_area
        PRINT, 'Fractional area     = ', frac
        PRINT, 'Fraction of bin so far...', frac_attributed
        READ,s
      ENDIF
      
      ;test for total bin allocation
      IF frac_attributed LT 0.95 THEN BEGIN
        num = num + 1
        IF (num EQ 7) THEN BEGIN
          PRINT,'WARNING...  full weight not acheived'
          PRINT,'            weight attributed = ', frac_attributed
          GOTO, next
        ENDIF
        GOTO, find_area
      ENDIF
      next:
    ENDFOR
  ENDFOR
  
  FOR ien=0, nEps-1 DO BEGIN
    FOR iQ=0, nQ-1 DO BEGIN
      IF (n_out[ien,iQ] GT 0.) THEN BEGIN
        w_out[ien,iQ] = w_out[ien,iQ]/n_out[ien,iQ]
        e_out[ien,iQ] = SQRT(e_out[ien,iQ])/n_out[ien,iQ]
      ENDIF ELSE BEGIN
        w_out[ien,iQ] =  0.
        e_out[ien,iQ] = -1.
      ENDELSE
    ENDFOR
  ENDFOR
  
  IF (iprint GT 0) THEN PRINT,'sqw_rebin_v2: End of "rebin data" section'
  ; -----------------------------------------------------------------------
  ; Return workspaces
  
  datp.y_tit      = 'Wavevector Transfer (A^-1)'
  datp.x_tit      = 'Energy Transfer (meV)'
  
  IF NOT KEYWORD_SET(swap_QE) THEN BEGIN
    w_out = TRANSPOSE(w_out)
    e_out = TRANSPOSE(e_out)
    temp = x_out
    x_out = y_out
    datp.x_tit  = 'Wavevector Transfer (A^-1)'
    datp.y_tit  = 'Energy Transfer (meV)'
    y_out = temp
  ENDIF
  
  dQs = STRING_ROUND(STRTRIM(STRING(dQ),2))
  
  datp.other_tit = datp.other_tit + ' -sr2(dQ='+dQs+')'
  
  MOD_DATP, datp, "x", x_out
  MOD_DATP, datp, "y", y_out
  MOD_DATP, datp, "e", e_out
  GIVE_DATP, datp
  
  PRINT, 'sqw_rebin_v2: data rebinned to dQ='+dQs
  toc = SYSTIME(1)
  IF iprint THEN PRINT, 'Time elapsed = ', toc - tic, 'secs'
  
  RETURN, w_out
END

; -----------------------------------------------------------------------
