;+
; NAME:
;   RECT_FUNCT
;
; PURPOSE:
;   EVALUATE THE SUM OF A CONVOLUTION OF A GAUSSIAN AND A 
;	RECTANGLE AND A 2ND ORDER POLYNOMIAL
;   AND OPTIONALLY RETURN THE VALUE OF IT'S PARTIAL DERIVATIVES.
;   NORMALLY, THIS FUNCTION IS USED BY CURVEFIT TO FIT THE
;   SUM OF A LINE AND A VARYING BACKGROUND TO ACTUAL DATA.
;
; CATEGORY:
;   E2 - CURVE AND SURFACE FITTING.
; CALLING SEQUENCE:
;   FUNCT,X,A,F,PDER
; INPUTS:
;   X = VALUES OF INDEPENDENT VARIABLE.
;   A = PARAMETERS OF EQUATION DESCRIBED BELOW.
; OUTPUTS:
;   F = VALUE OF FUNCTION AT EACH X(I).
;
; OPTIONAL OUTPUT PARAMETERS:
;   PDER = (N_ELEMENTS(X),7) ARRAY CONTAINING THE
;       PARTIAL DERIVATIVES.  P(I,J) = DERIVATIVE
;       AT ITH POINT W/RESPECT TO JTH PARAMETER.
; COMMON BLOCKS:
;   NONE.
; SIDE EFFECTS:
;   NONE.
; RESTRICTIONS:
;   NONE.
; PROCEDURE:
;   F = A(0)*EZ + A(4) + A(5)*X + A(6)*X^2
;   EZ= (ERF(Dp)-ERF(Dm))/(2*A(3))
;	Dp =(AA+CC)*BB
;	Dm =(AA-CC)*BB
;	AA =X-A(1)
;	BB =2*SQRT(ALOG(2))/A(2)
;	CC =A(3)/2
;   Elements beyond A(3) are optional.
; MODIFICATION HISTORY:
;   WRITTEN, DMS, RSI, SEPT, 1982.
;   Modified, DMS, Oct 1990.  Avoids divide by 0 if A(2) is 0.
;   Added to Gauss_fit, when the variable function name to
;       Curve_fit was implemented.  DMS, Nov, 1990.
;   GAUSS_FUNCT modified to RECT_FUNCT, GAUSSFIT modified to
;		RECTFIT. TCH, ILL, JAN, 2003.
;-

PRO RECT_FUNCT,X,A,F,PDER
;    COMPILE_OPT idl2, hidden
    ON_ERROR,2                      ;Return to caller if an error occurs
    n = n_elements(a)
	X=double(x)
    ln2=double(alog(2.))
	bb = 2.*double(Sqrt(ln2))/a[2]
	aa = x-a[1]
	cc = a[3]/2.
	Dp=(aa+cc)*bb
	Dm=(aa-cc)*bb
	Ep = erf(Dp) ; temporary variables
	Em = erf(Dm)
	Ez = 0.5/a[3]*(Ep-Em)
    case n of
        4:  F = A[0]*EZ
        5:  F = A[0]*EZ + A[4]
        6:  F = A[0]*EZ + A[4] + A[5]*X
        7:  F = A[0]*EZ + A[4] + A[5]*X + A[6]*X^2 
    ENDCASE

    IF N_PARAMS(0) LE 3 THEN RETURN ;NEED PARTIAL?
    PDER = double(FLTARR(N_ELEMENTS(X),n)) ;YES, MAKE ARRAY.
	Ep = exp(-(Dp*Dp))
	Em = exp(-(Dm*Dm))
    PDER[*,0] = EZ      ;COMPUTE PARTIALS
    if a[3] ne 0.      then PDER[*,1] = A[0]/A[3]     /!PI*BB  *(   Em-   Ep)
    if a[3]*a[2] ne 0. then PDER[*,2] = A[0]/A[3]     /!PI/A[2]*(Dm*Em-Dp*Ep)
    if a[3]*a[2] ne 0. then PDER[*,3] = A[0]/A[3]*(ln2/!PI/A[2]*(   Em+   Ep)-Ez)/1.e0
    if n gt 4 then PDER[*,4] = 1.
    if n gt 5 then PDER[*,5] = X
    if n gt 6 then PDER[*,6] = X^2
    RETURN
END
