;+
; NAME:
;   RECTFIT
;
; PURPOSE:
;   Fit the equation y=f(x) where:
;
;       F(x) = A0*EXP(-z^2/2) + A3 + A4*x + A5*x^2
;           and
;       z=(x-A1)/A2
;
;   A0 = height of exp, A1 = center of exp, A2 = sigma (the width).
;   A3 = constant term, A4 = linear term, A5 = quadratic term.
;   Terms A3, A4, and A5 are optional.
;   The parameters A0, A1, A2, A3 are estimated and then CURVEFIT is
;   called.
;
; CATEGORY:
;   ?? - fitting
;
; CALLING SEQUENCE:
;   Result = GAUSSFIT(X, Y [, A])
;
; INPUTS:
;   X:  The independent variable.  X must be a vector.
;   Y:  The dependent variable.  Y must have the same number of points
;       as X.
;
; KEYWORD INPUTS:
;
;   CHISQ: Set this keyword to a named variable that will contain
;      the value of the chi-square goodness-of-fit.
;
;   ESTIMATES = optional starting estimates for the parameters of the
;       equation.  Should contain NTERMS (6 if NTERMS is not
;       provided) elements.
;
;   NTERMS = Set NTERMS to 3 to compute the fit: F(x) = A0*EXP(-z^2/2).
;      Set it to 4 to fit:  F(x) = A0*EXP(-z^2/2) + A3
;      Set it to 5 to fit:  F(x) = A0*EXP(-z^2/2) + A3 + A4*x
;
;   SIGMA: Set this keyword to a named variable that will contain
;      the 1-sigma error estimates of the returned parameters.
;
;   YERROR: The standard error between YFIT and Y.
;
; OUTPUTS:
;   The fitted function is returned.
;
; OPTIONAL OUTPUT PARAMETERS:
;   A:  The coefficients of the fit.  A is a three to six
;       element vector as described under PURPOSE.
;
; COMMON BLOCKS:
;   None.
;
; SIDE EFFECTS:
;   None.
;
; RESTRICTIONS:
;   The peak or minimum of the Gaussian must be the largest
;   or smallest point in the Y vector.
;
; PROCEDURE:
;   The initial estimates are either calculated by the below procedure
;   or passed in by the caller.  Then the function CURVEFIT is called
;   to find the least-square fit of the gaussian to the data.
;
;  Initial estimate calculation:
;   If NTERMS>=4 then a constant term is subtracted first.
;   If NTERMS>=5 then a linear term is subtracted first.
;   If the (MAX-AVG) of Y is larger than (AVG-MIN) then it is assumed
;   that the line is an emission line, otherwise it is assumed there
;   is an absorbtion line.  The estimated center is the MAX or MIN
;   element.  The height is (MAX-AVG) or (AVG-MIN) respectively.
;   The width is found by searching out from the extrema until
;   a point is found less than the 1/e value.
;
; MODIFICATION HISTORY:
;   DMS, RSI, Dec, 1983.
;   DMS, RSI, Jun, 1995, Added NTERMS keyword.  Result is now float if
;               Y is not double.
;   DMS, RSI, Added ESTIMATES keyword.
;   CT, RSI, Feb 2001: Change the way estimates are computed.
;         If NTERMS>3 then a polynomial of degree NTERMS-4 is subtracted
;         before estimating Gaussian coefficients.
;   CT, RSI, Nov 2001: Slight change to above modification:
;         Because a Gaussian and a quadratic can be highly correlated,
;         do not subtract off the quadratic term,
;         only the constant and linear terms.
;         Also added CHISQ, SIGMA and YERROR output keywords.
;-

Function Rectfit, x, y, a, $
	  WEIGHTS=weights,$
    CHISQ=chisq, $
    ESTIMATES = est, $
    NTERMS=nt, $
    SIGMA=sigma, $
    YERROR=yerror,$
	  ITMAX=itmax

;    COMPILE_OPT idl2

    on_error,2                      ;Return to caller if an error occurs

    if n_elements(nt) eq 0 then nt = 7
    if nt lt 4 or nt gt 7 then $
       message,'NTERMS must have values from 4 to 7.'
    n = n_elements(y)       ;# of points.
    dbl = SIZE(y, /TNAME) eq 'DOUBLE'

    if n_elements(est) eq 0 then begin  ;Compute estimates?
print,'no estimates given!'
        if (nt gt 4) then begin
            ; For a Gaussian + polynomial, we need to subtract off either
            ; a constant or a straight line to get good estimates.
            ; NOTE: Because a Gaussian and a quadratic can be highly correlated,
            ; we do not want to subtract off the quadratic term.
            c = POLY_FITd20[x, y, (nt eq 5) ? 0 : 1, yf]
            yd = y - yf
        endif else begin
            ; Just fitting a Gaussian. Don't need to subtract off anything.
            yd = y
            c = 0d
        endelse

        if not dbl then begin    ;If Y is not double, use float
            yd = float(yd)
            c = float(c)
        endif

        ;x,y and subscript of extrema
        ymax=max(yd, imax)
        xmax=x[imax]
        ymin=min(yd, imin)
        xmin=x[imin]
        if abs(ymax) gt abs(ymin) then i0=imax else i0=imin ;emiss or absorp?
        i0 = i0 > 1 < (n-2)     ;never take edges
        dy=yd[i0]           ;diff between extreme and mean
        del = dy/exp(1.)        ;1/e value
        i=0
        while ((i0+i+1) lt n) and $ ;guess at 1/2 width.
        ((i0-i) gt 0) and $
        (abs(yd[i0+i]) gt abs(del)) and $
        (abs(yd[i0-i]) gt abs(del)) do i=i+1
        a = [yd[i0], x[i0], abs(x[i0]-x[i0+i]), abs(x[i0]-x[i0+i])]
        if nt gt 4 then a = [a, c[0]] ; estimate for constant term
        if nt gt 5 then a = [a, c[1]] ; estimate for linear term
        if nt gt 6 then a = [a, 0.]   ; assume zero for quadratic estimate
    endif else begin
        if nt ne n_elements(est) then message, 'ESTIMATES must have NTERM elements.'
        a = est
    endelse
	iterations=1
	if not keyword_set(itmax) then itmax=20
	if not keyword_set(noder) then noder=0
	if not keyword_set(weights) then weights=replicate(1.,n)
    yfit = CURVEFIT(x,y,weights,a,sigma, $
        CHISQ=chisq, YERROR=yerror,itmax=itmax,iter=iterations,tol=1e-9, $
        FUNCTION_NAME = "rect_FUNCT",/double,noder=noder) ;call curvefit
	;help,iterations
    return, yfit
end
