;+
; NAME:
;   STRAIN
;
; PURPOSE:
;   Gaussian or Gaussian-Rectangle convolution (Hansen) fit 
;   with linear background
;   on only ONE single Bragg peak to determine d.
; 2D workspaces supported (scans: sequential fitting) 
;   p(33) contains wavelength
;   error estimation based on Withers, Daymond et al. 
;   with detector instability consideration (Hansen)
;
; CATEGORY:
;   E2 - CURVE AND SURFACE FITTING.
;
; CALLING SEQUENCE:
;   Wm=STRAIN(Wn)
;
; INPUTS:
;   Wn = Workspace n (1<=n<=20) in LAMP
;  
; OPTIONAL INPUTS:
; none.
; 
; KEYWORD PARAMETERS: 
; PLot :  plots data and fit in new window.
; PRint:  prints starting guess, fitted parameters and sigma in terminal.
; Rectangle:  Rectangular convolution of Gaussian shape (default)
; Gauss:    Gaussian peak shape
; Detector_instability: 5e-3 by default, used for error estimation
;
; OUTPUTS:
;   Wm =  Workspace m (1<=m<=20) in LAMP containing d(s) in AA from fit
;     and sigma(s) in Em
;
; OPTIONAL OUTPUT PARAMETERS:
;   NONE.
;
; COMMON BLOCKS:
;   NONE.
;
; SIDE EFFECTS:
;   NONE.
;
; RESTRICTIONS:
;   NONE.
;
; PROCEDURE:
; use of GAUSSFIT (IDL/RSI) or RECTFIT (TCH/ILL)
; EXAMPLE:
;
;  W6 = RDRUN(9135)
;  W7 = W6 & BACON, W7, /NOP, WR = [17, 24],PAR = [17, 18]
; 
; MODIFICATION HISTORY:
;   Written by Thomas C Hansen (TCH), Institut Laue-Langevin (ILL), Grenoble, France
;   January 2003
;   Gaussian function now taking intensity and FWHM
;
;********
;**
;** The call is w6=strainfit(...)
;-

FUNCTION strain,	w,$
					gauss=gauss,rectangle=rect,$
					detector_instability=instability,guess=guess,$
					print=print,plot=plot,noder=noder,itmax=itmax,trials=trials
;+
;******* ******
;** User callable
;-

COMMON rect_share,H_Rect,sqrtln2_HG

take_datp,d
;if keyword_set(rect) then gauss=0 else gauss=1
if keyword_set(gauss) and keyword_set(rect) then begin
	print,'Only one keyword /gauss or /rect can be given!'
	return,0
endif
if NOT keyword_set(gauss) then rect=1 else rect=0
if NOT keyword_set(trials) then trials=1
if not keyword_set(instability) then instability=5e-3
if not keyword_set(itmax) then itmax=20
if not keyword_set(guess) then a=fltarr(6) else a=[guess,fltarr(6)]
a=a[0:5]
mini=1.
minx=1.
chisq=1.
if not keyword_set(noder) then noder=0
fit  =fltarr(n_elements(w[0,*]))
err  =fltarr(n_elements(w[0,*]))
ampli=fltarr(n_elements(w[0,*]))
int  =fltarr(n_elements(w[0,*]))
width=fltarr(n_elements(w[0,*]))
fwhm= fltarr(n_elements(w[0,*]))
bg   =fltarr(n_elements(w[0,*]))
HR   =fltarr(n_elements(w[0,*]))
if keyword_set(plot) then window,0
for i=0,n_elements(w[0,*])-1 do begin
	b=i<(n_elements(d.x[0,*])-1)
    maxi=max(w[*,i],c,min=mini)
	sig=a
	index=where((w[*,i] ge ((maxi-mini)/2+mini)),cnt)
	if a[1] eq 0 THEN a[1]=d.x[c,b]
	fwhm[i]=max(d.x[index,b],min=minx)-minx
	if a[2] eq 0 THEN begin
		if a[3] eq 0 then a[2]=fwhm[i] else a[2]=fwhm[i]/100.
		if trials lt 2 then trials = 2
	endif
    if a[0] eq 0 THEN a[0]=(maxi-mini)/2./Sqrt(alog(2.)/!pi)*a[2]
	if keyword_set(plot) then begin
		plot,d.x[*,b],w[*,i]
	endif
	if keyword_set(gauss) then begin
		if a[3] eq 0 THEN a[3]=mini
		if a[4] eq 0 THEN a[4]=0.0
		if keyword_set(print) then print,[d.y[i],a]
		a=a[0:4]
		f=curvefit(d.x[*,b],w[*,i],1./d.e[*,i]/d.e[*,i],a,sig,func='gaussian_funct',chisq=chisq)
		ampli[i]=a[0]*2.*Sqrt(alog(2.)/!pi)/fwhm[i]
		int[i]=a[0]*10. ; D20 specific, needed for Withers error estimation
		bg[i]   =a[3]+a[1]*a[4]
		fwhm[i]=a[2]
		width[i]=fwhm[i]/2./SQRT(2.*ALOG(2))
	endif else begin
		if a[3] eq 0 THEN begin
			a[3]=fwhm[i]/100.
			if trials lt 2 then trials = 2
		endif
		if a[4] eq 0 THEN a[4]=mini
		if a[5] eq 0 THEN a[5]=0.
		a=double(a)
		if keyword_set(plot) then begin
			rect_funct,d.x[*,b],a,f
			oplot,d.x[*,b],f
		endif
		trial=fltarr(trials,3)
		for j=0,trials-1 do begin
			if trials gt 2 then a[3]=a[3]+a[2]*j/trials*2 else begin
				if j eq 1 then begin
					tmp=a[2]
					a[2]=a[3]
					a[3]=tmp
				endif
			endelse
			if keyword_set(print) then print,[d.y[i],a]
			f=rectfit(d.x[*,b],w[*,i],a,nterms=6,est=a,sigma=sig,itmax=itmax,chisq=chisq,w=1./d.e[*,i]/d.e[*,i])
			if trials gt 1 then if keyword_set(print) then print,[d.y[i],a,chisq]
        	trial[j,*]=[a[2:3],chisq]
		endfor
		if trials gt 1 then begin ; retain the best fit
			maxt=min(trial[*,2],maxt_index)
			a[2:3]=trial[maxt_index,0:1]
			f=rectfit(d.x[*,b],w[*,i],a,nterms=6,est=a,sigma=sig,itmax=itmax,chisq=chisq,w=1./d.e[*,i]/d.e[*,i])
			if keyword_set(print) then print,[d.y[i],a,chisq]
		endif
		int[i]=a[0]*10. ; D20 specific for Withers' error estimation !!!!!!!
		HR[i]=a[3]
		bg[i]   =a[4]+a[1]*a[5]
		ampli[i]=a[0]/a[3]*erf(a[3]/a[2]*sqrt(alog(2)))
   		maxi=max(f,c,min=mini)
		index=where((f ge ((maxi-mini)/2+mini)),cnt)
		width[i]=max(d.x[index,b],min=minx)-minx
		H_Rect =a[3]
		sqrtln2_HG=sqrt(alog(2))/a[2]
		width[i]=fx_root([width[i],min(a[2:3]),max(a[2:3])],'rect_fwhm')
	endelse
	if keyword_set(plot) then begin
		oplot,d.x[*,b],f
	endif
	if keyword_set(print) then BEGIN
		print,[d.y[i],a,chisq]
		print,[d.y[i],sig]
		print
	endif
	;width(i)=a(2)
	fit[i]  =a[1]
	err[i]  =sig[1]
endfor
mod_datp,d,'x',d.y
;mod_datp,d,'e',err/360.0*!PI/tan(fit/360.0*!PI)
if keyword_set(gauss) then begin
;	fwhm=2.*SQRT(2.*ALOG(2))*width
;	int=ampli/2./Sqrt(alog(2.)/!pi)*fwhm*10.
endif else begin
	eta=HR/(width+HR)
	fwhm=width/(1.82-1.9857*eta)
	;ampli=int/10.*2.*Sqrt(alog(2.)/!pi)/fwhm
	fwhm=width
	if keyword_set(print) then print,bg
	if keyword_set(print) then print,ampli
	if keyword_set(print) then print,fwhm
endelse
mod_datp,d,'e',(fwhm/2./sqrt(alog(2.))/360.*!pi*(1./sqrt(int)+instability)/tan(fit/360.*!pi)*sqrt(1.+2.*sqrt(2.)*bg/ampli))
;mod_datp,d,'e',err
d.x_tit=d.y_tit
d.y_tit='d/AA'
give_datp,d
guess=a
return,d.p[33]/2.0/sin(fit/360.0*!PI)
end
