;+
;
; PURPOSE:
;   CW_BGROUP is a compound widget that simplifies creating
;   a base of buttons. It handles the details of creating the
;   proper base (standard, exclusive, or non-exclusive) and filling
;   in the desired buttons. Events for the individual buttons are
;   handled transparently, and a CW_BGROUP event returned. This
;   event can return any one of the following:
;       - The Index of the button within the base.
;       - The widget ID of the button.
;       - The name of the button.
;       - An arbitrary value taken from an array of User values.
;
;   To get or set the value of a CW_BGROUP, use the GET_VALUE and
;   SET_VALUE keywords to WIDGET_CONTROL. The value of a CW_BGROUP
;   is:
;
;       -----------------------------------------------
;       Type        Value
;       -----------------------------------------------
;       normal      None
;       exclusive       Index of currently set button
;       non-exclusive   Vector indicating the position
;               of each button (1-set, 0-unset)
;       -----------------------------------------------
;
;
; INPUTS:
;       Parent:     The ID of the parent widget.
;   Names:      A string array, containing one string per button,
;           giving the name of each button.
;
; KEYWORD PARAMETERS:
;
;   BUTTON_UVALUE:  An array of user values to be associated with
;           each button and returned in the event structure.
;   COLUMN:     Buttons will be arranged in the number of columns
;           specified by this keyword.
;   EVENT_FUNCT:    The name of an optional user-supplied event function
;           for buttons. This function is called with the return
;           value structure whenever a button is pressed, and
;           follows the conventions for user-written event
;           functions.
;   EXCLUSIVE:  Buttons will be placed in an exclusive base, with
;           only one button allowed to be selected at a time.
;   FONT:       The name of the font to be used for the button
;           titles. If this keyword is not specified, the default
;           font is used.
;   FRAME:      Specifies the width of the frame to be drawn around
;           the base.
;   IDS:        A named variable into which the button IDs will be
;           stored, as a longword vector.
;   LABEL_LEFT: Creates a text label to the left of the buttons.
;   LABEL_TOP:  Creates a text label above the buttons.
;   MAP:        If set, the base will be mapped when the widget
;           is realized (the default).
;   NONEXCLUSIVE:   Buttons will be placed in an non-exclusive base.
;           The buttons will be independent.
;   NO_RELEASE: If set, button release events will not be returned.
;   RETURN_ID:  If set, the VALUE field of returned events will be
;           the widget ID of the button.
;   RETURN_INDEX:   If set, the VALUE field of returned events will be
;           the zero-based index of the button within the base.
;           THIS IS THE DEFAULT.
;   RETURN_NAME:    If set, the VALUE field of returned events will be
;           the name of the button within the base.
;   ROW:        Buttons will be arranged in the number of rows
;           specified by this keyword.
;   SCROLL:     If set, the base will include scroll bars to allow
;           viewing a large base through a smaller viewport.
;   SET_VALUE:  The initial value of the buttons. This is equivalent
;           to the later statement:
;
;           WIDGET_CONTROL, widget, set_value=value
;
;   SPACE:      The space, in pixels, to be left around the edges
;           of a row or column major base. This keyword is
;           ignored if EXCLUSIVE or NONEXCLUSIVE are specified.
;   UVALUE:     The user value to be associated with the widget.
;   UNAME:      The user name to be associated with the widget.
;   XOFFSET:    The X offset of the widget relative to its parent.
;   XPAD:       The horizontal space, in pixels, between children
;           of a row or column major base. Ignored if EXCLUSIVE
;           or NONEXCLUSIVE are specified.
;   XSIZE:      The width of the base.
;   X_SCROLL_SIZE:  The width of the viewport if SCROLL is specified.
;   YOFFSET:    The Y offset of the widget relative to its parent.
;   YPAD:       The vertical space, in pixels, between children of
;           a row or column major base. Ignored if EXCLUSIVE
;           or NONEXCLUSIVE are specified.
;   YSIZE:      The height of the base.
;   Y_SCROLL_SIZE:  The height of the viewport if SCROLL is specified.
;
; OUTPUTS:
;       The ID of the created widget is returned.
;
; SIDE EFFECTS:
;   This widget generates event structures with the following definition:
;
;       event = { ID:0L, TOP:0L, HANDLER:0L, SELECT:0, VALUE:0 }
;
;   The SELECT field is passed through from the button event. VALUE is
;   either the INDEX, ID, NAME, or BUTTON_UVALUE of the button,
;   depending on how the widget was created.
;
; RESTRICTIONS:
;   Only buttons with textual names are handled by this widget.
;   Bitmaps are not understood.
;
; MODIFICATION HISTORY:
;   15 June 1992, AB
;   7 April 1993, AB, Removed state caching.
;   6 Oct. 1994, KDB, Font keyword is not applied to the label.
;       10 FEB 1995, DJC  fixed bad bug in event procedure, getting
;                         id of stash widget.
;   11 April 1995, AB Removed Motif special cases.
;-


pro button_group_setv, id, value

  compile_opt hidden
  
  on_error, 2
  
  stash = widget_info(id, /CHILD)
  widget_control, stash, GET_UVALUE=state, /NO_COPY
  
  case state.type of
    0: message,'unable to set plain button group value'
    1: begin
      widget_control, SET_BUTTON=0, state.ids[state.excl_pos]
      state.excl_pos = value
      widget_control, /SET_BUTTON, state.ids[value]
    end
    2: begin
      n = n_elements(value)-1
      for i = 0, n do begin
        state.nonexcl_curpos[i] = value[i]
        widget_control, state.ids[i], SET_BUTTON=value[i]
      endfor
    end
  endcase
  
  widget_control, stash, SET_UVALUE=state, /NO_COPY
  
end

function button_group_getv, id, value

  compile_opt hidden
  on_error, 2
  
  stash = widget_info(id, /CHILD)
  widget_control, stash, GET_UVALUE=state, /NO_COPY
  
  case state.type of
    0: message,'unable to get plain button group value'
    1: ret = state.excl_pos
    2: ret = state.nonexcl_curpos
  endcase
  
  widget_control, stash, SET_UVALUE=state, /NO_COPY
  
  return, ret
  
end

function button_group_event, ev

  compile_opt hidden
  
  widget_control, ev.handler, GET_UVALUE=stash
  widget_control, stash, GET_UVALUE=state, /NO_COPY
  widget_control, ev.id, get_uvalue=uvalue
  
  ret = 1           ;Assume we return a struct
  case state.type of
    0:
    1: if (ev.select eq 1) then begin
      state.excl_pos = uvalue
    endif else begin
      if (state.no_release ne 0) then ret = 0
    endelse
    2: begin
      ; Keep track of the current state
      state.nonexcl_curpos[uvalue] = ev.select
      if (state.no_release ne 0) and (ev.select eq 0) then ret = 0
    end
  endcase
  
  if ret then begin     ;Return a struct?
    ret = {WIDGET_BGROUP, ID:state.base, TOP:ev.top, HANDLER:0L, SELECT:ev.select, $
      VALUE:state.ret_arr[uvalue] }
    efun = state.efun
    epro = state.epro
    widget_control, stash, SET_UVALUE=state, /NO_COPY
    if efun ne '' then begin
    
      return, call_function(efun, ret)
      
    endif else if epro ne '' then begin
    
      call_procedure, epro, ret
      return, 0
      
    endif else begin
    
      return, ret
      
    endelse
    
  endif else begin      ;Trash the event
    widget_control, stash, SET_UVALUE=state, /NO_COPY
    return, 0
  endelse
end

function button_group, parent, names, button_uvalue = button_uvalue, column = column, event_funct = efun     , $
                       event_pro = epro, exclusive = excl, font = font, frame = frame, ids = ids             , $
                       label_top = label_top, label_left = label_left, map = map, nonexclusive = nonexcl     , $
                       no_release = no_release, return_id = return_id, return_index = return_index           , $
                       return_name = return_name, row = row, scroll = scroll, set_value = sval, space = space, $
                       tab_mode = tab_mode, uvalue=uvalue, xoffset = xoffset, xpad = xpad, xsize = xsize     , $
                       x_scroll_size = x_scroll_size, yoffset = yoffset, ypad = ypad, ysize = ysize          , $
                       y_scroll_size = y_scroll_size, uname = uname
    
    
  if (n_params() ne 2) then message, 'Incorrect number of arguments'
  
  on_error, 2                       ;return to caller
  
  ; Set default values for the keywords
  version = widget_info(/version)
  if (version.toolkit eq 'OLIT') then def_space_pad = 4 else def_space_pad = 3
  if (n_elements(column) eq 0)      then column = 0
  if (n_elements(excl) eq 0)        then excl = 0
  if (n_elements(frame) eq 0)       then frame = 0
  if (n_elements(map) eq 0)     then map=1
  if (n_elements(nonexcl) eq 0)     then nonexcl = 0
  if (n_elements(no_release) eq 0)  then no_release = 0
  if (n_elements(row) eq 0)     then row = 0
  if (n_elements(scroll) eq 0)      then scroll = 0
  if (n_elements(space) eq 0)       then space = def_space_pad
  if (n_elements(uname) eq 0)      then uname = 'CW_BGROUP_UNAME'
  if (n_elements(uvalue) eq 0)      then uvalue = 0
  if (n_elements(xoffset) eq 0) then xoffset=0
  if (n_elements(xpad) eq 0) then xpad = def_space_pad
  if (n_elements(xsize) eq 0) then xsize = 0
  if (n_elements(x_scroll_size) eq 0) then x_scroll_size = 0
  if (n_elements(yoffset) eq 0)     then yoffset=0
  if (n_elements(ypad) eq 0)        then ypad = def_space_pad
  if (n_elements(ysize) eq 0)       then ysize = 0
  if (n_elements(y_scroll_size) eq 0)   then y_scroll_size = 0
  
  top_base = 0L
  if (n_elements(label_top) ne 0) then begin
    next_base = widget_base(parent, XOFFSET=xoffset, YOFFSET=yoffset, /COLUMN)
    if(keyword_set(font))then $
      junk = widget_label(next_base, value=label_top,font=font) $
    else    junk = widget_label(next_base, value=label_top)
    top_base = next_base
  endif else next_base = parent
  
  if (n_elements(label_left) ne 0) then begin
    next_base = widget_base(next_base, XOFFSET=xoffset, YOFFSET=yoffset, /ROW)
    if(keyword_set(font))then $
      junk = widget_label(next_base, value=label_left, font=font) $
    else junk = widget_label(next_base, value=label_left)
    if (top_base eq 0L) then top_base = next_base
  endif
  ; We need some kind of outer base to hold the users UVALUE
  if (top_base eq 0L) then begin
    top_base = widget_base(parent, XOFFSET=xoffset, YOFFSET=yoffset)
    next_base = top_base
  endif
  if (top_base eq next_base) then $
    next_base = widget_base(top_base, Xpad=1, Ypad=1, Space=1)
    
  ; Set top level base attributes
  widget_control, top_base, MAP=map, $
    FUNC_GET_VALUE='CW_BGROUP_GETV', PRO_SET_VALUE='CW_BGROUP_SETV', $
    SET_UVALUE=uvalue, SET_UNAME=uname
    
  ; Tabbing
  if (n_elements(tab_mode) ne 0) then begin
    widget_control, top_base, TAB_MODE=tab_mode
    widget_control, next_base, TAB_MODE=tab_mode
  end
  
  ; The actual button holding base
  base = widget_base(next_base, COLUMN=column, EXCLUSIVE=excl, FRAME=frame, $
    NONEXCLUSIVE=nonexcl, ROW=row, SCROLL=scroll, SPACE=space, $
    XPAD=xpad, XSIZE=xsize, X_SCROLL_SIZE=x_scroll_size, $
    YPAD=ypad, YSIZE=ysize, Y_SCROLL_SIZE=y_scroll_size, $
    EVENT_FUNC='CW_BGROUP_EVENT', $
    UVALUE=widget_info(top_base, /child))
    
    
  n = n_elements(names)
  ids = lonarr(n)
  for i = 0, n-1 do begin
    if (n_elements(font) eq 0) then begin
      ids[i] = widget_button(base, value=names[i], UVALUE=i, $
        UNAME=uname+'_BUTTON'+strtrim(i,2))
    endif else begin
      ids[i] = widget_button(base, value=names[i], FONT=font, $
        UVALUE=i, UNAME=uname+'_BUTTON'+strtrim(i,2))
    endelse
  endfor
  
  ; Keep the state info in the real (inner) base UVALUE.
  ; Pick an event value type:
  ; 0 - Return ID
  ; 1 - Return INDEX
  ; 2 - Return NAME
  ret_type = 1
  if keyword_set(RETURN_ID) then ret_type = 0
  if keyword_set(RETURN_NAME) then ret_type = 2
  if keyword_set(BUTTON_UVALUE) then ret_type = 3
  case ret_type of
    0: ret_arr = ids
    1: ret_arr = indgen(n)
    2: ret_arr = names
    3: ret_arr = button_uvalue
  endcase
  type = 0
  if (excl ne 0) then type = 1
  
  if (nonexcl ne 0) then type = 2
  if n_elements(efun) le 0 then efun = ''
  if n_elements(epro) le 0 then epro = ''
  state = { type:type, $    ; 0-Standard, 1-Exclusive, 2-Non-exclusive
    base: top_base, $   ; cw_bgroup base...
    ret_arr:ret_arr, $  ; Vector of event values
    efun : efun, $  ; Name of event fcn
    epro : epro, $  ; Name of event pro
    nonexcl_curpos:intarr(n), $ ; If non-exclus, tracks state
    excl_pos:0, $           ; If exclusive, current button
    ids:ids, $          ; Ids of buttons
    no_release:no_release }
  widget_control, widget_info(top_base, /CHILD), SET_UVALUE=state, /NO_COPY
  
  if (n_elements(sval) ne 0) then cw_bgroup_setv, top_base, sval
  
  return, top_base
  
end
