;+
; :Description:
;    Init somme common and compile related modules.
;
; :Params:
;    run_galaxy : in, required, type="Integer"
;       If set to 1, performs the init for a GALAXY run, otherwise for a COSMOS run.
;
; :Author: pellegrini
;
; 23 nov. 2010
;-
pro cosmos_init

  forward_function build_path  
  forward_function label_frame
  forward_function cosmos_get_default_settings  
  forward_function cosmos_get_settings
  forward_function cosmos_get_table_labels  
  forward_function cosmos_getnumors
  forward_function cosmos_load_settings
  forward_function cosmos_str_make  
  forward_function cosmos_str_split
  forward_function cosmos_table_check
  forward_function str_centerize

  gui_utilities
  paths_utilities  
  strings_utilities
  variables_utilities
  web_utilities
  
  cosmos_globals
  cosmos_str
  cosmos_io  
  cosmos_analysis
  cosmos_widgets
  
end

pro cosmos_cleanup, tlb

  validPtrs = ptr_valid()
  ptr_free, validPtrs

end

;+
; :Description:
;    The handler for the events coming from the "about" dialog.
;
; :Params:
;    event : in, required, type="Event structure"
;       The event.
;
; :Author: pellegrini
;
; 23 nov. 2010
;-
pro cosmos_about_events, event

  ;forward_function build_path
  
  ;web_utilities
    
  ; The type of the event.
  eventType = tag_names(event, /structure_name)
      
  ; Case of a WIDGET_TIMER event. The only one considered for that dialog.
  if eventType eq "WIDGET_TIMER" then begin
  
    p = file_dirname(file_which('cosmos.pro'))  
    rcDir = build_path([p, "rc"])
        
    widget_control, event.top, get_uvalue = imageIndex
    
    ; Increment the timer and mod it by 11 (the # of small images to be displayed in the about dialog).
    imageIndex = 1 + imageIndex mod 11

    widget_control, event.top, set_uvalue = imageIndex
    
    filename = build_path([rcDir, "about" + strtrim(imageIndex,2) + ".jpg"])
    res = query_jpeg(filename)
    if (res eq 0) then return
    
    ; Read the image corresponding to the timer.
    read_jpeg, filename, image
    
    draw = widget_info(event.top, find_by_uname = 'STATUSDRAW')
        
    ; Store the IDL window ID of the drawing area in |winVis|
    widget_control, draw, get_value = winVis
    
    ; Select it as the active one for drawing.
    wset, winVis
    
    ; Display the loaded image in it.
    tvscl, image, true = 1
    
    ; Generate another timer event that will the next image in 3s.
    widget_control, event.top, timer = 2
    
  endif else if eventType eq 'WIDGET_BUTTON' then begin
  
    uname = widget_info(event.id, /uname)
  
    case uname of    
      'ILL': open_url, "http://www.ill.eu/"      
      'LSS': open_url, "http://www.ill.eu/instruments-support/instruments-groups/groups/lss/"      
      'CS':  open_url, "http://www.ill.eu/instruments-support/computing-for-science/home/"      
      else:        
    endcase
  
  endif
  
end

;+
; :Description:
;    Build up the "About" dialog.
;    It is made of a draw area where small images are displayed one after the other and of
;    general informations about the application.
;    This dialog is displayed as a pop-up window with its own event handler.
;
; :Params:
;    event : in, required, type="Event structure"
;       The event.
;
; :Author: pellegrini
;
; 26 nov. 2010
;-
pro cosmos_about_dialog, event
    
  ; The main base. Will host all the child widgets.
  main = widget_base(group_leader = event.top, /base_align_center, /row, title = 'About', tlb_frame_attr = 1, $
                     uvalue = 0, uname = 'MAIN')
  
  ; The main base is actually displayed.
  widget_control, main, /realize
  
  ; The drawing area where the images will be displayed.
  w = widget_draw(main, xsize = 200, ysize = 200, uname = 'STATUSDRAW')
  
  b = widget_base(main, /base_align_center, /column)
  
  ; The labels that will show general information about the application.
  l = widget_label(b, value = "", ysize = 10)
  l = widget_label(b, value = "", ysize = 10)
  l = widget_label(b, value = "Cosmos: A Program To Analyse Neutron Reflectometry Data", font = "Times*20*Italic")
  l = widget_label(b, value = "", ysize = 20)
  l = widget_label(b, value = "A Joined Effort Of", font = "Times*18")
  
  ; Clicking on this button will open the LSS webpage.
  w = widget_button(b, value = "Large Scale Structures Group", font = "Times*18*Bold", xsize = 250, uname = 'LSS')
  ; Clicking on this button will open the CS webpage.
  w  = widget_button(b, value = "Computing For Science Group", font = "Times*18*Bold", xsize = 250, uname = 'CS')
  
  l = widget_label(b, value = "", ysize = 20)
  l = widget_label(b, value = "Developped At", font = "Times*18")
  
  ; Clicking on this button will open the ILL webpage.
  w = widget_button(b, value = "Institut Laue Langevin", font = "Times*18*Bold", xsize = 250, uname = 'ILL')
  
  l = widget_label(b, value = "", ysize = 10)
        
  ; A timer event is generated at 0s.
  widget_control, main, timer = 0
  
  xmanager, "About", main, event_handler = "cosmos_about_events", /just_reg
  
end

;+
; :Description:
;    Reset button.
;
; :Params:
;    event : in, required, type="Event structure"
;       The event.
;
; :Author: pellegrini
;
; 6 déc. 2010
;-
pro cosmos_set_defaults, event

  s = dialog_message('Do you really want to reset to default values ?', /center, /default_no, dialog_parent = event.top, /question, title = "Quit")
  
  s = strupcase(s)

  if (s eq 'NO') then return
  
  settings = cosmos_get_default_settings()

  ; Apply the default settings.
  cosmos_set_settings, event.top, settings
  
end

function cosmos_get_settings, main

  ;forward_function cosmos_get_default_settings
    
  ;variables_utilities
  
  widget_control, main, get_uvalue = galaxyrun
    
  settings = cosmos_get_default_settings()
  
  tags = tag_names(settings)
    
  for i = 0, n_elements(tags) - 1 do begin
    
    ;special treatment of calculation method (droplist)
    if (tags[i] eq 'METHOD1') then begin
      wid = widget_info(main, find_by_uname = 'CALCMETHOD1')
      val = widget_info(wid,/droplist_select)
      widget_control, wid, Get_UValue = list_methods
      method_str = strtrim(list_methods[val],2)
      settings.(i) = method_str
    endif else if (tags[i] eq 'METHOD2') then begin
      wid = widget_info(main, find_by_uname = 'CALCMETHOD2')
      val = widget_info(wid,/droplist_select)
      widget_control, wid, Get_UValue = list_methods
      method_str = strtrim(list_methods[val],2)
      settings.(i) = method_str
    endif else if (tags[i] eq 'METHOD3') then begin
      wid = widget_info(main, find_by_uname = 'CALCMETHOD3')
      val = widget_info(wid,/droplist_select)
      widget_control, wid, Get_UValue = list_methods
      method_str = strtrim(list_methods[val],2)
      settings.(i) = method_str
    endif
    
    wid = widget_info(main, find_by_uname = tags[i])
  
    if not widget_info(wid, /valid_id) then continue
             
    if (tags[i] eq 'TABLE' or tags[i] eq 'GTABLE') then begin
        widget_control, wid, get_value = val
        *settings.(i) = val
    endif else if (tags[i] eq 'MFTPARAMSFIG') then begin
        widget_control, wid, get_uvalue = val
        settings.(i) = val
    endif else if (tags[i] eq 'MFTPARAMSD17') then begin
        widget_control, wid, get_uvalue = val        
        settings.(i) = val
    endif else if (tags[i] eq 'FOREGDWIDTH1') then begin
        widget_control, wid, get_value = val & val = val[0]
        val1 = 0 & val2 = 0
        reads, val+',0', val1, val2
        if val2 eq 0 then begin & val2 = val1/2 & val1 = val2 & endif
        settings.(i) = [val1, val2] 
    endif else if (tags[i] eq 'FOREGDWIDTH2') then begin
        widget_control, wid, get_value = val & val = val[0]
        val1 = 0 & val2 = 0
        reads, val+',0', val1, val2 
        if val2 eq 0 then begin & val2 = val1/2 & val1 = val2 & endif
        settings.(i) = [val1, val2]  
    endif else if (tags[i] eq 'FOREGDWIDTH3') then begin
        widget_control, wid, get_value = val & val = val[0]
        val1 = 0 & val2 = 0
        reads, val+',0', val1, val2 
        if val2 eq 0 then begin & val2 = val1/2 & val1 = val2 & endif
        settings.(i) = [val1, val2] 
    endif else begin
        widget_control, wid, get_value = val
        settings.(i) = val
    endelse

  endfor
    
  return, settings
  
end

pro cosmos_save_input_file, event
  
  settings = cosmos_get_settings(event.top)    

  widget_control, widget_info(event.top, find_by_uname = 'OUTPUTDIR'), get_value = path

  widget_control, widget_info(event.top, find_by_uname = 'OUTPUTBASE'), get_value = basename
  
  widget_control, event.top, get_uvalue = galaxyrun
    
  path = strtrim(path,2)
  
  filename = dialog_pickfile(title = "Select file", dialog_parent = event.top, file = strtrim(basename,2)+".dat", $
                             path = path, /overwrite_prompt, /write)
  
  cosmos_save_settings, event.top, filename, settings, galaxyrun = galaxyrun
  
end

pro cosmos_load_input_file, event

  forward_function cosmos_load_settings
        
  widget_control, widget_info(event.top, find_by_uname = 'OUTPUTDIR'), get_value = path
  
  widget_control, event.top, get_uvalue = galaxyrun
    
  path = strtrim(path,2)
              
  filename = dialog_pickfile(title = "Select input file", dialog_parent = event.top, path = path)
  
  settings = cosmos_load_settings(filename, galaxyrun = galaxyrun)
        
  if (size(settings, /type) ne 8) then return
              
  cosmos_set_settings, event.top, settings
  
  cosmos_log_flush, event.top
  
  widget_control, widget_info(event.top, find_by_uname = 'TABS'), set_tab_current = 0
  
end

pro cosmos_log_flush, main

  common cosmos_message, cosmos_info, cosmos_warn
                  
  widget_control, widget_info(main, find_by_uname = 'LOGINFO'), SET_VALUE = cosmos_info
  widget_control, widget_info(main, find_by_uname = 'LOGWARN'), SET_VALUE = cosmos_warn
  
end

pro cosmos_quit, event

  s = dialog_message('Do you really want to quit ?', /center, /default_no, dialog_parent = event.top, /question, title = "Quit")
  
  s = strupcase(s)
  
  if s eq 'YES' then widget_control, event.top, /destroy
  
end

pro cosmos_set_settings, main, settings

  forward_function cosmos_get_table_labels
          
  tags = tag_names(settings)
            
  for i = 0, n_elements(tags) - 1 do begin
    if tags[i] eq 'METHOD1' or tags[i] eq 'METHOD2' or tags[i] eq 'METHOD3' then begin
      calcname = 'CALC' + tags[i]
      wid = widget_info(main, find_by_uname = calcname)
    endif else wid = widget_info(main, find_by_uname = tags[i])
          
    if not widget_info(wid, /valid_id) then continue
                                              
    if tags[i] eq 'TABLE' or tags[i] eq 'GTABLE' then begin
    
        val = *settings.(i)
        nCols = n_elements(val[*,0])
        nRows = n_elements(val[0,*])
        rowLabels = strtrim(indgen(nRows)+1,2)
        tLabels = cosmos_get_table_labels(nCols)
        widget_control, wid, xsize = nCols, ysize = nRows, row_labels = rowLabels, column_labels = tLabels, $
                        set_value = val

    endif else if tags[i] eq 'MFTPARAMSFIG' then begin
        
        val = settings.(i)
        widget_control, wid, set_uvalue = val
        
        buttons = widget_info(wid, /all_children)
        
        for j=0, n_elements(val)-1 do begin

            but = widget_info(buttons[j],/child)
        
            if strlen(val[j]) eq 0 then begin
                widget_control, but, set_value='Figaro Parameter ...                                                     '
            endif else begin
                widget_control, but, set_value=val[j]            
            endelse
           
        endfor
                                           
        widget_control, wid, sensitive=~settings.mftinstr

    endif else if tags[i] eq 'MFTPARAMSD17' then begin
        
        val = settings.(i)
        widget_control, wid, set_uvalue = val
        
        buttons = widget_info(wid, /all_children)
        
        for j=0, n_elements(val)-1 do begin
                                
            but = widget_info(buttons[j],/child)        

            if strlen(val[j]) eq 0 then begin
                widget_control, but, set_value='D17 Parameter ...                                                     '
            endif else begin
                widget_control, but, set_value=val[j]            
            endelse
           
        endfor
                                           
        widget_control, wid, sensitive=settings.mftinstr
        
    endif else if tags[i] eq 'FOREGDWIDTH1' or tags[i] eq 'FOREGDWIDTH2' or tags[i] eq 'FOREGDWIDTH3' then begin
        
        val = settings.(i)
        if val[1] gt 0 then temp = strtrim(string(val[0]),2) + ',' + strtrim(string(val[1]),2) else temp = val[0]
        widget_control, wid, set_value = temp  
        
    endif else if tags[i] eq 'METHOD1' or tags[i] eq 'METHOD2' or tags[i] eq 'METHOD3' then begin   
       calcname = 'CALC' + tags[i]
       wid = widget_info(main, find_by_uname = calcname)
       val = settings.(i)
       if val eq 'coherent' then widget_control, wid, set_droplist_select = 1 else  widget_control, wid, set_droplist_select = 0
                      
    endif else begin
    
        val = settings.(i)
        widget_control, wid, set_value = val                     
    
    endelse
                                     
  endfor
              
  cosmos_change_number_of_angles, main

;  widget_control, widget_info(main, find_by_uname = 'GROUPVAR1'), sensitive = settings.group
;  widget_control, widget_info(main, find_by_uname = 'GROUPVAR2'), sensitive = settings.group
;  widget_control, widget_info(main, find_by_uname = 'GROUPVAR3'), sensitive = settings.group

  widget_control, widget_info(main, find_by_uname = 'SEARCHPIXELMIN1'), sensitive = 1 - settings.autosearchpixel1
  widget_control, widget_info(main, find_by_uname = 'SEARCHPIXELMAX1'), sensitive = 1 - settings.autosearchpixel1
  widget_control, widget_info(main, find_by_uname = 'SEARCHPIXELMIN2'), sensitive = 1 - settings.autosearchpixel2
  widget_control, widget_info(main, find_by_uname = 'SEARCHPIXELMAX2'), sensitive = 1 - settings.autosearchpixel2
  widget_control, widget_info(main, find_by_uname = 'SEARCHPIXELMIN3'), sensitive = 1 - settings.autosearchpixel3
  widget_control, widget_info(main, find_by_uname = 'SEARCHPIXELMAX3'), sensitive = 1 - settings.autosearchpixel3

  widget_control, widget_info(main, find_by_uname = 'SEARCHTIMEMIN1'), sensitive = 1 - settings.autosearchtime1
  widget_control, widget_info(main, find_by_uname = 'SEARCHTIMEMAX1'), sensitive = 1 - settings.autosearchtime1
  widget_control, widget_info(main, find_by_uname = 'SEARCHTIMEMIN2'), sensitive = 1 - settings.autosearchtime2
  widget_control, widget_info(main, find_by_uname = 'SEARCHTIMEMAX2'), sensitive = 1 - settings.autosearchtime2
  widget_control, widget_info(main, find_by_uname = 'SEARCHTIMEMIN3'), sensitive = 1 - settings.autosearchtime3
  widget_control, widget_info(main, find_by_uname = 'SEARCHTIMEMAX3'), sensitive = 1 - settings.autosearchtime3

  widget_control, widget_info(main, find_by_uname = 'DETPIXEL1'), sensitive = 1 - settings.autosearchtime2
  widget_control, widget_info(main, find_by_uname = 'DETPIXEL2'), sensitive = 1 - settings.autosearchtime3
  widget_control, widget_info(main, find_by_uname = 'DETPIXEL3'), sensitive = 1 - settings.autosearchtime3

  base = widget_info(main, find_by_uname = 'MACHBASE')
  widget_control, base, sensitive = settings.override
                
end

pro cosmos_startup_dialog, msg, title, im

  ;gui_utilities

  res = query_jpeg(im)
  if (res eq 0) then return

  c_font_big = "arial*bold*15"
  
  tlb = widget_base(/COLUMN, TITLE = title, TLB_FRAME_ATTR = 13)
    
  read_jpeg, im, image
  
  s = size(image)
  
  sx = s[2]
  sy = s[3]
  
  aspect = 1.0*sy/sx
  
  winxsize = 400
  winysize = fix(aspect*winxsize)
  
  winbase = widget_base(tlb, XSIZE = winxsize, YSIZE = winysize)
  win = widget_draw(winbase, XSIZE = winxsize, YSIZE = winysize)
  
  void = widget_text(tlb, VALUE = msg, FONT = c_font_big)
  
  centertlb, tlb
  
  widget_control, tlb, /realize
  
  ; Get the the IDL window ID for the drawing idea.
  widget_control, win, GET_VALUE = winVis
  
  ; Create a window.
  window, /FREE, /PIXMAP, XSIZE = winxsize, YSIZE = winysize
  
  winPix = !d.window
  
  ; color values will be interpreted as 24-bit color specification.
  device, DECOMPOSED = 1
  
  tvimage, image
  
  wset, winVis
  
  device, COPY = [0, 0, !d.x_size, !d.y_size, 0, 0, winPix]
  
  wdelete, winPix
  
  wait, 1
  
  widget_control, tlb, /destroy
  
  device, DECOMPOSED = 0
  
end

function cosmos_table_check, main, table
    
  nRows = n_elements(table[0,*])  
  nCols = n_elements(table[*,0])  
  
  tLabels = cosmos_get_table_labels(nCols)
  
  on_ioerror, bad_row
  
  checkedRows = make_array(nRows, /int, value = 0)

  for r = 0, nRows - 1 do begin
    
    if array_equal(strcompress(table[*,r], /remove_all), "") then begin
      checkedRows[r] = 2
      continue    
    endif
    
    valid = 0

    badCell = 0
          
    for c = 0, nCols - 1 do begin    
      
      ;check that cell contains something after removing spaces
      if strlen(strtrim(table[c,r],2)) le 0 then continue
          
      case 1 of

        (stregex(tLabels[c], "^(position)", /fold_case) ne -1) : begin
          temp = fix(table[c,r])
          if (temp le 0) or (temp gt 6) then begin
            badCell = 1
            break
          endif        
        end
      
        (stregex(tLabels[c], "^(date)", /fold_case) ne -1) : begin
          temp = strsplit(table[c,r], '/', /extract)        
          date = julday(temp[1], temp[0], temp[2])        
        end
                      
        (stregex(tLabels[c], "^(time)", /fold_case) ne -1) : begin
          temp = strsplit(table[c,r], ':', /extract)
          hour = fix(temp[0])
          if (hour lt 0) or (hour ge 24) then begin
            badCell = 1
            break
          endif
          minute = fix(temp[1])
          if (minute lt 0) or (minute ge 60) then begin
            badCell = 1
            break
          endif          
        end              
      
        (stregex(tLabels[c], "^(direct|reflect|instr)", /fold_case) ne -1) : begin
          val = cosmos_getnumors(table[c,r])
          s = size(val)
          if s[0] eq 0 then begin
            badCell = 1
            break          
          endif          
        end
                       
        (stregex(tLabels[c], "^(normalise|factor)", /fold_case) ne -1) : temp = float(table[c,r])
        
        (stregex(tLabels[c], "out lamp", /fold_case) ne -1) : begin
          temp = fix(table[c,r])
          if (temp le 0) or (temp gt 60) then begin
            badCell = 1
            break          
          endif
        end
               
        else:
              
      endcase
        
    endfor
        
    if not badCell then valid = 1 
    
    bad_row:
      if not valid then checkedRows[r] = 1
        
  endfor
                
  return, checkedRows
           
end

pro cosmos_run, event

  cosmos_globals
  
  common cosmos_params, c_params
      
  common cosmos_settings, c_settings
  
  common cosmos_results, c_results
  
  widget_control, widget_info(event.top, find_by_uname = 'PLOTLIST'), set_value = ""
  
  widget_control, widget_info(event.top, find_by_uname = 'PLOTWIN'), get_value = wid
  wset, wid
  erase, 0
    
  c_settings = cosmos_get_settings(event.top)

  ;polarization file
  pf = widget_info(event.top, find_by_uname = 'POLCORRFILE')
  widget_control, pf, get_value = polFile
  c_settings.polcorrfile = polFile
    
  wTable = widget_info(event.top, find_by_uname = 'TABLE')
    
  selCells = widget_info(wTable, /table_select)  
    
  widget_control, wTable, get_value = table
          
  checkedRows = cosmos_table_check(event.top, table)
  
  selRows = selCells[1] + indgen(selCells[3]-selCells[1]+1)
  
  idx = where(checkedRows[selRows] eq 2, nEmptyRows)

  nSelRows = n_elements(table[0,selRows])
  
  if nEmptyRows eq nSelRows then begin  
    s = dialog_message('Please enter some data.', /center, /default_no, dialog_parent = event.top, $
                       title = "Warning")
    return    
  endif

  idx = where(checkedRows[selRows] eq 1, nInvalidRows)
              
  if nInvalidRows gt 0 then begin  
    mess = 'Error when checking rows.' + string(10b) + $
           'The following rows seem to be misformatted: ' + string(10b) + $
           strcompress(strjoin(selRows[idx]+1)) + string(10b) + 'Continue skipping those rows ?' 
           
    cancel = dialog_message(mess, /cancel, /default_cancel, title = 'Warning')
    cancel = strupcase(cancel)
    if cancel eq "CANCEL" then return
        
  endif
    
  if nEmptyRows + nInvalidRows eq nSelRows then return

;  idx = where(checkedRows[selRows] ne 1)
              
;  table = table[*,idx]
    
;  nRowsNew = n_elements(table[0,*])
    
;  rowLabels = strtrim((indgen(nRowsNew) + 1),2)
      
;  widget_control, wTable, row_labels = rowLabels, ysize = nRowsNew, alignment = 2, /editable, set_value = table
          
  nCols = n_elements(table[*,0])
                      
  if selCells[0] le 0 and selCells[2] ge nCols - 1 then begin
  
    rowSel = [selCells[1], selCells[3]+1] 
  
  endif else begin

    rowSel = [0, n_elements(table[0,*])] 
  
  endelse
    
  *c_settings.table = table
    
  c_params.rowinit = rowSel[0]
  c_params.row = rowSel[0]
  c_params.rowfinish = rowSel[1]
  
  cosmos_run_setup

  cosmos_logmessage, ''
  cosmos_logmessage, 'Analyzing selected rows ' + cosmos_str_make(c_params.row+1) + ' to ' + cosmos_str_make(c_params.rowfinish)

  draw =  widget_info(event.top, find_by_uname = 'STATUSDRAW')

  cosmos_init_statusbar, event
  
  widget_control, event.top, timer = 0.0
  
end

pro cosmos_eval_row, event

  common cosmos_params, c_params
  common cosmos_results, c_results

  draw =  widget_info(event.top, find_by_uname = 'STATUSDRAW')
  cancel =  widget_info(event.top, find_by_uname = 'STOP')

  widget_control, cancel, get_uvalue = abortRun
      
  if abortRun eq 0 then begin
    
    ;Initialize Lamp workspace w61 to store all the results
    if c_params.row eq c_params.rowinit then cosmos_lamp_init

    if c_params.row eq c_params.rowfinish then begin
      if n_elements(c_results) gt 0 then begin
        plotNames = tag_names(c_results)
        widget_control, widget_info(event.top, find_by_uname = 'PLOTLIST'), set_value = plotNames
        cosmos_show_plot, event, n_elements(plotNames) - 1
        cosmos_log_flush, event.top
      endif
      cosmos_lamp_end
    endif

    if c_params.row lt c_params.rowfinish then begin
      cosmos_run_background
      widget_control, event.top, timer = 0.0
      cosmos_update_statusbar, event
    endif
                      
  endif else begin

    cosmos_init_statusbar, event
              
    widget_control, cancel, set_uvalue = 0
        
  endelse

end

pro cosmos_cancel_run, event

  s = dialog_message('Do you really want to cancel the job ?', /center, /default_no, dialog_parent = event.top, $
                     /question, title = "Abort Run")
  s = strupcase(s)
  if s eq 'YES' then begin        
    widget_control, widget_info(event.top, find_by_uname = 'STOP'), set_uvalue = 1
  endif
          
end

pro cosmos_news_event, event

  ; The type of event that occured.
  eventType = tag_names(event, /structure_name)
  
  uname = widget_info(event.id, /uname)
  
  widget_control, event.top, get_uvalue = galaxyrun
    
  ; A dialog resizing event.
  if eventType eq 'WIDGET_BASE' then begin
 
    ; The dialog can not be smaller than 600 along x. 
    xSize = event.x > 600
    ; The dialog can not be smaller than 400 along y. 
    ySize = event.y > 400
  
    widget_control, event.top, scr_xsize = xSize, scr_ysize = ySize
    
    text = widget_info(event.top, find_by_uname = 'COSMOS_NEWS_TEXT')
  
    ; The tabs are resized according to the new dimensions.  
    widget_control, text, SCR_XSIZE=xSize, SCR_YSIZE=ySize
    ;widget_control, text, SCR_XSIZE=500, SCR_YSIZE=300 
          
  ; A dialog killing event.
  endif else if (eventType eq 'WIDGET_KILL_REQUEST') then begin

    widget_control, event.top, /destroy

  endif

end

pro cosmos_display_news, event

  common cosmos_help, cosmos_help_message, cosmos_version

  ; If the widget is already opened, then do nothing.
  if xregistered("cosmos_news") then return
      
  ; The main base. Will host all the child widgets.
  tlb = widget_base(/BASE_ALIGN_CENTER, /ROW, TITLE="News and Updates", /TLB_SIZE_EVENTS, /TLB_KILL_REQUEST_EVENTS)
  
  ; The main base is actually displayed.
  widget_control, tlb, /realize

  w = widget_text(tlb, VALUE=cosmos_help_message, /SCROLL, UNAME='COSMOS_NEWS_TEXT')
            
  cosmos_news_event, {WIDGET_BASE, ID : tlb, TOP : tlb, HANDLER : tlb, X : 600, Y : 400}            
  
  xmanager, "cosmos_news", tlb, event_handler = "cosmos_news_event", /just_reg

end

pro cosmos_select_mft_parameter, event
     
    child = widget_info(event.id, /child)

    widget_control, event.id, get_uvalue=id
        
    widget_control, child, set_value=event.value

    inst = widget_info(event.top, find_by_uname='MFTINSTR')
    
    widget_control, inst, get_value=val
                
    if (val eq 0) then begin
        base = widget_info(event.top, find_by_uname="MFTPARAMSFIG")
    endif else begin
        base = widget_info(event.top, find_by_uname="MFTPARAMSD17")    
    endelse
    widget_control, base, get_uvalue=mftparams
    mftparams[id] = event.value
    widget_control, base, set_uvalue=mftparams

                  
end

pro cosmos_select_instrument, event
    
    basefig = widget_info(event.top, find_by_uname="MFTPARAMSFIG")
    based17 = widget_info(event.top, find_by_uname="MFTPARAMSD17")    

    widget_control, basefig, sensitive=~event.value
    widget_control, based17, sensitive=event.value

end

pro cosmos_events, event

  catch, stat & if stat ne 0 then begin print,!error_state.msg & return & endif
    
  ; The type of event that occured.
  eventType = tag_names(event, /structure_name)
  
  uname = widget_info(event.id, /uname)
        
  widget_control, event.top, get_uvalue = galaxyrun
      
  ; A dialog resizing event.
  if eventType eq 'WIDGET_BASE' then begin
  
    cosmos_resize, event

  ; A dialog killing event.
  endif else if eventType eq 'WIDGET_KILL_REQUEST' then begin
  
    cosmos_quit, event
    
  endif else if eventType eq 'WIDGET_TIMER' then begin
  
    case uname of
      'MAIN': cosmos_eval_row, event
      'PLAY': cosmos_run_animation, event
      else:
    endcase

  endif else if eventType eq 'WIDGET_CONTEXT' then begin
    table = widget_info(event.top, find_by_uname = 'TABLE')
    tablemenu = widget_info(event.top, find_by_uname = 'TABLEMENU')

    widget_displaycontextmenu, table, event.x, event.y, tablemenu

  endif else if eventType eq 'WIDGET_TRACKING' then begin
  
    if (event.ENTER eq 1) then begin
      cosmos_display_doc, event
    endif else begin
      widget_control, widget_info(event.top, find_by_uname = 'STATUSLINE'), set_value=''
    endelse   
     
  endif else begin
      
    case uname of
      'HELP'     : begin
                     help_message = [$
                         'For information on Cosmos please see FIGARO and D17 webpages.',$
                         'You are welcome to contact the instrument responsibles or',$
                         'Miguel A. Gonzalez (gonzalezm@ill.eu) with information about',$
                         'bugs and any other suggestion.']
                      w = dialog_message(help_message, /center, /information)  
                   end     
      'QUIT'     : cosmos_quit, event
      'RESET'    : cosmos_set_defaults, event      
      'SAVE'     : cosmos_save_input_file, event      
      'LOAD'     : cosmos_load_input_file, event      
      'RUN'      : begin
                     if galaxyrun then begin
                       galaxy_run, event
                     endif else begin
                       cosmos_run, event
                     end
                   end
      'NEWS'     : cosmos_display_news, event       
      'ABOUT'    : cosmos_about_dialog, event       
      'CANCEL'   : cosmos_cancel_run, event

      'CLEARALL': cosmos_data_clear, event      
      'CLEAR':    cosmos_data_clear, event
      'INSROW':   cosmos_data_insert_rows, event
      'DELROW':   cosmos_data_delete_rows, event 
      'SETVAL':   cosmos_data_set, event      
      'INCRVAL':  cosmos_data_increment, event
      'SELALL':   cosmos_data_selectall, event     
      'COPY':     cosmos_data_copy, event      
      'PASTE':    cosmos_data_paste, event
      'NANGLES':  cosmos_data_update_columns, event
      'INSTBG':   cosmos_data_update_columns, event
      'EXPORTCSV': cosmos_data_export, event
      'IMPORTCSV': cosmos_data_import, event
      'FILLINTERVAL': cosmos_data_fill, event

      'GROUP' : begin
                  widget_control, widget_info(event.top, find_by_uname = 'GROUPVAR1'), sensitive = event.select
                  widget_control, widget_info(event.top, find_by_uname = 'GROUPVAR2'), sensitive = event.select
                  widget_control, widget_info(event.top, find_by_uname = 'GROUPVAR3'), sensitive = event.select
                end

      'QRANGESEL' : cosmos_select_qrange, event
      'SETNORMVAR': cosmos_normalization_run, event
      'TRANSFERDB': cosmos_transfer_db, event
      'TRANSFERTHETA': cosmos_transfer_theta, event
      'TRANSFERNORMVAR': cosmos_transfer_normalization_factors, event

      'AUTOSEARCHPIXEL1': begin
        widget_control, widget_info(event.top, find_by_uname = 'SEARCHPIXELMIN1'), sensitive = 1 - event.select
        widget_control, widget_info(event.top, find_by_uname = 'SEARCHPIXELMAX1'), sensitive = 1 - event.select
      end
      'AUTOSEARCHPIXEL2': begin
        widget_control, widget_info(event.top, find_by_uname = 'SEARCHPIXELMIN2'), sensitive = 1 - event.select
        widget_control, widget_info(event.top, find_by_uname = 'SEARCHPIXELMAX2'), sensitive = 1 - event.select
      end
      'AUTOSEARCHPIXEL3': begin
        widget_control, widget_info(event.top, find_by_uname = 'SEARCHPIXELMIN3'), sensitive = 1 - event.select
        widget_control, widget_info(event.top, find_by_uname = 'SEARCHPIXELMAX3'), sensitive = 1 - event.select
      end
      'AUTOSEARCHTIME1': begin
        widget_control, widget_info(event.top, find_by_uname = 'SEARCHTIMEMIN1'), sensitive = 1 - event.select
        widget_control, widget_info(event.top, find_by_uname = 'SEARCHTIMEMAX1'), sensitive = 1 - event.select
      end
      'AUTOSEARCHTIME2': begin
        widget_control, widget_info(event.top, find_by_uname = 'SEARCHTIMEMIN2'), sensitive = 1 - event.select
        widget_control, widget_info(event.top, find_by_uname = 'SEARCHTIMEMAX2'), sensitive = 1 - event.select
      end
      'AUTOSEARCHTIME3': begin
        widget_control, widget_info(event.top, find_by_uname = 'SEARCHTIMEMIN3'), sensitive = 1 - event.select
        widget_control, widget_info(event.top, find_by_uname = 'SEARCHTIMEMAX3'), sensitive = 1 - event.select
      end

      'TAKE': begin
           cosmos_take_machine_settings, event
      end
      'OVERRIDE': begin    
        base = widget_info(event.top, find_by_uname = 'MACHBASE')
        widget_control, base, sensitive = event.select
      end  

      'LOGCLEAN': cosmos_clean_log, event
      'LOGSAVE':  cosmos_save_log, event
      'LOGREFRESH':  cosmos_log_flush, event.top
       
      'PATHBROWSE': begin
          pathDir = widget_info(event.top, find_by_uname = 'PATHDIR')
          widget_control, pathDir, get_value=initDir      
          dirname = dialog_pickfile(title = 'Select Directory', dialog_parent = event.top, /directory, path=initDir)
          if strlen(strtrim(dirname,2)) gt 0 then begin
            widget_control, pathDir, set_value = strtrim(dirname,2)
          endif      
        end

      'OUTPUTDIRBROWSE': begin
        outputDir = widget_info(event.top, find_by_uname = 'OUTPUTDIR')
        widget_control, outputDir, get_value=initDir      
        dirname = dialog_pickfile(title = 'Select Directory', dialog_parent = event.top, /directory, path=initDir)      
        if strlen(strtrim(dirname,2)) gt 0 then begin
          widget_control, outputDir, set_value = strtrim(dirname,2)
        endif      
      end

      'POLFILEBROWSE': begin
        outputFile = widget_info(event.top, find_by_uname = 'POLCORRFILE')
        polFile = dialog_pickfile(title = 'Select file', dialog_parent = event.top)
        if strlen(strtrim(polFile,2)) gt 0 then begin
          widget_control, outputFile, set_value = strtrim(polFile,2)
        endif
      end
      
      'PLOTLIST'   : cosmos_show_plot, event
      'FIRST'      : cosmos_show_first_plot, event
      'PREV'       : cosmos_show_previous_plot, event
      'NEXT'       : cosmos_show_next_plot, event
      'LAST'       : cosmos_show_last_plot, event
      'STOP'       : cosmos_stop_animation, event
      'PLAY'       : cosmos_play_animation, event
      'QPLOTSTYLE' : cosmos_show_plot, event
      'RPLOTSTYLE' : cosmos_show_plot, event
      'MFTINSTR'   : cosmos_select_instrument, event
      'MFTPARAM1': cosmos_select_mft_parameter, event
      'MFTPARAM2': cosmos_select_mft_parameter, event
      'MFTPARAM3': cosmos_select_mft_parameter, event
      'MFTPARAM4': cosmos_select_mft_parameter, event
      'MFTPARAM5': cosmos_select_mft_parameter, event
      'MFTPARAM6': cosmos_select_mft_parameter, event
      'MFTPARAM7': cosmos_select_mft_parameter, event
      'MFTPARAM8': cosmos_select_mft_parameter, event
      'MFTPARAM9': cosmos_select_mft_parameter, event
     
      else:     

    endcase
      
  
  endelse
  
end

pro cosmos, group_leader = group_leader, compile= compile
  
  cosmos_init

  common cosmos_help, cosmos_help_message, cosmos_version
  
  if keyword_set(compile) then return
  
  if xregistered('cosmos') then return
      
  title = 'COSMOS version ' + strtrim(cosmos_version,2)

  p = file_dirname(file_which('cosmos.pro'))  
  rcDir = build_path([p, "rc"])
    
  im = build_path([rcDir, 'cosmos_logo.jpg'])
      
  ; Sets the default group_leader if none is given.
  if n_elements(group_leader) eq 0 then group_leader = 0L
  
  cosmos_startup_dialog, 'Building the ' + title + ' user interface ... please wait', title, im
      
  ; create main window
  main = widget_base(group_leader = group_leader, /base_align_left, /column, title = title, mbar = bar, $
                     /tlb_kill_request_events, /tlb_size_events, uvalue = 0, uname = 'MAIN', resource_name='lamptouch',rname_mbar='lampben')
    
  b = widget_base(main, /row, /toolbar)

;  w = widget_button(b, value = build_path([rcDir, "loadfile.bmp"]), /bitmap, tooltip = "Load input file", uname = 'LOAD')
;  w = widget_button(b, value = build_path([rcDir, "savefile.bmp"]), /bitmap, tooltip = "Save input file", uname = 'SAVE')
;  w = widget_button(b, value = build_path([rcDir, "default.bmp"]), /bitmap, tooltip = "Back to default values", uname = 'DEFAULT')
;  w = widget_button(b, value = build_path([rcDir, "run.bmp"]), /bitmap, tooltip = "Run", uname = 'RUN')
;  w = widget_button(b, value = build_path([rcDir, "quit.bmp"]), /bitmap, tooltip = "Quit", uname = 'QUIT')
;  w = widget_button(b, value = build_path([rcDir, "help.bmp"]), /bitmap, tooltip = "Help", uname = 'HELP')
;  w = widget_button(b, value = build_path([rcDir, "info.bmp"]), /bitmap, tooltip = "About", uname = 'ABOUT')

  w = widget_button(b, value = str_centerize('Load',8),  xsize=50, tooltip = "Load input file", uname = 'LOAD')
  w = widget_button(b, value = str_centerize('Save',8),  xsize=50, tooltip = "Save input file", uname = 'SAVE')
  w = widget_button(b, value = str_centerize('Reset',9), xsize=50, tooltip = "Back to default values", uname = 'RESET')
  w = widget_button(b, value = str_centerize('Help',8),  xsize=50, tooltip = "Help pages", uname = 'HELP')
  w = widget_button(b, value = str_centerize('About',9), xsize=50, tooltip = "About application", uname = 'ABOUT')
  w = widget_button(b, value = str_centerize('News',8), xsize=50, tooltip = "News and updates", uname = 'NEWS')
  w = widget_button(b, value = str_centerize('Quit',8),  xsize=50, tooltip = "Quit", uname = 'QUIT')
  
  ; The 'File' menu button.
  b = widget_button(bar, value = 'File', /menu)
  w = widget_button(b, value = 'Load' , accelerator = "Ctrl+O", /checked_menu, uname = 'LOAD')
  w = widget_button(b, value = 'Save' , accelerator = "Ctrl+S", /checked_menu, uname = 'SAVE')
  w = widget_button(b, value = 'Reset', accelerator = "Ctrl+D", /checked_menu, uname = 'DEFAULT')
  w = widget_button(b, value = 'Run'  , accelerator = "Ctrl+R", /checked_menu, uname = 'RUN')
  w = widget_button(b, value = 'Quit' , accelerator = "Ctrl+Q", /checked_menu, uname = 'QUIT')
  
  ; The 'Help' menu button.
  b = widget_button(bar, value = 'Help', /menu, /help)
  w = widget_button(b, value = 'Help', accelerator = "Ctrl+H", /checked_menu, uname = 'HELP')
  w = widget_button(b, value = 'News and updates', accelerator = "Ctrl+U", /checked_menu, uname = 'NEWS')
  w = widget_button(b, value = 'About', /checked_menu, uname = 'ABOUT')
      
  tabs = widget_tab(main, uname = 'TABS')    
  cosmos_data_tab,          parent = tabs  
  cosmos_calculation_tab,   parent = tabs
  cosmos_normalization_tab, parent = tabs  
  cosmos_detector_tab,      parent = tabs  
  cosmos_machine_tab,       parent = tabs    
  cosmos_io_tab,            parent = tabs  
  cosmos_log_tab,           parent = tabs  
  cosmos_plot_tab,          parent = tabs

  b = label_frame(main, /row, /base_align_center, value = " Job status ", font = c_font_big, ysize=40, /frame, uname = 'STATUSBASE')
  w  = widget_button(b, value = string('Stop', FORMAT='(3X,A-6,3X)'), tooltip = "Stop current execution", xsize = 50, uvalue = 0, uname = 'CANCEL')
  w = widget_draw(b, ysize = 20, uname = 'STATUSDRAW')
  w = widget_button(b, value = string('Run', FORMAT='(3X,A-6,3X)'), tooltip = "Run Cosmos", uname = 'RUN')

  l = widget_label(main, value='', ysize=20, /sunken_frame, /align_left, uname='STATUSLINE')

  settings = cosmos_get_default_settings()
  cosmos_set_settings, main, settings

  widget_control, main, /realize
  
  plotwin = widget_info(main, find_by_uname = 'PLOTWIN')        
  widget_control, plotwin, get_value = wid
  wset, wid
            
  centertlb, main
      
  cosmos_resize, {WIDGET_BASE, ID : main, TOP : main, HANDLER : main, X : 900, Y : 700}  
        
  xmanager, 'cosmos', main, event_handler = 'cosmos_events', /no_block, /just_reg
  
end
