;+
; :Description:
;    Takes a string and turns it into a number, depending upon the required data type.
;
; :Params:
;    inputValue : in, required, type="str"
;    dataType : in, required
;
; :Returns:
;     The number resulting from the conversion of the input string.
;     'NULLVALUE' if the conversion failed.
;
; :Author: fanning
;
; :History:
;    fanning, 17 Nov. 1999
;    pellegrini, 22 Feb. 2011
;
; :Categories: gui
;
; :private:
;-

function input_field_returnvalue, inputValue, dataType

  on_ioerror, CatchIt
  
  if (byte(inputValue))[0] eq 32B then return, ""
  
  case dataType of
  
    'INT': if inputValue eq "" or inputValue eq "-" or inputValue eq "+" then $
      retValue = 'NULLVALUE' else retValue = fix(inputValue)
      
    'LONG': if inputValue eq "" or inputValue eq "-" or inputValue eq "+" then $
      retValue = 'NULLVALUE' else retValue = long(inputValue)
      
    'FLOAT' : if inputValue eq "" or inputValue eq "-" or inputValue eq "+" then $
      retValue = 'NULLVALUE' else retValue = float(inputValue)
      
    'DOUBLE': if inputValue eq "" or inputValue eq "-" or inputValue eq "+" then $
      retValue = 'NULLVALUE' else retValue = double(inputValue)
      
    'STRING' : retValue = inputValue
    
  endcase
  
  return, retValue
  
  CatchIt:
  retValue = 'NULLVALUE'
  return, retValue
  
end

;+
; :Description:
;    Eliminates illegal characters from a string that represents a number.
;    The return value is a properly formatted string that can be turned into
;    an INT, LONG, FLOAT, or DOUBLE value.
;
; :Params:
;    value : in, optional, type="str"
;       The input string.
;       Default = ""
;    dataType : in, optional, type="str"
;       The output type.
;       Default = "STRING"
;
; :Keywords:
;    decimal : in, optional, type="int"
;       The number of decimal in case of a floating-string.
;    digits : in, optional, type="int"
;       The number of digits in case of a integer-string.
;    positive : in, required, type="bool"
;       If set, treat the string as positive number.
;
; :Returns:
;     The formatted string.
;
; :Author: fanning
;
; :History:
;    fanning, 17 Nov. 1999
;    pellegrini, 22 févr. 2011
;
; :Categories: strings
;-
function input_field_validate, value, dataType, decimal = decimal, digits = digits, positive = positive

  if n_elements(value) eq 0 then value = ""
  
  value = value[0]
  
  if value eq "" then return, string(value)
  
  ; No leading or trainnig blank characters to evaluate.
  value = strtrim(value, 2)
  
  ; The default value for dataType is set.
  if n_elements(dataType) eq 0 then dataType = 'STRING'
  
  ; A string value should be returned at once. Nothing to check.
  if strupcase(dataType) eq 'STRING' then return, value
  
  ; Check integers and longs. A "-" or "+" in the first character is allowed. Otherwise,
  ; only number between 0 and 9, or 43B to 57B.
  if strupcase(dataType) eq 'INT' or strupcase(dataType) eq 'LONG' then begin
  
    returnValue = ptr_new(/Allocate_Heap)
    asBytes = byte(value)
    if positive then begin
    
      if (asBytes[0] eq 43B) or (asBytes[0] ge 48B and asBytes[0] le 57B) then *returnValue = [asBytes[0]]
      
    endif else begin
    
      if (asBytes[0] eq 45B) or (asBytes[0] eq 43B) or (asBytes[0] ge 48B and asBytes[0] le 57B) then *returnValue = [asBytes[0]]
      
    endelse
    
    length = strlen(asBytes)
    if length eq 1 then begin
    
      if n_elements(*returnValue) eq 0 then *returnValue = [32B] else *returnValue = [asBytes[0]]
      
    endif else begin
    
      for j = 1, length-1 do begin
      
        if (asBytes[j] ge 48B and asBytes[j] le 57B) then begin
          if n_elements(*returnValue) eq 0 then *returnValue = [asBytes[j]] else *returnValue = [*returnValue, asBytes[j]]
        endif
      endfor
      
    endelse
    
    if n_elements(*returnValue) ne 0 then retValue = string(*returnValue) else retValue = ""
    
    ptr_free, returnValue
    
    ; Check for digit restrictions.
    if digits gt 0 then begin
      retValue = strtrim(retValue, 2)
      if strmid(retValue, 0, 1) eq "-" then digits = digits + 1
      retValue = strmid(retValue, 0, digits)
    endif
    
    return, retValue
    
  endif
  
  ; Check floating and double values. (+,-) in first character or after 'eEdD'.
  ; Only numbers, signs, decimal points, and 'eEdD' allowed.
  if strupcase(dataType) eq 'FLOAT' or strupcase(dataType) eq 'DOUBLE' then begin
  
    returnValue = ptr_new(/Allocate_Heap)
    asBytes = byte(value)
    
    if positive then begin
    
      if (asBytes[0] eq 43B) or (asBytes[0] ge 48B and asBytes[0] le 57B) or $
        (asBytes[0] eq 46B) then *returnValue = [asBytes[0]]
        
      if (asBytes[0] eq 46B) then haveDecimal = 1 else haveDecimal = 0
      
    endif else begin
    
      if (asBytes[0] eq 45B) or (asBytes[0] eq 43B) or (asBytes[0] ge 48B and asBytes[0] le 57B) or $
        (asBytes[0] eq 46B) then *returnValue = [asBytes[0]]
        
      if (asBytes[0] eq 46B) then haveDecimal = 1 else haveDecimal = 0
      
    endelse
    
    haveExponent = 0
    length = strlen(asBytes)
    prevByte = asBytes[0]
    exponents = byte('eEdD')
    
    if length eq 1 then begin
    
      if n_elements(*returnValue) eq 0 then  *returnValue = [32B] else $
        *returnValue = [asBytes[0]]
        
    endif else begin
    
      for j = 1, length - 1 do begin
      
        if (asBytes[j] ge 48B and asBytes[j] le 57B) then begin
        
          if n_elements(*returnValue) eq 0 then *returnValue = [asBytes[j]] else *returnValue = [*returnValue, asBytes[j]]
          prevByte = asBytes[j]
          
        endif else begin
        
          ; What kind of thing is it?
          ; A decimal point.
          if (asBytes[j] eq 46B) then begin
          
            if haveDecimal eq 0 then begin
              *returnValue = [*returnValue, asBytes[j]]
              haveDecimal = 1
              prevByte = asBytes[j]
            endif
          endif
          
          ; A + or - sign.
          if (asBytes[j] eq 45B) or (asBytes[j] eq 43B) then begin
            index = where(exponents eq prevByte, count)
            
            if count eq 1 and haveExponent then begin
            
              *returnValue = [*returnValue, asBytes[j]]
              haveDecimal = 1
              prevByte = asBytes[j]
              
            endif
            
          endif
          
          index = where(exponents eq asBytes[j], count)
          
          ; An exponent
          if count eq 1 and haveExponent eq 0 then begin
            *returnValue = [*returnValue, asBytes[j]]
            haveExponent = 1
            prevByte = asBytes[j]
          endif
          
        endelse
        
      endfor
      
    endelse
    
    if n_elements(*returnValue) ne 0 then begin
    
      retValue = string(*returnValue)
      retValue = strtrim(retValue, 2)
      
      ; Check for decimal restrictions
      if decimal ge 0 then begin
        theDecimalPt = strpos(retValue, '.')
        if theDecimalPt ne -1 then retValue = strmid(retValue, 0, theDecimalPt + decimal + 1)
      endif
      
    endif else retValue = ""
    
    ptr_free, returnValue
    
    ; Is this a representable number?
    testValue = input_field_returnvalue(retValue, dataType)
    if string(testValue) ne 'NULLVALUE' then numCheck = finite(testValue) else numCheck = 1
    
    if numCheck then begin
      return, retValue
    endif else begin
      message, 'The requested number is not representable.', /Informational
      return, ""
    endelse
    
  endif
  
end

pro input_field__define

  ; The Input_Field Event Structure.
  event = {INPUT_FIELD     , $ ; The name of the event structure.
    ID: 0L          , $ ; The ID of the compound widget's top-level base.
    TOP: 0L         , $ ; The widget ID of the top-level base of the hierarchy.
    HANDLER: 0L     , $ ; The event handler ID. Filled out by IDL.
    Value: ptr_new(), $ ; A pointer to the widget value.
    Type:""           $ ; A string indicating the type of data in the VALUE field.
    }                    ; Values are "INT", "LONG", "FLOAT", "DOUBLE", or "STRING".
    
end

;+
; :Description:
;    Cleans up the pointer when the compound widget is destroyed.
;
; :Params:
;    id : in, required, type="int"
;       The compound widget id.
;
; :Author: fanning
;
; :History:
;    fanning, 17 Nov. 1999
;    pellegrini, 22 Feb. 2011
;
; :Categories: gui
;
; :private:
;-
pro input_field_kill_notify, id

  widget_control, id, get_uvalue = info, /no_copy
  ptr_free, info.theValue
  
end

;+
; :Description:
;    Sets a value for the compound widget. The value
;    is a value appropriate for the data type or a string.
;
; :Params:
;    cw_tlb : in, required, type="int"
;       The id of the compound widget.
;    value : in, required, type="any"
;       The value to set the input field with.
;
; :Author: fanning
;
; :History:
;    fanning, 17 Nov. 1999
;    pellegrini, 22 Feb. 2011
;
; :Categories: gui
;
; :private:
;-
pro input_field_set_value, cw_tlb, value

  ; Get info structure.
  info_carrier = widget_info(cw_tlb, find_by_uname = 'INFO_CARRIER')
  widget_control, info_carrier, get_uvalue = info, /no_copy
  
  ; Validate the value.
  theText = strtrim(value, 2)
  theText = input_field_validate(theText, info.dataType, decimal = info.decimal, digits = info.digits, positive = info.positive)
  
  ; Load the value in the widget.
  widget_control, info.textID, set_value = theText, set_text_select = [strlen(theText),0]
  info.theText = theText
  
  ; Set the actual value of the compound widget.
  *info.theValue = input_field_returnvalue(info.theText, info.dataType)
  widget_control, info_carrier, set_uvalue = info, /no_copy
  
end

;+
; :Description:
; This function returns the numerical or string value of the input field.
;
; :Params:
;    cw_tlb : in, required, type="int"
;       The id of the compound widget.
;
; :Returns:
;    The input stored by the input field.
;
; :Author: fanning
;
; :History:
;    fanning, 17 Nov. 1999
;    pellegrini, 22 Feb. 2011
;
; :Categories: gui
;
; :private:
;-
function input_field_get_value, cw_tlb

  info_carrier = widget_info(cw_tlb, find_by_uname = 'INFO_CARRIER')
  widget_control, info_carrier, get_uvalue = info, /no_copy
  value = info.theValue
  widget_control, info_carrier, set_uvalue = info, /no_copy
  
  return, value
  
end

;+
; :Description:
;    The main event handler for the input field.
;
; :Params:
;    event : in, required, type="struct"
;       The IDL event structure.
;
; :Author: fanning
;
; :History:
;    fanning, 17 Nov. 1999
;    pellegrini, 22 Feb. 2011
;
; :Categories: gui
;
; :private:
;-
pro input_field_event_handler, event


  ; Get the info structure. Get the previous text, the current
  ; cursor location in the text widget, and indicate this is not
  ; a Carriage Return event.
  widget_control, event.ID, get_uvalue = info, /no_copy
  previousText = info.theText
  textLocation = widget_info(event.id, /Text_Select)
  cr_event = 0
  
  ; What kind of event is this?
  possibleTypes = ['INSERT SINGLE CHARACTER', 'INSERT MULTIPLE CHARACTERS', 'DELETE TEXT', 'SELECT TEXT']
  thisType = possibleTypes[event.type]
  
  ; Branch on event type.
  case thisType of
  
    'INSERT SINGLE CHARACTER': begin
    
      ; Get the current contents of text widget. Validate it.
      widget_control, info.textID, get_value = newText
      newText = newText[0]
      validText = input_field_validate(newText, info.dataType, decimal = info.decimal, digits = info.digits, positive = info.positive)
      
      ; If it is valid, leave it alone. If not, go back to previous text.
      if validText ne newText then begin
        widget_control, info.textID, set_value = previousText, set_text_select = [textLocation[0]-1,0]
      endif else begin
        info.theText = validText
        testValue  = input_field_returnvalue(validText, info.dataType)
        if string(testValue) eq "NULLVALUE" then begin
          ptr_free, info.theValue
          info.theValue = ptr_new(/Allocate_Heap)
        endif else *info.theValue = testValue
      endelse
      
      ; Is this a Carriage Return event?
      if event.ch eq 10B then cr_event = 1
      
    end
    
    'INSERT MULTIPLE CHARACTERS': begin
    
      ; Same thing as above, but for all the characters you are inserting.
      widget_control, info.textID, get_value = newText
      newText = newText[0]
      validText = input_field_validate(newText, info.dataType, decimal = info.decimal, digits = info.digits, positive = info.positive)
      if validText ne newText then begin
        widget_control, info.textID, set_value = previousText, set_text_select = [textLocation[0]-1,0]
      endif else begin
        info.theText = validText
        testValue  = input_field_returnvalue(validText, info.dataType)
        if string(testValue) eq "NULLVALUE" then begin
          ptr_free, info.theValue
          info.theValue = ptr_new(/Allocate_Heap)
        endif else *info.theValue = testValue
      endelse
      
    end
    
    'DELETE TEXT': begin
    
      ; Just get the new text and update the info stucture.
      widget_control, info.textID, get_value = theText
      theText = theText[0]
      validText = input_field_validate(theText, info.dataType, decimal = info.decimal, digits = info.digits, positive = info.positive)
      
      ; Load the valid text.
      widget_control, info.textID, set_value = validText, set_text_select = [textLocation[0],0]
      info.theText = validText
      testValue  = input_field_returnvalue(info.theText, info.dataType)
      
      if string(testValue) eq "NULLVALUE" then begin
        ptr_free, info.theValue
        info.theValue = ptr_new(/Allocate_Heap)
      endif else *info.theValue = testValue
      
    end
    
    'SELECT TEXT': ; Nothing to do.
    
  endcase
  
  ; Do you report all events, or only Carriage Return events?
  if info.cr_only then begin
    if info.event_func ne "" then begin
      thisEvent = {Input_Field, info.cw_tlb, event.top, 0L, info.theValue, info.dataType}
      if cr_event then widget_control, info.cw_tlb, send_event = thisEvent
    endif
    
    if info.event_pro ne "" then begin
      thisEvent = {Input_Field, info.cw_tlb, event.top, 0L, info.theValue, info.dataType}
      if cr_event then widget_control, info.cw_tlb, send_event = thisEvent
    endif
    
  endif else begin
    if info.event_func ne "" then begin
      thisEvent = {Input_Field, info.cw_tlb, event.top, 0L, info.theValue, info.dataType}
      widget_control, info.cw_tlb, send_event = thisEvent
    endif
    
    if info.event_pro ne "" then begin
      thisEvent = {Input_Field, info.cw_tlb, event.top, 0L, info.theValue, info.dataType}
      widget_control, info.cw_tlb, Send_Event=thisEvent
    endif
    
  endelse
  
  ; Out of here.
  widget_control, event.ID, set_uvalue = info, /no_copy
  
end

;+
; :Description:
;     The purpose of this compound widget is to provide an alternative
;     to the CW_FIELD widget offered in the IDL distribution. What has
;     always annoyed me about CW_FIELD is that the text widgets do not
;     look editable to the users on Windows platforms. This program
;     corrects that deficiency and adds some features that I think
;     would be helpful. For example, you can now assign an event handler
;     to the compound widget.
;
;   All events are handled internally unless either the Event_Pro or Event_Func
;   keywords are used to assign an event handler to the compound widget. By
;   default all events generated by the text widget are passed to the assigned
;   event handler. If you wish to receive only Carriage Return events, set the
;   CR_Only keyword.
;
;   event = { INPUT_FIELD     , $ ; The name of the event structure.
;             ID: 0L          , $ ; The ID of the compound widget's top-level base.
;             TOP: 0L         , $ ; The widget ID of the top-level base of the hierarchy.
;             HANDLER: 0L     , $ ; The event handler ID. Filled out by IDL.
;             Value: Ptr_New(), $ ; A pointer to the widget value.
;             Type:""             ; A string indicating the type of data in the VALUE field.
;           }                     ; Values are "INT", "LONG", "FLOAT", "DOUBLE", or "STRING".
;
; :Params:
;    parent : in, required, type="long"
;       The parent widget ID of the compound widget
;
; :Keywords:
;    Column : in, optional, type="bool"
;       Set this keyword to have the Label Widget above the Text Widget.
;    CR_Only : in, optional, type="bool"
;       Set this keyword if you only want Carriage Return events. If
;       this keyword is not set, all events are returned. No events
;       are returned unless the EVENT_PRO or EVENT_FUNC keywords are used.
;    Digits : in, optional, type="int"
;       Set this keyword to the number of digits permitted in INTERGERVALUE and LONGVALUE numbers.
;    Decimal : in, optional, type="int"
;       Set this keyword to the number of digits to the right of the decimal point in FLOATVALUE and DOUBLEVALUE numbers.
;    DoubleValue : in, optional, type="bool"
;       Set this keyword if you want DOUBLE values returned.
;    Event_Func : in, optional, type="str"
;       Set this keyword to the name of an Event Function. If this
;       keyword is undefined and the Event_Pro keyword is undefined,
;       all compound widget events are handled internally and not
;       passed on to the parent widget.
;    Event_Pro : in, optional, type="str"
;       Set this keyword to the name of an Event Procedure. If this
;       keyword is undefined and the Event_Func keyword is undefined,
;       all compound widget events are handled internally and not
;       passed on to the parent widget.
;    FieldFont : in, optional, type="font str"
;       The font name for the text in the Text Widget.
;    FloatValue : in, optional, type="bool"
;       Set this keyword for FLOAT values.
;    Frame : in, optional, type="bool"
;       Set this keyword to put a frame around the compound widget.
;    IntegerValue : i, optional, type="bool"
;       Set this keyword for INTEGER values.
;    LabelFont : in, optional, type="font str"
;       The font name for the text in the Label Widget.
;    LabelSize : in, optional, type="int"
;       The X screen size of the Label Widget.
;    LongValue : in, required, type="bool"
;       Set this keyword for LONG values.
;    Positive
;    Row : in, optional, type="bool"
;       Set this keyword to have the Label beside the Text Widget. (The default.)
;    Scr_XSize : in, optional, type="int"
;       The X screen size of the compound widget.
;    Scr_YSize : in, optional, type="int"
;       The Y screen size of the compound widget.
;    StringValue : in, optional, type="bool"
;       Set this keyword for STRING values. (The default.)
;    Title : in, required, type="str"
;       The text to go on the Label Widget.
;    UName : in, optional, type="str"
;       The widget uname.
;    UValue : in, optional, type="any"
;       A user value for any purpose.
;    Value : in, optional, type="any"
;       The "value" of the compound widget.
;    XSize : in, optional, type="int"
;       The X size of the Text Widget.
; :Returns:
;    The widget id.
;
; :Author: fanning
;
; :History:
;    fanning, 17 Nov. 1999
;    pellegrini, 22 févr. 2011
;
; :Categories: widgets
;
; :private:
;-
function input_field, parent, column = column, cr_only = cr_only, digits = digits, decimal = decimal        , $
    doublevalue = doublevalue, event_func = event_func, event_pro = event_pro, fieldFont = fieldfont   , $
    floatvalue = floatvalue, frame = frame, integervalue = integervalue, labelfont = labelfont         , $
    labelsize = labelsize, longvalue = longvalue, positive = positive, row = row, scr_xSize = scr_xsize, $
    scr_ySize = scr_ysize, stringvalue = stringvalue, title = title, uname = uname, uvalue = uvalue    , $
    value = value, xsize = xsize
    
  ; A parent is required.
  if n_elements(parent) eq 0 then begin
    message, 'A PARENT argument is required. Returning...', /Informational
    return, -1L
  endif
  
  ; Check keyword values.
  if n_elements(column) eq 0 then column = 0
  if n_elements(digits) eq 0 then digits = 0 else digits = fix(digits)
  if n_elements(decimal) eq 0 then decimal = -1 else decimal = fix(decimal)
  if n_elements(event_func) eq 0 then event_func = ""
  if n_elements(event_pro) eq 0 then event_pro = ""
  if n_elements(fieldfont) eq 0 then fieldfont = ""
  if n_elements(frame) eq 0 then frame = 0
  if n_elements(labelfont) eq 0 then labelfont = ""
  if n_elements(labelsize) eq 0 then labelsize = 0
  if n_elements(scr_xsize) eq 0 then scr_xsize = 0
  if n_elements(scr_ysize) eq 0 then scr_ysize = 0
  if n_elements(title) eq 0 then title = "Input Value: "
  if n_elements(uvalue) eq 0 then uvalue = ""
  if n_elements(uname) eq 0 then uname = ""
  if n_elements(value) eq 0 then value = "" else value = strtrim(value,2)
  if n_elements(xsize) eq 0 then xsize = 0
  if n_elements(row) eq 0 and column eq 0 then row = 1 else row = 0
  
  positive = keyword_set(positive)
  
  ; What data type are we looking for?
  
  dataType = 'STRING'
  if keyword_set(stringvalue) then dataType = 'STRING'
  if keyword_set(integervalue) then dataType = 'INT'
  if keyword_set(longvalue) then dataType = 'LONG'
  if keyword_set(floatvalue) then dataType = 'FLOAT'
  if keyword_set(doublevalue) then dataType = 'DOUBLE'
  
  ; Validate the input value.
  
  value = input_field_validate(value, dataType, Decimal=decimal, Digits=digits, Positive=positive)
  
  ; Create the widgets.
  
  cw_tlb = widget_base(parent, pro_set_value = 'Input_Field_Set_Value' , $
                               func_get_value = 'Input_Field_Get_Value', $
                               frame = frame                           , $
                               row = row                               , $
                               column = keyword_set(column)            , $
                               base_align_center = 1                   , $
                               uname = uname                           , $
                               uvalue = uvalue                         , $
                               event_pro = event_pro                   , $
                               event_func = event_func)
    
  labelID = widget_label(cw_tlb, value = title, font = labelfont, scr_xsize = labelsize)
    
  textID = widget_text(cw_tlb, value = value                          , $
                               xsize = xsize                          , $
                               ysize = 1                              , $
                               scr_xsize = scr_xsize                  , $
                               scr_ysize = scr_ysize                  , $
                               font = fieldfont                       , $
                               all_events = 1                         , $
                               event_pro = 'Input_Field_Event_Handler', $
                               uname = 'INFO_CARRIER'                 , $
                               kill_notify = 'Input_Field_Kill_Notify', $
                               editable = 1 )
    
  ; Set the actual return value of the compound widget.
  theValue = input_field_returnvalue(value, dataType)
  
  ; The info structure.  
  info = {theText    : value               , $ ; The text in the Text Widget.
          theValue   : ptr_new(theValue)   , $ ; The real value of the Text Widget.
          cw_tlb     : cw_tlb              , $ ; The top-level base of the compound widget.
          event_func : event_func          , $ ; The name of the event handler function.
          event_pro  : event_pro           , $ ; The name of the event handler procedure.
          cr_only    : keyword_set(cr_only), $ ; A flag to return events only on CR events.
          dataType   : dataType            , $ ; The type of data wanted in the compound widget.
          decimal    : decimal             , $ ; The number of digits in decimal numbers.
          digits     : digits              , $ ; The number of digits in integer numbers.
          positive   : positive            , $ ; Flag to indicate positive number values.
          textID     : textID }                ; The widget identifier of the Text Widget.
    
  ; Store info structure in Text Widget.    
  widget_control, textID, set_uvalue = info, /no_copy
  return, cw_tlb
  
end
