;+
; :Description:
;    Delete or undefine an IDL program variable from within an IDL program or at the 
;    IDL command line. It is a more powerful delvar.
;    Pointer and structure variables are traversed recursively to undefine any variables 
;    pointed to in the pointer or in a structure dereference.
;    
;    This code was taken from an implementation of David Fanning.
;
; :Params:
;    var : in, required, type="any"
;       The variable to be deleted.
;
; :Author: pellegrini
;
; :History:
;    Andrew Cool
;    David Fanning, 8 Jun. 1997
;    Eric Pellegrini, 14 Feb. 2011
;
; :Categories: variables manipulation
;-

pro undefine, var
   
    if n_elements(var) ne 0 then begin
   
        dataType = size(var, /TNAME)
        
        if dataType eq 'POINTER' then begin
        
            for j = 0L, n_elements(var)-1 do begin
                if ptr_valid(var[j]) then undefine, *var[j]
                ptr_free, var[j]
            endfor
            
        endif
        
        if dataType eq 'OBJREF' then obj_destroy, var
        
        if dataType eq 'STRUCT' then begin
        
            for j = 0, n_tags(var)-1 do undefine, var.(j)
            
        endif
        
        var = 0
        
        dummy = temporary(var)
        
   endif

end

;+
; :Description:
;    This function will merge two IDL array of structures.
;    
;    If s1 and s2 share some tags, it will be the ones of s2 that will be kept.
;    
; :Params:
;    s1 : in, required, type="struct"
;       The first input IDL structure.        
;    s2 : in, required, type="struct"
;       The second IDL structure.
;
; :Returns:
;     The results of the merging of the two input structures.
;     Returns -1 if something failed during the merging.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 18 Feb. 2011
;
; :Categories: variables manipulation
;-
function struct_modif, s1, s2

    ; There must be two input paramaters.
    if n_params() ne 2 then return, -1
            
    ; The input parameters must be array of structures.
    if (size(s1, /TYPE) ne 8) or (size(s2, /TYPE) ne 8) then return, -1
            
    ; The array of structures must be of the same size.
    if n_elements(s1) ne n_elements(s2) then return, -1

    ; The tag names of s1 structure.
    t1 = tag_names(s1)    

    ; The tag names of s2 structure.
    t2 = tag_names(s2)

    ; A new structure is created with the union of the tag names of s1 and s2.    
    newStruct = {}    
    for i = 0, n_elements(t1) - 1 do begin
    
      idx = where(t2 eq t1[i])
              
      if idx[0] eq -1 then newStruct = create_struct(newStruct, t1[i], s1[0].(i))
                                
    endfor
            
    for i = 0, n_elements(t2) - 1 do newStruct = create_struct(newStruct, t2[i], s2[0].(i))

    ; The output array of structures is created.                
    newStruct = replicate(newStruct, n_elements(s2))

    ; First, fills the output structure with s1.
    comp = 0    
    ; Loop over the tag names of s1.
    for i = 0, n_elements(t1) - 1 do begin
    
      ; Check whether the tag name is also a tag name of s2.
      ; If not, use the value of s1 otherwise skip it, it will be the values of s2 that will be used instead.  
      idx = where(t2 eq t1[i])                    
      if idx[0] eq -1 then begin
          newStruct.(comp) = s1.(i)
          comp = comp + 1
      endif
                                
    endfor

    ; Then fills the output structure with the values of s2.
    for i = 0, n_elements(t2) - 1 do newStruct.(comp + i) = s2.(i)
            
    ; The output structure is returned.
    return, newStruct
        
end

;+
; :Description:
;    The module constructor.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 18 Feb. 2011
;
; :Categories: web utilities
;-
pro variables_utilities

end
