function elascan_abscor_correct, w_in, w_bg, w_corr = w_corr, fBS = fBS, transm = transm
;+
;******* ************
;** User callable
;*******************************************************************************
;**
;** Background subtraction for an elastic (inelastic) scan
;** Equivalent to background_abscor_correct, but taking into account that
;** w_in and w_bg may have different dimensions and that needs to subtract
;** the closest temperature.
;**
;** MAG (12/4/2015)
;**
;**
;*******************************************************************************
;-

;-------------------------------------------------------------------------------
; Get sizes and check dimensions and values

s_in=size(w_in)
if (s_in[0] ne 2) then begin
  print
  print, ' Data workspace is not a 2D array. Not valid input!'
  print, ' NOTHING DONE!!!'
  return, w_in
endif

take_datp, sample  ;take parameters corresponding to w_in
x_out = sample.x ;temperatures
y_out = sample.y ;angles
w_out = w_in
if n_elements(sample.e) eq n_elements(w_in) then e_out = sample.e else e_out = sqrt(w_in) ;errors

if n_elements(w_bg) ne 0 then begin ;check background dimensions and get parameters
  
  s_bg = size(w_bg)
  
  if (s_bg[0] eq 1) then begin
    
    print, ' Background measurement contains a single temperature!'
    if s_bg[1] ne s_in[2] then begin
      print, ' The number of angles (X axis) in background workspace is different from '
      print, ' the number of angles (Y axis) in the data workspace!'
      print, ' NOTHING DONE!!!'
      return, w_in
    endif
    take_datp, bgr, /third
    angles_bg = bgr.x ;angles
    if n_elements(bgr.e) eq n_elements(w_bg) then e_bg = bgr.e else e_bg = sqrt(w_bg) ;errors
   
    ;replicate single measurement to data dimensions
    bgrd = transpose(rebin(w_bg, s_bg[1], s_in[1]))
    e_bg = transpose(rebin(e_bg, s_bg[1], s_in[1]))
    
  endif else begin
    
    if (s_bg[2] ne s_in[2]) then begin
      print, ' The number of angles (Y axis) in background workspace is different from '
      print, ' the number of angles (Y axis) in the data workspace!'
      print, ' NOTHING DONE!!!'
      return, w_in
    endif
    take_datp, bgr, /third
    temp_bg = bgr.x   ;temperatures
    angles_bg = bgr.y ;angles
    if n_elements(bgr.e) eq n_elements(w_bg) then e_bg = bgr.e else e_bg = sqrt(w_bg) ;errors

    ;interpolate background temperatures to sample temperatures
    xvalues = interpol(findgen(n_elements(temp_bg)), temp_bg, x_out)
    yvalues = interpol(findgen(n_elements(angles_bg)), angles_bg, y_out)
    bgrd = interpolate(w_bg, xvalues, yvalues, /grid)
    e_bg = interpolate(e_bg, xvalues, yvalues, /grid)
    
  endelse
  
endif else begin
  print, ' No background given. Only SSF scaling will be applied'
endelse

if n_elements(fBS) ne 0 then begin
  if fBS lt 0.0 then begin & print, 'fBS cannot be < 0' & fBS = 0.0 & endif
  if fBS gt 1.0 then begin & print, 'fBS cannot be > 1' & fBS = 1.0 & endif
endif

if n_elements(transm) ne 0 then begin
  if transm lt 0.0 then begin & print, 'transmission cannot be < 0' & transm = 0.0 & endif
  if transm gt 1.0 then begin & print, 'transmission cannot be > 1' & transm = 1.0 & endif
endif


;-------------------------------------------------------------------------------
; Get absorption correction factors and interpolate to sample angles/energies

if n_elements(w_corr) gt 0 then begin

  take_w, corr, W=w_corr
  take_datp, corr_params, W=w_corr
  sw = size(corr)

  case sw[0] of

    1: begin ;Only SSF(2-theta) given
      ssf_values = corr
      saf_values = corr*0.0 + 1.0
      ssf_angles = corr_params.x
    end

    2: begin ;SSF(2-theta) and SAF(2-theta) given
      ssf_values = corr[*,0]
      saf_values = corr[*,1]
      ssf_angles = corr_params.x
    end

    3: begin ;SSF(2-theta,w) and SAF(2-theta,w) given
      ssf_values = corr[*,*,0]
      saf_values = corr[*,*,1]
      ssf_angles = corr_params.x
      ssf_energies = corr_params.y
    end

  endcase

  ;Interpolate to scattering angles and if necessary to energy = 0
  if n_elements(ssf_energies) eq 0 then begin

    print, 'No w dependence on correction factor. Interpolating to experimental angles ...'
    print, 'Correction file --> 2-theta range     = ', min(ssf_angles), max(ssf_angles)
    print, 'Experimental angles --> 2-theta range = ', min(y_out), max(y_out)
    print
    ssf = interpol(ssf_values, ssf_angles, y_out)
    saf = interpol(saf_values, ssf_angles, y_out)
    ssf = transpose(rebin(ssf, n_elements(y_out), n_elements(x_out)))
    saf = transpose(rebin(saf, n_elements(y_out), n_elements(x_out)))

  endif else begin

    print, 'Interpolating to experimental angles and energy = 0.'
    print, 'Correction file -->     2-theta range = ', min(ssf_angles), max(ssf_angles)
    print, 'Experimental angles --> 2-theta range = ', min(y_out), max(y_out)
    print
    print, 'Correction file -->       w range = ', min(ssf_energies), max(ssf_energies)
    print, 'Experimental energies --> w range = ', 0.0
    print
    xvalues = interpol(findgen(n_elements(ssf_energies)), ssf_energies, [0.0])
    yvalues = interpol(findgen(n_elements(ssf_angles)), ssf_angles, y_out)
    ssf = interpolate(transpose(ssf_values), xvalues, yvalues, /grid)
    saf = interpolate(transpose(saf_values), xvalues, yvalues, /grid)
  endelse

endif


;-------------------------------------------------------------------------------
; Correct data (different options depending on existence of w_corr and w_bg)

if n_elements(w_corr) eq 0 then begin

  ;if transmission is not given, set to 1
  if n_elements(transm) eq 0 then transm = 1.0

  ;No absorption correction - Check if background / backscattering att factor given.
  if n_elements(bgrd) ne 0 then begin

    w_out = w_in - transm*bgrd
    e_out = sqrt(e_out^2 + (transm*e_bg)^2)

    if n_elements(fBS) ne 0 then begin
      print, ' Subtracting background (no abscor) and applying backscattering attenuation factor '
      att = (1.0-fBS) + fBS * transm
      w_out = w_out / att
      e_out = e_out / att
    endif else begin
      print, ' Subtracting background (no abscor) only '
    endelse

  endif else begin ;No background given

    if n_elements(fBS) ne 0 then begin
      att = (1.0-fBS) + fBS * transm
      w_out = w_out / att
      e_out = e_out / att
      print, ' No background given. Applying only backscattering attenuation factor = ', att
    endif else begin
      print, ' Nothing to do!, Returning input workspace'
    endelse

  endelse

endif else begin ;Subtract background using SSF and SAF factors

  ;if transmission is not given, approximate as SSF(2-theta~0,w~0)
  if n_elements(transm) eq 0 then begin
    wmin = min(abs(x_out), x_index)
    tmin = min(abs(y_out), y_index)
    transm = ssf[x_index, y_index]
  endif

  if n_elements(bgrd) eq 0 then begin ;No background to subtract. Only SSF applied.

    w_out = w_out / ssf
    e_out = e_out / ssf

    if n_elements(fBS) ne 0 then begin
      print, ' No background. Applying SSF and backscattering attenuation factors '
      att = (1.0-fBS) + fBS * transm
      w_out = w_out / att
      e_out = e_out / att
    endif else begin
      print, ' No background. Applying SSF attenuation factor only '
    endelse

  endif else begin

    w_out = ( w_out - saf * bgrd ) / ssf
    e_out = sqrt(e_out^2 + (saf*e_bg)^2) / ssf

    if n_elements(fBS) ne 0 then begin
      print, ' Background subtracted applying SAF + SSF and backscattering attenuation factors '
      att = (1.0-fBS) + fBS * transm
      w_out = w_out / att
      e_out = e_out / att
    endif else begin
      print, ' Background subtracted applying SAF + SSF attenuation factor  '
    endelse

  endelse

endelse


;-------------------------------------------------------------------------------
; Return corrected data

mod_datp, sample, 'X', x_out
mod_datp, sample, 'Y', y_out
mod_datp, sample, 'E', e_out
give_datp, sample
return, w_out

end
