;+
;*******************************************************************************
;
; Evaluation of correction coefficients in the case of a cylindrical sample.
; Both absorption and self shielding are taken into account.
; Self-shielding factors for each region are computed using a Montecarlo method
; by adapting the routines of R.M.Dimeo and John Copley.
;
; Call w6=elascan_cyl (sample_w=1, background_w=2, Rin=10.0, Rout=11.0, t_sample=0.90, t_container=0.99,$
;                      thick1=0.1, thick2=0.1, twice=0.5, f=1.0, nmc=10000, corr_w=20)
;
; The corrected data will then be stored in w6. Additionally an ASCII file called 'correction_factors.dat' is written.
; This file contains the self-shielding factors Assc, Acsc, and Acc applied in the correction. Another ASCII file
; named 'backthrough.dat' containing the attenuation factor applied to the neutrons passing a second time through
; the sample is also written.
; If the background workspace contains more than one temperature measurement, the subtracted background is that
; measured at the closest T at which the sample has been measured. A file 'temperatures.dat' is also written.
; It contains a list of the sample temperatures and the corresponding background temperature used in the subraction.
;
; MAG (04/5/2009)
;
;
;*******************************************************************************
;-

pro insect,nann,rann,x0,y0,u,v,dism,disp,nint
;+
;** ******
;** Calculates the intersections of a directed line with a set of
;** concentric circles centered at the origin.  The number of circles
;** is nann and their radii, in increasing order, are (rann(i),i=1,nann).
;** The line goes through the point (x0,y0) and its direction is (u,v).
;** The calculated distances to intersection with circle k are dism(k),
;** disp(k), such that dism(k).le.disp(k).  If there is no intersection
;** both quantities are set equal to zero.  The number of intersections is
;** nint.
;
;-
front = -x0*u-y0*v
disc0 = front^2-x0^2-y0^2
disc = (disc0+rann[0:nann-1]^2) > 0.0
root = sqrt(disc)
dism = front-root
disp = front+root
ind = where(disc gt 0.0,nint)
nint = nint*2
return
end


pro ninsect,nann,rann,x0,y0,nphi,u,v,dism,disp,nint
;+
;** *******
;** Calculates the intersections of nphi directed lines with a set of
;** concentric circles centered at the origin.  The number of circles
;** is nann and their radii, in increasing order, are (rann(i),i=1,nann).
;** The lines go through the point (x0,y0) and their directions are (u,v).
;** The calculated distances to intersection with circle k are dism(k),
;** disp(k), such that dism(k).le.disp(k).  If there is no intersection
;** both quantities are set equal to the same value.
;** The numbers of intersections are nint.
;
;-
front = (-x0*u-y0*v)##(intarr(nann)+1)
disc0 = front^2-x0^2-y0^2
disc = (disc0+(intarr(nphi)+1)##(rann^2)) > 0.0
root = sqrt(disc)
dism = front-root
disp = front+root
nint = 2*round(total(disc gt 0.0,2))
return
end


pro nout_calc,nann,dism,disp,sigma,nphi,arg_out
;+
;** *********
;** calculates the attenuation from scattering point to all detectors
;-

dm=dism>0
dp=disp>0
sg=sigma[0:nann-1]#(intarr(nphi)+1)
s1=total((dp-dm)*sg,1)
ssg=shift(sg,-1)
s2=total((dp-dm)*ssg,1)
s3=(dp[nann-1,*]-dm[nann-1,*])*ssg[nann-1]
arg_out=s1-s2+s3
return
end


pro calc_area,rann,d,area
;+
;** *********
;** elascan_cyc stuff
;-
nann=n_elements(rann)
area=fltarr(nann)
if (d eq 0) then return
sd=round(d/abs(d))
ad=abs(d)
for k=0,nann-1 do begin
   if (ad gt rann[k]) then begin
     area[k]=0.5*!pi*rann[k]^2
   endif else begin
     cth=ad/rann[k]
     sth=sqrt(1.0-cth^2)
     theta=acos(cth)
     area[k]=(0.5*!pi-theta+cth*sth)*rann[k]^2
   endelse
   if (k gt 0) then area[k]=area[k]-total(area[0:k-1])
endfor
area=sd*area
end


function elascan_cyl, w_data, background_w=n_container, Rin=Rin, Rout=Rout, t_sample=ts, $
                       t_container=tc, thick1=thick1, thick2=thick2, twice=back, $
                       f=f, nmc=nmc, corr_w=n_corr
;+
;******* ***********
;** User callable
;-


; ------------------------------------------------------------------------------
;Read data and background workspaces (for background use only first spectrum at the moment)

;take sample data
take_datp, datp & x_data=datp.x & y_data=datp.y & e_data=datp.e & n_data=datp.n & par = datp.p

!except=0
print
; check dimensions
sw=n_elements(w_data) & se=n_elements(e_data)
sx=n_elements(x_data) & sy=n_elements(y_data) & sn=n_elements(n_data)
if (sw ne se) then begin & print,'Elascan_cyl: Problem! w & e do not have the same number of elements' & return,1 & endif
if (sx*sy ne sw) then begin & print,'Elascan_cyl: Problem! Incompatible sizes of x, y and w' & return,1 & endif
if (sx ne sn) then begin & print,'Elascan_cyl: Problem! x & n do not have the same dimensions' & return,1 & endif
print,'Elascan_cyl:'
print,' Number of temperatures in sample measurement = ',sx

if n_elements(n_container) gt 0 then begin
   ;get background data
   take_datp, datb, W=n_container & take_w,w_background,W=n_container
   par = datb.p
   sb = size(w_background)
   if (sb[2] ne sy) then begin
      print,'Elascan_cyl:'
      print,' ERROR! Different number of detectors in sample and background measurements'
      return,0
   endif
   x_background = datb.x  ;temperatures
   y_background = datb.y  ;scattering angles
   e_background = datb.e
   n_background = datb.n
   svx=n_elements(x_background)
   print,' Number of temperatures in background measurement = ',svx
endif else begin
   print,'Elascan_cyl:'
   print,' No background measurement given!'
   print,' Applying self shielding corrections to sample measurement.'
endelse

w_out = w_data
e_out = e_data
x_out = x_data
y_out = y_data


; ------------------------------------------------------------------------------
; Check input

if (n_elements(ts) eq 0) then ts=1.0
if (n_elements(tc) eq 0) then tc=1.0
if (n_elements(back) eq 0) then back=0.0
if (n_elements(f) eq 0) then f=1.0

if ((ts le 0.0) or (ts gt 1.0)) then begin
   print,'Elascan_cyl:'
   print,' ERROR! Sample transmission must be between 0 and 1'
   return,0
endif

if ((tc le 0.0) or (tc gt 1.0)) then begin
   print,'Elascan_cyl:'
   print,' ERROR! Container transmission must be between 0 and 1'
   return,0
endif

if ((back lt 0.0) or (back gt 1.0)) then begin
   print,'Elascan_cyl:'
   print,' ERROR! Backthrough fraction must be betwen 0 and 1'
   return,0
endif

; ------------------------------------------------------------------------------
;  Sample is cylinder (hollow)
; ------------------------------------------------------------------------------
if keyword_set(Rin) && keyword_set(Rout) then begin
   print,'Sample is HOLLOW CYLINDER'
   shape = 'hollow'
endif
if keyword_set(Rin) && ~keyword_set(Rout) then begin
   print,'Elascan_cyl:'
   print,' ERROR! Rout is missing for HOLLOW CYLINDER.'
   return,0
endif

; ------------------------------------------------------------------------------
;  Sample is cylinder (full)
; ------------------------------------------------------------------------------
if ~keyword_set(Rin) && keyword_set(Rout) then begin
  print,'Sample is FULL CYLINDER'
  shape = 'full' & Rin = 0.0
endif

if ~keyword_set(shape) then begin
  print,'Elascan_cyl:'
  print,'ERROR! Container type unknown, check parameters!'
  return,0
endif

if ~keyword_set(nmc) then begin
   print,'Elascan_cyl:'
   print,'WARNING: Number of MC events not given. Set to 10000!'
   nmc = 1e4
endif
if (nmc gt 32767) then begin
   print,'Elascan_cyl:'
   print,'WARNING: Maximum number of MC events is 32767. nmc set to this value!'
   nmc = 32767
endif

if (shape eq 'hollow') then begin
   if ~keyword_set(thick1) then begin
      if n_elements(n_container) gt 0 then begin
         print,'Elascan_cyl:'
         print,'WARNING: No thickness given for inner cylinder of container. Value set to 0.1 mm!'
         thick1 = 0.1
      endif else thick1 = 0.0
   endif
endif
if ~keyword_set(thick2) then begin
   if n_elements(n_container) gt 0 then begin
      print,'Elascan_cyl:'
      print,'WARNING: No thickness given for outer cylinder of container. Value set to 0.1 mm!'
      thick2 = 0.1
   endif else thick2 = 0.0
endif


; ------------------------------------------------------------------------------
; Define number of regions

if (Rin le 0.0) then begin
   if (thick2 le 0.0) then begin
         ;sample without container
         ncase = 0 & nann = 1 & radius = dblarr(nann) & tk =dblarr(nann)
         radius[0] = Rout & tk[0] = ts ;sample
      endif else begin
         ;sample in a container
         ncase = 1 & nann = 2 & radius = dblarr(nann) & tk =dblarr(nann)
         radius[0] = Rout & tk[0] = ts ;sample
         radius[1] = Rout + thick2 & tk[1] = tc ;container
      endelse
endif else begin
   if ((Rin-thick1) le 0.0) then begin
      ;bulk inner cylinder
      ncase = 2 & nann = 3 & radius = dblarr(nann) & tk =dblarr(nann)
      radius[0] = Rin & tk[0] = tc ;inner cylinder
      radius[1] = Rout & tk[1] = ts ;sample
      radius[2] = Rout + thick2 & tk[2] = tc ;outer cylinder
   endif else begin
      ;hollow inner cylinder
      ncase = 3 & nann = 4 & radius = dblarr(nann) & tk =dblarr(nann)
      radius[0] = Rin - thick1 & tk[0] = 1.0;empty space
      radius[1] = Rin & tk[1] = tc ;inner cylinder
      radius[2] = Rout & tk[2] = ts ;sample
      radius[3] = Rout + thick2 & tk[3] = tc ;outer cylinder
   endelse
endelse

;Declare arrays
wid = radius[nann-1]*2.0 ;beam width
hwid = wid*0.5
print
print,'Beam width used = ',wid,' [L]'
print,'WARNING: This code assumes that the beam has exactly the dimensions of the sample+container.'
print,'If this is not the case, the macro should be slightly modified!'

;phi: detector angles --> makes the calculation between 0 and 359 degrees every 1 degree
phi = findgen(360)
phi1 = min(phi)
phi2 = max(phi)
nphi = n_elements(phi)
cphi     = dblarr(nphi)
sphi     = dblarr(nphi)
dis_out  = dblarr(nphi,3)
sum    = dblarr(nphi)
ttotal    = dblarr(nann,nphi)
etotal    = dblarr(nann,nphi)
ssf    = dblarr(nann,nphi)
esf    = dblarr(nann,nphi)
ssf0    = dblarr(nann,nphi)
esf0    = dblarr(nann,nphi)

dism_in  = dblarr(nann)
disp_in  = dblarr(nann)
dism_out    = dblarr(nann)
disp_out    = dblarr(nann)
tot0     = dblarr(nann)
etot0      = dblarr(nann)
path     = dblarr(nann)
attenh    = dblarr(nann)
area   = dblarr(nann)
prob1    = dblarr(nann)
prob2    = dblarr(nann)

phird = dblarr(nphi)
phird = phi*!dtor
cphi = cos(phird)
sphi = sin(phird)


y_min = -hwid
y_max = +hwid
y_wid = y_max - y_min

prand=ptr_new(randomu(100l,nmc,1+2*nann)) ;generate same array of random numbers

; ------------------------------------------------------------------------------
;Loop over y-coordinate to determine the path length taken by the transmitted beam
;in each region and estimate the total scattering section (in cm-1) for each of them.
;Using only y=0 to estimate total scattering section, so the transmission value
;given in the call to the routine should be the transmission along the diameter.

ny=1
y = dblarr(ny)
dy = y_wid/ny
;y = y_min+0.5*dy+dy*dindgen(ny)
y=0.
totpath = dblarr(nann)
totsigma = dblarr(nann)
cumpath = 0.d
for iy = 0,ny-1 do begin
   ; calculate distances from the origin to inter-region interfaces
   insect,nann,radius,0.d,y[iy],1.d,0.d,dism_in,disp_in,nint
   ; calculate contribution to paths for this value of y
   for k = 0,nann-1 do begin
      if (k eq 0) then begin
         path[0] = disp_in[0]-dism_in[0]
         cumpath = path[0]
      endif else begin
         path[k] = disp_in[k] - dism_in[k] - cumpath
         cumpath = cumpath+path[k]
      endelse
   endfor
   totpath = totpath + path
endfor
totpath = totpath / ny

sigma = dblarr(nann)
case ncase of
   0: begin ;only sample
         ssample = -alog(ts)/totpath[0]
         sigma[0] = ssample
      end
   1: begin ;sample + container
         ssample = -alog(ts)/totpath[0]
         scont = -alog(tc)/totpath[1]
         sigma[0] = ssample
         sigma[1] = scont
      end
   2: begin ;container+sample + container
         ssample = -alog(ts)/totpath[1]
         scont = -alog(tc)/(totpath[0]+totpath[2])
         sigma[0] = scont
         sigma[1] = ssample
         sigma[2] = scont
      end
   3: begin ;empty+container+sample + container
         ssample = -alog(ts)/totpath[2]
         scont = -alog(tc)/(totpath[1]+totpath[3])
         sigma[0] = 0.0
         sigma[1] = scont
         sigma[2] = ssample
         sigma[3] = scont
      end
   else: begin
         print,'Strange problem: Case not valid!'
         return,0
         end
endcase

print
print,'Number of scattering regions = ',nann
for k = 0,nann-1 do print,'Region ',k,':      Radius = ',radius[k],' [L]    Sigma = ',sigma[k], ' [L]^-1.'
print

; ------------------------------------------------------------------------------
; Monte Carlo loop for sample + container
progress = nmc/20
;prand=ptr_new(randomu(100l,nmc,1+2*nann)) ;generate same array of random numbers
prand=ptr_new(randomu(unknown,nmc,1+2*nann))
trans = 0.0
cumpath = 0.d

for imc = 0,nmc-1 do begin

   if ((imc mod progress) eq 0) then print,'MC loop S+C, progress = ',fix(100.*float(imc)/float(nmc)),'%'

   y=y_min+y_wid*(*prand)[imc,0]

   ; calculate distances from the origin to inter-region interfaces
   insect,nann,radius,0.d,y,1.d,0.d,dism_in,disp_in,nint

   ; calculate contribution to transmission for this value of y
   for k = 0,nann-1 do begin
      if (k eq 0) then begin
         path[0] = disp_in[0]-dism_in[0]
         cumpath = path[0]
         endif else begin
         path[k] = disp_in[k] - dism_in[k] - cumpath
         cumpath = cumpath+path[k]
      endelse
      attenh[k] = exp(-sigma[k]*path[k]/2.d)
   endfor

   prob_thru=1.d0
   for k=nann-1,0,-1 do begin
      prob1[k]=prob_thru*(1.d0-attenh[k])
      prob_thru=prob_thru*attenh[k]
   endfor
   for k=0,nann-1 do begin
      prob2[k]=prob_thru*(1.d0-attenh[k])
      prob_thru=prob_thru*attenh[k]
   endfor
   trans=trans+prob_thru

   ; calculation for scattering points within each region
   for k = 0,nann-1 do begin

      ; only do calculation if region has sigma>0 and neutron actually enters region
      if (sigma[k])*path[k] gt 0.0 then begin

         ; pick a distance between 0 and 0.5*path[k]
         dis=-alog(1.d0-(*prand)[imc,k+1]*(1.d0-attenh[k]))/sigma[k]

         ; decide between the two parts of the trajectory in region k. hence x.
         prob12=prob1[k]+prob2[k]
         if ((*prand)[imc,k+nann+1] lt prob1[k]/prob12) then begin
            x=dism_in[k]+dis
         endif else begin
            if (k eq 0) then x=dis else x=disp_in[k-1]+dis
         endelse

         ; for each scattering angle calculate contribution to SSF
         ninsect,nann,radius[0:nann-1],x,y,nphi,cphi,sphi,dism_out,disp_out,nint
         nout_calc,nann,dism_out,disp_out,sigma,nphi,arg_out
         term=prob12*exp(-arg_out)
         ttotal[k,*] = ttotal[k,*] + term
         etotal[k,*] = etotal[k,*] + term^2

      endif

      tot0[k] = tot0[k] + path[k]
      etot0[k] = etot0[k] + path[k]^2

   endfor

endfor

ptr_free,prand

trans = trans/nmc
s1=total(radius*sigma)
ssg=shift(sigma,-1)
s2=total(radius*ssg)
s3=(radius[nann-1])*ssg[nann-1]
arg_out=s1-s2+s3
transc=exp(-2.0*arg_out)
print,'Transmission S+C (average) = ',trans
print,'Transmission S+C (diameter) = ',transc

ttotal=ttotal/nmc; "<x>"
etotal=etotal/nmc; "<x^2>"
etotal=sqrt((etotal-ttotal^2)/(nmc-1)); "std dev=sqrt((<x^2>-<x>^2)/(nmc-1))"

for k = 0,nann-1 do begin
   for j = 0,nphi-1 do begin
      ttotal[k,j] = ttotal[k,j]*y_wid/sigma[k]
      etotal[k,j] = etotal[k,j]*y_wid/sigma[k]
   endfor
endfor

tot0=tot0/nmc; "<x>"
etot0=etot0/nmc; "<x^2>"
etot0=sqrt((etot0-tot0^2)/(nmc-1)); "std dev=sqrt((<x^2>-<x>^2)/(nmc-1))"
tot0=tot0*y_wid
etot0=etot0*y_wid

calc_area,radius[0:nann-1],y_max,area1
calc_area,radius[0:nann-1],y_min,area2
area=area1-area2

for j = 0,nphi-1 do begin
   for k = 0,nann-1 do begin
      ssf[k,j] = 0.0
      esf[k,j] = 0.0
      if sigma[k] gt 0.d then begin
         ssf[k,j] = ttotal[k,j]/area[k]
         esf[k,j] = etotal[k,j]/area[k]
      endif
   endfor
endfor

;print ssf
openw, ssf_lun, 'ssf1_raw.dat', /get_lun
printf, ssf_lun, 'Angle, SSF_reg1, SSF_reg2, ...'
for j = 0,nphi-1 do begin
   printf, ssf_lun, phi[j],ssf[*,j]
endfor
close, ssf_lun
free_lun, ssf_lun
;smooth ssf
ssf_smoothed = ssf
for k = 0,nann-1 do begin
   ssf_smoothed[k,*] = smooth(ssf[k,*],30)
endfor
ssf = ssf_smoothed
;print ssf
openw, ssf_lun, 'ssf1.dat', /get_lun
printf, ssf_lun, 'Angle, SSF_reg1, SSF_reg2, ...'
for j = 0,nphi-1 do begin
   printf, ssf_lun, phi[j],ssf[*,j]
endfor
close, ssf_lun
free_lun, ssf_lun

; ------------------------------------------------------------------------------
; Monte Carlo loop for empty container
progress = nmc/20
;prand=ptr_new(randomu(100l,nmc,1+2*nann)) ;generate same array of random numbers
prand=ptr_new(randomu(unknown,nmc,1+2*nann))
trans = 0.0
cumpath = 0.d
tot0[*] = 0.
etot0[*] = 0.
ttotal[*,*] = 0.
etotal[*,*] = 0.
case ncase of
   0: ssf0[*] = 0. ;no container
   1: begin ;sample + container
         sigma[0] = 0.0
         sigma[1] = scont
      end
   2: begin ;container+empty + container
         sigma[0] = scont
         sigma[1] = 0.0
         sigma[2] = scont
      end
   3: begin ;empty+container+empty + container
         sigma[0] = 0.0
         sigma[1] = scont
         sigma[2] = 0.0
         sigma[3] = scont
      end
endcase

if (ncase gt 0) then begin

   for imc = 0,nmc-1 do begin

      if ((imc mod progress) eq 0) then print,'MC loop EC, progress = ',fix(100.*float(imc)/float(nmc)),'%'

      y=y_min+y_wid*(*prand)[imc,0]

      ; calculate distances from the origin to inter-region interfaces
      insect,nann,radius,0.d,y,1.d,0.d,dism_in,disp_in,nint

      ; calculate contribution to transmission for this value of y
      for k = 0,nann-1 do begin
         if (k eq 0) then begin
            path[0] = disp_in[0]-dism_in[0]
            cumpath = path[0]
         endif else begin
            path[k] = disp_in[k] - dism_in[k] - cumpath
            cumpath = cumpath+path[k]
         endelse
         attenh[k] = exp(-sigma[k]*path[k]/2.d)
      endfor

      prob_thru=1.d0
      for k=nann-1,0,-1 do begin
         prob1[k]=prob_thru*(1.d0-attenh[k])
         prob_thru=prob_thru*attenh[k]
      endfor
      for k=0,nann-1 do begin
         prob2[k]=prob_thru*(1.d0-attenh[k])
         prob_thru=prob_thru*attenh[k]
      endfor
      trans=trans+prob_thru

      ; calculation for scattering points within each region
      for k = 0,nann-1 do begin

         ; only do calculation if region has sigma>0 and neutron actually enters region
         if (sigma[k])*path[k] gt 0.0 then begin

            ; pick a distance between 0 and 0.5*path[k]
            dis=-alog(1.d0-(*prand)[imc,k+1]*(1.d0-attenh[k]))/sigma[k]

            ; decide between the two parts of the trajectory in region k. hence x.
            prob12=prob1[k]+prob2[k]
            if ((*prand)[imc,k+nann+1] lt prob1[k]/prob12) then begin
               x=dism_in[k]+dis
            endif else begin
               if (k eq 0) then x=dis else x=disp_in[k-1]+dis
            endelse

            ; for each scattering angle calculate contribution to SSF
            ninsect,nann,radius[0:nann-1],x,y,nphi,cphi,sphi,dism_out,disp_out,nint
            nout_calc,nann,dism_out,disp_out,sigma,nphi,arg_out
            term=prob12*exp(-arg_out)
            ttotal[k,*] = ttotal[k,*] + term
            etotal[k,*] = etotal[k,*] + term^2

         endif

         tot0[k] = tot0[k] + path[k]
         etot0[k] = etot0[k] + path[k]^2

      endfor

   endfor

   ptr_free,prand

   trans = trans/nmc
   s1=total(radius*sigma)
   ssg=shift(sigma,-1)
   s2=total(radius*ssg)
   s3=(radius[nann-1])*ssg[nann-1]
   arg_out=s1-s2+s3
   transc=exp(-2.0*arg_out)
   print,'Transmission EC (average) = ',trans
   print,'Transmission EC (diameter) = ',transc

   ttotal=ttotal/nmc; "<x>"
   etotal=etotal/nmc; "<x^2>"
   etotal=sqrt((etotal-ttotal^2)/(nmc-1)); "std dev=sqrt((<x^2>-<x>^2)/(nmc-1))"

   for k = 0,nann-1 do begin
      for j = 0,nphi-1 do begin
         ttotal[k,j] = ttotal[k,j]*y_wid/sigma[k]
         etotal[k,j] = etotal[k,j]*y_wid/sigma[k]
      endfor
   endfor

   tot0=tot0/nmc; "<x>"
   etot0=etot0/nmc; "<x^2>"
   etot0=sqrt((etot0-tot0^2)/(nmc-1)); "std dev=sqrt((<x^2>-<x>^2)/(nmc-1))"
   tot0=tot0*y_wid
   etot0=etot0*y_wid

   calc_area,radius[0:nann-1],y_max,area1
   calc_area,radius[0:nann-1],y_min,area2
   area=area1-area2

   for j = 0,nphi-1 do begin
      for k = 0,nann-1 do begin
         ssf0[k,j] = 0.0
         esf0[k,j] = 0.0
         if sigma[k] gt 0.d then begin
            ssf0[k,j] = ttotal[k,j]/area[k]
            esf0[k,j] = etotal[k,j]/area[k]
         endif
      endfor
   endfor

   ;print ssf
   openw, ssf_lun, 'ssf2_raw.dat', /get_lun
   printf, ssf_lun, 'Angle, SSF_reg1, SSF_reg2, ...'
   for j = 0,nphi-1 do begin
      printf, ssf_lun, phi[j],ssf0[*,j]
   endfor
   close, ssf_lun
   free_lun, ssf_lun
   ;smooth ssf
   ssf_smoothed = ssf0
   for k = 0,nann-1 do begin
      ssf_smoothed[k,*] = smooth(ssf0[k,*],30)
   endfor
   ssf0 = ssf_smoothed
   ;print ssf
   openw, ssf_lun, 'ssf2.dat', /get_lun
   printf, ssf_lun, 'Angle, SSF_reg1, SSF_reg2, ...'
   for j = 0,nphi-1 do begin
      printf, ssf_lun, phi[j],ssf0[*,j]
   endfor
   close, ssf_lun
   free_lun, ssf_lun

endif


;  Store ssf factors in a workspace
if (n_elements(n_corr) gt 0) then begin
   datc={w_tit:'SSF[k]', x:phi, y:findgen(2*nann), y_tit:'SSFs S+C - SSFs EC', x_tit: 'Angle'}
   give_datp, datc, W=n_corr
   corr=fltarr(360,2*nann)
   for k=0,nann-1 do begin
      corr[*,k] = transpose(ssf[k,*])
   endfor
   for k=0,nann-1 do begin
      corr[*,nann+k] = transpose(ssf0[k,*])
   endfor
   give_w,corr,W=n_corr
endif


;phi: Interpolate ssf values to current detector angles
phi_s = y_data
ssf_s = fltarr(nann,sy) & esf_s = fltarr(nann,sy)
for k = 0,nann-1 do begin
   ssf_s[k,*] = interpol(ssf[k,*],phi,phi_s)
   esf_s[k,*] = interpol(esf[k,*],phi,phi_s)
endfor
;print ssf
openw, ssf_lun, 'ssf_sample.dat', /get_lun
printf, ssf_lun, 'Angle, SSF_reg1, SSF_reg2, ...'
for j = 0,sy-1 do begin
   printf, ssf_lun, phi_s[j],ssf_s[*,j]
endfor
close, ssf_lun
free_lun, ssf_lun
if ncase gt 0 then begin
   ssf0_s = fltarr(nann,sy) & esf0_s = fltarr(nann,sy)
   for k = 0,nann-1 do begin
      ssf0_s[k,*] = interpol(ssf0[k,*],phi,phi_s)
      esf0_s[k,*] = interpol(esf0[k,*],phi,phi_s)
   endfor
   ;print ssf
   openw, ssf_lun, 'ssf_ec.dat', /get_lun
   printf, ssf_lun, 'Angle, SSF_reg1, SSF_reg2, ...'
   for j = 0,sy-1 do begin
      printf, ssf_lun, phi_s[j],ssf0_s[*,j]
   endfor
   close, ssf_lun
   free_lun, ssf_lun
endif

; ------------------------------------------------------------------------------
;Return parameters and exit

datp.e = e_out ; mod_datp,datp,'E',e_out
give_datp, datp
RETURN, w_out

END
