function elascan_groupp, w_in, dP=dP, average=average, sort=srt
;+
;******* **************
;** User callable
;*******************************************************************************
;**
;** Group points of an elastic scan having the same pressure (y+-dP)
;**
;** Call: w2=elascan_groupp(w1)
;**
;** Spectra are added unless average is set.
;** If input spectra are not sorted you can use the /sort keyword
;-

COMMON c_lamp_access, inst

if strupcase(inst) ne 'IN13' then begin
  print, 'Elascan_groupp only available for IN13!'
  print, 'For the moment, pressures are not read/available for other instruments.'
  return, w_in
endif

if n_elements(dP) eq 1 then d=dP else d=0.0

;take sample data
take_datp, datp & x=datp.x & y=datp.y & e=datp.e & n=datp.n & par = datp.p
w_tmp = w_in

print
; check all dimensions are OK
sx=n_elements(x) & sy=n_elements(y) & sw=n_elements(w_tmp) & se=n_elements(e) & sn=n_elements(n)
if (sw ne se) then begin & print,'Elascan_groupp: Problem! w & e do not have the same number of elements' & return,1 & endif
if (sx*sy ne sw) then begin & print,'Elascan_groupp: Problem! Incompatible sizes of x, y and w' & return,1 & endif
if (sx ne sn) then begin & print,'Elascan_groupp: Problem! x & n do not have the same dimensions' & return,1 & endif
print,'Elascan_groupp:'
print,' Note that X axis will become the pressure axis. Temperatures are not used and could be summed together if same pressure!'
print,' Number of numors/pressures before grouping = ',sx

; Get pressures (default/minimum value is 1 bar) & change X axis from T to P
pressure = (datp.pv[7,*] > 1.0) & x = transpose(pressure[0,*])
datp.x_tit = 'p/bar'

; Sort pressures?
if keyword_set(srt) then begin & idx=sort(x) & x=x[idx] & w_tmp=w_tmp[idx,*] & e=e[idx,*] & n=n[idx] & endif

; Prepare output
flg =lonarr(sx)+1
w_out=w_tmp*0 & w_out[0,*]=w_tmp[0,*]
e_out=e*0 & e_out[0,*]=e[0,*]^2 ;add square errors into e_out
x_out=x*0 & x_out[0]=x[0]
n_out=n*0 & n_out[0]=n[0]
j = 0 & xj=x[0]

; Grouping
for i=1,sx-1 do begin
   if abs(x[i]-xj) le d then begin
     w_out[j,*] = w_out[j,*] + w_tmp[i,*]
     x_out[j] = x_out[j] + x[i]
     e_out[j,*] = e_out[j,*] + e[i,*]^2 ;add square errors into e_out
     n_out[j] = n_out[j]+n[i]
     flg[j]=flg[j]+1
   endif else begin
     j=j+1 & xj = x[i]
     w_out[j,*] = w_tmp[i,*]
     x_out[j] = x[i]
     e_out[j,*] = e[i,*]^2 ;add square errors into e_out
     n_out[j] = n[i]
   endelse
endfor

; If only one point remains after grouping, duplicate it to keep same structure of arrays
if j+1 eq 1 then begin
   w_out[j+1,*] = w_out[j,*]
   x_out[j+1] = x_out[j]
   e_out[j+1,*] = e_out[j,*]
   n_out[j+1] = n_out[j]
   flg[j+1] = flg[j]
   j = j+1
   print,' One single pressure after grouping. Duplicated in output workspace'
endif else print,' Number of PRESSURES after grouping = ',j+1

;Set error bars:
;  e_out contains the sum of the squared standard deviations passed in datp.e, so
;  sigma = sqrt(sigma_1^2 + sigma_2^2 + ...)
e_out = sqrt(e_out[0:j,*])

; Perform average?
if keyword_set(average) then for i=0,j do begin
   w_out[i,*] = w_out[i,*]/flg[i]
   e_out[i,*] = e_out[i,*]/flg[i]
   n_out[i] = n_out[i]/flg[i]
endfor

x_out = x_out[0:j]/flg[0:j]
w_out = w_out[0:j,*]
e_out = e_out[0:j,*]
n_out = n_out[0:j]


; Return parameters
mod_datp, datp, 'X', x_out
mod_datp, datp, 'Y', y_out
mod_datp, datp, 'E', e_out
mod_datp, datp, 'N', n_out
give_datp, datp

return,w_out
end

