function elascan_vnorm, w_in, w_van, remove=fremove
;+
;******* *************
;** User callable
;*******************************************************************************
;**
;** Normalize by vanadium.
;** If the vanadium has been measured at several temperatures, then for each
;** sample temperature it looks for the closest vanadium temperature and it
;** uses this one to normalize.
;** If the keyword 'remove' is given with a value between 0 and 1, then all
;** the detectors where the vanadium counts are < than the average value
;** multiplied by this factor are removed.
;**
;** Call w10=elascan_vnorm(w1,w2) or w10=elascan_vnorm(w1,w2,remove=0.1)
;**
;*******************************************************************************
;-

; ------------------------------------------------------------------------------
; Check for consistencies

w_out=w_in
s_in=size(w_in) & s_vn=size(w_van)
if (s_in[0] ne 2) then begin
   print
   print,'Elascan_vnorm:'
   print,' Data workspace is not a 2D array. Not valid input!'
   print,' NOTHING DONE!!!'
   return,0
endif
if (s_vn[0] ne 2) then begin
   print
   print,'Elascan_vnorm:'
   print,' Vanadium workspace is not a 2D array. Not valid input!'
   print,' NOTHING DONE!!!'
   return,0
endif
ntemperatures  = s_in[1] & ndetectors = s_in[2]
if (ndetectors ne s_vn[2]) then begin
   print
   print,'Elascan_vnorm:'
   print,' Different number of detectors in sample and vanadium measurements!'
   print,' ndetectors sample = s_in[2] = ',ndetectors
   print,' ndetectors vana = s_vn[2] = ',s_vn[2]
   print,' NOTHING DONE!!!'
   return,0
endif

take_datp, sample         ;take parameters corresponding to w_in
take_datp, vana, /third   ;take parameters corresponding to w_van

x_out = sample.x ;temperatures
y_out = sample.y ;angle or Q
e_out = sample.e ;errors
n_out = sample.n ;monitor

if (n_elements(fremove) gt 0) then begin

   ;Check vanadium detectors (for all temperatures) and find bad spectra
   ntv  = s_vn[1] ;vanadium temperatures
   average_counts = total(w_van) / ndetectors / ntv
   det_counts = total(w_van[*,*],1) / ntv
   idx = where( det_counts[*] ge (average_counts*fremove))
   iremoved = where( det_counts[*] lt (average_counts*fremove))
   nremoved = n_elements(iremoved)
   if nremoved eq 1 then if iremoved eq -1 then nremoved = 0
   print
   print,'Elascan_vnorm:'
   print,' Number of detectors removed = ',nremoved
   if nremoved gt 0 then print,' Detectors removed (first det = 1): ',iremoved+1

endif

for i=0,ntemperatures-1 do begin

   ; Find index corresponding to smaller T difference between sample and background
   diff_temp = abs(vana.x - x_out[i])
   min_temp = min(diff_temp)
   itemp = where(diff_temp eq min_temp)
   ivana = itemp[0]
   w_out[i,*] = w_in[i,*] / w_van[ivana,*]
   e_out[i,*] = sqrt( (sample.e[i,*]/w_van[ivana,*])^2 + $
                      (w_in[i,*]*vana.e[ivana,*]/w_van[ivana,*]^2)^2 )

endfor

if (n_elements(idx) gt 0) then begin

   ;Remove bad detectors
   w_out = w_out[*,idx]
   e_out = e_out[*,idx]
   y_out = y_out[idx]

endif


; Return parameters
mod_datp, sample, 'X', x_out
mod_datp, sample, 'Y', y_out
mod_datp, sample, 'E', e_out
mod_datp, sample, 'N', n_out
give_datp, sample
return,w_out

end
