function quasi_slab, w_data, background_w=n_container, angle=angle, t_sample=ts, $
                     t_container=tc, twice=back, f=f, corr_w=n_corr
;+
;******* **********
;** User callable
;*******************************************************************************
;**
;** Evaluation of correction coefficients for transmission in the case of a
;** flat sample. Both absorption and self shielding are taken into account.
;** Calculations are made according to reference [1] (chapter 4) and following
;** the fortran implementation in [2]. However some equations have been changed,
;** as there are some errors in the expressions given in [1] for the reflection
;** case. The factors obtained agree with the results given by the Self-shielding
;** calculator for slab geometry of DAVE.
;**
;**  Is:    theoretical scattering from sample, without absorption
;**  Ic:    theoretical scattering from container, without absorption
;**  Iscsc: scattering from sample plus container (i.e. measured scattering)
;**  Icc:   scattering from container when alone (i.e. measured)
;**  Issc:  scattering from sample itself when inside container
;**  Icsc:  scattering from can itself when sample inside
;**
;**  measured scattering Iscsc = Issc + Icsc
;**
;**  Issc  = Assc*Is , where Assc is the self-shielding of scattering by the sample (in sample measurement)
;**  Icsc  = Acsc*Ic , where Acsc is the self-shielding of scattering by the container (in sample measurement)
;**  Icc   = Acc*Ic  , where Acc is the self-shielding of scattering by the container (in empty cell measurement)
;**
;**   Is = (1/Assc) * [Iscsc - (Acsc/Acc)*Icc ]
;**
;**   Also defined sample attenuation factor, SAF = Acsc/Acc, and
;**   self-shielding factor, SSF = Assc, so
;**
;**   Is = (1/SSF) * (Iscsc - SAF*Icc)
;**
;** [1] "Quasielastic neutron scattering", M. Bee, Adam Hilger (1988)
;** [2] Elascan, subroutine elsc_slab
;**
;** Call w6=elascan_slab (w1, background_w=2, angle=45, t_sample=0.90, t_container=0.99,twice=0.5, f=1.0, corr_w=20)
;** Short call: w6=elascan-slab(w1,b=2,a=45,t_s=0.9,t_c=0.99,tw=0.5,f=1,c=20)
;**
;**  where the data are in w1, 2 indicates that the background measurement to subtract is in w2,
;**  the slab is oriented pointing to 45 degrees, the transmission of the sample (without container) is 0.90,
;**  and the transmission of the empty cell is 0.99.
;**
;**  'twice' is the fraction of neutrons passing a second time through the sample in the way back from
;**  the analysers to the detectors.
;**
;**  'f' is a scaling factor for the background that can be used for example when the measured background
;**  does not correspond exactly to the real background during the sample measurement.
;**
;**  'corr_w' serves to indicate a workspace to store the correction factors:
;**  A_c,c , A_c,s+c , A_s,s+c , SAF = A_c,s+c / A_c,c , SSF = A_s,s+c.
;**
;** Apart from the input workspace containing the data, the only compulsory parameter to pass when calling
;** the routine is the angle corresponding to the orientation of the slab.
;** If the background workspace is not passed, then only the self-shielding correction is applied:
;** Is = Iscsc / SSF.
;** If a transmission value is not given, then its value is set to 1 (so no correction will take place),
;** while the default values for twice and f are 0 (so no correction for passing a second time through
;** the sample applies) and 1.
;**
;** With the above call the routine will give as output the corrected data in w6.
;** Additionally an ASCII file called 'attenuation_factors.dat' is written. This file contains the
;** self-shielding factors Assc, Acsc, and Acc applied in the correction.
;** If twice > 0, another ASCII file named 'backthrough.dat' is written. It contains the attenuation
;** factor applied to the neutrons passing a second time through the sample.
;**
;** If the background workspace contains more than one temperature measurement, for each point corresponding
;** to a sample temperature the subtracted background is the empty cell measurement taken at the closest T.
;** The file 'temperatures.dat' contains tje list of the sample temperatures and the corresponding
;** background temperature used in the subraction.
;**
;** MAG (07/5/2009)
;** JC  (02/8/2012) MAG version modified for quasielastic measurements
;**
;*******************************************************************************
;-


; ------------------------------------------------------------------------------
;Read data and background workspaces (for background use only first spectrum at the moment)

;take sample data
take_datp, datp & x_data=datp.x & y_data=datp.y & e_data=datp.e & n_data=datp.n & par = datp.p

!except=0
print
; check dimensions
sw=n_elements(w_data) & se=n_elements(e_data)
sx=n_elements(x_data) & sy=n_elements(y_data) & sn=n_elements(n_data)
if (sw ne se) then begin & print,'Quasi_slab: Problem! w & e do not have the same number of elements' & return,1 & endif
if (sx*sy ne sw) then begin & print,'Quasi_slab: Problem! Incompatible sizes of x, y and w' & return,1 & endif
if (sx ne sn) then begin & print,'Quasi_slab: Problem! x & n do not have the same dimensions' & return,1 & endif
print,'Quasi_slab:'
print,' Number of energy channels in sample measurement = ',sx

if n_elements(n_container) gt 0 then begin
   ;get background data
   take_datp, datb, W=n_container & take_w,w_background,W=n_container
   par = datb.p
   sb = size(w_background)
   if (sb[2] ne sy) then begin
      print,'Quasi_slab:'
      print,' ERROR! Different number of detectors in sample and background measurements'
      return,0
   endif
   x_background = datb.x  ;energy channels
   y_background = datb.y  ;scattering angles
   e_background = datb.e
   n_background = datb.n
   svx=n_elements(x_background)
   print,' Number of energy channels in background measurement = ',svx
endif else begin
   print,'Quasi_slab:'
   print,' No background measurement given!'
   print,' Applying self shielding corrections to sample measurement.'
endelse

w_out = w_data
e_out = e_data
x_out = x_data
y_out = y_data

; ------------------------------------------------------------------------------
; Check input

if (n_elements(ts) eq 0) then ts=1.0
if (n_elements(tc) eq 0) then tc=1.0
if (n_elements(back) eq 0) then back=0.0
if (n_elements(f) eq 0) then f=1.0

if (angle lt 1.0) then begin
   print,'Quasi_slab:'
   print,' ERROR! Incident beam within the plane of slab'
   return,0
endif

if (angle gt 359.0) then begin
   print,'Quasi_slab:'
   print,' ERROR! Give angle between 1 and 359 degrees'
   return,0
endif

if ((ts le 0.0) or (ts gt 1.0)) then begin
   print,'Quasi_slab:'
   print,' ERROR! Sample transmission must be between 0 and 1'
   return,0
endif

if ((tc le 0.0) or (tc gt 1.0)) then begin
   print,'Quasi_slab:'
   print,' ERROR! Container transmission must be between 0 and 1'
   return,0
endif

if ((back lt 0.0) or (back gt 1.0)) then begin
   print,'Quasi_slab:'
   print,' ERROR! Backthrough fraction must be betwen 0 and 1'
   return,0
endif


; ------------------------------------------------------------------------------
; Angle from the sample normal to the incident neutron wavevector

phi0 = 90.0 - angle


; ------------------------------------------------------------------------------
; Define a0c and a0s (eqs. A.4.7 and A.4.9 in [1])
; Note that the definition for the container includes a factor 1/2 as a0c corresponds
; to a single wall of the container.

a0c = -0.5 * alog(tc) / (2.0 * cos(phi0*!dtor))
a0s = -alog(ts) / (2.0 * cos(phi0*!dtor))


phi = y_data + phi0 ;vector containing the angle between the sample normal and each of the detectors

;avoid divergences by moving angles corresponding to 90 and 270 degrees
idx=where( (phi eq 90.0) or (phi eq 270.0))
if idx[0] ge 0 then phi[idx]=phi[idx]-0.1

; Define ac and as (eqs. A.4.7 and A.4.9 in [1])
; Note that the definition for the container includes a factor 1/2 as ac corresponds
; to a single wall of the container.

ac = -0.5 * alog(tc) / (2.0 * cos(phi*!dtor))
as = -alog(ts) / (2.0 * cos(phi*!dtor))

; u(x) = [exp(x)-exp(-x)]/2x
; uaa0c: u(ac-a0c) used in defining H1(a0c,ac)
; u2a0c: u(2ac-2a0c) used in defining H1(2a0c,2ac)
; uaa0s: u(as-a0s) used in defining H1(a0s,as)

uaa0c = (exp(ac-a0c) - exp(-ac+a0c)) / (2.0*(ac-a0c)) > 1 ;function > 1 for all (ac-a0c) values and = 1 when ac=a0c (indeterminate 0/0)
u2a0c = (exp(2.0*ac-2.0*a0c) - exp(-2.0*ac+2.0*a0c) ) / (2.0*(2.0*ac-2.0*a0c)) > 1 ;function > 1 for all (ac-a0c) values and = 1 when ac=a0c (indeterminate 0/0)
uaa0s = (exp(as-a0s) - exp(-as+a0s)) / (2.0*(as-a0s)) > 1 ;function > 1 for all (ac-a0c) values and = 1 when ac=a0c (indeterminate 0/0)

; H1(a0s,as) = exp(-|a0s|-|as|) * u(as-a0s)
; ha0ac: H1(a0c,ac) used in 4.A.13 and 4.A.15 in [1]
; h2a02ac: H1(2a0c,2ac) defined in 4.A.6 in [1]
; ha0as: H1(a0s,as) defined in 4.A.8 and used in 4.A.10 and 4.A. 14 in [1]

ha0ac = exp(-abs(ac)-abs(a0c)) * uaa0c
h2a02ac = exp(-2.0*abs(ac)-2.0*abs(a0c)) * u2a0c
ha0as = exp(-abs(as)-abs(a0s)) * uaa0s


; Transmission geometry
; acc = A_c,c given by eq. 4.A.6 in [1]
;       equivalent to A_c,c = 0.5 * ha0ac * (exp(-2.0*(abs(as)) + exp(-2.0*abs(a0c)))
; acsc = A_c,s+c given by eq. 4.A.13 in [1], but doing the sum of absolute values
;       (exp(-2.0*(abs(as)+abs(ac))) + exp(-2.0*(abs(a0s)+abs(a0c)))) instead of
;       (exp(-2.0*abs(as+ac)) + exp(-2.0*abs(a0s+a0c))). But they give the same
;       result.
; assc = A_s,s+c given by eq. 4.A.10 in [1]

idx=where( (phi le 90.0) or (phi ge 270) )
if idx[0] ge 0 then begin
   acc = h2a02ac
   acsc = 0.5 * ha0ac * (exp(-2.0*(abs(as)+abs(ac))) + exp(-2.0*(abs(a0s)+abs(a0c))))
   assc = exp(-2.0*abs(a0c)) * ha0as * exp(-2.0*abs(ac))
endif

; Reflection geometry
; acc = A_c,c given by eq. 4.A.15 in [1], but note that there are several mistakes.
;       A factor of 0.5 missing and the exponent should be exp(-2.0*(abs(a0c)+abs(ac))).
;       The 1/2 factor is correctly added in the fortran code [2], but not the other correction.
; acsc = A_c,s+c given by eq. 4.A.16 in [1], but note that there are several mistakes.
;        A factor of 0.5 missing and the exponent should be exp(-2.0*(abs(a0c)+abs(a0s)+abs(as)+abs(ac))).
;       The 1/2 factor is correctly added in the fortran code [2], but then it considers that
;       abs(a0c+ac) = abs(a0c)+abs(ac) and that abs(a0s+as) = abs(a0s)+abs(as), which is wrong
; assc = A_s,s+c given by eq. 4.A.14 = 4.A.10 in [1]

idx=where( (phi gt 90.0) or (phi lt 270) )
if idx[0] ge 0 then begin
   acc = 0.5 * ha0ac * (1.0 + exp(-2.0*(abs(a0c)+abs(ac))))
   acsc = 0.5 * ha0ac * (1.0 + exp(-2.0*(abs(a0c)+abs(a0s)+abs(as)+abs(ac))))
   assc = exp(-2.0*abs(a0c)) * ha0as * exp(-2.0*abs(ac))
endif

; Sample attenuation factor, SAF = Acsc/Acc, and
; self-shielding factor, SSF = Assc

saf = acsc / acc
ssf = assc

;  Write correction factors

openw,u,'attenuation_factors.dat',/get_lun
printf,u,'      Angle        A_c,c        A_c,s+c      A_s,s+c      SAF          SSF'
for i = 0,sy-1 do begin
   printf,u,y_data[i],acc[i],acsc[i],assc[i],saf[i],ssf[i]
endfor
free_lun,u

;  Store correction factors in a workspace
if (n_elements(n_corr) gt 0) then begin
   datc={w_tit:'Attenuation factors', x:y_data, y:[1,2,3,4,5], y_tit:'1.Acc,2.Acsc,3.Assc,4.SAF,5.SSF', x_tit: 'Angle'}
   give_datp, datc, W=n_corr
   corr=fltarr(sy,5)
   corr[*,0]=acc
   corr[*,1]=acsc
   corr[*,2]=assc
   corr[*,3]=saf
   corr[*,4]=ssf
   give_w,corr,W=n_corr
endif

;  Apply corrections for each temperature. Background substracted is the one measured
;  at the closest T at which the sample has been measured.
if n_elements(n_container) eq 0 then begin

   ;No background. Apply only self-shielding corrections for each temperature.
   for i=0,sx-1 do begin
      w_out[i,*] = w_data[i,*] / ssf[*]
      e_out[i,*] = e_data[i,*] / abs(ssf[*])
   endfor

   ; Apply attenuation factor due to second pass through sample.
   ; Computed following the same scheme as in Marc Bee's program (subroutine elsc_subslab).
   if (back gt 0) then begin
      phin = 90.0 - phi
      fcor = exp( alog(ts*tc) / abs(sin(phin*!dtor)) )
      openw,u,'backthrough.dat',/get_lun
      printf,u,'      Angle        fcor'
      for i = 0,sy-1 do begin
         printf,u,y_data[i],fcor[i]
      endfor
      free_lun,u
      for i=0,sx-1 do begin
         w_out[i,*] = (1.0-back)*w_out[i,*] + back*w_out[i,*]*fcor
         e_out[i,*] = sqrt( ((1.0-back)*e_out[i,*])^2 + (back*e_out[i,*]*fcor)^2 )
      endfor
   endif

endif else begin

   ;Apply the corrections for each energy channel and subtract background
 
   for i=0,sx-1 do begin

      w_out[i,*] = ( w_data[i,*] - saf[*] * w_background[i,*] * f) / ssf[*]
      e_out[i,*] = sqrt( e_data[i,*]^2 + (saf[*]*e_background[i,*]*f)^2 ) / abs(ssf[*])


   endfor
   free_lun,u

   ; Apply attenuation factor due to second pass through sample.
   ; Computed following the same scheme as in Marc Bee's program (subroutine elsc_subslab).
   if (back gt 0) then begin
      phin = 90.0 - phi
      fcor = exp( alog(ts*tc) / abs(sin(phin*!dtor)) )
      openw,u,'backthrough.dat',/get_lun
      printf,u,'      Angle        fcor'
      for i = 0,sy-1 do begin
         printf,u,y_data[i],fcor[i]
      endfor
      free_lun,u
      for i=0,sx-1 do begin
         w_out[i,*] = (1.0-back)*w_out[i,*] + back*w_out[i,*]*fcor
         e_out[i,*] = sqrt( ((1.0-back)*e_out[i,*])^2 + (back*e_out[i,*]*fcor)^2 )
      endfor
   endif

endelse

;Return parameters and exit
mod_datp, datp, 'X', x_out
mod_datp, datp, 'Y', y_out
mod_datp, datp, 'E', e_out
mod_datp, datp, 'N', n_out
give_datp, datp

return,w_out
end
