function in16b_fws, w_in, plotcolumn = plotcolumn, plot3d = plot3d
;+
;******* ***
;** User callable
;** Generates a 3D array corresponding to a fixed window scan performed on IN16B.
;** The input should be a 2D array [2theta, numor] read with the option RDSET, fws=1
;** and the output will be a 3D array containing:
;**
;**       Intensity[temperature, energy defined by doppler speed, detector angle]
;**
;** Possible calls are:
;**
;**       w2 = in16b_fws(w1)
;**       w2 = in16b_fws(w1, /plotcol)
;**       w2 = in16b_fws(w1, /plot3d)
;**
;** The last two commands are optional and will make the function to make an automatic plot
;** of the output array, either as several plots (one per doppler energy) or as a single 3D plot.

forward_function plot3d

    W_in =reform(W_in)
    w_out=float(w_in)
    s    =size( w_in)
    
;** Check for consistencies (workspace dimension s(0)=1 or 2)
    if (s[0] gt 0) and (s[0] lt 3) then begin
      TAKE_DATP,P
    endif
    
    Err = P.E
    y_out = P.X[*,0]
    T = P.PV[9,*]
    E = p.PV[14,*]; --> sort + uniq
  
    b = E[UNIQ(E, SORT(E))]; gets list of different doppler energies and orders them
    d = fltarr(n_elements(b)); energy count
    for i=0,n_elements(b)-1 do begin 
      c = 0
      c = where (E eq b[i], count)
      d[i] = n_elements(c)
    endfor

    temp = fltarr(n_elements(b), max(d)); temperature values for intensity
    
    Int = fltarr(s[1], n_elements(b), max(d)); int[detector angle, Energy, Temp] 
    Interr = fltarr(s[1], n_elements(b), max(d));Error[detector angle, Energy, Temp] 
    
    for i=0, n_elements(E)-1 do begin
    
      En_pos = WHERE(b eq E[i]);find pos in energy array
      temp_index = min(where(temp[En_pos,*] eq 0)); returns the lowest value element that hasnt been inputed yet

      Int[*,En_pos, temp_index] = W_in[*, i]
      Interr[*,En_pos, temp_index] = Err[*, i]
      temp[En_pos,temp_index] = T[i]

    endfor
   
    s = size(Int)
    if s[0] ne 3 then begin
      print, 'This macro is expected to create a 3D array: I[T, energy, 2theta]'
      print, 'Not enough dimensions with these data.'
      print, 'Returning input workspace'  
      return, W_in    
    endif
    
    w_out = transpose(Int,[2,1,0]);w_wout = intensity[temperature, doppler energy, detector ]
    E_out = transpose(Interr,[2,1,0]);w_wout = Error[temperature, doppler energy, detector ]
    x_out = transpose(temp)

    P.x_tit = 'T/K'
    MOD_DATP ,P, 'x', x_out; temp
    P.z_tit = '2-theta (degrees)'
    MOD_DATP, P, 'z', y_out; angles
    P.y_tit = 'Delta Energy (microeV)'
    MOD_DATP, P, 'y', b; doppler energies - e.g. 0,1,3,5,7,9,11
    MOD_DATP, P, 'e', E_out; Intensity error
    GIVE_DATP,P


    ;;;;plotting
    colours = ['b','g','r','c','m','y','k']
    line_col = make_array(n_elements(y_out), /STRING)
    for k = 0, n_elements(line_col)-1 do begin
       kk = k - ((k/7)*7); kk = 0,1,2,3...,6,7,0,1...,7,0
       line_col[k] = colours[kk]
    endfor

    if n_elements(plotcolumn) ne 0 then begin
      len = n_elements(b)
      for i = 0, len-1 do begin
          for j = 0, n_elements(y_out)-1 do begin
             if j eq 0 then begin      
                id2 = errorplot(x_out[*,i],w_out[*,i,j],e_out[*,i,j],line_col[j],XTITLE="Temperature(K)", YTITLE="Intensity",layout = [1,len,i+1], current = id2)
                ;id2 = plot(x_out[*,i],w_out[*,i,j],layout = [1,len,i+1], /overplot)
                id2.SYM_INCREMENT = 10     
                id2.SYM_COLOR = "blue"          
                id2.SYM_FILLED = 1
                id2.SYM_FILL_COLOR = 0
             endif else begin
                id2 = errorplot(x_out[*,i],w_out[*,i,j],e_out[*,i,j],line_col[j],XTITLE="Temperature(K)", YTITLE="Intensity",layout = [1,len,i+1], /overplot, current = id2)
             endelse
          endfor
      endfor  
    endif

    energy_array3d = transpose(rebin(b, n_elements(b), max(d))) 
    if n_elements(plot3d) ne 0 then begin
       len = n_elements(b)
       for i = 0, len-1 do begin
          for j = 0, n_elements(y_out)-1 do begin
             if i eq 0 and j eq 0 then begin
                id = plot3d(x_out[*,i],energy_array3d[*,i],w_out[*,i,j],line_col[j])
             endif else begin
                id = plot3d(x_out[*,i],energy_array3d[*,i],w_out[*,i,j],line_col[j], /overplot, current = id)
             endelse
           endfor    
        endfor
    endif
       
    return, w_out

end

