;+
;****************************************************************************
;
;PRO fitpho, x_in, y_in, e_in, xmin, xmax, par, dpar, temp, debyeE, $
;            all=allpoints, plot=plot
;
; Fits a Gaussian plus a Debye phonon term (w[n(w)+1]) for use in the PDOS 
; calculation in remove_phonon.pro
;
;ARGUMENTS
; xmin     :min limit of fit range
; xmax     :max limit of fit range
; par     :output of fitted Gaussian parameters :-- par(1) - height
;                           par(2) - centre
;                           par(3) - FWHM (sigma)
;                           par(4) - Debye phonon term
; dpar     :errors in fitted parameters
; temp   :temperature of data
; debyeE :Cutoff energy below which low-E phonons are fitted
;
;KEYWORDS
; /all    : keeps all points in range xmin<x<xmax.
;       otherwise, Gaussian fitted in range x0-FWHM<x<x0+FWHM (default)
; /plot : plot data and fit while executing
;
;DIMENSIONS
; x_in, y_in and e_in ar all 1D arrays of the same dimension
;
;COMMAND SYNTAX
; fitpho, x1, y1, e1, ,<min> ,<max>,<temp>,<debyeE>,[,/all][,/plot]
;
; (optional keywords/args shown in square brackets) 
;
;                        KHA,JRS 7/4/07
;-

PRO gaussian_debye, x, par, gausspho, PD
;+
;** **************
;** stuff for fitpho
;-
    
    COMMON phonon_params, temperature, kb

    height=par[0]
    centre=par[1]
    sigma=par[2]
    pho=par[3]

    peak=EXP(-0.5*((x-centre)/sigma)^2)
    gauss=height*peak
    db=x/(1.-EXP(-x/(temperature/kb)))
    i = WHERE(ABS(x) LT 0.001)
    
    IF i[0] NE -1 THEN db[i] = temperature/kb

    gausspho=gauss + pho*db

    IF (N_PARAMS() GE 4) THEN BEGIN
        dfda=peak
        dfdb=height*peak*(x-centre)/sigma^2
        dfdc=height*peak*(x-centre)^2/sigma^3
        dfdd=db
        PD=[[dfda],[dfdb],[dfdc],[dfdd]]
    ENDIF
END

; ---------------------------------------------------------------------------
;****************************************************************************

PRO fitpho, x_in, y_in, e_in, xmin, xmax, par, dpar, temp, debyeE, $
            all=all, plot=plot
;+
;** ******
;** User callable
;-

    COMMON phonon_params, temperature, kb

    iprint = 0

    kb = 11.60998    
    temperature = temp

    IF KEYWORD_SET(plot) THEN iplot = 1 ELSE iplot = 0

    IF iprint THEN PRINT,'Start fitpho:'

    sx=SIZE(x_in)
    npts=sx[1]

    IF KEYWORD_SET(all) THEN allpoints=1 ELSE allpoints=0
    IF iprint THEN PRINT,'npts=',npts

; --------------------------------------------------------------------

    IF iprint THEN PRINT,'Starting x-range extraction section'

    i=WHERE(x_in GE xmin AND x_in LE xmax,ndat)
    xdat=x_in[i]
    ydat=y_in[i]
    edat=e_in[i]
    i1=WHERE(edat EQ 0.,n1)

    IF (n1 GE 1) THEN BEGIN
        i2=WHERE(edat GT 0.,n2)
        IF (n2 LE 0) THEN BEGIN
            par=[0.,0.,(xmin+xmax)/2.,xmax-xmin]
            dpar=[0.,0.,xmax-xmin,xmax-xmin]
            GOTO, finished
        ENDIF
        emin=MIN(edat[i2])
        edat[i1]=emin
    ENDIF

    height=MAX(ydat,imax)
    centre=xdat[imax]
    hh=height/2.
    hh1=WHERE(ydat[0:imax] LT hh, n)
    IF (n EQ 0) THEN i1=0 ELSE i1=MAX(hh1)
    hh2=WHERE(ydat[imax:ndat-1] LT hh, n)
    IF (n EQ 0) THEN i2=0 ELSE i2=MIN(hh2)    & i2=i2+imax
    FWHM=xdat[i2]-xdat[i1+1]

    pho_cutoff   = debyeE
    pho_position = WHERE(xdat GE pho_cutoff)
    pho_value    = ydat[pho_position[0]]
    pho          = pho_value / (pho_cutoff/(1.-exp(-pho_cutoff/(temp/kb))))

    IF (allpoints EQ 0) THEN BEGIN
        i=WHERE(ABS(xdat-centre) LE pho_cutoff, n)
        IF (n LE 4) THEN BEGIN
            n=5
            i=[imax-2,imax-1,imax,imax+1,imax+2]
            IF (i[0] LT 0) THEN i=i+1
            IF (i[0] LT 0) THEN i=i+1
            IF (i[4] GE ndat) THEN i=i-1
            IF (i[4] GE ndat) THEN i=i-1
            FWHM=xdat[i[4]]-xdat[i[0]]
            centre=xdat[i[2]]
        ENDIF
        ndat=n
        xdat=xdat[i]
        ydat=ydat[i]
        edat=edat[i]
    ENDIF
    wdat=1./edat^2

    IF iprint THEN BEGIN
        PRINT, 'Phonon cutoff = ', pho_cutoff
        PRINT, 'Phonon value  = ', pho_value
        PRINT, 'pho           = ', pho
    ENDIF

    IF iplot THEN BEGIN
        PLOT, xdat, ydat, PSYM=4, SYMSIZE=0.4, $
            TITLE='Spectrum '+STRING(STRTRIM(ispec,2))
        ERRPLOT, xdat, ydat-edat/2., ydat+edat/2.
    ENDIF

; --------------------------------------------------------------------

    IF iprint THEN PRINT,'Starting fitting section'

    par=FLTARR(4)    & dpar=par
    par[0]=height    & par[1]=centre    & par[2]=FWHM/2.355 & par[3]=pho

    IF iplot THEN BEGIN
        dx=(xmax-xmin)/1000.
        xplot=xmin+FLOAT(INDGEN(1000))*dx
        gaussian_debye, xplot, par, ystart, PD
        OPLOT, xplot, ystart, LINE=1
    ENDIF

    ymean=TOTAL(ydat,1)/FLOAT(ndat)
    i=WHERE(ABS(ydat-ymean) GT 2.*edat, n)
    yfit=CURVEFIT(xdat,ydat,wdat,par,dpar,FUNCTION_NAME='gaussian_debye')

    IF iplot THEN BEGIN
        gaussian_debye, xplot, par, yfit, PD
        OPLOT, xplot, yfit, LINE=0
        PRINT,'Hit return to continue'    & wait=''
        READ, wait
    ENDIF

    IF iprint THEN PRINT,'End of fitting section'

; --------------------------------------------------------------------

    IF iprint THEN BEGIN
        pname=STRARR(4)
        pname[0]='Height'
        pname[1]='Centre'
        pname[2]='Sigma'
        pname[3]='Pho'
        FOR i=0,3 DO PRINT, pname[i], par[i], dpar[i], $
            FORMAT='(A10," =",F7.2," +/-",F6.2)'
    ENDIF

finished:
    RETURN
    END
