FUNCTION remove_phonons, w_in, pdos=pdos, a=a, b=b, Q=Q, multi_E=multi_E, $
                         debye_cutoff=debye_cutoff, phonons=phonons, dos=dos
;+
;******* **************
;** User callable
;** Removes incoherent phonons using the A + BQ^2 approximation.  Calls fitpho.pro
;** to remove the elastic line. Can output either the subtraction, the phonons
;** or the phonon DOS.****
;**
;** ARGUMENTS
;** ---------
;**    pdos    :    Workspace containing a high-Q cut of the data
;**    a,b    :    A and B parameters (A + BQ^2)
;**    Q    :    Q position of the high-Q cut pdos data
;**    multi_E    :    Energy beyond which the pdos data approximate to multi-phonons
;**    debye_cutoff:    Maximum energy for the Debye approximation of the PDOS
;**            around the elastic line
;**
;** KEYWORDS
;** --------
;**    /phonons:    outputs the phonon S(Q,w)
;**    /dos    :    outputs the phonon density of states
;**
;**                         JRS 10/4/07
;-

    iprint = 0

    TAKE_DATP, datp
    TAKE_DATP, datpdos, /THIRD

    temp = datp.p[11]

    q_in  = datp.x
    en_in = datp.y
    e_in  = datp.e

    p_en_in = datpdos.x
    edos    = datpdos.e

    IF iprint THEN PRINT, 'remove_phonons: End of "read in data" section'
; ----------------------------------------------------------------------------
;Get PDOS into shape (remove elastic line and multiphonons)

;remove elastic line
    pdos_out = pdos
    edos_out = edos
        y0   = MAX(pdos_out,i0)
        xmin = p_en_in[(i0 - 40) > 0]
        xmax = p_en_in[(i0 + 40) < (SIZE(pdos_out))[1]]

        FITPHO, p_en_in, pdos_out, edos_out, xmin, xmax, gauss, dgauss, temp, debye_cutoff

    centre = gauss[1]
    height = gauss[0]
    sigma  = gauss[2]
    pdos_buf   = pdos_out - height*EXP(-0.5*((p_en_in - centre)/sigma)^2)
    edos_buf   = edos_out

;Put on correct energy axis
    pdos_out = INTERPOL(pdos_buf, p_en_in, en_in)
    edos_out = INTERPOL(edos_buf, p_en_in, en_in)
    
;Add multiphonon tail beyond E cutoff
    IF N_ELEMENTS(multi_E) NE 0 THEN BEGIN
        ii   = WHERE(en_in GT multi_E)
        IF ii[0] NE -1 THEN BEGIN
            endy = [pdos_out[ii[0]-1], 0]
            endx = [en_in[ii[0]],en_in[(SIZE(en_in))[1]-1]]
            endp = INTERPOL(endy, endx, en_in[ii])
            pdos_out[ii]   = endp
            edos_out[ii]   = 0.0
        ENDIF
    ENDIF

    IF iprint THEN PRINT, 'remove_phonons: End of PDOS creation section'

; -----------------------------------------------------------------------------
;Make phonon array and subtract

    phonon_Q = a + b*(q_in^2)

    ii = WHERE(q_in GE Q)
    phonon_Q = phonon_Q / phonon_Q[ii[0]]

    phonon_sqw = phonon_Q # pdos_out
    phonon_err = phonon_Q # edos_out
    ii = WHERE(e_in EQ -1)
    phonon_sqw[ii] = 0.
    phonon_err[ii] = -1.

;Subtract phonons

    ii = WHERE(e_in NE -1)
    psub = w_in
    esub = e_in
    psub[ii] = w_in[ii] - phonon_sqw[ii]
    esub[ii] = SQRT(e_in[ii]^2 + phonon_err[ii]^2)

    IF iprint THEN PRINT, 'remove_phonons: End of phonon subtraction section'

; -----------------------------------------------------------------------------

    

    IF KEYWORD_SET(phonons) THEN BEGIN
        w_out = phonon_sqw
        e_out = phonon_err
        datp.e = e_out
        GIVE_DATP, datp    
    ENDIF ELSE IF KEYWORD_SET(dos) THEN BEGIN
        MOD_DATP, datpdos, "x", en_in
        MOD_DATP, datpdos, "e", edos_out
        w_out = pdos_out
        GIVE_DATP, datpdos
    ENDIF ELSE BEGIN
        w_out = psub
        e_out = esub
        datp.e = e_out
        GIVE_DATP, datp    
    ENDELSE

    RETURN, w_out
    END
