FUNCTION debyescherrer, win, interpol=inter, distance=e_d, datp=datp, $
                           verbose=verbose
; ----------------------------------------------------------------------------------
;
;+
;
; NAME:
;
;
;  debyescherrer
;
;
; USAGE:
;
;  w2 = debyescherrer(w1, distance=400.0)
;
; ARGUMENTS:
;
;  distance: detector distance in [cm]
;  /interpol: set keyword if interpolation needed
;  /verbose: set keyword if verbosity required
;
;
;
; win  is a 2D diffractogram i.e. I(2theta,h) where h
;      is the physical height of the multidetector
; wout is a 2D diffractogram i.e. I(2theta,h)
; in which the DS cones should be straightened out
; For each detector height calculate a vector of 2theta
; angles and interpolate onto equatorial plane
; using INTERPOL
;
; h is the physical height of the multidetector
; r is the physical radius of the multidetector
; set them
;
;
; VERSION HISTORY:
;
; Johnson MR. 2005?
; DRD, 2006 improve speed
; DRD, 2008/09 improve for IN5 data
; JOR, 2008/09 try to improve (?) change the name, improve speed
;               by changing the last loop by a *
;-
;
; ----------------------------------------------------------------------------------
;
;
;   The call is w6 = debyescherrer(w1, distance=400.0, /verbose)
;
; win  is a 2D (3D) diffractogram i.e. I(2theta,h)
;      where h is the physical height of the multidetector
; wout is a 2D diffractogram i.e. I(2theta,h)
; in which the DS cones should be straightened out
; For each detector height calculate a vector of 2theta
; angles and interpolate onto equatorial plane
;
; h is the physical height of the multidetector
; r is the physical radius of the multidetector
; set them
; ----------------------------------------------------------------------------------
FORWARD_FUNCTION interpol
st1=systime(1)
if (!version.release lt '5.3') then inter=1 ;***VALUE_LOCATE function***

; ----------------------------------------------------------------------------------
; h= 30cm, defined in reading D2B   and carried forward in Y
; h=  8cm, defined in reading SALSA and carried forward in Y
; h= 40cm, defined in reading D19   and carried forward in Y
; h=300cm, defined in reading IN5   and carried forward in Y
; ----------------------------------------------------------------------------------
r =130.0 ;D2B
if n_elements(e_d) eq 1 then $
	r =e_d  $;for SALSA 100.0 , D19 76.3 , IN5 400.
else $
	print,'DEBYESCHERRER: Warning: distance not given -> set to ',strtrim(r,2),' cm!'
if (size(win))[0]  eq 3 then kz=(size(win))[3]-1 else kz=0

; ----------------------------------------------------------------------------------
; get variables associated with win
; ----------------------------------------------------------------------------------
pasp=1 & if n_elements(datp) eq 0 then take_datp,datp else pasp=0
; ----------------------------------------------------------------------------------
; if x is bidim then take the middle. Not the best correction. to be improved!!!
; ----------------------------------------------------------------------------------
if (size(datp.x))[0] eq 2 then $
	XX=datp.x[*,(((size(datp.x))[2])-1)/2] $
else $
	XX=datp.X

; ----------------------------------------------------------------------------------
;don't use negative angles !!!
; ----------------------------------------------------------------------------------
wzero = where(XX ge 0.) & indPhiEqZero = wzero[0]
; ----------------------------------------------------------------------------------
;wzero=lindgen((size(datp.x))(1))
; ----------------------------------------------------------------------------------

rcos2t  =  r*cos(XX[wzero]*!pi/180.0)
rsin2t2 = (r*sin(XX[wzero]*!pi/180.0))^2
RAD2DEG  = 180.0/!pi
y2       = datp.y^2

; ----------------------------------------------------------------------------------
;  2D array, number of pixels vertically defined here (21)
; ----------------------------------------------------------------------------------
h_pixels     = (size(datp.y))[1]
glob_low_idx = fltarr(h_pixels) 
glob_hi_idx  = glob_low_idx

; ----------------------------------------------------------------------------------
;  Define output array and initialise 
; ----------------------------------------------------------------------------------
wout    = win*0.
win_s1  =(size(win))[1]     -1
rc_s1   = n_elements(rcos2t)-1

; ----------------------------------------------------------------------------------
;  Find how many times data has been accumulated in input workspace
; ----------------------------------------------------------------------------------
n_acc_sc=1 & Eok=0
if n_elements(datp.e) gt 1 then begin Eok=1 & Eout=Wout
   idx= where(datp.e[*,h_pixels/2,0] gt 0.)
   if idx[0] ge 0 then n_acc_sc = ceil(total(win[idx,h_pixels/2,0]/datp.e[idx,h_pixels/2]^2,0)/n_elements(idx))
endif
; if keyword_set(verbose) then $
;     print,'DEBYESCHERRER: number of scans = ',strtrim(n_acc_sc,2)
if keyword_set(verbose) then $
    print,'DEBYESCHERRER: straightening '  ,strtrim(h_pixels,2),' pixels * ',strtrim(rc_s1+1,2),' angles: ',strtrim(XX[0],2),' ->',strtrim(XX[win_s1],2),' deg.'

; ----------------------------------------------------------------------------------
;  For each detector height calculate the 2theta vector
; ----------------------------------------------------------------------------------
for j=0,h_pixels-1 do begin
	zq    = sqrt(rsin2t2 +  y2[j])       ; height -h/2 to +h/2
	Alpha = atan(zq,rcos2t)*RAD2DEG      ; calculate 2theta values for the equatorial plane 

	if Alpha[0] lt Alpha[rc_s1] then $
		xdx = where((XX ge Alpha[0]) and (XX le Alpha[rc_s1])) $ ;find corresponding indices Alpha->XX
	else $
		xdx = where((XX le Alpha[0]) and (XX ge Alpha[rc_s1]))   ;(ascending or descending)

	if xdx[0] ge 0 then begin 
		nxe = n_elements(xdx)-1
		phi = XX[xdx]
		glob_low_idx[j] = xdx[0]
		glob_hi_idx[ j] = xdx[nxe] 		;keep first and last 2theta index (straight_1d)
; ----------------------------------------------------------------------------------
;	INTERPOLATE ONTO EQUATORIAL PLANE
; ----------------------------------------------------------------------------------
		if keyword_set(inter) then begin
			for KK=0,kz do begin
				v = win[wzero,j,KK]
				v[0,*]=v[1,*] & v[nxe,*]=v[nxe-1,*] ;Will prevent from interpol bounds artifacts
				wout[xdx,j,KK]=INTERPOL(v,Alpha,phi)>0.
				if Eok then $
					Eout[xdx,j,KK]=INTERPOL(datp.e[wzero,j,KK],Alpha,phi)>0.   ;Ok I know!
			endfor
		endif else begin
			s  = VALUE_LOCATE(phi, Alpha)       ; index alpha between index phi
			si = where((s ge 0) and (s lt nxe)) ; Subscript intervals for index alpha
			s  = s[si]
			for k=0,n_elements(si)-1 do begin
				i  = si[k] 
				sk = s[k]
				ik = sk + xdx[0]
				PA = ( Alpha[i]-phi[sk] )/( phi[sk + 1] - phi[ sk ] )
                unMoinsPA = 1.0 - PA
				i  = i + indPhiEqZero
				if Eok then begin
					v = win[i  ,j,*]
					wout[ik  ,j,*] = wout[ik  ,j,*] + v * unMoinsPA
	        		wout[ik+1,j,*] = wout[ik+1,j,*] + v * PA
					e               = datp.e[i,j,*]
					Eout[ik   ,j,*] = Eout[ik  ,j,*] + e * unMoinsPA    ;Increase errors
					Eout[ik+1 ,j,*] = Eout[ik+1,j,*] + e * PA
                endif else begin
					v = win[i  ,j,*]
					wout[ik  ,j,*] = wout[ik  ,j,*] + v * unMoinsPA
	        		wout[ik+1,j,*] = wout[ik+1,j,*] + v * PA
                endelse    
			endfor
		endelse
		if xdx[0]    gt 0      then wout[0:xdx[0]-1       ,j,*] = -1.
		if xdx[nxe]  lt win_s1 then wout[xdx[nxe]+1:win_s1,j,*] = -1.
	endif
endfor
; ----------------------------------------------------------------------------------
; ----------------------------------------------------------------------------------

if Eok then datp.e=Eout
if keyword_set(verbose) then $
    print, 'DEBYESCHERRER: straightening done in ',strtrim(round( 1000*(systime(1)-st1) ) ,2),' ms'

datp.x_tit = 'ToF channels'
datp.y_tit = 'Scatt. ang. [deg]'
if pasp and Eok then give_datp,datp

return, wout

end
