;----------------------------------------------------------------------------------------------------------------------
;*********************************************************************************************************************
;+
;
;<h2>NAME:</h2>
;
;	corrtof, w_in, deteff = deteff, background = background, psd = psd,  winProp=winProp, frameoverlap = frameoverlap
;
;For IN4, IN5, IN6 and D7 data.
;
;<h2>KEYWORDS:</h2>
; <b>/deteff</b>           : corrects for energy-variation of detector efficiency
; <b>/frameoverlap</b>	   : subtracts a t^-4 tail from the beginning
;		             of the time frame & shift the elastic peak
; <b>/background </b> 	   : subtracts a flat background in each detector,
;		             found using a moving filter (For IN4, IN5 and IN6 only)
;                            NO MORE VALID on the transformed data. Must be applied before
;                            on the raw time-of-flight data.
;
;                            The moving window width can be changed with the keyword:
;<b> window = [value] </b> : default is 20. Must be used in conjunction with <b>/background</b>
;           
;<b> winProp = [value] </b> : default is 1. Multiply the background to subtract.Must be used in 
;                             conjunction with <b>/background</b>. <b>window=[value]</b> can also be used.
;
;<b> /verbose </b>         : More info during execution.
;
;
;<h2>DIMENSIONS:</h2>
;   w_in=w_out(nchannels,nphases*nspectra)
;
;<h2>COMMAND SYNTAX:</h2>
;   w2 = corrtof(w1[,/deteff][,/frameoverlap])
;   w2 = corrtof( w1,/deteff, window=15,/background, /verbose )
;
;  Since 10/2008 : 
;   w2 = corrtof( w1,/deteff, /psd, window=15,/background, /verbose )
; 
;   w2 = corrtof( w1,/deteff, /psd, window=15, elp=252, /background, /verbose )
;
;   Keyword PSD means using the new PSD detectors.
;
;   (optional keywords shown in square brackets)
;
;
;
;<h2>HISTORY:</h2>
;
;	written by:  KHA,JRS 23/6/03
;    revisions:  JO, 20/12/04 detector efficiency correction as implemented in
;                          INX for IN4, IN5, IN6 and MIBEMOL.
;                JO, Mon Nov 24 14:17:47 CET 2008: Add detector efficiency as for 1 inch tubes 
;                                                  IN5/IN6 (wait for Van Esch calculation ?)
;                JO, Fri Jan 23 10:35:44 CET 2009: Correct the w_in background substraction: W_in
;                                                  was affected by the substraction, not normal...
;                JO, Thu Aug 27 16:05:23 CEST 2009: Main at  bottom to avoid uncompiled functions
;                                                   forward_function out.
;                JO, Thu Jul 26 15:26:51 CEST 2012: Add ELP keyword to force the energy scale when elastic peak
;                                                   is non-existent or less than inelastic.
;                JO, Thu Sep 25 15:00:11 CEST 2014: Add keyword "winProp" such as W_out = W_in - winProp * BkGd

;
;
;-
;
; ----------------------------------------------------------------------------------------------------------------------
;*********************************************************************************************************************
; ----------------------------------------------------------------------------------------------------------------------
;
;               corr_tof
;
;
; WARNING: main at  bottom to avoid undefined routines
; ----------------------------------------------------------------------------------------------------------------------
function in5_corr_tof, w_in, window, winProp, deteff, frameoverlap, psd, elp, background, verbose
;-----------------------------------------------------------------------------------------------------------------------
common c_lamp_access, inst

if keyword_set(verbose) then begin
	print, 'CORRTOF: Instrument is ',inst
	if keyword_set(deteff) then $
		print, 'CORRTOF: Correct for energy/wavelength-dependent efficiency.'
	if keyword_set(frameoverlap) then $
		print, 'CORRTOF: Correct for frame overlap.'		
endif
;---------------------------------------------
;set up input workspace and energy parameters
;---------------------------------------------
take_datp, datp
nspectra  = n_elements(datp.y)
nchannels = n_elements(datp.x)

;-------------------------------------------------------------------------------
; Check if the data concistency and convert 
; in energy if necessary
;-------------------------------------------------------------------------------
if ~strcmp(datp.x_tit[0], 'Energy Transfer (meV)', 4, /FOLD_CASE) then begin
    if keyword_set(verbose) then $
        print, 'CORRTOF: WARNING: data in time channels -> compute energies.'    
      x_in = datp.x
      p_in = datp.p
      
;---------------------------------------------------------------------------
;
; Since recently the MIBEMOL parameters are stored in the same place as others
; instruments at the ILL 
;
; Fri May 11 17:03:57 CEST 2007
; ------------------------------
;
;      if (inst eq 'mibemol') then begin  
;         eelast = 81.8049/p_in(0)^2                  ; Ei in meV
;         p_in(27) = 3.580;                           ; Sample to detector distance
;         telast = p_in(27)*p_in(0)/3956.035          ; TOF in sec
;         cwidth = p_in(29)*0.1*1e-6                  ; chw in sec
;      endif else begin
;---------------------------------------------------------------------------
    eelast = 81.8049/p_in[21]^2                 ; Ei in meV
    telast = p_in[27]*p_in[21]/3956.035         ; TOF in sec
    cwidth = p_in[18]*1e-6                      ; chw in sec
;---------------------------------------------------------------------------
;      endelse
;           Find the elastic peak position for each group          
;---------------------------------------------------------------------------
      if (size(w_in))[0] eq 2 then $
          ytot   = total(w_in,2) $
      else $
          ytot   = w_in
      y0    = max(ytot,i0)
      xmin  = x_in[(i0 - 30) > 0]
      xmax  = x_in[(i0 + 30) < (nchannels - 1)]
      ind   = where(x_in gt xmin and x_in lt xmax)  
      epp   = fltarr(nspectra)
      if keyword_Set(ELP) then begin
         epp = epp + ELP
         if keyword_set(verbose) then $
            print, 'CORRTOF: ELP forced to user value ', strtrim( string(min(ELP),FORMAT = '(F8.0)') ,2 ),' for energy calculation'
       endif else begin
          gauss = epp
          gauss = str_fit(w_in[ind,*],Bkgd=0,Xin=x_in[ind])
          epp   = gauss[*,2] - 1.0
       endelse   

       x_dummy = w_in*0.0
       flt_time = fltarr(nspectra)
       j = indgen(nchannels)
       for i=0,nspectra-1 do begin
          flt_time  = telast - (epp[i]-j)*cwidth
          lam       = flt_time*3956.035/p_in[27]
          x_dummy[*,i]= eelast - 81.8049/lam^2
;---------------------------------------------------------------------------
;         In (rare) cases where the counting frame start before 
;         the neutron arrives on the sample
;---------------------------------------------------------------------------
;         x_dummy = x_dummy(where(flt_time>0.0),*)
      endfor
;---------------------------------------------------------------------------
;     interpolate to have same x at all angles
;---------------------------------------------------------------------------
      if (size(x_dummy))[0] gt 1 then begin
           sx   = size(x_dummy)
           hw = total(x_dummy,2)/sx[2]
      endif else hw = x_dummy
endif else begin
      hw = datp.x
endelse

if keyword_set(verbose) then $
    print, 'CORRTOF: Perform detector efficiency corrections in range: [',strtrim( string(min(hw),FORMAT = '(F8.2)') ,2 ),',',strtrim( string(max(hw),FORMAT = '(F8.2)'),2 ),'] meV'   
;-------------------------------------------------------------------------------------
;   Perform background correction
;-------------------------------------------------------------------------------------
w_out	  = w_in*1.0
if ~keyword_set(window) then window = 20
if ~keyword_set(winProp) then winProp = 1.0



 if keyword_set(background) then begin
         if keyword_set(verbose) then print,'CORRTOF: perform BG correction with a ',strtrim(string(window),2),' channels moving window.'
         if keyword_set(verbose) then print,'CORRTOF: a multiplication factor of ',strtrim(string(winProp),2),' is applied to the resulting constant.'
         for ispec=0,nspectra-1 do begin
                 S    = w_out[*,ispec]
                 Ss   = smooth(S,2*window+1)
                 bkgd = min(Ss[window:nchannels-window-1])>0.
                 w_out[*,ispec] = S - winProp*bkgd
         endfor
 endif
;-------------------------------------------------------------------------------
; Correct for detector efficiency if any.
; ---------------------------------------
;
; Corrections are exactly the same as in 
; the INX programme. They are updated for 
; IN5 scattering angles.
;
; The attenuation function for the 1cm tube is:
; eff=.951/eff0*exp(-.0887/sqrt(e))*(1.-exp(-4.07/sqrt(e)))
; For the big 1 inch tubes it is similar to other instruments:
; eff=0.951/eff0*exp(-0.0887/sqrt(e))*(1.0-exp(-5.597/sqrt(e))) 
; The wavelength steel cutoff is not apparent in these formula:
; it has been removed since it produces a discontinuity 
; in the detector efficiencies which is not present on real spectra 
;
; Mon Nov 24 14:17:47 CET 2008
; -------------------------------
; Takes the same expression as for 1 inch tubes for the new IN5 PSD tubes
; P = 4.75 bar He + 1.25 bar CF4
;
; The expressions in the Rieutord report:
; ---------------------------------------
;
; are the above function exactly
;
;-------------------------------------------------------------------------------
if keyword_set(verbose) then $
    if keyword_set(psd) then $
        print, 'CORRTOF: Warning: IN5 with PSD -> uses same energy correction than IN5/IN6 1 inch tubes.'

yerr      = datp.e*1.0

if keyword_set(deteff) then begin
        if (size(yerr))[0] le 1 then yerr = w_out*0.0
        
	case strlowcase(inst) of
	'in4':begin
			e0 = 81.799/datp.p[21]^2
			eff0=0.951*exp(-.0887/sqrt(e0))*(1.-exp(-5.597/sqrt(e0))) 
			if min([abs(min(hw)),e0]) lt e0 then $
				e = e0 + hw $
			else $
				e = e0 - hw
			for i=0,nspectra-1 do begin
				eff=0.951/eff0*exp(-0.0887/sqrt(e))*(1.0-exp(-5.597/sqrt(e))) 
				w_out[*,i]=w_out[*,i]/eff 
				yerr[*,i]=yerr[*,i]/eff 
			endfor 
	end
	'in5':begin
            ;-------------------------------------------------------------------------------
            ; In case of PSDs do the same calculation than for all 1 inch tubes IN5/6
            ; Don't take into account the stainless steel cutoff as was initialy implemented 
            ; in the old INX versions that doesn't work well at all and introduces 
            ; discontinuities in the spectra. 
            ;-------------------------------------------------------------------------------
            if keyword_set(psd) then begin
			    e0 = 81.799/datp.p[21]^2
			    if min([abs(min(hw)),e0]) lt e0 then $
				    e = e0 + hw $
			    else $
				    e = e0 - hw
			    eff0   = exp(-.0565/sqrt(e0))*(1.-exp(-3.284/sqrt(e0))) 
                
			    for i=0,nspectra-1 do begin
				    eff       = 1./eff0*exp(-.0565/sqrt(e))*(1.-exp(-3.284/sqrt(e))) 
				    w_out[*,i]    = w_out[*,i]/eff 
				    yerr[*,i] = yerr[*,i]/eff 
			    endfor 
            ;-------------------------------------------------------------------------------
            ; If no PSDs, i.e., for old secondary spectrometer, take into account the 
            ; 3 types of detector.
            ; Don't take into account the stainless steel cutoff as was initialy implemented 
            ; in the old INX versions that doesn't work well at all and introduces 
            ; discontinuities in the spectra. 
            ;-------------------------------------------------------------------------------
            endif else begin
                e0 = 81.799/datp.p[21]^2
                eff0=0.951*exp(-.0887/sqrt(e0))*(1.-exp(-4.07/sqrt(e0))) 
                if min([abs(min(hw)),e0]) lt e0 then $
                    e = e0 + hw $
                else $
				    e = e0 - hw
                for i=0,nspectra-1 do begin
                    if ( datp.y[i] ge 11.0 && datp.y[i] le 63.0 ) then  begin
                        eff0      = .951*exp(-.0887/sqrt(e0))*(1.-exp(-4.07/sqrt(e0))) 
                        eff       = .951/eff0*exp(-.0887/sqrt(e))*(1.-exp(-4.07/sqrt(e))) 
                        w_out[*,i]    = w_out[*,i]/eff 
                        yerr[*,i] = yerr[*,i]/eff 
                    endif
				if ( datp.y[i] gt 63.0 ) then begin
					eff0  = exp(-0.0565/sqrt(e0))*(1.-exp(-3.284/sqrt(e0))) 
					eff   = 1./eff0*exp(-0.0565/sqrt(e))*(1.0-exp(-3.284/sqrt(e))) 
					w_out[*,i]=w_out[*,i]/eff 
					yerr[*,i]=yerr[*,i]/eff 
				endif
			endfor 
            endelse
	end
	'in6':begin
            ;-------------------------------------------------------------------------------
            ; If no PSDs, i.e., for old secondary spectrometer, take into account the 
            ; 3 types of detector.
            ; Remove the possibility to take into account the stainless steel cutoff as was 
            ; initialy implemented in the old INX versions that doesn't work well at all 
            ; and introduces discontinuities in the spectra. 
            ;-------------------------------------------------------------------------------
			e0 = 81.799/datp.p[21]^2
			if min([abs(min(hw)),e0]) lt e0 then $
				e = e0 + hw $
			else $
				e = e0 - hw
			; eseuil=5.1124     ! this is the 4 angstroms limit. 
			; if (e0.le.eseuil) then 
			eff0   = exp(-.0565/sqrt(e0))*(1.-exp(-3.284/sqrt(e0))) 
			; else 
			; eff0=.94*(1.-exp(-3.284/sqrt(e0))) 
			; end if 
			for i=0,nspectra-1 do begin
				; if (e.le.eseuil) then 
				eff    = 1./eff0*exp(-.0565/sqrt(e))*(1.-exp(-3.284/sqrt(e))) 
				w_out[*,i] = w_out[*,i]/eff 
				yerr[*,i]=yerr[*,i]/eff 
				; else 
				; eff=.94/eff0*(1.-exp(-3.284/sqrt(e))) 
				; w_out(j,i)=w_out(j,i)/eff 
				; yerr(j,i)=yerr(j,i)/eff 
				; end if 
			endfor 
	end
	'mibemol':begin
			e0 = 81.799/datp.p[21]^2
;			e0 = 81.799/datp.p(0)^2
; same reason as above
			if min([abs(min(hw)),e0]) lt e0 then $
				e = e0 + hw $
			else $
				e = e0 - hw
			eff0=(1.-exp(-2.49/sqrt(e0))) 
			for i=0,nspectra-1 do begin
				eff=1.0/eff0*(1.-exp(-2.49/sqrt(e))) 
				w_out[*,i]=w_out[*,i]/eff 
				yerr[*,i]=yerr[*,i]/eff 
			endfor 
	end
	'dcsasc':begin
	end
	else:begin
	end
	endcase
endif

catch:
    
mod_datp,datp,"e",yerr
give_datp, datp

return, w_out

end

; ----------------------------------------------------------------------------------------------------------------------
;
;               corr_tof_d7
;
; ----------------------------------------------------------------------------------------------------------------------
function corr_tofD7, w_in, window, ieff, iover, itrans

common c_lamp_access, inst
;common printing, iprint, outstring

if iprint then print,'start CORRTOF_d7:'

take_datp, datp

; -------------------------------------------------------------------------------
;set up input file and energy parameters
; -------------------------------------------------------------------------------

; -------------------------------------------------------------------------------
;   find the analyser transmission files
; -------------------------------------------------------------------------------

ii = ''
ij = ''
root = lamp_macro + '/d7'

trans_file     = filepath('transmission.dat',     root_dir = root)
new_trans_file = filepath('new_transmission.dat', root_dir = root)
ii = file_search(trans_file)
ij = file_search(new_trans_file)
root = './'
trans_file     = filepath('transmission.dat',     root_dir = root)
new_trans_file = filepath('new_transmission.dat', root_dir = root)

if strlen(ii[0]) lt 1 then ii = file_search(trans_file)
if strlen(ij[0]) lt 1 then ij = file_search(new_trans_file)

if strlen(ii[0]) lt 1 and strlen(ij[0]) lt 1 then begin
s = strarr(2)
s[0] = 'CORRTOF: cannot perform analyser efficiency correction
s[1] = '          transmission files not found'
ii = DIALOG_MESSAGE(s, /error)
itrans = 0
endif else begin
trans_file     = ii[0] ;analyser transmission from h2 experiment of mompean
new_trans_file = ij[0] ;new analyser transmission
endelse

par=datp.p
lambda = par[4]
freq   = par[5]
chw    = par[7]
chel   = par[9]
tof=fix(par[8])

if not tof then begin
s = ' CORRTOF: workspace data is not in tof'
ii = DIALOG_MESSAGE(s, /error)
return, w_in
endif

if (freq le 0. or freq gt 12000.) then begin
print,'CORRTOF_d7: freq = 0 in workspace parameters. assume 8000rpm'
freq = 8000.
endif

nspectra  = fix(par[1])
nphases   = fix(par[2])
nchannels = fix(par[6])
y_in      = datp.y
e_in      = datp.e

sw = size(w_in)
se = size(e_in)

if (se[0] ne sw[0] or se[1] ne sw[1]) then begin
print,'error - size(w_in)=',sw
print,'	  size(e_in)=',se
print,'	- must be the same size'
return,w_in
endif

zeroed = where(e_in le -1.,n) ; zeroed channels
if iprint then print,n/nchannels,' zeroed spectra

if n_elements(ieff)   ne 0 then eff0   = ieff   else eff0   = 0
if n_elements(itrans) ne 0 then trans0 = itrans else trans0 = 0
if n_elements(iover)  ne 0 then over0  = iover  else over0  = 0

if keyword_set(verbose) then begin
print,'efficiency   = ', eff0
print,'transmission = ', trans0
print,'overlap      = ', over0
endif

if (eff0 eq 0 and trans0 eq 0 and over0 eq 0) then return,w_in

if keyword_set(verbose) then print,'corr_tof_d7: end of "w_in dimensions check etc." section'

; -------------------------------------------------------------------------------------
;   set constants and prepare arrays
; -------------------------------------------------------------------------------------

eff  = fltarr(nchannels)
wave = fltarr(nchannels)

const1 = 5.22697	; e(mev)=const1*v(m/ms)^2 for neutron
const2 = 2.07193571	; e(mev)=const2*q(a^-1)^2 for neutron
const3 = 3.956076	; v(m/ms)=const3/lambda(a) for neutron
const4 = 81.8066	; e(mev)=const4/lambda(a)^2 for neutron

l2 = 1500.0	 ; sample-detector distance (mm)

ei  = const4/lambda^2
vi  = sqrt(ei/const1)
tel = l2/vi

time = chw*float(indgen(nchannels) - chel) + tel
ef   = const1*(l2/time)^2

eff = (1. - exp(-5.6/sqrt(ef)))/(1.-exp(-5.6/sqrt(ei)))

wave = sqrt(const4/ef)

if (over0 eq 1) then begin
freq   = freq/60.		; convert to hz
period = 1.e+06/freq		; period in mcs
period = period/4.		; since there are four slits in chopper disk
nfit   = 10			; no.of points for fit of y=a*t^-4
ifit1  = nchannels - nfit
ifit2  = nchannels - 2
t = chw*(float(ifit1 + ifit2)/2. - chel)+tel ; mean time of integrating region
time1  = time+period		; frame-overlap time-of-flight
endif

if iprint then begin
print,'lambda=',lambda,' ei=',ei,' freq=',freq,' chw=',chw
print,'ef=',ef
print,'eff=',eff
print,'wave=',wave
endif

if iprint then print,'corr_tof_d7: end of "set constants etc." section'
; -------------------------------------------------------------------------------
;   open and read from input file
; -------------------------------------------------------------------------------
if trans0 then begin
line=''
if strlen(trans_file) lt 5 then begin
s    = strarr(2)
s[0] = 'corr_tof: old analyser transmission curve not available'
s[1] = '          will use new analyser curve'
old = 0
ii = DIALOG_MESSAGE(s)
endif else begin
old = 1
trans = read_trans[trans_file, nchannels, lambda, wave, 0]
endelse
if strlen(new_trans_file) lt 5 then begin
s    = strarr(2)
s[0] = 'CORRTOF: new analyser transmission curve not available'
s[1] = '          will use old analyser curve'
new = 0
ii = DIALOG_MESSAGE(s)
endif else begin
new = 1
new_trans = read_trans[new_trans_file, nchannels, lambda, wave, 1]
endelse


if keyword_set(verbose) then print,'corr_tof_d7: end of "read from input files" section'
endif

; -------------------------------------------------------------------------------
;   perform correction
; -------------------------------------------------------------------------------
s=fltarr(nchannels) & ds=s
w_out=w_in
e_out=e_in

ecorr=fltarr(nchannels) & ecorr[*]=1.

if eff0 eq 1 then ecorr=ecorr/eff

for ispec=0,nphases*nspectra-1 do begin
s = w_out[*,ispec]	& ds = e_out[*,ispec]
if over0 eq 1 then begin
y=total(s[ifit1:ifit2],1)/float(nfit)
a=y*t^4
s=s-a/time1^4
endif
if trans0 eq 1 then begin
idet = ispec mod nspectra
if idet lt 32 and old then tr = trans
if not new then tr = trans
if not old then tr = new_trans
if idet ge 32 and new then tr = new_trans
endif
w_out[*,ispec] = s*ecorr*tr
e_out[*,ispec] = ds*ecorr*tr
endfor

if(n gt 0) then begin
w_out[zeroed]=0.	& e_out[zeroed]=-1.
endif

if keyword_set(verbose) then print,'corr_tof_d7: end of correction section'

; -------------------------------------------------------------------------------
;   return parameters and exit
; -------------------------------------------------------------------------------
datp.e=e_out

s = ''
if eff0 then s = s + '/eff'
if over0 then s = s + ',/frame'
if trans0 then s = s + ',/trans'

datp.other_tit = datp.other_tit+' -ct('+s+')'

if (eff0 eq 0) then estring='no det eff corr' else estring='with det eff corr'
if (over0 eq 0) then fstring='no frame ovlp corr' else fstring='with frame ovlp corr'
if (trans0 eq 0) then tstring='no ana trans corr' else tstring='with ana trans corr'
outstring=estring+', '+fstring+', '+tstring

finished:
if keyword_set(verbose) then print,'end corr_tof_d7.'

give_datp, datp

return, w_out
end



;*********************************************************************************************************************
;*********************************************************************************************************************
function in5_corrtof, w_in, window = window, winprop = winprop, deteff = deteff, frameoverlap = frameoverlap, $
			background = background, psd=psd, elp=elp, verbose=verbose

common c_lamp_access, inst

;;forward_function str_fit, in5_corr_tof, corr_tof_d7
;-----------------------------------------------------
;check instrument name and call appropriate function
;-----------------------------------------------------
if strcmp(inst,'in4',/fold_case) ||  strcmp(inst,'in5',/fold_case) || $
   strcmp(inst,'in6',/fold_case) ||  strcmp(inst,'mibemol',/fold_case) || $
	strcmp(inst,'dcsasc',/fold_case) then $
	w_out = in5_corr_tof(w_in, window, winprop, deteff, frameoverlap, psd, elp, background, verbose) $
else if strcmp(inst,'d7',/fold_case) then $
	w_out = corr_tofD7(w_in, window, deteff, frameoverlap, background, ana_eff) $
else begin
;	s = 'corr_tof: instrument must be in4, in5, in6, d7, mibemol or dcsasc'
;	ii = DIALOG_MESSAGE(s, /error)
    print, 'CORRTOF: ERROR: instrument must be IN4, IN5, IN6, D7, MIBEMOL or DCSASC.'
	return, w_in
endelse
;-----------------------------
;return parameters and exit
;-----------------------------
;print,'corrtof: '+outstring

finished:
return, w_out

end
























