FUNCTION in5_distancecorrection, W_in, W_elp, datp=datp, verbose=verbose
; ----------------------------------------------------------------------------------
;
;+
;
; NAME:
;
;
;  distanceCorrection
;
;
; USAGE:
;
;  w2 = distanceCorrection(w1, /verbose)
;
; ARGUMENTS:
;
;  /verbose: set keyword if verbosity required
;
;
;
; win  is a 2D diffractogram i.e. I(2theta,h) where h
;      is the physical height of the multidetector
; wout is a 2D diffractogram i.e. I(2theta,h)
; in which the DS cones should be straightened out
; For each detector height calculate a vector of 2theta
; angles and interpolate onto equatorial plane
; using INTERPOL
;
; h is the physical height of the multidetector
; r is the physical radius of the multidetector
; set them
;
;
; VERSION HISTORY:
;
; JOR, Thu Oct  2 10:38:40 CEST 2008: Creation (inspired from blabla by DRD)
; JOR, Tue Nov 18 15:15:02 CET 2008 : Use value_locate instead of interpol: 
;                                     improve speed and avoid noise at largwe ToF
; JOR, Fri Apr 10 18:24:52 CEST 2009: Add the possibility to ELP on the vana because 
;                                     sometime no ELP after BG subtraction.
;                                     Keyword: W_elp where elp is a workspace with raw 
;                                     vanadium. 
; JOR, Fri Jul 27 17:32:40 CEST 2012: Add the propagation of the errors  if any (in case
;                                     that errors are defined before distancecorrection.
;
;
;-
;
; ---------------------------------------------------------------------------------- 
forward_function str_fit

    if keyword_set(verbose) then $
        print,'DISTANCECORRECTION: version: Fri Apr 10 19:18:12 CEST 2009 ...'
        
    st1=systime(1)

    take_datp,datp

    phi = datp.X
    

    COMMON c_lamp_access, inst

    take_datp, datp

	n_in = datp.n
	pv_in= datp.pv     
	p_in = datp.p       
	x_in = datp.x
	y_in = datp.y
	z_in = datp.z
	e_in = datp.e
;  Removed because normalise test if error exist, JO 20012-07-30        
;    if n_elements(e_in) ne n_elements(W_in) then $
;        e_in = sqrt(W_in)

	; set defaults            
	w_out = W_in*0.0
	n_out = n_in
	p_out = p_in
	pv_out= pv_in
	x_out = x_in
	y_out = y_in
	z_out = z_in
	e_out = e_in*1.0

    sizeWin    = size(W_in)
        
    NbSpectra  = sizeWin[1]
    NbZ        = sizeWin[2] 
    NbChannels = sizeWin[3] 
  
; ---------------------------------------------------------------------------------- 
;  Case where vanadium is for ELP search
; ---------------------------------------------------------------------------------- 
    elp_vana = 0 
    if (size(W_elp))[0] ne 0  then begin
        sizeVana       = size(W_elp)
        NbSpectraVana  = sizeVana[1]
        NbZVana        = sizeVana[2] 
        NbChannelsVana = sizeVana[3] 
        if ( NbSpectraVana ne NbSpectra ) or $
           ( NbZVana ne NbZ ) or $
           ( NbChannelsVana ne NbChannels ) then begin
            print,'DISTANCECORRECTION: Error: Call to vanadium, workspaces must have same format!' 
            w_out = W_in
            goto, exitJump   
        endif else begin
            elp_vana = 1   
            if keyword_set(verbose) then $
                print,'DISTANCECORRECTION: Distance correction according to VANADIUM elastic reference' 
        endelse
    endif 
    
; ---------------------------------------------------------------------------------- 
;  1) Compute the 1/Vo for scattering plane
;  2) Compute the 1/Vo for scattering out-of-plane     
;  3) Interpol out of plane onto the in-plane ToF
; ---------------------------------------------------------------------------------- 
;   Compute elastic peak position at Z
;   Order: [phi,Z,ToF]
; ---------------------------------------------------------------------------------- 
    if keyword_set(verbose) then $
        print,'DISTANCECORRECTION: Compute elastic peak position ...'
; ---------------------------------------------------------------------------------- 
    elp   = fltarr(NbZ)
    gauss = elp
    
    for i = 0,NbZ-1 do begin
        if elp_vana eq 1 then $
            w_buf   = total(W_elp[*,i,*],1) $
        else $
            w_buf   = total(W_in[*,i,*],1) 
                
	    W0      = max(w_buf,i0)
    	tofMin  = z_in[(i0 - 30) > 0]
	    tofMax  = z_in[(i0 + 30) < (NbChannels - 1)]
        ind     = where( z_in gt tofMin and z_in lt tofMax )  
        gauss   = str_fit( W_buf[ind],Bkgd=0,Xin = z_in[ind] )
        elp[i]  = gauss[*,2] - 1.0
;;        print, 'No ',i,' ElasticPeakPosition: ',elp(i)
    endfor
; ---------------------------------------------------------------------------------- 
;  1) Compute the 1/Vo for scattering plane and out-of-plane
;
;  WARNING: y_in in [cm] at ther momenty should be set to [m] in the future
; ---------------------------------------------------------------------------------- 
    Ro     = 4.0                            ; average in-plane distance [m]
    R      = sqrt(400.0^2 + y_in^2) / 100.  ; Each height R distance [m]
 	cwidth = p_in[18]*1e-6                  ; chw in sec
; ---------------------------------------------------------------------------------- 
    if keyword_set(verbose) then $
        print,'DISTANCECORRECTION: Interpolate to distance ',strtrim( Ro ,2),' m ...'
; ---------------------------------------------------------------------------------- 
; Quick and dirty correction: sometime at tube edges it cannot find an ELP 
; => min(elp) could be anywhere...
; Restrict to around the  middle of the tube
; ---------------------------------------------------------------------------------- 
;    ind       = where( elp(28:228) eq min(elp(28:228)) )       ; Find mid-plane (always shorter distance) 
    X = indgen(NbZ)
    ind       = where( elp eq min(elp) )                       ; Find mid-plane (always shorter distance) 
    elp_o     = elp[ind[0]]
    if keyword_set(verbose) then $
        print,'DISTANCECORRECTION: Elastic peak position: ',strtrim( elp_o ,2)
    
    indChannels = indgen(NbChannels)
	telast     = R  * p_in[21]/3956.035                                 ; TOF in sec for each height
    flt_time_o = Ro * p_in[21]/3956.035 - (elp_o - indChannels)*cwidth  ; flight time for Ro 
    invSpeed_o =  flt_time_o/Ro                                         ; inverse speed [ps/m]
     
; ---------------------------------------------------------------------------------- 
; Innitial loop (to be improved)
; ---------------------------------------------------------------------------------- 
;     for i = 0,NbZ-1 do begin
;          invSpeed =  ( telast(i) - (elp(i) - indChannels)*cwidth ) / R(i)
;          for ii=0,NbSpectra-1 do $        
;              W_out(ii,i,*) = interpol(W_in(ii,i,*), invSpeed, invSpeed_o)
;      endfor
;  
; ---------------------------------------------------------------------------------- 
;test loop: seems to work perfectly, including intensities, with the above algo
; ---------------------------------------------------------------------------------- 
    for j = 0,NbZ-1 do begin
        invSpeed =  ( telast[j] - (elp[j] - indChannels)*cwidth ) / R[j]
        ind    = VALUE_LOCATE(invSpeed_o, invSpeed) 
        indLim = where( ( ind ge 0 ) and ( ind lt NbChannels -1) ) 
        ind    = ind[indLim]
        for k=0,n_elements(indLim)-1 do begin
            i     = indLim[k] 
	    ind_k = ind[k]
	    PA    = ( invSpeed[i] - invSpeed_o[ind_k] )/( invSpeed_o[ind_k + 1] - invSpeed_o[ ind_k ] )
            unMoinsPA = 1.0 - PA
	    v                  = W_in[ *,j,   i]
	    W_out[*,j,  ind_k] = W_out[*,j,  ind_k] + v * unMoinsPA
	    W_out[*,j,ind_k+1] = W_out[*,j,ind_k+1] + v * PA
            ;  try to do similar for the error if any
            if (size(e_in))[0] gt 0 then begin
	        v                  = e_in[ *,j,   i]
	        e_out[*,j,  ind_k] = e_out[*,j,  ind_k] + v * unMoinsPA
	        e_out[*,j,ind_k+1] = e_out[*,j,ind_k+1] + v * PA
            endif
        endfor
    endfor
; ---------------------------------------------------------------------------------- 
    
    if keyword_set(verbose) then $
        print, 'DISTANCECORRECTION: Done in ',strtrim(round( 1000*(systime(1)-st1) ) ,2),' ms'

; ---------------------------------------------------------------------------------- 
; Spectra number in LAMP mode as expected by t2e 
; ---------------------------------------------------------------------------------- 
    z_out = findgen(NbChannels)+1
    mod_datp, datp, "x_tit", ' Scattering angles [deg]'
    mod_datp, datp, "y_tit", ' Tube height [cm]'
    mod_datp, datp, "z_tit", ' Time of flight channel'
    mod_datp, datp, "p", p_out 
    mod_datp, datp, "pv",pv_out
    mod_datp, datp, "x", x_out 
    mod_datp, datp, "y", y_out 
    mod_datp, datp, "z", z_out 
    mod_datp, datp, "e", e_out
    give_datp,datp

 exitJump:    

    return, W_out

end
