 FUNCTION in5_energy_rebin, w_in, dE=dE, Emin=Emin, $
                         force=force, $
                         keepelastic=keepelastic, $
                         verbose = verbose
;-------------------------------------------------------------------------------
;
;+
;
;<H2>NAME:</H2>
;
;   energy_rebin
;
;For IN4, IN5, IN6 and D7 data.
;
; rebins output workspace from t2e to constant energy bin width.
;
;
;<H2>ARGUMENTS:</H2>
;
; dE	: required energy bin width
;
;
;<H2>KEYWORDS:</H2>
;
; <b>/force</b>	: rebin over entire energy range
;		  otherwise: rebin only where dE is > point spacing (default)
; <b>/keepelastic</b>	: do not rebin over the elastic peak
;
; Emin = energy minimal to start the exit workspace.
;
;
;<H2>DIMENSIONS:</H2>
;
; w_in=(nE,nspectra) -> w_out(dE, nspectra)
;
;<H2>COMMAND SYNTAX:</H2>
;
; w10 = in5_energy_rebin(w9,dE=<dE>[Emin=-100.0, /force, /keepelastic])
;
; (optional keywords shown in square brackets)
;
;
;<h2>VERSION HISTORY:</h2>
;
;  KHA,JRS 16/03/05
;
;  Modifications: JOR, ??? add verbosity 
;                 JOR, Tue Nov 25 11:12:49 CET 2008: Improve output parameters 
;                                                   presentation
;                 JOR, Tue Jan 13 17:13:04 CET 2015 Add the 3D (single crystal case for ToF)
;
;-
;
;-------------------------------------------------------------------------------
forward_function string_round

iprint=0	; if iprint>0, show debugging messages

;	IF (iprint GT 0) THEN PRINT,'Start reb:'

take_datp, datp

; -------------------------------------------------------------------------------
;   Set up input workspace
;   
;   If 3D "single crystal" type of data
;   
;   w_in in format [En, Phi, Z]
;   x_in =- energy (meV), y_in = 2htheta (deg), Z = height (cm) 
; -------------------------------------------------------------------------------
par   = datp.p
w_out = 0

; Keep a version of the input to avoid dammages during process
w_sav = w_in
sw        = SIZE(w_in)
nchannels = sw[1]
x_in      = datp.x
e_in      = datp.e
se        = SIZE(e_in)
errors    = 1
threeD    = 0   ; Flag if 3D / single crystal
IF (se[0] NE sw[0] OR se[1] NE sw[1] OR se[2] NE sw[2]) THEN errors = 0

CASE sw[0] OF
  1: BEGIN
        nspectra = 1
  END
  2: BEGIN
        nspectra = sw[2]
  END
  3: BEGIN
        NbPhi = sw[2]
        NbZ   = sw[3]
        threeD = 1
        nspectra = LONG( NbPhi*NbZ )
        error = 1
        w_in = REFORM(w_in,  nchannels, NbPhi*NbZ, /OVERWRITE)
        e_in = REFORM(e_in,  nchannels, NbPhi*NbZ, /OVERWRITE)
     END
  ELSE: BEGIN
        PRINT, 'ENERGY_REBIN: ERROR: Unknown data type/size.'
        RETURN, w_in
  END
ENDCASE

; Nb points in energy
IF (N_ELEMENTS(dE) EQ 0) THEN BEGIN
    PRINT,'ENERGY_REBIN: ERROR: dE not specified'
    return, REFORM(w_in,  nchannels, NbPhi, NbZ, /OVERWRITE)
ENDIF

;	IF (N_ELEMENTS(force) EQ 0) THEN force = 0
IF keyword_set(verbose) THEN $
     PRINT,'ENERGY_REBIN: nchannels = ', strtrim(nchannels,2),' / nspectra = ', $
           strtrim(nspectra,2),' force = ', strtrim(force,2)

; -------------------------------------------------------------------------------
;   Determine elastic peak region only when "keepelastic" set
; -------------------------------------------------------------------------------
IF KEYWORD_SET(keepelastic) THEN BEGIN
    IF sw[0] GT 2 THEN BEGIN
        print, 'ENERGY_REBIN: "Keepelastic" does not work for single crystal'
        return, REFORM(w_in,  nchannels, NbPhi, NbZ, /OVERWRITE)
    ENDIF
    Ei=81.81/datp.p[21]^2
    x_buf=x_in
    w_buf=TOTAL(w_in,2)
    e_buf=SQRT(TOTAL(e_in^2,2))
    gauss  = FLTARR(4)
    dgauss = FLTARR(4)
    FITGAUSS, x_buf, w_buf, e_buf, -Ei/10., Ei/10., gauss, dgauss
    IF keyword_set(verbose) THEN PRINT,' Gauss pars =',gauss
    xElmin=gauss[2]-gauss[3]*2.355
    xElmax=gauss[2]+gauss[3]*2.355
    iEl=WHERE(x_buf GT xElmin AND x_buf LT xElmax)
    x_buf=x_in[iEl]
    w_buf=w_in[iEl,*]
    e_buf=e_in[iEl,*]
ENDIF

;-------------------------------------------------------------------------------
;   Set up rebin arrays
;-------------------------------------------------------------------------------
IF keyword_set(force) THEN $
    IF keyword_set(Emin) THEN $
        xmin = MAX([x_in[0],-abs(Emin)]) $
    ELSE $
        xmin = MAX([x_in[0],-100]) $
ELSE $
    xmin = x_in[0] 
        
xmax = x_in[nchannels-1]
dx_in = x_in
dx_in[1:nchannels-2] = (x_in[2:nchannels-1]-x_in[0:nchannels-3])/2.
dx_in[0]             = x_in[1]-x_in[0]	
dx_in[nchannels-1]   = x_in[nchannels-1]-x_in[nchannels-2]

x_in0 = FLTARR(nchannels+1)	; histogram
x_in0[1:nchannels-1] = (x_in[0:nchannels-2]+x_in[1:nchannels-1])/2.
x_in0[0] = x_in0[1]-(x_in[1]-x_in[0])
x_in0[nchannels] = x_in0[nchannels-1]+(x_in[nchannels-1]-x_in[nchannels-2])
nbin = LONG((xmax-xmin)/dE)+2

IF keyword_set(force)  THEN BEGIN
;		IF keyword_set(verbose) THEN PRINT,'force=1'
neverwider:
    i0 = 0
;		IF keyword_set(verbose) THEN PRINT,'rebin everything'
    xstart = dE*FLOAT(FIX(xmin/dE)-1)
    x_bin = FLOAT(INDGEN(nbin))*dE+xstart
ENDIF ELSE BEGIN
;		IF keyword_set(verbose) THEN PRINT,'force=0'
    wider = WHERE(dx_in GT dE, nw)
    IF (nw LE 0) THEN GOTO, neverwider
    x_bin = FLTARR(nbin)
    i0 = wider[nw-1]	
    x0 = x_in[i0]
    IF keyword_set(verbose) THEN PRINT,'ENERGY_REBIN: Not forced, rebin from E = ', strtrim(string(x0),2)
    x_bin[0:i0] = x_in[0:i0]
    n = FIX((xmax-x0)/dE)
    IF (x0 LT 0.) THEN $
        xstart = dE*FLOAT(FIX(x0/dE)+1) $
    ELSE $
        xstart = dE*FLOAT(FIX(x0/dE)+2)
    x_bin[i0+1:i0+n] = FLOAT(INDGEN(n))*dE+xstart
    x_bin[i0+1] = (x_bin[i0]+x_bin[i0+2])/2.
    nbin=i0+n+1
    x_bin=x_bin[0:nbin-1]
ENDELSE
dx_bin = x_bin
dx_bin[1:nbin-2] = (x_bin[2:nbin-1]-x_bin[0:nbin-3])/2.
dx_bin[0] = x_bin[1]-x_bin[0]	
dx_bin[nbin-1] = x_bin[nbin-1]-x_bin[nbin-2]

x_bin0 = FLTARR(nbin+1)	; histogram
x_bin0[1:nbin-1] = (x_bin[0:nbin-2]+x_bin[1:nbin-1])/2.
x_bin0[0]        = x_bin0[1]-(x_bin[1]-x_bin[0])
x_bin0[nbin]     = x_bin0[nbin-1]+(x_bin[nbin-1]-x_bin[nbin-2])

IF (iprint GT 0) THEN PRINT,'End of rebin array setup section'

; -------------------------------------------------------------------------------
;   Perform rebinning
; -------------------------------------------------------------------------------
w_out=FLTARR(nbin,nspectra)
IF (errors EQ 1) THEN e_out = FLTARR(nbin,nspectra)

IF (i0 GT 0) THEN BEGIN	; do not rebin first points
    w_out[0:i0-1,*] = w_in[0:i0-1,*]
    IF (errors EQ 1) THEN e_out[0:i0-1,*]=e_in[0:i0-1,*]
ENDIF

iptmin = 0
ysum = FLTARR(nspectra)	
esum2 = ysum

FOR ibin=i0,nbin-1 DO BEGIN
      xmin = x_bin0[ibin]	
      xmax = x_bin0[ibin+1]
      xsum = 0.		
      ysum[*] = 0.	
      esum2[*] = 0.
      FOR ipt = iptmin,nchannels-1 DO BEGIN
            x2 = x_in0[ipt+1]
            IF (x2 GT xmin) THEN BEGIN
                x1 = x_in0[ipt]
                IF (x1 GT xmax) THEN GOTO, nomore
                IF (x2 LT xmax) THEN xhi = x2 ELSE xhi = xmax
                xhi = MIN([x2,xmax])
                xlo = MAX([x1,xmin])
                deltax = xhi-xlo
                IF (deltax LT 0.0) THEN PRINT,'ENERGY_REBIN: Error in rebinning'
                xsum = xsum+deltax
                ysum = ysum+deltax*w_in[ipt,*]
                IF (errors EQ 1) THEN esum2 = esum2+(deltax*e_in[ipt,*])^2
            ENDIF
      ENDFOR
nomore:   
      IF (xsum EQ 0.) THEN BEGIN                                  ; no point in the new bin
          w_out[ibin,*] = 0.
          IF (errors EQ 1) THEN e_out[ibin,*] = -1.
      ENDIF ELSE BEGIN
          w_out[ibin,*] = ysum/xsum                                ; normalise intensity to x bin
          IF (errors EQ 1) THEN e_out[ibin,*] = SQRT(esum2)/xsum
      ENDELSE
      iptmin = (ipt-1)>0
ENDFOR

x_out = x_bin

; IF KEYWORD_SET(verbose) THEN PRINT,'End of rebinning section'

; -------------------------------------------------------------------------------
;   Remove zeroed points
; -------------------------------------------------------------------------------
IF (errors EQ 1) THEN BEGIN
    WHILE (TOTAL(e_out[0,*]) LT -FLOAT(nspectra)+0.01) DO BEGIN
        x_out=x_out[1:nbin-1]
        w_out=w_out[1:nbin-1,*]
        e_out=e_out[1:nbin-1,*]
        nbin=nbin-1
    ENDWHILE
    WHILE (TOTAL(e_out[nbin-1,*]) LT -FLOAT(nspectra)+0.01) DO BEGIN
        x_out=x_out[0:nbin-2]
        w_out=w_out[0:nbin-2,*]
        e_out=e_out[0:nbin-2,*]
        nbin=nbin-1
    ENDWHILE
ENDIF

FOR ispec=0,nspectra-1 DO BEGIN
    IF (TOTAL(w_in[*,ispec],1) EQ 0.) THEN BEGIN
        w_out[*,ispec]=0.
        IF (errors EQ 1) THEN e_out[*,ispec]=-1.
    ENDIF
ENDFOR

IF KEYWORD_SET(verbose) THEN PRINT,'End of remove zeros section'

; -------------------------------------------------------------------------------
;   Splice unbinned data back in
; -------------------------------------------------------------------------------
IF KEYWORD_SET(keepelastic) THEN BEGIN
    i1=WHERE(x_out LT xElmin)
    i2=WHERE(x_out GT xElmax)
    x_out=[x_out[i1],x_buf,x_out[i2]]
    w_out=[w_out[i1,*],w_buf,w_out[i2,*]]
    e_out=[e_out[i1,*],e_buf,e_out[i2,*]]
    IF iprint THEN HELP, x_out, w_out, e_out
ENDIF

if threeD THEN BEGIN
    w_out = REFORM(w_out, nbin, NbPhi, NbZ, /OVERWRITE)
;    w_out = REVERSE(w_out,2) ; not necessary in fact ??? strange ...
    e_out = REFORM(e_out, nbin, NbPhi, NbZ, /OVERWRITE)
;    e_out = REVERSE(e_out,2) ; not necessary in fact ??? strange ...
ENDIF

IF KEYWORD_SET(force) THEN BEGIN
  s = STRTRIM(STRING(x_out[i0]),2)
  Elim = string_round(s)
  range='where E > '+Elim+' meV'
ENDIF ELSE $
  range='everywhere'

IF KEYWORD_SET(verbose) THEN BEGIN
  PRINT, 'ENERGY_REBIN: Rebin '+range+' in range [',STRTRIM( STRING(x_out[0],FORMAT = '(F8.2)'),2 ),' , ',STRTRIM( STRING(x_out[N_ELEMENTS(x_out)-1],FORMAT = '(F8.2)'),2 ),'] meV'
  PRINT, 'ENERGY_REBIN: Rebin to constant energy bin width of ', STRTRIM( STRING(dE,FORMAT = '(F10.4)'),2 ),'meV.'
ENDIF
IF KEYWORD_SET(force)  THEN fs = ',/force' ELSE fs = ''

; Recover the initial w_in to remove damages during process
w_in = w_sav

; -------------------------------------------------------------------------------
;   Return parameters and exit
; -------------------------------------------------------------------------------
mod_datp, datp, "x", x_out
IF (errors EQ 1) THEN mod_datp, datp, "e", e_out
datp.other_tit = datp.other_tit+' -rb(dE='+strtrim( string(dE),2)+fs+')'
give_datp, datp

finished:
RETURN, w_out


END
