FUNCTION in5_group,w_in,ngroups, filename=filename, verbose=verbose
;--------------------------------------------------------------------------------
;
;+ 
;<h2>NAME:</h2>
;
;       group
;
;<h2>PURPOSES:</h2>
; Group spectra in scattering angle or Q in a smaller number of groups.
; Intensity is the averaged value.
;
;<h2>COMMAND SYNTAX:</h2>
;  w2 = sgroup(w1,10, /verbose) ; group spectra in 10 groups
;  w2 = sgroup(w1,filename='angles.ang', /verbose) ; group spectra according to the file
;                                                  ; 'angles.ang' 
;<h2>ARGUMENTS:</h2>
;<b> number of groups</b> : final group number
; <b>filename</b>         : name of a file where the spectra numbers are given 
;                           for each group.
;                           Angles must be in column and in LAMP  format, i.e., starting from 1 
;                           and going to 88 (90 before 2006).   
;                           e.g.:  1    7
;                                  8    15
;                                  ...   
;                           Means: group 1 include spectra 1 to 7, group 2, spectra 8 to 15, etc.
;                           Last value lower than the number of remaining groups
;                           after <b>remove_spectra</b>
;                           Bad groups must have been removed 
;                           with <b>remove_spectra</b> first  
; 
; <b>/verbose</b>         : more info during execution
;
;<h2>VERSION HISTORY:</h2>
;       Written by . S. Rols 11/01 
;       Revisions:  
;                    + S. Rols 10.08.02 rols@gdpc.univ-montp2.fr
;                    + M.Plazanet 2005: add the possibility to group according to 
;                      an angle group file 'angles.ang'
;                    + JOR 2006-11-22: Improved version for the group file, verbosity added.
;                    + JOR Wed Nov 16 07:34:53 CET 2011: Correction when file contain only 1 element.
;
;-
;--------------------------------------------------------------------------------

COMMON c_lamp_access, inst
COMMON printing, iprint, outstring

;--------------------------------------------
; Donnees experimentales
take_datp, datp
par  = datp.p
x_in = datp.x
y_in = datp.y
e_in = datp.e

nchannels = n_elements(x_in) 
nspectra  = n_elements(y_in)

;--------------------------------------------
; tests sur la dimensions des erreurs

sein=n_elements(e_in) & swin=n_elements(w_in)

if (sein ne swin) then begin
	e_in=SQRT(w_in)
	print,"GROUP: warning - errors were not calculated --> err = sqrt(w)"
endif

;------------------------------------------
; groups definition by file
;------------------------------------------
if keyword_set(filename) then begin
    angles = intarr(2,1000)
    dummy  = intarr(2)
    openr, fID, filename,/get_lun
    i = 0
    while not eof(fID) do begin
        readf, fID, dummy
        angles[*,i]= dummy
        i = i+1
    endwhile
    free_lun, fID
    angles = 0 > angles < nspectra  ; angles between the spectra numbers
    angles = angles[*,0:i-1] - 1    ; take account on the indices instead of spectrum number
    if (size(angles))[0] eq 1 then $
        ngroups = 1 $
    else $
        ngroups = (size(angles))[2]
endif

;--------------------------------------------
; Qqes tests sur ngroups et nspectra
;------------------------------------------

ngroups = round(ngroups)

if(ngroups le 0) then begin
    PRINT,'GROUP: Error - Number of groups MUST be > 0'
    return,w_in
endif

if(nspectra lt ngroups) then begin
    PRINT,'GROUP: Error - Number of groups MUST be <= initial number of spectra'
    return,w_in
endif

ratio = FIX(nspectra/ngroups)
rest  = nspectra - ratio*ngroups
if keyword_set(verbose) then begin
    print,'GROUP: Grouping ',strtrim(string(nspectra),2),' spectra in ',strtrim(string(ngroups),2),' groups (',strtrim(string(ratio),2),' spectra/group).'
endif

w_buf = FLTARR(nchannels,ngroups)
e_buf = FLTARR(nchannels,ngroups)
y_buf = FLTARR(ngroups)

if keyword_set(filename) then begin
    if keyword_set(verbose) then print, 'Group spectra according to file: ', filename
    for i=0,ngroups-1 do begin
        w_buf[*,i] = total(w_in[*,angles[0,i]:angles[1,i]],2)/( angles[1,i] - angles[0,i] + 1 )
        e_buf[*,i] = SQRT(total((e_in[*,angles[0,i]:angles[1,i]])^2,2))/( angles[1,i] - angles[0,i] + 1 )
        y_buf[i]   = total(y_in[angles[0,i]:angles[1,i]])/( angles[1,i] - angles[0,i] + 1 )
        if keyword_set(verbose) then $
            print,' Group No ',strtrim(string(i),2),' phi = ',strtrim(string(y_buf[i]),2),' deg contains: ',strtrim(string(angles[1,i] - angles[0,i] + 1),2),' spectra.' 
    endfor
endif else begin
    imin = 0 
    imax = 0
    for i=0,ngroups-1 do begin
        ajou=0
        if rest gt 0 then ajou = ratio else ajou = ratio-1
        if imin ne imax then imin=imax+1
        imax = imin + ajou
        w_buf[*,i] = total(w_in[*,imin:imax],2)/(ajou+1)
        e_buf[*,i] = SQRT(total((e_in[*,imin:imax])^2,2))/(ajou+1)
        y_buf[i]   = total(y_in[imin:imax])/(ajou+1)
        if keyword_set(verbose) then $
            print,'GROUP No ', string(i+1,format='(I2)'),' : [ Phi_min, Phi_max ] = ',  $
                              strtrim( string(y_in[imin], format='(F8.3)'),2), ', ', $
                              strtrim( string(y_in[imax], format='(F8.3)'),2), $
                             ' / average = ', strtrim( string(y_buf[i],format='(F8.3)'),2),' deg'
        rest = rest - 1
    endfor
endelse 

x_out=x_in
y_out=y_buf
e_out=e_buf
w_out=w_buf
mod_datp, datp, "e", e_out
mod_datp, datp, "y", y_out
give_datp, datp
return, w_out
end
