; -----------------------------------------------------------------------------------------------
;************************************************************************************************
;
;+
;
;<H2>NAME:</H2>
;     FUNCTION normalise, w_in, raw=raw, monitor=monitor, time=time,  $
;			  detector=detector, alldetectors=alldetectors, ei=ei, $
;			  madangles=madangles, zeroshift=zeroshift, nopo = nopo, $
;			f  inorm, ikeep
;
;For IN4, IN5, IN6, HET and D7 data.
;
;For IN4, IN5 and IN6:
; ---------------------
; Normalises raw data to monitor or counting time, depending on value of inorm.
; Finds the position of the elastic peak.
;
;<b>KEYWORDS:</b>
; /raw	        : no normlisation (error bars calculated)
; /monitor      : normalise data to 1000 monitor1 counts (DEFAULT)
; /time         : normalise data to counting time (will not work for summed data)
;
; (inorm and ikeep are obsolete, kept for backwards compatability)
;
;For HET:
; --------
; Normalises raw data to monitor 1 (must be monitor 1 for ISIS data).  Removes
; empty spectra.  User must input incident energy.
;
;<b>ARGUMENTS:</b>
; ei	: incident energy of measurement
;
;For D7:
; --------
; Normalises raw data to monitor (default) or counting time or individual
; detector.  Extracts every other spectrum if desired (=> nbSpectra=32).
; Recalculates detector angles based on a YIG calibration by default.
;
;<b>ARGUMENTS:</b>
; detector      : detector number to normalise to
; zeroshift     : angle in degrees of 2theta = 0
;
; (inorm and ikeep are obsolete, kept for backwards compatability)
;
;<b>KEYWORDS:</b>
; /raw	        : no normlisation (error bars calculated)
; /monitor      : normalise data to 1000 monitor1 counts (DEFAULT)
; /time	        : normalise data to counting time
; /alldetectors : supresses the removal of the odd numbered detectors
; /madangles    : supresses the recalculation of the detector angles using the
;		  current YIG calibration
;
;<b>DIMENSIONS:</b>
;   non-TOF data: w_out(nbSpectra,nphases,nruns)           -unless nphases is 1
;	TOF data: w_out(nbChannels,nbSpectra*nphases,nruns)
;
;<b>DCOMMAND SYNTAX:</b>
;  w2=normalise(w1[,/raw][,/monitor][,/time][,detector=#][,/alldetectors][,/rawangles][,ei=#])
;
;  (optional keywords/arguments shown in square brackets)
;
;
;<h2>VERSION HISTORY:</h2>
;
;  Writtne by:    KHA,JRS 29/7/04
;
;  Modifications: S. Rols 09/01 srols@anl.gov: Normalization for mibemol data and DCS 
;                 JO, ??? Add verbosity and use str_fit instead of fitgauss (due to)
;                         problems with the errors computation.
;                 JO, Tue Dec  9 18:12:50 CET 2008: correct normalise for the 3D raw data
;                                                   with the new PSDs. Add ELP calculation
;                                                   in that case.
;                 JO, Sat Oct 30 08:51:43 CEST 2010: Correction return w_in after error.
;                 JO, Thu Nov  4 09:51:19 CET 2010: Correction on ELP in parameter vector.
;                 JO, Fri Jul 27 17:17:21 CEST 2012: Add test on errors crwation in case of 
;                                                    errors created earlier, should not be re-created 
;                                                    but only normalised.
;-
;
; -----------------------------------------------------------------------------------------------
;************************************************************************************************
	FUNCTION in5_normalise, w_in, inorm, verbose=verbose
; -----------------------------------------------------------------------------------------------
;	FUNCTION normalise_tof, w_in, inorm, verbose=verbose
; -----------------------------------------------------------------------------------------------
	COMMON c_lamp_access, inst
	COMMON printing, iprint, outstring
	forward_function strfit
        
        iprint = 0  
        

	IF keyword_set(verbose) THEN PRINT,'NORMALISE: WARNING: Development version 2008-12-09.'
	IF keyword_set(verbose) THEN PRINT,'NORMALISE: Instrument is: ',strupcase(inst)
	take_datp, datp
; -------------------------------------------------------------------------------
;Set up start parameters

	mon  = datp.n
 	par  = datp.p
	x_in = datp.x
	y_in = datp.y
	z_in = datp.z
        e_in = datp.e
        e_out = e_in
    
; -------------------------------------------------------------------------------
;  Treat the case where the data are already normalised --> set wout to win
; -------------------------------------------------------------------------------
    if mean(mon) eq 1 then begin
        print,'NORMALISE: WARNING:  Data already normalised!'
;;	return, win
        w_out = w_in
        goto, finished
    endif
; -------------------------------------------------------------------------------
;  Treat the case where the vector monitor is not properly inserted in the Ni var
; -------------------------------------------------------------------------------
    if (size(mon))[0] EQ 0 then begin
        print,'NORMALISE: WARNING:  Monitor missing, data not normalised !'
        w_out = w_in
        goto, finished
    endif

    sizeWin = SIZE(w_in)

	IF sizeWin[0] EQ 1 THEN begin
        nbSpectra  = 1  
        nbChannels = sizeWin[1] 
    endif ELSE IF sizeWin[0] EQ 2 THEN begin
        nbSpectra  = sizeWin[2] 
        nbChannels = sizeWin[1]
    endif ELSE IF sizeWin[0] EQ 3 THEN begin
        nbSpectra = sizeWin[1]
        nbChannels = sizeWin[3]
    endif ELSE BEGIN 
;	    i = WIDGET_MESSAGE('NORMALISE: unknown data format',/ERROR)
        print, "NORMALISE: ERROR: unknown data format for W_in."
;;	    RETURN, w_in
            w_out = w_in
            goto, finished
	ENDELSE
 ; -------------------------------------------------------------------------------
;   For the new PSDs : [Phi, Z, ToF]
;   --> change x_in to fit with ToF as before
; -------------------------------------------------------------------------------
    if sizeWin[0] eq 3 then $
        x_in = z_in 

	norm = 1
	IF N_ELEMENTS(inorm) GE 1 THEN norm = inorm

;;	IF keyword_set(verbose) THEN PRINT, 'norm = ',strtrim(norm,2) 

	gauss  = FLTARR(4)
	dgauss = FLTARR(4)

; -------------------------------------------------------------------------------
;       Find elastic peak
;
; JO, Tue Dec  9 18:12:50 CET 2008
;  Elastic peak position cannot be computed before 'distanceCorrection'
;  Since we don't know if the distance corection is done before normalise, it
;  is no more possible to compute the average ELP.
;  ELP is set to min(ELP) corresponding to the minimal distance , 4.0m at the
;  center of the detectors.
;
; -------------------------------------------------------------------------------
;       elimine les petits angles scat. ang. < 10 deg (old IN5)
; -------------------------------------------------------------------------------
    IF inst EQ 'IN5' THEN $
        i = WHERE(y_in GT 12.) $
    ELSE $
        i = INDGEN(nbSpectra)
    xtot  = x_in
    if sizeWin[0] eq 3 then begin
        elp   = fltarr(sizeWin[2])
        gauss = elp
        for i = 0,sizeWin[2]-1 do begin
            ytot  = total(w_in[*,i,*],1)
	        W0    = max(ytot,i0)
    	    xmin  = xtot[(i0 - 30) > 0]
	        xmax  = xtot[(i0 + 30) < (nbChannels - 1)]
            ind   = where( x_in gt xmin and x_in lt xmax )  
            gauss = str_fit( ytot[ind],Bkgd=0,Xin = x_in[ind] )
            elp[i]  = gauss[*,2] - 1.0
        endfor
        ind  = where( elp eq min(elp) )    ; Find mid-plane (always shorter distance) 
        chel = elp[ind[0]]
    endif else begin
        ytot  = TOTAL(w_in[*,i],2)
        etot  = SQRT(ytot)
        y0    = MAX(ytot,i0)
        xmin  = xtot[ (i0-30) > 0 ]
        xmax  = xtot[ (i0+30) < (nbChannels-1) ]
        ind   = where(x_in gt xmin and x_in lt xmax)  
        gauss = str_fit(ytot[ind],Bkgd=0,Xin=xtot[ind])
        chel   = gauss[2] - 1.0
    end
    par[9] = chel   ; Set the ELP in the parameter vector

    if keyword_set(verbose) then $
        print, 'NORMALISE: Data elastic peak position   : ',strtrim( string(chel,format='(F6.2)'),2)
        
; -------------------------------------------------------------------------------
;   Normalization for mibemol data and DCS
;   added by S. Rols 09/01 srols@anl.gov
; -------------------------------------------------------------------------------
	CASE inst OF
	'mibemol':BEGIN
		normf = TOTAL(mon)/N_ELEMENTS(mon)
		normf = normf/1000.
		par[9] = chel
		mon = 0.*mon+1000 & datp.n = mon
		GOTO, mibdcs
		END
	'DCSasc':BEGIN
		normf = TOTAL(mon)
		normf = normf/1000.
		chel  = gauss[2] & par[10] = chel
		mon = 0.*mon+1000  & datp.n = mon
		GOTO, mibdcs
		END
	'focus':BEGIN
		normf = par[3]  ; FOCUS: monitor is a parameter , no dedicated spectrum
		normf = normf/1000.
		mon = 0.*mon+1000  & datp.n = mon
		GOTO, mibdcs
		END
	ELSE:
	ENDCASE
; ------------------------------------------------------------------------------
;   Find monitor peak, integrate and normalise
;
;  JO, Mon Feb  8 15:25:13 CET 2010: correct for xmon for 2D data
; ------------------------------------------------------------------------------
	IF NOT norm THEN $
		normf = 1. $
	ELSE IF norm EQ 1 THEN BEGIN
		if sizeWin[0] ge 3 then xmon = z_in else xmon = x_in 
		ymon = mon[*,0]
		emon = SQRT(ymon)
		y0   = MAX(ymon,i0)
		xmin = xtot[(i0 - 20) > 0]
		xmax = xtot[(i0 + 20) < (nbChannels - 1)]
                if sizeWin[0] ne 3 then ind = where(x_in gt xmin and x_in lt xmax)  $
                else  ind = where(z_in gt xmin and z_in lt xmax) 
                gauss = str_fit(ymon[ind],Bkgd=1,Xin=xtot[ind])
;;		FITGAUSS, xmon, ymon, emon, xmin, xmax, gauss, dgauss
		bkgd   = gauss[4]
		height = gauss[1]
		centre = gauss[2]
		sigma  = gauss[3]
                integral = gauss[7]
		irange  = WHERE(ABS(xmon - centre) LT 8.*sigma, nrange)
		peaksum = TOTAL(ymon[irange],1)
		bkgdsum = TOTAL(ymon,1) - peaksum
		bkgd    = bkgdsum/FLOAT(nbChannels - nrange)
		normf   = (peaksum - bkgd*FLOAT(nrange)) / 1000.
		datp.n  = 1.
	ENDIF ELSE IF norm EQ 2 THEN BEGIN
		IF inst EQ 'IN5' THEN normf = par[0] ELSE normf = par[0]/60.
	ENDIF

mibdcs:
; -------------------------------------------------------------------------------
;   If  normf is 0 but data not previouosly normalised
; -------------------------------------------------------------------------------
    if normf le 0.0 then begin
		print,'NORMALISE: ERROR:  Monitor is <= 0!, Abort...'
		return, w_in
    endif

   IF  keyword_set(verbose) THEN BEGIN
    ;      PRINT, 'NORMALISE: monitor bkgd   = ',strtrim(bkgd,2)
    ;      PRINT, 'NORMALISE: monitor height = ',strtrim(height,2)
       PRINT, 'NORMALISE: Monitor elastic peak position: ',strtrim( string(centre,format='(F6.2)'), 2 )
       PRINT, 'NORMALISE: Monitor FWHM                 : ',strtrim(string(sigma,format='(F6.3)'),2),' channels'
       PRINT, 'NORMALISE: Monitor integral             : ',strtrim( string(normf * 1000.,format='(F12.1)'),2)
   ENDIF
   
   w_out = w_in / normf
   if (size(e_in))[0] eq 0 then begin
        e_out = SQRT(w_in) / normf
        if keyword_set(verbose) then $
            print, 'NORMALISE: create the error matrix as sqrt(N).'
   endif else begin
        e_out = e_in / normf
        if keyword_set(verbose) then $
            print, 'NORMALISE: Errors already exist, just normalise.'
   endelse
      
; -------------------------------------------------------------------------------
;   Return parameters and exit
; -------------------------------------------------------------------------------
	s = STRTRIM(STRING(chel),2)  & i = STRPOS(s,'.') & chel  = STRMID(s,0,i[0]+3)
	s = STRTRIM(STRING(normf),2) & i = STRPOS(s,'.') & normf = STRMID(s,0,i[0]+3)

	CASE norm OF
		0: BEGIN
		   nors = '/raw'
		   nstring = 'not normalised. '
		   END
		1: BEGIN
		   nors = '/m'
		   nstring = 'normalised to '+normf+'/1000 M1 counts. '
		   END
		2: BEGIN
		   nors = '/t'
		   nstring='normalised to '+normf+' minutes. '
		   END
	ENDCASE
	normalisation = ' no('+nors + ' ,ce='+chel+')'
	cstring='Elastic channel=' + chel
	outstring=nstring+cstring

	datp.p = par

	s = datp.other_tit & i = RSTRPOS(s,' ') & n = STRLEN(s) & numor = STRMID(s,i+1,n-1)
	IF (STRPOS(numor,'>') EQ -1) THEN $
		numor = STRTRIM(STRING(FIX(par[10])),2)
	datp.other_tit = inst + ' #' + numor + normalisation

    
     if sizeWin[0] eq 3 then begin
 	    datp.x_tit = 'Scattering angle'
	    datp.y_tit = 'Detector height Z [cm]'
	    datp.z_tit = 'Channel number'
     endif else begin
 	    datp.x_tit = 'Channel number'
	    datp.y_tit = 'Scattering angle'
	    datp.z_tit = 'Counts / 1000 M1'
     end

	mod_datp,  datp, "e", e_out
	give_datp, datp

finished:
	RETURN, w_out
	END

