; ----------------------------------------------------------------------------------------------------------
;+
;  in5_outputspe
;
;  Create the .spe files to be used by ISIS Horace suite.
;
;
;
;  Usage:
;   outputspe, W1, W57, file = 'filename.spe', dZCells = 40, /par, /phx, /verbose
;
;   Keyword /phx : produces the (mSlice) .phx file
;           /par : produces the (Horace) .par file
;           dZCells: creates the vertical slicing (optional)
;           W57 : a workspace containing the 2D (nSpectra,nZ) mask. The mask can be 
;                 defined by hand or can be obtained using the "in5_mask" routine.
;
;    Incoming workspace: [hw,Phi] or [hw, Phi, Z] (single crystal)
;
; PHX structure:
;
;  (Phi,ToF): [detNumber, Obsolete, twoTheta', Psi, dTwoTheta, dPsi, detNumber]
;
;  PSD      : [detNumber, Obsolete, twoTheta', Psi, dTwoTheta, dPsi, detNumber]
;
;
; PAR structure:
;
;  (Phi,ToF): [detDistance, twoTheta', Beta, dTwoTheta, dBeta, detNumber]
;
;  PSD      : [detDistance, twoTheta', Beta, dTwoTheta, dBeta, detNumber]
;
; -
;
;  Written by: JRS, 2007-04-09
;  Changes   : JOR, Wednesday, June 24 2009 16:25:  Adapted for IN5 PSD ( and 3D data)
;                                                   produce .PHX files for use with mSlice
;              JOR, Mon Oct  5 09:39:43 CEST 2009:  Produce Horace .PAR file
;              JOR, Wed Jan 20 10:53:38 CET 2010:   Change indices for long int for large files     
;                                                   when no pixel grouping is made.  
;              JOR, Tue Apr 27 09:49:17 CEST 2010   Correct phi when no pixels grouping and z == 0.
;              JOR, Thu Oct 13 13:08:45 CEST 2011   Correct a bug on naming.
;              JOR, Thu Apr 26 17:35:56 CEST 2012   Add detector masking 'w_mask' to remove masked detectors
;                                                   after defining a 2D w_mask(nSpectra,nZ) from the vanadium 
;                                                   standard.
;              JOR, Tue Oct 23 15:34:32 CEST 2012   Small corrections for powders files.
;              JOR  Tue Sep 17 16:58:05 CEST 2013   Bug corerection on variables 'path' and 'file'
;              JOR, Wed Jan 25 10:07:24 CET 2017    Correct error in error writting: reverse e_in as w_in
;
; -----------------------------------------------------------------------------------------------------------------
PRO in5_outputspe, w_in, w_mask, file = file, dZCells = dZCells, phx = phx, par=par, $
                   wout=wout, $
                   verbose = verbose
    
    COMMON c_lamp_access, inst
    
    DEG2RAD = !pi/180.0
    RAD2DEG = 180.0/!pi

    st1 = systime(1)

    TAKE_DATP, datp
    p_in = datp.p    
    e_in = datp.e
    
    Dsd  = p_in[27] * 100.0             ;  distance sample-detector [cm], unit same as Z
    heightPerCell = 12.2407/1000.0      ; cell height; 1.224 cm/ cell ((3030 - 2*40)/241)  

    sizeWin   = size(w_in)
    if sizeWin[0] GE 3 then PSD = 1 else PSD = 0
    nChannels = sizeWin[1]
    nSpectra  = sizeWin[2]
    if PSD EQ 1 then nZ = sizeWin[3] else nZ = 0 ; WARNING: cas non traite en mask si pas PSD

; --------------------------------------------------------------------------------------------
;   Test if  Mask (through w_mask) is set up    
; --------------------------------------------------------------------------------------------
if PSD EQ 1 then begin
    sizeWmask = size(w_mask)
    if  sizeWmask[0] ne 0 then begin
        if ( sizeWmask[1] ne sizeWin[2] ) or ( sizeWmask[2] ne sizeWin[3] ) then begin
                print,'OUTPUTSPE: When calling a mask, workspaces must have same format!' 
                w_mask = fltarr( sizeWin[2],sizeWin[3] )+1
                if keyword_set(verbose) then begin
                print,'OUTPUTSPE: Create a dummy ',strtrim(string(sizeWin[2]),2),' x ',strtrim(string(sizeWin[3]),2),'  full mask.'
                endif
        endif        
    endif else begin
        w_mask = fltarr( sizeWin[2],sizeWin[3] )+1
        print,'OUTPUTSPE: No Mask, create a dummy ',strtrim(string(sizeWin[2]),2),' x ',strtrim(string(sizeWin[3]),2),'  full mask.'
    endelse    
endif
; --------------------------------------------------------------------------------------------
;   Test the positive direction of the 2Theta axis: 
;
;   if small indices are with the 
;   2Theta = 134.8, inverse the direction to have an increasing 2Theta (datp.y) 
;   vector [-11.8;+134.8]
; --------------------------------------------------------------------------------------------
if PSD EQ 1 then $
    if datp.y[1] GT datp.y[nSpectra-1] then begin
        w_in   = reverse(w_in,2)
        e_in   = reverse(e_in,2)
        datp.y = reverse(datp.y)
         if  sizeWmask[0] ne 0 then w_mask = reverse(w_mask,1)
    endif
    
; --------------------------------------------------------------------------------------------
;  First extract the filename and the path
; --------------------------------------------------------------------------------------------
i=strpos(file,'/',/REVERSE_SEARCH)
if i[0] NE -1 then path = strmid(file,0,i[n_elements(i)-1]+1) else path = ''

i = strpos(file,'.')   
if i[0] NE -1 then file = strmid( file,0,i[ n_elements(i) -1] ) else file = 'IN5_outputfile'

; --------------------------------------------------------------------------------------------
;  Total number of SQW groups
; --------------------------------------------------------------------------------------------
if PSD EQ 1 then IndNonZero = where(w_mask GT 0, NbNonZero)  

; --------------------------------------------------------------------------------------------
;  Total number of SQW groups
; --------------------------------------------------------------------------------------------
if keyword_set(verbose) then begin
    print, 'OUTPUTSPE: writting file ',file,'.spe ...'
    if PSD EQ 1 then $
        print,'OUTPUTSPE: Applying mask, ',STRTRIM(STRING(nSpectra*nZ - NbNonZero),2), $
              ' pixels discarded (',STRTRIM(STRING(100.0*(1 - float(NbNonZero)/float((nSpectra*nZ)))),2),'% of total)'
endif

; --------------------------------------------------------------------------------------------
;   Create the SPE file
; --------------------------------------------------------------------------------------------
OPENW, lun, file + '.spe', /GET_LUN

; --------------------------------------------------------------------------------------------
;  Header : Total number of SQW groups
; --------------------------------------------------------------------------------------------
if PSD EQ 1 then $
    PRINTF, lun, FORMAT = '(2I5)', NbNonZero, nChannels $
else $
    PRINTF, lun, FORMAT = '(2I5)', nSpectra, nChannels

; --------------------------------------------------------------------------------------------
; Store SQW accordingly
; 
; 2Theta (Phi) and Energy grid , no Psi grid
; Phi grid is as large as the total number of Phi * Psi pixels
; 
;
; JOR, Mon Apr 23 18:54:56 CEST 2012  Add the mask feature if w_mask is defined. To optimize
;                                     if it doesn't exist.
; --------------------------------------------------------------------------------------------
if PSD EQ 1 then begin
    dummy = fltarr(nSpectra * nZ)
;        dummy = fltarr(NbNonZero)
    for i = 0, nZ-1 do dummy[i*nSpectra:(i+1)*nSpectra-1] = datp.y
    dummy = dummy[IndNonZero]
endif else $
    dummy  = datp.y

; --------------------------------------------------------------------------------------------
st2 = systime(1)

PRINTF, lun, '### Phi Grid'
PRINTF, lun, FORMAT = '(8f12.4)', [dummy, 0.0]
PRINTF, lun, '### Energy Grid'
PRINTF, lun, FORMAT = '(8f12.4)', [datp.x, datp.x[n_elements(datp.x)-1]]


if PSD EQ 1 then begin
   FOR iphi = 0, nSpectra-1 DO BEGIN
        for iPsi = 0, nZ-1 do begin
            if w_mask[iphi,iPsi] then begin
               PRINTF, lun, '### S(Phi,w)'
               PRINTF, lun, FORMAT = '(8e11.3)', w_in[*,iphi, iPsi]
               PRINTF, lun, '### Errors'
               PRINTF, lun, FORMAT = '(8e11.3)', e_in[*,iphi, iPsi]
            endif
       endfor
   ENDFOR
endif else begin
   FOR iphi = 0, nSpectra-1 DO BEGIN
       PRINTF, lun, '### S(Phi,w)'
       PRINTF, lun, FORMAT = '(8e11.3)', w_in[*,iphi]
       PRINTF, lun, '### Errors'
       PRINTF, lun, FORMAT = '(8e11.3)', e_in[*,iphi]
   ENDFOR    
endelse
FREE_LUN, lun

if keyword_set(verbose) then $
    print, 'OUTPUTSPE: .spe done in ',strtrim(round( 1000*(systime(1)-st2) ) ,2),' ms'

; for test purposes
if PSD EQ 1 then begin
    wout = total(w_in,1)
    IndZero = where(w_mask EQ 0)
    wout[IndZero] = 0.
endif                   

; --------------------------------------------------------------------------------------------
; Create the appropriate PHX file
; --------------------------------------------------------------------------------------------
IF KEYWORD_SET(phx) THEN BEGIN
    ; ------------------------------------------------------------------------------------
    ;        PSD case
    ; ------------------------------------------------------------------------------------
    if PSD EQ 1 then begin
        if keyword_set(verbose) then $
            print, 'OUTPUTSPE: writting file in5psd_'+string(strtrim(nChannels,2))+'_'+string(strtrim(nSpectra,2))+'_'+string(strtrim(nZ,2))+'.phx ...'
        delphi = FLTARR(nSpectra)
        delpsi = fltarr(nZ)
        kk     = 0
        ; --------------------------------------------------------------------------------
        ; Compute all Psi values (same for all tubes)
        ; delPsi as Ps(i+1) - Psi(i) as Phi
        ; --------------------------------------------------------------------------------
        Psi    = 2*atan( 0.5 * datp.z/Dsd ) * RAD2DEG
        ; --------------------------------------------------------------------------------
        ; Write PHX file
        ; --------------------------------------------------------------------------------
	OPENW, lun, path+'in5psd_'+string(strtrim(nChannels,2))+'_'+string(strtrim(nSpectra,2))+'_'+string(strtrim(nZ,2))+'.phx', /GET_LUN
	PRINTF, lun, FORMAT = '(I6)', nSpectra * nZ
	FOR iphi = 0, nSpectra - 1 DO BEGIN
	    IF iphi LT nSpectra - 1 THEN delphi[iphi] = datp.y[iphi+1] - datp.y[iphi] $
            ELSE delphi[iphi] = delphi[iphi-1]
            for iPsi = 0, nZ - 1 do begin 
                if iPsi LT nZ - 1 then delpsi[iPsi] = Psi[iPsi+1] -  Psi[iPsi] $
                else delpsi[iPsi] = delpsi[iPsi - 1]      
		        PRINTF, lun, FORMAT = '(I10,1X,I10,1X,F10.4,1X,F10.4,F10.6,1X,F10.6,1X,I10)', $
		                kk+1,0,datp.y[iphi],Psi[iPsi],delphi[iphi], delpsi[iPsi],kk+1
                kk = kk + 1        
            endfor
	ENDFOR
    ; ------------------------------------------------------------------------------------
    ;        2Theta case
    ; ------------------------------------------------------------------------------------
    endif else begin
        if keyword_set(verbose) then $
            print, 'OUTPUTSPE: writting file in5Phi_'+string(strtrim(nChannels,2))+'_'+string(strtrim(nSpectra,2))+'.phx ...'
        if keyword_Set(dZCells) then $
            dPsi = 2.0 * atan( dZCells * heightPerCell / 2.0 / Dsd ) * RAD2DEG $
        else  begin
            print, 'OUTPUTSPE: Warning: dZ unknown (keyword: dZcells=<value>), set to 20 cells!'
            dPsi = 2.0 * atan( 20.0 * heightPerCell / 2.0 / Dsd ) * RAD2DEG 
        endelse
        delphi = FLTARR(nSpectra)
        ; --------------------------------------------------------------------------------
        ; Write PHX file
        ; --------------------------------------------------------------------------------
        OPENW, lun,  path+'in5Phi_'+string(strtrim(nChannels,2))+'_'+string(strtrim(nSpectra,2))+'.phx', /GET_LUN
        PRINTF, lun, FORMAT = '(I3)', nSpectra
        FOR iphi = 0, nSpectra - 1 DO BEGIN
            IF iphi LT nSpectra - 1 THEN $
                delphi[iphi] = datp.y[iphi+1] - datp.y[iphi] $
            ELSE $
                delphi[iphi] = delphi[iphi-1]
            PRINTF, lun, $
                    FORMAT = '(I10,1X,I10,1X,F10.4,1X,I10,F10.3,1X,F10.6,1X,I10)', $
                    iphi+1,0,datp.y[iphi],0,delphi[iphi], dPsi,iphi+1
        ENDFOR
    endelse    
    FREE_LUN, lun
ENDIF


; --------------------------------------------------------------------------------------
; Create the appropriate PAR file for use with Horace
; --------------------------------------------------------------------------------------------
IF KEYWORD_SET(par) THEN BEGIN
    ; ------------------------------------------------------------------------------------
    ;        PSD case only
    ; ------------------------------------------------------------------------------------
    if PSD EQ 1 then begin
        if keyword_set(verbose) then $
            print, 'OUTPUTSPE: writting file in5psd_'+string(strtrim(nChannels,2))+'_'+string(strtrim(nSpectra,2))+'_'+string(strtrim(nZ,2))+'.par ...' 
        ; -----------------------------------------------------------------------------------
        ; extend value for each bin (number = nZ * nSpectra)
        ; -----------------------------------------------------------------------------------
        phiPlan  = fltarr(NbNonZero)   ;  nSpectra*nZ
        z        = fltarr(NbNonZero)
        dphiPlan = FLTARR(NbNonZero)
        dz   = fltarr(NbNonZero)
        kkk  = 0L
        for k = 0L,nSpectra-1 do begin
            for kk = 0L,nZ-1 do begin
                if w_mask[k,kk] then begin
                    phiPlan[kkk] = datp.y[k]
                    z[kkk]   = datp.z[kk]      
                    kkk      = kkk + 1L
                endif
           endfor
        endfor
        phiPlan = phiPlan*DEG2RAD
        z       = z/100.0                   ; z in [m] 
        
        ind = where(z GT 0.)
        dz   = z[1L:N_ELEMENTS(z)-1L] - z[0L:N_ELEMENTS(z)-2L]
        ind  = where(dz gt 0.)           
        dz   = min(dz[ind]) 
        
        ;  ------------------------------------------------------------------------------------
        ;       dPhi calculus not reliable (needed when grouping)
        ;  ------------------------------------------------------------------------------------            
        dphiPlan = phiPlan[1:N_ELEMENTS(z)-1L] - phiPlan[0L:N_ELEMENTS(z)-2L] 
        ind      = where(dphiPlan gt 0.)           
        dphiPlan = min(dphiPlan[ind])

        ;  ------------------------------------------------------------------------------------
        ;       Compute the essential inverse transformations 
        ;  ------------------------------------------------------------------------------------
        ind  = WHERE(phiPlan LE !PI/2., nInd)
        phiReal = fltarr(NbNonZero)     ; nSpectra*nZ - discarded pixels
        beta    = fltarr(NbNonZero)     ; 
        R    = Dsd/100.0                ; i.e. p(27) but in [m]
        beta = ATAN(z, R*sin(phiPlan)) 
        ; Below Pi/2:
        phiReal[ind]  = asin( z[ind] / (sin(beta[ind]) * sqrt( R^2 + z[ind]^2) ) )
        ; Above Pi/2:
        phiReal[ind[nInd-1]+1:N_ELEMENTS(phiReal)-1]  = !PI - asin( z[ind[nInd-1]+1:N_ELEMENTS(z)-1]         $
                                                    / ( sin(beta[ind[nInd-1]+1:N_ELEMENTS(beta)-1] )         $
                                                    * sqrt(R^2 + z[ind[nInd-1]+1:N_ELEMENTS(z)-1]^2) ) )

        ;  ------------------------------------------------------------------------------------
        ;      Treat the NaN in phiReal: due to z == 0 (if no pixel grouping, a pixel at z = 0 exists)  
                    ;      The phiReal are all positive
        ;  ------------------------------------------------------------------------------------
        ind = where(z eq 0.0, ncount)
        if ind[0] gt 0 then phiReal[ind] = abs( phiPlan[ind] )

        ; -------------------------------------------------------------------------------------
        ;           Write PAR file
        ; -------------------------------------------------------------------------------------
                OPENW, lun,  path+'in5psd_'+string(strtrim(nChannels,2))+'_'+string(strtrim(nSpectra,2))+'_'+string(strtrim(nZ,2))+'.par', /GET_LUN
                PRINTF, lun, FORMAT = '(I6)', NbNonZero
                FOR i = 0L, NbNonZero - 1L DO BEGIN
                         PRINTF, lun, FORMAT = '(F10.4,1X,F10.4,F10.4,1X,F10.4,1X,F10.4,1X,I10)', $
                                 R, phiReal[i]*RAD2DEG, beta[i]*RAD2DEG, dphiPlan*RAD2DEG, dZ,i+1L
                ENDFOR
                FREE_LUN, lun
    ENDIF ELSE PRINT, 'OUTPUTSPE: Not a PSD, no PAR file output.'
endif  ; Par file     

    ; ----------------------------------------------------------------------------------       
    ;  Elapsed time
    ; ----------------------------------------------------------------------------------       
if keyword_set(verbose) then $
    print, 'OUTPUTSPE: Everything done in ',strtrim(round( 1000*(systime(1)-st1) ) ,2),' ms'

    

END
