; ------------------------------------------------------------------------------------
;         in5_phiGrouping
;         
;         
;         Group a given range in Q on the (Phi, energy/ToF) data in order to have a 
;         well defined Q range when summing on Phi (2 theta) data. 
;
;         ex:
;               w20 = in5_phiGrouping(w10, Q=0.5, dQ=0.2, /verbose)
;
;
; 
;     JO, Mon Feb  8 18:03:01 CET 2010
; -----------------------------------------------------------------------------------
FUNCTION in5_phiGrouping, w_in, Q=Q, dQ=dQ, verbose=verbose, help=help

    if keyword_set(help) then begin
        print,"  Group a given range in Q on the (Phi, energy/ToF) data in order to have a "
        print," well defined Q range when summing on Phi (2 theta) data. "
        print," "
        print,"  ex:"
        print,"                w20 = in5_phiGrouping(w10, Q=0.5, dQ=0.2, /verbose)"
        print," "
        goto, errorExit
    endif

	common c_lamp_access, inst

    DEG2RAD = !pi/180.
    RAD2DEG = 180./!pi

	sw=SIZE(w_in)
    
    if sw[0] gt 2 then begin
        print,'GROUPINGPHI: ERROR: Size of w_in not compatible with this function.'
        goto, errorExit
    endif
        
	take_datp, datp
	x_in = datp.x	& sx=SIZE(x_in)
	y_in = datp.y	& sy=SIZE(y_in)
    p_in = datp.p
    e_in = datp.e
	nx = sw[1]
	ny = sw[2]

    lambda = p_in[21]

    Qmin = (Q - dQ/2.) > 0.
    Qmax = (Q + dQ/2.) < 4*!pi/lambda * sin( y_in[ny-1] * DEG2RAD/2.0)
    
    Phimin = 2 * asin(Qmin*lambda/4/!pi) * rad2deg
    Phimax = 2 * asin(Qmax*lambda/4/!pi) * rad2deg
    
    ind = where(y_in ge Phimin and y_in lt Phimax)
    
    if ind[0] ge 0 then begin
        nSpectra = n_elements(ind)
        w_out = total(w_in[*,ind[0]:ind[n_elements(ind)-1]],2)/nSpectra
        e_out = sqrt( total( e_in[ *,ind[0]:ind[ n_elements(ind)-1] ]^2 ,2) )/nSpectra
        y_out = mean( y_in[ ind[0]:ind[n_elements(ind)-1] ] )
        x_out = x_in
        if keyword_set(verbose) then $
            print, format = '("GROUPINGPHI: <Q> = ",F4.2," in [",F4.2," ; ", F4.2,"] --> <Phi> = ",F6.2," in [",F5.1," ; ", F5.1,"] deg.")', $
                             Q, Qmin, Qmax, mean(y_in[ind]), y_in[ ind[0] ], y_in[ ind[n_elements(ind)-1] ]
    endif else begin
        print, format = '("GROUPINGPHI: ERROR: No data point in the specified Q-range: <Q> = ",F4.2," in [",F4.2," ; ", F4.2,"] --> <Phi> = ",F6.2," in [",F5.1," ; ", F5.1,"] deg.")',$
                         Q, Qmin, Qmax, mean([Phimin, Phimax]), Phimin, Phimax
        goto, errorExit    
    endelse

	mod_datp, datp, "x", x_out
	mod_datp, datp, "y", y_out
	mod_datp, datp, "e", e_out
    give_datp, datp
    
    return, w_out

errorExit:
    return, w_in

END

