;------------------------------------------------------------------------------
;
	FUNCTION in5_rescale_t, w_in, Told=Told, Tnew=Tnew, verbose=verbose
;
;+
;     rescale_t
; 
;For IN5 and IN6 data
;
; Rescales data taken at a temperature Told to a temperature Tnew, using
; the Bose thermal population factor. Data either in energy or time-of-flight
;
; ARGUMENTS
;
; Told	: Temperature of input data
; Tnew	: Desired rescaling temperature
;
;DIMENSIONS
; w_in(dE,nspectra)=w_out
;
; SYNTAX
; w10 = rescale_t(w9,Told=<Told>,Tnew=<Tnew>, [/verbose]
;
;
; HISTORY
;
; Created: 	KHA,JRS 7/8/00
; Modified: JOR, 2010-07-20 17:43:51 : Add verbosity, change flt to dbl for 
;                                      correction "error" at larger energy.
;									   Treat the case where data in ToF.
;                                      Debug for max values in Bose factor.
;
; -
;
;------------------------------------------------------------------------------
forward_function str_fit

	take_datp, datp

;-------------------------------------------------------------------------------
;Set up input workspace and energy parameters

	p_in = datp.p

	sw=SIZE(w_in)
	nchannels=sw[1]
	IF (sw[0] EQ 1) THEN nspectra=1     ELSE $
	IF (sw[0] EQ 2) THEN nspectra=sw[2] ELSE return,w_in

	x_in = datp.x
	y_in = datp.y
	e_in = datp.e
	se=SIZE(e_in)
	IF (se[0] NE sw[0] OR se[1] NE sw[1]) THEN e_in=SQRT(w_in)


; ---------------------------------------------------------------------------
;    If data in ToF , compute the energy data for better 
;    rescaling application        
; ---------------------------------------------------------------------------
	if ~strcmp(datp.x_tit[0], 'Energy Transfer (meV)', 4, /FOLD_CASE) then begin
    if keyword_set(verbose) then $
        print, 'RESCALE_T: WARNING: data in time channels -> compute energies.'    
      x_in = datp.x
      p_in = datp.p
    eelast = 81.8049/p_in[21]^2                 ; Ei in meV
    telast = p_in[27]*p_in[21]/3956.035         ; TOF in sec
    cwidth = p_in[18]*1e-6                      ; chw in sec
; ---------------------------------------------------------------------------
;           Find the elastic peak position for each group          
; ---------------------------------------------------------------------------
      if (size(w_in))[0] eq 2 then $
          ytot   = total(w_in,2) $
      else $
          ytot   = w_in
      y0    = max(ytot,i0)
      xmin  = x_in[(i0 - 30) > 0]
      xmax  = x_in[(i0 + 30) < (nchannels - 1)]
      ind   = where(x_in gt xmin and x_in lt xmax)  
      epp   = fltarr(nspectra)
      gauss = epp
      gauss = str_fit(w_in[ind,*],Bkgd=0,Xin=x_in[ind])
      epp   = gauss[*,2] - 1.0
      x_dummy = w_in*0.0
      flt_time = fltarr(nspectra)
      j = indgen(nchannels)
      for i=0,nspectra-1 do begin
          flt_time  = telast - (epp[i]-j)*cwidth
          lam       = flt_time*3956.035/p_in[27]
          x_dummy[*,i]= eelast - 81.8049/lam^2
      endfor
;---------------------------------------------------------------------------
;     interpolate to have same x at all angle
;---------------------------------------------------------------------------
      if (size(x_dummy))[0] gt 1 then begin
           sx   = size(x_dummy)
           hw = total(x_dummy,2)/sx[2]
      endif else hw = x_dummy
	endif else begin
      hw = datp.x
	endelse
	if keyword_set(verbose) then begin
		print, 'RESCALE_T: Perform temperature rescaling in range: [',strtrim( string(min(hw),FORMAT = '(F16.2)') ,2 ),',',strtrim( string(max(hw),FORMAT = '(F16.2)'),2 ),'] meV'   
		if max( abs(hw) ) gt 1000.0 then $
			print, 'RESCALE_T: WARNING: Rescaling might be wrong for large energy values (> +/-1000meV)!'
	endif

; -------------------------------------------------------------------------------------
;	Perform correction
; -------------------------------------------------------------------------------------

	kB=0.0861735	; Boltzmann constant in meV/K

	w_out = w_in
	e_out = e_in
	x_out = fltarr(nchannels)
	y_out = datp.y

	arg  = double( hw/(kB*Told) )
	arg  = -80.0 > arg < 80.0
	Bose = arg
	i=WHERE(arg NE 0.,n)
	Bose[i]=1./(1.-EXP(-arg[i]))
	i=WHERE(arg EQ 0.,n)
	IF (n GE 1) THEN Bose[i] = max( ABS(Bose) )
	i = WHERE(ABS(Bose) eq max( ABS(Bose) ),n)
	IF (n GE 1) THEN Bose[i]=Told
	Bold=Bose

	arg  = double( hw/(kB*Tnew) )
	arg  = -80.0 > arg < 80.0

	Bose = arg
	i=WHERE(arg NE 0.,n)
	Bose[i]=1./(1.-EXP(-arg[i]))
	i=WHERE(arg EQ 0.,n)
	IF (n GE 1) THEN Bose[i] = max( ABS(Bose) )
	i = WHERE(ABS(Bose) eq max( ABS(Bose) ),n)
	IF (n GE 1) THEN Bose[i]=Tnew
	Bnew=Bose

	corr=Bnew/Bold	; temperature correction factor

; -------------------------------------------------------------------------------------
; end of correction calculation
; -------------------------------------------------------------------------------------
	corr=corr#(FLTARR(1,nspectra)+1)

; --------------------------------------------------------------------------------
;  corr is now okay, but data should be always in energy to benefit from the zeros 
; at high energies.
; Work should then be done back and forth to get back the TOF data.
; ----------------------------------------------------------------------------------       
;       Transform S(2theta, ToF) into S(2theta,W) 
; ----------------------------------------------------------------------------------       
	eelast = 81.8049/p_in[21]^2                 ; Ei in meV
	telast = p_in[27]*p_in[21]/3956.035         ; TOF in sec
	cwidth = p_in[18]*1e-6                      ; chw in sec
    w_elp = w_in
    if (size(w_elp))[0] eq 2 then $
        ytot = total(w_elp,2) $
    else $
        ytot = w_elp
	y0    = max(ytot,i0)
    xmin  = x_in[(i0 - 30) > 0]
	xmax  = x_in[(i0 + 30) < (nchannels - 1)]
    ind   = where(x_in gt xmin and x_in lt xmax)  
    
    epp   = fltarr(nspectra)
	gauss = epp
    gauss = str_fit(w_elp[ind,*],Bkgd=0,Xin=x_in[ind])
    epp   = gauss[*,2] - 1.0
    x_dummy     = w_out*0.0
    flt_time    = fltarr(nspectra)
    j           = indgen(nchannels)
    ind   = where(epp gt xmin and epp lt xmax)  
    average_elp = mean(epp[ind])
   for i=0,nspectra-1 do begin
         flt_time  = telast - (epp[i]-j)*cwidth
         lam  = flt_time*3956.035/p_in[27]
         x_dummy[*,i] = eelast - 81.8049/lam^2
         bodge = 1.0e6/telast
        for jj=0,nchannels-1 do begin
            w_out[jj,i]= w_out[jj,i]*flt_time[jj]^4*bodge
            if n_elements(e_out) gt 1 then e_out[jj,i]=e_out[jj,i]*flt_time[jj]^4*bodge
        endfor
    endfor

; ----------------------------------------------------------------------------------       
;     END  Transform S(2theta, ToF) into S(2theta,W) 
;
;    Apply corrections ...
; ----------------------------------------------------------------------------------       
	w_out = w_out*corr
	e_out = e_out*corr


; -------------------------------------------------------------------------------------
;  ... and transform back data in S(2theta, ToF)
; -------------------------------------------------------------------------------------
   for i=0,nspectra-1 do begin
		flt_time  = telast - (epp[i]-j)*cwidth
        lam  = flt_time*3956.035/p_in[27]
        x_dummy[*,i] = eelast - 81.8049/lam^2
        bodge = 1.0e6/telast
        for jj=0,nchannels-1 do begin
            w_out[jj,i]= w_out[jj,i]/flt_time[jj]^4/bodge
            if n_elements(e_out) gt 1 then e_out[jj,i]=e_out[jj,i]/flt_time[jj]^4/bodge
         endfor
    endfor
; -------------------------------------------------------------------------------------
;  END ... and transform back data in S(2theta, ToF)
; -------------------------------------------------------------------------------------
	datp.e = e_out

	s = STRTRIM(STRING(FLOAT(Told)),2) & i=RSTRPOS(s,'.') & Told=STRMID(s,0,i+2)
	s = STRTRIM(STRING(FLOAT(Tnew)),2) & i=RSTRPOS(s,'.') & Tnew=STRMID(s,0,i+2)
	datp.other_tit = datp.other_tit+' rt('+Told+','+Tnew+')'

	give_datp, datp

	if keyword_set(verbose) then $
		PRINT,'RESCALE_T: Rescaled from T='+Told+'K to T='+Tnew+'K'

finished:
	RETURN, w_out
	END


