; -------------------------------------------------------------------------------------------------------------
;*************************************************************************************************************
;
;+
;
;<h2>NAME:</h2>
;
;	safcorr
;
;
;<h2>PURPOSES:</h2>
;
; Perform the Self Attenuation Factor corrections on data in w_in 
; similarly to INX routines.
;
; For slab samples, the calculation is identical to INX as taken from the Sears 
; calculations (V.F. Sears Adv. Phys. (1975)).
; For cylinders (full/hollow) the calculation is similar to those of J.Wuttke
; (ILL report WU....) and differs slightly formt the calculation from Rieutord 
; (ILL report RI.... 1991) as implemented in INX but the results are similar.
;
; In all cases, contrarily to the INx and SQW routines, the SAF corrections are 
; computed for each energy.
;
;
;<h2>ARGUMENTS:</h2>
;
; <b>sigma_a:</b> 	Absorption cross-section (barns)
; <b>sigma_s:</b>	Total scattering cross-section (barns)
; <b> rho:</b>		sample density (g/cm^3)
; <b> Mass:</b>		molar mass (g/mol)
; <b>Rin:</b> 		Inner radius (full/hollow cylinders). Unit: [mm]
; <b>Rout:</b>		outer radius (full/hollow cylinders). Unit: [mm] 
; <b>thick:</b> 	Slab thickness (flat plate -- slab -- samples). Unit: [mm]
; <b>angle:</b>		Slab setting angle (flat plate -- slab -- samples). Unit: [deg.]
;
;<h2>KEYWORDS:</h2>

; <b>verbose:</b>	Information during the process
; <b>corrections:</b>	Gives the Self Attenuation Factors instead of the corrected intensities 
;			Size = size(w_in)
; <b>hlp = hlp</b>	Gives this help
;
;
;<h2>DIMENSIONS:</h2>
;
; Default: w_in=w_out(nchannels, nspectra)
; If keyword '/corrections': gives the correction map Corr such as: 
; W_out = W_in/Corr instead of W_out.
;
;
;<h2>COMMAND SYNTAX:</h2>
;
; For a SLAB sample:
; w8=safcorr(w7,thick=0.5,angle=135.0,sigma_a=5.08,sigma_s=5.205,rho=5.96,Mass=50.941)
; Performs self-absorption / self-attenuation for w7 
; (here parameters are for vanadium slab at an angle of 135 deg) 
; (optional arguments shown in square brackets)
;
; For a cylinder (full or hollow) sample:
; w8=safcorr(w7,Rin=10.0,Rout=11.0,sigma_a=5.08,sigma_s=5.205,rho=5.96,Mass=50.941)
; For a full cylinder set Rin=0 or cancel the keyword in the routine:
; w8=safcorr(w7,Rout=11.0,sigma_a=5.08,sigma_s=5.205,rho=5.96,Mass=50.941)
;
; w3 = safcorr(w2,Rin=0,Rout=12,sigma_a=5.08,sigma_s=5.205,rho=5.96,Mass=50.941,/verbose,/corrections)
; for a vanadium full cylinder, gives the corrections factors instead of the corrected intensities
;
;
;<h2>VERSION HISTORY:</h2>
;
; Written by : J.Ollivier 2006-03-23
;
; Modified by: 
;
;               JO-2006-06-15: Corrected a bug 2 times win/H1 in safcyl
;               JO-2006-11-10: Compute energies. Data need no more to be in energy
;                              mode. Warning: all cases are not treated , MIBEMOL
;                              should not work.
;               JO-2008-11-25: Treat correctly the case where all is in time channel.
;                              Improve verbosity.
;
;               JO-2012-11-27: Introduce the w_van for energy transfo. for sample without well defined ELP.
;                              Correct the error propagation.
;
;               JO-2012-12-06: Correction for NaNs in full cylinder correction: add an Eps to Rin
;               JO-2013-01-30: Correction bug on error exit in datp
;
;-
;
; -------------------------------------------------------------------------------------------------------------
;*************************************************************************************************************
; -------------------------------------------------------------------------------------------------------------
;	safcyl
; -------------------------------------------------------------------------------------------------------------
FUNCTION safcyl, w_in, datp, Rin, Rout, sigma_a, sigma_s, Mass, rho,  verbose=verbose, $
		corrections=corrections

	COMMON c_lamp_access, inst

	t0=systime(/seconds)

	EPS  = 1.0e-8	 	    ; avoid problems with zeros 
	w_out= w_in*0.0 	    ; define exit matrices
	H1   = w_out
	x    =  datp.x     	    ;  means after t2e for energies
	phi  =  datp.y*!pi/180  ; y must be angles in degrees
        e_in = datp.e

	if strcmp(inst,'mibemol',4,/FOLD_CASE) then $
		ei = 81.805/datp.p[0]^2 $
	else $
		ei = 81.805/datp.p[21]^2

	if keyword_set(verbose) then $
	    print,'SAFCYL: Rin =: ',strtrim(Rin,2),' mm, Rout = ',strtrim(Rout,2),' mm'
; ---------------------------------------------------------------------------------------
;       Create steps in R (20 steps):
; ---------------------------------------------------------------------------------------
	dR     = (float(Rout) - float(Rin))/(20.0-1.0)
	Rbase  = Rin+indgen(20,/float)*dR 
; ---------------------------------------------------------------------------------------
;	Create steps in angle alpha (100 steps)
; ---------------------------------------------------------------------------------------
	dAlpha = 2.0*!PI/(100.0-1.0)
	alpha  = indgen(100,/float)*dAlpha
	R = Rbase#(fltarr(n_elements(alpha)*n_elements(phi))+1.0)
;	R = reform(R,n_elements(Rbase),n_elements(alpha),n_elements(phi))

; ---------------------------------------------------------------------------------
;	incident beam:
; ---------------------------------------------------------------------------------
	dummy = fltarr(n_elements(alpha)*n_elements(phi))
	ddum = cos(alpha)
	n_alpha = n_elements(alpha)
	for k=0,n_elements(phi)-1 do $
		dummy[k*n_alpha:(k+1)*n_alpha-1] = ddum
	x1  = Rbase#dummy
	ddum = sin(alpha)
	for k=0,n_elements(phi)-1 do $
		dummy[k*n_alpha:(k+1)*n_alpha-1] = ddum
	y1  = Rbase#dummy
	psi = x1*0.0
	R_out= Rout + psi
	R_in = Rin  + psi
	ind = where(x1 le 0.0)
	if ind[0] ge 0 then $
		psi[ind] = sqrt( R_out[ind]^2-y1[ind]^2 ) - sqrt( R[ind]^2-y1[ind]^2  )
	ind = where(x1 gt 0.0 AND abs(y1) le Rin)
	if ind[0] ge 0 then $
		psi[ind] = sqrt( R_out[ind]^2-y1[ind]^2 ) + sqrt( R[ind]^2-y1[ind]^2  ) - $
                         	2*sqrt( R_in[ind]^2-y1[ind]^2 )
	ind= where(x1 gt 0.0 AND abs(y1) gt Rin)
	if ind[0] ge 0 then $
		psi[ind] = sqrt( R_out[ind]^2-y1[ind]^2 ) + sqrt( R[ind]^2-y1[ind]^2  ) 
; ---------------------------------------------------------------------------------
;	scattered beam:
; ---------------------------------------------------------------------------------
	dummy = fltarr(n_elements(alpha)*n_elements(phi))
	for k=0,n_elements(phi)-1 do $
		dummy[k*n_alpha:(k+1)*n_alpha-1] = cos(alpha + !PI - phi[k])
	x2  = Rbase#dummy
	pso = x2*0.0
	R_out= Rout + pso
	R_in = Rin + pso
	for k=0,n_elements(phi)-1 do $
		dummy[k*n_alpha:(k+1)*n_alpha-1] = sin(alpha + !PI - phi[k])
	y2  = Rbase#dummy
	ind = where(x2 le 0.0)
	if ind[0] ge 0 then $
		pso[ind] = sqrt( R_out[ind]^2-y2[ind]^2 ) - sqrt( R[ind]^2-y2[ind]^2  )
	ind = where(x2 gt 0.0 AND abs(y2) le Rin)
	if ind[0] ge 0 then $
		pso[ind] = sqrt( R_out[ind]^2-y2[ind]^2 ) + sqrt( R[ind]^2-y2[ind]^2  ) - $
			         2*sqrt( R_in[ind]^2-y2[ind]^2 )
	ind= where(x2 gt 0.0 AND abs(y2) gt Rin)
	if ind[0] ge 0 then $
		pso[ind] = sqrt( R_out[ind]^2-y2[ind]^2 ) + sqrt( R[ind]^2-y2[ind]^2  )

; ---------------------------------------------------------------------------------------------
;	reshape for  summimg over energies
; ---------------------------------------------------------------------------------------------
	psi = reform(psi,n_elements(Rbase),n_elements(alpha),n_elements(phi))
	pso = reform(pso,n_elements(Rbase),n_elements(alpha),n_elements(phi))

; ---------------------------------------------------------------------------------------------
;	Create the attenuation coefficients matrices
; ---------------------------------------------------------------------------------------------
	e0 = 81.8049/1.8^2	; reference energy for the absorption coefficients
	avog = 6.022e-2 	; attenuation factors in mm^-2
 	ef = ei-x
	ainc = float(rho)/float(Mass)*avog*(sigma_s+sigma_a*sqrt(e0/ei))
	adif = float(rho)/float(Mass)*avog*(sigma_s+sigma_a*sqrt(e0/ef))

	t1=systime(/seconds)
	if keyword_set(verbose) then $
		print,'SAFCYL: elapsed time first part:', strtrim(t1-t0,2),' sec.'

	TS = fltarr(n_elements(x),n_elements(phi))
	exp_ainc_psi = exp(-ainc*temporary(psi))
	exp_adif_pso = exp_ainc_psi
	
	for k=0,n_elements(x)-1 do begin
		exp_adif_pso[*] = pso
		exp_adif_pso = -adif[k]*temporary(exp_adif_pso)
		exp_adif_pso = exp(temporary(exp_adif_pso))
		TS[k,*] = total(total(exp_ainc_psi*exp_adif_pso,2)*R,1)
	endfor
 	TS = TS/total(Rbase)/n_elements(alpha)
	if keyword_set(verbose) then $
		print,'SAFCYL: elapsed time in the loop:', strtrim(systime(/seconds)-t1,2),' sec.'

	if keyword_set(corrections) then begin
	    w_out = TS
            e_out = 0.
	endif else begin
	    w_out = w_in/TS
            e_out = e_in/TS
        endelse

	mod_datp,datp,'x',x
	mod_datp,datp,'y',phi*180/!PI	
        mod_datp,datp,'e',e_out
	give_datp,datp
    
	return, w_out

END
; -------------------------------------------------------------------------------------------------------
;	safslab
; -------------------------------------------------------------------------------------------------------
FUNCTION safslab, w_in, datp , thick, angle, sigma_a, sigma_s, Mass, rho, verbose=verbose, $
		  corrections=corrections

	COMMON c_lamp_access, inst

	EPS = 1.0e-8	        ; avoid problems with zeros 
        e_in = datp.e
	w_out = w_in*0.0        ; define exit matrices
	H1 = w_out
	x   =  datp.x           ;  means after transformation to energies
	phi =  datp.y           ; y must be angles in degrees
	phi = (fltarr(n_elements(x),1)+1.0)#phi ; extends phi for all points
    
	if strcmp(inst,'mibemol',4,/FOLD_CASE) then $
	    einc = 81.805/datp.p[0]^2 $
	else $
	    einc = 81.805/datp.p[21]^2
	einc = fltarr(n_elements(x),1)+einc
	einc = einc#(fltarr(n_elements(datp.y),1)+1.0)
	e0   = 81.805/1.8^2
	ef   = (einc-x)#(fltarr(n_elements(datp.y),1)+1.0)	
	avog = 6.022e-2 ; attenuation factors in mm^-1 (thick in mm)
	if keyword_set(verbose) then $
	    print,'SAFSLAB: Slab thickness: ',strtrim(thick,2),' mm'
 	ainc = float(rho)/float(Mass)*avog*(sigma_s+sigma_a*sqrt(e0/einc))
	adif = float(rho)/float(Mass)*avog*(sigma_s+sigma_a*sqrt(e0/ef))
;    ----------------------------------------------------------------------    
;	Attenuation factor "atten" + self-shielding for 1 layer sample
;	The multi-layer case (in, e.g., INX) is not treated here.
;    ----------------------------------------------------------------------    
        x1 = ainc/(sin(float(angle)*!PI/180.0) > EPS)*thick/2.0      ;  0<alpha<180 
        x2 = adif/(sin(abs(float(angle-phi))*!PI/180.0))*thick/2.0   ; !! half thickness in each case
	ind = where(sin((angle-phi)*!PI/180.0) ge 0.0)
	if ind[0] ge 0 then $ 
		H1[ind] = (exp(-2*x1[ind])-exp(-2*x2[ind]))/2.0/((x2[ind] - x1[ind]))
	ind = where(sin((angle-phi)*!PI/180.0) lt 0.0)
	if ind[0] ge 0 then $ 
		H1[ind] = (1.0 - exp(-2*x1[ind]-2*x2[ind]))/2.0/(x2[ind] + x1[ind])
        ind = where(abs((x2 - x1)/(x2 + x1)) lt EPS)
        if ind[0] gt 0 then $ 
    	H1[ind] = exp(-2*x2[ind]);

;	returned values:
	if keyword_set(corrections) then begin
	    w_out = H1
            e_out = 0.
	endif else begin
	    w_out = w_in/H1
            e_out = e_in/H1
	endelse
	mod_datp,datp,'x',x
;;	mod_datp,datp,'y',phi*180.0/!PI	
        mod_datp,datp,'e',e_out
	give_datp,datp
    
	return, w_out
		
END

; -------------------------------------------------------------------------------------------------------
;
;	safcorr  ( main routine )
;
; -------------------------------------------------------------------------------------------------------
FUNCTION in5_safcorr, w_in, w_van, $
		  sigma_a=sigma_a, sigma_s=sigma_s, $
		  Rin=Rin, Rout=Rout, $
		  thick=thick, angle=angle, $
		  rho=rho, Mass=Mass, $
		  verbose=verbose, $
		  corrections=corrections, $
		  help = help

	if keyword_set(help) then begin
		print,''		
		print,'SAFFCORR (function)'		
		print,''		
		print,'Perform the Self Attenuation Factor corrections on data in w_in  similarly to INX routines.'
		print,''		
		print,'Usage:'		
		print,'------'		
		print,'	Slab sample:'		
		print,'	 w8=safcorr(w7,thick=0.5,angle=135.0,sigma_a=5.08,sigma_s=5.205,rho=5.96,Mass=50.941)'		
		print,'	Hollow cylinder:'		
		print,'	 w3 = safcorr(w2,Rin=0,Rout=12,sigma_a=5.08,sigma_s=5.205,rho=5.96,Mass=50.941,/verbose)'		
		print,''		
		print,'Arguments:'		
		print,'----------'		
		print,'<b>sigma_a:</b> 	Absorption cross-section (barns)'		
		print,'<b>sigma_s:</b>	Total scattering cross-section (barns)'		
		print,'<b> rho:</b>	sample density (g/cm^3)'		
		print,'<b> Mass:</b>	molar mass (g/mol)'		
		print,'<b>Rin:</b> 	Inner radius (full/hollow cylinders). Unit: [mm]'		
		print,'<b>Rout:</b>	outer radius (full/hollow cylinders). Unit: [mm] '		
		print,'<b>thick:</b> 	Slab thickness (flat plate -- slab -- samples). Unit: [mm]'		
		print,'<b>angle:</b>	Slab setting angle (flat plate -- slab -- samples). Unit: [deg.]'		
		print,'<b>verbose:</b>	Information during the process'		
		print,'<b>corrections:</b> Gives the Self Attenuation Factors instead of the corrected intensities'		
		print,''		
		print,'<b>hlp = hlp</b>	This help'		
		print,''		
		return,-1
	end

	COMMON c_lamp_access, inst

;   for functions that has to appear later but are unknown from here when saving
;   using: save,/routines,'t2e',filename='t2e.sav'
;   Appesars to be unuseful later on when compiling correctly
    forward_function str_fit, safslab, safcyl 

    t0 = systime(/seconds)

;	IF keyword_set(verbose) THEN PRINT,'Start safcorr...'
; -------------------------------------------------------------------------------------------------------
;	Check for keywords	
; -------------------------------------------------------------------------------------------------------
; -------------------------------------------------------------------------------------------------------
; 	Sample is SLAB
; -------------------------------------------------------------------------------------------------------
	if keyword_set(thick) && keyword_set(angle) then begin
		IF keyword_set(verbose) THEN print,'SAFCORR: sample is SLAB'
		shape = 'slab'
	endif
	if keyword_set(thick) && ~keyword_set(angle) then begin
		print,'SAFCORR error: Angle is missing for SLAB'
		return,w_in
	endif
	if ~keyword_set(thick) && keyword_set(angle) then begin
		print,'SAFCORR error: Thickness is missing for SLAB'
		return,w_in
	endif
; -------------------------------------------------------------------------------
; 	Sample is cylinder (hollow)
; -------------------------------------------------------------------------------
	if keyword_set(Rin) && keyword_set(Rout) then begin
		IF keyword_set(verbose) THEN print,'SAFCORR: sample is HOLLOW CYLINDER'
		shape = 'hollow'
	endif
	if keyword_set(Rin) && ~keyword_set(Rout) then begin
		print,'SAFCORR error: Rout is missing for HOLLOW CYLINDER --> Abort: Wout = Win...'
		return,w_in
	endif
; -------------------------------------------------------------------------------
; 	Sample is cylinder (full)
; -------------------------------------------------------------------------------
	if ~keyword_set(Rin) && keyword_set(Rout) then begin
		IF keyword_set(verbose) THEN print,'SAFCORR: sample is FULL CYLINDER'
		shape = 'full'
                Rin = 0.01
	endif
; -------------------------------------------------------------------------------
; 	Check for other parameters
; -------------------------------------------------------------------------------
	if ~keyword_set(sigma_a) then begin
		print,'SAFCORR error: sigma_a requested! --> Abort: Wout = Win...'
		return,w_in
	endif
	if ~keyword_set(sigma_s) then begin
		print,'SAFCORR error: sigma_s requested! --> Abort: Wout = Win...'
		return,w_in
	endif
	if ~keyword_set(rho) then begin
		print,'SAFCORR error: density rho requested! --> Abort: Wout = Win...'
		return,w_in
	endif
	if ~keyword_set(Mass) then begin
		print,'SAFCORR error: Molar mass requested! --> Abort: Wout = Win...'
		return,w_in
	endif
	if ~keyword_set(shape) then begin
		print,'SAFCORR error: container type unknown, check parameters! --> Abort: Wout = Win...'
		return,w_in
	endif	
; -------------------------------------------------------------------------------
;	open data
; -------------------------------------------------------------------------------
    take_datp, datp
    p_in = datp.p
    x_in = datp.x
    y_in = datp.y
    e_in = datp.e
;	take_datp, datvan, /third
; -------------------------------------------------------------------------------
;	Check data concistancy and compute energies if necessary
; -------------------------------------------------------------------------------
    if ~strcmp(datp.x_tit[0], 'Energy Transfer (meV)', 4, /FOLD_CASE) then begin
        if keyword_set(verbose) then $
            print, 'SAFCORR: WARNING: data in time channels -> compute energies.'    
        nchannels = n_elements(x_in) 
        nspectra  = n_elements(y_in)
	    eelast    = 81.8049/p_in[21]^2                 ; Ei in meV
	    telast    = p_in[27]*p_in[21]/3956.035         ; TOF in sec
	    cwidth    = p_in[18]*1e-6                      ; chw in sec
; -------------------------------------------------------------------------------
;       Find the elastic peak position for each group          
; -------------------------------------------------------------------------------
        w_elp = w_in
        if  (size(w_van))[0] ne 0 then begin
           if ( (size(w_van))[1] ne (size(w_elp))[1] ) or $
              ( (size(w_van))[2] ne (size(w_elp))[2] ) then $
                  print,'SAFCORR: When calling vanadium, workspaces must have same format!' $
           else begin
              w_elp = w_van    
              if keyword_set(verbose) then $
                  print,'SAFCORR: Energy transformation according to VANADIUM elastic peak' 
           endelse
        endif else $
           print,'SAFCORR: Energy transformation according to SAMPLE elastic peak'
           
        if (size(w_elp))[0] eq 2 then $
            ytot   = total(w_elp,2) $
        else $
            ytot   = w_elp
	    y0    = max(ytot,i0)
        xmin  = x_in[(i0 - 30) > 0]
	    xmax  = x_in[(i0 + 30) < (nchannels - 1)]
        ind   = where(x_in gt xmin and x_in lt xmax)  
        epp   = fltarr(nspectra)
	    gauss = epp
        gauss = str_fit(w_elp[ind,*],Bkgd=0,Xin=x_in[ind])
        epp   = gauss[*,2] - 1.0
        x_dummy = w_elp*0.0
        flt_time = fltarr(nspectra)
        j = indgen(nchannels)
        for i=0,nspectra-1 do begin
            flt_time  = telast - (epp[i]-j)*cwidth
            lam       = flt_time*3956.035/p_in[27]
            x_dummy[*,i] = eelast - 81.8049/lam^2
;           In (rare) cases where the counting frame start before the neutron arrives on the sample
;           x_dummy = x_dummy(where(flt_time>0.0),*)
        endfor
; -------------------------------------------------------------------------------
;       interpolate to have same x at all angle
; -------------------------------------------------------------------------------
        if (size(x_dummy))[0] gt 1 then begin
            sx   = size(x_dummy)
           XIN = total(x_dummy,2)/sx[2]
        endif else x_in = x_dummy
	    mod_datp,datp,'x',XIN
	    give_datp,datp
	endif 
    if keyword_set(verbose) then begin
        print, 'SAFCOR: ELP in channel: ',strtrim( string(round(mean(epp)),FORMAT = '(I8)') ,2 )
        print, 'SAFCORR: Energy range: [',strtrim( string(min(XIN),FORMAT = '(F8.2)') ,2 ),',',strtrim( string(max(XIN),FORMAT = '(F8.2)'),2 ),'] meV'   
       
    endif
; -------------------------------------------------------------------------------
;	Check workspace sizes
; -------------------------------------------------------------------------------
	size_win=SIZE(w_in)

	nchannels = size_win[1]
	IF size_win[0] EQ 1 THEN nspectra = 1 ELSE nspectra = size_win[2]
	size_err_in=SIZE(e_in)

	IF (size_err_in[0] NE size_win[0]) OR $
	   (size_err_in[1] NE size_win[1]) OR $
	   (size_err_in[2] NE size_win[2]) THEN e_in = SQRT(w_in)

; -------------------------------------------------------------------------------
;	Start correction routine
; -------------------------------------------------------------------------------
	w_out = w_in
	if strcmp(shape, 'slab', 4, /FOLD_CASE) then $
		w_out = safslab(w_in, datp , thick, angle, sigma_a, sigma_s, Mass, rho, $
                                verbose=verbose, corrections=corrections)
	if strcmp(shape, 'full', 4, /FOLD_CASE) || $
	   strcmp(shape, 'hollow', 4, /FOLD_CASE) then $
		w_out = safcyl(w_in, datp , Rin, Rout, sigma_a, sigma_s, Mass, rho, $
                               verbose=verbose, corrections=corrections)

;	IF keyword_set(verbose) THEN PRINT,'... end of safcorr'

	if keyword_set(verbose) then begin
	    t1 = systime(/seconds)
	    dt = t1 - t0
	    print,'SAFCORR: Self attenuation corrections performed in ', strtrim(string(dt*1000.),2),' ms.'
	endif
; -------------------------------------------------------------------------------
; In case x is not in energy store it back in channels 
; -------------------------------------------------------------------------------
    mod_datp,datp,'x',x_in
    give_datp,datp
    
	RETURN, w_out

END


