FUNCTION in5_straight_2d, win, interpol = interpol, distance = distance, $
                           verbose = verbose, datp = datp
; ----------------------------------------------------------------------------------
;
;+
;
; NAME:
;
;  straight_2d
;
;
; USAGE:
;
;  w2 = straight_2d(w1, distance=400.0)
;
; ARGUMENTS:
;
;  distance: detector distance in [cm]
;  /interpol: set keyword if interpolation needed
;  /verbose: set keyword if verbosity required
;
;
;
; win  is a 2D diffractogram i.e. I(2theta,h) where h
;      is the physical height of the multidetector
; wout is a 2D diffractogram i.e. I(2theta,h)
; in which the DS cones should be straightened out
; For each detector height calculate a vector of 2theta
; angles and interpolate onto equatorial plane
; using INTERPOL
;
; h is the physical height of the multidetector
; r is the physical radius of the multidetector
; set them
;
;
; VERSION HISTORY:
;
; Johnson MR. 2005?
; DRD, 2006 improve speed
; JOR, Thu Sep 11 16:23:54 CEST 2008, "humanise" and adapt to IN5
;
;-
;
; ----------------------------------------------------------------------------------
FORWARD_FUNCTION interpol
sysTime1 = systime(1)
if (!version.release lt '5.3') then interpol=1 ;***VALUE_LOCATE function***

; ----------------------------------------------------------------------------------
; h = 30cm, defined in rdid_d2b.pro  and carried forward in Y
; h =  8cm, defined in reading Salsa and carried forward in Y
; h = 40cm, defined in reading D19   and carried forward in Y
; Values en cm !!!
; ----------------------------------------------------------------------------------
;  R = 130.0 ; D2B
R = 400.0 ; IN5
if n_elements(distance) eq 1 then R = distance ;for SALSA 100.0 , D19 74.3
; ----------------------------------------------------------------------------------
; If 3rd dimension : eg., time-of-flight
; ----------------------------------------------------------------------------------
if (size(win))[0]  eq 3 then sizeZ = ( size(win) )[3] - 1 else sizeZ = 0

; ----------------------------------------------------------------------------------
; get variables associated with win
; ----------------------------------------------------------------------------------
noDatp = 1 
if n_elements(datp) eq 0 then take_datp,datp else noDatp = 0
; ----------------------------------------------------------------------------------
; if x is bidim then take the middle. Not the best correction. to be improved!!!
; ----------------------------------------------------------------------------------
if (size(datp.x))[0] eq 2 then $
    phi = datp.x[*,((( size(datp.x) )[2])-1)/2] $
else $
    phi = datp.x

; ----------------------------------------------------------------------------------
; find index for 2theta=0 and start straightening from this point
; don't use negative angles
; ----------------------------------------------------------------------------------
indPhiGEZero = ( where(phi ge 0.) )[0]

RAD2DEG  = 180.0/!pi
DEG2RAD  = !pi/180.0

rCosPhi  =   R*cos(phi[indPhiGEZero:*]*DEG2RAD)
rSinPhi2 = ( R*sin(phi[indPhiGEZero:*]*DEG2RAD) )^2
y2       = datp.y^2

; ----------------------------------------------------------------------------------
; 2D array, number of pixels vertically defined here (21)
; ----------------------------------------------------------------------------------
nbVerticalPixels  = ( size(datp.y) )[1]
phiEquatorialMin  = fltarr(nbVerticalPixels) 
phiEquatorialMax  = phiEquatorialMin

; ----------------------------------------------------------------------------------
; get size of 1D array, and min and max values
; ----------------------------------------------------------------------------------
indPhiMax  = ( size(phi) )[1] - 1
phiMin     = phi[0]
phiMax     = phi[indPhiMax]
phiRange   = phiMax - phiMin

; ----------------------------------------------------------------------------------
; define output array and initialise it
; ----------------------------------------------------------------------------------
wout = win*0.
; ----------------------------------------------------------------------------------
; find how many times data has been accumulated in input workspace
; ----------------------------------------------------------------------------------
nbAccumulatedScans = 1 
errorsOk = 0
if n_elements(datp.e) gt 1 then begin 
   errorsOk  = 1
   Eout = Wout
   idx  = where(datp.e[*,nbVerticalPixels/2,0] gt 0.)
   if idx[0] ge 0 then nbAccumulatedScans = ceil( total(win[idx,nbVerticalPixels/2,0]/datp.e[idx,nbVerticalPixels/2]^2,0)/n_elements(idx) )
endif
if keyword_set(verbose) then begin
    print,'STRAIGHT_2D: Number of scans = ',strtrim(nbAccumulatedScans,2)
    print,'STRAIGHT_2D: straightening '  ,strtrim(nbVerticalPixels,2),' pixels from scatt. angle ',strtrim(phiMin,2),' deg. to ',strtrim(phiMax,2),' deg.'
endif
; ----------------------------------------------------------------------------------
; for each detector height calculate the 2theta vector
; ----------------------------------------------------------------------------------
for j = 0,nbVerticalPixels-1 do begin
   zq = sqrt( rSinPhi2 +  y2[j] )                          ; height -h/2 to +h/2
   phiEquatorial = atan(zq,rCosPhi)*RAD2DEG               ; calculate 2theta values for the equatorial plane phiEquatorial

   ; ----------------------------------------------------------------------------------
   ; define phi = 2theta vector onto which all data is to be interpolated
   ; ----------------------------------------------------------------------------------
   phiEquatorialMin[j] = phiEquatorial[0]                                         ; get first and
   phiEquatorialMax[j] = phiEquatorial[indPhiMax-indPhiGEZero]                    ; last 2theta angles
   indPhiEquatorialMin = ceil( indPhiMax*(phiEquatorialMin[j]-phiMin)/phiRange) > 0           ; find corresponding indices
   indPhiEquatorialMax  = floor(indPhiMax*(phiEquatorialMax[ j]-phiMin)/phiRange) < indPhiMax  ; for first and last 2theta
      
   ; ----------------------------------------------------------------------------------
   ; interpolate onto equatorial plane
   ; ----------------------------------------------------------------------------------
   phi_Z_Equatorial   = phi[indPhiEquatorialMin:indPhiEquatorialMax] 
   Nbphi_Z_Equatorial = n_elements(phi_Z_Equatorial) - 1
   if keyword_set(interpol) then begin
        for kk=0,sizeZ do begin
            v = win[indPhiGEZero:*,j,kk]
            v[0,*]   = v[1,*] 
            v[Nbphi_Z_Equatorial,*] = v[Nbphi_Z_Equatorial-1,*]  ; Will prevent from interpol bounds artifacts
            wout[indPhiEquatorialMin:indPhiEquatorialMax,j,kk] = INTERPOL[v,phiEquatorial,phi_Z_Equatorial] > 0.
	        if errorsOk then $
                Eout[indPhiEquatorialMin:indPhiEquatorialMax,j,kk]=INTERPOL[datp.e[indPhiGEZero:*,j,kk],phiEquatorial,phi_Z_Equatorial]>0.   ;Ok I know!
	    endfor
   endif else begin
        indInterval  = VALUE_LOCATE(phi_Z_Equatorial, phiEquatorial) 
        indSubscriptIntervals = where( (indInterval ge 0) and (indInterval lt Nbphi_Z_Equatorial) )   ;Subscript intervals to fill
        
        for k = 0,n_elements(indSubscriptIntervals)-1 do begin
	        i  = indSubscriptIntervals[k] 
            sk = indInterval[i] 
            ik = sk + indPhiEquatorialMin 
            PA = ( phiEquatorial[i] - phi_Z_Equatorial[sk] )/( phi_Z_Equatorial[sk+1] - phi_Z_Equatorial[sk] )
            
;           if k eq n_elements(indSubscriptIntervals)-1 then $
;                if keyword_set(verbose) then print, j,': ', i, sk, ik, phiEquatorial(i),phi_Z_Equatorial(sk), phi_Z_Equatorial(sk+1), PA
           
	        for kk = 0,sizeZ do begin
                v = win[indPhiGEZero:*,j,kk]
                wout[ik,j,kk]   = wout[ik,j,kk]   + v[i]*( 1.0 - PA )
	            wout[ik+1,j,kk] = wout[ik+1,j,kk] + v[i]*PA
	            if errorsOk then begin
                    e = datp.e[indPhiGEZero:*,j,kk]
                    Eout[ik,j,kk]  = Eout[ik,j,kk]  + e[i]*( 1.0 - PA )             ;Increase errors
	                Eout[ik+1,j,kk]= Eout[ik+1,j,kk]+ e[i]*PA
	            endif
	        endfor
        endfor
   endelse
   ; ----------------------------------------------------------------------------------
   ; zeroes the angles outside the interval 
   ; ----------------------------------------------------------------------------------
   if indPhiEquatorialMin gt 0 then wout[0:indPhiEquatorialMin-1,j,*]    = -1.
   if indPhiEquatorialMax  lt indPhiMax then wout[indPhiEquatorialMax+1:indPhiMax,j,*]= -1.
endfor

if errorsOk then datp.e = Eout
if keyword_set(verbose) then $
    print, 'STRAIGHT_2D: Straightening done in ',strtrim( systime(1) - sysTime1 ,2),'s'

if noDatp and errorsOk then give_datp,datp
return, wout
end
