function in5_t2epsd,w_in, w_van, elp=elp, average_elp=average_elp, verbose=verbose
; --------------------------------------------------------------------------------------------------------
;+
;
;<h2>NAME:</h2>
;
;	t2epsd 
;
;<h2>PURPOSES:</h2>
; Converts array w_in  to energy in w_out fpr IN5 PSD in single Xtal mode
; Based on the original "tee" procedure instead of on "t2e". 
;
;<h2>COMMAND SYNTAX:</h2>
;	w2 = t2epsd(w1)
;         convert tof spectrum in w1 to energy and output in w2
;         Energies are in x2.
;
;<h2>ARGUMENTS:</h2>
; (all optional)
;
; <b>/average_elp</b>     : take average elp instead of 
; <b> W_van</b>           : Workspace for the (raw) vanadium that gives the elastic peak reference 
; <b>/verbose</b>         : more info during execution 
; 
;<h2>DIMENSIONS:</h2>
;
; w_in = w_out(nchannels, nspectra, nZ)
;
;<h2>VERSION HISTORY:</h2>
;
;	Written by  GJK 1994
; 	Revisions:     JOR 2004-11-03 Include other Tof instruments (e.g.: MIBEMOL, LLB)
;		           JOR 2006-03-28 Html help, more verbosity
;                  JOR 2006-10-23 tee -> t2e with the same properties 
;                                in case the original t2e fails.
;                  JOR 2006-11-10 gaussian fit with str_fit instead of fitgauss (due to 
;                                problems when error bars are not computed), with the same restriction
;                                to the elastic peak than before +/- 30 channels from  
;                                mean elastic maximum.
;                  JOR 2006-12-14 Reference of the elastic line by the vanadium (keyword: Wvan)
;                                Vanadium data must be in the same format than sample data.
;                  JOR 2008-07-01 This version T2EPSD adapted for IN5 PSD but other instruments
;                                removed for clarity.
;                  JOR 2011-09-14 Correction error on the ELP. Add user defined ELP when not enough
;                                ELP and no vanadium.  ELP might be average after distance correction.
;                  JOR Jul  5 17:37:46 CEST 2012: Minor correction on the verbosity of ELP when elp or average_elp set
;                  JOR Thu Nov 21 15:34:21 CET 2013: Correct the routine for absolute normalisation. 
;                                                    Using both vnorm with abs. norm. and t2epsd gives intensities
;                                                    in Barns/Sr/meV.
;
;-
;
; --------------------------------------------------------------------------------------------------------
;;	@lamp.cbk

; for functions that has to appear later but are unknown from here when saving
; using: save,/routines,'t2e',filename='t2e.sav'
forward_function str_fit

    st1=systime(1)

    COMMON c_lamp_access, inst

    take_datp, datp

; --------------------------------------------------------------------------------------------------------
;
;  Win order is: [Phi, z, ToF] after regularise
;
; --------------------------------------------------------------------------------------------------------
    sizeWin = size(w_in)
    
    if sizeWin[0] NE 3 then begin
        print, 'T2EPSD: ERROR: Data are not 3D PSD data, use t2e instead!' 
        return, w_in
    endif    
    
    x = 0.0
    y = 0.0
    z = 0.0
    w = 0.0
    n_in = datp.n
    pv_in= datp.pv     
    p_in = datp.p       
    x_in = datp.x
    y_in = datp.y
    z_in = datp.z
    e_in = datp.e
    if n_elements(e_in) ne n_elements(w_in) then $
        e_in = sqrt(w_in)
; --------------------------------------------------------------------------------------------------------
;
;  Reorder W_out as: [ToF, Phi, z] 
;
; --------------------------------------------------------------------------------------------------------
    w_out = transpose(w_in,[2,0,1])    

    ; set defaults
    n_out = n_in
    p_out = p_in
    pv_out= pv_in
    x_out = z_in
    y_out = x_in
    z_out = y_in
    e_out = transpose(e_in*1.0,[2,0,1]) 
	
    sizeWout  = size(w_out)	
    nchannels = sizeWout[1]
    nspectra  = sizeWout[2]
    nz        = sizeWout[3]     
       
	coef  = fltarr(nchannels)
	x_out = fltarr(nchannels,nspectra, nz)
	if n_elements(e_in)  ne n_elements(w_in) then e_out=0
	
; --------------------------------------------------------------------
;
;       Convert to Energy
;
; --------------------------------------------------------------------
	Ei = 81.8049/p_in[21]^2                     ; Ei in meV
	telast = p_in[27]*p_in[21]/3956.035         ; TOF in sec
	cwidth = p_in[18]*1e-6                      ; chw from mu-sec to sec

    if keyword_set(verbose) then begin
		print, 'T2EPSD: Instrument is                 : ',strupcase(inst)
		print, 'T2EPSD: Sample-detector distance      : ',strtrim(string(p_in[27],format='(F8.2)'),2),' m'
    endif        
; --------------------------------------------------------------------
;       Find the elastic peak position for each group     
;       ELP coming possibily from the vanadium (or low temp) run  
; --------------------------------------------------------------------
    w_elp = total(w_out,3)
    if  (size(w_van))[0] ne 0 then begin
        if ( (size(w_van))[1] ne (size(w_out))[1] ) or $
           ( (size(w_van))[2] ne (size(w_out))[2] ) then $
            print,'T2EPSD: When call to vanadium, workspaces must have same format!' $
        else begin
            w_elp = w_van    
            if keyword_set(verbose) then $
               print,'T2EPSD: Transformation according to VANADIUM elastic peak' $
            else $
               print,'T2EPSD: Transformation according to SAMPLE elastic peak'   
        endelse    
    endif
    ytot  = total(w_elp,2) 
    y0    = max(ytot,i0)
    xmin  = z_in[(i0 - 30) > 0]
    xmax  = z_in[(i0 + 30) < (nchannels - 1)]
    ind   = where(z_in gt xmin and z_in lt xmax)  
    epp   = fltarr(nspectra)
    gauss = epp
    gauss = str_fit(w_elp[ind,*],Bkgd = 0,Xin = z_in[ind])
    epp   = gauss[*,2] - 1.0

; ----------------------------------------------------------------------------------       
;       Transform into S(2theta,W) 
; ----------------------------------------------------------------------------------       
    x_dummy     = w_out*0.0
    j           = indgen(nchannels)

    if keyword_set(average_elp) then begin
        ind         = where(epp gt xmin and epp lt xmax)  
        average_elp = mean(epp[ind])
       if keyword_set(verbose) then  $
            print, 'T2EPSD: Elastic peak position forced to (AVERAGE_ELP): ', strtrim(string( average_elp ),2)   
    endif 
    
    if keyword_set(elp) then begin
        average_elp = elp 
        if keyword_set(verbose) then $
           print, 'T2EPSD: Elastic peak position forced to user value (ELP) : ', strtrim(string( elp ),2)
    endif
            
   if keyword_set(verbose) then begin
      if keyword_set(elp) then $
           print, 'T2EPSD: Average elastic peak position : ',strtrim(string( round( average_elp ) ),2) $
      else $
           print, 'T2EPSD: Average elastic peak position : ',strtrim(string( round( mean(epp[ind]) ) ),2) 
           print, 'T2EPSD: Incident energy               : ',strtrim( string(Ei) ,2),' meV'
           print, 'T2EPSD: Channel width                 : ',strtrim(string(cwidth*1e6),2),' mu-sec'
   endif

    for i=0,nspectra-1 do begin
         if keyword_set(average_elp) or keyword_set(elp)then $
            flightTime  = telast - (average_elp-j)*cwidth $
         else $
             flightTime  = telast - (epp[i]-j)*cwidth
         
         Ef   = Ei*(telast/flightTime)^2
         dtdE = flightTime/(2.*Ef)  * 1e6  ; to be concistent with vnorm absolute calcualtion
         kikf = SQRT(Ei/Ef)                ; all times above ccalculated in sec.
              
        for jj=0,nz-1 do begin
            x_dummy[*,i,jj] = Ei - 81.8049/(flightTime*3956.035/p_in[27])^2
            w_out[*,i,jj]   = w_out[*,i,jj] * dtdE * kikf
            if n_elements(e_out) gt 1 then $
               e_out[*,i,jj] = e_out[*,i,jj] * dtdE * kikf
        endfor
    endfor

; ----------------------------------------------------------------------------------       
;       interpolate to have same x at all angle
; ----------------------------------------------------------------------------------       
    sx    = size(x_dummy)
    x_out = total(total(x_dummy,3),2)/( sx[2]*sx[3] )
    for j = 0,sx[2]-1 do begin
        for jj = 0,sx[3]-1 do begin
            w_out[*,j,jj] = interpol(w_out[*,j,jj],x_dummy[*,j,jj],x_out)
            e_out[*,j,jj] = interpol(e_out[*,j,jj],x_dummy[*,j,jj],x_out)
        endfor
    endfor
; ----------------------------------------------------------------------------------       
; To avoid data greter than 1000 meV that are not written in the INX format data files
; ----------------------------------------------------------------------------------       
    ind = where(abs(x_out) lt 999.0)
    x_out = x_out[ind]
    w_out = w_out[ind,*,*]       
    e_out = e_out[ind,*,*]       

; ----------------------------------------------------------------------------------       
;  Elased time
; ----------------------------------------------------------------------------------       
    if keyword_set(verbose) then $
        print, 'T2EPSD: Done in ',strtrim(round( 1000*(systime(1)-st1) ) ,2),' ms'

; ----------------------------------------------------------------------------------       
;       Copies work-space stuff from simple arrays back to main LAMP arrays
; ----------------------------------------------------------------------------------       
    ; After the for loop
;    print, ' --------------------------------------------------------'
;    print, '  After the energy homogeneisation (exit routine)'
;    print, '  T2EPSD: total wout * dedT * kikf  = ',strtrim(total(double(w_out)),2)
;    print, '  T2EPSD: mean  wout * dedT * kikf  = ',strtrim(total(mean(double(w_out))),2)
;    print, ' --------------------------------------------------------'

    mod_datp, datp, "x_tit", ' Energy [meV]'
    mod_datp, datp, "y_tit", ' Scatt. Ang. [deg]'
    mod_datp, datp, "z_tit", ' det.height [m]'
    mod_datp, datp, "p", p_out 
    mod_datp, datp, "pv",pv_out
    mod_datp, datp, "x", x_out 
    mod_datp, datp, "y", y_out 
    mod_datp, datp, "z", z_out 
    mod_datp, datp, "e", e_out
    give_datp,datp
    return, w_out
   
end

