;+
;
; read_mupho
; 
; Read the input_mupho.txtx inut files for MUPHOCOR routine
; and exit the "Z00" time-of-flight distribution in a workspace.
;
;  Example:
;
;  w1 = read_muffle('input_mupho.txt')
;  
;  
;  
; :Author: ollivier 2016-12-23
;-
FUNCTION read_mupho, file=file

  fmt20="(' PROBLEM:',A)"
  fmt30="(16I5)"
  fmt40="(8F10.4)"
  fmt45="(' *********************************************************"+ $
    "*************************************************************')"

  u=-1 
  OPENR,u,file,/get_lun
  TITLE='' 
  READF,u,TITLE  
  PRINT,'' & PRINT,TITLE,format=fmt20
  
  ; ---------------------------------------------------------------------------
  ;     E0:INC. ENERGY(MEV)             FP:FLIGHTPATH(CM)
  ;     XNEL:CHANNEL OF ELASTIC LINE    CW:CHANNEL WIDTH(MYS)
  ;     FIMI:MIN.SCATT.ANGLE            FIMA:MAX.SCATT.ANGLE
  ;     UNT:CONST.BACKGROUND
  ; ---------------------------------------------------------------------------
  E0=0. & FP=0. & CW=0. & XNEL=0. & FIMI=0. & FIMA=0. & ABK=0. & AEMP=0.
  READF,u ,E0,FP,CW,XNEL,FIMI,FIMA,ABK,AEMP

  ; ---------------------------------------------------------------------------
  ;     NSPEC: NUMBER OF DIFFERENT ATOMIC SPECIES
  ; ---------------------------------------------------------------------------
  LM=0L & NSPEC=0L & IQUO=0L & LPAR=0L
  READF,u,LM,NSPEC,IQUO,LPAR ;,format="(4I5)"

 
  ; ---------------------------------------------------------------------------
  ;     HOX:   UPPER LIMIT OF DENSITY OF STATES
  ;     TEMPO: TEMPERATURE IN KELVIN
  ;     DW:    MEAN DEBY WALLER COEFFICIENT
  ; ---------------------------------------------------------------------------
  HOX=0. & TEMP0=0. & DW=0.
  READF,u,HOX,TEMP0,DW ;,format="(3F10.4)"

  ; ---------------------------------------------------------------------------
  ;     AMASI: ATOMIC MASS
  ;     SIGI : SIGMA
  ;     CONC:  CONCENTRATION
  ;     ALFI:  SCATTERING POWER
  ; ---------------------------------------------------------------------------
  tmp=[0.,0.,0.]
  SIGI =fltarr(2) & CONCI=SIGI & ALFI=SIGI & AMASI=SIGI
  FOR I=0,NSPEC-1 DO BEGIN
    READF,u,tmp 
    AMASI[I]=tmp[0] 
    SIGI[I]=tmp[1]
    CONCI[I]=tmp[2]
  ENDFOR
 

  IF (NSPEC EQ 2) THEN BEGIN
    tmp=fltarr(LPAR)
    READF,u,tmp
    PARQ[0]=tmp
  ENDIF


  ; ---------------------------------------------------------------------------
  ;     NPHO:     NUMBER OF MULTI PHONON TERMS,ITM:NUMBER OF ITERATIONS
  ;     ITM:      TOTAL NUMBER OF ITERATIONS
  ;     IVIT=0(1):ITERATION BY DIFFERENCE(QUOTIENT) METHOD
  ;     IDW=0:    DW COEFF. KEPT CONST.=INPUT VALUE,DW=1:DW COEFF.ITERATED
  ;     IEMP=1(0):CORRECTIONS FOR COUNTER EFFICIENCY WILL BE(NOT BE) DONE
  ;     IRES=1(0):DATA WILL BE(NOT BE) CORRECTED FOR SPECTR. RESOLUTION
  ; ---------------------------------------------------------------------------
  NPHO=0L & ITM=0L & IVIT=0L & IDW=0L & IEMP=0L & IRES=0L & IPR=0L & ILOSS=0L
  READF,u,NPHO,ITM,IVIT,IDW,IEMP,IRES,IPR,ILOSS

  ; ---------------------------------------------------------------------------
  ;     NU:  First channel of spektrum NO: Last channel of spektrum
  ;     NUU: First channel of TOF distribution used for calculation
  ;     NOO: Last channel  of TOF distribution used for calculation
  ;
  ; ---------------------------------------------------------------------------
  ISOUR=0L & NUU=0L & NOO=0L & NU=0L & NO=0L & IGLU=0L
  READF,u,ISOUR,NUU,NOO,NU,NO,IGLU

  ; ---------------------------------------------------------------------------
  ;     UNT: CONSTANT BACKGROUND
  ;     FUN: MULTIPLICATION FACTOR FOR TIME DEPENDENT BACKGROUND
  ;     Z00: TIME OF FLIGHT DISTRIBUTION
  ;     UN0: TIME DEPENDENT BACKGROUND
  ; ---------------------------------------------------------------------------
  UNT=0. & FUN=0.
  READF,u,UNT,FUN
  
  
  Z00=fltarr(1024) & UN0=Z00
 
  IF (ISOUR NE 1) THEN BEGIN
    tmp=fltarr(NOO-NUU+1) 
    READF,u,tmp,format=fmt40 
    Z00[NUU-1]=tmp
    IF (FUN NE 0.) THEN BEGIN
      READF,u,tmp,format=fmt40 
      UN0[NUU-1]=tmp 
    ENDIF
  ENDIF
  
  ; ---------------------------------------------------------------------------
  ; Print the input parameters
  ; ---------------------------------------------------------------------------
  IF (IEMP EQ 0) THEN BEGIN & PRINT,''
  PRINT,'INPUT DISTRIBUTION HAS ALREADY BEEN CORRECTED FOR COUNTER EFFICIENCY' & ENDIF
  IF (IEMP EQ 1) THEN BEGIN & PRINT,''
  PRINT,'CORRECTION FOR COUNTER EFFICIENCY IS DONE IN THIS PROGRAM' & ENDIF
  PRINT,E0,FP,CW,ABK,format="('INC. ENERGY=',F10.5,'   FLIGHT PATH=',F10.5,"+ $
    "'   CHANNEL WIDTH=',F10.5,'   ABS.COEFF.=',F10.5)"
  PRINT,FIMI,FIMA   ,format="('RANGE OF SCATTERING ANGLES:',F10.3,' TO ',F10.3)"
  PRINT,HOX         ,format="('UPPER LIMIT OF PHONON DENSITY OF STATES:',F10.4)"
  PRINT,TEMP0,DW    ,format="(' TEMPERATURE:',F10.3,'   MEAN DEBYE-WALLER COEFF.:',E14.5)"
  PRINT,' IN THIS PROGRAM VALUES FOR DW-COEFF. ARE IN (1/MEV), TO '+ $
    'OBTAIN VALUES IN (ANG**2) MULTIPLY BY 2.0717 '
  IF (IDW EQ 0)   THEN PRINT,'DEBYE-WALLER COEF. FIXED=INPUT VALUE                '
  IF (IDW EQ 1)   THEN PRINT,'DEBYE-WALLER COEF. DETERM. FROM ITERATED DISTR.     '
  IF (NSPEC NE 1) THEN BEGIN & PRINT,''
  PRINT,' MULTI-PHONON CONTRIBUTIONS CALCULATED SEPARATELY FOR BOTH COMPONENTS '
  muph_CONST & ENDIF
  FOR I=0,NSPEC-1 DO PRINT, AMASI[I],SIGI[I],CONCI[I],ALFI[I],format= $
    "(' ATOMIC MASS:',F10.3,'   SIGMA:',F8.4,'   CONC:',F8.4,'   SCATT.POWER:',F8.4)"
  RMA=1.0087/AMASI[0]
  IF (NSPEC NE 1) THEN BEGIN & PRINT,' PARAMETERS FOR FNU/GNU:'
  PRINT,PARQ[0:7],format=fmt40 & RMA=1.0087/AMEFF & ENDIF
  PRINT,NPHO        ,format="('NUMBER OF MULTI-PHONON-TERMS:',I3)"
  PRINT,ITM         ,format="(' TOTAL NUMBER OF ITERATIONS :',I3)"
  IF (IVIT EQ 0) THEN PRINT,'     ITERATION BY DIFFERENCE METHOD'
  IF (IVIT GT 0) THEN PRINT,IVIT,format="('     ITERATION BY QUOTIENT METHOD UP TO IT. NR. :',I3)"

  PRINT,''
  PRINT,XNEL,NU,NO,format="('CHAN.OF EL.LINE:',F10.2,'   FIRST CHANNEL:',I5,'   LAST CHANNEL:',I5)"
  PRINT,UNT       ,format="('CONSTANT BACKGROUND:',F10.3)"
  PRINT,''
  PRINT,NUU,NOO   ,format="(' INPUT TIME OF FLIGHT DISTRIBUTION FROM CHANNEL :',I4,' TO CHANNEL :',I4)"

  FREE_LUN,u

; Output in a workspace

  mod_datp, datp, "e", e_out
  mod_datp, datp, "y", y_out
  give_datp, datp
  return, Z00





END