FUNCTION write_mupho, w_in, file=file, phimin=phimin, phimax=phimax, abs=abs, $
                       aemp=aemp, emax = emax, sam=sam, npho=npho, itm=itm, $
                       ivit=ivit, idw=idw, iemp=iemp, ires=ires, ipr=ipr, iloss=iloss, $
                       amass=amass, sig=sig, conc=conc, $
                       unt=unt, fun=fun
;+
;
; write_mupho(w_in, file='input_mupho.txt')
;
; w_in: Spectrum in time-of-flight
; 
; write_mupho sum the sin(theta)-weighted intensities of the instrument-free 
; time-of-flight data and create the input_mupho.txt input file for the routine 
; MUPHOCOR.
; 
; prox example:
; 
; RDSET,inst="IN5"
; RDSET,base="Current Path"
; w_clear, /all
;
;  ;  read the time-of-flight reduced data
;  ; BGA at room temperature
;  w1 = rdrun('152965_152990.hdf')
;
;  ; Create the MUPHOCOR input with various options
;  w2 = write_mupho(w1, file='input_mupho.txt', emax=40.0, abs=0.0, phimin=31.0, phimax=134.8, iemp=0, amass=44.8, sig=3.9, conc=1.0, unt=0.015)
;
;  ; Compute the GDOS with the MUPHOCOR routine
;  w10 = muphocor('input_test.txt') & see, w10, /below
; 
; 
; 
; input_mupho.txt example and parameters:
; 
;!     E0:INC. ENERGY(MEV)             FP:FLIGHTPATH(CM)
;!     XNEL:CHANNEL OF ELASTIC LINE    CW:CHANNEL WIDTH(MYS)
;!     FIMI:MIN.SCATT.ANGLE            FIMA:MAX.SCATT.ANGLE
;!     UNT:CONST.BACKGROUND            ABK: Absorption coefficient
;!     AEMP: Coeff. for calculating the detector efficiency
;
; e0,     fp,         cw,      xnel,     fimi,    fima,       abk,      aemp
; 4.77    248.30      3.91    794.08     10.33    115.05      0.00      3.30
;
;
;!       Correction OF constant backgroud FOR counter efficiency
;  bac=unt/(1.0-EXP(-aemp/SQRT(e0-hot(n))))
;
;
;
;!     NSPEC: NUMBER OF DIFFERENT ATOMIC SPECIES
;!     IF IQUO is unequal 0 an external model FOR the ratio OF F/G has
;!     to be provided. The model is given through the array PARQ
; lm,     nspec,  iquo,lpar
; 500     1       0    1
;
;!     HOX:   UPPER LIMIT OF DENSITY OF STATES
;!     TEMPO: TEMPERATURE IN KELVIN
;!     DW:    MEAN DEBY WALLER COEFFICIENT
;
;hox,    temp0,      dw
;40.00    298.52      0.03
;
;
;!     AMASI: ATOMIC MASS
;!     SIGI : SIGMA
;!     CONCI: CONCENTRATION
;!     ALFI:  SCATTERING POWER
;DO  i=1,nspec <-- 1 in our CASE so far
;READ 40,amasi(i),sigi(i),conci(i)
;END DO
;amasi(i),  sigi(i),  conci(i)
;44.80      3.90      1.00
;
;
;!     NPHO:     NUMBER OF MULTI PHONON TERMS
;!     ITM:      TOTAL NUMBER OF ITERATIONS
;!     IVIT=0(1):ITERATION BY DIFFERENCE(QUOTIENT) METHOD
;!     IDW=0:    DW COEFF. KEPT CONST.=INPUT VALUE,DW=1:DW COEFF.ITERATED
;!     IEMP=1(0):CORRECTIONS FOR COUNTER EFFICIENCY WILL BE(NOT BE) DONE
;!     IRES=1(0):DATA WILL BE(NOT BE) CORRECTED FOR SPECTR. RESOLUTION
;!     IPR=1(0): Convolutions integrals o the multiphonon term are printed
;!     ILOSS=1:  Analysis OF the imcomplete energy loss spectrum will be done
;
;npho,itm, ivit, idw, iemp, ires, ipr,iloss
;5    10   10    1    1     0     0   0
;
;
;
;!     NU:  First channel OF spektrum NO: Last channel OF spektrum
;!     NUU: First channel OF TOF distribution used FOR calculation
;!     NOO: Last channel  OF TOF distribution used FOR calculation
;!     IGLU: Number OF smoothing processes in CASE OF a time-dependant background
;
;isour,nuu, noo, nu,  no,    iglu
;0     200  750  200  750    0
;
;
;!     UNT: CONSTANT BACKGROUND
;!     FUN: MULTIPLICATION FACTOR FOR TIME DEPENDENT BACKGROUND
;
;unt,      fun
;1.0000    0.0000
;
;
;;     Z00: TIME OF FLIGHT DISTRIBUTION
;;     UN0: TIME DEPENDENT BACKGROUND (if fun = 1)
;

; :Author: ollivier (2016-12-23)
;-

COMMON c_lamp_access, inst

  s = SIZE(w_in)
  if s[0] NE 2 THEN GOTO, ERROR
  
  TAKE_DATP, datp
  par  = datp.p
  x_in = datp.x
  y_in = datp.y
  title= datp.w_tit
  
  
  IF (NOT KEYWORD_SET(phimin)) THEN phimin = min(y_in) 
  IF (NOT KEYWORD_SET(phimax)) THEN phimax = max(y_in)
  idx = where(y_in GT phimin AND y_in LT phimax)
  
  phimin = y_in[idx[0]]
  phimax = y_in[idx[n_elements(idx)-1]]
  
  ; ----------------------------------------------------------------------------
  ; From the Reichardt report:
  ; --------------------------
  ; "The analysis performed by MUPHCOR starts from the sin(theta) weighted 
  ; angle integrated time-of-flight distribution."  
  ; WARNING: y_in is 2theta => theta = y_in/2
  ; ----------------------------------------------------------------------------
  w_buf = w_in[*,idx] * ((fltarr(s[1])+1.0)#y_in[idx]/2.0/180.0*!PI)
  w_buf = total(w_buf,2)
  
  ; WARNING: y_in is 2theta => theta = y_in/2
  
; ----------------------------------------------------------------------------
;  Not to do?: the position of ELP gives the information to the routine
;  From other tests input this is not to be done.
;  IF STRUPCASE(inst) NE 'IN4' THEN  w_buf = REVERSE(w_buf)
; ----------------------------------------------------------------------------
  imax = WHERE(w_buf EQ MAX(w_buf))
  idx  = WHERE(w_buf GE w_buf[imax[0]]/3.0)
 
; p1 = plot(x_in[idx], w_buf[idx], 'Db')

; Doesn't work directly on IN4 data at least
;  res = str_fit(w_buf(idx), Bkgd="flat", Xin=x_in[idx])
; this better works:  
  gs = GAUSSFIT(x_in[idx], w_buf[idx], coeff, NTERMS=3)  
  gs = GAUSS(x_in, coeff)
  imax = WHERE(gs EQ MAX(gs))
  idx  = WHERE(gs GE gs[imax[0]]/3.0)
  
  IF STRUPCASE(inst) EQ 'IN4' THEN $
     dN = ROUND(0.5*(x_in[idx[n_elements(idx)-1]] - x_in[idx[0]])) $ 
  ELSE $; Nb channels above/below ELP
     dN = ROUND((x_in[idx[n_elements(idx)-1]] - x_in[idx[0]])) ; Nb channels above/below ELP

  print, '; ---------------------------------------------------------------------------'
  print, ' dN = ', strtrim(string(dN),2)
  print, '; ---------------------------------------------------------------------------'
  
 
  
; ----------------------------------------------------------------------------
; MUPHOCOR doen't work with neuton energy loss (ENERGY LOSS mode) ???    
; Keep the neutron energy gain for all
; ----------------------------------------------------------------------------
;  IF STRUPCASE(inst) EQ 'IN4' THEN $
;     w_buf = w_buf(imax[0]+dN:n_elements(w_buf)-1) $
;  ELSE $
     w_buf = w_buf[0:imax[n_elements(imax)-1]-dN]

  ; ------------------------------------------------------------------------
  ; Get all parameters from the w_in to feed the MUPHO input file
  ; ------------------------------------------------------------------------
  

  ; ---------------------------------------------------------------------------
  ;     E0:INC. ENERGY(MEV)             FP:FLIGHTPATH(CM)
  ;     XNEL:CHANNEL OF ELASTIC LINE    CW:CHANNEL WIDTH(MYS)
  ;     FIMI:MIN.SCATT.ANGLE            FIMA:MAX.SCATT.ANGLE
  ;     UNT:CONST.BACKGROUND            ABK: Absorption coeff. (0.0 ?)
  ;     AEMP: Coeff. for calculating detector efficiency:  
  ;           Correction of constant backgroud for counter efficiency
  ;     IF(iemp == 0) bac=unt/(1.0-EXP(-aemp/SQRT(e0-hot(n))))
  ;     AEMP = 3.3 in De Boissieu data and also in others fo IN6 data
  ; ---------------------------------------------------------------------------
  E0=0. & FP=0. & CW=0. & XNEL=0. & FIMI=0. & FIMA=0. & ABK=0. 
  E0 = 81.8045/par[21]^2   ; incident energy
  FP = par[27]*100         ; unit [cm]
  CW   = par[18]           ; channel width [mu-s]
  XNEL = par[9]            ; ELP (as computed after "normalise")  
  FIMI = phimin            ; min. scatering angle
  FIMA = phimax            ; max. scatering angle
  IF (NOT KEYWORD_SET(abs)) THEN abs = 0.
  ABK  = abs
  
  ; Coef for detector wefficiency: value from De Boissieu et al. taken (IN6 data)
  IF (NOT KEYWORD_SET(aemp)) THEN AEMP = 3.3

  
  ; ---------------------------------------------------------------------------
  ;     NSPEC: NUMBER OF DIFFERENT ATOMIC SPECIES
  ;     LM: Number of points
  ;     IQUO: Indicator to select model F(w)/G(w)  = 0
  ;     LPAR: Nb parameters for calculation of  F(w)/G(w) = 1
  ;     
  ;     LM must be limited to 512 (LM2 and LM5 to 1024 in MUPHOCOR)
  ; ---------------------------------------------------------------------------
  LM = n_elements(w_buf)<512

;  print, '-------------------------------------------------------'
;  print, 'LM = ', strtrim(string(LM),2)
;  print, '-------------------------------------------------------'

  NSPEC=1L & IQUO=0L & LPAR= 1L

  ; ---------------------------------------------------------------------------
  ;     HOX:   UPPER LIMIT OF DENSITY OF STATES
  ;     TEMPO: TEMPERATURE IN KELVIN
  ;     DW:    MEAN DEBY WALLER COEFFICIENT
  ; ---------------------------------------------------------------------------
  HOX=0. & TEMP0=0. & DW=0.

  IF (NOT KEYWORD_SET(emax)) THEN HOX = 100. ELSE HOX = emax
  IF (NOT KEYWORD_SET(DW)) THEN DW = 0.
  IF par[11] GT 0. THEN TEMP0 = par[11] ELSE TEMP0 = 295.
  
  ; ---------------------------------------------------------------------------
  ;     AMASI: ATOMIC MASS
  ;     SIGI : SIGMA
  ;     CONC:  CONCENTRATION
  ;     ALFI:  SCATTERING POWER
  ; ---------------------------------------------------------------------------
  SIGI =fltarr(2) & CONCI=SIGI & AMASI=SIGI
  IF (NOT KEYWORD_SET(emax)) THEN emax=max(x_in)
  sam = fltarr(3)
  IF (NOT KEYWORD_SET(amass)) THEN amass = 123.0 
  IF (NOT KEYWORD_SET(sig)) THEN sig = 3.9
  IF (NOT KEYWORD_SET(conc)) THEN conc = 1.0
  IF (NOT KEYWORD_SET(amass)) OR  (NOT KEYWORD_SET(sig)) OR (NOT KEYWORD_SET(conc)) THEN $
         print, 'WARNING: Sample parameters: Sigma, concentration or Atomic mass not defined'
  
  sam[0] = amass & sam[1] = sig & sam[2] = conc

  FOR I=0,NSPEC-1 DO BEGIN
    AMASI[I] = sam[0]
    SIGI[I]  = sam[1]
    CONCI[I] = sam[2]
  ENDFOR
 
  
  ; ---------------------------------------------------------------------------
  ;     NPHO:     NUMBER OF MULTI PHONON TERMS,ITM:NUMBER OF ITERATIONS
  ;     ITM:      TOTAL NUMBER OF ITERATIONS
  ;     IVIT=0(n):ITERATION BY DIFFERENCE(QUOTIENT) METHOD UP TO iteration n
  ;     IDW=0:    DW COEFF. KEPT CONST.=INPUT VALUE,DW=1:DW COEFF.ITERATED
  ;     IEMP=1(0):CORRECTIONS FOR COUNTER EFFICIENCY WILL BE(NOT BE) DONE
  ;     IRES=1(0):DATA WILL BE(NOT BE) CORRECTED FOR SPECTR. RESOLUTION
  ; ---------------------------------------------------------------------------
  IF (NOT KEYWORD_SET(nphon)) THEN  NPHO = 5L 
  IF (NOT KEYWORD_SET(itm))   THEN  ITM  = 10L
  IF (NOT KEYWORD_SET(ivit))  THEN  IVIT = 10L
  IF (NOT KEYWORD_SET(idw))   THEN  IDW  = 1L
  IF (NOT KEYWORD_SET(iemp))  THEN  IEMP = 0L
  IF (NOT KEYWORD_SET(ires))  THEN  IRES = 0L
  IF (NOT KEYWORD_SET(ipr))   THEN  IPR  = 0L
  IF (NOT KEYWORD_SET(iloss)) THEN  ILOSS= 0L
 
  ; ---------------------------------------------------------------------------
  ;     NU:  First channel of spektrum NO: Last channel of spektrum
  ;     NUU: First channel of TOF distribution used for calculation
  ;     NOO: Last channel  of TOF distribution used for calculation
  ;
  ; ---------------------------------------------------------------------------
  ISOUR=0L 
;  MUPHOCOR doesn't work with neutron energy loss (ENERGY LOSS mode)???
;  IF (NOT KEYWORD_SET(nuu)) THEN  $
;    IF STRUPCASE(inst) EQ 'IN4' THEN NUU=imax[0]+dN+1 ELSE NUU = 1L
;  IF (NOT KEYWORD_SET(noo)) THEN  $
;    IF STRUPCASE(inst) EQ 'IN4' THEN NOO= n_elements(w_buf)-1+NUU ELSE NOO=n_elements(w_buf)
    
  NUU = 1L
  NOO=n_elements(w_buf) 
   
  NU=NUU
  NO=NOO
  IGLU=0L
 
  ; ---------------------------------------------------------------------------
  ;     UNT: CONSTANT BACKGROUND
  ;     FUN: MULTIPLICATION FACTOR FOR TIME DEPENDENT BACKGROUND
  ; ---------------------------------------------------------------------------
  IF (NOT KEYWORD_SET(unt)) THEN  UNT=0.0
  IF (NOT KEYWORD_SET(fun)) THEN  FUN=0.0
   
  ; ---------------------------------------------------------------------------
  ;     Z00: TIME OF FLIGHT DISTRIBUTION (spectrum 1)
  ;     UN0: TIME DEPENDENT BACKGROUND   (spectrum 2?)
  ; ---------------------------------------------------------------------------
;  Z00=fltarr(1024) & UN0=Z00
  
  ; ---------------------------------------------------------------------------
  ; open the output file and write parameters and intensities
  ; ---------------------------------------------------------------------------
  OPENW,  lun, file, /GET_LUN
  PRINTF, lun, title,format = "(A)"
  PRINTF, lun, E0, FP, CW, XNEL, FIMI, FIMA, ABK, AEMP, format="(8F10.2)"
  PRINTF, lun, LM, NSPEC, IQUO, LPAR, format="(4I5)"
  PRINTF, lun, HOX, TEMP0, DW, format="(3F10.2)"
  FOR I=0,NSPEC-1 DO BEGIN
    PRINTF, lun, AMASI[I], SIGI[I], CONCI[I], format="(3F10.2)"
  ENDFOR
  PRINTF, lun, NPHO,ITM,IVIT,IDW,IEMP,IRES,IPR,ILOSS, format="(8I5)"
  PRINTF, lun, ISOUR,NUU,NOO,NU,NO,IGLU, format="(6I5)"
  PRINTF, lun, UNT, FUN, format="(2F10.4)"
    
  IF (ISOUR NE 1) THEN BEGIN
    PRINTF,lun, w_buf, format="(8F10.4)"
    IF (FUN NE 0.) THEN BEGIN
      PRINTF,lun,tmp,format="(2F10.4)"
    ENDIF
  ENDIF
  PRINTF, lun, 0., format="(1F10.4)"


ERROR:
  FREE_LUN, lun
  RETURN, 0

END
