;+
; :Description:
;    Computes the indexes from a ranges string.
;    A ranges string is a string with the following format 'a:b,c:d,e:f'
;    Parsing such a string will produce indexes resulting from the concatenation of [a,b], [c,d], [e,f] intervals.
;    Example: parsing '1:5,7:10' will yield [1,2,3,4,5,7,8,9,10]
;
; :Params:
;    str : in, required, type=string
;       The input ranges string.
;       
;    indexes : out, required, type=indexes array
;       The output indexes.
;       
; :Keywords:
;    mini : in, optional, type=integer
;       If provided, only those indexes strictly upper to 'mini' will be kept.
;       
;    maxi : in, optional, type=integer
;       If provided, only those indexes strictly lower to 'maxi' will be kept.
;       
;    value : in, optional, type=numeric
;       If provided, will translate the output indexes by 'value'
;
;    
; :Author: pellegrini
; 
; :Categories: strings
;-
pro process_ranges_string, str, indexes, mini=mini, maxi=maxi, value=value

    ; A flag to monitor any ioerror.
    valid = 0

    ; If an ioerror occurs, jump to 'error' label.
    on_ioerror, error

    ; This array will store the indexes corresponding to the parsed ranges string.
    temp = [-1]

    ; First extract the ranges out of the ranges string.
    ranges = strsplit(str,',',/EXTRACT)
    
    if (n_elements(ranges) eq 0) then return
    
    ; Loop over the extracted ranges.
    for i = 0, n_elements(ranges)-1 do begin

        ; For each range, extract its lower and upper bounds.
        bounds = long(strsplit(ranges[i],':',/EXTRACT))

        if n_elements(bounds) ne 2 then return

        ; The lower and upper bounds are sorted.
        bounds = bounds[sort(bounds)]
            
        ; The indexes corresponding to the running ranges are generated.
        bounds = bounds[0] + lindgen(bounds[1] + 1 - bounds[0])

        ; And added to the indexes array.
        temp = [temp,bounds]    
    
    endfor

    ; If the indexes array just contain one element (i.e the initial -1), return with an error.
    if (n_elements(temp) eq 1) then return

    ; Removed the leading -1 from the indexes array.
    temp = temp[1:*]
    
    ; Keep the unique values.
    temp = temp[uniq(temp, sort(temp))]

    ; If the 'value' keyword is set, translate all the indexes by its value.
    if (n_elements(value) ne 0) then temp = temp + value

    ; If the 'mini' keyword is set, keep those indexes that are strictly upper than 'mini'
    if (n_elements(mini) ne 0) then begin
        sel = where(temp ge mini, count)
        if (count eq 0) then return
        temp = temp[sel]
    endif

    ; If the 'maxi' keyword is set, keep those indexes that are strictly lower than 'maxi'
    if (n_elements(maxi) ne 0) then begin
        sel = where(temp le maxi, count)
        if (count eq 0) then return
        temp = temp[sel]
    endif

    if (n_elements(temp) eq 0) then return

    ; Everything was fine, set the flag to monitor ioerror to 1.
    valid = 1
    
    ; The labels to handle an ioerror. If an error occured during the procedure, return with an error.
    error: if (~ valid) then return

    indexes = temporary(temp)
    
    return
    
end

;+
; :Description:
;    Reformat the superADAM so that it always has 3 dimensions.
;
; :Params:
;    data : inout, required, type=array
;       The input suerADAM data.
;
; :Author: pellegrini
;-
pro reformat_superadam_data, data

    ; The number of dimension for the workspace. Must be 2 or 3 on superAdam otherwise throw an error.
    s = size(data,/N_DIMENSIONS) 

    ; Check the workspace dimensions.
    case s of
        ; If dimension is 2, just one angle has been measured. Add an extra third dimension of 1 
        ; to keep consistent with the standard dimension 3 case.        
        2: begin
               data = reform(data,[size(data,/DIMENSIONS),1],/OVERWRITE)
           end
        ; Dimension 3 is OK. Do nothing.
        3:
        ; Otherwise, the numors may not be proper superADAM data. Return with the -1 error code.
        else: data = -1
    endcase

end

;+
; :Description:
;    Generates the q values given an array of angles. 
;
; :Params:
;    angles : in, required, type=array
;       The input angles.
;
;    wavelength : in, required, type=float
;       The wavelength.
;
;    q : out, required, type=vector
;       The q values.
;
; :Keywords:
;    /two_thetas : in, optional
;       If set, specify that the input angles are two thetas.
;
;    /radians : in, optional
;       If set, the input angles are in radians otherwise in degrees.
;
; :Author: pellegrini
;
; :Categories: reflectivity
;
;-
pro thetas_to_q, thetas, wavelength, q, radians=radians, two_thetas=two_thetas

    ; If two_thetas parameter is set, the angles will be divided by two in the q values formula
    if keyword_set(two_thetas) then tt_factor = 0.5 else tt_factor = 1.0

    ; If the radians parameter is not set, the angles will be converted in degrees
    if keyword_set(radians) then conv = 1.0 else conv = !DTOR 

    ; The well-known formula for the q values
    q = reform(4.0*!PI*sin(tt_factor*conv*thetas)/wavelength)
        
    return

end

;+
; :Description:
;    Correct some input datas from detector efficiency.
;
; :Params:
;    data : inout, required, type=float array
;       The data to correct from efficiency
;       
;    efficiency : in, required, type=numor string
;       The numor from which the efficiency matrix will be computed.
;
; :Keywords:
;    efficiency_range_x : in, optional, type=string
;       If provided, the ranges string that define the x region for the efficiency factor computation.
;       
;    efficiency_range_y
;       If provided, the ranges string that define the y region for the efficiency factor computation.
;
; :Author: pellegrini,gonzalez
;-
pro efficiency_correction, data, data_error, efficiency_file, x_range=x_range, y_range=y_range

    if (n_elements(efficiency_file) eq 0) then efficiency_file = ''
    
    ; If the efficiency numor is an empty string, skip the efficiency correction.
    if (strlen(strtrim(efficiency_file,2)) eq 0) then begin
        lamp_message, "No efficiency numor provided. Efficiency correction skipped."
        return
    endif
  
    ; A flag to monitor any ioerror.
    valid = 0

    ; If an ioerror occurs, jump to error' label.
    on_ioerror, error

    ; The efficiency numor is read.
    efficiency = rdopr(efficiency_file)
    
    reformat_superadam_data, efficiency

    ; The efficiency matrix is summed over the q axis.
    efficiency = total(efficiency,3)

    ; The dimension of the efficiency matrix.
    s = size(efficiency, /DIMENSIONS)

    ; If the efficiency indexes array for X axis is not set, exit with an error.
    if (n_elements(x_range) eq 0) then x_range = lindgen(s[0])

    ; If the efficiency indexes array for Y axis is not set, exit with an error.
    if (n_elements(y_range) eq 0) then y_range = lindgen(s[1])   
       
    ; The efficiency factor is defined as the average over the x and y efficiency regions.
    factor = median(efficiency[x_range[0]:x_range[-1],y_range[0]:y_range[-1]])
    
    ; Check that the efficiency factor is not <= 0. Otherwise, exit with an error.
    if (factor le 0.0) then return
    
    ; The efficiency matrix is divided by the efficiency factor.
    efficiency = efficiency/factor
    efficiency_error = sqrt(efficiency)/factor
    
    ; The dimension of the input data.
    s = size(data, /DIMENSIONS)
                   
    sel = where(efficiency gt 0, counts)

    nPixels = s[0]*s[1]

    for i = 0, s[2]-1 do begin
        if (counts gt 0) then begin
           data[nPixels*i + sel] = data[nPixels*i + sel]/efficiency[sel]
           data_error[nPixels*i + sel] = sqrt( (data_error[nPixels*i + sel]/efficiency[sel])^2 + (data[nPixels*i + sel]*efficiency_error[sel]/efficiency[sel]^2)^2 ) 
        endif   
    endfor                   
                    
    ; Everything was fine.
    valid = 1

    ; The error label: if an ioerror occured, exit with an error. 
    error: if (~ valid) then data = -1

    return

end

;+
; :Description:
;    Normalize data from monitor.
;
; :Params:
;    data : inout, required, type=float array
;       The data to be normalized from monitor
;       
;    data_error : inout, required, type=float array
;       The error for the normalized data.
;
;    monitor : in, required, type=float vector
;       The monitor value for each q values.
;       
; :Author: pellegrini,gonzalez
;-
pro monitor_correction, data, data_error, monitor
    
    ; Reformat the input data.
    reformat_superadam_data, data
    if (size(data, /N_DIMENSIONS) eq 0) then begin
        data = -1
        return
    endif

    ; The input data dimensions.
    s = size(data, /DIMENSIONS)

    ; If there is a discrepancy between the monitor dimension and the q-axis defined for the input data, exit with an error.
    if (s[2] ne n_elements(monitor)) then begin
        data = -1
        return
    endif

    ; If the monitor values are not positive, exit with a warning message.
    index = where(monitor GT 0, positive_values)
    if (s[2] ne positive_values) then begin
        print, 'Normalization to monitor skipped because some values are null or negative'
        return
    endif
    
    ; Loop over the q axis.
    for i = 0, s[2]-1 do begin

        ; The data are normalized to the monitor.
        data[*,*,i] = data[*,*,i]/monitor[i]

        ; Monitor error.        
        dM = sqrt(monitor[i])

        ; The error for monitor-normalized data is computed using the error propagation formula.
        data_error[*,*,i] = sqrt((data_error[*,*,i]/monitor[i])^2 + (data[*,*,i]*dM/monitor[i]^2)^2)
        
    endfor
        
    return

end

;+
; :Description:
;    Integrate the data for each q value around the corresponding peak. 
;
; :Params:
;    data : in, required, type=array
;       The input data.
;       
;    error : in, required, type=array.
;       The error corresponding to the input data.
;       
;    peaks : in, required, type=vector
;       The indexes of the peaks found for each q values.
;    
;    integrated_peak : out, required, type=vector
;        The integrated peak.
;
; :Author: pellegrini,gonzalez
;-
pro integrate_superadam_data, data, data_error, integrated_data, integrated_data_error, x_range=x_range, y_range=y_range, peaks=peaks, background=background
    
    ; The dimension of the input data.
    s = size(data, /DIMENSIONS)

    if (n_elements(x_range) eq 0) then x_range = lindgen(s[0])

    if (n_elements(y_range) eq 0) then y_range = lindgen(s[1])
    
    ; The background.
    bkg = 0.0 & bkg_err = 0.0
    
    ; The array that will store the result of the integration for each q values.
    integrated_data = [-1.0] & integrated_data_error = [-1.0]
    
    peak_pos = 0L  

    ; Loop over the q axis.
    for i = 0, s[2]-1 do begin
    
        ; The input data is integrated over the y range.
        temp  = total(data[*,y_range,i],2)
        temp_error = sqrt( total(data_error[*,y_range,i]^2, 2) )
    
        if (n_elements(peaks) gt 0) then peak_pos = peaks[i]
        
        ; If the 'background' keyword is set, computes the background as the mean value over the defined background range.
        if (n_elements(background) ne 0) then begin
           bkg = mean(temp[peak_pos + background])
           bkg_err = sqrt( total(temp_error[peak_pos + background]^2) ) / n_elements(background)
        endif   

        ; Substract the background from the integrated peak value.        
        integrated_data = [integrated_data, total(temp[peak_pos + x_range]-bkg)]
        
        integrated_data_error = [integrated_data_error, sqrt(total(temp_error[peak_pos + background]^2+bkg_err^2)) ]

    endfor
    
    ; If the integrated peak vector has no data, exit with an error.
    if (n_elements(integrated_data) le 1) then return

    ; The leading dummy value is removed.
    integrated_data = integrated_data[1:*] & integrated_data_error = integrated_data_error[1:*]

    return

end


;+
; :Description:
;    Computes the reflectivity for superADAM instrument.
;
; :Params:
;    reflect : in, required, type=string
;       The reflected beam numors.
;
;    integ_range_x : in, required, type=string
;       The integration range that will surround the detected peaks.
;       Must be of the form 'a:b' where a<0 and b>0 so that for each q frame, 
;       the integration will be performed on the range [p+a,p+b] where p is the index
;       of the reflected peak.
;
;    wavelength : in, required, type=float
;       The wavelength.
;
;    q_values : out, required, type=vector
;        The q values for which the reflectivity is computed.
;
;    reflectivity : out, required, type=vector
;        The reflectivity defined for all the q values.
;
; :Keywords:
;    direct : in, optional, type=string
;       The direct beam numors.
;
;    efficiency : in, required, type=string
;       The incoherent scatterer numors.
;
;    eff_range_x : in, optional, type=string
;       The X range used for defining the range over which the efficiency will be computed.
;       Must be of the form 'a:b', where a and b specified the minimum and maximum indexes to consider.
;       For example, 10:40 will correspond indexes 10,11,12...40.
;       If not set, 'eff_range_y' will be used.
;       If not set, use the whole data X dimension.
;
;    eff_range_y : in, optional, type=string
;       The X range used for defining the range over which the efficiency will be computed.
;       Must be of the form 'a:b', where a and b specified the minimum and maximum indexes to consider.
;       For example, 10:40 will correspond indexes 10,11,12...40.
;       If not set, 'eff_range_y' will be used.
;       If not set, use the whole data Y dimension.
;
;    integ_range_y : in, optional, type=string
;       The Y range used for integrating the peak.
;       Must be of the form 'a:b', where a and b specified the minimum and maximum indexes to consider.
;       For example, 10:40 will correspond indexes 10,11,12...40.
;       If not set, 'eff_range_y' will be used.
;
;    background_range : in, optional, type=string
;       The background range used for substracting the background from the integrated peak.
;       Must be of the form 'a:b' where a<0 and b>0 so that for each q frame, 
;       the background will be computed in the range [p+a,p+b] where p is the index
;       of the reflected peak.
;       
; :Author: pellegrini
;-
pro superadam_reflectivity, reflect                           , $
                            q_values                          , $
                            reflectivity                      , $
                            reflectivity_error                , $
                            integ_range_x=integ_range_x       , $
                            integ_range_y=integ_range_y       , $
                            direct=direct                     , $
                            efficiency=efficiency             , $
                            eff_range_x=eff_range_x           , $
                            eff_range_y=eff_range_y           , $
                            background_range=background_range , $
                            peaksearch_range=peaksearch_range , $
                            wavelength=wavelength
         
    forward_function rdopr, str_fit
                        
    ; Display a starting message in LAMP.
    lamp_message, 'Starting superADAM data reducing on ' + systime()
    
    ; Init all the input variables that would not have been set.
    if (n_elements(direct)           eq 0) then direct = ''
    if (n_elements(eff_range_x)      eq 0) then eff_range_x = ''
    if (n_elements(eff_range_y)      eq 0) then eff_range_y = ''
    if (n_elements(integ_range_x)    eq 0) then integ_range_x = ''
    if (n_elements(integ_range_y)    eq 0) then integ_range_y = ''
    if (n_elements(background_range) eq 0) then background_range = ''
    if (n_elements(peaksearch_range) eq 0) then peaksearch_range = ''
    if (n_elements(wavelength)       eq 0) then wavelength = ''

    ; If no wavelength was provided, set it to the superADAM default.
    ; if (strlen(strtrim(wavelength,2)) eq 0) then wavelength = 4.4
                                                         
    ; Read the reflect numors. 
    data = rdopr(reflect, datp=datp)    

    ; To be used in future to differentiate between polarizations
    ; itmp = where(strmid(datp.par_txt,0,9) eq 'spinstate')
    ; spinstate = strmid(datp.par_txt[itmp],11,2) --> 'uu', 'du', 'dd', or 'ud'

    ; Reformat the workspace to (nx,ny,nframes) where nframes will be 1 in case of a 2D workspace.
    reformat_superadam_data, data
    if (size(data, /N_DIMENSIONS) eq 0) then return
    
    ; Deal with errors
    data_error = sqrt(data)

    ; The data darray dimensions.
    s = size(data, /DIMENSIONS)    
    
    if (strlen(strtrim(integ_range_x,2))    gt 0) then process_ranges_string, integ_range_x, integration_x, mini=0, maxi=s[0]-1
    if (strlen(strtrim(integ_range_y,2))    gt 0) then process_ranges_string, integ_range_y, integration_y, mini=0, maxi=s[1]-1
    if (strlen(strtrim(eff_range_x,2))      gt 0) then process_ranges_string, eff_range_x,efficiency_x, mini=0,maxi=s[0]-1
    if (strlen(strtrim(eff_range_y,2))      gt 0) then process_ranges_string, eff_range_y, efficiency_y, mini=0,maxi=s[1]-1
    if (strlen(strtrim(background_range,2)) gt 0) then process_ranges_string, background_range, background

    if (strlen(strtrim(peaksearch_range,2)) gt 0) then begin
        process_ranges_string, peaksearch_range, peaks_pos, mini=0,maxi=s[0]-1
        if (n_elements(peaks_pos) eq 0) then return
        fit = str_fit(total(data[peaks_pos[0]:peaks_pos[-1],*,*],2), BKGD='flat', /PSEUDO_VOIGT)
        peaks = peaks_pos[0] + round(fit[*,2])
    endif

    ; Correct the data from efficiency.    
    efficiency_correction, data, data_error, efficiency, x_range=efficiency_x, y_range=efficiency_y 
    if (size(data, /N_DIMENSIONS) eq 0) then return

    ; Normalize the data from monitor. Alternative could be use counting time.
    MonPos = where(datp.par_txt eq 'Monitor =')
    TimePos = where(datp.par_txt eq 'Seconds =')
    monitor_values = transpose(datp.pv[MonPos,*])
    time_values = transpose(datp.pv[timePos,*])
    monitor_correction, data, data_error, monitor_values
    if (size(data, /N_DIMENSIONS) eq 0) then return
    
    ; Compute the q values and the number of q values from the scattering angles.
    if (strlen(strtrim(wavelength,2)) eq 0) then begin
       q_values = datp.pv[14,*] ;If no wavelength given, keep the angles
       q_values = reform(q_values, n_elements(q_values))
    endif else begin
       thetas_to_q, transpose(datp.pv[14,*]), wavelength, q_values ;Transform to Q
    endelse
                            
    ; Integrates the corrected reflected beam over the integration range.
    ; If no direct beam is defined, this is the computed reflectivity.
    integrate_superadam_data, data, data_error     , $
                              reflectivity         , $ 
                              reflectivity_error   , $
                              x_range=integration_x, $
                              y_range=integration_y, $
                              peaks=peaks          , $
                              background=background

    
    ; If a direct beam is defined, correct it and integrate it over the integration range.                                 
    if (strlen(strtrim(direct,2)) ne 0) then begin
    
        ; Read the direct numors. 
        data = rdopr(direct, datp=datp)    
        data_error = sqrt(data)

        ; Reformat the workspace to (nx,ny,nframes) where nframes will be 1 in case of a 2D workspace.
        reformat_superadam_data, data
        if (size(data, /N_DIMENSIONS) eq 0) then return

        ; Correct the direct data from efficiency.
        efficiency_correction, data, data_error, efficiency, x_range=efficiency_x, y_range=efficiency_y
        if (size(data, /N_DIMENSIONS) eq 0) then return

        ; Normalize the data from monitor.
        monitor_values = transpose(datp.pv[62,*])
        monitor_correction, data, data_error, monitor_values
        if (size(data, /N_DIMENSIONS) eq 0) then return

        ; Integrated the corrected direct beam over the integration range.
        integrate_superadam_data, data, data_error       , $
                                  integrated_direct      , $
                                  integrated_direct_error, $
                                  x_range=integration_x  , $
                                  y_range=integration_y  , $
                                  peaks=peaks            , $
                                  background=background
        
        ; If the direct beam is defined, the integrated signal for the reflected beam has to be divided by the integrated
        ; and averaged one for the direct beam.
        direct_flux = mean(integrated_direct)
        reflectivity = reflectivity/direct_flux 
        
        ; Needs to add errors here!
        direct_error = sqrt(total(integrated_direct_error^2))/n_elements(integrated_direct_error)
        reflectivity_error = reflectivity_error/direct_flux + reflectivity*direct_error/direct_flux^2
    
    endif
        
    return
                                    
end