;+
; Compute automatically direct beam area and neutron counts in that area 
; in a series of numors.
;
; Call:
;
; sans_directBeam, #numor1, #numor2 [, refTitle = 'title of reference measuremnt'] [, percentBgr = N] [, /show]

;
;    where #numor1 and #numor2 are the initial and final numors.
;    The optional parameters are
;       refTitle = title of the numor to be used as reference to determine 'integration window'.
;                  If not given, the title of the first file is used.
;       percentBgr = background percentage to add to the background in order to define
;                    threshold for direct beam (default = 10%)
;       The keyword /show can be used to display the file immediately
;
;-


pro sans_directBeam, numor_ini, numor_end, status, refTitle = refTitle, percentBgr = percentBgr, show=show
;+
;** ********
;** User callable
;** Write a log file for a series of SANS numors. The name of the log file is 'logDirectBeam_numor1_numor2.txt'
;** Call: sans_directBeam, #numor1, #numor2 [, refTitle = 'title of reference measuremnt'] [, percentBgr = N] [, /show]
;**
;** status=0 Program error
;** status=1 Success
;** status=2 First numor missing
;** status=3 Last  numor missing
;** status=4 Write error
;-

common c_lamp_font

on_ioerror,mislog & u=-1 & status=0

catch, stat & if stat ne 0 then begin & rdset,nodata=0 & lamp_error & endif

if n_elements(numor_end) ne 1 then numor_end = numor_ini

file = 'logDirectBeam_' + strtrim(numor_ini,2) + '_' + strtrim(numor_end,2) + '.txt'


; Check if first and last numors can be read
numor = rdrun(numor_ini, datp = datp_ini)
s = size(numor)
if s[0] eq 0 then begin
   print, 'ERROR: First numor does not exist or cannot be read! Returning ...'
   status=2
   return
endif
numor = rdrun(numor_end)
s = size(numor)
if s[0] eq 0 then begin
   print, 'ERROR: Last numor does not exist or cannot be read! Returning ...'
   status=3
   return
endif

; Check if 'over background' threshold given, otherwise set to 10%
if n_elements(percentBgr) ne 1 then percentBgr = 10.

; Check if reference title given or not. If not given, take title of first numor!
if n_elements(refTitle) ne 1 then refTitle = datp_ini.w_tit

;Open log file 
openw, u, file, /get_lun, width=180
printf, u, 'SANS LOG FILE - DIRECT BEAM RUNS:'
printf, u
printf, u, format = '("Run #", 6x, "Title", 5x, "x_i", 1x, "x_f", 2x, "y_i", 1x, "y_f", 4x, "Time", 5x, "Area", 2x, " CountRate ")'
printf, u


; Go through the list of numors

for i = numor_ini, numor_end do begin

    ; Check if numor is valid 
    valid = 0
    while not valid  and i le numor_end do begin
        numor = rdrun(i,datp=datp)
        s = size(numor)
        if s[0] gt 0 then valid = 1 else i = i + 1
    endwhile   
 
    ; Define integration window (if numor_ini or refTitle)  
    if i eq numor_ini or strtrim(datp.w_tit) eq strtrim(refTitle) then begin
        x_proj = total(numor,2)
        y_proj = total(numor,1)
        x_threshold = max(x_proj, xmax_index) / 100.
        y_threshold = max(y_proj, ymax_index) / 100.
        x_over = where(x_proj ge x_threshold)
        x_tmp = x_over[0] - 1
        y_over = where(y_proj ge y_threshold)
        y_tmp = y_over[0] -1
        if x_tmp lt n_elements(x_proj)-1 then x_bgr = mean(x_proj[0:x_tmp]) else x_bgr = 0.
        if y_tmp lt n_elements(y_proj)-1 then y_bgr = mean(y_proj[0:y_tmp]) else y_bgr = 0.
        x_threshold = x_bgr + x_bgr*percentBgr/100.
        y_threshold = y_bgr + y_bgr*percentBgr/100.
        ; Starting from maximum find first pixel in each direction below threshold  
        x_over = where(x_proj[0:xmax_index] lt x_threshold)
        x_ini = x_over[-1]
        x_over = where(x_proj[xmax_index:*] lt x_threshold)
        x_end = xmax_index + x_over[0]
        y_over = where(y_proj[0:ymax_index] lt y_threshold)
        y_ini = y_over[-1]
        y_over = where(y_proj[ymax_index:*] lt y_threshold)
        y_end = ymax_index + y_over[0]

    endif

    ; Area of integration region (in cm^2)
    x_pixel = 0.75
    y_pixel = 0.75
    if n_elements(datp.x) eq 256 then x_pixel = 0.5 * x_pixel
    if n_elements(datp.y) eq 256 then y_pixel = 0.5 * y_pixel
    area = (x_end - x_ini) * x_pixel * (y_end - y_ini) * y_pixel
    
    ; Counting range in integration window
    if i le numor_end then begin
      
        time = datp.p[2]                                            ;run time in seconds
        countRate = total(numor[x_ini:x_end, y_ini:y_end]) / time
     
        ; Write log
   
        printf, u, format = '(I06, 3x, A10, 2x, I3, 1x, I3, 2x, I3, 1x, I3, 2x, F6.1, 2x, F7.2, 2x, F10.2)', $
           i, strmid(datp.w_tit,0,10), x_ini, x_end, y_ini, y_end, time, area, countRate

      endif

   endfor

free_lun, u
rdset,nodata=0
if keyword_set(show) then xdisplayfile, file, font=ft_propor, width=90
status=1
return

mislog: if u lt 0 then lamp_error

rdset,nodata=0
status=4

end
