;+
; Write a log file for a series of SANS numors. The name of the log file is 'log_numor1_numor2.txt'
;
; Call:
;
; If ASCII numors: sans_log, #numor1, #numor2 [, param = #P1] [, step=N], [tol=F] [, /show]
;
;    where #numor1 and #numor2 are the initial and final numors.
;    The optional parameters are
;       param = integer corresponding to the position in the parameters block of the additional parameter to be written in the log,
;       step  = integer to indicate the step in the loop over numors (e.g. N=2 will skip 1 numor every 2) (default is 1),
;       tol   = float to give the tolerance to determine when a parameter has changed value (default is 0.1).
;       The keyword /show can be used to display the file immediately
;
; If nexus numors: sans_log, #numor1, #numor2 [, param = 'keyword'] [, step=N], [tol=F] [, /show]
;
;    In this case with the keyword the user must give the name given in the nexus file to the additional parameter to be written in the log.
;    The other parameters work as for the ASCII case.
;-

pro get_parameters_nexus, numor, run_number, instrument, user, localcontact, title, date, detector_distance, detector_angle, detector_offset, $
                                 collimation, selector_speed, selector_tilt, attenuator, sample, position, time, monitor, counting_time, $
                                 lambda, total_counts, counts_per_second, tof_mode, d11_dataType, value
;+
;** ********************
;** Get all parameters written in the nexus file
;-

   number_params = n_elements(numor)
   value = fltarr(number_params)
   for i = 0, n_elements(numor)-1 do begin
      tmp = strpos(numor[i], '=', /reverse_search)
      if tmp ge 0 then strvalue = strtrim(strmid(numor[i], tmp+1), 2) else strvalue = ''
      if strlen(strvalue) gt 0 then value[i] = float(strvalue) 
   endfor

   idx0 = strpos(numor, 'run_number') & idx0 = where(idx0 ge 0) & idx0 =idx0[0] 
   if idx0 ge 0 then run_number = value[idx0] else run_number = 0.0

   idx1 = strpos(numor, '(Instrument) name') & idx1 = where(idx1 ge 0) & idx1 =idx1[0] 
   if idx1 ge 0 then begin
      tmp1 = strsplit(numor[idx1],/extract)
      instrument = tmp1[n_elements(tmp1)-1]
   endif else instrument = ''
   
   idx2 = strpos(numor, '(User) name') & idx2 = where(idx2 ge 0) & idx2 =idx2[0] 
   if idx2 ge 0 then begin
      tmp2 = strpos(numor[idx2], 'name:')
      user = strmid(numor[idx2], tmp2+5)
   endif else user = ''
   
   idx3 = strpos(numor, '(User) namelocalcontact') & idx3 = where(idx3 ge 0) & idx3 =idx3[0] 
   if idx3 ge 0 then begin
     tmp3 = strpos(numor[idx3], 'namelocalcontact:')
     localcontact = strmid(numor[idx3], tmp3+17)
   endif else localcontact = ''
   
   idx4 = strpos(numor, 'experiment_title') & idx4 = where(idx4 ge 0) & idx4 =idx4[0] 
   if idx4 ge 0 then begin
      tmp4 = strpos(numor[idx4], 'title:')
      title = strmid(numor[idx4], tmp4+6)
   endif else title = ''
   
   idx5 = strpos(numor, 'start_time') & idx5 = where(idx5 ge 0) & idx5 =idx5[0] 
   if idx5 ge 0 then begin
      tmp5 = strpos(numor[idx5], 'time:')
      date = strmid(numor[idx5], tmp5+5)
   endif else date = ''
   
   if strlowcase(instrument) eq 'd33' then begin
      idx6 = strpos(numor, 'det2_actual') & idx6 = where(idx6 ge 0) & idx6 =idx6[0]
   endif else begin
      idx6 = strpos(numor, 'det_actual') & idx6 = where(idx6 ge 0) & idx6 =idx6[0] 
   endelse   
   if idx6 ge 0 then detector_distance = value[idx6] else detector_distance = 0.0

   idx7 = strpos(numor, 'det_angle') & idx7 = where(idx7 ge 0) & idx7 =idx7[0] 
   if idx7 ge 0 then detector_angle = value[idx7] else detector_angle = 0.0

   idx8 = strpos(numor, 'det_offset') & idx8 = where(idx8 ge 0) & idx8 =idx8[0] 
   if idx8 ge 0 then detector_offset = value[idx8] else detector_offset = 0.0

   idx9 = strpos(numor, '(I collimator) actual_position') & idx9 = where(idx9 ge 0) & idx9 =idx9[0] 
   if idx9 ge 0 then collimation = value[idx9] else collimation = 0.0

   idx10 = strpos(numor, '(I velocity_selector) rotation_speed') & idx10 = where(idx10 ge 0) & idx10 =idx10[0] 
   if idx10 ge 0 then selector_speed = value[idx10] else selector_speed = 0.0

   idx11 = strpos(numor, '(I velocity_selector) selrot_actual') & idx11 = where(idx11 ge 0) & idx11 =idx11[0] 
   if idx11 ge 0 then selector_tilt = value[idx11] else selector_tilt = 0.0

   idx12 = strpos(numor, '(I attenuator) position') & idx12 = where(idx12 ge 0) & idx12 =idx12[0] 
   if idx12 ge 0 then attenuator = value[idx12] else attenuator = 0.0


   ; Get values for log and conditions checking 

   idx13 = strpos(numor, 'sample_description') & idx13 = where(idx13 ge 0) & idx13 =idx13[0] 
   if idx13 ge 0 then begin
      tmp13 = strpos(numor[idx13], 'description:')
      sample = strmid(numor[idx13], tmp13+12)
   endif else sample = ''

   ;sample_changer_slot_value added to NeXus files (only D11) in June 2016
   changer_text = 'sample_changer_value'
   if strlowcase(instrument) eq 'd11' then changer_text = 'sample_changer_slot_value'
   idx14 = strpos(numor, changer_text) & idx14 = where(idx14 ge 0) & idx14 = idx14[0]
   if idx14 ge 0 then position = value[idx14] else position = 0.0
   
   idx15 = strpos(numor, 'start_time') & idx15 = where(idx15 ge 0) & idx15 =idx15[0] 
   if idx15 ge 0 then begin
      tmp15 = strpos(numor[idx15], 'time:')
      time = strmid(numor[idx15], tmp15+16)
   endif else time = ''

   idx16 = strpos(numor, 'monsum') & idx16 = where(idx16 ge 0) & idx16 =idx16[0] 
   if idx16 ge 0 then monitor = value[idx16] else monitor = 0.0

   idx17 = strpos(numor, 'duration') & idx17 = where(idx17 ge 0) & idx17 =idx17[0] 
   if idx17 ge 0 then counting_time = value[idx17] else counting_time = 0.0

   idx18 = strpos(numor, 'velocity_selector) wavelength') & idx18 = where(idx18 ge 0) & idx18 =idx18[0] 
   if idx18 ge 0 then lambda = value[idx18] else lambda = 0.0

   idx19 = strpos(numor, 'detsum') & idx19 = where(idx19 ge 0) & idx19 =idx19[0] 
   if idx19 ge 0 then total_counts = value[idx19] else total_counts = 0.0

   idx20 = strpos(numor, 'detrate') & idx20 = where(idx20 ge 0) & idx20 =idx20[0] 
   if idx20 ge 0 then counts_per_second = value[idx20] else counts_per_second = 0.0
    
   ;For D33 determine if working mode is TOF
   if strlowcase(instrument) eq 'd33' then begin
      idx22 = strpos(numor, '(Entry) mode') & idx22 = where(idx22 ge 0) & idx22 =idx22[0]
      if idx22 ge 0 then tof_mode = value[idx22] else tof_mode = 0.0
   endif else tof_mode = 0.0

   ; Determine data type in D11 (128x128 or 256x256)
   if strlowcase(instrument) eq 'd11' then begin
     idx23 = strpos(numor, '(I detector) data') & idx23 = where(idx23 ge 0) & idx23 =idx23[0]
     if idx23 ge 0 then begin
        tmp23 = strpos(numor[idx23], 'data')
        numberOfData = strmid(numor[idx23], tmp23+4, 7)
        if numberOfData eq '(16384)' then begin
          d11_dataType = 128
        endif else if numberOfData eq '(65536)' then begin
          d11_dataType = 256
        endif else d11_dataType = 0
     endif
   endif

end

pro sans_log, numor_ini, numor_end, status, param=np, step=step, tol=tolerance, show=show
;+
;** ********
;** User callable
;** Write a log file for a series of SANS numors. The name of the log file is 'log_numor1_numor2.txt'
;** Call for ASCII numors: sans_log, #numor1, #numor2 [, param = #P1] [, step=N], [tol=F] [, /show]
;** Call for NeXus numors: sans_log, #numor1, #numor2 [, param = 'keyword'] [, step=N], [tol=F] [, /show]
;**
;** status=0 Program error
;** status=1 Success
;** status=2 First numor missing
;** status=3 Last  numor missing
;** status=4 Write error
;-

common c_lamp_font

on_ioerror,mislog & u=-1 & status=0

catch, stat & if stat ne 0 then begin & rdset,nodata=0 & lamp_error & endif

if n_elements(step)      ne 1 then step = 1 
if n_elements(numor_end) ne 1 then numor_end = numor_ini
if n_elements(tolerance) ne 1 then tolerance = 0.1 ; Tolerance to check if parameters have changed

file = 'log_' + strtrim(numor_ini,2) + '_' + strtrim(numor_end,2) + '.txt'

; Set reading to read only numor parameters
rdset, nodata=1

; Check if first and last numors can be read
numor = rdrun(numor_ini)
if not (size(numor,/type) eq 7 or size(numor,/type) eq 8) then begin
   print, 'ERROR: First numor does not exist or cannot be read! Returning ...'
   rdset,nodata=0
   status=2
   return
endif
numor2 = rdrun(numor_end)
if not (size(numor2,/type) eq 7 or size(numor2,/type) eq 8) then begin
   print, 'ERROR: Last numor does not exist or cannot be read! Returning ...'
   rdset,nodata=0
   status=3
   return
endif

; Determine if numor is ASCII (in this case numor = structure) or Nexus (in this case numor = string array)
if size(numor,/type) eq 7 then nexus_mode = 1 else nexus_mode = 0

;Open log file 
openw, u, file, /get_lun, width=180
printf, u, 'SANS LOG FILE'
printf, u

if not nexus_mode then begin

   ;Write general information extracted from first numor
 
   printf, u, 'Instrument         : ', numor.instrument

   tmp = strmid(numor.experiment[0], 0, 6)   & printf, u, 'User               : ', tmp
   tmp = strmid(numor.experiment[0], 6, 4)   & printf, u, 'LC                 : ', tmp
   tmp = strmid(numor.experiment[0], 10, 50) & printf, u, 'Title              : ', tmp
   tmp = strmid(numor.experiment[1], 0, 9)   & printf, u, 'Start date/time    : ', tmp

   printf, u
   printf, u, format = '("Detector distance (read)    = ", F8.2)', numor.param[18]
   printf, u, format = '("Detector angle (read)       = ", F8.2)', numor.param[14]
   printf, u, format = '("Detector translation (read) = ", F8.2)', numor.param[61]
   printf, u, format = '("Collimation (read)          = ", F8.2)', numor.param[57]
   printf, u, format = '("Selector speed (read)       = ", F8.2)', numor.param[19]
   printf, u, format = '("Selector tilt angle         = ", F8.2)', numor.param[62]
   printf, u, format = '("Attenuator                  = ", I8)', long(numor.param[58])
   printf, u

   ; Get values for log and conditions checking 

   sample            = strmid(numor.experiment[0], 60, 20)
   position          = numor.param[65]
   time              = strmid(numor.experiment[1], 10, 8)
   monitor           = numor.param[4]
   counting_time     = numor.param[2] / 10.
   selector_speed    = numor.param[19]
   lambda            = numor.param[52]
   total_counts      = numor.param[3]
   counts_per_second = numor.param[3] / counting_time

   okInd=0 & on_ioerror, misindex 
   if n_elements(np) eq 1 then begin
      index = long(np)-1
      okInd = 1
      misindex: if oKind eq 0 then printf, u, 'Invalid parameter given. Needs an integer for ASCII file!' & printf, u, ''
   endif
   on_ioerror,mislog 
   if okInd then selected_param = numor.param[index] else selected_param = 0.0 

   detector_distance = numor.param[18]
   detector_angle = numor.param[14]
   collimation = numor.param[57]
   attenuator = numor.param[58]

   ; Write info header and information for 1st numor

   printf, u, format = '("Run #", 4x, "Sample", 14x, "Pos", 4x, "Start", 3x, "Monitor", 2x, "Time", 3x, "RPM", 2x, "Lambda", 1x, "Total cts.", 4x, "c/s", 5x, "Param")'
   printf, u, format = '(I06, 3x, A-20, 1x, I-3, 1x, A-8, 1x, I8, 1x, I5, 2x, I5, 2x, F4.1, 1x, I10, 1x, I8, 1x, F8.2)', $
     numor.numor, sample, position, time, monitor, counting_time, selector_speed, lambda, total_counts, counts_per_second, selected_param

   ; Go through the list of numors, check for changes and write info

   for i = numor_ini+step, numor_end, step do begin

      ;rdset, nodata=1    ;Added because some times it seems as if reset to nodata=0. Why?
      ; Check if numor is valid 
      valid = 0
      while not valid  and i le numor_end do begin
        numor = rdrun(i)
        if size(numor,/type) eq 8 then valid = 1 else i = i + 1
      endwhile   
   
      if i le numor_end then begin
      
         ; Check for changes in conditions
         
         change = 0
         if abs(numor.param[18]-detector_distance) gt tolerance then change=1
         if abs(numor.param[14]-detector_angle) gt tolerance then change=1
         if abs(numor.param[57]-collimation) gt tolerance then change=1
         if abs(numor.param[58]-attenuator) gt tolerance then change=1
         if abs(numor.param[52]-lambda) gt tolerance then change=1

         if change then begin
            printf, u, " "
               if abs(numor.param[18]-detector_distance) gt tolerance then printf, u, format='("Detector distance changed. New detector distance = ", F8.2)', numor.param[18]
               if abs(numor.param[14]-detector_angle) gt tolerance then printf, u, format='("Detector angle changed. New detector angle = ", F8.2)', numor.param[14]
               if abs(numor.param[57]-collimation) gt tolerance then printf, u, format='("Collimation changed. New collimation = ", F8.2)', numor.param[57]
               if abs(numor.param[58]-attenuator) gt tolerance then printf, u, format='("Attenuator changed. New attenuator = ", I1)', long(numor.param[58])
               if abs(numor.param[52]-lambda) gt tolerance then printf, u, format='("Wavelength changed. New wavelength = ", F8.2)', numor.param[52]
            printf, u, " "
         endif
         
         ; Update values
   
         sample            = strmid(numor.experiment[0], 60, 20)
         position          = numor.param[65]
         time              = strmid(numor.experiment[1], 10, 8)
         monitor           = numor.param[4]
         counting_time     = numor.param[2] / 10.
         selector_speed    = numor.param[19]
         lambda            = numor.param[52]
         total_counts      = numor.param[3]
         counts_per_second = numor.param[3] / counting_time

         if okInd then selected_param = numor.param[index] else selected_param = 0.0 
            
         detector_distance = numor.param[18]
         detector_angle = numor.param[14]
         collimation = numor.param[57]
         attenuator = numor.param[58]
   
         ; Write log
   
         printf, u, format = '(I06, 3x, A-20, 1x, I-3, 1x, A-8, 1x, I8, 1x, I5, 2x, I5, 2x, F4.1, 1x, I10, 1x, I8, 1x, F8.2)', $
           numor.numor, sample, position, time, monitor, counting_time, selector_speed, lambda, total_counts, counts_per_second, selected_param

      endif

   endfor

endif else begin ;NEXUS file

   get_parameters_nexus, numor, run_number, instrument, user, localcontact, title, date, detector_distance, detector_angle, detector_offset, $
                                collimation, selector_speed, selector_tilt, attenuator, sample, position, time, monitor, counting_time, $
                                lambda, total_counts, counts_per_second, tof_mode, d11_dataType, value

   ;Write general information extracted from first numor

   printf, u, 'Instrument         : ', instrument
   printf, u, 'User               : ', user
   printf, u, 'LC                 : ', localcontact
   printf, u, 'Title              : ', title
   printf, u, 'Start date/time    : ', date   
   printf, u
   printf, u, 'Detector distance (read) = ', detector_distance
   printf, u, 'Detector angle (read)    = ', detector_angle
   printf, u, 'Detector offset (read)   = ', detector_offset
   printf, u, 'Collimation (read)       = ', collimation
   printf, u, 'Selector speed (read)    = ', selector_speed
   printf, u, 'Selector tilt angle      = ', selector_tilt
   printf, u, 'Attenuator               = ', long(attenuator)
   if strlowcase(instrument) eq 'd33' then printf, u, 'TOF mode                 = ', long(tof_mode)
   expt_day = strmid(strtrim(date, 1), 0, 9)

   ; Get values for extra parameter (if given in input)
   
   if n_elements(np) ne 0 then begin
      idx21 =strpos(numor,np) & idx21 = where(idx21 ge 0) & idx21 =idx21[0] 
      if idx21 ge 0 then selected_param = value[idx21] else selected_param = 0.0
   endif else selected_param = 0.0


   ; Write info header and information for 1st numor
   
   printf, u
   if strlowcase(instrument) eq 'd11' then begin
     printf, u, format = '("Run #", 4x, "Sample", 5x, "DataType", 1x, "Pos", 4x, "Start", 3x, "Monitor", 2x, "Time", 3x, "RPM", 2x, "Lambda", 1x, "Total cts.", 4x, "c/s", 5x, "Param")'
     printf, u, format = '(I06, 1x, A-17, 1x, I-3, 2x, I-3, 1x, A-8, 1x, I8, 1x, I5, 2x, I5, 2x, F4.1, 1x, I10, 1x, I8, 1x, F8.2)', $
       run_number, sample, d11_dataType, position, time, monitor, counting_time, selector_speed, lambda, total_counts, counts_per_second, selected_param
   endif else begin
     printf, u, format = '("Run #", 4x, "Sample", 14x, "Pos", 4x, "Start", 3x, "Monitor", 2x, "Time", 3x, "RPM", 2x, "Lambda", 1x, "Total cts.", 4x, "c/s", 5x, "Param")'
     printf, u, format = '(I06, 1x, A-22, 1x, I-3, 1x, A-8, 1x, I8, 1x, I5, 2x, I5, 2x, F4.1, 1x, I10, 1x, I8, 1x, F8.2)', $
       run_number, sample, position, time, monitor, counting_time, selector_speed, lambda, total_counts, counts_per_second, selected_param
   endelse

   ; Go through the list of numors, check for changes and write info

   for i = numor_ini+step, numor_end, step do begin

      numor = rdrun(i)
      ; Check if numor is valid 
      valid = 0
      while not valid and i le numor_end do begin
        numor = rdrun(i)
        if size(numor,/type) eq 7 then valid = 1 else i = i + 1
      endwhile   
      
      if i le numor_end then begin
      
         get_parameters_nexus, numor, run_number, instrument, user, localcontact, title, date, new_detector_distance, new_detector_angle, detector_offset, $
                                      new_collimation, selector_speed, selector_tilt, new_attenuator, sample, position, time, monitor, counting_time, $
                                      new_lambda, total_counts, counts_per_second, new_tof_mode, d11_dataType, value
 
         ; Get values for extra parameter (if given in input)
         if n_elements(np) ne 0 then begin
            idx21 =strpos(numor,np) & idx21 = where(idx21 ge 0) & idx21 =idx21[0] 
            if idx21 ge 0 then selected_param = value[idx21] else selected_param = 0.0
         endif else selected_param = 0.0

         ; Check for changes in conditions
   
         change = 0
         new_expt_day = strmid(strtrim(date, 1), 0, 9)
         if abs(new_detector_distance - detector_distance) gt tolerance then change = 1
         if abs(new_detector_angle - detector_angle) gt tolerance then change = 1
         if abs(new_collimation - collimation) gt tolerance then change = 1
         if abs(new_attenuator - attenuator) gt tolerance then change = 1
         if abs(new_lambda - lambda) gt tolerance then change = 1
         if strlowcase(instrument) eq 'd33' then begin
           if abs(new_tof_mode - tof_mode) gt tolerance then change = 1
         endif
         if (new_expt_day  ne expt_day) then change = 1
         
         if change then begin
            printf, u, " "
            if abs(new_detector_distance - detector_distance) gt tolerance then printf, u, "Detector distance changed. New detector distance = ", new_detector_distance
            if abs(new_detector_angle - detector_angle) gt tolerance then printf, u, "Detector angle changed. New detector angle = ", new_detector_angle
            if abs(new_collimation - collimation) gt tolerance then printf, u, "Collimation changed. New collimation = ", new_collimation
            if abs(new_attenuator - attenuator) gt tolerance then printf, u, "Attenuator changed. New attenuator = ", long(new_attenuator)
            if abs(new_lambda - lambda) gt tolerance then printf, u, "Wavelength changed. New wavelength = ", new_lambda
            if strlowcase(instrument) eq 'd33' then begin
               if abs(new_tof_mode - tof_mode) gt tolerance then printf, u, "D33 mode changed. New mode = ", new_tof_mode
            endif
            if (new_expt_day ne expt_day) then printf, u, "Day changed. New day = ", new_expt_day
            printf, u, " "
         endif 
   
         ; Update values
   
         detector_distance = new_detector_distance
         detector_angle = new_detector_angle
         collimation = new_collimation
         attenuator = new_attenuator
         lambda = new_lambda
         if strlowcase(instrument) eq 'd33' then tof_mode = new_tof_mode
         expt_day = new_expt_day
   
         ; Write log
     
         if strlowcase(instrument) eq 'd11' then begin
            printf, u, format = '(I06, 1x, A-17, 1x, I-3, 2x, I-3, 1x, A-8, 1x, I8, 1x, I5, 2x, I5, 2x, F4.1, 1x, I10, 1x, I8, 1x, F8.2)', $
             run_number, sample, d11_dataType, position, time, monitor, counting_time, selector_speed, lambda, total_counts, counts_per_second, selected_param
         endif else $
           printf, u, format = '(I06, 1x, A-22, 1x, I-3, 1x, A-8, 1x, I8, 1x, I5, 2x, I5, 2x, F4.1, 1x, I10, 1x, I8, 1x, F8.2)', $
           run_number, sample, position, time, monitor, counting_time, selector_speed, lambda, total_counts, counts_per_second, selected_param

      endif
      
   endfor

endelse

free_lun, u
rdset,nodata=0
if keyword_set(show) then xdisplayfile, file, font=ft_propor
status=1
return

mislog: if u lt 0 then lamp_error

rdset,nodata=0
status=4

end
