FUNCTION auto_remove_spectra, w_in, n_sdev = n_sdev, nolog = nolog
;+
;******* *******************
;** User callable
;** Removes all spectra with a peak-height/background ratio of less than
;** the mean value minus user given number of standard deviations
;**
;** KEYWORDS
;** /nolog       : if set, supresses output of badspecs in other_tit
;**                This is necessary in cases where other_tit exceeds 160 characters
;**
;** ARGUMENTS
;** n_sdev       : sets the number of standard deviations from the mean
;**                from which to remove the spectra (default = 2.0)
;**
;** DIMENSIONS
;** w_in(nchannels,nspectra) -> w_out(nchannels,nspectra-n) - n=number of bad specs
;**
;** COMMAND SYNTAX
;** e.g. w2=remove_spectra(w1 [,n_sdev = <s>] [,/nolog])
;**
;**                                        JRS 27/2/07
;-


       iprint = 0       ; if iprint>0, show debugging messages

       IF iprint THEN PRINT,'Start auto_remove_specs:'

       TAKE_DATP, dat

; ------------------------------------------------------------------------------
;Set up input workspace

       nspectra = (SIZE(w_in))[2]
       xmin = MIN(dat.x)
       xmax = MAX(dat.x)
       IF ((SIZE(dat.e))[0] EQ 0) THEN e = SQRT(w_in) ELSE e = dat.e
       bkgd   = FLTARR(nspectra) & dbkgd   = FLTARR(nspectra)
       height = FLTARR(nspectra) & dheight = FLTARR(nspectra)
       IF N_ELEMENTS(n_sdev) EQ 0 THEN n_sdev = 2.0

; -----------------------------------------------------------------------------
;Find height and background level of spectra

       FOR ispec = 0,nspectra - 1 DO BEGIN
              FITGAUSS,/all, dat.x, w_in[*,ispec], e[*,ispec], xmin, xmax, gauss, dgauss
              bkgd[ispec]   = gauss[0] & dbkgd[ispec]   = dgauss[0]
              height[ispec] = gauss[1] & dheight[ispec] = dgauss[1]
       ENDFOR
       ratio = height/bkgd

; -----------------------------------------------------------------------------
;Locate bad specs and remove from data

       i = WHERE(bkgd GT 0)
       mom = MOMENT(ratio[i],SDEV = sdev)
       ii = WHERE(ratio GT mom[0] - n_sdev*sdev AND bkgd GT 0)
       badspecs = WHERE(ratio LE mom[0] - n_sdev*sdev) + 1
;include dead spectra
       i = WHERE(bkgd EQ 0)
       badspecs = [badspecs,i+1]
       isort = SORT(badspecs)
       badspecs = badspecs[isort]

       w_out = w_in[*,ii,*]
       e_out = dat.e[*,ii,*]
       y_out = dat.y[ii]
       z_out = dat.z[ii]

; ------------------------------------------------------------------------------
;Return parameters and exit

       mod_datp, dat, "y", y_out
       mod_datp, dat, "e", e_out
       mod_datp, dat, "z", z_out

       bad = STRTRIM(badspecs,2) & badstring = bad[0]
       FOR i = 1, N_ELEMENTS(badspecs) - 1 DO badstring = badstring+','+bad[i]
       badstring='('+badstring+')'

       PRINT,'auto_remove_specs: Removed spectra ' + badstring
       IF NOT KEYWORD_SET(nolog) THEN $
              dat.other_tit=dat.other_tit+' -ars'+badstring

       GIVE_DATP, dat
       RETURN,w_out
       END
