;+
;
; :Description:
;   Function to subtract the background applying the self-absorption and self-shielding factors.
;   The operation done is w_out = (w_in - SAF*background) / SSF (but see other possibilities below)
;
; :Params:
;    w_in   : in, required, type="float array" - 2D workspace with the sample data
;    w_bg   : in, optional, type="float array" - 2D workspace with background data 
;                                                (should have same dimensions as w_in)
;    w_corr : in, optional, type="integer" - Number of the workspace containing the 
;                                            absorption correction factors
;    fBS    : in, optional, type="float" - Value between 0 and 1 giving the fraction of 
;                                          backscattered neutrons that pass a second time 
;                                          through the sample
;    transm : in, optional, type="float" - Value between 0 and 1 giving the 'average' sample 
;                                          transmission
;
; :Info:
; 
;   The values of the correction factor SAF and SSF are given in a workspace, w_corr,
;   which may have 1, 2 or 3 dimensions:
;
;     - If w_corr is 1D (a vector), then it is assumed that it contains only A_S,SC i.e.
;       the self-shielding factor, SSF.
;
;     - If w_corr is 2D, then w_corr = [[2theta], [SSF,SAF, ...]], i.e.
;       w_corr[*,0] will contain SSF(2theta), and w_corr[*,1] SAF(2theta).
;       Any w_corr[*,>1] are not used, but can be used to keep other information,
;       e.g. the different absorption factors A_S,SC , A_C,SC , and A_C,C.
;
;     - If w_corr is 3D, then w_corr = [[2theta], [energy], [SSF,SAF, ...]], i.e.
;       w_corr[*,*,0] will contain SSF(2theta,w), and w_corr[*,*,1] SAF(2theta,w).
;       The energy is assumed to be given in the same units as in w_in (normally in meV).
;       Otherwise the energy units in w_in or in w_corr should be converted before calling
;       this function. As before, any data stored in w_corr[*,*,>1] will not be used.
;
;   In all the cases, the values of SSF and SAF will be interpolated to the scattering
;   angles (and energies) available in the sample workspace w_in.
;
;   fBS should be a float between 0 and 1 and corresponds to the fraction of backscattered
;   neutrons that traverse the sample a second way on their way from analyzer to detector.
;   If not given, the default value is 0.
;
;   transm should be also a float between 0 and 1 corresponding to the sample transmission, T.
;   If fBS is not 0, then the final intensity is divided by the attenuation factor
;   (1-fBS) + fBS*T in order to correct for the attenuation due to the second passage through
;   the sample. It should be noted that this is a crude approximation (see e.g. J. Wuttke:
;   'Absorption-Correction Factor for Scattering from Flat or Tubular Samples: Open-Source
;   Implementation libabsco, and Why it Should be Used with Caution',
;   http://apps.jcns.fz-juelich.de/doku/sc/_media/abs00.pdf), but taking into account
;   the uncertainties in the determination of fBS, probably this is not a big issue.
;   If fBS > 0, but transm is not given, T is approximated as SSF(2-theta = 0).
;
; :Command syntax: 
; 
;   This function admits different calls, and the operation done and the workspace returned
;   will depend on the parameters passed to the function.
;
;   w10 = background_subtract(w1, fBS=0.5, transm=0.9) ,
;   No background or abscor factors given, so only backscattering attenuation applied: w10 = w1 / [(1-fBS)+(fBS*T)]
;
;   w10 = background_subtract(w1, w2, transm=0.9) ,
;   No abscor factors given: w10 = w1 - T*w2
;
;   w10 = background_subtract(w1, w2, transm=0.9, fBS=0.5) ,
;   No abscor factors given: w10 = (w1 - T*w2) / [(1-fBS)+(fBS*T)]
;
;   w10 = background_subtract(w1, w_corr=3) ,
;   No background: w10 = w1/SSF
;
;   w10 = background_subtract(w1, w2, w_corr=3) ,
;   w10 = (w1-SAF*w2)/SSF
;
;   w10 = background_subtract(w1, w2, w_corr=3, fBS=0.5) ,
;   w10 = [(w1-SAF*w2)/SSF] / [(1-fBS)+(fBS*SSF(0))]
;
;   w10 = background_subtract(w1, w2, w_corr=3, fBS=0.5, transm=0.9) ,
;   w10 = [(w1-SAF*w2)/SSF] / [(1-fBS)+(fBS*T)]
;
; :Author: M. A. Gonzalez
; 
; :History:
;    M. A. Gonzalez, 31 March 2015
;
;-

function background_abscor_correct, w_in, w_bg, w_corr = w_corr, fBS = fBS, transm = transm

;-------------------------------------------------------------------------------
; Get sizes and check dimensions and values

s_in=size(w_in) 
if (s_in[0] ne 2) then begin
   print
   print, ' Data workspace is not a 2D array. Not valid input!'
   print, ' NOTHING DONE!!!'
   return, w_in
endif

take_datp, sample  ;take parameters corresponding to w_in
x_out = sample.x ;energies
y_out = sample.y ;angles
w_out = w_in     
if n_elements(sample.e) eq n_elements(w_in) then e_out = sample.e else e_out = sqrt(w_in) ;errors

if n_elements(w_bg) ne 0 then begin ;check background dimensions and get parameters
   s_bg=size(w_bg) 
   if (s_bg[0] ne 2) then begin
      print, ' Background workspace is not a 2D array. Not valid input!'
      print, ' NOTHING DONE!!!'
      return, w_in
   endif
   if (s_bg[1] ne s_in[1]) then begin
      print, ' Background workspace has a different number of X values than data workspace!'
      print, ' NOTHING DONE!!!'
      return, w_in
   endif
   if (s_bg[2] ne s_in[2]) then begin
      print, ' Background workspace has a different number of Y values than data workspace!'
      print, ' NOTHING DONE!!!'
      return, w_in
   endif
   take_datp, bgr, /third
   x_bg = bgr.x ;energies
   y_bg = bgr.y ;angles
   if n_elements(bgr.e) eq n_elements(w_bg) then e_bg = bgr.e else e_bg = sqrt(w_bg) ;errors
endif else begin
   print, ' No background given. Only SSF scaling will be applied'
endelse

if n_elements(fBS) ne 0 then begin
   if fBS lt 0.0 then begin & print, 'fBS cannot be < 0' & fBS = 0.0 & endif
   if fBS gt 1.0 then begin & print, 'fBS cannot be > 1' & fBS = 1.0 & endif
endif

if n_elements(transm) ne 0 then begin
  if transm lt 0.0 then begin & print, 'transmission cannot be < 0' & transm = 0.0 & endif
  if transm gt 1.0 then begin & print, 'transmission cannot be > 1' & transm = 1.0 & endif
endif

;-------------------------------------------------------------------------------
; Get absorption correction factors and interpolate to sample angles/energies

if n_elements(w_corr) gt 0 then begin
  
   take_w, corr, W=w_corr
   take_datp, corr_params, W=w_corr
   sw = size(corr)

   case sw[0] of

      1: begin ;Only SSF(2-theta) given
         ssf_values = corr
         saf_values = corr*0.0 + 1.0
         ssf_angles = corr_params.x
      end

      2: begin ;SSF(2-theta) and SAF(2-theta) given
         ssf_values = corr[*,0]
         saf_values = corr[*,1]
         ssf_angles = corr_params.x
      end

      3: begin ;SSF(2-theta,w) and SAF(2-theta,w) given
         ssf_values = corr[*,*,0]
         saf_values = corr[*,*,1]
         ssf_angles = corr_params.x
         ssf_energies = corr_params.y
      end

   endcase

   ;Interpolate to scattering angles and energies available in w_in
   if n_elements(ssf_energies) eq 0 then begin

      print, 'No w dependence on correction factor. Interpolating to experimental angles ...'
      print, 'Correction file --> 2-theta range     = ', min(ssf_angles), max(ssf_angles)
      print, 'Experimental angles --> 2-theta range = ', min(y_out), max(y_out)
      print
      ssf = interpol(ssf_values, ssf_angles, y_out)
      saf = interpol(saf_values, ssf_angles, y_out)
      ssf = transpose(rebin(ssf, n_elements(y_out), n_elements(x_out)))
      saf = transpose(rebin(saf, n_elements(y_out), n_elements(x_out)))

   endif else begin

      print, 'Interpolating to experimental angles and energies.'
      print, 'Correction file -->     2-theta range = ', min(ssf_angles), max(ssf_angles)
      print, 'Experimental angles --> 2-theta range = ', min(y_out), max(y_out)
      print
      print, 'Energies should have same units in w_in and in file containing the correction factors!'
      print, 'Correction file -->       w range = ', min(ssf_energies), max(ssf_energies)
      print, 'Experimental energies --> w range = ', min(x_out), max(x_out)
      print
      xvalues = interpol(findgen(n_elements(ssf_energies)), ssf_energies, x_out)
      yvalues = interpol(findgen(n_elements(ssf_angles)), ssf_angles, y_out)
      ssf = interpolate(transpose(ssf_values), xvalues, yvalues, /grid)
      saf = interpolate(transpose(saf_values), xvalues, yvalues, /grid)
   endelse

endif

;-------------------------------------------------------------------------------
; Correct data (different options depending on existence of w_corr and w_bg)


if n_elements(w_corr) eq 0 then begin

   ;if transmission is not given, set to 1
   if n_elements(transm) eq 0 then transm = 1.0

   ;No absorption correction - Check if background / backscattering att factor given.
   if n_elements(w_bg) ne 0 then begin
    
      w_out = w_in - transm*w_bg
      e_out = sqrt(e_out^2 + (transm*e_bg)^2)

      if n_elements(fBS) ne 0 then begin
         print, ' Subtracting background (no abscor) and applying backscattering attenuation factor '
         att = (1.0-fBS) + fBS * transm
         w_out = w_out / att
         e_out = e_out / att
      endif else begin
         print, ' Subtracting background (no abscor) only '
      endelse
    
   endif else begin ;No background given
      
      if n_elements(fBS) ne 0 then begin
         att = (1.0-fBS) + fBS * transm
         w_out = w_out / att
         e_out = e_out / att
         print, ' No background given. Applying only backscattering attenuation factor = ', att
      endif else begin
         print, ' Nothing to do!, Returning input workspace'
      endelse
   
   endelse
   
endif else begin ;Subtract background using SSF and SAF factors 

   ;if transmission is not given, approximate as SSF(2-theta~0,w~0)
   if n_elements(transm) eq 0 then begin
      wmin = min(abs(x_out), x_index)
      tmin = min(abs(y_out), y_index)
      transm = ssf[x_index, y_index]
   endif
   
   if n_elements(w_bg) eq 0 then begin ;No background to subtract. Only SSF applied.

      w_out = w_out / ssf
      e_out = e_out / ssf

      if n_elements(fBS) ne 0 then begin
        print, ' No background. Applying SSF and backscattering attenuation factors '
        att = (1.0-fBS) + fBS * transm
        w_out = w_out / att
        e_out = e_out / att
      endif else begin
        print, ' No background. Applying SSF attenuation factor only '
      endelse

  endif else begin

      w_out = ( w_out - saf * w_bg ) / ssf
      e_out = sqrt(e_out^2 + (saf*e_bg)^2) / ssf

      if n_elements(fBS) ne 0 then begin
        print, ' Background subtracted applying SAF + SSF and backscattering attenuation factors '
        att = (1.0-fBS) + fBS * transm
        w_out = w_out / att
        e_out = e_out / att
      endif else begin
        print, ' Background subtracted applying SAF + SSF attenuation factor  '
      endelse

  endelse
    
endelse


;-------------------------------------------------------------------------------
; Return corrected data

mod_datp, sample, 'X', x_out
mod_datp, sample, 'Y', y_out
mod_datp, sample, 'E', e_out
give_datp, sample
return, w_out

end

