pro insect,nann,rann,x0,y0,u,v,dism,disp,nint
  ;+
  ;** ******
  ;** Calculates the intersections of a directed line with a set of
  ;** concentric circles centered at the origin.  The number of circles
  ;** is nann and their radii, in increasing order, are (rann(i),i=1,nann).
  ;** The line goes through the point (x0,y0) and its direction is (u,v).
  ;** The calculated distances to intersection with circle k are dism(k),
  ;** disp(k), such that dism(k).le.disp(k).  If there is no intersection
  ;** both quantities are set equal to zero.  The number of intersections is
  ;** nint.
  ;
  ;-
  front = -x0*u-y0*v
  disc0 = front^2-x0^2-y0^2
  disc = (disc0+rann[0:nann-1]^2) > 0.0
  root = sqrt(disc)
  dism = front-root
  disp = front+root
  ind = where(disc gt 0.0,nint)
  nint = nint*2
  return
end

pro ninsect,nann,rann,x0,y0,nphi,u,v,dism,disp,nint
  ;+
  ;** *******
  ;** Calculates the intersections of nphi directed lines with a set of
  ;** concentric circles centered at the origin.  The number of circles
  ;** is nann and their radii, in increasing order, are (rann(i),i=1,nann).
  ;** The lines go through the point (x0,y0) and their directions are (u,v).
  ;** The calculated distances to intersection with circle k are dism(k),
  ;** disp(k), such that dism(k).le.disp(k).  If there is no intersection
  ;** both quantities are set equal to the same value.
  ;** The numbers of intersections are nint.
  ;
  ;-
  front = (-x0*u-y0*v)##(intarr(nann)+1)
  disc0 = front^2-x0^2-y0^2
  disc = (disc0+(intarr(nphi)+1)##(rann^2)) > 0.0
  root = sqrt(disc)
  dism = front-root
  disp = front+root
  nint = 2*round(total(disc gt 0.0,2))
  return
end

pro nout_calc,nann,dism,disp,sigma,nphi,arg_out
  ;+
  ;** *********
  ;** calculates the attenuation from scattering point to all detectors
  ;-

  dm=dism>0
  dp=disp>0
  sg=sigma[0:nann-1]#(intarr(nphi)+1)
  
  s1=total((dp-dm)*sg,1)
  ssg=shift(sg,-1)
  s2=total((dp-dm)*ssg,1)
  s3=(dp[nann-1,*]-dm[nann-1,*])*ssg[nann-1]
  arg_out=s1-s2+s3
  
  return
end

pro nout_calc_nw,nann,dism,disp,sigma,nphi,arg_out
  ;+
  ;** *********
  ;** calculates the attenuation from scattering point to all detectors
  ;** works with a 2D sigma array, corresponding to sigma[region,energy]
  ;-

  dm=dism>0
  dp=disp>0
  ;sg=sigma[0:nann-1]#(intarr(nphi)+1)
  sg = rebin(sigma, nann, n_elements(sigma[0,*]), nphi) & sg = transpose(sg, [0,2,1])
  dm = rebin(dm, nann, nphi, n_elements(sigma[0,*])) 
  dp = rebin(dp, nann, nphi, n_elements(sigma[0,*])) 
  s1 = total((dp-dm)*sg,1)
  nw = (size(sg))[3]
  ssg = sg & for i = 0, nw-1 do ssg[*,*,i] = shift(sg[*,*,i], -1)
  s2 =total((dp-dm)*ssg,1)
  s3 = ssg[nann-1,*,*] & for i = 0, nw-1 do s3[0,*,i] =(dp[nann-1,*,i]-dm[nann-1,*,i])*ssg[nann-1,0,i]
  arg_out=s1-s2+s3
  
  return
end


;+
;
; :Description:
;   Function to compute the attenuation factors for a cylindrical geometry.
;   This function will return a vector, a 2D array or a 3D array (depending on
;   the input parameters) with the values of SSF, SAF, Assc, Acsc, and Acc (see
;   below for explanations) as a function of the scattering angles (and the energy
;   transfer, if given).
;
; :Params:
;    angles : in, required, type="float array" - 1D array with values of 2-theta (scattering angles)
;    diameterOutSample : in, required, type="float" - external diameter of the sample 
;                        (or internal diameter of out container)
;    sampleSigmaS : in, required, type="float" - Scattering cross section of the sample
;                   (in units consistent with those employed for the sample thickness, usually cm^-1)
;    sampleSigmaA : in, required, type="float" - Absorption cross section of the sample
;                   (in units consistent with those employed for the sample thickness, usually cm^-1)
;    sampleThickness : in, optional, type="float" - Thickness of the sample
;                      (in units consistent with those employed for the cross sections, usually cm)
;                      (if not given, a massive sample is considered)
;    cellOutThickness : in, optional, type="float" - Thickness of the wall of the external container
;                      (in units consistent with those employed for the cross sections, usually cm)
;                      (if not given, no external container is used)
;    cellInThickness : in, optional, type="float" - Thickness of the wall of the internal container
;                      (in units consistent with those employed for the cross sections, usually cm)
;                      (if not given, no internal container is used)
;                      (a value >= internal radius corresponds to a massive container)
;    cellSigmaS : in, optional, type="float" - Scattering cross section of the container
;                 (in units consistent with those employed for the sample thickness, usually cm^-1)
;    cellSigmaA : in, optional, type="float" - Absorption cross section of the container
;                 (in units consistent with those employed for the sample thickness, usually cm^-1)
;    w  : in, optional, type="float array" - 1D array with values containing energy transfers
;         (w > 0 corresponds to neutron energy gains)
;    E0 : in, optional, type="float" - Incident neutron energy   
;         (if not given, set to 25.3 meV, corresponding to a wavelength = 1.798 angstroms) 
;    nr : in, optional, type="integer" - number of points to integrate over dr (if not given, a default value of 10 is used) 
;    nomega : in, optional, type="integer" - number of points to integrate over domega (if not given, a default value of 360 is used)
;                                            
; :Info:
;
;   Evaluation of correction coefficients for transmission in the case of a
;   cylindrical sample. Both absorption and self shielding are taken into account.
;   Simple approach using equation (5) in reference [1].
;
;   Crude numerical approximation to the integration by evaluating the input and
;   output pahs on a regular grid. 
;   To test in the future numerical integration using Gaussian quadrature --> int2d.
;   Factors obtained are in reasonable agreement with the results obtained
;   with the libabsco library [1].
;   Using some geometrical routines from DAVE to compute intersection points of the
;   beam with the different circles defining the sample.
;
;   Is:   theoretical scattering from sample, without absorption
;   Ic:   theoretical scattering from container, without absorption
;   Iscsc:  scattering from sample plus container (i.e. measured scattering)
;   Icc:  scattering from container when alone (i.e. measured)
;   Issc: scattering from sample itself when inside container
;   Icsc: scattering from can itself when sample inside
;
;   measured scattering Iscsc = Issc + Icsc
;
;   Issc  = Assc*Is , where Assc is the self-shielding of scattering by the sample (in sample measurement)
;   Icsc  = Acsc*Ic , where Acsc is the self-shielding of scattering by the container (in sample measurement)
;   Icc   = Acc*Ic  , where Acc is the self-shielding of scattering by the container (in empty cell measurement)
;
;   Is = (1/Assc) * [Iscsc - (Acsc/Acc)*Icc ]
;
;   It is possible to define the sample attenuation factor, SAF = Acsc/Acc, and
;   the self-shielding factor, SSF = Assc, so
;
;   Is = (1/SSF) * (Iscsc - SAF*Icc)
;
; [1] Libabsco (http://apps.jcns.fz-juelich.de/doku/sc/absco)
; [2] DAVE (http://www.ncnr.nist.gov/dave)
;
; :Command syntax:
;
;   This function admits different calls, and the operation done and the dimensions
;   of the workspace returned will depend on the parameters passed to the function.
;
;   w10 = calc_abscor_cyl(x1, ) ,
;   ...
;
; :Author: M. A. Gonzalez
;
; :History:
;    M. A. Gonzalez, 7 April 2015
;
;-


function calc_abscor_cyl, angles, diameterOutSample, sampleSigmaS, sampleSigmaA, $
                          sampleThickness = sampleThickness, cellOutThickness = cellOutThickness,  $
                          cellInThickness = cellInThickness, cellSigmaS = cellSigmaS, $
                          cellSigmaA = cellSigmaA, w = w, E0 = E0, nr = nr, nomega = nomega

lamp_journal, 'Calculation of absorption corrections for a cylinder:'

;-------------------------------------------------------------------------------
; Check if input is correct

if n_elements(diameterOutSample) eq 0 then begin
  print, 'External diameter of the sample missing!'
  lamp_journal, '  External diameter of the sample missing!'
  return, 0
endif else begin
  if diameterOutSample lt 0.0 then begin
    print, 'External diameter of the sample cannot be negative!'
    lamp_journal, '  External diameter of the sample cannot be negative!'
    return, 0
  endif
endelse

if n_elements(sampleSigmaS) eq 0 then begin
  print, 'Sample scattering cross section missing!'
  lamp_journal, '  Sample scattering cross section missing!'
  return, 0
endif else begin
  if sampleSigmaS lt 0 then begin
    print, 'Sample scattering cross section < 0!'
    lamp_journal, '  Sample scattering cross section < 0!'
    return, 0
  endif
endelse

if n_elements(sampleSigmaA) eq 0 then begin
  print, 'Sample absorption cross section missing!'
  lamp_journal, '  Sample absorption cross section missing!'
  return, 0
endif else begin
  if sampleSigmaA lt 0 then begin
    print, 'Sample absorption cross section < 0!'
    lamp_journal, '  Sample absorption cross section < 0!'
    return, 0
  endif
endelse

;If incident energy not given, assume E0 = 25.3 meV, i.e. lambda = 1.798 A
E0ref = 81.8049/1.798^2  
if n_elements(E0) eq 0 then E0 = E0ref

;Number of points in r and omega to compute  integral
if n_elements(nr) eq 0 then nr = 50
if n_elements(nomega) eq 0 then nomega = 360

;No container 
if n_elements(cellInThickness) eq 0 then cellInThickness = 0.0
if n_elements(cellOutThickness)eq 0 then cellOutThickness = 0.0
if n_elements(cellSigmaS) eq 0 then cellSigmaS = 0.0
if n_elements(cellSigmaA)eq 0 then cellSigmaA = 0.0
if cellInThickness lt 0 or cellOutThickness lt 0 then begin
  print, 'Negative thickness not allowed!'
  lamp_journal, '  Negative thickness not allowed!'
  return, 0
endif

;-------------------------------------------------------------------------------
; Determine type of output (1D, 2D, 3D)

if n_elements(w) gt 0 then begin ;energies given --> 3D workspace (even if only SSF computed)
   w_out = fltarr(n_elements(angles), n_elements(w), 5)
endif else begin
   if cellOutThickness eq 0 and cellInThickness eq 0 then begin ;no background --> 1D wks (only SSF)
       w_out = fltarr(n_elements(angles))
   endif else begin; 2D wks (SSF, SAF, A_S,SC, A_C,SC, A_C,C)
       w_out = fltarr(n_elements(angles), 5)
   endelse
endelse

;-------------------------------------------------------------------------------
; Compute scattering and absorption cross sections for sample and container
; Create the attenuation coefficients matrices
  
  if n_elements(w) gt 0 then Ef = E0 - w else Ef = E0
  
  sigma0_s = sampleSigmaS + sampleSigmaA * sqrt(E0ref/E0)
  sigmaf_s = sampleSigmaS + sampleSigmaA * sqrt(E0ref/Ef)
  sigma0_c = cellSigmaS + cellSigmaA * sqrt(E0ref/E0)
  sigmaf_c = cellSigmaS + cellSigmaA * sqrt(E0ref/Ef)


;-----------------------------------------------------------------------------------------
; Define the geometry case requested, number of regions and characteristics of each region
; Generate also grid of (X,Y) points for each region 
; geometry = 1 : Massive sample (1 region)
; geometry = 2 : Massive sample + container_out (2 regions)
; geometry = 3 : Hollow sample (no container) (2 regions)
; geometry = 4 : Hollow sample + container out (3 regions)
; geometry = 5 : Hollow sample + massive inner container (2 regions); unusual geometry!
; geometry = 6 : Hollow sample + massive inner container + container out (3 regions)
; geometry = 7 : Hollow sample + hollow inner container & no external container (3 regions); unusual geometry!
; geometry = 8 : Hollow sample + hollow inner container + container out (4 regions)

if n_elements(sampleThickness) eq 0 then begin ;sample is a massive cylinder

  if cellInThickness gt 0.0 then begin

    print, 'Massive sample (no sample thickness) given incompatible with internal cell thickness!'
    lamp_journal, 'Massive sample (no sample thickness) given incompatible with internal cell thickness!'
    return, 0

  endif else begin

    if cellOutThickness eq 0 then begin ;massive sample, no container

      geometry = 1
      nann = 1 & radius = dblarr(nann) & sigma0 = dblarr(nann) 
      radius[0] = 0.5 * diameterOutSample
      sigma0[0] = sigma0_s
      if n_elements(w) eq 0 then begin
        sigmaf = dblarr(nann) 
        sigmaf[0] = sigmaf_s 
      endif else begin
        sigmaf = dblarr(nann, n_elements(w))
        sigmaf[0,*] = sigmaf_s
      endelse  
      
      r1s = 0.0 & r2s = radius[0] 
      
      ;sigmas for measurements with no sample
      sigma0_noS = sigma0 & sigmaf_noS = sigmaf
      sigma0_noS[0] = 0.0
      if n_elements(w) eq 0 then sigmaf_noS[0] = 0.0 else sigmaf_noS[0,*] = 0.0

    endif else begin ;massive sample, external container

      geometry = 2
      nann = 2 & radius = dblarr(nann) & sigma0 = dblarr(nann) 
      radius[0] = 0.5 * diameterOutSample
      radius[1] = radius[0] + cellOutThickness
      sigma0[0] = sigma0_s
      sigma0[1] = sigma0_c
      if n_elements(w) eq 0 then begin
        sigmaf = dblarr(nann)
        sigmaf[0] = sigmaf_s
        sigmaf[1] = sigmaf_c
      endif else begin
        sigmaf = dblarr(nann, n_elements(w))
        sigmaf[0,*] = sigmaf_s
        sigmaf[1,*] = sigmaf_c
      endelse
      
      r1s = 0.0        & r2s = radius[0] 
      r1co = radius[0] & r2co = radius[1] 

      ;sigmas for measurements with no sample
      sigma0_noS = sigma0 & sigmaf_noS = sigmaf
      sigma0_noS[0] = 0.0
      if n_elements(w) eq 0 then sigmaf_noS[0] = 0.0 else sigmaf_noS[0,*] = 0.0

    endelse ;end cellOutThickness

  endelse ;end check cellInThickness is zero for massive sample
    
endif else begin ;sample is a hollow cylinder

  if cellInThickness eq 0 then begin ;no inner container
    
    if cellOutThickness eq 0 then begin ;sample hollow cylinder, no container

      geometry = 3
      nann = 2 & radius = dblarr(nann) & sigma0 = dblarr(nann) 
      radius[0] = 0.5 * diameterOutSample - sampleThickness
      radius[1] = 0.5 * diameterOutSample
      sigma0[0] = 0.0
      sigma0[1] = sigma0_s
      if n_elements(w) eq 0 then begin
        sigmaf = dblarr(nann)
        sigmaf[0] = 0.0
        sigmaf[1] = sigmaf_s
      endif else begin
        sigmaf = dblarr(nann, n_elements(w))
        sigmaf[0,*] = 0.0
        sigmaf[1,*] = sigmaf_s
      endelse
      
      r1s = radius[0] & r2s = radius[1] 

      ;sigmas for measurements with no sample
      sigma0_noS = sigma0 & sigmaf_noS = sigmaf
      sigma0_noS[1] = 0.0
      if n_elements(w) eq 0 then sigmaf_noS[1] = 0.0 else sigmaf_noS[1,*] = 0.0
            
    endif else begin ;sample hollow cylinder in a external container

      geometry = 4
      nann = 3 & radius = dblarr(nann) & sigma0 = dblarr(nann)
      radius[0] = 0.5 * diameterOutSample - sampleThickness
      radius[1] = 0.5 * diameterOutSample
      radius[2] = radius[1] + cellOutThickness
      sigma0[0] = 0.0
      sigma0[1] = sigma0_s
      sigma0[2] = sigma0_c
      if n_elements(w) eq 0 then begin
        sigmaf = dblarr(nann)
        sigmaf[0] = 0.0
        sigmaf[1] = sigmaf_s
        sigmaf[2] = sigmaf_c
      endif else begin
        sigmaf = dblarr(nann, n_elements(w))
        sigmaf[0,*] = 0.0
        sigmaf[1,*] = sigmaf_s
        sigmaf[2,*] = sigmaf_c
      endelse
      
      r1s = radius[0]  & r2s = radius[1] 
      r1co = radius[1] & r2co = radius[2] 

      ;sigmas for measurements with no sample
      sigma0_noS = sigma0 & sigmaf_noS = sigmaf
      sigma0_noS[1] = 0.0
      if n_elements(w) eq 0 then sigmaf_noS[1] = 0.0 else sigmaf_noS[1,*] = 0.0

    endelse  
    
  endif else begin ;there is an inner container
    
    if (cellInThickness ge (0.5*diameterOutSample-sampleThickness)) then begin ;massive inner container
      
      if cellOutThickness eq 0 then begin ;no external container, but very unusual case
        
        geometry = 5
        nann = 2 & radius = dblarr(nann) & sigma0 = dblarr(nann) 
        radius[0] = 0.5 * diameterOutSample - sampleThickness
        radius[1] = 0.5 * diameterOutSample
        sigma0[0] = sigma0_c
        sigma0[1] = sigma0_s
        if n_elements(w) eq 0 then begin
          sigmaf = dblarr(nann)
          sigmaf[0] = sigmaf_c
          sigmaf[1] = sigmaf_s
        endif else begin
          sigmaf = dblarr(nann, n_elements(w))
          sigmaf[0,*] = sigmaf_c
          sigmaf[1,*] = sigmaf_s
        endelse
        
        r1ci = 0.0      & r2ci = radius[0] 
        r1s = radius[0] & r2s = radius[1] 

        ;sigmas for measurements with no sample
        sigma0_noS = sigma0 & sigmaf_noS = sigmaf
        sigma0_noS[1] = 0.0
        if n_elements(w) eq 0 then sigmaf_noS[1] = 0.0 else sigmaf_noS[1,*] = 0.0
     
      endif else begin ;massive inner container + sample + out container
      
        geometry = 6
        nann = 3 & radius = dblarr(nann) & sigma0 = dblarr(nann) 
        radius[0] = 0.5 * diameterOutSample - sampleThickness
        radius[1] = 0.5 * diameterOutSample
        radius[2] = radius[1] + cellOutThickness
        sigma0[0] = sigma0_c
        sigma0[1] = sigma0_s
        sigma0[2] = sigma0_c
        if n_elements(w) eq 0 then begin
          sigmaf = dblarr(nann)
          sigmaf[0] = sigmaf_c
          sigmaf[1] = sigmaf_s
          sigmaf[2] = sigmaf_c
        endif else begin
          sigmaf = dblarr(nann, n_elements(w))
          sigmaf[0,*] = sigmaf_c
          sigmaf[1,*] = sigmaf_s
          sigmaf[2,*] = sigmaf_c
        endelse
        
        r1ci = 0.0       & r2ci = radius[0] 
        r1s = radius[0]  & r2s = radius[1]       
        r1co = radius[1] & r2co = radius[2] 

        ;sigmas for measurements with no sample
        sigma0_noS = sigma0 & sigmaf_noS = sigmaf
        sigma0_noS[1] = 0.0
        if n_elements(w) eq 0 then sigmaf_noS[1] = 0.0 else sigmaf_noS[1,*] = 0.0
        
      endelse  
      
    endif else begin ;inner empty space
      
      if cellOutThickness eq 0 then begin ;no external container, but very unusual case

        geometry = 7
        nann = 3 & radius = dblarr(nann) & sigma0 = dblarr(nann) 
        radius[0] = 0.5 * diameterOutSample - sampleThickness - cellInThickness
        radius[1] = 0.5 * diameterOutSample - sampleThickness
        radius[2] = 0.5 * diameterOutSample
        sigma0[0] = 0.0
        sigma0[1] = sigma0_c
        sigma0[2] = sigma0_s
        if n_elements(w) eq 0 then begin
          sigmaf = dblarr(nann)
          sigmaf[0] = 0.0
          sigmaf[1] = sigmaf_c
          sigmaf[2] = sigmaf_s
        endif else begin
          sigmaf = dblarr(nann, n_elements(w))
          sigmaf[0,*] = 0.0
          sigmaf[1,*] = sigmaf_c
          sigmaf[2,*] = sigmaf_s
        endelse
        
        r1ci = radius[0] & r2ci = radius[1] 
        r1s = radius[1]  & r2s = radius[2] 

        ;sigmas for measurements with no sample
        sigma0_noS = sigma0 & sigmaf_noS = sigmaf
        sigma0_noS[2] = 0.0
        if n_elements(w) eq 0 then sigmaf_noS[2] = 0.0 else sigmaf_noS[2,*] = 0.0

      endif else begin ;empty + inner container + sample + out container

        geometry = 8
        nann = 4 & radius = dblarr(nann) & sigma0 = dblarr(nann) 
        radius[0] = 0.5 * diameterOutSample - sampleThickness - cellInThickness
        radius[1] = 0.5 * diameterOutSample - sampleThickness
        radius[2] = 0.5 * diameterOutSample
        radius[3] = 0.5 * diameterOutSample + cellOutThickness
        sigma0[0] = 0.0
        sigma0[1] = sigma0_c
        sigma0[2] = sigma0_s
        sigma0[3] = sigma0_c
        if n_elements(w) eq 0 then begin
          sigmaf = dblarr(nann)
          sigmaf[0] = 0.0
          sigmaf[1] = sigmaf_c
          sigmaf[2] = sigmaf_s
          sigmaf[3] = sigmaf_c
        endif else begin
          sigmaf = dblarr(nann, n_elements(w))
          sigmaf[0,*] = 0.0
          sigmaf[1,*] = sigmaf_c
          sigmaf[2,*] = sigmaf_s
          sigmaf[3,*] = sigmaf_c
        endelse
       
        r1ci = radius[0] & r2ci = radius[1] 
        r1s = radius[1]  & r2s = radius[2] 
        r1co = radius[2] & r2co = radius[3] 

        ;sigmas for measurements with no sample
        sigma0_noS = sigma0 & sigmaf_noS = sigmaf
        sigma0_noS[2] = 0.0
        if n_elements(w) eq 0 then sigmaf_noS[2] = 0.0 else sigmaf_noS[2,*] = 0.0

      endelse

    endelse
    
  endelse
    
endelse

;print, 'geometry = ', geometry, nann
;print, 'radius = ', radius
;print, 'sigma0 = ', sigma0
;print, 'sigmaf = ', sigmaf

;detector angles
nphi  = n_elements(angles)
cphi  = dblarr(nphi)
sphi  = dblarr(nphi)
phird = dblarr(nphi)
phird = angles*!dtor
cphi  = cos(phird)
sphi  = sin(phird)

;geometry (insect) arrays
dism_in  = dblarr(nann)
disp_in  = dblarr(nann)
dism_out = dblarr(nann)
disp_out = dblarr(nann)
path     = dblarr(nann)
attenh   = dblarr(nann)

;output arrays
if n_elements(w) eq 0 then acc   = dblarr(nphi) else acc   = dblarr(nphi, n_elements(w))
if n_elements(w) eq 0 then acsc  = dblarr(nphi) else acsc  = dblarr(nphi, n_elements(w))
if n_elements(w) eq 0 then acci  = dblarr(nphi) else acci  = dblarr(nphi, n_elements(w))
if n_elements(w) eq 0 then acsci = dblarr(nphi) else acsci = dblarr(nphi, n_elements(w))
if n_elements(w) eq 0 then acco  = dblarr(nphi) else acco  = dblarr(nphi, n_elements(w))
if n_elements(w) eq 0 then acsco = dblarr(nphi) else acsco = dblarr(nphi, n_elements(w))
if n_elements(w) eq 0 then assc  = dblarr(nphi) else assc  = dblarr(nphi, n_elements(w))
transmission_c = 0.0
transmission_s = 0.0

;define points for integration

domega = 2.0d0 * !pi / float(nomega)
omega = findgen(nomega) * domega
comega = cos(omega)
somega = sin(omega)

;weight = dblarr(nr,nomega) + 1.0 ;weights for trapezoidal rule
;for i = 0, nr-1 do begin
;  for j = 0, nomega-1 do begin
;    if (i eq 0) or (i eq nr-1) then weight[i,j] *= 0.5
;    if (j eq 0) or (j eq nr-1) then weight[i,j] *= 0.5
;  endfor
;endfor

;weight = dblarr(nr,nomega) + 1.0 ;weights for Simpson's rule in 2D
;for i = 0, nr-1 do begin
;  for j = 0, nomega-1 do begin
;    if (i eq 0) or (i eq nr-1) then weight[i,j] *= 1.0 else $
;      if ((i mod 2) eq 1) then weight[i,j] *= 4.0 else weight[i,j] *= 2.0
;    if (j eq 0) or (j eq nomega-1) then weight[i,j] *= 1.0 else $
;      if ((j mod 2) eq 1) then weight[i,j] *= 4.0 else weight[i,j] *= 2.0
;  endfor
;endfor
;weight /= 9.0

;weight = dblarr(nr,nomega) + 1.0 ;weights for Simpson's rule in 1D --> apply only to r integral
;for i = 0, nr-1 do begin
;  if (i eq 0) or (i eq nr-1) then weight[i,*] *= 1.0 else $
;    if ((i mod 2) eq 1) then weight[i,*] *= 4.0 else weight[i,*] *= 2.0
;endfor
;weight /= 3.0

weight = dblarr(nr,nomega) + 1.0 ;unit weights --> best agreement with Libabsco?

; sample points
dr_s = (r2s-r1s)/double(nr)
r_s = r1s + findgen(nr) * dr_s + 0.5 * dr_s
xs = dblarr(nr,nomega) & ys = dblarr(nr,nomega) & rs = dblarr(nr,nomega)
for i = 0, nr-1 do begin
  for j = 0, nomega-1 do begin
    xs[i,j] = r_s[i] * comega[j]
    ys[i,j] = r_s[i] * somega[j]
    rs[i,j] = r_s[i]
  endfor
endfor

; points in inner container
if n_elements(r1ci) gt 0 then begin
  dr_ci = (r2ci-r1ci)/float(nr)
  r_ci = r1ci + findgen(nr) * dr_ci + 0.5 * dr_ci
  xci = fltarr(nr,nomega) & yci = fltarr(nr,nomega) & rci = fltarr(nr,nomega)
  for i = 0, nr-1 do begin
    for j = 0, nomega-1 do begin
      xci[i,j] = r_ci[i] * comega[j]
      yci[i,j] = r_ci[i] * somega[j]
      rci[i,j] = r_ci[i]
    endfor
  endfor
endif  

; points in external container
if n_elements(r1co) gt 0 then begin
  dr_co = (r2co-r1co)/float(nr)
  r_co = r1co + findgen(nr) * dr_co + 0.5 * dr_co
  xco = fltarr(nr,nomega) & yco = fltarr(nr,nomega) & rco = fltarr(nr,nomega)
  for i = 0, nr-1 do begin
    for j = 0, nomega-1 do begin
      xco[i,j] = r_co[i] * comega[j]
      yco[i,j] = r_co[i] * somega[j]
      rco[i,j] = r_co[i]
    endfor
  endfor
endif

; ------------------------------------------------------------------------------
; Transmission of sample and container

npTrans = 1000
max_diameter = diameterOutSample + 2.0 * cellOutThickness
dx = max_diameter / float(npTrans)
;pvector = findgen(npTrans+1, start=-npTrans/2)*dx  ;but start keyword only valid from IDL 8.2
pvector = findgen(npTrans+1)*dx - (npTrans/2)*dx  

for ipoint = 0,n_elements(pvector)-1 do begin
 
  ; calculate intersection points with each region
  ; dism_in[nann regions] = value of X of enter point into region
  ; disp_in[nann regions] = value of X of leaving point out of region
  ; nint = number of intersections
  insect, nann, radius, 0.d, pvector[ipoint], 1.d, 0.d, dism_in, disp_in, nint

  ; calculate contribution to transmission for this value of y (only container --> sigma0_noS)
  ; path[k] = path of transmitted beam in each region
  ; attenh[k] = attenuation in region when going through half of the path
  
  cumulPath = 0.d
  for k = 0,nann-1 do begin
    path[k] = disp_in[k] - dism_in[k] - cumulPath
    cumulPath += path[k]
    attenh[k] = exp(-sigma0_noS[k]*path[k]/2.d)
  endfor

  ; transmission of container
  prob_thru = 1.d0
  ; attenuation in first half of sample (X < 0, from external to internal shell)
  for k = nann-1,0,-1 do begin
    prob_thru *= attenh[k]
  endfor
  ; attenuation in second half of sample (X > 0, from internal to external shell)
  for k=0,nann-1 do begin
    prob_thru *= attenh[k]
  endfor
  transmission_c += prob_thru

  ; calculate contribution to transmission for this value of y (sample + container --> sigma0)
  for k = 0,nann-1 do begin
    attenh[k] = exp(-sigma0[k]*path[k]/2.d)
  endfor

  ; transmission of container + sample
  prob_thru = 1.d0
  ; attenuation in first half of sample (X < 0, from external to internal shell)
  for k = nann-1,0,-1 do begin
    prob_thru *= attenh[k]
  endfor
  ; attenuation in second half of sample (X > 0, from internal to external shell)
  for k=0,nann-1 do begin
    prob_thru *= attenh[k]
  endfor
  transmission_s += prob_thru

endfor

s1 = total(radius*sigma0_noS)
ssg = shift(sigma0_noS,-1)
s2 = total(radius*ssg)
s3 = (radius[nann-1])*ssg[nann-1]
arg_out = s1-s2+s3
transd = exp(-2.0*arg_out)
transmission_c /= float(n_elements(pvector))
print, 'Transmission Container (average)  = ', transmission_c
print, 'Transmission Container (diameter) = ', transd
s1 = total(radius*sigma0)
ssg = shift(sigma0,-1)
s2 = total(radius*ssg)
s3 = (radius[nann-1])*ssg[nann-1]
arg_out = s1-s2+s3
transd = exp(-2.0*arg_out)
transmission_s /= float(n_elements(pvector))
print, 'Transmission Sample + Container (average)  = ', transmission_s
print, 'Transmission Sample + Container (diameter) = ', transd

; ------------------------------------------------------------------------------
; Points in inner container --> Acc + Acsc
 
for ipoint = 0,n_elements(xci)-1 do begin

  ;path of incident beam in each region before scattering
  insect, nann, radius, xci[ipoint], yci[ipoint], 1.d, 0.d, dism_in, disp_in, nint
  cumulPath = 0.d
  for k = 0, nann-1 do begin
    if dism_in[k] gt 0 then path[k] = 0.0 else path[k] = (disp_in[k]<0) - dism_in[k] - cumulPath
    cumulPath += path[k]
  endfor

  ;for each scattering angle calculate intersection points with each region.
  ninsect, nann, radius[0:nann-1], xci[ipoint], yci[ipoint], nphi, cphi, sphi, dism_out, disp_out, nint

  ;sigma*path for incident beam (for Acc --> no sample, using sigma0_noS)
  arg_in = 0.d
  for k = 0, nann-1 do begin
    arg_in += sigma0_noS[k] * path[k]
  endfor

  ;get attenuation factor from out path through all regions  (Acc --> sigmaf_noS)
  if n_elements(w) eq 0 then nout_calc, nann, dism_out, disp_out, sigmaf_noS, nphi, arg_out $
    else nout_calc_nw, nann, dism_out, disp_out, sigmaf_noS, nphi, arg_out

  acci += weight[ipoint] * rci[ipoint] * exp(-arg_in) * exp(-arg_out) * dr_ci * domega

  ;sigma*path for incident beam (for Acsc --> with sample, using sigma0 and sigmaf)
  arg_in = 0.d
  for k = 0, nann-1 do begin
    arg_in += sigma0[k] * path[k]
  endfor

  if n_elements(w) eq 0 then nout_calc, nann, dism_out, disp_out, sigmaf, nphi, arg_out $
    else nout_calc_nw, nann, dism_out, disp_out, sigmaf, nphi, arg_out

  acsci += weight[ipoint] * rci[ipoint] * exp(-arg_in) * exp(-arg_out) * dr_ci * domega

endfor

; ------------------------------------------------------------------------------
; Points in outer container --> Acc + Acsc

for ipoint = 0,n_elements(xco)-1 do begin

  ;path of incident beam in each region before scattering
  insect, nann, radius, xco[ipoint], yco[ipoint], 1.d, 0.d, dism_in, disp_in, nint
  cumulPath = 0.d
  for k = 0, nann-1 do begin
    if dism_in[k] gt 0 then path[k] = 0.0 else path[k] = (disp_in[k]<0) - dism_in[k] - cumulPath
    cumulPath += path[k]
  endfor

  ;for each scattering angle calculate intersection points with each region.
  ninsect, nann, radius[0:nann-1], xco[ipoint], yco[ipoint], nphi, cphi, sphi, dism_out, disp_out, nint

  ;sigma*path for incident beam (for Acc --> no sample, using sigma0_noS)
  arg_in = 0.d
  for k = 0, nann-1 do begin
    arg_in += sigma0_noS[k] * path[k]
  endfor

  ;get attenuation factor from out path through all regions  (Acc --> sigmaf_noS)
  if n_elements(w) eq 0 then nout_calc, nann, dism_out, disp_out, sigmaf_noS, nphi, arg_out $
  else nout_calc_nw, nann, dism_out, disp_out, sigmaf_noS, nphi, arg_out

  acco += weight[ipoint] * rco[ipoint] * exp(-arg_in) * exp(-arg_out) * dr_co * domega

  ;sigma*path for incident beam (for Acsc --> with sample, using sigma0 and sigmaf)
  arg_in = 0.d
  for k = 0, nann-1 do begin
    arg_in += sigma0[k] * path[k]
  endfor

  if n_elements(w) eq 0 then nout_calc, nann, dism_out, disp_out, sigmaf, nphi, arg_out $
  else nout_calc_nw, nann, dism_out, disp_out, sigmaf, nphi, arg_out

  acsco += weight[ipoint] * rco[ipoint] * exp(-arg_in) * exp(-arg_out) * dr_co * domega

endfor

; ------------------------------------------------------------------------------
; Points in sample --> Assc

for ipoint = 0,n_elements(xs)-1 do begin

  ;path of incident beam in each region before scattering
  insect, nann, radius, xs[ipoint], ys[ipoint], 1.d, 0.d, dism_in, disp_in, nint
  cumulPath = 0.d
  for k = 0, nann-1 do begin
    if dism_in[k] gt 0 then path[k] = 0.0 else path[k] = (disp_in[k]<0) - dism_in[k] - cumulPath
    cumulPath += path[k]
  endfor

  ;for each scattering angle calculate intersection points with each region.
  ninsect, nann, radius[0:nann-1], xs[ipoint], ys[ipoint], nphi, cphi, sphi, dism_out, disp_out, nint

  ;sigma*path for incident beam
  arg_in = 0.d
  for k = 0, nann-1 do begin
    arg_in += sigma0[k] * path[k]
  endfor

  ;get attenuation factor from out path through all regions
  if n_elements(w) eq 0 then nout_calc, nann, dism_out, disp_out, sigmaf, nphi, arg_out $
    else nout_calc_nw, nann, dism_out, disp_out, sigmaf, nphi, arg_out
  
  assc += weight[ipoint] * rs[ipoint] * exp(-arg_in) * exp(-arg_out) * dr_s * domega

endfor


;Divide attenuation factors by area
assc /= !pi * (r2s^2 - r1s^2)
if n_elements(r1ci) gt 0 then begin
  if n_elements(r1co) gt 0 then begin
    acc  = (acci + acco)   / (!pi * (r2ci^2 - r1ci^2 + r2co^2 - r1co^2)) ;inner + outer container
    acsc = (acsci + acsco) / (!pi * (r2ci^2 - r1ci^2 + r2co^2 - r1co^2))
  endif else begin
    acc  = acci  / (!pi * (r2ci^2 - r1ci^2)) ;only inner container
    acsc = acsci / (!pi * (r2ci^2 - r1ci^2))
  endelse
endif else begin
  if n_elements(r1co) gt 0 then begin
    acc  = acco  / (!pi * (r2co^2 - r1co^2)) ;only outer container
    acsc = acsco / (!pi * (r2co^2 - r1co^2))
  endif  
endelse

; Sample attenuation factor, SAF = Acsc/Acc, and
; self-shielding factor, SSF = Assc

if total(acc) gt 0 then saf = acsc/acc else saf = acsc
ssf = assc

if n_elements(w) gt 0 then begin ;energies given --> 3D workspace (even if only SSF computed)
  w_out[*,*,0] = ssf
  w_out[*,*,1] = saf
  w_out[*,*,2] = assc
  w_out[*,*,3] = acsc
  w_out[*,*,4] = acc
  x_out = angles & x_tit = 'Scattering angle'
  y_out = w & y_tit = 'Energy'
  z_out = indgen(5)+1 & z_tit = 'SSF, SAF, ASSC, ACSC, ACC'
  w_tit = 'Attenuation correction factors [2theta, w]'
  other_tit = 'T_S = ' + strtrim(string(transmission_s/transmission_c),2) + $
              '  T_C = ' + strtrim(string(transmission_c),2) + $
              '  T = ' + strtrim(string(transmission_s),2)
endif else begin
  if n_elements(xci) eq 0 and n_elements(xco) eq 0 then begin ;no background --> 1D wks (only SSF)
    w_out = ssf
    x_out = angles & x_tit = 'Scattering angle'
    y_out = '' & y_tit = ''
    z_out = '' & z_tit = ''
    w_tit = 'Self-shielding factor [2theta]'
    other_tit = 'T_S = ' + strtrim(string(transmission_s),2)
  endif else begin; 2D wks (SSF, SAF, A_S,SC, A_C,SC, A_C,C)
    w_out[*,0] = ssf
    w_out[*,1] = saf
    w_out[*,2] = assc
    w_out[*,3] = acsc
    w_out[*,4] = acc
    x_out = angles & x_tit = 'Scattering angle'
    y_out = indgen(5)+1 & y_tit = 'SSF, SAF, ASSC, ACSC, ACC'
    z_out = '' & z_tit = ''
    w_tit = 'Attenuation correction factors [2theta]'
  other_tit = 'T_S = ' + strtrim(string(transmission_s/transmission_c),2) + $
              '  T_C = ' + strtrim(string(transmission_c),2) + $
              '  T = ' + strtrim(string(transmission_s),2)
  endelse
endelse

datp = {X: x_out, Y: y_out, Z: z_out, X_TIT: x_tit, Y_TIT: y_tit, Z_TIT: z_tit, W_TIT: w_tit, OTHER_TIT: other_tit}
give_datp, datp
return, w_out

end





