;+
;
; :Description:
;   Function to compute the attenuation factors for an infinite slab.
;   This function will return a vector, a 2D array or a 3D array (depending on
;   the input parameters) with the values of SSF, SAF, Assc, Acsc, and Acc (see
;   below for explanations) as a function of the scattering angles (and the energy
;   transfer, if given).
;
; :Params:
;    angles : in, required, type="float array" - 1D array with values of 2-theta (scattering angles)
;    orient : in, required, type="float" - angle (in degrees) between the slab and the incident beam
;             (should be a value between 0 and 180 degrees, 90 corresponding to the slab perpendicular to the beam)
;    sampleThickness : in, required, type="float" - Sample thickness 
;                      (in units consistent with those employed for the cross sections, usually cm)
;    sampleSigmaS : in, required, type="float" - Scattering cross section of the sample
;                   (in units consistent with those employed for the sample thickness, usually cm^-1)
;    sampleSigmaA : in, required, type="float" - Absorption cross section of the sample
;                   (in units consistent with those employed for the sample thickness, usually cm^-1)
;    w  : in, optional, type="float array" - 1D array with values containing energy transfers
;         (w > 0 corresponds to neutron energy gains)
;    E0 : in, optional, type="float" - Incident neutron energy   
;         (if not given, set to 25.3 meV, corresponding to a wavelength = 1.798 angstroms)  
;    cellWallThickness : in, optional, type="float" - Thickness of a single container wall
;                       (in units consistent with those employed for the cross sections, usually cm)
;    cellSigmaS : in, optional, type="float" - Scattering cross section of the container
;                 (in units consistent with those employed for the sample thickness, usually cm^-1)
;    cellSigmaA : in, optional, type="float" - Absorption cross section of the container
;                 (in units consistent with those employed for the sample thickness, usually cm^-1)
;                                            
; :Info:
;
;   Evaluation of correction coefficients for transmission in the case of a
;   flat sample. Both absorption and self shielding are taken into account.
;   Calculations are made according to reference [1] (chapter 4) and following
;   the fortran implementation in [2]. However some equations have been changed,
;   as there are some errors in the expressions given in [1] for the reflection
;   case. The factors obtained have been checked by comparing with the results
;   obtained using the Self-shielding calculator for slab geometry in DAVE [3] 
;   and the libabsco library [4].
;
;   Is:   theoretical scattering from sample, without absorption
;   Ic:   theoretical scattering from container, without absorption
;   Iscsc:  scattering from sample plus container (i.e. measured scattering)
;   Icc:  scattering from container when alone (i.e. measured)
;   Issc: scattering from sample itself when inside container
;   Icsc: scattering from can itself when sample inside
;
;   measured scattering Iscsc = Issc + Icsc
;
;   Issc  = Assc*Is , where Assc is the self-shielding of scattering by the sample (in sample measurement)
;   Icsc  = Acsc*Ic , where Acsc is the self-shielding of scattering by the container (in sample measurement)
;   Icc   = Acc*Ic  , where Acc is the self-shielding of scattering by the container (in empty cell measurement)
;
;   Is = (1/Assc) * [Iscsc - (Acsc/Acc)*Icc ]
;
;   It is possible to define the sample attenuation factor, SAF = Acsc/Acc, and
;   the self-shielding factor, SSF = Assc, so
;
;   Is = (1/SSF) * (Iscsc - SAF*Icc)
;
; [1] "Quasielastic neutron scattering", M. Bee, Adam Hilger (1988)
; [2] Elascan, subroutine elsc_slab
; [3] DAVE (http://www.ncnr.nist.gov/dave)
; [4] Libabsco (http://apps.jcns.fz-juelich.de/doku/sc/absco)
;
; :Command syntax:
;
;   This function admits different calls, and the operation done and the dimensions
;   of the workspace returned will depend on the parameters passed to the function.
;
;   w10 = calc_abscor_flat(x1, 135.0, 0.02, 5.6, 0.2) ,
;   ...
;
; :Author: M. A. Gonzalez
;
; :History:
;    M. A. Gonzalez, February 2008 (for elascan routines)
;    M. Boulanger, July 2014
;    M. A. Gonzalez, 3 April 2015
;
;-


function calc_abscor_flat, angles, orient, sampleThickness, sampleSigmaS, sampleSigmaA, $
                           w = w, E0 = E0, cellWallThickness = cellWallThickness,  $
                           cellSigmaS = cellSigmaS, cellSigmaA = cellSigmaA

lamp_journal, 'Calculation of absorption corrections for a slab:'

;-------------------------------------------------------------------------------
; Check if input is correct

if n_elements(orient) eq 0 then begin
  print, 'Sample orientation missing!'
  lamp_journal, '  Sample orientation missing!'
  return, 0
endif else begin
  if orient lt 0.0 or orient ge 180.0 then begin
    print, 'Sample orientation expected to be an angle in degrees between 0 and 180!'
    print, 'Current value is ', orient
    lamp_journal, '  Sample orientation expected to be an angle in degrees between 0 and 180!'
    lamp_journal, '  Current value is ' + string(orient)
    tmp = orient mod 180
    if orient lt 0.0 then begin
      if tmp eq 0 then orient = orient + 180.0 * fix(abs(orient)/180) else $
        orient = orient + 180.0 * (1 + fix(abs(orient)/180))
    endif else orient = tmp
    print, 'Continue using orient = ', orient
    lamp_journal, '  Continue using orient =  ' + string(orient)
  endif
endelse

if n_elements(sampleThickness) eq 0 then begin
  print, 'Sample thickness missing!'
  lamp_journal, '  Sample thickness missing!'
  return, 0
endif else begin
  if sampleThickness lt 0 then begin
    print, 'Sample thickness < 0!'
    lamp_journal, '  Sample thickness < 0!'
    return, 0  
  endif
endelse

if n_elements(sampleSigmaS) eq 0 then begin
  print, 'Sample scattering cross section missing!'
  lamp_journal, '  Sample scattering cross section missing!'
  return, 0
endif else begin
  if sampleSigmaS lt 0 then begin
    print, 'Sample scattering cross section < 0!'
    lamp_journal, '  Sample scattering cross section < 0!'
    return, 0
  endif
endelse

if n_elements(sampleSigmaA) eq 0 then begin
  print, 'Sample absorption cross section missing!'
  lamp_journal, '  Sample absorption cross section missing!'
  return, 0
endif else begin
  if sampleSigmaA lt 0 then begin
    print, 'Sample absorption cross section < 0!'
    lamp_journal, '  Sample absorption cross section < 0!'
    return, 0
  endif
endelse

;If incident energy not given, assume E0 = 25.3 meV, i.e. lambda = 1.798 A
E0ref = 81.8049/1.798^2  
if n_elements(E0) eq 0 then E0 = E0ref

;No container
if n_elements(cellWallThickness) eq 0 then cellWallThickness = 0.0
if n_elements(cellSigmaS) eq 0 then cellSigmaS = 0.0
if n_elements(cellSigmaA)eq 0 then cellSigmaA = 0.0

;-------------------------------------------------------------------------------
; Determine type of output (1D, 2D, 3D)

if n_elements(w) gt 0 then begin ;energies given --> 3D workspace (even if only SSF computed)
   w_out = fltarr(n_elements(angles), n_elements(w), 5)
endif else begin
   if cellWallThickness eq 0 then begin ;no background --> 1D wks (only SSF)
       w_out = fltarr(n_elements(angles))
   endif else begin; 2D wks (SSF, SAF, A_S,SC, A_C,SC, A_C,C)
       w_out = fltarr(n_elements(angles), 5)
   endelse
endelse

;-------------------------------------------------------------------------------
; Compute scattering and absorption cross sections for sample and container
; Create the attenuation coefficients matrices
  
  if n_elements(w) gt 0 then Ef = E0 - w else Ef = E0
  
  sigma0_s = sampleSigmaS + sampleSigmaA * sqrt(E0ref/E0)
  sigmaf_s = sampleSigmaS + sampleSigmaA * sqrt(E0ref/Ef)
  sigma0_c = cellSigmaS + cellSigmaA * sqrt(E0ref/E0)
  sigmaf_c = cellSigmaS + cellSigmaA * sqrt(E0ref/Ef)


;-------------------------------------------------------------------------------
; Angle from the sample normal to the incident and output neutron wavevectors

phi0 = 90.0 - orient
phi = angles + phi0 ;vector containing the angle between the sample normal and each of the detectors
;phi between 0 and 360 degrees, necessary to choose between transmission / reflection
idx = where(phi lt 0.0 or phi ge 360.0, count)
if count gt 0 then begin
  tmp = phi[idx] mod 360
  for i = 0, n_elements(tmp)-1 do begin
    if phi[idx[i]] lt 0.0 then begin
      if tmp[i] eq 0 then phi[idx[i]] += 360.0 * fix(abs(phi[idx[i]])/360) else $
        phi[idx[i]] += 360.0 * (1 + fix(abs(phi[idx[i]])/360))
    endif else phi[idx[i]] = tmp[i]
  endfor
endif

;-------------------------------------------------------------------------------
; Define a0c, ac, a0s and as (eqs. A.4.7 and A.4.9 in [1])

a0c = (sigma0_c * cellWallThickness) / (2.0 * cos(phi0*!dtor))
a0s = (sigma0_s * sampleThickness) / (2.0 * cos(phi0*!dtor))

; Transmissions - Path of incident beam = thickness / cos(phi0)
t_s = exp(-2.0*a0s)
t_c = exp(-4.0*a0c)  ;two container walls
print, 'Sample transmission    = ', t_s
print, 'Container transmission = ', t_c
print, 'Total transmission     = ', t_s * t_c
lamp_journal, '  Sample transmission    = ' + string(t_s)
lamp_journal, '  Container transmission = ' + string(t_c)
lamp_journal, '  Total transmission     = ' + string(t_s * t_c)


;Avoid divergences by moving angles corresponding to 90 and 270 degrees
idx = where( (phi eq 90.0) or (phi eq 270.0), count)
if count gt 0 then phi[idx] -= 0.1

;ac  = (sigmaf_c * cellWallThickness) / (2.0 * cos(phi*!dtor))
;as  = (sigmaf_s * sampleThickness) / (2.0 * cos(phi*!dtor))
tmp1 = 1.0 / (2.0 * cos(phi*!dtor))
tmp2 = sigmaf_c * cellWallThickness 
tmp3 = sigmaf_s * sampleThickness 
ac  = (tmp1 # tmp2) > (-60) < 60 ;alpha_c [phi,omega] - use minimum = -60 to avoid under-/over-flows and Inf numbers
as  = (tmp1 # tmp3) > (-60) < 60 ;alpha_s [phi,omega]

; u(x) = [exp(x)-exp(-x)]/2x
; uaa0c: u(ac-a0c) used in defining H1(a0c,ac)
; u2a0c: u(2ac-2a0c) used in defining H1(2a0c,2ac)
; uaa0s: u(as-a0s) used in defining H1(a0s,as)

; u(x) > 1 for all (ac-a0c) values and = 1 when ac=a0c (indeterminate 0/0 --> Floating illegal operand message)
uaa0c = (exp(ac-a0c) - exp(-ac+a0c)) / (2.0*(ac-a0c)) > 1                          
u2a0c = (exp(2.0*ac-2.0*a0c) - exp(-2.0*ac+2.0*a0c) ) / (2.0*(2.0*ac-2.0*a0c)) > 1 
uaa0s = (exp(as-a0s) - exp(-as+a0s)) / (2.0*(as-a0s)) > 1                          

; H1(a0s,as) = exp(-|a0s|-|as|) * u(as-a0s)
; ha0ac: H1(a0c,ac) used in 4.A.13 and 4.A.15 in [1]
; h2a02ac: H1(2a0c,2ac) defined in 4.A.6 in [1]
; ha0as: H1(a0s,as) defined in 4.A.8 and used in 4.A.10 and 4.A. 14 in [1]

ha0ac = exp(-abs(ac)-abs(a0c)) * uaa0c
h2a02ac = exp(-2.0*abs(ac)-2.0*abs(a0c)) * u2a0c
ha0as = exp(-abs(as)-abs(a0s)) * uaa0s

if n_elements(w) gt 0 then acc = fltarr(n_elements(phi),n_elements(w))else acc = fltarr(n_elements(phi))
if n_elements(w) gt 0 then acsc = fltarr(n_elements(phi),n_elements(w))else acsc = fltarr(n_elements(phi))
if n_elements(w) gt 0 then assc = fltarr(n_elements(phi),n_elements(w))else assc = fltarr(n_elements(phi))

; Transmission geometry
; acc = A_c,c given by eq. 4.A.6 in [1]
;       equivalent to A_c,c = 0.5 * ha0ac * (exp(-2.0*(abs(as)) + exp(-2.0*abs(a0c)))
; acsc = A_c,s+c given by eq. 4.A.13 in [1], but doing the sum of absolute values
;       (exp(-2.0*(abs(as)+abs(ac))) + exp(-2.0*(abs(a0s)+abs(a0c)))) instead of
;       (exp(-2.0*abs(as+ac)) + exp(-2.0*abs(a0s+a0c))). But they give the same
;       result.
; assc = A_s,s+c given by eq. 4.A.10 in [1]

trans = where( (phi lt 90.0) or (phi ge 270), count, complement = refl, ncomplement = nrefl )
if count gt 0 then begin
  if n_elements(w) gt 0 then begin
    acc[trans,*] = h2a02ac[trans,*]
    acsc[trans,*] = 0.5 * ha0ac[trans,*] * (exp(-2.0*(abs(as[trans,*])+abs(ac[trans,*]))) + $
                                            exp(-2.0*(abs(a0s)+abs(a0c))))
    assc[trans,*] = exp(-2.0*abs(a0c)) * ha0as[trans,*] * exp(-2.0*abs(ac[trans,*]))
    
  endif else begin
    acc[trans] = h2a02ac[trans]
    acsc[trans] = 0.5 * ha0ac[trans] * (exp(-2.0*(abs(as[trans])+abs(ac[trans]))) + $
                                        exp(-2.0*(abs(a0s)+abs(a0c))))
    assc[trans] = exp(-2.0*abs(a0c)) * ha0as[trans] * exp(-2.0*abs(ac[trans]))
  endelse 
endif

; Reflection geometry
; acc = A_c,c given by eq. 4.A.15 in [1], but note that there are several mistakes.
;       A factor of 0.5 missing and the exponent should be exp(-2.0*(abs(a0c)+abs(ac))).
;       The 1/2 factor is correctly added in the fortran code [2], but not the other correction.
; acsc = A_c,s+c given by eq. 4.A.16 in [1], but note that there are several mistakes.
;        A factor of 0.5 missing and the exponent should be exp(-2.0*(abs(a0c)+abs(a0s)+abs(as)+abs(ac))).
;       The 1/2 factor is correctly added in the fortran code [2], but then it considers that
;       abs(a0c+ac) = abs(a0c)+abs(ac) and that abs(a0s+as) = abs(a0s)+abs(as), which is wrong
; assc = A_s,s+c given by eq. 4.A.14 = 4.A.10 in [1]

if nrefl gt 0 then begin
  if n_elements(w) gt 0 then begin
    acc[refl,*] = 0.5 * ha0ac[refl,*] * (1.0 + exp(-2.0*(abs(a0c)+abs(ac[refl,*]))))
    acsc[refl,*] = 0.5 * ha0ac[refl,*] * (1.0 + exp(-2.0*(abs(a0c)+abs(a0s)+abs(as[refl,*])+abs(ac[refl,*]))))
    assc[refl,*] = exp(-2.0*abs(a0c)) * ha0as[refl,*] * exp(-2.0*abs(ac[refl,*]))
  endif else begin
    acc[refl] = 0.5 * ha0ac[refl] * (1.0 + exp(-2.0*(abs(a0c)+abs(ac[refl]))))
    acsc[refl] = 0.5 * ha0ac[refl] * (1.0 + exp(-2.0*(abs(a0c)+abs(a0s)+abs(as[refl])+abs(ac[refl]))))
    assc[refl] = exp(-2.0*abs(a0c)) * ha0as[refl] * exp(-2.0*abs(ac[refl]))
  endelse 
endif

; Sample attenuation factor, SAF = Acsc/Acc, and
; self-shielding factor, SSF = Assc

saf = acsc / acc
ssf = assc

if n_elements(w) gt 0 then begin ;energies given --> 3D workspace (even if only SSF computed)
  w_out[*,*,0] = ssf
  w_out[*,*,1] = saf
  w_out[*,*,2] = assc
  w_out[*,*,3] = acsc
  w_out[*,*,4] = acc
  x_out = angles & x_tit = 'Scattering angle'
  y_out = w & y_tit = 'Energy'
  z_out = indgen(5)+1 & z_tit = 'SSF, SAF, ASSC, ACSC, ACC'
  w_tit = 'Attenuation correction factors [2theta, w]'
  other_tit = 'T_S = ' + strtrim(string(t_s),2) + '  T_C = ' + strtrim(string(t_c),2) + $
    '  T = ' + strtrim(string(t_s*t_c),2)
endif else begin
  if cellWallThickness eq 0 then begin ;no background --> 1D wks (only SSF)
    w_out = ssf
    x_out = angles & x_tit = 'Scattering angle'
    y_out = '' & y_tit = ''
    z_out = '' & z_tit = ''
    w_tit = 'Self-shielding factor [2theta]'
    other_tit = 'T_S = ' + strtrim(string(t_s),2)
  endif else begin; 2D wks (SSF, SAF, A_S,SC, A_C,SC, A_C,C)
    w_out[*,0] = ssf
    w_out[*,1] = saf
    w_out[*,2] = assc
    w_out[*,3] = acsc
    w_out[*,4] = acc
    x_out = angles & x_tit = 'Scattering angle'
    y_out = indgen(5)+1 & y_tit = 'SSF, SAF, ASSC, ACSC, ACC'
    z_out = '' & z_tit = ''
    w_tit = 'Attenuation correction factors [2theta]'
    other_tit = 'T_S = ' + strtrim(string(t_s),2) + '  T_C = ' + strtrim(string(t_c),2) + $
      '  T = ' + strtrim(string(t_s*t_c),2)
  endelse
endelse

datp = {X: x_out, Y: y_out, Z: z_out, X_TIT: x_tit, Y_TIT: y_tit, Z_TIT: z_tit, W_TIT: w_tit, OTHER_TIT: other_tit}
give_datp, datp
return, w_out

end

