FUNCTION elastic, w_in, vanadium = v_in, min=xmin, max=xmax, $
                  save=save, align=align, read=read
;+
;******* *******
;** Use callable
;** For IN4, IN5, IN6 and D7 data.
;** 
;** Finds the elastic peak position in each spectrum by fitting a Gaussian
;** and a flat background. Looks for peak in region xmin < x < xmax.
;** If /align keyword is used, then the elastic lines are aligned to the mean
;** elastic peak position.  If a vanadium workspace is provided, then the
;** elastic lines are set to the mean vanadium elastic peak position.  Normalise
;** should have been previously run on sample and vanadium input data.
;** 
;** ARGUMENTS:
;**     min:    minimum x-value for peak search
;**     max:    maximum x-value for peak search
;**     vanadium:    use an input vanadium workspace for elastic lineup (in conjunction
;**                  with /align)
;** 
;** KEYWORDS:
;**      /save     : Write out results to file 'elastic.dat'
;**      /read     : read in positions from file 'elastic.dat'
;**      /align    : Lineup elastic positions
;** 
;** DIMENSIONS:
;** w_in(nchannels,nspectra) OR w_in(nchannels)
;** 
;** COMMAND SYNTAX:
;** w3=elastic(w2[, min = <min>][, max = <max>][, /save][, /align][, vanadium = w10])
;** 
;** (optional arguments and keywords shown in square brackets)
;** 
;**                             KHA,JRS 28/11/06
;-
        
    COMMON c_lamp_access, inst
    
    iprint = 0    ; if iprint > 0, show debugging messages
    
    IF (iprint GT 0) THEN PRINT,'Start elastic:'
    
    TAKE_DATP, datp
    
    s = STRPOS(datp.other_tit, 'no')
    IF s EQ -1 THEN BEGIN
        s = 'Elastic: Please normalise data before running Elastic'
        ii = dialog_message(s, /ERROR)
        RETURN, w_in
    ENDIF
    
    w_oper = w_in
    dat = datp
    ; ------------------------------------------------------------------------------
    ;Set up input workspace
    
    IF N_ELEMENTS(v_in) GT 2 THEN BEGIN
        TAKE_DATP, vdatp, /third
        s = STRPOS(vdatp.other_tit, 'no')
        IF s EQ -1 THEN BEGIN
            s = 'Elastic: Please normalise vanadium data before running Elastic'
            ii = dialog_message(s, /ERROR)
            RETURN, w_in
        ENDIF
        IF NOT KEYWORD_SET(align) THEN BEGIN
            s = STRARR(2)
            s[0] = 'Elastic:  Vanadium data only used for alignment'
            s[1] = '          Please set /lineup keyword to enable this'
            ii = dialog_message(s, /ERROR)
            RETURN, w_in
        ENDIF
        w_oper = v_in
        dat    = vdatp
    ENDIF
    
    IF N_ELEMENTS(xmin) EQ 0 THEN BEGIN
        xmin = dat.p[9] - 30.
        xmax = dat.p[9] + 30.
    ENDIF
    
    sw        = SIZE(w_oper)
    IF sw[0] EQ 3 THEN nruns = sw[3] ELSE nruns = 1
    nchannels = sw[1]
    IF sw[0] EQ 1 THEN nspectra = 1     ELSE $
        IF sw[0] GE 2 THEN nspectra = sw[2] ELSE RETURN,0
        
    IF iprint THEN PRINT,'nchannels=',nchannels,' nspectra=',nspectra, ' nruns=',nruns
    phi  = dat.y
    x_oper = dat.x
    e_oper = dat.e
    
    IF iprint THEN PRINT,'End of setup section'
    
    ; ------------------------------------------------------------------------------------
    ;    Fit Gaussians or read elastic positions from file
    
    IF NOT KEYWORD_SET(read) THEN BEGIN
        x_out = phi
        w_out = FLTARR(nspectra)    & e_out=w_out
        
        gauss = FLTARR(4)    & dgauss = gauss
        
        FOR ispec = 0, nspectra - 1 DO BEGIN
            y = w_oper[*,ispec]
            e = e_oper[*,ispec]
            IF iprint THEN PRINT,'Fitting Gaussian to s',ispec
            FITGAUSS, x_oper, y, e, xmin, xmax, gauss, dgauss, ispec
            w_out[ispec] = gauss[2]    & e_out[ispec] = dgauss[2]
        ENDFOR
    ENDIF ELSE BEGIN
        arse = READ_ASCII('elastic.dat')
        phi = REFORM(arse.field1[0,*],(SIZE(arse.field1[0,*]))[2])
        w_out = REFORM(arse.field1[1,*],(SIZE(arse.field1[0,*]))[2])
        e_out = REFORM(arse.field1[2,*],(SIZE(arse.field1[0,*]))[2])
        numin = (SIZE(phi))[1]-3
        phi = phi[0:numin-1]
        w_out = w_out[0:numin-1]
        e_out = e_out[0:numin-1]
        PRINT,'elastic: Elastic channels read from file "elastic.dat"'
    ENDELSE
    IF iprint THEN PRINT,'End of fitting/reading section'
    
    ; ------------------------------------------------------------------------------------
    ;    Save to file
    
    IF KEYWORD_SET(save) THEN BEGIN
        OPENW, lun, 'elastic.dat', /GET_LUN
        FOR i = 0,nspectra - 1 DO PRINTF, lun, phi[i], w_out[i], e_out[i]
        PRINTF, lun, 'Elastic channels'
        PRINTF, lun, 'Scattering Angle'
        PRINTF, lun, 'Elastic Channel'
        FREE_LUN, lun
        PRINT,'elastic: Elastic channels saved to file "elastic.dat"'
    ENDIF
    
    ; ------------------------------------------------------------------------------------
    ;       Lineup section
    
    IF KEYWORD_SET(align) THEN BEGIN
        el_in = w_out
        w_out = FLTARR(nchannels, nspectra, nruns)
        e_out = FLTARR(nchannels, nspectra, nruns)
        nf = 10        ;number of overlap frames either side of spectra
        mean_elastic = datp.p[9]
        w_dev = el_in - mean_elastic
        FOR irun = 0, nruns - 1 DO BEGIN
            w_int = [w_in[(nchannels - nf):nchannels - 1,*,irun],w_in[*,*,irun],w_in[0:nf - 1,*,irun]]
            x_int = INDGEN(nchannels + 2*nf)
            e_int = [datp.e[(nchannels - nf):nchannels - 1,*,irun],datp.e[*,*,irun],datp.e[0:nf - 1,*,irun]]
            w_buf = w_int
            e_buf = e_int
            
            FOR ispec = 0, nspectra - 1 DO BEGIN
                t_arr = x_int + w_dev[ispec]
                w_buf[*,ispec] = INTERPOL(w_int[*,ispec],x_int,t_arr)
                e_buf[*,ispec] = SQRT(INTERPOL(e_int[*,ispec]^2,x_int,t_arr))
            ENDFOR
            w_out[*,*,irun] = w_buf[nf:nf+nchannels - 1,*]
            e_out[*,*,irun] = e_buf[nf:nf+nchannels - 1,*]
        ENDFOR
    ENDIF
    
    ; ------------------------------------------------------------------------------------
    IF NOT KEYWORD_SET(align) THEN BEGIN
        datp.x_tit='Scattering Angle'
        datp.y_tit='Elastic Channel'
        mod_datp, datp, "x", x_out
    ENDIF
    mod_datp, datp, "e", e_out
    
    x1=STRTRIM(STRING(FIX(xmin)),2) & x2=STRTRIM(STRING(FIX(xmax)),2)
    
    IF NOT KEYWORD_SET(lineup) THEN $
        datp.other_tit=datp.other_tit+' -el('+x1+','+x2+')' ELSE $
        datp.other_tit = datp.other_tit + ' -el(/align'
    IF N_ELEMENTS(v_in) GT 2 THEN datp.other_tit = datp.other_tit + '/van)' $
    ELSE datp.other_tit = datp.other_tit + ')'
    
    IF NOT KEYWORD_SET(align) OR NOT KEYWORD_SET(read) THEN PRINT,'elastic: Gaussians fitted from channels '+x1+' to '+x2
    IF KEYWORD_SET(align) THEN BEGIN
        IF N_ELEMENTS(v_in) GT 2 THEN PRINT, 'elastic:  Data lined up with vanadium' ELSE $
            PRINT, 'elastic: Data lined up with mean elastic position'
    ENDIF
    
    give_datp, datp
    
    finished:
    RETURN, w_out
END


