; ---------------------------------------------------------------------------
FUNCTION input_spe, f_name, phx = phx, Ei = Ei
;+
;******* *********
;** User callable
;** Function to input spe files from HOMER. 
;** 
;** ARGUMENTS
;** ----------
;**    phx : name of phx mapping file.  Default is 'mari_resa.phx'
;**    Ei  : Incident energy in meV
;**
;**                        JRS 15/4/06
;-

    TAKE_DATP, datp
    iprint = 0

; ---------------------------------------------------------------------------
;Check input
    IF N_ELEMENTS(phx) LT 1 THEN phx = 'mari_resa.phx'
    IF N_ELEMENTS(Ei)  LT 1 THEN BEGIN
        s  = 'input_spe: Ei must be specified'
        ii = dialog_message(s,/ERROR)
        RETURN, 0
    ENDIF

; ---------------------------------------------------------------------------
; Read in .spe file section

    PRINT, 'Reading run: ', f_name + '.spe'
    line = ''
    OPENR, lun, f_name + '.spe', /GET_LUN
    READF, lun, nspectra, nchannels
    READF, lun, line

    angarr_1 = FLTARR(nspectra)
    enarr    = FLTARR(nchannels)

    lines    = FIX(nspectra/8)
    remain   = FIX(nspectra - (8*lines))
    IF remain EQ 0 THEN BEGIN
        remain = 8
        lines = lines - 1
    ENDIF
    lines_s  = STRTRIM(STRING(lines),2)
    remain_s = STRTRIM(STRING(remain),2)
    format_str = '(' + lines_s + '(8G10.4,/),' + remain_s + 'G10.4)'

    READF, lun, FORMAT = format_str, angarr_1
    READF, lun, line
    
    lines    = FIX(nchannels/8)
    remain   = FIX(nchannels - (8*lines))
    IF remain EQ 0 THEN BEGIN
        remain = 8
        lines = lines - 1
    ENDIF
    lines_s  = STRTRIM(STRING(lines),2)
    remain_s = STRTRIM(STRING(remain),2)
    format_str = '(' + lines_s + '(8G10.4,/),' + remain_s + 'G10.4)'

    READF, lun, FORMAT = format_str, enarr

    IF iprint THEN BEGIN
        PRINT, 'nchannels = ', nchannels
        PRINT, 'nspectra = ', nspectra
    ENDIF    

    w_out = FLTARR(nchannels,nspectra)
    e_out = FLTARR(nchannels,nspectra)
    arse  = FLTARR(nchannels)
    
    IF remain EQ 8 THEN READF, lun, line

    FOR ispec = 0, nspectra - 1 DO BEGIN
        READF, lun, line
        READF, lun, FORMAT = format_str, arse
        w_out[*,ispec] = arse
        READF, lun, line
        READF, lun, FORMAT = format_str, arse
        e_out[*,ispec] = arse
    ENDFOR

    FREE_LUN, lun
    IF iprint THEN PRINT,'input_spe: finished reading .spe file'

; ---------------------------------------------------------------------
; Read in .phx file

    OPENR, lun, phx, /GET_LUN
    READF, lun, nspectra_phx
    IF nspectra_phx NE nspectra THEN BEGIN
        s  = 'input_spe: phx file incompatible with data'
        ii = dialog_message(s, /ERROR)
        RETURN, 0
    ENDIF
    phxarr = FLTARR(7, nspectra)
    READF, lun, phxarr

    FREE_LUN, lun
    angarr = REFORM(phxarr[2,*],nspectra,1)
    L2arr  = REFORM(phxarr[5,*],nspectra,1)
    widarr = REFORM(phxarr[4,*],nspectra,1)

; ----------------------------------------------------------------------
; find gaps and insert into data set

    gap = 0
    gapang = FLTARR(100)
    FOR ispec = 1, nspectra - 1 DO BEGIN
        IF angarr[ispec] GT (angarr[ispec - 1] + widarr[ispec - 1] + 0.1) THEN BEGIN
            IF iprint THEN PRINT, $
            'Gap found between ', angarr[ispec-1], ' and ', angarr[ispec]    
            gap = gap + 1
            gapang[gap - 1] = (angarr[ispec-1] + angarr[ispec]) / 2.
            IF iprint THEN PRINT, 'Gap value = ', gapang[gap-1]
        ENDIF
    ENDFOR 
    gapdata = FLTARR(nchannels,gap)
    angles = [angarr,gapang[0:gap-1]]
    w_out  = [[w_out],[gapdata]]
    e_out  = [[e_out],[gapdata - 1]]
    i = SORT(angles)
    angles = angles[i]
    w_out = w_out[*,i]
    e_out = e_out[*,i]

; ---------------------------------------------------------------------
;remove unreasonable numbers

    i = WHERE(ABS(w_out) GT 1.e10)
    IF N_ELEMENTS(i) LE 0 THEN BEGIN
        w_out[i] = 0.0
        e_out[i] = 0.0
    ENDIF
; ---------------------------------------------------------------------
; Prepare data parameters

    par = FLTARR(31)
    par[2]  = nchannels
    par[21] = SQRT(81.8066/FLOAT(Ei))
    par[27] = MEAN(L2arr)
    par[30] = nspectra + gap
    partxt  = [$
    'Run duration (seconds)            = ',$
    'Counting Time in seconds               = ',$
    'Number of reserved channels            = ',$
    'Counts in M1                           = ',$
    'Counts in M2                           = ',$
    'Counts in M3                           = ',$
    'Total counts                           = ',$
    'Not used                               = ',$
    'Sample angle (deg.)                    = ',$
    'Elastic peak position (channel)        = ',$
    'Numor                                  = ',$
    'Sample temperature (K)                 = ',$
    'Digital Voltmeter Reading              = ',$
    'Repetition period (microsec.)          = ',$
    'Multiplier for repetition period       = ',$
    'Number of detector spectra         = ',$ 
    'Number of monitor spectra          = ',$
    'Not used                           = ',$ 
    'Channel width (microsec.)          = ',$ 
    'Number of channels used            = ',$
    'TOF delay (microsec.)             = ',$             
    'Wavelength (angstroms)            = ',$             
    'Distance CH4 - M1     (meter)        = ',$      
    'Distance CH4 - Sample (meter)        = ',$      
    'Not used                           = ',$    
    'Distance M1  - M2     (meter)        = ',$      
    'Not used                           = ',$    
    'Distance Det - Sample (meter)        = ',$      
    'Contents scaler 1            = ',$                  
    'Contents scaler 2            = ',$                  
    'Number of angles            = ']           
            
; ---------------------------------------------------------------------
; Prepare arrays and output data

    MOD_DATP, datp, "e", e_out
    MOD_DATP, datp, "x", enarr
    MOD_DATP, datp, "y", angles
    MOD_DATP, datp, "p", par
    MOD_DATP, datp, "par_txt", partxt
    
    datp.w_tit = f_name + '.spe'
    datp.x_tit = 'Energy Transfer (meV)'
    datp.y_tit = '2*theta (deg)'
    GIVE_DATP, datp

    RETURN, w_out
    
    END
