;+
;    FUNCTION intersection_points, a1,b1,c1,d1,a2,b2,c2,d2
;
;    Finds intersection points between 2 quadrilaterals
;    Input quadrilateral points must be ordered
;
;    returns an array (axis, npoints)
;    (if no intersection, then returns 0)
;
;    where the intersecting polygon has npoints vertices. 
;    Note that the output polygon points are not ordered
;
;                            JRS 21/10/2009
;-

FUNCTION inside_quad, p, a, b, c, d
;+
;******* ***********
;** Decides whether point p is inside quadrilateral with vertices (a,b,c,d)
;** Uses barycentric coordinates technique
;** (see www.blackpawn.com/texts/pointinpoly)
;-

    tp = DOUBLE(p)

;first triangle test
    ta = DOUBLE(a)
    tb = DOUBLE(b)
    tc = DOUBLE(d)
    v0 = tc - ta
    v1 = tb - ta
    v2 = tp - ta
    dot00 = v0(0)*v0(0) + v0(1)*v0(1)
    dot01 = v0(0)*v1(0) + v0(1)*v1(1)
    dot02 = v0(0)*v2(0) + v0(1)*v2(1)
    dot11 = v1(0)*v1(0) + v1(1)*v1(1)
    dot12 = v1(0)*v2(0) + v1(1)*v2(1)
    invDenom = 1.0D / (dot00*dot11 - dot01*dot01)
    u = (dot11*dot02 - dot01*dot12) * invDenom
    v = (dot00*dot12 - dot01*dot02) * invDenom
    ans1 = (u GE 0) && (v GE 0) && (u + v LE 1.0)

;2nd triangle test
    ta = DOUBLE(c)
    v0 = tc - ta
    v1 = tb - ta
    v2 = tp - ta
    dot00 = v0(0)*v0(0) + v0(1)*v0(1)
    dot01 = v0(0)*v1(0) + v0(1)*v1(1)
    dot02 = v0(0)*v2(0) + v0(1)*v2(1)
    dot11 = v1(0)*v1(0) + v1(1)*v1(1)
    dot12 = v1(0)*v2(0) + v1(1)*v2(1)
    invDenom = 1.0D / (dot00*dot11 - dot01*dot01)
    u = (dot11*dot02 - dot01*dot12) * invDenom
    v = (dot00*dot12 - dot01*dot02) * invDenom
    ans2 = (u GE 0) && (v GE 0) && (u + v LE 1.0)

    RETURN, ans1 || ans2

    END

; -------------------------------------------------------------------------

FUNCTION between, a, b, p
;+
;******* *******
;** checks that point p is between a and b
;-

    ba = [b[0] - a[0], b[1] - a[1]]
    pa = [p[0] - a[0], p[1] - a[1]]

    ba_dot_pa = ba[0]*pa[0] + ba[1]*pa[1]
    ba_dot_ba = ba[0]^2 + ba[1]^2

    RETURN, (ba_dot_pa GT 0) && (ba_dot_pa LT ba_dot_ba)

    END
; -------------------------------------------------------------------------

FUNCTION crossing_lines, a, b, c, d
;+
;******* **************
;** finds crossing points of 2 lines a-b and c-d
;-

    aa = DOUBLE(b[0]) - DOUBLE(a[0])
    bb = DOUBLE(d[0]) - DOUBLE(c[0])
    cc = DOUBLE(b[1]) - DOUBLE(a[1])
    dd = DOUBLE(d[1]) - DOUBLE(c[1])

    det = aa*dd - bb*cc

    IF ABS(det) LT 1.e-8 THEN BEGIN
        RETURN, 0
    ENDIF ELSE BEGIN
        cross_x = (DOUBLE(a[1]) - DOUBLE(c[1]))*aa*bb - DOUBLE(a[0])*bb*cc + DOUBLE(c[0])*aa*dd
        cross_y = (DOUBLE(a[1])*aa - DOUBLE(a[0])*cc)*dd - (DOUBLE(c[1])*bb - DOUBLE(c[0])*dd)*cc
        ps = [cross_x,cross_y] / det
        IF between(DOUBLE(b),DOUBLE(a),ps) AND between(DOUBLE(d),DOUBLE(c),ps) THEN RETURN, ps ELSE RETURN, 0
    ENDELSE

    END 

; ------------------------------------------------------------------------

FUNCTION intersection_points, a1,b1,c1,d1,a2,b2,c2,d2
;+
;******* *******************
;** User callable
;** finds intersection points between 2 quadrilaterals
;-

    intpoints = [0,0]        ; initialise intersection array

    IF inside_quad(a1,a2,b2,c2,d2) THEN intpoints= [[intpoints],[a1]]
    IF inside_quad(b1,a2,b2,c2,d2) THEN intpoints= [[intpoints],[b1]]
    IF inside_quad(c1,a2,b2,c2,d2) THEN intpoints= [[intpoints],[c1]]
    IF inside_quad(d1,a2,b2,c2,d2) THEN intpoints= [[intpoints],[d1]]
    IF inside_quad(a2,a1,b1,c1,d1) THEN intpoints= [[intpoints],[a2]]
    IF inside_quad(b2,a1,b1,c1,d1) THEN intpoints= [[intpoints],[b2]]
    IF inside_quad(c2,a1,b1,c1,d1) THEN intpoints= [[intpoints],[c2]]
    IF inside_quad(d2,a1,b1,c1,d1) THEN intpoints= [[intpoints],[d2]]

    aa = crossing_lines(a1,b1,a2,b2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(a1,b1,b2,c2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(a1,b1,c2,d2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(a1,b1,d2,a2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(b1,c1,a2,b2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(b1,c1,b2,c2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(b1,c1,c2,d2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(b1,c1,d2,a2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(c1,d1,a2,b2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(c1,d1,b2,c2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(c1,d1,c2,d2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(c1,d1,d2,a2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(d1,a1,a2,b2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(d1,a1,b2,c2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(d1,a1,c2,d2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]
    aa = crossing_lines(d1,a1,d2,a2)
    IF N_ELEMENTS(aa) EQ 2 THEN intpoints= [[intpoints],[aa]]

    IF (SIZE(intpoints))[0] EQ 1 THEN BEGIN
        intpoints = 0
        numpoints = 0
    ENDIF ELSE BEGIN
        numpoints = (SIZE(intpoints))[2]
        intpoints = intpoints[*,1:numpoints-1]
    ENDELSE

    RETURN, intpoints
    END

;****************************************************************************
; ---------------------------------------------------------------------------
