;+
;    FUNCTION normalise, w_in, raw=raw, monitor=monitor, time=time,  $
;        detector=detector, alldetectors=alldetectors, ei=ei, $
;        madangles=madangles, zeroshift=zeroshift, nopo = nopo, $
;        inorm, ikeep
;
;For IN4, IN5, IN6, D7, HET, Mibemol, DCS, NEAT, FOCUS and TOFTOF data.
;
;For IN4, IN5, IN6, Mibemol, NEAT, FOCUS and TOFTOF:
;***************************************************
; Normalises raw data to monitor or counting time, depending on value of inorm.
; Finds the position of the elastic peak.
;
;KEYWORDS
; /raw          : no normlisation (error bars calculated)
; /monitor      : normalise data to 1000 monitor1 counts (DEFAULT)
; /time         : normalise data to counting time (will not work for summed data)
;
; (inorm and ikeep are obsolete, kept for backwards compatability)
;
;For HET:
;*******
; Normalises raw data to monitor 1 (must be monitor 1 for ISIS data).  Removes
; empty spectra.  User must input incident energy.
;
;ARGUMENTS
; ei    : incident energy of measurement
;
;For D7:
;******
; Normalises raw data to monitor (default) or counting time or individual
; detector.  Extracts every other spectrum if desired (=> nspectra=32).
; Recalculates detector angles based on a YIG calibration by default.
;
;ARGUMENTS
; detector      : detector number to normalise to
; zeroshift     : angle in degrees of 2theta = 0
;
; (inorm and ikeep are obsolete, kept for backwards compatability)
;
;KEYWORDS
; /raw          : no normlisation (error bars calculated)
; /monitor      : normalise data to 1000 monitor1 counts (DEFAULT)
; /time         : normalise data to counting time
; /alldetectors : supresses the removal of the odd numbered detectors
; /madangles    : supresses the recalculation of the detector angles using the
;       current YIG calibration
;
;DIMENSIONS
;   non-TOF data: w_out(nspectra,nphases,nruns)           -unless nphases is 1
;   TOF data: w_out(nchannels,nspectra*nphases,nruns)
;
;COMMAND SYNTAX
;  w2=normalise(w1[,/raw][,/monitor][,/time][,detector=#][,/alldetectors][,/rawangles][,ei=#])
;
;  (optional keywords/arguments shown in square brackets)
;
; KHA,JRS,PPD 28/05/08
; Modified by SR to incorporate Mibemol, DCS and TOFTOF data
; Last modification 24/03/09 by SR
; Modified by SR to incorporate FOCUS data 06/11/2013
; Altered  detector positions by PD 03/06/09 
; par -> parp (due to a DIFF procedure having the name PAR !!!!)
; FIX -> long (FIX is a short integer !!!)
; Modified by SR to correct from inproper TOFTOF data M1 normalisation 06/02/14
;*******************************************************************************
;-

FUNCTION normalise_tof, w_in, inorm, keepmons
;+
;******* *************
;** normalise for TOF
;-

    COMMON c_lamp_access, inst
    COMMON printing, iprint, outstring
    
    IF iprint THEN PRINT, 'Start normalise_tof:'
    
    TAKE_DATP, datp

    insU=strupcase(inst)
; ------------------------------------------------------------------------------
;Set up start parameters

    sw = SIZE(w_in)
    nchannels = sw[1]
    IF sw[0] EQ 3 THEN nruns = sw[3] ELSE nruns = 1
    IF iprint THEN BEGIN
        PRINT, ' nchannels = ', nchannels
        PRINT, '     nruns = ', nruns
    ENDIF

    parp = datp.p
    parv = datp.pv
    IF nruns EQ 1 THEN parv = parp

    IF (insU EQ 'IN4') THEN BEGIN
        nspectra = parp[15]
        nmon = parp[16]
        mon = w_in[*,nspectra:nspectra+nmon-1,*]
        monarr = datp.n
    ENDIF ELSE BEGIN    
        IF sw[0] EQ 1 THEN nspectra = 1 ELSE $
        IF sw[0] GE 2 THEN nspectra = sw[2]
        mon = datp.n 
        monarr = mon
    ENDELSE
    x_in = datp.x
    y_in = datp.y

    norm = 1
    IF N_ELEMENTS(inorm) GE 1 THEN norm = inorm
    IF iprint THEN PRINT, '      norm = ', norm
    gauss  = FLTARR(4)
    dgauss = FLTARR(4)

    IF iprint THEN PRINT, 'normalise_tof: End of setup data section'
; ------------------------------------------------------------------------------
;Find elastic peak in detectors

    i = WHERE(y_in GT 10. AND y_in LT 120.)
    xtot = x_in
    IF nruns EQ 1 THEN BEGIN
        ytot = TOTAL(w_in[*,i],2)
    ENDIF ELSE BEGIN
        ytot = TOTAL(TOTAL(w_in[*,i,*],3),2)
    ENDELSE
    etot = SQRT(ytot)
    y0   = MAX(ytot,i0)
    xmin = xtot[(i0-20)>0]
    xmax = xtot[(i0+20)<(nchannels-1)]

    FITGAUSS, xtot, ytot, etot, xmin, xmax, gauss, dgauss

    chel = gauss[2]

    IF (insU NE 'MIBEMOL') AND (insU NE 'DCSASC') THEN parv[9,*] = chel

    IF iprint THEN PRINT, 'End of elastic peak section'


; ** Normalization for mibemol data and DCS
; ** added by S. Rols 09/01 srols@anl.gov
; ** *************************************

    CASE insU OF
      
        'MIBEMOL': BEGIN
            normff = TOTAL(monarr)/N_ELEMENTS(monarr)
            normff = normff/1000.
            chel = gauss[2] & parp[9] = chel
            mon = 1000 
            mod_datp, datp, "n",mon
        END
          
        'DCSASC': BEGIN
            normff = TOTAL(monarr)
            normff = normff/1000.
            chel  = gauss[2] & parp[10] = chel
            mon = 1000 
            mod_datp, datp, "n",mon
        END
        
        'FOCUS': BEGIN
            normff = TOTAL(monarr)/N_ELEMENTS(monarr)
            normff = normff/1000.
            chel = gauss[2] & parp[9] = chel
            mon = 1000 
            mod_datp, datp, "n",mon
        END
        
        'TOFTOF': BEGIN          
            CASE (norm) OF
                0: normff = 1 
                1: normff = monarr[1]/1000.
                2: normff = monarr[0]/60 
                else:
            ENDCASE
            chel = gauss[2] & parp[9] = chel
        END
        
        'NEAT': BEGIN
            normff = TOTAL(monarr)/N_ELEMENTS(monarr)
            normff = normff/1000.
            chel = gauss[2] & parp[9] = chel
            mon=1000 
            mod_datp, datp, "n",mon    
        END
        
        ELSE:
        
    ENDCASE
; -----------------------------------------------------------------------------
;Find monitor peak, integrate and normalise

    normf = FLTARR(nruns)
    w_out = FLTARR(nchannels,nspectra,nruns)
    e_out = FLTARR(nchannels,nspectra,nruns)

    FOR irun = 0, nruns - 1 DO BEGIN
      
        IF (insU EQ 'MIBEMOL') OR (insU EQ 'FOCUS') OR $
           (insU EQ 'DCSASC')  OR (insU EQ 'TOFTOF') OR (insU EQ 'NEAT') THEN BEGIN
            normf[irun]=normff
            GOTO,mibdcs
        ENDIF
        IF norm EQ 0 THEN normf = 1. ELSE IF norm EQ 1 THEN BEGIN
            xmon = x_in
            ymon = mon[*,0,irun]
            emon = SQRT(ymon)
            y0   = MAX(ymon,i0)
            xmin = xtot[(i0 - 20) > 0]
            xmax = xtot[(i0 + 20) < (nchannels - 1)]

            FITGAUSS, xmon, ymon, emon, xmin, xmax, gauss, dgauss

            bkgd   = gauss[0]
            height = gauss[1]
            centre = gauss[2]
            sigma  = gauss[3]
            IF  iprint THEN BEGIN
                PRINT, 'bkgd =',bkgd
                PRINT, 'height =',height
                PRINT, 'centre =',centre
                PRINT, 'sigma =',sigma
            ENDIF

            irange      = WHERE(ABS(xmon - centre) LT 8.*sigma, nrange)
            peaksum     = TOTAL(ymon[irange],1)
            bkgdsum     = TOTAL(ymon,1) - peaksum
            bkgd        = bkgdsum/FLOAT(nchannels - nrange)
            normf[irun] = (peaksum - bkgd*FLOAT(nrange)) / 1000.
        ENDIF ELSE IF norm EQ 2 THEN BEGIN
            IF insU EQ 'IN5' THEN normf[irun] = parv[0] ELSE normf = parv[0]/60.
            IF insU EQ 'IN4' THEN normf = monarr[1,*]/60.
        ENDIF

mibdcs:
        w_out[*,*,irun] = w_in[*,0:nspectra-1,irun] / normf[irun]
        e_out[*,*,irun] = SQRT(w_in[*,0:nspectra-1,irun]) / normf[irun]

    ENDFOR
    IF iprint THEN PRINT, 'End of monitor peak section'
; ------------------------------------------------------------------------------

;For IN4, calibrate wavelength

    IF (insU EQ 'IN4') AND (norm EQ 1) THEN BEGIN
      
        chw = parv[18]         ;channel time width
        rpm = parv[19]*2.     ;pulse frequency
        IF centre GT chel THEN BEGIN
            ;monitor measure in pulse i, detector in pulse i+1
            time_det_mon = (chw/1.e6) * (centre-chel)
            diff_time = (60./rpm) - time_det_mon ;dt = time between pulses - (M-D) time
        ENDIF ELSE BEGIN
            ;monitor and detector in same pulse
            diff_time = (chw/1.e6) * (chel-centre)
       ENDELSE
       L_mon2dets = 2.3039                       ; distance between monitor and detector
       speed  = L_mon2dets / (diff_time*1000.)   ; in km/s
       lambda = 3.956076 / speed                 ; in Angstroms
       if lambda lt 0.7 then begin
           ; need to add an extra pulse
           diff_time = diff_time + (60./rpm)
           speed  = L_mon2dets / (diff_time*1000.) ; in km/s
           lambda = 3.956076 / speed               ; in Angstroms
       endif

       parv[21,*] = lambda

       IF iprint THEN PRINT, 'End of wavelength calibration section'
       
    ENDIF
; ------------------------------------------------------------------------------
;Return parameters and exit

    s = STRTRIM(STRING(chel),2)  & i = STRPOS(s,'.') & chel  = STRMID(s,0,i[0]+3)
    s = STRTRIM(STRING(normf),2) & i = STRPOS(s,'.') & normf = STRMID(s,0,i[0]+3)

    lambdast = ''
    lstring  = ''
    CASE norm OF
      
        0: BEGIN
            nors = '/raw'
            nstring = 'not normalised. '
            datp.z_tit = 'Counts '
        END
        
        1: BEGIN
            nors = '/m'
            IF (insU EQ 'IN4') THEN BEGIN
                s = STRTRIM(STRING(lambda),2)
                i = STRPOS(s,'.')
                lambda  = STRMID(s,0,i[0]+4)
                lambdast = ', lambda = ' + lambda
                lstring = ' Calculated wavelength = ' + lambda
            ENDIF
            nstring = 'normalised to ' + normf + '/1000 M1 counts. '
            datp.z_tit = 'Counts / 1000 M1'
        END
        
        2: BEGIN
            nors = '/t'
            nstring = ' normalised to ' + normf + ' minutes. '
            datp.z_tit = 'Counts / min'
        END
        
        ELSE:
        
    ENDCASE

    normalisation = ' no(' + nors + ', ce = ' + chel + lambdast[0] + ')'
    cstring = 'Elastic channel = ' + chel + '. '
    outstring = nstring + cstring + lstring

    IF nruns EQ 1 THEN datp.p = parv ELSE BEGIN
        datp.pv = parv
        datp.p  = parv[*,nruns-1]
    ENDELSE
    s = datp.other_tit & i = strpos(/reverse_search,s,' ') & n = STRLEN(s) & numor = STRMID(s,i+1,n-1)
    IF (STRPOS(numor,'>') EQ -1) THEN numor = STRTRIM(STRING(LONG(parp[10])),2)
    datp.other_tit = inst + ' #' + numor + normalisation

    datp.x_tit = 'Channel number'
    datp.y_tit = 'Scattering angle'

    IF NOT keepmons THEN MOD_DATP, datp, "n", 1
    MOD_DATP,  datp, "y", y_in[0:nspectra-1]
    MOD_DATP,  datp, "e", e_out
    GIVE_DATP, datp

finished:
    RETURN, w_out
    END


; ------------------------------------------------------------------------------
;*******************************************************************************

FUNCTION normalise_d7, w_in0, inorm, ikeep, zeroshift
;+
;******* ************
;** normalise for D7
;-

    COMMON c_lamp_access, inst
    COMMON printing, iprint, outstring

    IF iprint THEN PRINT, 'Start normalise_d7:'
    TAKE_DATP, datp

; ------------------------------------------------------------------------------
;Check dimensions of input workspaces

    m_in0     = datp.n
    x_in0     = datp.x
    y_in0     = datp.y
    z_in0     = datp.z
    e_in0     = datp.e
    parp      = datp.p
    parv      = datp.pv
    e_in0     = SQRT(w_in0)
    TOF       = LONG(parp[8])
    nchannels = LONG(parp[6])
    nspectra  = LONG(parp[1])
    nphases   = LONG(parp[2])
    se        = SIZE(e_in0)
    sw        = SIZE(w_in0)
    print, datp.p[48]
    par_dim = (SIZE(datp.p))[1]

    IF nspectra LT 90 THEN BEGIN
        IF par_dim EQ 34 THEN nvers = 0   ; Pre-1996 data
        IF par_dim EQ 49 THEN nvers = 1   ; mid-1996 data
        IF par_dim EQ 50 THEN nvers = 2   ; end-1996 to 2003 data
        IF par_dim EQ 47 THEN nvers = 3   ; 2003 (Prototype analyser installed)
    ENDIF ELSE IF par_dim GE 48 THEN BEGIN
        IF nspectra LT 114 THEN nvers = 4   ; 2004 (1st MP detector bank installed)
        IF nspectra EQ 114 THEN nvers = 5   ; 2006 (2nd MP detector bank installed) 
        IF nspectra EQ 148 THEN nvers = 6   ; 2007 (3rd MP detector bank installed)
        IF nspectra EQ 132 THEN nvers = 7   ; 2007 (Bank 1 removed)
        IF TOF AND nspectra EQ 150 THEN nvers = 6  ; TOF 
        IF TOF AND nspectra EQ 135 THEN nvers = 7
        IF nspectra EQ 135 AND datp.p[48] EQ 6.0 THEN nvers = 8 ; March 2009-
        IF nspectra EQ 150 AND datp.p[48] EQ 6.0 THEN nvers = 9 ; TOF
    ENDIF
    IF datp.p[48] EQ 6.0 THEN nvers=8  ; March 2009 and onwards
    IF datp.p[48] EQ 7.0 THEN nvers=10 ;June and onwards

    rearrange = 0

    IF NOT TOF AND (nphases EQ 1) THEN BEGIN
        IF N_ELEMENTS(sw) EQ 4 THEN nruns = 1 ELSE nruns = sw[sw[0]]
        PRINT,'Normalise_d7: Assuming workspace contains a series of no-PA runs'
        rearrange = 1
        w_in = FLTARR(nspectra,nphases,nruns) & w_in[*,0,*]=w_in0[*,*]
        e_in = FLTARR(nspectra,nphases,nruns) & e_in[*,0,*]=e_in0[*,*]
        y_in = y_in0
        sw   = size(w_in)
    ENDIF ELSE IF sw[0] EQ 2 THEN nruns = 1 ELSE nruns = sw[3]

    IF nruns EQ 1 THEN parv = parp

    norm = inorm
    keep = ikeep
    nspec_in = nspectra

; Choose appropriate detectors------------------------------------------------

    IF nvers LE 2 THEN BEGIN
        IF TOF AND (norm EQ 1) AND (nspectra EQ 66) THEN norm = -65
        IF keep LE 0 THEN BEGIN
            nspectra  = 32            ;data pre-2003 POL
            goodspecs = INDGEN(32)*2 + 1
        ENDIF ELSE BEGIN
            nspectra  = 64            ;data pre-2003 NOPO/ALL
            goodspecs = INDGEN(64)
        ENDELSE
    ENDIF ELSE IF nvers EQ 3 THEN BEGIN
        IF TOF AND (norm EQ 1) THEN norm = -75
        IF keep LE 0 THEN BEGIN
            nspectra  = 42            ;data 2003 (prototype bank) POL
            goodspecs =  [(INDGEN(32)*2 + 1),(INDGEN(10) + 64)]
        ENDIF ELSE BEGIN
            IF keep GT 2 THEN nspectra = 64 ELSE nspectra = 74   ;data 2003 (prototype bank) NOPO/ALL
            goodspecs = INDGEN(nspectra)
        ENDELSE
    ENDIF ELSE IF nvers EQ 4 THEN BEGIN
        IF nspec_in GE 86 THEN BEGIN        ;data from cycle 043 onwards
            IF TOF AND (norm EQ 1) THEN norm = -87
            IF keep LE 0 THEN BEGIN
                nspectra = 70        ;data cycle 043 (1st bank) POL
                goodspecs =  [INDGEN(54), (INDGEN(16)*2 + 55)]
            ENDIF ELSE BEGIN
                IF keep GT 2 THEN BEGIN        ;nopo
                    nspectra  = 32        ;data cycle 043 (1st bank) NOPO
                    goodspecs = INDGEN(nspectra) + 54
                ENDIF ELSE BEGIN
                    nspectra  = 86        ;data cycle 043 (1st bank) ALL
                    goodspecs = INDGEN(nspectra)
                ENDELSE
            ENDELSE
        ENDIF
        IF nspec_in GE 92 THEN BEGIN                  ;data from cycle 042
            IF TOF AND (norm EQ 1) THEN norm = -93
            IF keep LE 0 THEN BEGIN
                nspectra = 68          ;data cycle 042 (1st bank) POL
                goodspecs =  [INDGEN(44), (INDGEN(24)*2 + 45)]
            ENDIF ELSE BEGIN
                IF keep GT 2 THEN BEGIN        ;nopo
                    nspectra  = 48        ;data cycle 042 (1st bank) NOPO
                    goodspecs = INDGEN(nspectra) + 44
                ENDIF ELSE BEGIN
                    nspectra  = 92        ;data cycle 042 (1st bank) ALL
                    goodspecs = INDGEN(nspectra)
                ENDELSE
            ENDELSE
        ENDIF
    
    ENDIF ELSE IF nvers EQ 5 THEN BEGIN        ;data from cycle 061
        IF TOF AND (norm EQ 1) THEN norm = -115
        IF keep LE 0 THEN BEGIN
            nspectra = 106            ;data cycle 061 (2nd bank) POL
            goodspecs =  [INDGEN(98), (INDGEN(8)*2 + 99)]
        ENDIF ELSE BEGIN
            IF keep GT 2 THEN BEGIN        ;nopo
                nspectra  = 16        ;data cycle 061 (2nd bank) NOPO
                goodspecs = INDGEN(nspectra) + 99
            ENDIF ELSE BEGIN
                nspectra  = 114        ;data cycle 061 (2nd bank) ALL
                goodspecs = INDGEN(nspectra)
            ENDELSE
        ENDELSE
    
    ENDIF ELSE IF nvers EQ 6 THEN BEGIN              ;data from cycle 071
        IF TOF AND (norm EQ 1) THEN norm = -149
        IF keep LE 0 THEN BEGIN
            nspectra = 140            ;data cycle 071 (3rd bank) POL ; only 8 dets on bank1 are usefull
            goodspecs =  [INDGEN(132), (INDGEN(8)*2 + 133)]
        ENDIF ELSE BEGIN
            IF keep GT 2 THEN BEGIN        ;nopo
                nspectra  = 16        ;data cycle 071 (3rd bank) NOPO
                goodspecs = INDGEN(nspectra) + 133
            ENDIF ELSE BEGIN
                nspectra  = 148        ;data cycle 071 (3rd bank) ALL
                goodspecs = INDGEN(nspectra)
            ENDELSE
        ENDELSE
    
    ENDIF ELSE IF nvers GE 7 THEN BEGIN              ;data from cycle 073 onwards
            IF TOF AND (norm EQ 1) THEN norm = -133
        IF keep LE 0 THEN BEGIN
            nspectra = 132            ;data cycle 073 
            goodspecs =  [INDGEN(132)]
        ENDIF ELSE BEGIN
            IF keep GT 2 THEN BEGIN        ;nopo
                nspectra  = 132        ;data cycle 073 
                goodspecs = INDGEN(nspectra)
            ENDIF ELSE BEGIN
                nspectra  = 132        ;data cycle 073 
                goodspecs = INDGEN(nspectra)
            ENDELSE
        ENDELSE
    ENDIF

    parp[1] = nspectra
    parp[3] = nruns

    IF iprint THEN BEGIN
        PRINT,'size(w_in)= ',sw
        PRINT,'nvers     = ',nvers
        PRINT,'TOF       = ',TOF
        PRINT,'nchannels = ',nchannels
        PRINT,'nspectra  = ',nspectra
        PRINT,'nphases   = ',nphases
        PRINT,'nruns     = ',nruns
        PRINT,'norm      = ',norm
        PRINT,'keep      = ',keep
    ENDIF

    m_in = m_in0
    x_in = x_in0
    sz   = SIZE(z_in0)
    IF sz[0] EQ 3 THEN BEGIN
        ns   = sz[1]
        nr   = sz[3]
        z_in = REFORM(z_in0,ns,nr)
    ENDIF ELSE z_in = z_in0
    IF NOT rearrange THEN BEGIN
        w_in = w_in0
        e_in = e_in0
        y_in = y_in0
    ENDIF

    IF iprint THEN PRINT,'End of "w_in dimensions check etc." section'

; ------------------------------------------------------------------------------
;For TOF data, find elastic peak

    IF TOF THEN BEGIN

        specs = goodspecs
        FOR iphase = 1,nphases-1 DO specs = [specs,goodspecs + nspec_in*iphase]
        ytot   = TOTAL(w_in[*,specs,*],2)
        IF nruns GT 1 THEN ytot = TOTAL(ytot,2)
        etot   = SQRT(ytot)
        xtot   = x_in
        y0     = MAX(ytot,i0)
        xmin   = xtot[(i0 - 20) > 0]
        xmax   = xtot[(i0 + 20) < (nchannels - 1)]
        gauss  = FLTARR(4)
        dgauss = FLTARR(4)

        FITGAUSS, xtot, ytot, etot, xmin, xmax, gauss, dgauss

        chel   = gauss[2]
        parp[9]= chel

        IF iprint THEN PRINT, 'Normalise: Elastic channel = ' + $
                    STRTRIM(STRING(chel),2)
        IF iprint THEN PRINT, 'End of "elastic peak" section'
    ENDIF

; ------------------------------------------------------------------------------
;Perform normalisation

    IF NOT TOF THEN BEGIN

        x_out = x_in[goodspecs]
        IF nruns GT 1 THEN z_out = z_in[goodspecs,*] ELSE z_out = x_out

        iphase = INDGEN(nphases)
        irun   = INDGEN(nruns)

        w_in = w_in[goodspecs,*,*]
        e_in = e_in[goodspecs,*,*]

        IF norm EQ 0 THEN normf = 1.
        IF norm EQ 1 THEN normf = m_in[1,iphase+1,irun]/1000.
        IF norm EQ 2 THEN normf = m_in[0,iphase+1,irun]/100.
        IF norm LT 0 THEN normf = w_in[-norm-1,iphase,irun]/1000.

        nspecs = (SIZE(w_in))[1]
        normf = normf[INTARR(nspecs),*,*]
        IF iprint THEN HELP, normf

        w_out = w_in/normf
        e_out = e_in/normf

        IF (nruns EQ 1) OR (nphases NE 1) THEN y_out = y_in $
        ELSE BEGIN
            y_out    = INTARR(nruns)
            y_out    = LONG(parv[0,*])
        ENDELSE
        
    ENDIF ELSE BEGIN
        x_out    =  x_in
        y_out    = FLTARR(nspectra*nphases)
        iphase   = INDGEN(nphases)
        irun     = INDGEN(nruns)
        tofspecs = INTARR(nspectra*nphases)

        FOR iiphase = 0, nphases - 1 DO BEGIN
            ii = iiphase*nspectra
            tofspecs[ii:ii + nspectra - 1] = goodspecs + nspec_in*iiphase
        ENDFOR

        y_out = y_in[tofspecs]
        z_out = (y_out)[*,INTARR(nruns)]

; Build normalisation array

        CASE 1 OF
            norm EQ 0: normf = 1.
            norm EQ 1: normf = m_in[1,iphase + 1,irun]/1000.
            norm EQ 2: normf = m_in[0,iphase + 1,irun]/100.  ; time
            norm LT 0: BEGIN     ; detector
                ispec = -norm - 1 + iphase*nspec_in
                normf = TOTAL(w_in[*,ispec,irun],1)/1000.
                w_in[*,ispec,irun] = 0.
                      e_in[*,ispec,irun] = -1.
            END
            else:
        ENDCASE

        IF norm NE 0 THEN BEGIN
            a = (FLTARR(nspec_in) + 1.) # TRANSPOSE(iphase)
            b = REFORM(a,nspec_in*nphases)
            c = REFORM(normf[b,*],1,nspec_in*nphases,nruns)
            normf = c[INTARR(nchannels),*,*]
        ENDIF

        w_out  = w_in/normf
        e_out  = e_in/normf
        parp[6]= nchannels
        w_out  = w_out[*,tofspecs,*]
        e_out  = e_out[*,tofspecs,*]
        ENDELSE

    IF iprint THEN BEGIN
        PRINT,'size(w_out)=',size(w_out)
        PRINT,'size(e_out)=',size(e_out)
        PRINT,'size(x_out)=',size(x_out)
        PRINT,'size(y_out)=',size(y_out)
        PRINT,'size(z_out)=',size(z_out)
    ENDIF

    IF iprint THEN PRINT, 'End of "Perform normalisation" section'

; ------------------------------------------------------------------------------
;Recalculate and sort detector angles from YIG calibration
;MOST RECENT CORRECTION; YIG scan March 2009

    IF (keep EQ 0) OR (keep EQ 2) OR (keep EQ 4) THEN BEGIN
        IF iprint THEN PRINT,'Recalculating detector angles'

        IF parp[16] EQ 0. THEN BEGIN
            i = dialog_message('Normalise: Cannot recalibrate detector angles',/ERROR)
            GOTO, endangles
        ENDIF
        IF nvers EQ 10 THEN BEGIN   ; Data from March 2009 onwards
            bank = FLTARR(4,nruns)
            IF nruns EQ 1 THEN bank =   parp[[16,17,18,19]] $
                               ELSE bank =   parv[[16,17,18,19],*]
            offset4 = FLTARR(44)
            FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
                offset4[ispec] = -1.003*(ispec + 1) + 21.894 ELSE $
                offset4[ispec] = -1.005*(ispec + 1) + 22.313
                offset3 = FLTARR(44)
                        FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
                offset3[ispec] = -1.0012*(ispec + 1) + 19.65 ELSE $
                offset3[ispec] = -1.006*(ispec + 1) + 20.005
            offset2 = FLTARR(44)
            FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
                offset2[ispec] = -0.9927*(ispec + 1) + 18.33 ELSE $
                offset2[ispec] = -0.974*(ispec + 1) + 17.998

            bank4 = FLTARR(44,nruns)
            bank3 = FLTARR(44,nruns)
            bank2 = FLTARR(44,nruns)

            FOR irun = 0, nruns - 1 DO BEGIN
                bank4[*,irun] = bank[3,irun] - offset4
                bank3[*,irun] = bank[2,irun] - offset3
                bank2[*,irun] = bank[1,irun] - offset2
            ENDFOR
            phi = [bank4,bank3,bank2]
            z_out = phi[goodspecs,*]
        ENDIF ELSE IF nvers EQ 8 THEN BEGIN   ; Data from March 2009 onwards
            bank = FLTARR(4,nruns)
            IF nruns EQ 1 THEN bank =   parp[[16,17,18,19]] $
                               ELSE bank =   parv[[16,17,18,19],*]
            offset4 = FLTARR(44)
            FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
                offset4[ispec] = -1.0204*(ispec + 1) + 23.905 ELSE $
                offset4[ispec] = -0.9955*(ispec + 1) + 23.377
                offset3 = FLTARR(44)
                        FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
                offset3[ispec] = -0.99762*(ispec + 1) + 20.566 ELSE $
                offset3[ispec] = -0.99237*(ispec + 1) + 20.75
            offset2 = FLTARR(44)
            FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
                offset2[ispec] = -0.99383*(ispec + 1) + 19.192 ELSE $
                offset2[ispec] = -1.0035*(ispec + 1) + 19.827

            bank4 = FLTARR(44,nruns)
            bank3 = FLTARR(44,nruns)
            bank2 = FLTARR(44,nruns)

            FOR irun = 0, nruns - 1 DO BEGIN
                bank4[*,irun] = bank[3,irun] - offset4
                bank3[*,irun] = bank[2,irun] - offset3
                bank2[*,irun] = bank[1,irun] - offset2
            ENDFOR

            phi = [bank4,bank3,bank2]
            z_out = phi[goodspecs,*]
        ENDIF ELSE IF nvers EQ 7 THEN BEGIN   ; Data from May 2007 onwards
            bank = FLTARR(4,nruns)
            IF nruns EQ 1 THEN bank =   parp[[16,17,18,19]] $
                          ELSE bank =   parv[[16,17,18,19],*]
            offset4 = FLTARR(44)
            FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
                offset4[ispec] = -0.999537*(ispec + 1) + 28.504 ELSE $
                offset4[ispec] = -0.999752*(ispec + 1) + 28.243
                offset3 = FLTARR(44)
            FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
                offset3[ispec] = -0.99647*(ispec + 1) + 25.419 ELSE $
                offset3[ispec] = -0.99469*(ispec + 1) + 25.106 
                offset2 = FLTARR(44)
            FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
                offset2[ispec] = -0.99776*(ispec + 1) + 19.083 ELSE $
                offset2[ispec] = -0.99828*(ispec + 1) + 18.805

            bank4 = FLTARR(44,nruns)
            bank3 = FLTARR(44,nruns)
            bank2 = FLTARR(44,nruns)

            FOR irun = 0, nruns - 1 DO BEGIN
                bank4[*,irun] = bank[3,irun] - offset4
                bank3[*,irun] = bank[2,irun] - offset3
                bank2[*,irun] = bank[1,irun] - offset2
            ENDFOR

            phi = [bank4,bank3,bank2]
            z_out = phi[goodspecs,*]
        ENDIF ELSE IF nvers EQ 6 THEN BEGIN   ; Data from January 2007 onwards
            bank = FLTARR(4,nruns)
            IF nruns EQ 1 THEN bank =   parp[[16,17,18,19]] $
                          ELSE bank =   parv[[16,17,18,19],*]
            offset4 = FLTARR(44)
            FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
                offset4[ispec] = -0.999537*(ispec + 1) + 28.504 ELSE $
                offset4[ispec] = -0.999752*(ispec + 1) + 28.243
                offset3 = FLTARR(44)
            FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
                offset3[ispec] = -0.99647*(ispec + 1) + 25.419 ELSE $
                offset3[ispec] = -0.99469*(ispec + 1) + 25.106 
                offset2 = FLTARR(44)
            FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
                offset2[ispec] = -0.99776*(ispec + 1) + 19.083 ELSE $
                offset2[ispec] = -0.99828*(ispec + 1) + 18.805

            bank4 = FLTARR(44,nruns)
            bank3 = FLTARR(44,nruns)
            bank2 = FLTARR(44,nruns)
            bank1 = FLTARR(16,nruns)

            FOR irun = 0, nruns - 1 DO BEGIN
                bank4[*,irun] = bank[3,irun] - offset4
                bank3[*,irun] = bank[2,irun] - offset3
                bank2[*,irun] = bank[1,irun] - offset2
                bank1[*,irun] = INDGEN(16)*2.8675 + bank[0,irun] - 24.736
            ENDFOR

            phi = [bank4,bank3,bank2,bank1]
            z_out = phi[goodspecs,*]
       
        ENDIF ELSE IF nvers EQ 5 THEN BEGIN   ; Data from June 2006 onwards
            bank = FLTARR(4,nruns)
            IF nruns EQ 1  THEN bank =   parp[[16,17,18,19]] $
                           ELSE bank =   parv[[16,17,18,19],*]
            offset4 = FLTARR(44)
            FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ float(ispec + 1)/2. THEN $
                offset4[ispec] = -1.002*(ispec + 1) + (15.0*2.) ELSE $
                offset4[ispec] = -1.002*(ispec + 1) + (14.851*2.)
                offset3 = FLTARR(44)
            FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ float(ispec + 1)/2. THEN $
                offset3[ispec] = -1.002*(ispec + 1) + (13.4725*2.) ELSE $
                offset3[ispec] = -1.002*(ispec + 1) + (13.434*2.)
                offset2 = FLTARR(10)
            FOR ispec = 0,9 DO IF (ispec + 1)/2 EQ float(ispec + 1)/2. THEN $
                offset2[ispec] = -1.002*(ispec + 1) + 1. ELSE $
                offset2[ispec] = -1.002*(ispec + 1) + 1.

            bank4 = FLTARR(44,nruns)
            bank3 = FLTARR(44,nruns)
            bank2 = FLTARR(10,nruns)
            bank1 = FLTARR(16,nruns)

            FOR irun = 0, nruns - 1 DO BEGIN
                bank4[*,irun] = bank[3,irun] - offset4
                bank3[*,irun] = bank[2,irun] - offset3
                bank2[*,irun] = bank[1,irun] - offset2
                bank1[*,irun] = INDGEN(16)*2.8675 + bank[0,irun] - 27.613
            ENDFOR

            phi = [bank4,bank3,bank2,bank1]
            z_out = phi[goodspecs,*]

        ENDIF ELSE IF nvers EQ 4 THEN BEGIN     ; Data from Mar 2004 till Aug 2005
            bank = FLTARR(4,nruns)
            IF nruns EQ 1  THEN bank =   parp[[16,17,18,19]] $
                           ELSE bank =   parv[[16,17,18,19],*]
            offset4 = FLTARR(44)
            FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ float(ispec + 1)/2. THEN $
                offset4[ispec] = -1.002*(ispec + 1) + (15.0*2.) ELSE $
                offset4[ispec] = -1.002*(ispec + 1) + (14.851*2.)

            bank4 = FLTARR(44,nruns)
            bank3 = FLTARR(16,nruns)

            IF nspec_in LT 90 THEN nbank2 = 10   ELSE nbank2 = 16
            IF nspec_in GT 90 THEN gbank2 = 2.85 ELSE gbank2 = 1.002
            bank2 = FLTARR(nbank2,nruns)
            bank1 = FLTARR(16,nruns)

            FOR irun = 0, nruns - 1 DO BEGIN
                bank4[*,irun] = bank[3,irun] - offset4
                bank3[*,irun] = INDGEN(16)*2.85       + bank[2,irun] - 26.39
                bank2[*,irun] = INDGEN(nbank2)*gbank2 + bank[1,irun] + 1.
                bank1[*,irun] = INDGEN(16)*2.85       + bank[0,irun] - 25.68
            ENDFOR

            phi = [bank4,bank3,bank2,bank1]
            IF nspec_in EQ 86 THEN phi = [bank4,bank2,bank3,bank1]
            z_out = phi[goodspecs,*]

        ENDIF ELSE BEGIN
            bank = FLTARR(5,nruns)
            IF nruns EQ 1  THEN bank =   parp[[16,17,18,19,16]] $
                       ELSE bank =   parv[[16,17,18,19,16],*]
            angle = FLTARR(16,5) - zeroshift
            angle[*,0] = angle[*,0] + [206.71,203.12,200.23,196.93,194.38,191.62,188.85,185.86, $
                           182.89,179.78,177.33,174.77,171.79,168.95,165.74,163.03]
            angle[*,1] = angle[*,1] + [198.06,194.34,192.01,188.62,186.48,182.86,180.63,177.44, $
                           174.81,171.50,169.22,165.60,163.51,159.88,157.71,154.19]
            angle[*,2] = angle[*,2] + [204.73,200.55,198.87,195.18,193.16,189.76,187.48,183.90, $
                           181.55,178.29,175.91,172.69,170.22,166.79,164.58,161.16]
            angle[*,3] = angle[*,3] + [199.26,196.81,193.42,191.08,187.72,185.17,182.06,179.64, $
                           176.37,173.99,170.69,168.34,164.99,162.66,159.30,156.86]
            angle[*,4] = angle[*,4] + [154.70,153.96,152.74,152.01,150.81,150.00,148.83,148.13, $
                           146.83,146.29,  0.00,  0.00,  0.00,  0.00,  0.00,  0.00]
            FOR irun = 0, nruns - 1 DO BEGIN
                phi = REFORM((INTARR(16) + 1) # bank[*,irun] - angle, 5*16)
                IF NOT TOF THEN z_out[*,irun] = phi[goodspecs]
            ENDFOR
        ENDELSE
    
    phi = phi[goodspecs]
    IF NOT TOF THEN BEGIN
        x_out = phi
    ENDIF ELSE BEGIN
        IF nphases EQ 1 THEN y_out = phi $
            ELSE IF nphases EQ 2 THEN y_out = [phi,phi] $
                 ELSE y_out = [phi,phi,phi,phi,phi,phi]
         IF iprint THEN PRINT,'y_out=',y_out
    ENDELSE

    IF iprint EQ 2 THEN PRINT,'z_out=',z_out
    ENDIF

endangles:

    IF iprint THEN PRINT,'End of "Calibrate angles" section'


; ------------------------------------------------------------------------------
;Return parameters and exit

    datp.p = parp
    MOD_DATP, datp, "x", x_out
    MOD_DATP, datp, "z", z_out

    IF NOT TOF AND nphases EQ 1 THEN BEGIN
        w_out0 = FLTARR(nspectra,nruns) & e_out0 = w_out0
        w_out0[*,*] = w_out[*,0,*]
        e_out0[*,*] = e_out[*,0,*]
        IF nruns GT 1 THEN MOD_DATP, datp, "y", y_out
    ENDIF ELSE BEGIN
        w_out0 = w_out
        e_out0 = e_out
        MOD_DATP, datp, "y", y_out
    ENDELSE

    MOD_DATP, datp, "e", e_out0

    label=' '

    CASE 1 OF
        norm EQ 0: BEGIN
            nors = '/r'
            nstring = 'no normalisation'
        END
        norm EQ 1: BEGIN
            nors = '/m'
            nstring = 'normalised to M1'
        END
        norm EQ 2: BEGIN
            nors = '/t'
            nstring = 'normalised to counting time'
        END
              (norm EQ -65) OR (norm EQ -75) OR (norm EQ -115) OR (norm EQ -149) OR (norm EQ -93): BEGIN
            nors = '/m'
            nstring = 'normalised to TOF-M1'
        END
        ELSE: BEGIN
            nors = 'det='+STRTRIM(STRING(-norm),2)
            nstring='normalised to s'+STRTRIM(STRING(-norm),2)
        END
    ENDCASE

    CASE keep OF
        1: BEGIN
            kees = ',/all,/mad'
            kstring = 'all detectors kept, raw det angles used'
        END
        -1: BEGIN
            kees = ',/mad'
            kstring = 'hidden detectors discarded, raw det angles used'
        END
        2: BEGIN
            kees = ',/all'
            kstring = 'all detectors kept, det angles recalculated'
        END
        3: BEGIN
            kees = ',/nopo,/mad'
            kstring = '"nopo" detectors kept, raw det angles used'
        END
        4: BEGIN
            kees = ',/nopo'
            kstring = '"nopo" detectors kept, det angles recalculated'
        END

        ELSE: BEGIN
            kees = ''
            kstring = 'hidden detectors discarded, det angles recalculated'
        END
    ENDCASE

    normalisation = ' no(' + nors + kees
    IF zeroshift GT 0.0 THEN BEGIN
        s=STRTRIM(STRING(zeroshift),2) & i=STRPOS(s,'.') & zers=STRMID(s,0,i+3)
        normalisation = normalisation+',zs='+zers
    ENDIF
    IF NOT TOF THEN normalisation = normalisation+')' ELSE BEGIN
        s=STRTRIM(STRING(chel),2) & i=STRPOS(s,'.') & chel=STRMID(s,0,i+3)
        normalisation = normalisation+',ce='+chel+')'
        cstring = '.  Elastic Channel = '+chel
    ENDELSE

    IF NOT TOF THEN BEGIN
        cstring = ''
        IF (nruns EQ 1) THEN BEGIN
            IF (nphases EQ 1) THEN $
            datp.y_tit = STRTRIM(label,2) $
            ELSE BEGIN
                datp.y_tit = 'Phase'
                datp.z_tit = STRTRIM(label,2)
            ENDELSE
        ENDIF ELSE BEGIN
            IF (nphases EQ 1) THEN BEGIN
                datp.y_tit = 'Run Number'
                datp.z_tit = STRTRIM(label,2)
            ENDIF ELSE BEGIN
                datp.y_tit = 'Phase'
                datp.z_tit = 'Run Number'
            ENDELSE
        ENDELSE
    ENDIF ELSE BEGIN
        datp.y_tit = 'Scattering Angle'
        IF (nruns EQ 1) THEN datp.z_tit = STRTRIM(label,2) $
             ELSE datp.z_tit = 'Numor'
    ENDELSE

    s=datp.other_tit & i= strpos(/reverse_search,s,' ') & n=STRLEN(s) & numor=STRMID(s,i+1,n-1)
    IF (nruns EQ 1) AND (STRPOS(numor,'>') EQ -1) THEN numor = STRTRIM(STRING(LONG(parp[0])),2)
    IF (nruns EQ 1) AND (STRPOS(numor,'>') NE -1) THEN numor = numor
    IF nruns NE 1 THEN $
    numor=STRTRIM(STRING(LONG(parv[0,0])),2)+':'+STRTRIM(STRING(LONG(parv[0,nruns-1])),2)

    IF (STRPOS(s,' -ax') EQ -1 AND STRPOS(s,' -cc') EQ -1) THEN $  ; data not previously concatenated, or add_xyz'd
        datp.other_tit='D7 #'+numor+normalisation $
    ELSE datp.other_tit=datp.other_tit+normalisation

    outstring = nstring + ', ' + kstring + cstring

finished:
    IF iprint THEN PRINT,'End normalise:'

    GIVE_DATP, datp

    RETURN, w_out0
    END
; ----------------------------------------------------------------------------
;*****************************************************************************

FUNCTION normalise_het, w_in, ei = ei
;+
;******* *************
;** normalise for HET
;-

    COMMON printing, iprint, outstring
    take_datp, datp

    n    = datp.n
    e_in = datp.e
    y_in = datp.y
    z_in = datp.z
    x_in = datp.x
    s    = SIZE(w_in)
    nchannels = s[1]
    nspectra = s[2]
    IF iprint THEN BEGIN
        PRINT, 'Nspectra  = ', nspectra
        PRINT, 'Nchannels = ', nchannels
    ENDIF

    IF iprint THEN PRINT, 'Normalise_HET: End of check dimensions, etc. section'
; ------------------------------------------------------------------------------------
; Find elastic lines

    offset = 100      ; offset required to ignore prompt peak

;2.5m bank
    i = WHERE(z_in LT 2.6 AND z_in GT 2.4)
    xtot = x_in[offset : nchannels - 1]
    ytot = TOTAL(w_in[offset:nchannels-1,i],2)
    etot = SQRT(TOTAL(e_in[offset:nchannels-1,i]^2,2))
    y0   = MAX(ytot,i0)
    xmin = xtot[(i0 - 20) > 0]
    xmax = xtot[(i0 + 20) < (nchannels - 1)]
    FITGAUSS, xtot, ytot, etot, xmin, xmax, gauss, dgauss
    chel1   = gauss[2]
    datp.p[9] = chel1

;4m bank
    i = WHERE(z_in LT 4.1 AND z_in GT 3.9)
    xtot = x_in[offset:nchannels - 1]
    ytot = TOTAL(w_in[offset:nchannels-1,i],2)
    etot = SQRT(TOTAL(e_in[100:nchannels-1,i]^2,2))
    y0   = MAX(ytot,i0)
    xmin = xtot[(i0 - 20) > 0]
    xmax = xtot[(i0 + 20) < (nchannels - 1)]
    FITGAUSS, xtot, ytot, etot, xmin, xmax, gauss, dgauss
    chel2   = gauss[2]
    datp.p[8] = chel2

    IF (iprint GT 0) THEN PRINT,'End of elastic peak section'

; ---------------------------------------------------------------------------
; Put Incident energy into parameter block

    IF N_ELEMENTS(ei) EQ 0 THEN BEGIN
        s = STRARR(3)
        s[0] = 'You need to include the incident energy'
        s[1] = 'for HET data'
        s[2] = 'e.g.   w2 = normalise(w1, Ei = 100)'
        i = dialog_message(s, /ERROR)
        GOTO, finished
    ENDIF ELSE BEGIN
        ei = FLOAT(ei)
        lambda = SQRT(81.8066/ei)
        datp.p[21] = lambda
    ENDELSE
; ---------------------------------------------------------------------------
; Sort data in ascending 2-theta order

    sorty = SORT(y_in)
    w_buf = w_in[*,sorty]
    e_buf = e_in[*,sorty]
    z_buf = z_in[sorty]
    y_buf = y_in[sorty]

; Perform normalisation to M1 (must use M1 for HET data)

    narr = INTARR(nspectra)
    w_out = w_buf/n[*,narr]
    e_out = SQRT( (e_buf/n[*,narr])^2 + (w_buf*n[*,narr+1]/(n[*,narr]^2))^2 )

; Remove zero spectra
    i = WHERE(y_buf NE 0, nnz)
    w_out = w_out[*,i]
    e_out = e_out[*,i]
    y_out = y_buf[i]
    z_out = z_buf[i]
    s = SIZE(w_out)
    nspectra = s[2]

    IF iprint THEN PRINT, 'Normalise_HET: End of normalisation section'

; --------------------------------------------------------------------------
; Return data and paramaters

    s=STRTRIM(STRING(ei),2)     & i=STRPOS(s,'.') & ei    =STRMID(s,0,i+3)
    s=STRTRIM(STRING(lambda),2) & i=STRPOS(s,'.') & lambda=STRMID(s,0,i+3)
    s=STRTRIM(STRING(chel1),2)  & i=STRPOS(s,'.') & chel1 =STRMID(s,0,i+3)
    s=STRTRIM(STRING(chel2),2)  & i=STRPOS(s,'.') & chel2 =STRMID(s,0,i+3)
    datp.other_tit = datp.other_tit + ' no(Ei='+ei+') '
    mod_datp, datp, "e", e_out
    mod_datp, datp, "y", y_out
    mod_datp, datp, "z", z_out
    give_datp, datp
    outstring = ' Normalised to HET monitor 1, Lambda = '+ lambda
    cstring = ', Elastic times: 2.5m bank - ' + chel1 + ', 4m bank - ' + chel2
    outstring = outstring + cstring
    RETURN, w_out

finished:
    END
; ------------------------------------------------------------------------------
;*******************************************************************************

FUNCTION normalise, w_in, RAW=raw, MONITOR=monitor, TIME=time, $
                    DETECTOR=detector, ALLDETECTORS=alldetectors, EI=ei, $
                    MADANGLES=madangles, ZEROSHIFT=zeroshift, KEEPMONITORS=keepmonitors,$
                    NOPO = nopo, inorm, ikeep
;+
;******* *********
;** User callable
;** normalise for TOF , D7 , HET
;-

    COMMON c_lamp_access, inst
    COMMON printing, iprint, outstring

    iprint = 0  ; if iprint>0, show debugging messages

    IF iprint THEN PRINT, 'Start normalise:'

    IF N_ELEMENTS(inorm) EQ 0 THEN inorm = 1        ;default
    IF KEYWORD_SET(raw)       THEN inorm = 0
    IF KEYWORD_SET(monitor)   THEN inorm = 1
    IF KEYWORD_SET(time)      THEN inorm = 2
    IF KEYWORD_SET(detector)  THEN BEGIN
        IF(detector NE 0) THEN inorm = -detector
    ENDIF

    IF N_ELEMENTS(ikeep) EQ 0 THEN ikeep = 0            ;default
    IF KEYWORD_SET(madangles) AND     KEYWORD_SET(alldetectors) THEN ikeep = 1
    IF KEYWORD_SET(madangles) AND NOT KEYWORD_SET(alldetectors) THEN ikeep = -1
    IF NOT KEYWORD_SET(madangles) AND KEYWORD_SET(alldetectors) THEN ikeep = 2
    IF KEYWORD_SET(madangles) AND     KEYWORD_SET(nopo)         THEN ikeep = 3
    IF NOT KEYWORD_SET(madangles) AND KEYWORD_SET(nopo)         THEN ikeep = 4

    IF N_ELEMENTS(zeroshift) EQ 0 THEN zeroshift = 0.0
    IF KEYWORD_SET(keepmonitors) THEN keepmons = 1 ELSE keepmons = 0
; ------------------------------------------------------------------------------
;Check instrument name and call appropriate function
        insU=strupcase(inst)
    IF (insU EQ 'IN4') OR (insU EQ 'IN5') OR (insU EQ 'IN6') OR (insU EQ 'DCSASC') OR $
       (insU EQ 'MIBEMOL') OR (insU EQ 'MIBENEX') OR (insU EQ 'TOFTOF') OR (insU EQ 'NEAT') OR (insU EQ 'FOCUS') THEN $
        w_out=normalise_tof(w_in,inorm,keepmons) $
    ELSE IF (insU EQ 'D7') THEN $
        w_out=normalise_d7(w_in,inorm,ikeep,zeroshift) $
    ELSE IF (insU EQ 'HET') THEN $
        w_out=normalise_HET(w_in, ei = ei) $
    ELSE BEGIN
        s = STRARR(13)
        s[0] = 'Normalise: Instrument must be specified as one of the following'
        s[1] = ''
        s[2] = 'IN4'
        s[3] = 'IN5'
        s[4] = 'IN6'
        s[5] = 'D7'
        s[6] = 'HET'
        s[7] = 'mibemol'
        s[8] = 'DCSasc'
        s[9] = 'mibenex'
        s[10]= 'TOFTOF'
        s[11]= 'NEAT'
        s[11]= 'FOCUS'        
        i = dialog_message(s,/ERROR)
        return,w_in
    ENDELSE

; ------------------------------------------------------------------------------
;Return parameters and exit

    PRINT, 'normalise: ' + outstring

finished:
    RETURN, w_out
    END
