;
FUNCTION slab_tof, w_in, angle=alpha, inc_xs=sigmai, abs_xs=sigmaa, thick=thick, N=N
;+
;******* ********
;** User callable
;** For D7, IN4, IN5, IN6 and else!
;**
;** Assuming par[9:ElasPos , 18:ChanWidth , 21:WaveL , 27:Distance]
;**
;** Takes a 2-D tof workspace and corrects for sample attenuation of the
;** scattered neutrons assuming infinite slab geometry.
;** -------------------------------
;** NB!  Not yet tested on D7 data
;** -------------------------------
;**
;** ARGUMENTS
;** angle : the angle in degrees of sample normal rel. to incident beam
;** inc_xs: inchorent cross-section of sample (barns)
;** abs_xs: absorption cross-section for thermal neutrons (lambda=1.8�) (barns)
;** thick : thickness of sample (mm)
;** N     : number density of sample (*10^22 per cm^3)
;**
;** DIMENSIONS
;** w_in(nchannels,nspectra) -> w_out(nchannels,nspectra)
;**
;** COMMAND SYNTAX
;** w7=slab_tof(w6,angle=<angle>,inc_xs=<inc_xs>,abs_xs=<abs_xs>,thick=<thick>,N=<N>)
;**
;** Modified by S.Rols Feb-2014
;**                            JRS, 15/11/02
;** Modified by S.Rols Dec-2014: to account for an error on L2 (detected by B. Fak)
;-

    iprint = 0    ; if iprint > 0, show debugging messages

    COMMON c_lamp_access, inst
    IF iprint THEN PRINT,'Start slab_tof:'

    take_datp, datp

    alpha  = FLOAT(alpha)
    sigmai = FLOAT(sigmai)
    sigmaa = FLOAT(sigmaa)
    thick  = FLOAT(thick)
    N      = FLOAT(N)
    alphar = alpha*!pi/180.
    Nr     = N*1.e22

; ------------------------------------------------------------------------------
;Check dimensions of input workspaces

    par = datp.p
    IF(inst NE 'D7') THEN BEGIN
        lambda = par[21]
        chw    = par[18]    ; channelwidth in microseconds
        chel   = par[9]
        L2     = par[27]*1000
    ENDIF ELSE BEGIN
        lambda = par[4]
        chw    = par[7]
        chel   = par[9]
        L2     = 1500       
    ENDELSE

    sw        = SIZE(w_in)
    nspectra  = sw[2]
    nchannels = sw[1]
    ei        = 81.8066/(lambda^2)

    IF (iprint GT 0) THEN BEGIN
        PRINT,'lambda=',lambda
        PRINT,'ei=',ei
        PRINT,'nspectra=',nspectra
        PRINT,'nchannels=',nchannels
    ENDIF

    x_in = datp.x    ;time-of-flight
    y_in = datp.y    ;scattering angle
    e_in = datp.e

    const1 = 5.22697      ; E(meV)=const1*V(mm/mus)^2 for neutron
    Vi     = SQRT(ei/const1)
    Tel    = L2/Vi        ; time in microseconds
    time   = chw*FLOAT(x_in - chel)+Tel
    Ef     = const1*(L2/time)^2
    Eps    = Ei - Ef

    IF (iprint GT 0) THEN PRINT,'End of "w_in dimensions check etc." section'

; ------------------------------------------------------------------------------
;Perform correction

    w_out  = w_in
    e_out  = FLTARR(nchannels,nspectra)

    ttheta = y_in*!pi/180.    ; anticlockwise is +ve

    sigmaa1 = sigmaa*1.E-24
    sigmai1 = sigmai*1.E-24
    sigmaar = sigmaa1*lambda/1.8
    mu      = Nr*(sigmaar + sigmai1)
    mu2     = Nr*sigmaar*(SQRT(Ei/(Ei - Eps)))
    beta    = mu/SIN(alphar)
    t       = thick/10.

    IF iprint THEN BEGIN
        PRINT,'sigmai  =',sigmai
        PRINT,'sigmaa  =',sigmaa
        PRINT,'sigmaar =',sigmaar
        PRINT,'mu      =',mu
        PRINT,'t       =',t
    ENDIF
    gama = FLTARR(nchannels,nspectra)
    A    = FLTARR(nchannels,nspectra)
    FOR iang = 0, nspectra - 1 DO BEGIN
        gama[*,iang] = mu2[*]/SIN(alphar - ttheta[iang])
    ENDFOR

    A = (EXP(-DOUBLE(beta*t)) - EXP(-DOUBLE(gama*t))) / $
        (DOUBLE(gama) - DOUBLE(beta))    

    i = WHERE( A EQ 0.)
    IF (i[0] NE -1) THEN A[i] = A[i - 1]    ;this removes the possible zeros

    w_out = w_in*t/A
    e_out = e_in*t/A
    x_out = x_in
    y_out = y_in

    IF iprint THEN PRINT,'End of Correction section'

; ------------------------------------------------------------------------------
;Return parameters and exit

    datp.e = e_out
    
    als = STRTRIM(STRING(alpha),2)   & als = string_round(als)
    sis = STRTRIM(STRING(sigmai),2)  & sis = string_round(sis)
    aas = STRTRIM(STRING(sigmaa),2)  & aas = string_round(aas)
    ths = STRTRIM(STRING(thick),2)   & ths = string_round(ths)
    Ns  = STRTRIM(STRING(N),2)       & Ns  = string_round(Ns)
    
    datp.other_tit=datp.other_tit+' -st(alp='+als+',inc='+sis+',abs=' $
                  +aas+',t='+ths+',N='+Ns+')'

finished:
    IF (iprint GT 0) THEN PRINT,'End slab_tof:'

    give_datp, datp

    RETURN, w_out
    END
