
FUNCTION sqw_window, Ei = Ei, lambda = lambda, inst = inst, $
                     angles=angles
;+
;******* **********
;** User callable
;** Plots out the S(Q,w) limits for given energy/wavelength, and angular ranges.
;** Instruments can be given in place of angles. 
;**
;** ARGUMENTS
;** -----------
;**    Ei:     incident energy
;**    lambda: incident wavelength
;**            (One of these is necessary. If both are given then Ei 
;**            is used.)
;**    inst:   equals 'in4' or 'in6' ('in5' not implemented)
;**    angles: an array of angles over which to caluculate the Q-w
;**            trajectories - supercedes those determined by inst.
;**
;**                        JRS 14/11/06
;**
;-

;Check input arguments and calculate constants
  
    TAKE_DATP, datp    
    
    IF N_ELEMENTS(Ei) GT 0 AND N_ELEMENTS(lambda) GT 0 THEN BEGIN
        PRINT, 'sqw_window: Both Ei and lambda specfied.  Using Ei.'
    ENDIF
    IF N_ELEMENTS(Ei) GT 0 THEN lambda = SQRT(81.81/Ei)
    IF N_ELEMENTS(Ei) LT 1 THEN BEGIN
        IF N_ELEMENTS(lambda) GT 0 THEN Ei = 81.81/lambda^2
        IF N_ELEMENTS(lambda) LT 1 THEN BEGIN
            s = 'sqw_window: must specify Ei or Lambda'
            ii = dialog_message(s, /ERROR)
            RETURN, 1
        ENDIF
    ENDIF
    
    instfl = 0
    IF N_ELEMENTS(inst) LT 1 THEN BEGIN
        IF N_ELEMENTS(angles) LT 2 THEN BEGIN
            s = 'sqw_window: at least 2 2theta angles must be given'
            ii = dialog_message(s,/ERROR)
            RETURN, 1
        ENDIF
    ENDIF ELSE BEGIN
        IF N_ELEMENTS(angles) GT 1 THEN BEGIN
            PRINT, 'sqw_window: using user input angles'
        ENDIF ELSE BEGIN
            instfl = 1
            CASE inst OF
                   'in4':    angles = [2.435,8.738,13.075,120.325]
                   'in6':    angles = [10.,115.]
                   'IN4':    angles = [2.435,8.738,13.075,120.325]
                   'IN6':    angles = [10.,115.]
            ENDCASE    
        ENDELSE
    ENDELSE

; ---------------------------------------------------------------------------
;Calculate trajectories    

    ki   = SQRT(Ei/2.0719)
    delE = 2.*Ei*FINDGEN(100)/100. - Ei
    Ef   = Ei -delE
    kf   = SQRT(Ef/2.0719)

    x_out = [-1]
    w_out = [-1]
    FOR iangle = 0,N_ELEMENTS(angles)-1 DO BEGIN
        Q = SQRT(ki^2 + kf^2 -2*ki*kf*cos(angles[iangle]*!pi/180.))
        IF FLOAT(iangle)/2. EQ FLOAT(iangle/2) THEN BEGIN
            x_out = [x_out,Q] 
            w_out = [w_out,delE]
        ENDIF ELSE BEGIN 
            x_out = [x_out,ROTATE(Q,2)]
            w_out = [w_out,ROTATE(delE,2)]
        ENDELSE
    ENDFOR
    x_out = x_out[1:*]
    w_out = w_out[1:*]

; ---------------------------------------------------------------------------
;Return workspace and exit

    datp.x_tit = 'Wavevector transfer (A^-1)'
    datp.y_tit = 'Energy transfer (meV)'

    dummy = STRTRIM(STRING(ROUND(lambda*100.)),2)
    Eis = STRMID(dummy,0,STRLEN(dummy)-2)+'.'+STRMID(dummy,STRLEN(dummy)-2)

    IF instfl THEN angless = 'INST = '+inst ELSE BEGIN
        angless = 'angles = ['
        FOR iangle = 0,N_ELEMENTS(angles)-1 DO BEGIN
            dummy = STRTRIM(STRING(ROUND(angles[iangle]*10.)),2)
            angless = angless + STRMID(dummy,0,STRLEN(dummy)-1)+'.'+STRMID(dummy,STRLEN(dummy)-1)+','
        ENDFOR
        angless = STRMID(angless,0,STRLEN(angless)-1) + ']'
    ENDELSE

    datp.w_tit     = 'S[Q,w] window for incident wavelength '+Eis    
    datp.other_tit = angless    
    MOD_DATP, datp, "x", x_out
    GIVE_DATP, datp

    RETURN, w_out
    END
