;
FUNCTION tof_Qrebin, w_in, omegashift=omegashift, fold=fold, $
                     nQ=nQ, Qmax=Qmax
;+
;******* **********
;** User callable
;**For TOF data
;**
;** Takes input workspace of energy (x-axis) 2theta (y-axis) and omega (z-axis)
;** and performs a Qrebin at each energy.
;**
;** ARGUMENTS:
;** omegashift    : offset in degrees of omega=0 position
;** nQ        : number of Qx/Qy bins per quadrant
;**
;** KEYWORDS:
;** fold        : folds quadrants into +,+
;**
;** DIMENSIONS:
;** w_in(nE,nspectra,nruns) -> w_out(nQ,nQ,nE) (folded), w_out(nE,2*nQ,2*nQ) (unfolded)
;**
;** COMMAND SYNTAX:
;** w10=tof_qrebin(w9,[nQ=<nQ>][,omegashift=<omegashift>][,/fold])
;**
;**                        KHA,JRS 30/05/08
;-

CATCH, errors

IF errors NE 0 THEN BEGIN
    print, ix, iy, de,qx[itheta,iomega], qy[itheta,iomega], SQRT(Qx[itheta,iomega]^2 + Qy[itheta,iomega]^2)
ENDIF

    TAKE_DATP, datp

    iprint = 0    ; if iprint > 0, show debugging messages

    fld = 1        ; don't fold data (default)
    IF N_ELEMENTS(omegashift) LE 0 THEN omegashift = 0. 
    IF N_ELEMENTS(oms) GT 0        THEN omegashift = oms
    IF N_ELEMENTS(nQ) LE 0           THEN nQ           = 26
    IF KEYWORD_SET(fold)           THEN fld        = 0

    e_in      = datp.e
    en_in     = datp.x
    lambda    = datp.p[21]
    ntheta    = (SIZE(w_in))[2]
    nomega    = (SIZE(w_in))[3]
    twotheta  = datp.y*!pi/180.        ; detector angle (rad)
    omega     = datp.z - omegashift        ; sample rotation angle (degrees)

    const2 = 2.07194            ; hbar^2/2m

    ki = 2.*!pi/lambda

    IF N_ELEMENTS(Qmax) LE 0 THEN BEGIN
        kfmax = SQRT(ki^2 - (MIN(en_in)/const2))
        Qmax = 1.1*SQRT(ki^2 + kfmax^2 - 2*ki*kfmax*COS(MAX(twotheta)))
    ENDIF

    dQ   = Qmax / FLOAT(nQ)

    IF iprint THEN BEGIN
        PRINT, 'ntheta = ', ntheta
        PRINT, 'nomega = ', nomega
        PRINT, 'lambda = ', lambda
        PRINT, 'ki     = ', ki
    ENDIF

    If iprint THEN PRINT,'tof_Qrebin: End of "set constants" section'

; ------------------------------------------------------------------------------
;loop over energies and perform q-rebinning

    nenergies = (SIZE(en_in))[1]
    w_out=FLTARR(nenergies,(fld+1)*nQ,(fld+1)*nQ)    & w_out[*,*,*]=0.
    e_out=FLTARR(nenergies,(fld+1)*nQ,(fld+1)*nQ)    & e_out[*,*,*]=0.
    n_out=FLTARR(nenergies,(fld+1)*nQ,(fld+1)*nQ)    & n_out[*,*,*]=0.

    FOR ienergy = 0, nenergies - 1 DO BEGIN
        dE = en_in[ienergy]
        w_buf = REFORM(w_in[ienergy,*,*],ntheta,nomega)
        e_buf = REFORM(e_in[ienergy,*,*],ntheta,nomega)
        kf = SQRT(ki^2 - (dE/const2))
        Qmag = SQRT(ki^2 + kf^2 - 2*ki*kf*COS(twotheta))
    
        beta  = (twotheta/(ABS(twotheta)))*(180./!pi)*ACOS((ki^2 - kf^2 + Qmag^2)/(2*ki*Qmag))
        ; beta is angle between ki and Q (degrees)

        alpha = 180. + ((FLTARR(ntheta) + 1) # TRANSPOSE(omega)) + (beta # TRANSPOSE(FLTARR(nomega) + 1))

        Qx = (Qmag # TRANSPOSE(FLTARR(nomega) + 1))* COS(alpha * !pi / 180.)
        Qy = (Qmag # TRANSPOSE(FLTARR(nomega) + 1))* SIN(alpha * !pi / 180.)

; ------------------------------------------------------------------------------
;Rebin data

        FOR itheta=0,ntheta-1 DO BEGIN
            FOR iomega=0,nomega-1 DO BEGIN
                IF (ABS(Qx[itheta,iomega]) GT 0.99*Qmax) OR (ABS(Qy[itheta,iomega]) GT 0.99*Qmax) THEN GOTO, next
                IF (fld EQ 1) THEN BEGIN
                    ix=FIX(((Qx[itheta,iomega]+dQ/2.)/dQ)+nQ)
                    iy=FIX(((Qy[itheta,iomega]+dQ/2.)/dQ)+nQ)
                ENDIF ELSE BEGIN
                    ix=FIX(ABS((Qx[itheta,iomega])+dQ/2.)/dQ)
                    iy=FIX(ABS((Qy[itheta,iomega])+dQ/2.)/dQ)
                ENDELSE
                w_out[ienergy,ix,iy]=w_out[ienergy,ix,iy]+w_buf[itheta,iomega]
                e_out[ienergy,ix,iy]=e_out[ienergy,ix,iy]+e_buf[itheta,iomega]^2
                n_out[ienergy,ix,iy]=n_out[ienergy,ix,iy]+1.
next:            
            ENDFOR
        ENDFOR
        FOR ix=0,(fld+1)*nQ-1 DO BEGIN
            FOR iy=0,(fld+1)*nQ-1 DO BEGIN
                IF (n_out[ienergy,ix,iy] GT 0.) THEN BEGIN
                    w_out[ienergy,ix,iy]=w_out[ienergy,ix,iy]/n_out[ienergy,ix,iy]
                    e_out[ienergy,ix,iy]=SQRT(e_out[ienergy,ix,iy])/n_out[ienergy,ix,iy]
                ENDIF ELSE BEGIN
                    w_out[ienergy,ix,iy]=0.
                    e_out[ienergy,ix,iy]=-1.
                ENDELSE
            ENDFOR
        ENDFOR
    ENDFOR

    y_out=FLOAT((INDGEN((fld+1)*nQ)-(fld*nQ))*dQ)    & z_out=y_out
    
    IF iprint THEN PRINT,'Qrebin: End of "rebin data" section'

; ------------------------------------------------------------------------------

finished:
    MOD_DATP, datp, "y", y_out
    MOD_DATP, datp, "z", z_out
    MOD_DATP, datp, "e", e_out
    datp.y_tit = 'Qx (Angstroms^-1)'
    datp.z_tit = 'Qy (Angstroms^-1)'

    d=STRING_ROUND(STRING(dQ))
    s=STRING_ROUND(STRING(omegashift))

        PRINT,'tof_Qrebin: rebinned to Qx-Qy grid with dQ='+d+'A-1 and omega-shift = '+s+' degrees'
        datp.other_tit=datp.other_tit+' -tqr(dQ='+d+',o='+s
        IF(fld=0) THEN datp.other_tit=datp.other_tit+',/fold)' ELSE $
                       datp.other_tit=datp.other_tit+')'


    GIVE_DATP, datp

    RETURN, w_out
    END
