;
FUNCTION vnorm2, w_in, w_van0, min=ch1, max=ch2, tv=tv, Ns=Ns, $
                ts=ts, cb1, cb2, ierr=ierr, verbose=verbose , Temp_van=Temp_van, no_DW=no_DW
;+
;******* *****
;** User callable
;** For IN4, IN5, IN6 and D7 data, TOF...
;**
;** Normalises data in w_in vanadium data in w_van0 integrated between given limits.
;**
;** ARGUMENTS
;** min   : lower time channel limit of integration over vanadium elastic peak
;** max   : upper time channel limit
;** tv    : thickness of vanadium sample
;** Ns    : number density of sample (*10^22 per cm^3)
;** ts    : effective sample thickness
;** (cb1 and cb2 are obsolete and kept for backwards compatability)
;** verbose : to print message (default = no verbose mode)
;** Tvan  : to specify the temperature of the vanadium scan (default = Tvan read in w_van0 parameters)
;**       : if Tvan < 1e-3 K then Tvan set to 300 K    
;**       
;** DIMENSIONS
;** w_in=w_out(nchannels, nspectra, nruns) (deals with multiple runs on TOF data)
;**
;** COMMAND SYNTAX
;** w8=vnorm(w7,w20,min=<min>,max=<max>[,tv=<tv>,Ns=<Ns>,ts=<ts>])
;**      - normalises data in w7 to vanadium data in w20,
;** (optional arguments shown in square brackets)
;**
;**                JRS,KHA  7/12/07 + SR(for mibemol data) 4/12/03
;**                
;**                DR  25/06/12  Nsd = Ns*1.E22 ; to keep Ns unchanged
;**
;**                ARW 30/01/13  Modified to have correct channel width and L2 for D7
;**                ARW 01/03/13  Modified to multiply the vanadium intensity by 
;**                        L2/channel width (necessary for absolute calibration)
;**                        NOTE: 'PELICAN', 'QENS', 'QENS_raw' are NOT modified!
;**                SR  05/12/14  Modified to include DW correction for Vanadium
;**                DR  25/03//14 **********************************************
;
;**        VNORM2.pro is a demo version in order to be tested without compomising the vnorm function in lamp
;**        SR and BF 12/2014 
;-

    COMMON c_lamp_access, inst

    iprint = 0    ; if iprint>0, show debugging messages
    If(keyword_set(verbose)) then iprint=1
    IF iprint THEN PRINT,'Start vnorm:'

    take_datp, datp
    take_datp, datvan, /third
    p_van=datvan.p
    
; ------------------------------------------------------------------------------
;  Compute DW term for Vanadium
;  -> See F. Rieutord's description in INX documentation
;  Caution: 2W=2*[hbar^2/2M]*[2k/(kT_Debye_vana)^2]*[T*300/300]*Q^2=alpha*Q^2
;  with alpha = 0.0067 * T / 300
;  <=> alpha = 2* [hbar^2/2M]*[2k/(kT_Debye_vana)^2]
;  T_Debye_vana = 355 K
;  M = 50.9415*1.67265E-27 kg
;  
; put the Vana temp into variable T_van -> DW correction
;    
    IF (inst NE 'D7') THEN BEGIN
      If N_elements(Temp_van) GT 0 THEN T_van=Temp_van ELSE T_van=p_van[11]
      IF T_van le 1e-3 then T_van=300.
      IF iprint THEN PRINT,'*** vnorm: Vanadium temperature =',T_van,' K'
      y_van = datvan.y
      q_van=4*!pi*sin(y_van*!pi/360.)/p_van[21]
      DW_van=exp(0.0067*T_van*q_van^2/300.)
      if keyword_set(no_DW) then begin
        DW_van=DW_van*0.+1.
        IF iprint THEN PRINT,'*** vnorm: No DW correction for Vanadium'
      endif
    ENDIF  
; End of DW Vanadium section
; -------------------------------------------------------------------------------    
    
    IF N_ELEMENTS(cb1) GT 0 THEN ch1 = cb1
    IF N_ELEMENTS(cb2) GT 0 THEN ch2 = cb2

; ------------------------------------------------------------------------------
;Check workspace sizes

    sw=SIZE(w_in)
    IF iprint THEN PRINT,'SIZE(w_in)=',sw
    nchannels = sw[1]
    IF sw[0] EQ 1 THEN nspectra = 1 ELSE nspectra = sw[2]
    IF sw[0] EQ 3 THEN nruns = sw[3] ELSE nruns = 1

; Part added by SR on 14/03/02
; ** Decide if include the vanadium error or not (default=included)
; *****************************************************************

ierrtype=1
IF(N_ELEMENTS(ierr) GT 0) THEN ierrtype=2

    CASE inst OF
        'QENS_raw':BEGIN
                 w_out=vnorm_qens[w_in, w_van0, ch1, ch2]
                 GOTO, finished
                 END
        'QENS':  BEGIN
                 w_out=vnorm_qens[w_in, w_van0, ch1, ch2]
                 GOTO, finished0
                 END
        'DCSasc':BEGIN
                 par = datp.p
                 vpar= datvan.p
                 e_in= datp.e
                 se  = SIZE(e_in)
                 chw = FLOAT(par[14])
                 vchw= FLOAT(vpar[14])
                 L2  = FLOAT(par[6])
                 END
         'D7':   BEGIN
                 par = datp.p
                 vpar= datvan.p
                 e_in= datp.e
                 se  = SIZE(e_in)
                 chw = par[7]
                 vchw= vpar[7]
                 L2  = 1500.0
                 end
        ELSE:    BEGIN
                 par = datp.p
                 vpar= datvan.p
                 e_in= datp.e
                 se  = SIZE(e_in)
                 chw = par[18]       ; Data channel width
                 vchw= vpar[18]      ; Vana channel width
                 L2  = par[27]*1000. ; Distance sample-detector
                 IF strupcase(inst) eq 'PELICAN' then vchw = L2 ; So L2/vchw = 1. (GJK versus ARW)
                 END
    ENDCASE
; ------------------------------------------------------------------------------


    IF inst EQ 'D7' THEN nphases = par[2]

    se = SIZE(e_in)
    IF iprint THEN PRINT,'SIZE(e_in) = ',se
    IF (se[0] NE sw[0]) OR (se[1] NE sw[1]) OR (se[2] NE sw[2]) THEN e_in = SQRT(w_in)

    w_van = w_van0    & sv = SIZE(w_van)

    e_van = datvan.e
    se = SIZE(e_van)
    IF (se[0] NE sv[0]) OR (se[1] NE sv[1]) OR (se[2] NE sv[2]) THEN e_van = SQRT(w_van)

    IF inst EQ 'D7' THEN BEGIN
        nvs = sv[2]
        IF iprint THEN PRINT,'D7: nspectra=',nspectra,' nvs=',nvs
        IF nspectra EQ 2*nvs THEN BEGIN
            w_van=[[w_van],[w_van]]
            e_van=[[e_van],[e_van]]
        ENDIF ELSE IF nspectra EQ 6*nvs THEN BEGIN
            w_van=[[w_van],[w_van],[w_van],[w_van],[w_van],[w_van]]
            e_van=[[e_van],[e_van],[e_van],[e_van],[e_van],[e_van]]
        ENDIF ELSE IF nvs NE nspectra THEN BEGIN
            s = 'vnorm: Error - no. of V spectra must be 1,2 or 6 times no. of w_in spectra'
            i = WIDGET_MESSAGE(s, /ERROR)
            GOTO, fin
        ENDIF
        zeroed = WHERE(e_in LE -1.,nz) ; zeroed spectrum numbers
        IF iprint THEN PRINT,nz/nchannels,' zeroed spectra'
    ENDIF ELSE IF nruns EQ 1 AND (sv[0] NE sw[0]) OR (sv[1] NE sw[1]) OR (sv[2] NE sw[2]) THEN BEGIN
        s = 'vnorm: Error - w_in and V data not same format'
        i = WIDGET_MESSAGE(s, /ERROR)
        GOTO, fin
    ENDIF

    IF iprint THEN PRINT,'nchannels=',nchannels,' nspectra=',nspectra, 'nruns=',nruns

; ------------------------------------------------------------------------------------
;    Perform normalisation

;    FOR D7: reform vanadium into (nchannels, nspectra, nphases)
;    and sum over phases before normalisation

    IF inst EQ 'D7' THEN BEGIN
        wv_buf = REFORM(w_van,nchannels,nspectra/nphases,nphases)
        ev_buf = REFORM(e_van,nchannels,nspectra/nphases,nphases)
        V_buf  = TOTAL(wv_buf[ch1-1:ch2-1,*,*],1)         * L2/vchw ;* DW_van ; ARW: L2/vchw added SR: DW_Van added
        dV_buf = SQRT(TOTAL(ev_buf[ch1-1:ch2-1,*,*]^2,1)) * L2/vchw ;* DW_van ; ARW: L2/vchw added SR: DW_Van added
        IF nphases GT 1 THEN V1  = TOTAL(V_buf,2)
        IF nphases GT 1 THEN dV1 = SQRT(TOTAL(dV_buf^2,2))
        IF iprint THEN PRINT, 'nphases = ',nphases
        IF nphases EQ 6 THEN BEGIN
            V  = [V1,V1,V1,V1,V1,V1]
            dV = [dV1,dV1,dV1,dV1,dV1,dV1]
        ENDIF ELSE IF nphases EQ 2  THEN BEGIN
            V  = [V1,V1]
            dV = [dV1,dV1]
        ENDIF ELSE BEGIN
            V  = V_buf
            dV = dV_buf
        ENDELSE
    ENDIF ELSE BEGIN
        V  = TOTAL(w_van[ch1-1:ch2-1,*],1)         * DW_van ;* L2/vchw ; ARW: L2/vchw added SR: DW_Van added
        dV = SQRT(TOTAL(e_van[ch1-1:ch2-1,*]^2,1)) * DW_van ;* L2/vchw ; ARW: L2/vchw added SR: DW_Van added
    ENDELSE

    w_buf=FLTARR(nchannels,nspectra,nruns)
    e_buf=FLTARR(nchannels,nspectra,nruns)

    IF nspectra GT 1 THEN BEGIN
        V  = REFORM(V,1,nspectra)
        dV = REFORM(dV,1,nspectra)
        V  = (FLTARR(nchannels)+1.) # V
        dV = (FLTARR(nchannels)+1.) # dV

        FOR irun=0, nruns-1 DO BEGIN
            w_buf[*,*,irun] = w_in[*,*,irun]/V
            IF ierrtype EQ 1 THEN $
                e_buf[*,*,irun] = SQRT((e_in[*,*,irun]/V)^2 + (w_in[*,*,irun]*dV/V^2)^2) $
            ELSE BEGIN
                e_buf[*,*,irun] = e_in/V
                PRINT, 'vnorm:  Absolute errors not calcuated'
            ENDELSE
        ENDFOR
    ENDIF ELSE BEGIN
        w_buf = w_in/V
        IF ierrtype EQ 1 THEN $
            e_buf = SQRT((e_in/V)^2 + (w_in*dV/V^2)^2) $
        ELSE BEGIN
            e_buf = e_in/V
            PRINT, 'vnorm:  Absolute errors not calcuated'
        ENDELSE
    ENDELSE

    w_buf = REFORM(w_buf,nchannels,nspectra,nruns)
    e_buf = REFORM(e_buf,nchannels,nspectra,nruns)

    IF(N_ELEMENTS(Ns) GT 0) THEN BEGIN
        Nsd = Ns*1.E22
        Nv  = 7.197E22
        tv  = FLOAT(tv)
        ts  = FLOAT(ts)
            IF iprint THEN BEGIN
            PRINT, 'Absolute Normalisation:'
            PRINT, 'Ns = ',Nsd
            PRINT, 'Nv = ',Nv
            PRINT, 'ts = ',ts
            PRINT, 'tv = ',tv
            ENDIF

        w_out = ((Nv*tv)/(Nsd*ts))*w_buf*5.08*L2/(4*!pi*chw)
        e_out = ((Nv*tv)/(Nsd*ts))*e_buf*5.08*L2/(4*!pi*chw)
    ENDIF ELSE BEGIN
        w_out = w_buf
        e_out = e_buf
    ENDELSE

    novan = WHERE(V LE 0., n)
    IF n GE 1 THEN BEGIN
        V[novan]=1.
        w_out[novan] = 0.
        e_out[novan] = -1.
    ENDIF

    IF inst EQ 'D7' THEN BEGIN
        IF N_ELEMENTS(zeroed) GT 2 THEN BEGIN
            w_out[zeroed] = 0.
            e_out[zeroed] = -1.
        ENDIF
    ENDIF

    IF iprint THEN PRINT,'End of main section'
; ------------------------------------------------------------------------------------

    if n_elements(datp.e) eq n_elements(e_out) then datp.e=e_out

finished0:
    PRINT,'vnorm: normalised to V data, channels ', $
          STRTRIM(STRING(ch1),2),' to ',STRTRIM(STRING(ch2),2)

    title=datvan.other_tit
    IF (inst EQ 'D7') THEN i=3 ELSE i=4
    IF (STRMID(title,i,1) EQ '#') THEN BEGIN
        n=STRLEN(title)        & title=STRMID(title,i+1,n-i-1)
        i=STRPOS(title,' ')    & numor=STRMID(title,0,i)
    ENDIF ELSE BEGIN
        i= strpos(/reverse_search,title,' ')    & n=STRLEN(title)
        numor=STRMID(title,i+1,n)
        IF (STRPOS(numor,'>') EQ -1) THEN $
            numor=STRTRIM(STRING(LONG(datvan.p[10])),2)
    ENDELSE
    numor = '#'+numor
    IF (N_ELEMENTS(Ns) EQ 0) THEN BEGIN
        s=' -vn('+numor+',min='+STRTRIM(STRING(ch1),2)+',max='+STRTRIM(STRING(ch2),2)+')'
    ENDIF ELSE BEGIN
        tvs=STRTRIM(STRING(tv) ,2)    & tvs = string_round(tvs)
        Nss=STRTRIM(STRING(Nsd),2)    & Nss = string_round(Nss)
        tss=STRTRIM(STRING(ts) ,2)    & tss = string_round(tss)
        abss='tv='+tvs+',Ns='+Nss+',ts='+tss
        s=' -vn('+numor+',min='+STRTRIM(STRING(ch1),2)+',max='+STRTRIM(STRING(ch2),2)+','+abss+')'
    ENDELSE

    datp.other_tit=datp.other_tit+s

    give_datp, datp

finished:
    RETURN, w_out
fin:
    END


