;+  Absorption correction for powder diffraction for annular cylinder
;****************************************************************************
;     By D.Schmitt and B. Ouladdiaf,  J.Appl.Crys.31(1998)620
;     Traduction by D.Richard (ILL 2006)
;****************************************************************************
;    integration par methode de Simpson
;    teta = angle de Bragg *2
;    xmu  = mu * RT
;    RT   = rayon exterieur du cylindre
;    RP   = rayon interieur du cylindre (normalise a RT ==> 0<RP<1)
;    R    = rayon du point courant M
;    FI   = angle polaire de M
;-

pro chimin,RP,RP2,r,cfi,sfi,par,t2t,t22,c2t,s2t,dsig,xl
;+
;** ******
;**
;** RP2=RP^2  cfi=cos(fi)  sfi=sin(fi)  par=sfi-cfi*t2t  dsig=+-1.
;** t2t=tan(teta)  t22=t2t^2 s2t=sin(teta)  c2t=cos(teta)
;-
precis=1.D-14

xm =r*cfi    & ym =r*sfi
xe =-sqrt(1D - ym^2)

if t22 gt 1./precis then begin & xs=xm & ys=sqrt(1D - xs^2)
endif else begin
  rpar = r*par
  disc =(1D + t22 - rpar^2)>0.
  xs   =   (-t2t*rpar + dsig*sqrt(disc))/(1D + t22)
  ys   = xs *t2t+rpar
endelse

xem=xm-xe
xms=sqrt((xs-xm)^2 + (ys-ym)^2)
xl =xem+xms

if RP lt precis then return

;Test of the outgoing beam
ps =abs(-xm * s2t + ym*c2t)
idx=where(ps lt RP)
if idx[0] ge 0 then begin
    ps2 =(xm*xs + ym*ys - r^2)[idx]
    idy=where(ps2 lt -precis)
    if idy[0] ge 0 then begin
        xv = 2.*sqrt(RP2 - ps[idx[idy]]^2)
        xl[idx[idy]] = xl[idx[idy]] - xv ;discard on the outgoing beam path
    endif
endif

;Test of the incoming beam
idx=where((abs(ym) lt RP) and (xm gt 0.))
if idx[0] ge 0 then begin
    xv2= 2.*sqrt(RP2 - ym[idx]^2)
    xl[idx] = xl[idx] - xv2              ;discard on the incoming beam path
endif

xl =xl>0.
end

function cylindcor, theta2,xmui,RP, STEPS=stp
;+
;******* **********
;** User callable
;** Input   theta2=Bragg angles vector (degrees*2)
;**         xmui  =Linear Absorption Coefficient mu*Rext
;**         RP    =Radius ratio: Rint/Rext
;** Outpout Wout  =Correction vector
;-

Wout  =theta2*0.
if (RP ge 1) or (RP lt 0) then return,Wout+1.

pi    =!Dpi      ;radial steps for integration
if n_elements(stp) eq 1 then nf=stp else nf=100.
dr    =(1D -RP)/nf & dr2=.5*dr
df    =(2D *pi)/nf & df2=.5*df
nteta =n_elements(theta2)
xmu   =double(xmui)
tetar =(theta2*pi/180.) mod (2*pi)
idx=where( tetar gt pi) & if idx[0] ge 0 then tetar[idx]=2*pi-tetar[idx]

ifi2=(findgen(nf+1))*df
ifi1=ifi2[1:*]      -df2
cfi1=cos(ifi1)  & cfi2=cos(ifi2)
sfi1=sin(ifi1)  & sfi2=sin(ifi2)
s2tv=abs(sin(tetar)) & c2tv=cos(tetar)
t2tv=tan(tetar) & t22v=t2tv^2 & RP2=RP^2

for it=0,nteta-1 do begin

  t2t=t2tv[it] & t22=t22v[it] & c2t=c2tv[it] & s2t=s2tv[it]
  xxl =0D  & r=RP & par1=sfi1-cfi1*t2t & par2=sfi2-cfi2*t2t
  if t2t ge 0. then dsig=1. else dsig=-1.

  chimin,RP,RP2,r,cfi2,sfi2,par2,t2t,t22,c2t,s2t,dsig,y2
  chimin,RP,RP2,r,cfi1,sfi1,par1,t2t,t22,c2t,s2t,dsig,y1

  bid2=exp(-xmu*y2) & bid1=exp(-xmu*y1)
  xl2 =r*total((bid2[0:nf-1]+4.*bid1+bid2[1:nf])/6.)

  for ir =1,nf do begin

    xl0=xl2 & r=RP+ ir*dr

    chimin,RP,RP2,r,cfi2,sfi2,par2,t2t,t22,c2t,s2t,dsig,y2
    chimin,RP,RP2,r,cfi1,sfi1,par1,t2t,t22,c2t,s2t,dsig,y1

    bid2=exp(-xmu*y2) & bid1=exp(-xmu*y1)
    xl2 =r*total((bid2[0:nf-1]+4.*bid1+bid2[1:nf])/6.)

    r=r-dr2

    chimin,RP,RP2,r,cfi2,sfi2,par2,t2t,t22,c2t,s2t,dsig,y2
    chimin,RP,RP2,r,cfi1,sfi1,par1,t2t,t22,c2t,s2t,dsig,y1

    bid2=exp(-xmu*y2) & bid1=exp(-xmu*y1)
    xl1 =r*total((bid2[0:nf-1]+4.*bid1+bid2[1:nf])/6.)

    xxl=xxl+(xl0+4.*xl1+xl2)/6.
  endfor

  Wout[it]=xxl

endfor

if RP lt 1. then Wout=Wout*dr/(1.-RP^2)
Wout=Wout*df/pi
if (nf lt 120) and (nteta gt 15) then Wout=smooth(Wout,3,/edge_t)
return, Wout
end
