pro lamp_elements_database_init

  common elements_database, elements_database
  io_utilities
  forward_function lamp_read_csv

end

;+
; :Description:
;    Load the LAMP database.
;    The loaded database is stored into "elements_database" common variable.
;    The database is searched in two alternative paths: the first one is in the HOME directory of the user (as defined by
;    the HOME IDL environment variable) and the second one is the default location defined in LAMP.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 14 Feb. 2011
;    pellegrini, 24 Oct. 2012
;
; :Categories: database
;-
pro load_lamp_elements_database

    common elements_database
        
    if (n_elements(elements_database) ne 0) then return    

    ; The first to search for the databse, the HOME directory.    
    path = filepath('lamp_elements_database.csv',root_dir=getenv('HOME'))

    ; Check that this path exists and if so that it actually store a valid csv file.
    q = query_csv(path)
    
    if (q ne 0) then begin
        ; Read the database.
        elements_database = lamp_read_csv(path)
    endif else begin
        ; The first to search for the databse, the HOME directory.    
        path = filepath('lamp_elements_database.csv',root_dir=sys_dep('GETENV','LAMP_DIR'),$
                         subdirectory=['lamp_mac','periodic_table','rc'])

        ; Read the database.
        elements_database = lamp_read_csv(path)    
    endelse
            
end 

;+
; :Description:
;    Save the LAMP database.
;    The database will be save in the HOME directory of the user (as defined by the HOME IDL environment variable).
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 14 Feb. 2011
;    pellegrini, 24 Oct. 2012
;
; :Categories: database
;-
pro save_lamp_elements_database, filename=filename, database=database, selection=selection

    common elements_database

    ; Build the absolute path.
    if n_elements(filename) eq 0 then filename = filepath('lamp_elements_database.csv',root_dir=getenv('HOME'))

    if n_elements(database) eq 0 then database = elements_database

    if n_elements(selection) eq 0 then selection = indgen(n_elements(database))
    
    ; Write the database to this path.
    lamp_write_csv, filename, database[selection]

end


;+
; :Description:
;    Returns the indexes that matches a query.
;
; :Params:
;    query : in, required, type="str"
;       The query to search the database for.
;       It must involve the database field name.
; 
; :Returns:
;     A list of the indexes of the entries that matches the query.
;     If no match was found, return -1
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 14 Feb. 2011
;    pellegrini, 24 Oct. 2012
;
; :Categories: database
;-
function query_lamp_elements_database, query
  
    common elements_database
    
    ; The elements database is loaded.
    load_lamp_elements_database
    
    ; The names of the database fields.
    tags = strjoin(tag_names(elements_database),'|')
    
    ; The query string is first splitted to isolate each of its words.
    query = strtrim(strsplit(query,/extract),2)
    
    ; Retrieve the words of the query that match one of the database field names.
    matches =  where(stregex(query,tags,/fold_case) ne -1, n_matches)
    
    ; If the query does not contain any database field, return -1.
    if n_matches eq 0 then return, -1
                
    ; Append the actual variable name storing the databse to the words that match the database field names.
    query[matches] = 'elements_database.'+query[matches]
    
    ; The query is converted back to a string.
    query = 'indexes = where('+strjoin(query,' ')+')'
    
    ; The query string is now evaluated to get the indexes of the matches.
    r = execute(query)
     
    ; If no matches was found, return -1
    if r eq 0 then return, -1
    
    ; Return the indexes of the matches.
    return, indexes       
            
end

pro lamp_elements_database
    lamp_elements_database_init
end
