pro parse_compound_init

    strings_utilities

    forward_function search_uppercase_letters, strreplace
    
end

; 
; 
; docformat = 'rst'
;
;+
; :Author: pellegrini
;
; :History:
;    pellegrini, 11 févr. 2011
;-

;+
; :Description:
;    Convert an element string from its database representation 
;    to its internal representation.
;    
;    An element string is the way an element is defined in the LAMP
;    elements database (e.g. C, H, C[14] ...). As the "[", "]" characters
;    can not be used in an IDL structure tag, those has to be converted in 
;    a suitable internal format.
;    
;    The "[" is replaced by "_" while #]" is replaced by "__". 
;
; :Params:
;    element : in, required, type="str"
;       The element string to convert.
;
; :Returns:
;    The internal representation of the element string.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 11 févr. 2011
;
; :Categories: apps, sld_calculator
; 
; :private:
;-
function internal_element_name, element

    element = strreplace(element, '[', '_')
    element = strreplace(element, ']', '_')        

    return, element

end

;+
; :Description:
;    Convert an element string from its internal representation 
;    to its database representation.
;
;    An element string is the way an element is defined in the LAMP
;    elements database (e.g. C, H, C[14] ...). As the "[", "]" characters
;    can not be used in an IDL structure tag, those has to be converted in 
;    a suitable internal format.
;    
;    The "[" is replaced by "_" while "]" is replaced by "__".
;     
; :Params:
;    element : in, required, type="str"
;       The element string to convert.
;
; :Returns:
;    The database representation of the element string.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 11 févr. 2011
;
; :Categories: apps, sld_calculator
;-
function database_element_name, element

    element = strreplace(element, '_', ']')
    element = strreplace(element, '_', '[')        

    return, element
    
end

;+
; :Description:
;    Parses a string defining a compound mixture and convert it
;    into an internal structure representation.
;    
;    A mixture string is the representation of a mixture of compounds.
;    
;    Its formats is C1%p1sepC2%p2sepC3%p3... where:
;        - sep can be ",", ";", ":".
;        - Ci defines the brute formula for compound i. To define a specific 
;          isotope use [] (e.g. C[14]H4 for a C14-methane).
;        - pi defines the proportion in % of compound i in the mixture.
;
; :Params:
;    mixt : in, required, type="str"
;       The input mixture string.
;
; :Returns:
;    An IDL structure storing the compound contents of the mixture. Each compound 
;    contents being stored in turn as another IDL structure storing the atomic 
;    contents of the compound and its proportion in %. 
;     
; :Author: pellegrini
;
; :History:
;    pellegrini, 11 Feb. 2011
;
; :Categories: apps, sld_calculator
;-
function parse_compound_string, mixt

    parse_compound_init

    ; The mixture string is splitted in its differents compounds.
    res = strtrim(strsplit(strcompress(mixt,/REMOVE_ALL), ",:;", /EXTRACT), 2)
        
    ; Detect the % character for each compound.
    pos = strpos(res, "(")
                       
    ; This is the output structure that will store the information about each compound found
    ; in the mixture.
    ; mixStruct = {}
    
    ; This will store the sum of the proportions of each compound. Will be checked to be 100 at the end. 
    totalProp = 0.0
                
    ; Loop over the compounds found in the mixture.
    for i = 0, n_elements(pos) - 1 do begin
                
        ; Case where no proportion was defied for the compound. Set it to 100.0.
        if pos[i] eq -1 then begin
        
            ; The name of the compound.
            cpnd = res[i]
            ; Its proportion.
            prop = 1.0
        
        endif else begin
        
            ; Extract the name of the compound and its proportion from the "Ci%pi" string.
            temp = stregex(res[i], "(.*)\((.+)", /EXTRACT, /FOLD_CASE, /SUBEXPR)
                        
            ; The name of the compound.
            cpnd = temp[1]
            ; Its proportion.
            prop = float(temp[2])
                                                                      
        endelse

        ; Updates the total proportion.        
        totalProp = totalProp + prop
        
        ; Detect the atom contents of the compound.
        ; An atom must start with a capital letter.
        ; Will store the indexes of the atom first (capital) letter in the compound string.
        match = search_uppercase_letters(cpnd)
        if match[0] eq -1 then begin        
            lamp_message, "The compound " + cpnd + " is not a valid compound chemical formula."
            return, -1        
        endif        
        match = [match, strlen(cpnd)]

        ; Creates a compound structute to store the info about the current compound.
        ; The props tags stands stores the proportion of the compound in the mixture.
        ;cpndStruct = create_struct("prop", prop)
        
        ; Loop over the atoms of the compound.
        for j = 0, n_elements(match) - 2 do begin
        
            len = match[j+1] - match[j]
        
            chemFrag = strmid(cpnd, match[j], len)
            
            ; Case where the atoms is a specific isotope (e.g. C[14]).
            if stregex(chemFrag, "^[A-Z][a-z]?\[[0-9]+\][0-9]*$", /BOOL) eq 1 then begin
            
                submatch = stregex(chemFrag, "^([A-Z][a-z]?\[[0-9]+\])([0-9]*)$", /EXTRACT, /SUBEXPR)
                                                                                                                                               
            ; Case where the atoms is not a specific isotope.
            endif else if stregex(chemFrag, "^[A-Z][a-z]?[0-9]*$", /BOOL) eq 1 then begin

                submatch = stregex(chemFrag, "^([A-Z][a-z]?)([0-9]*)$", /EXTRACT, /SUBEXPR)
                
            ; The other cases are considered to be a misformatted atom string.                  
            endif else begin
            
                lamp_message, "The compound " + cpnd + " is not a valid compound chemical formula."
                return, -1                
            
            endelse
                
            if n_elements(submatch) ne 3 then begin
                lamp_message, "The compound " + cpnd + " is not a valid compound chemical formula."
                return, -1                
            endif
                
            ; Case of a single atom (e.g. C of CH4 or N of NH3).
            if strlen(submatch[2]) eq 0 then submatch[2] = 1
                
            ; The elements is extracted from the atom string (e.g. C[14] from C[14]H4)
            element = strtrim(submatch[1],2)
            ; And converted to a string suitable for an IDL structure tag.
            element = internal_element_name(element)
            
            ; The number of atoms of that elements.
            number = fix(submatch[2])
            
            ; The compound structure is updated with the element and its number found in the compound tags.
            ;cpndStruct = create_struct(cpndStruct, element, number)
                                    
            if (n_elements(mixStruct) eq 0) then begin
                mixStruct = create_struct(element, number*prop)            
            endif else begin
                idx = where(tag_names(mixStruct) eq element)
                if (idx eq [-1]) then begin
                    mixStruct = create_struct(mixStruct, element, number*prop)            
                endif else begin
                    mixStruct.(idx) = mixStruct.(idx) + number*prop
                endelse
            endelse

            ;mixStruct = create_struct(mixStruct, cpnd, cpndStruct)
                                        
        endfor

        ; The name of the compound is converted to a string suitable for an IDL structure tag.        
        ;cpnd = internal_element_name(cpnd)
        ;mixStruct = create_struct(mixStruct, cpnd, cpndStruct)
                                                                        
    endfor

    ; Check that the sum of the proportions of each compound in the mixture is 1.    
    if abs(totalProp - 1.0) gt 0.01 then begin
    
        lamp_message, "The compound proportions of " + mixt + " do not sum to 1."
        return, -1
    
    endif
        
    return, mixStruct
            
end
