;+
;Show periodic table, Atoms properties, SLD calculations
;-

pro pt_init
  ;+
  ; :Description:
  ;    Init application.
  ;
  ; :Author: pellegrini
  ;
  ; :History:
  ;    pellegrini, Jan. 2013
  ;
  ; :Categories: apps, database
  ;-

    ; Imports some modules.
    lamp_elements_database
    io_utilities
    gui_utilities
    strings_utilities
    web_utilities
    
    ; Forwarded functions.
    forward_function parse_compound_string
    forward_function lamp_read_csv
    forward_function query_lamp_elements_database

    common pt_layout, pt_layout

    ; The element symbols of the periodic table. 
    pt_layout = [['H'  ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,'He' ]]
    pt_layout = [[pt_layout],['Li' ,'Be' ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,'B'  ,'C'  ,'N'  ,'O'  ,'F'  ,'Ne' ]]
    pt_layout = [[pt_layout],['Na' ,'Mg' ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,'Al' ,'Si' ,'P'  ,'S'  ,'Cl' ,'Ar' ]]
    pt_layout = [[pt_layout],['K'  ,'Ca' ,'Sc' ,'Ti' ,'V'  ,'Cr' ,'Mn' ,'Fe' ,'Co' ,'Ni' ,'Cu' ,'Zn' ,'Ga' ,'Ge' ,'As' ,'Se' ,'Br' ,'Kr' ]]
    pt_layout = [[pt_layout],['Rb' ,'Sr' ,'Y'  ,'Zr' ,'Nb' ,'Mo' ,'Tc' ,'Ru' ,'Rh' ,'Pd' ,'Ag' ,'Cd' ,'In' ,'Sn' ,'Sb' ,'Te' ,'I'  ,'Xe' ]]
    pt_layout = [[pt_layout],['Cs' ,'Ba' ,'La' ,'Hf' ,'Ta' ,'W'  ,'Re' ,'Os' ,'Ir' ,'Pt' ,'Au' ,'Hg' ,'Tl' ,'Pb' ,'Bi' ,'Po' ,'At' ,'Rn' ]]
    pt_layout = [[pt_layout],['Fr' ,'Ra' ,'Ac' ,'Rf' ,'Db' ,'Sg' ,'Bh' ,'Hs' ,'Mt' ,'Ds' ,'Rg' ,'Cn' ,'Uut','Uuq','Uup','Uuh','Uus','Uuo']]
    pt_layout = [[pt_layout],[''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ,''   ]]
    pt_layout = [[pt_layout],[''   ,''   ,''   ,'La' ,'Ce' ,'Pr' ,'Nd' ,'Pm' ,'Sm' ,'Eu' ,'Gd' ,'Tb' ,'Dy' ,'Ho' ,'Er' ,'Tm' ,'Yb' ,'Lu' ]]
    pt_layout = [[pt_layout],[''   ,''   ,''   ,'Ac' ,'Th' ,'Pa' ,'U'  ,'Np' ,'Pu' ,'Am' ,'Cm' ,'Bk' ,'Cf' ,'Es' ,'Fm' ,'Md' ,'No' ,'Lr' ]]

    pt_layout = transpose(pt_layout)
                
end

;+
; :Description:
;    Event handler that opens an element information page.
;
; :Params:
;    event : in, required, type="struct"
;       The event.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2013
;
; :Categories: apps, database
;-
pro pt_open_element, event

    ; Import the lamp font.
    common c_lamp_font
   
    ; Get the element symbol.
    widget_control, event.id, get_value=symb
    
    ; Search whether a page for this element is already opened.
    page = widget_info(event.top, find_by_uname='BASE_'+symb)
        
    ; If so, do nothing.
    if (page ne 0) then return
    
    ; Get the id of the database table widget.
    table = widget_info(event.top, find_by_uname='PT_DATABASE')
    
    ; Get the database.
    widget_control, table, get_value=table
      
    ; Get all the elements that match the selected symbol (natural + isotopes).
    idxs = where(table.symbol eq symb)

    ; If no element matches this symbol, do nothing.
    if (idxs eq [-1]) then return
                
    ; The atom long name corresponding to selected symbol.
    atom = (table[idxs].atom)[0]
    
    ; The url of the selected symbol.
    url = (table[idxs].url)[0]
                          
    ; The indexes of all the entries (mixture + isotopes) that matches the current atom long name.
    isotopes = where(table.atom eq atom)
  
    ; Sorted the index of of the element.  
    idxs = sort(tag_names(table))
    
    ; The names of the columns of the database.
    tagNames = tag_names(table)  

    ; This string array will store the information about the selected symbol.    
    info = ['']
        
    ; Build the information about this element (natural + isotopes). 
    for i = 0, n_elements(isotopes)-1 do begin
        
        name = strsplit(table[isotopes[i]].element, '_', /extract)
        name = strjoin(reverse(name))
            
        info = [info, 'Properties for ' + name + ':']

        for j=0, n_elements(idxs)-1 do begin
            info = [info,string([strlowcase(tagNames[idxs[j]]),(table.(idxs[j]))[isotopes[i]]],format='("    - ",A-20,A50)')]
        endfor

        info = [info, '']
        info = [info, '']
    
    endfor

    ; The id of the tab widget.
    tabs = widget_info(event.top, find_by_uname='PT_TABS')
       
    ; Its geometry.
    geom = widget_info(tabs, /geom)
    
    ; Create a new page for the tabs.
    base = widget_base(tabs, $
                       /column, $
                       title=symb, $
                       scr_xsize=geom.scr_xsize-10, $
                       scr_ysize=geom.scr_ysize-30, $ 
                       uname='BASE_'+symb)

    ; A button that can be used to open the url of the selected symbol.
    label = widget_button(base, $
                          value=symb, $
                          xsize=120, $
                          ysize=80, $
                          /align_center, $
                          font='Times*60*Bold', $
                          uvalue=url, $
                          uname='ELEMENT_URL', $
                          tooltip='Open ' + atom + ' wiki')
    
    ; The text widget that will display the information about the selected symbol.
    text = widget_text(base, $
                       value=info, $
                       /align_center, $
                       /scroll, $
                       font=ft_propor, $
                       scr_xsize=geom.scr_xsize-20, $
                       scr_ysize=geom.scr_ysize-170, $
                       uvalue=info)

    ; The close button for closing the selected symbol page.
    b = widget_button(base, value='Close', xsize=80, ysize=40, /align_center, uvalue=base, uname='CLOSE_ELEMENT')
     
end

;+
; :Description:
;    Event handler that closes an element information page.
;
; :Params:
;    event : in, required, type="struct"
;       The event.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2013
;
; :Categories: apps, database
;-
pro pt_close_element, event

    ; Get the page id.    
    widget_control, event.id, get_uvalue=base
    
    ; Destroy it.
    widget_control, base, /destroy

end

;+
; :Description:
;    Event handler that opens an url adress in a browser.
;
; :Params:
;    event : in, required, type="struct"
;       The event.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2013
;
; :Categories: apps, elements database
;-
pro pt_open_url, event

    ; Get the uvalue of the button that will open the url when clicked.
    widget_control, event.id, get_uvalue = url

    ; Open the url adress.
    open_url, url

end

;+
; :Description:
;    Event handler that opens the about dialog.
;
; :Params:
;    event : in, required, type="struct"
;       The event.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2013
;
; :Categories: apps, elements_database
;-
pro pt_open_about_dialog, event
    
    ; Opens the about dialog.
    periodic_table_about, group_leader = event.top

end

;+
; :Description:
;    Event handler that quit the application.
;
; :Params:
;    event : in, required, type="struct"
;       The event IDL structure. 
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2011
;
; :Categories: apps, database
;-
pro pt_quit, event

  ; Ask for confirmation before closing the window.
  q = dialog_message('Do you really want to quit ?', /center, /default_no, dialog_parent=event.top, /question, title="Quit")
                  
  ; If the user confirmed the closing, destroy the main window.
  if strlowcase(q) eq 'yes' then widget_control, event.top, /destroy

end

;+
; :Description:
;    Event handler that highlight the entry of the database that match a given query.
;
; :Params:
;    event : in, required, type="struct"
;       The event IDL structure. 
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2011
;
; :Categories: apps, database
;-
pro pt_highlight_query, event

    ; Get the query text.
    widget_control, event.id, get_value=query

    ; Search for the entries of the database that match the query.
    idxs = query_lamp_elements_database(query)

    ; If the query triggered no match, do nothing.
    if (idxs eq [-1]) then return

    ; The database table widget.
    table = widget_info(event.top, find_by_uname='PT_DATABASE')

    ; Set the selection corresponding to the matches.
    selection = make_array(2,n_elements(idxs),/integer,value=-1)
    selection[1,*] = idxs

    ; Highlight the selection in the table.
    widget_control, table, set_table_select=selection

end

;+
; :Description:
;    Event handler that save the database in a csv file.
;
; :Params:
;    event : in, required, type="struct"
;       The event IDL structure.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2011
;
; :Categories: apps, elements database
;-
pro pt_database_save, event

    ; Import the elements_database common.
    common elements_database, elements_database

    ; Get the uname of the widget that sent the event.
    uname = widget_info(event.id, /uname)
     
    ; If this is the File->Save database menu item, save the database to its default place.
    if (uname eq 'MENU_DATABASE_SAVE') then begin
        
        filename = filepath('lamp_elements_database.csv',root_dir=getenv('HOME'))
        
    ; If this is the File->Save database as ... menu item, save the database to a user defined place.
    endif else if (uname eq 'MENU_DATABASE_SAVEAS') then begin  
        
        ; Open a browser to select a file writing.
        filename = dialog_pickfile(Default_extension='.csv', dialog_parent = event.top, $
                                   file = 'lamp_elements_database.csv', title = 'Save database as ...', /write)
    
        ; If no file is selected, just return.
        if strlen(filename) eq 0 then return    

    endif else begin

        return

    endelse

    ; The id of the database table widget.
    id = widget_info(event.top, find_by_uname='PT_DATABASE')

    ; Get the database.
    widget_control, id, get_value=table
            
    ; The whole database is save to a file.
    save_lamp_elements_database, filename=filename, database=table

    ; Replace the elements_database by the new one.
    elements_database = table
    
end

;+
; :Description:
;    Event handler that calculate some molecular properties using the database.
;
; :Params:
;    event : in, required, type="struct"
;       The event IDL structure.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2011
;
; :Categories: apps, elements database
;-
pro pt_calc_molecular_properties, event

    ; The id of the database table widget.
    id = widget_info(event.top, find_by_uname='PT_DATABASE')

    ; Get the database.
    widget_control, id, get_value=table

    ; Get the ids of the all molecular properties widgets.
    wid_vm        = widget_info(event.top, find_by_uname='PT_MOLECULAR_VOLUME')   
    wid_sld       = widget_info(event.top, find_by_uname='PT_SLD')   
    wid_xs_coh    = widget_info(event.top, find_by_uname='PT_XS_COH')
    wid_xs_inc    = widget_info(event.top, find_by_uname='PT_XS_INC')
    wid_xs_abs    = widget_info(event.top, find_by_uname='PT_XS_ABS')
    wid_sigma_scat= widget_info(event.top, find_by_uname='PT_SIGMA_SCAT')
    wid_sigma_coh = widget_info(event.top, find_by_uname='PT_SIGMA_COH')
    wid_sigma_inc = widget_info(event.top, find_by_uname='PT_SIGMA_INC')
    wid_sigma_abs = widget_info(event.top, find_by_uname='PT_SIGMA_ABS')
    wid_thickness = widget_info(event.top, find_by_uname='PT_THICKNESS')
    wid_message   = widget_info(event.top, find_by_uname='PT_MESSAGE_CALCULATOR')
    
    ; Initialize their contents to NaN
    widget_control, wid_sld       , set_value='NaN'
    widget_control, wid_xs_coh    , set_value='NaN'
    widget_control, wid_xs_inc    , set_value='NaN'
    widget_control, wid_xs_abs    , set_value='NaN'
    widget_control, wid_sigma_scat, set_value='NaN'
    widget_control, wid_sigma_coh , set_value='NaN'
    widget_control, wid_sigma_inc , set_value='NaN'
    widget_control, wid_sigma_abs , set_value='NaN'
    widget_control, wid_thickness , set_value='NaN'
    widget_control, wid_message   , set_value=' '

    ; Gets the id of the current compound widget.
    id = widget_info(event.top, find_by_uname='PT_CURRENT_COMPOUND')
     
    ; And its value.
    widget_control, id, get_value=compound & compound=strcompress(compound,/remove_all)
     
    ; Parse the compound string.
    s = parse_compound_string(compound)

    ; If the results of the parsing is not a valid structure, do nothing.
    if (size(s,/TYPE) ne 8) then begin
        widget_control, wid_message  , set_value='Invalid compound/mixture definition.'
        return
    endif
   
    ; Get the id of the density entry widget.
    id = widget_info(event.top, find_by_uname='PT_DENSITY')
     
    ; And its value.
    widget_control, id, get_value=density
     
    ; Cast it to a float.
    density = float(density)
   
    ; Get the id of the wave length entry widget.
    id = widget_info(event.top, find_by_uname='PT_WAVE_LENGTH')
     
    ; And its value.
    widget_control, id, get_value=wavel
     
    ; Cast it to a float.
    wavel   = float(wavel)

    ; Initialize the molecular properties values.
    prop   = [0.0]
    vm     = [0.0]
    b_coh  = [0.0]
    xs_abs = [0.0]
    xs_coh = [0.0]
    xs_inc = [0.0]

    ; List of the elements found in the compound strings.
    elements = tag_names(s)
    ; Loop over the elements of the compound string.
    for i=0, n_tags(s)-1 do begin
        idx = where(strupcase(table.element) eq elements[i])
        ; If the elemet is unknown, stop the calculation.
        if (idx eq [-1]) then begin
            widget_control, wid_message  , set_value="The element " + elements[i] + " is not stored in the database."
            return
        endif
        ; Get the atomic properties corresponding to the current element.
        prop   = [prop,   s.(i)] 
        vm     = [vm,     table[idx].atomic_weight]
        b_coh  = [b_coh,  table[idx].b_coherent] 
        xs_abs = [xs_abs, table[idx].xs_absorption]
        xs_coh = [xs_coh, table[idx].xs_coherent]
        xs_inc = [xs_inc, table[idx].xs_incoherent]     
    endfor

    ; Compute the molecular volume.
    idx = where(vm eq -9999, cnt)
    if (cnt eq 0) then begin
        vm = 1./.6022 * total(vm*prop)/density 
        widget_control, wid_vm, set_value=strtrim(vm,2)
    endif

    ; Compute the sld.
    idx = where(b_coh eq -9999, cnt)
    if (cnt eq 0) then begin
        val = total(b_coh*prop)/vm
        widget_control, wid_sld, set_value=strtrim(val,2)    
    endif
    
    ; Compute the coherent sigma and xs.
    idx = where(xs_coh eq -9999, cnt)
    if (cnt eq 0) then begin
        val = total(xs_coh*prop)
        widget_control, wid_sigma_coh, set_value=strtrim(val,2)
        val = val/vm
        sigma_tot = [val]
        widget_control, wid_xs_coh, set_value=strtrim(val,2)    
    endif

    ; Compute the incoherent sigma and xs.
    idx = where(xs_inc eq -9999, cnt)
    if (cnt eq 0) then begin
        val = total(xs_inc*prop)
        widget_control, wid_sigma_inc, set_value=strtrim(val,2)    
        val = val/vm
        sigma_tot = [sigma_tot,val]
        widget_control, wid_xs_inc, set_value=strtrim(val,2)    
    endif

    ; Compute the scattering sigma (coherent+incoherent)
    idx = where(xs_inc eq -9999, cnt)
    if (cnt eq 0) then begin
        val = total(sigma_tot) * vm
        widget_control, wid_sigma_scat, set_value=strtrim(val,2)
    endif
    
    ; Compute the absorption sigma and xs.
    idx = where(xs_abs eq -9999, cnt)
    if (cnt eq 0) then begin
        val = total(xs_abs*prop) * wavel / 1.798
        widget_control, wid_sigma_abs, set_value=strtrim(val,2)    
        val = val/vm
        sigma_tot = [sigma_tot,val]
        widget_control, wid_xs_abs, set_value=strtrim(val,2)    
    endif

    ; Compute the sigma tot.
    if (n_elements(sigma_tot) eq 3) then begin
        val = 1.0/total(float(sigma_tot)) * (-alog(.9)) ; 90% transmission
        widget_control, wid_thickness, set_value=strtrim(val,2)    
    endif

    ; Get the id of the previous compound list widget.
    id = widget_info(event.top,find_by_uname='PT_PREVIOUS_COMPOUNDS')

    ; And its values.
    widget_control, id, get_value=cpnds  & cpnds=strcompress(cpnds,/remove_all)

    ; Add the current compound to the previous compounds list if it not already stored in it.
    idx = where(cpnds eq compound[0])        
    if (idx[0] eq -1) then begin
        cpnds = [cpnds,compound]
        widget_control, id, set_value=cpnds
    endif
    
    return
     
end

;+
; :Description:
;    Event handler that select a compound from the previous compounds list and update the current compound entry.
;
; :Params:
;    event : in, required, type="struct"
;       The event IDL structure. 
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2013
;
; :Categories: apps, database
; 
;-
pro pt_select_compound, event

    ; Get the compounds stored in the previous compounds list.
    widget_control, event.id, get_value=compounds
    
    ; Get the id of the current compound text widget.
    id = widget_info(event.top, find_by_uname='PT_CURRENT_COMPOUND')

    ; Set the current compound widget with the selected compound.
    widget_control, id, set_value=compounds[event.index]

    return
    
end

;+
; :Description:
;    Event handler that saves the previous compounds list to an ascii file.
;
; :Params:
;    event : in, required, type="struct"
;       The event IDL structure. 
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2013
;
; :Categories: apps, database
; 
;-
pro pt_save_compounds_list, event

    ; The id of the previous compounds list widget.
    id = widget_info(event.top, find_by_uname="PT_PREVIOUS_COMPOUNDS")

    ; Get the compounds list.
    widget_control, id, get_value=compounds

    ; If there is no compound, do nothing.
    if n_elements(compounds) eq 0 then return

    ; Open a browser to select a compounds file.
    filename = dialog_pickfile(Default_extension='.dat', dialog_parent = event.top, $
                               title = 'Save compounds list', /write, /overwrite_prompt)

    ; If no file is selected, just return.
    if strlen(filename) eq 0 then return

    ok = 0
    
    on_ioerror, exit

    ; Open the compound file for writing.
    openw, unit, filename, /get_lun

    for i=0, n_elements(compounds)-1 do begin
        ; Write the list of current list of compounds.
        printf, unit, compounds[i]
    endfor


    ok = 1

    exit:
        ; If something went wrong when writing the file, throw a lamp error.
        if not ok then lamp_error  

        ; Free the logical unit of the compounds file.
        free_lun, unit

end

;+
; :Description:
;    Event handler that loads the previous compounds list to an ascii file.
;
; :Params:
;    event : in, required, type="struct"
;       The event IDL structure. 
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2013
;
; :Categories: apps, database
; 
;-
pro pt_load_compounds_list, event

    ; Open a browser to select a compounds file.
    filename = dialog_pickfile(Default_extension='.dat', dialog_parent = event.top, $
                               title = 'Load compounds list', /read)
    
    ; If no file is selected, just return.
    if strlen(filename) eq 0 then return

    ok = 0
    
    on_ioerror, exit

    ; The number of lines of the compounds file.
    n_lines = file_lines(filename)
    
    ; Allocate a string array that will store all the compounds.
    arr = strarr(n_lines)
    
    ; Open the compound file for reading.
    openr, unit, filename, /get_lun

    ; Reads and store its contents in the string array.
    readf, unit, arr
    
    ; The id of the previous compounds list widget.
    id = widget_info(event.top, find_by_uname="PT_PREVIOUS_COMPOUNDS")

    ; Set the previous compounds list with the contents of the file.
    widget_control, id, set_value=arr

    ok = 1

    exit:
        ; If something went wrong when loading the file, throw a lamp error.
        if not ok then lamp_error  

        ; Free the logical unit of the compounds file.
        free_lun, unit

end

;+
; :Description:
;    Event handler that resizes some of the components of the dialog.
;
; :Params:
;    event : in, required, type="struct"
;       The event IDL structure. 
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2013
;
; :Categories: apps, database
; 
;-
pro pt_resize, event

    ; Import the pt_layout common.
    common pt_layout

    ; The new X and Y dimensions of the main window.
    dims = [event.x,event.y]
    
    ; Sort the dimensions in ascending order.
    dims = dims[reverse(sort(dims))]

    ; Set the new actual X and Y size of the main window.
    x_size = max([600, dims[0]])
    y_size = max([600, dims[1]])
    y_size = max([y_size,x_size/1.8])
    n_rows = n_elements(pt_layout[*,0])
    n_cols = n_elements(pt_layout[0,*])
    x_size = n_cols*ceil(x_size/n_cols) + 10
    y_size = n_rows*ceil(y_size/n_rows) + 75

    ; Resize the tab widget.
    tab = widget_info(event.top, find_by_uname='PT_TABS')

    widget_control, event.top, scr_xsize=x_size, scr_ysize=y_size, /realize
    widget_control, tab, scr_xsize=x_size-10, scr_ysize=y_size-75

    ; Get the new dimension of the tab widget.    
    s = widget_info(tab, /geom)

    ; The id of the periodic table page.
    pte_page = widget_info(event.top, find_by_uname='PT_PTE_PAGE')
    
    ; Get all the bases contained in this page.
    pte_bases = widget_info(pte_page, /all_children)

    ; The new size for the buttons of the periodic table. 
    b_size = long(0.98*ceil((s.scr_xsize)/n_cols))

    ; The resize will be silentful.
    widget_control, event.top, update=0
            
    ; Loop over all the lines of the periodic table.
    for i=0, n_elements(pte_bases)-1 do begin
        ; Resize the corresponding base.
        widget_control, pte_bases[i], scr_xsize=s.scr_xsize-10, scr_ysize=b_size
        ; Get all the widgets contained in this base (periodic table button + labels)
        w = widget_info(pte_bases[i], /all_children)
        ; Loop overs the button and labels of this line.
        for j=0, n_elements(w)-1 do begin
            ; Resize the buttons and labels of this periodic table line.
            widget_control, w[j], xsize=b_size, ysize=b_size        
        endfor
    endfor

    ; Resize some components of the sld calculator page.
    w = widget_info(event.top, find_by_uname='PT_DATABASE')
    widget_control, w, scr_xsize=s.scr_xsize-10, scr_ysize=s.scr_ysize-70 
    w = widget_info(event.top, find_by_uname='PT_QUERY')
    widget_control, w, scr_xsize=s.scr_xsize-60 
    w = widget_info(event.top, find_by_uname='PT_PREVIOUS_COMPOUNDS')
    widget_control, w, xsize=s.scr_xsize-160
    w = widget_info(event.top, find_by_uname='PT_CURRENT_COMPOUND')
    widget_control, w, scr_xsize=s.scr_xsize-160 

    ; Resize the elements info pages.
    el_pages = widget_info(tab, /all_children)
    for i=3, n_elements(el_pages)-1 do begin
        widget_control, el_pages[i], scr_xsize=s.scr_xsize-10
        base = widget_info(el_pages[i], /all_children)
        widget_control, base[1], scr_xsize=s.scr_xsize-30
    endfor

    ; Update the dialog display.
    widget_control, event.top, /update
    
end

;+
; :Description:
;    Global handler for all the dialog events that will dispatch incoming event to specific handlers.
;
; :Params:
;    event : in, required, type="struct"
;       The event IDL structure. 
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2011
;
; :Categories: apps, database
;-
pro pt_events, event

    CATCH,stat & if stat ne 0 then return

    ; The event type.
    event_type = tag_names(event, /structure_name)

    ; The uname of the widget that triggered the event.
    uname = widget_info(event.id, /uname)
                        
    ; Case of a resizing event.
    if event_type eq 'WIDGET_BASE' then begin

        pt_resize, event
        
    ; Case of a button event.
    endif else if event_type eq 'WIDGET_BUTTON' then begin
    
        case uname of        
            'CLOSE_ELEMENT'                : pt_close_element, event
            'ELEMENT_URL'                  : pt_open_url, event
            'MENU_ABOUT'                   : pt_open_about_dialog, event
            'MENU_AINST'                   : pt_open_url, event
            'MENU_COMPOUNDS_LOAD'          : pt_load_compounds_list, event
            'MENU_COMPOUNDS_SAVE'          : pt_save_compounds_list, event
            'MENU_DATABASE_SAVE'           : pt_database_save, event
            'MENU_DATABASE_SAVEAS'         : pt_database_save, event
            'MENU_NIST'                    : pt_open_url, event
            'MENU_QUIT'                    : pt_quit, event
            'PT_CALC_MOLECULAR_PROPERTIES' : pt_calc_molecular_properties, event

        else: begin
                 ; The id of the database table widget.
                 id = widget_info(event.top, find_by_uname='PT_DATABASE')
 
                 ; Get the database.
                 widget_control, id, get_value=table

                 idxs = where(table.symbol eq uname)
                 if (idxs ne [-1]) then pt_open_element, event
              end
        
        endcase
 
    ; Case of a drop list event.       
    endif else if event_type eq 'WIDGET_DROPLIST' then begin
    
        case uname of
            'PT_PREVIOUS_COMPOUNDS' : pt_select_compound, event

        else:
        endcase

    ; Case of a keypressed event.
    endif else if event_type eq 'WIDGET_TEXT_CH' then begin

        case uname of
            'PT_QUERY'            : pt_highlight_query, event
            'PT_CURRENT_COMPOUND' : pt_calc_molecular_properties, event

        else:
        endcase
    
    endif
    
end

;+
; :Description:
;    Build all the components of the main dialog.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2013
;
; :Categories: apps, database
;-
pro pt_build_application , group_leader

    ; Import some commons
    common c_lamp_font
    common elements_database, elements_database
    common pt_layout

    ; Set the group_leader to a default value if none provided as input.
    if (n_elements(group_leader) eq 0) then group_leader = 0L

    ; The main base.
    main_base = widget_base(title="The LAMP elements database", /column, /tlb_size_events , resource_name='lampmic' , $
                            space=0, xpad=5, ypad=5, mbar=main_menu, scr_xsize=600, scr_ysize=600, uname='PT_TLB')

    ; The File menu and its buttons.
    file_menu = widget_button(main_menu, value = 'File', /menu)
    but = widget_button(file_menu, value='Save database', uname='MENU_DATABASE_SAVE', accelerator="Ctrl+S")
    but = widget_button(file_menu, value='Save database as', uname='MENU_DATABASE_SAVEAS', accelerator="Ctrl+Alt+S")
    but = widget_button(file_menu, value='Load compounds list', uname='MENU_COMPOUNDS_LOAD', /separator, accelerator="Ctrl+L")
    but = widget_button(file_menu, value='Save compounds list', uname='MENU_COMPOUNDS_SAVE', accelerator="Ctrl+F")
    but = widget_button(file_menu, value='Quit', accelerator="Ctrl+Q", /checked_menu, /separator, uname='MENU_QUIT')

    ; The Help menu and its buttons.
    help_menu = widget_button(main_menu, value='Help', /menu)
    but = widget_button(help_menu, value='About', accelerator="Ctrl+A", uname='MENU_ABOUT')
    res_menu = widget_button(help_menu, /separator, Value='Resources', /menu)                      
    but = widget_button(res_menu, value='NIST elements database', uname='MENU_NIST', uvalue = 'http://www.ncnr.nist.gov/resources/n-lengths/')                      
    but = widget_button(res_menu, value='Atominstitut elements database', uname='MENU_AINST', uvalue='http://www.ati.ac.at/~neutropt/scattering/table.html')

    ; The main tab widget.
    ; *******************
    tab = widget_tab(main_base, uname='PT_TABS', scr_xsize=590, scr_ysize=525)
        
    ; The size of the tab widget.
    s = widget_info(tab, /geom)
    
    ; The number of rows of the periodic table.
    n_rows = n_elements(pt_layout[*,0])

    ; The number of columns of the periodic table.
    n_cols = n_elements(pt_layout[0,*])

    ; Guess an element button size.
    b_size = long(0.98*ceil((s.scr_xsize)/n_cols))





    ; ***********************************
    ; The base storing the periodic table.
    ; ***********************************
    base = widget_base(tab, title='Periodic table   ', /column, /align_center, xpad=0, ypad=0, space=0, uname='PT_PTE_PAGE')

    n_rows = n_rows-1
    n_cols = n_cols-1
                                                             
    ; Loop over the pce rows.        
    for i = 0, n_rows do begin
        
        ; A base storing the buttons and labels in the current row.
        row_base = widget_base(base, /row, /align_center, space=0, xpad=0, ypad=0)

        ; Loop over the pce columns.        
        for j = 0, n_cols do begin
                                                                                    
            ; If the element of the layout is an empty string, create a dummy label widget.
            if strlen(pt_layout[i,j]) eq 0 then begin
            
                l = widget_label(row_base, $
                                 value=pt_layout[i,j], $
                                 /align_center, $
                                 xsize=b_size, $
                                 ysize=b_size, $
                                 font=c_lamp_big, $
                                 /dynamic_resize)
                                 
            ; Otherwise, create a button labeled with the element symbol.
            endif else begin
                        
                b = widget_button(row_base, $
                                  value=pt_layout[i,j], $
                                  uvalue=pt_layout[i,j], $
                                  uname=pt_layout[i,j], $
                                  font=c_lamp_big, $
                                  /dynamic_resize)
                                                                       
            endelse
                        
        endfor
            
    endfor
    
    ; The column names for the table.
    properties = tag_names(elements_database)





    ; *****************************
    ; The base storing the database.
    ; *****************************
    database_page = widget_base(tab, $
                                title='Database   ', $
                                /column, $
                                /align_center, $
                                space=0)
  
    ; A table is created and filled with the database.
    table = widget_table(database_page, $
                         column_labels=properties, $
                         column_width=100, $
                         /resizeable_columns, $
                         /editable, $
                         /scroll, $
                         value=elements_database, $
                         /disjoint_selection, $
                         uname='PT_DATABASE')

    b = widget_base(database_page, /row)
    l = widget_label(b,value="Query ")
    t = widget_text(b, /editable, value='element eq "H"', uname='PT_QUERY', font=ft_propor)





    ; ***********************************
    ; The base storing the sld calculator.
    ; ***********************************
    calc_page = widget_base(tab, $
                            title='Calculator   ', $
                            /column, $
                            space=0)

    ; The previous compounds list widget.
    base = widget_base(calc_page, /column)
    base11 = widget_base(base, /row)
    l = widget_label(base11, value='Previous compounds' , font=ft_propor)
    d = widget_droplist(base11, value=['H[1]2O(0.7),H[2]2O(0.3)'], font=ft_propor, /dynamic_resize, uname='PT_PREVIOUS_COMPOUNDS')    

    ; An empty label for adding extra space.
    l = widget_label(base, value=' ', ysize=10)

    ; The current compound entry widget.
    base12 = widget_base(base, /row)
    l = widget_label(base12, value='Current compound  ' , font=ft_propor)
    t = widget_text(base12 , /editable, font=ft_propor,uname='PT_CURRENT_COMPOUND')

    ; An empty label for adding extra space.
    l = widget_label(calc_page, value=' ', ysize=20)

    ; The density entry widget.
    b = widget_base(calc_page, /row)
    base = widget_base(b, /row)
    l = widget_label(base  , value='Density (g/cm^3)  ' , font=ft_propor)
    t = widget_text(base   , value='1.0', font=ft_propor, xsize=16, /editable, uname='PT_DENSITY')
    l = widget_label(b, value=' ', xsize=30)
    base = widget_base(b, /row)
    l = widget_label(base  , value='Wave Length (A)   ' , font=ft_propor)
    l = widget_text(base   , value='1.798', font=ft_propor, xsize=16, uname='PT_WAVE_LENGTH', /editable)

    ; An empty label for adding extra space.
    l = widget_label(calc_page, value=' ', ysize=20)

    ; The calculate button widget.
    base = widget_base(calc_page, /row, /align_center)
    b = widget_button(base, value='Calculate', xsize=160, ysize=40, font=ft_propor, uname='PT_CALC_MOLECULAR_PROPERTIES')

    ; An empty label for adding extra space.
    l = widget_label( calc_page, value=' ', xsize=500 ,font=ft_b_normal, uname='PT_MESSAGE_CALCULATOR', /align_center)

    ; The molecular volume entry widget.
    b = widget_base(calc_page, /column)
    base = widget_base(b, /row)
    l = widget_label(base   , value='Molecular volume  ' , font=ft_propor)
    l = widget_text( base   , value=''    , font=ft_propor, xsize=16, uname='PT_MOLECULAR_VOLUME')
    l = widget_label(base   , value='A^3' , font=ft_propor)

    ; An empty label for adding extra space.
    l = widget_label(calc_page, value=' ', ysize=15)

    ; The sld entry widget and Sigma Scat.
    b = widget_base(calc_page, /row)
    base = widget_base(b, /row)
    l = widget_label(base  , value='SLD    (1/A^2)    ' , font=ft_propor)
    l = widget_text(base   , value=''   , font=ft_propor, xsize=16, uname='PT_SLD')
    l = widget_label(b, value=' ', xsize=30)
    base = widget_base(b, /row)
    l = widget_label(base  , value='Sigma Scat.(barn) ' , font=ft_propor)
    l = widget_text(base   , value=''   , font=ft_propor, xsize=16, uname='PT_SIGMA_SCAT')

    ; An empty label for adding extra space.
    l = widget_label(calc_page, value=' ', ysize=15)

    ; The coherent XS and sigma entry widgets.
    b = widget_base(calc_page, /row)
    base = widget_base(b, /row)
    l = widget_label(base  , value='Coh. XS (1/cm)    ' , font=ft_propor)
    l = widget_text(base   , value=''   , font=ft_propor, xsize=16, uname='PT_XS_COH')
    l = widget_label(b, value=' ', xsize=30)
    base = widget_base(b, /row)
    l = widget_label(base  , value='Sigma Coh. (barn) ' , font=ft_propor)
    l = widget_text(base   , value=''   , font=ft_propor, xsize=16, uname='PT_SIGMA_COH')

    ; An empty label for adding extra space.
    l = widget_label(calc_page, value=' ', ysize=15)

    ; The incoherent XS and sigma entry widgets.
    b = widget_base(calc_page, /row)
    base = widget_base(b, /row)
    l = widget_label(base  , value='Inc. XS (1/cm)    ' , font=ft_propor)
    l = widget_text(base   , value=''   , font=ft_propor, xsize=16, uname='PT_XS_INC')
    l = widget_label(b, value=' ', xsize=30)
    base = widget_base(b, /row)
    l = widget_label(base  , value='Sigma Inc. (barn) ' , font=ft_propor)
    l = widget_text(base   , value=''   , font=ft_propor, xsize=16, uname='PT_SIGMA_INC')

    ; An empty label for adding extra space.
    l = widget_label(calc_page, value=' ', ysize=15)

    ; The absorption XS and sigma entry widgets.
    b = widget_base(calc_page, /row)
    base = widget_base(b, /row)
    l = widget_label(base  , value='Abs. XS (1/cm)    ' , font=ft_propor)
    l = widget_text(base   , value=''   , font=ft_propor, xsize=16, uname='PT_XS_ABS')
    l = widget_label(b, value=' ', xsize=30)
    base = widget_base(b, /row)
    l = widget_label(base  , value='Sigma Abs. (barn) ' , font=ft_propor)
    l = widget_text(base   , value=''   , font=ft_propor, xsize=16, uname='PT_SIGMA_ABS')

    ; An empty label for adding extra space.
    l = widget_label(calc_page, value=' ', ysize=15)

    ; The thickness entry widgets.
    b = widget_base(calc_page, /row)
    base = widget_base(b, /row)
    l = widget_label(base  , value='Thickness (cm)    '   , font=ft_propor)
    l = widget_text( base  , value=''    , font=ft_propor, xsize=16, uname='PT_THICKNESS')
    l = widget_label(base  , value='for 90% transmission' , font=ft_propor)
    l = widget_label(b, value=' ', xsize=30)

    ; Initial resizing of the dialog.
    pt_resize, {WIDGET_BASE, ID : main_base, TOP : main_base, HANDLER : main_base, X : 650, Y : 600}  

    ; Realize the dialog.
    widget_control, main_base, /realize

    ; Register the dialog.
    xmanager, 'periodic_table', main_base, group_leader=group_leader, /just_reg, event_handler='pt_events'

end

;+
; :Description:
;    Start the application.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, Jan. 2013
;
; :Categories: apps, database
;-
pro periodic_table , group_leader
    ; If the widget is already opened then do nothing.
    if xregistered('periodic_table') then return
        
    ; Init the module.
    pt_init

    ; Load the elements database.        
    load_lamp_elements_database
             
    ; Build the application.                                                                                                
    pt_build_application , group_leader
                                
end
 
