;+
; :Author: pellegrini
;-

function search_uppercase_letters, str
;+
; :Description:
;    Returns the indexes of a string corresponding to upper case letter.
;
; :Params:
;    str : in, required, type="str"
;       The input string to search for capital letters.
;
; :Returns:
;    A 1D integer array storing the indexes of the upper case letters 
;    found in the string. -1 if no upper case letter was found.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 11 févr. 2011
;
; :Categories: core, strings
;-

    ; A lower-case version of 'str' input string.
    lower = strlowcase(str)

    ; The array storing the indexes of the capital letters.
    match = [-1]

    ; Loop over the character of the string.
    for i = 0, strlen(lower) - 1 do begin
        
        ; If the current character is different from its lowerized version then it is a capital letter.
        ; Append its index to the 'match' array.
        if strmid(lower, i, 1) ne strmid(str,i, 1) then match = [match, i]
    
    endfor
    
    ; If match contains just -1 then no capital letter was found. Returns -1.
    if n_elements(match) eq 1 then begin
    
        match = -1
        
    ; Else remove the first dummy element. 
    endif else begin
    
        match = match[1:*]
    
    endelse
    
    return, match

end

;+
; :Description:
;    Replace all occurences of substring 'substr1' with 'substr2'.
;
; :Params:
;    str : in, required, type="str"
;       The input string.
;    substr1 : in, required, type="str"
;       The substring to be replaced.
;    substr2 : in, required, type="str"
;       The new substring.
;
; :Returns:
;    A string where all occurences of substr1 substring have been replaced by substr2 substring.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 11 févr. 2011
;    
; :Categories: core, string
;-
function strreplace, str, substr1, substr2

    ; If the substring to search for is empty, returns the initial string as results.
    if strlen(substr1) eq 0 then return, str

    ; Search for occurences of 'substr1'.
    pos = strpos(str, substr1, /REVERSE_SEARCH)
    
    ; An occurence is found.    
    while pos ne -1 do begin
    
       ; Take the part of the string preceeding the match.
       lead = strmid(str, 0, pos)
       
       ; Take the part of the string following the match.
       trail = strmid(str, pos + strlen(substr1))
              
       ; Build the string where 'substr1' has been replaced by substr2.
       str = lead + substr2 + trail
       
       ; Search for a new occurence of 'substr1'.
       pos = strpos(strmid(str,0, strlen(lead)), substr1, /REVERSE_SEARCH)            
    
    endwhile
    
    ; Returns the string where all 'substr1' has been replaced by 'substr2'.    
    return, str

end

;+
; :Description:
;    Convert number to string as printed.
;
; :Params:
;    value : in, required, type="any"
;
; :Returns:
;    The string resulting from the conversion.
;
; :Author: unknown
;
; :History:
;    unknown, 11 févr. 2011
;    
; :Categories: core, string
;-
function valtostr, value

  return, strcompress(string(value, /PRINT), /REMOVE_ALL)
  
end

;+
; :Description:
;    Separate string into a 1D array of component strings
;    (separator is ',' or '+', keeping the separator at the front of each string)
;
; :Params:
;    original : in, required, type="str"
;       The input string.
;
; :Returns:
;    The 1D array resulting from the splitting of the input string.
;    
; :Author: unknown
;
; :History:
;    unknown, 11 févr. 2011
;    
; :Categories: core, string
;-
function strtoarr, original

  w = where((byte(original) eq (byte(','))[0]) or (byte(original) eq (byte('+'))[0]), count)
  arr = strarr(count + 1)
  spos = strlen(original)
  
  for i = 0, count - 1 do begin
  
    arr[count - i] = strmid(original, w[count - i - 1], spos - w[count - i - 1])
    spos = w[count - i - 1]
    
  endfor
  
  arr[0] = strmid(original, 0, spos)
  
  return, arr
  
end

;+
; :Description:
;    Capitalize the first letter of each element of an array of string.
;
; :Params:
;    strs : in, required, type="strarr"
;       The input array of strings.
;
; :Keywords:
;    lowerize : in, optional, type="int"
;       If set to 1. The rest of the string will be lowerized.
; 
; :Returns:
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 11 févr. 2011
;
; :Categories: core, strings
;-
function strcap, strs, Lowerize = Lowerize

    ; If strs is not defined, return -1.
    if n_elements(strs) eq 0 then return, -1
    
    ; If lowerize keyword is set, lowerized the string case.
    if keyword_set(lowerize) then strs = strlowcase(strs)

    ; Loop over each element of the array if strings.
    for i = 0, n_elements(strs) - 1 do begin
    
        temp = strs[i]
    
        ; The index of the first letter in the string.
        pos = stregex(temp, "[A-Z]", /FOLD_CASE)
            
        ; If the string does not contain any letter, returns it.    
        if pos eq -1 then continue
    
        ; Upperize the letter.
        letter = strupcase(strmid(temp, pos, 1))
            
        ; And replace the letter by it upperized version.
        strput, temp, letter, pos
        
        strs[i] = temp
        
    endfor
   
    ; Returns the capitalized string. 
    return, strs
   
end

;+
; :Description:
;    This function convert a string representing a complex number written in the form a + bi 
;    to the IDL complex number internal format.
;
; :Params:
;    str: in, required, type="str"
;       The input string representing the complex number to convert. 
;       MUST be in the form 'a + bi'
;
; :Returns:
;     The IDL complex number corresponding to the input string.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 11 févr. 2011
;
; :Categories: core, strings
;-
function str_to_complex, str

    ; The input string is first stripped of all its spaces.    
    str = strcompress(str,/REMOVE_ALL)
    
    ; The regular expression for the real part of the complex number.
    real_part = '[-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?'
    
    ; The regular expression for the imaginary part of the complex number.
    imag_part = '[-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?i'    
        
    ; Search for the imaginary part of the complex number.
    imag_pos = stregex(str, imag_part)
    
    ; Case of a pure real number.
    if imag_pos eq -1 then begin
    
        real = stregex(str, real_part, /Extract)
        
        imag = 0.0
    
    endif else begin
    
        ; The imaginary part is extracted from the complex number string.
        imag = strmid(str, imag_pos)
        
        ; and the rest is used as the real part.
        real = stregex(strmid(str, 0, imag_pos), real_part, /Extract)
    
    endelse
                          
    ; store the real and imaginary parts within a IDL complex type.
    z = dcomplex(real, imag)
    
    return, z

end

;+
; :Description:
;    Convert an IDL complex number to a complex string of the form a + bi.
;
; :Params:
;    complexes : in, required, type="Complex array"
;        The array of IDL complex number to convert to an array of complex string.
;
; :Keywords:
;    Format : in, optional, type="format str"
;        The format for the real and imaginary part of the complex.
;        Default is '(F8.4)'.
;
; :Returns:
;    An array of complex strings. 
;    If a single complex number is given as input return a single string.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 15 févr. 2011
;
; :Categories: core, strings
;-
function complex_to_str, complexes, format = format
    
    ; An array of IDL complexes must be provided as input.
    if n_elements(complexes) eq 0 then begin    
        print, "You must provide an array of IDL complexes."
        return, -1        
    endif
    
    if (size(complexes, /type) ne 6) and (size(complexes, /type) ne 9) then begin
        print, "The input is not a array of IDL complexes"
        return, -1    
    endif
    
    ; Defines a default value for format keyword if none was provided.
    if n_elements(format) eq 0 then format = '(F12.4)'
    
    ; The output array of string.
    strs = [""]
    
    ; Loop over the array complexes.
    for i = 0, n_elements(complexes) -1 do begin
    
        ; Case of zero.
        if complexes[i] eq 0 then begin
            strs = [strs, strtrim(string((0), Format = format),2)]
            continue        
        endif
    
        complexStr = ""
    
        ; The real part of the complex.            
        real = real_part(complexes[i])
    
        ; If the real part is non-null, converts it to a string.
        if real ne 0 then begin    
            realStr = strtrim(string((real), Format = format),2)        
            if strmid(realStr, 0, 1) eq '+' then realStr = strmid(realStr, 1)        
            complexStr = complexStr + realStr        
        endif
                    
        ; The real part of the complex.            
        imag = imaginary(complexes[i])
        
        ; If the imaginary part is non-null, converts it to a string.
        if imag ne 0 then begin     
            imagStr = strtrim(string(imag, Format = format),2) + "i"    
            if strmid(imagStr, 0, 1) ne '-' then imagStr = "+" + imagStr            
            complexStr = complexStr + imagStr            
        endif
                
        ; The complex string is appended to the output array.
        strs = [strs, complexStr]
        
    endfor
    
    ; The first dummy element is removed.
    strs = strs[1:*]
                                       
    return, strs
    
end

function str_array_split, array, separator=separator, reform=reform

    ; The error catcher
    catch, err
        
    ; If an error occured somewhere in the procedure, return -1.
    if err ne 0 then return, -1

    ; If no separator provided, set the default one.
    if (not keyword_set(separator)) then separator = ","
    
    ; If the input array is just a value, cast it to an array.
    if ((size(array))[0] eq 0) then array = [array]
    
    values = strarr(1)
    for i = 0, n_elements(array)-1 do begin   
        values = [values, strsplit(array[i], separator, /EXTRACT)]
    endfor
        
    ; If no match was found, return -1.
    if (n_elements(values) eq 1) then return, -1
    
    values = values[1:*]

    ; If the keyword REFORM is set, the output will be a 2D array of string.
    if (keyword_set(reform)) then begin
        nRows = n_elements(array)
        nCols = n_elements(values)/n_elements(array)         
        values = reform(values, nCols, nRows)
    endif
        
    return, values
    
end

function str_centerize, str, siz

    s = strlen(str)

    if (s gt siz) then return, str

    extra = (siz - s)/2
    
    new_str = strjoin(make_array(siz,/STRING,VALUE=' '))
            
    strput, new_str, str, extra
    
    return, new_str

end

;+
; :Description:
;    The module constructor.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 18 Feb. 2011
;
; :Categories: strings utilities
;-
pro strings_utilities

end
