;** *********
pro radial_AT, M, CENTER=xy0, INCREMENT=incr, MASK=masq, PixSize=xypsize, ratioXY=ratioxy, $
               input_weight=in_e, sector=sector, AZIMUTAL=azimut, thetrad=thetrad, atanGG=atanGG, $
               weight_mode=in_t, DIAGRAM=intensity, X=xx, ERROR=error, UNROLL=mat, UX=ux, $
               Dangle=dangle, distance=dst, Raxe=Raxe, minInt=minInt
;+
;** *********
;** User callable
;**
;** Radial integration:   "Aurelie Tallandier & D.R. July 2001 Grenoble".
;** ******************
;**     Input:  M      =2D or 3D data matrix.
;**     *****   xy0    =[x0,y0] center coordinates (default [n,m]-1 /2.)
;**             incr   =Radius increment in pixels units     (default  1.0)
;**             masq   =same as M(n,m), contains 1's for masked values otherwise 0's(default)
;**             xypsize=size of the pixel in Cm. if present will determine ratioXY.
;**             ratioXY=[1. , Yunit/Xunit] if Xunit>Yunit
;**             ratioXY=[Xunit/Yunit , 1.] if Yunit>Xunit.   Default is [1,1] or f(xypsize).
;**             in_e   =input weights of counts (M/E^2 or 1/E^2 or 1/E). Default is  1.
;**             in_t   =weighting mode           1        2        3     Default is  0.
;**             sector =[inner_radius, outer_radius, angle1_deg, angle2_deg, squared, totsec] (0->360)
;**             dangle =detector angle   (deg)
;**             dst    =detector distance (cm)
;**             Raxe   =rotation axe      (cm)
;**             thetrad=2theta matrix to use instead of increment pixels (M is not a flat image detector)
;**             minInt =minimum pixels to integrate, default is 1
;**
;**     Output: intensity =result of Radial integration.
;**     ******  xx        =diagram abscissa: radius in pixel unit.
;**             error     =diagram errors
;**             Mat,ux    =Unrolled  matrix if Unroll keyword is present.
;-

;** Updates:   "Didier Richard February 2005".
;** *******
;**             Accept a 3D input matrix M.
;**             UNROLL=mat, UX=ux

;**             Add ratioXY and xyPixSize "D R Jul 2005".

;**             Add input_weight          "D R May 2006".

;**             Add input sector          "D R Jul 2007".

;**             Add azimutal keyword      "D R Jul 2007".

;**             Add Dangle,distance kword "D R Oct 2007".

;**             Add thetrad keyword       "D R Mar 2010".

;**             Accept 3D weights         "D R Jul 2010".
;**             Add weight_mode keyword

;**             Add squared sector        "D R Sep 2011".
;**             Add totsec  sector        "D R Dec 2011".

;**             Add minInt                "D R Jul 2013".

common c_rad_at, RR,xc,yc,rmax,rmin
common c_tan_at, GG
common c_rat_at, ratio

;*****************************CHECK FOR PARAMETERS******************************
s=size(M)   & s12= s[1]* s[2]
IF (s[0] lt 2) or (s[1] lt 2) or (s[2] lt 2) THEN RETURN
IF (s[0] eq 3) then nz=s[3] else nz=1
Unit=1.
IF n_elements(xy0)     ne 2 then xy0    =([s[1],s[2]]-1.)/2.
IF n_elements(incr)    ne 1 then incr   =1.0
IF n_elements(xyPsize) eq 2 then begin &  Unit=float(max(xyPsize,min=mu))
                                 if xyPsize[0] eq mu then ratioxy=[mu/Unit,1.] $
                                                     else ratioxy=[1.,mu/Unit] & endif
IF n_elements( ratioxy)  ne  2  then ratioxy=[1.,1.]
IF n_elements( in_e)     ne s12 then IF n_elements( in_e) ne n_elements(M) then in_e=M[*,*,0]*0.+1.
IF n_elements( in_t)     ne  1  then in_t=0
IF n_elements( sector)   ge  4  then sect=1 else sect=0
IF n_elements( sector)   ge  5  then squared=sector[4] else squared=0 ;1 or 0
IF n_elements( sector)   ge  6  then totsec =sector[5] else totsec =0 ;1 or 0
IF n_elements( Dangle)   ne  1  then Dangle=0.
IF n_elements( dst)      ne  1  then dst =1.
IF n_elements( Raxe)     ne  1  then Raxe=0.
IF n_elements(minInt)    ne  1  then minInt=1
IF (size(in_e))[0]       eq  3  then E3d =1 else E3d =0
IF keyword_set(azimut)          then azim=1 else azim=0


if not keyword_set(azim) then incr=abs(incr) <(s[1]/2) <(s[2]/2) > 0.01 $
                         else incr=abs(incr) <180.               > 0.01

xo  =xy0[0]  & yo=xy0[1] & ok=1
IF n_elements(RR) eq s12 then if (xo eq xc) and (yo eq yc)  and $
  (ratioxy[0] eq ratio[0]) and (ratioxy[1] eq ratio[1]) and (ratio[2] eq dangle) and (ratio[3] eq dst) then ok=0

;*****************************MAKE MATRIX OF RADIUS*******************************
IF n_elements(thetrad) gt 1 then begin
   RR  =thetrad*dst/min(xypsize)
   rmax=MAX(RR,min=rmin)
   GG  =atanGG ;thetrad
   unit=1.
   xc=xo & yc=yo & ratio=[ratioxy,dangle,dst]
endif else $
IF ok then BEGIN
        xc=xo & yc=yo & ratio=[ratioxy,dangle,dst]

        x =findgen(s[1])+.5
        x =[[x],[x]]
        x =(congrid(x,s[1],s[2])-xo)*ratio[0]

        y =transpose(findgen(s[2])+.5)
        y =(congrid(y,s[1],s[2])-yo)*ratio[1]

        RR=sqrt(x^2+y^2) ;RADIUS

        if abs(ratio[2]) ge 1. then begin & alpha=abs(ratio[2])*!pi/180. ;We have a detector angle.
            a   =abs(xo-s[1]/2.+.5)*ratio[0]*sin(alpha)*Unit ;Use center offset
            cosa=cos(alpha)
            r   =Raxe*(1.-cosa)
            D   =ratio[3] & Dra=D+r+a
            if ratio[2] gt 0. then sgn=[-1,1] else  sgn=[1,-1] ;alpha > 0   !!
            if xo gt s[1]/2.  then sgn=[-1,1] else  sgn=[1,-1] ;=offset < 0 !!
            xoff=findgen(s[1])+.5 & idxl=where(xoff lt xo) & idxr=where(xoff ge xo)
            sina=sin(sgn*alpha)
            if idxl[0] ge 0 then begin & xt=abs(x[idxl,*])*Unit & x[idxl,*]=(xt*Dra*cosa)/(xt*sina[0]+Dra)/Unit & endif
            if idxr[0] ge 0 then begin & xt=abs(x[idxr,*])*Unit & x[idxr,*]=(xt*Dra*cosa)/(xt*sina[1]+Dra)/Unit & endif
            RR=sqrt(x^2+y^2)*D/Dra
        endif
        rmax=MAX(RR,min=rmin)
        GG  =(atan(y,x))
ENDIF

;*****************************MASK**********************************************
R= RR & cmax=rmax & cmin=rmin & em9=0.
IF n_elements(masq) eq s12 then begin
   if sect then begin
      em9=-1.e-9
      rRR=round(RR) & rGG=round(GG/!pi*180.) & masqq=masq
      ida=where(rGG lt 0.) & if ida[0] ge 0 then rGG[ida]=360.+rGG[ida]
      rr1=min(round(sector[0:1]),max=rr2)    ; Get inner, outer radius
      IF n_elements(thetrad) gt 1 then begin ; We may have an irregular grid of radius
         rr1=max([rRR[(xc+rr1)>0<(s[1]-1),yc>0<(s[2]-1)] , rRR[xc>0<(s[1]-1),(yc+rr1)>0<(s[2]-1)],$
                  rRR[(xc-rr1)>0<(s[1]-1),yc>0<(s[2]-1)] , rRR[xc>0<(s[1]-1),(yc-rr1)>0<(s[2]-1)]])
         rr2=max([rRR[(xc+rr2)>0<(s[1]-1),yc>0<(s[2]-1)] , rRR[xc>0<(s[1]-1),(yc+rr2)>0<(s[2]-1)],$
                  rRR[(xc-rr2)>0<(s[1]-1),yc>0<(s[2]-1)] , rRR[xc>0<(s[1]-1),(yc-rr2)>0<(s[2]-1)]])
        ;print,'Radius ',sector(0),sector(1),' --> ',rr1,rr2
      ENDIF
      idr=WHERE((rRR lt rr1) or  (rRR gt rr2))
      an1=sector[2]  & an2=sector[3] ; RGG IS NOW ONLY POSITIVE IN DEGRES
      if squared then begin
       ;Mask outside a squared sector, should use the code from "int_poly" (Bens.pro)
        if an1 gt an2 then an2=an2+360.
        a1=an1 & a2=(an1+an2)/2. & a3=an2 & a12=(a1+a2)/2. & a23=(a2+a3)/2.
        deg2rad=!pi/180. & r1=rr1 & r2=rr2
        
        temp2=[a3,a23,a2,a12,a1]*deg2rad          ;Same code starts here
        a3a1= a3-a1 & if a3a1 lt 0. then a3a1=360.+a3a1
        dx=( (a3a1*r2/(r1>1.) -a3a1)<90. )/2.
        b1=a1-dx & b12=a12-dx/2. & b23=a23+dx/2. & b3=a3+dx
        temp3=[b1,b12,a2,b23,b3]*deg2rad
        temp3=[b1,           b3]*deg2rad ; Modification for Bruno D...
        temp2=[a3,           a1]*deg2rad ; Modification for Bruno D...
        x=[r1*cos(temp3),r2*cos(temp2),r1*cos(b1*deg2rad)]
        y=[r1*sin(temp3),r2*sin(temp2),r1*sin(b1*deg2rad)]
   
        Tmasq =masqq*0 +1
        idxsqr=polyfillv(round(xc+x),round(yc+y),s[1],s[2])
        if idxsqr[0] ge 0 then Tmasq[idxsqr]=0 & ida=where(Tmasq)
      endif else begin
       ;Mask the angles not concerned using rGG matrix.
        if an1 gt an2 then ida=WHERE((rGG lt an1) and (rGG gt an2))
        if an1 le an2 then ida=WHERE((rGG lt an1) or  (rGG gt an2))
      endelse
      if idr[0] ge 0 then masqq[idr]=1
      if ida[0] ge 0 then masqq[ida]=1 & idm=WHERE(masqq)
   endif else idm= WHERE(masq)

endif else idm=[-1]

if azim then begin
   R=GG & idd=where(R lt 0.) & R[idd]=2*!pi+R[idd] & R=R/!pi*180. ; angles R from 0 to 360 deg
   if n_elements(sector) ge 2 then begin
      rr1 = min(round(sector[0:1]),max=rr2)                       ; Get inner, outer radius
      rRR = round(RR) & idr=WHERE((rRR lt rr1) or  (rRR gt rr2))
      if idr[0] ge 0 then R[idr]=-incr ; Mask radius limits giving a negative value
   endif
   cmax=360. & cmin=0. & Unit=1.
endif

if idm[0] ge 0 then R[idm] = -incr     ; Mask original Mask and/or Sectors giving a negative value

; on calcule le  nombre d'elements de R contenus dans un "cercle" d'epaisseur dr*2
; ce qui nous donne 2 valeurs : un tableau des valeurs qui sont dans R +|-  dr
; et le nombre d'elements de ce tableau. (Idem if R=angles azimutals)
dr  =incr/2.
eps =incr/8.  &  dreps =dr+eps

nx       =round((cmax+dr-cmin)/incr)+1
intensity=fltarr(nx,nz) + em9
xx       =fltarr(nx)
error    =intensity * 0.
IF arg_present(mat) then begin & pi2r=round(!pi*2*rmax+1)
                    mat=fltarr(pi2r,nx,nz) & pat=fltarr(pi2r) & endif
j=0
;*****************************CALCULATE INTENSITY*******************************
FOR i=cmin,cmax+dr,incr DO BEGIN

; where permet de determiner quelles sont les valeurs de la matrice R qui se
; trouvent dans le "cercle" d'epaisseur dr*2. Elle renvoie les indices des valeurs
; correspondantes (indice unique au lieu de l'indicage ligne-colonne)
; Ces indices sont places dans un tableau res. (Idem if R=angles azimutals)
 res=WHERE((R gt (i-dreps)) and (R lt (i+dreps)))

 IF (res[0] ge 0) and (n_elements(res) ge minInt) then begin ;We have unmasked points

; On veut recuperer les valeurs correspondantes aux indices res
; pour chaque i afin de calculer l'intensite => M(res)

; on pondere la valeur de chq element par rapport a sa position dans le pixel
; plus le point est proche du centre du pixel ie de i, plus la fraction du
; pixel utilise sera grande [1.->0.]
  wG  =(dreps -abs(i-R[res]) )/dreps        ;Geometry weighting

  if not totsec then wGt=total(Wg) else begin & wGt=1. & totw=1. & endelse

  if not E3d then begin & e_cur=in_e[res] & w= wG ;* e_cur ;multiply by weights !!!!!!!!!!!!! TODO ******
                        if not totsec then totw = total(w) & endif
  xx[j]= i
  IF arg_present(mat) then begin
         GI=GG*(i<rmax)+!pi*rmax   & GA=floor(GI) & GB=ceil(GI) & PA=1.-GI+GA & PB=(1.-PA)>.001
         GAr=GA[res] & PAr=PA[res] & GBr=GB[res]  & PBr=PB[res] & endif

  FOR z=0,nz-1 DO BEGIN
    off = res+s12*z
    if  E3d  then begin & e_cur=in_e[off] & w= wG ;* e_cur ;multiply by weights !!!!!!!!!!!!! TODO ******
                        if not totsec then totw = total(w) & endif
    totm= total(w*M[off])
   ;if j eq 0 then begin print,w & print,M(off) & print,i,j,dr,eps,ep2 & endif
    if totw ne 0.0 then begin & intensity[j,z]= totm/totw
                              if in_t eq 3 then error[j,z] = sqrt(total( (wG/e_cur)^2        )) /wGt else $ ;weight= 1/E
                              if in_t eq 2 then error[j,z] = sqrt(total( (wG/e_cur)          )) /wGt else $ ;weight= 1/E^2
                              if in_t eq 1 then error[j,z] = sqrt(total( (wG/e_cur * M[off]) )) /wGt $      ;weight= M/E^2
                                           else error[j,z] = sqrt(totm>0.)/totw                             ;weight= unknown
    endif
    IF arg_present(mat) then begin &  pat=pat*0.
       for k=0,n_elements(res)-1 do begin
           mat[GAr[k],j,z]=mat[GAr[k],j,z]+M[off[k]]*PAr[k] & pat[GAr[k]]=pat[GAr[k]]+PAr[k]
           mat[GBr[k],j,z]=mat[GBr[k],j,z]+M[off[k]]*PBr[k] & pat[GBr[k]]=pat[GBr[k]]+PBr[k]
       endfor
       idx=where(pat eq 0.) & if idx[0] ge 0 then pat[idx]=1. & mat[*,j,z]=mat[*,j,z]/pat
    ENDIF
  ENDFOR
  j=j+1
 ENDIF else if sect then begin & xx[j]=i & j=j+1 & endif
ENDFOR
xx=xx*Unit
;*****************************RETURN PARAMETERS********************************
IF (j ne n_elements(xx)) and (not sect) then begin & j=j-1
        intensity=intensity[0:j,*]
        error    =error[0:j,*]
        xx       =xx[0:j]
        IF arg_present(mat) then Mat=Mat[*,0:j,*]
ENDIF

IF arg_present(mat) then begin & mat=reform(mat) & ux=findgen(pi2r)-pi2r/2 & endif
intensity=reform(intensity)
error    =reform(error)

END
