;+
;
; :Description:
; 
;    Application to read and bin an event mode file.
;    
;    The data format is instrument dependent.
;    It consists of a common header containing (4 + N_boards) 32-bit words:
;        
;        Word 1 = Version
;        Word 2 = Time base
;        Word 3 = Number of events registered
;        Word 4 = Number of boards (N_boards)
;        Words 4+1 to 4+N_boards = Info about each board (not read)
;        
;    Then it follows the list of events, stored using 4 x 32 bits.
;    The format of each event depends on the card type, so this will be
;    instrument and even experiment dependent.
;    
;    Cases handled:
;    
;    1. D22 with 4 x 1740 QDIV (detector) + a GGPickup (monitor)
;    
;        Word 1 --> Crate                ( 4 bits: 31-28)
;                   Board                ( 6 bits: 27-22)
;                   Channel              ( 6 bits: 21-16)
;                   Timestamp bits 47-32 (16 bits: 15-0)
;                   
;        Word 2 --> Timestamp bits 31-0  (32 bits: 31-0) 
;        
;        Word 3 (only for 1740 QDIV) --> X position (5 bits: 20-16)
;                                        Y position (8 bits: 7-0)       
;
;        Word 4 (only for 1740 QDIV) --> A+B (16 bits: 31-16)
;                                        A   (16 bits: 15-0)
;
; :History:
;    M. A. Gonzalez, May 2018 --> Rewrite of event_file for new list mode format
;
;-

;****************************************************************************************
;****************************************************************************************


; External functions/pros used:
;   valid_num(): Check if a string is a valid number representation
;   modify_struc: Modifies a structure

;+
; NAME:
;     VALID_NUM()
; PURPOSE:
;     Check if a string is a valid number representation.
; EXPLANATION:
;     The input string is parsed for characters that may possibly
;     form a valid number.  It is more robust than simply checking
;     for an IDL conversion error because that allows strings such
;     as '22.3qwert' to be returned as the valid number 22.3
;
;     This function had a major rewrite in August 2008 to use STREGEX
;     and allow vector input.    It should be backwards compatible.
; CALLING SEQUENCE:
;     IDL> status = valid_num(string  [,value]  [,/integer])
;
; INPUTS:
;     string  -  the string to be tested, scalar or array
;
; RETURNS
;     status - byte scalar or array, same size as the input string
;              set to 1 where the string is a  valid number, 0 for invalid
; OPTIONAL OUTPUT:
;     value     - The value the string decodes to, same size as input string.
;           This will be returned as a double precision number unless
;           /INTEGER is present, in which case a long integer is returned.
;
; OPTIONAL INPUT KEYWORD:
;    /INTEGER   -  if present code checks specifically for an integer.
; EXAMPLES:
;     (1) IDL> print,valid_num(3.2,/integer)
;        --> 0     ;Since 3.2 is not an integer
;     (2) IDL> str =['-0.03','2.3g', '3.2e12']
;         IDL> test = valid_num(str,val)
;              test = [1,0,1]    &  val =  [-0.030000000 ,NaN ,3.2000000e+12]
; REVISION HISTORY:
;          Version 1, C D Pike, RAL, 24-May-93
;          Version 2, William Thompson, GSFC, 14 October 1994
;                       Added optional output parameter VALUE to allow
;                       VALID_NUM to replace STRNUMBER in FITS routines.
;          Version 3 Wayne Landsman rewrite to use STREGEX, vectorize
;          Version 4 W.L. (fix from C. Markwardt) Better Stregex expression,
;                    was missing numbers like '134.' before Jan 1 2010
;-
FUNCTION valid_num, string, value, INTEGER=integer
  On_error,2
  compile_opt idl2

  ; A derivation of the regular expressions below can be found on
  ; http://wiki.tcl.tk/989

  if keyword_set(INTEGER) then $
    st = '^[-+]?[0-9][0-9]*$'  else $                    ;Integer
    st = '^[-+]?([0-9]+\.?[0-9]*|\.[0-9]+)([eEdD][-+]?[0-9]+)?$' ;F.P.

  ;Simple return if we just need a boolean test.
  if N_params() EQ 1 then return, stregex(strtrim(string,2),st,/boolean)


  vv = stregex(strtrim(string,2),st,/boolean)
  if size(string,/N_dimen) EQ 0 then begin     ;Scalar
    if vv then $
      value= keyword_set(integer) ? long(string) : double(string)
  endif else begin                             ;Array

    g = where(vv,Ng)
    if Ng GT 0 then begin      ;Need to create output vector
      if keyword_set(integer) then begin
        value = vv*0L
        value[g] = long(string[g])
      endif else begin
        value = replicate(!VALUES.D_NAN,N_elements(vv))
        value[g] = double(string[g])
      endelse
    endif
  endelse

  return,vv
end


;+
; :Description:
;    Modifies a structure changing the type, dimensionality, and contents of an
;    existing structure field. It can go inside nested structures.
;
;    The call is
;       > modify_struc, struct, tagkeys, 0, data
;
; :Parameters (all required):
;    struct : in/out, type="structure" - structure to modify.
;    tagkeys: in, type="string" or "string array"  - case insensitive tag name describing
;             the structure field to modify. To modify a field in a structure inside a structure
;             give the ordered list of structure names and the field name at the end, e.g.
;             ['struct1', 'struct2', 'struct3', 'field'].
;    0      : in - needed for recursive calls.
;    data   : in, any type - data that will replace the current contents of the structure field,
;             modifying the type and dimensions of the field.
;
;  :Example:
;    To modify the data in a 'nexus' structure obtained from reading a NeXus file with h5_parse, use
;       > modify_struc, nexus, ['entry0', 'data', 'multidetector_linear_data', '_data'], 0, data
;-
pro modify_struc, struct, tagkeys, ilevel, data

  ;Check that input is a structure.
  if size(struct, /tname) ne 'STRUCT' then begin
    message, 'First argument is not a structure'
  endif

  ;Get list of structure tags.
  tags = tag_names(struct)
  ntags = n_elements(tags)

  ;Check that requested field exists in input structure.
  ctag = strupcase(strtrim(tagkeys, 2))   ;canoncial form of tag
  itag = where(tags eq ctag[ilevel], nmatch)
  if nmatch eq 0 then begin
    message, 'structure does not contain ' + ctag + ' field'
    return
  endif
  itag = itag[0]        ;convert to scalar

  ;Choose tag name for the output structure.
  ;if keyword_set(newtag) then otag = newtag else otag = ctag
  otag = ctag[ilevel]

  ;Copy any fields that precede target field. Then add target field.
  if itag eq 0 then begin

    tmpstruct = struct.(itag)
    if size(tmpstruct, /tname) ne 'STRUCT' then new = create_struct(otag, data) $
    else begin
      modify_struc, tmpstruct, tagkeys, ilevel+1, data
      new = create_struct(otag, tmpstruct)
    endelse

  endif else begin

    new = create_struct(tags[0], struct.(0))  ;initialize structure
    for i=1, itag-1 do begin                  ;insert leading unchange
      new = create_struct(new, tags[i], struct.(i))
    endfor

    ;insert new data
    tmpstruct = struct.(itag)
    if size(tmpstruct, /tname) ne 'STRUCT' then new = create_struct(new, otag, data) $
    else begin
      modify_struc, tmpstruct, tagkeys, ilevel+1, data
      new = create_struct(new, otag, tmpstruct)
    endelse

  endelse
  ;Replicate remainder of structure after desired tag.
  for i=itag+1, ntags-1 do begin
    new = create_struct(new, tags[i], struct.(i))
  endfor

  ;Replace input structure with new structure.
  struct = new

end


;****************************************************************************************
;****************************************************************************************


pro help_eventfile
;+
;** ****************
;** Give brief information about the program
;-

  forward_function finefile

  @lamp.cbk
  info_message = ['Program to read and manipulate ILL event mode (.lst) files.', $
                  'The present version (beta, released on 1/12/2018) is still', $
                  'under development and is able to read only data obtained on', $
                  'D22 from the 3rd cycle of 2018 onwards.', $
                  ' ', $ 
                  'For questions, bug reports or demands for additional functionality', $
                  '(e.g. to read data from other instruments), please contact', $
                  'Miguel A. Gonzalez (gonzalezm@ill.fr).', $
                  ' ', $
                  'A short user guide is available as a PDF file.', $
                  'Do you want to open it now?']

  see_guide = dialog_message(info_message, /question)
  
  if see_guide eq 'Yes' then begin
      guide = finefile(lamp_dir + lamp_dvd + 'manual' + lamp_dvd + 'EVENT_MODE_guide.pdf')
      online_help, book=guide, /full
  endif
  

end


pro event_log, mess
;+
;** ****************
;** Prints information and warning messages and sends them to lamp journal
;-

  @lamp.cbk
  common EV_common1, EV_struct
  
  lamp_journal, mess & lamp_journal, ''
  print, mess & print, ''

end


pro event_info, mess
;+
;** ****************
;** Writes information in corresponding message line on event_mode interface
;-

  @lamp.cbk
  common EV_common1, EV_struct
  event_log, mess
  widget_control, EV_struct.bmess4, set_value=mess
  
end


pro send_to_lamp, wi, data, XVAL=xval, XLABEL=xlabel, YLABEL=ylabel, ZVAL=zval, ZLABEL=zlabel, $
                            TITLE=title, SUBTITLE=subtitle
;+
;** ****************
;** Send data to a given workspace
;-

  @lamp.cbk

  give_w, data, W=wi
  take_datp, dp, W=wi

  if keyword_set(xval) then mod_datp, dp, 'X', xval
  if keyword_set(xlabel) then dp.X_tit = xlabel
  if keyword_set(ylabel) then dp.Y_tit = Ylabel
  if keyword_set(zval) then mod_datp, dp, 'Z', zval
  if keyword_set(zlabel) then dp.Z_tit = zlabel
  if keyword_set(title) then dp.w_tit = title
  if keyword_set(subtitle) then dp.Other_tit = subtitle

  give_datp, dp, W=wi
  see, W=wi

end


function check_file, key, file
;+
;** ****************
;** Check if selected file existst and can be read
;-

  @lamp.cbk
  common EV_common1, EV_struct

  case key of
    
    'file': begin
        if EV_struct.Cpath eq '' then begin
            cd, current = c
            EV_struct.Cpath = c
        endif
        file = EV_struct.Cpath + path_sep() + file
        check = file_test(file)
        if check then return, file else return, ' ' 
    end
    
    'browseW' : begin
        pthI = EV_struct.Wpath
        file = dialog_pickfile(filter='*.lst', path=pthI, get_path=pthO, /read, resource='lamptouch')
        if file gt ' ' then begin
          basename = file_basename(file)
          EV_struct.Wpath = pthO 
          EV_struct.Cpath = pthO
          widget_control, EV_struct.bfile, set_value=basename
          return, file
        endif else return, ' '
    end

    'browseD' : begin
      pthI = path_for_online
      file = dialog_pickfile(filter='*.lst', path=pthI, get_path=pthO, /read, resource='lamptouch')
      if file gt ' ' then begin
        basename = file_basename(file)
        EV_struct.Dpath = pthO
        EV_struct.Cpath = pthO
        widget_control, EV_struct.bfile, set_value=basename
        return, file
      endif else return, ' '
    end
    
    else: event_info, '  Not valid keyword in call to check_file'
    
  endcase

  ;if keyword ne 'expcur' and keyword ne 'explst' and old ne new then return, 2 ; [Input file] modified

  
end


pro initialize_event_arrays
;+
;** ****************
;** Initialize all counters and arrays before reading new file.
;-

  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger
  common EV_common3, out_array, mon_array, time_array, nbrepaints, total_counting_time, ticks, ticks_end

  Counts=0 
  Xdet=0 & Ydet=0 & Dtimer=0
  Ctimer=0 
  Stimer=0 
  Mtimer=0
  Dtimer_trigger=0 
  Mtimer_trigger=0
  
  out_array = 0
  mon_array = 0
  time_array = 0
  nbrepaints = 1
  total_counting_time = 0
  ticks = 0
  ticks_end = 0

end


pro initialize_info_messages
;+
;** ****************
;** Remove all messages about file statistics
;-

  common EV_common1, EV_struct

  Mess1 = '' & Mess2='' & Mess3='' & Mess4=''
  widget_control, EV_struct.bmess1, set_value=Mess1
  widget_control, EV_struct.bmess2, set_value=Mess2
  widget_control, EV_struct.bmess3, set_value=Mess3
  widget_control, EV_struct.bmess4, set_value=Mess4
  
  EV_struct.mxTime = [0.0, 0.0] 
  EV_struct.mimaxT = [0.0, 0.0]
  widget_control, EV_struct.bidmn, set_value=string(EV_struct.mxTime[0])
  widget_control, EV_struct.bidmx, set_value=string(EV_struct.mxTime[1])
  widget_control, EV_struct.bminT, set_value=string(EV_struct.mxTime[0])
  widget_control, EV_struct.bmaxT, set_value=string(EV_struct.mxTime[1])
  
  EV_struct.tcycle = 0.0
  widget_control, EV_struct.bcycle, set_value = strtrim(EV_struct.tcycle, 2)
  EV_struct.tslice = 0.0
  widget_control, EV_struct.btslice, set_value = strtrim(EV_struct.tslice, 2)

end


pro read_eventfile, key
;+
;** ****************
;** Read file and set corresponding messages for users
;** and minimum and maximum values in Time Range boxes.
;-

  forward_function check_file, read_lst

  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger
  
  widget_control, EV_struct.bfile, get_value=txt_file
  new_file = strtrim(txt_file[0], 2)

  initialize_info_messages
  initialize_event_arrays

  file = check_file(key, new_file) ;test if we have a file to read
  widget_control, /hourglass
  
  if file gt ' ' then begin
    
      Mess = '  Reading the file' 
      widget_control, EV_struct.bmess1, set_value=Mess

      ; read event mode file
      time_ini = systime(1)
      counts = read_lst(file)
      time_end = systime(1)
      
      if n_elements(counts) gt 1 then begin
        
         update_after_reading, file
                 
      endif else begin
      
        Mess = '  Bad input file: No counts read!'
        widget_control, EV_struct.bmess1, set_value=Mess
       
      endelse

  endif else begin
    
    Mess = '  Bad input: File could not be opened!'
    widget_control, EV_struct.bmess1, set_value=Mess
    
  endelse
  
end


pro update_after_reading, file
;+
;** ****************
;** Update EV_struct values for a new file 
;-

  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger

  EV_struct.curFile = file

  ;Set min and max Timer 
  mini = min([Dtimer],max=maxi) & EV_struct.mxTime = [mini,maxi] / EV_struct.clock
  widget_control, EV_struct.bidmn, set_value=string(EV_struct.mxTime[0]) 
  widget_control, EV_struct.bidmx, set_value=string(EV_struct.mxTime[1])
  widget_control, EV_struct.bminT, set_value=string(EV_struct.mxTime[0])
  widget_control, EV_struct.bmaxT, set_value=string(EV_struct.mxTime[1])
  EV_struct.mimaxT[0] = EV_struct.mxTime[0]
  EV_struct.mimaxT[1] = EV_struct.mxTime[1]

  if Stimer[0] ne -1 then EV_struct.Strigg  = n_elements(Stimer) else EV_struct.Strigg  = 0
  if Ctimer[0] ne -1 then EV_struct.Ctrigg  = n_elements(Ctimer) else EV_struct.Ctrigg  = 0
  if Dtimer[0] ne -1 then EV_struct.Ndata   = n_elements(Dtimer) else EV_struct.Ndata   = 0
  if Mtimer[0] ne -1 then EV_struct.Monitor = n_elements(Mtimer) else EV_struct.Monitor = 0
  
  update_info_array_contents
  update_cycle_time
  
  if EV_struct.Ndata gt 1 then uv = 'show' ;Force a plot

end


pro update_info_array_contents
;+
;** ****************
;** Get number of events in each array (Dtimer, Mtimer, Stimer, Ctimer)
;** and update messages
;-

  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger

  if n_elements(Dtimer) gt 1 then tmp1 = n_elements(Dtimer) else $
          if Dtimer[0] eq -1 then tmp1 = 0 else tmp1 = 1

  if n_elements(Mtimer) gt 1 then tmp2 = n_elements(Mtimer) else $
          if Mtimer[0] eq -1 then tmp2 = 0 else tmp2 = 1

  if n_elements(Stimer) gt 1 then tmp3 = n_elements(Stimer) else $
          if Stimer[0] eq -1 then tmp3 = 0 else tmp3 = 1

  if n_elements(Ctimer) gt 1 then tmp4 = n_elements(Ctimer) else $
          if Ctimer[0] eq -1 then tmp4 = 0 else tmp4 = 1

  Mess1 = '  XY counts  =   ' + strtrim(long(total(Counts)), 2)

  Mess2 = '  Detector events  =   ' + strtrim(tmp1, 2)  + $
          ' ,  Monitor events  =   ' + strtrim(tmp2, 2)

  Mess3 = '  Trigger 1 (sample) events   =   ' + strtrim(tmp3, 2) + $
          ' ,  Trigger 2 (chopper) events  =   ' + strtrim(tmp4, 2)
          
  Mess4 = ' '        

  widget_control, EV_struct.bmess1, set_value=Mess1
  widget_control, EV_struct.bmess2, set_value=Mess2
  widget_control, EV_struct.bmess3, set_value=Mess3
  widget_control, EV_struct.bmess4, set_value=Mess4

end


pro update_cycle_time
;+
;** ****************
;** Updates values shown in time/cycle and time/slice boxes
;-

  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger

  if EV_struct.USECTRIGG then begin

      if n_elements(Ctimer) ge 2 then begin
          time_diff = Ctimer[1:*] - Ctimer[0:n_elements(Ctimer)-2]
          EV_struct.tcycle = mean(time_diff)/EV_struct.clock
      endif else begin
          zero = max([Ctimer[0]/EV_struct.clock, EV_struct.mimaxT[0]])
          EV_struct.tcycle = EV_struct.mimaxT[1] - zero
      endelse

  endif else if EV_struct.USESTRIGG then begin

      if n_elements(Stimer) ge 2 then begin
          time_diff = Stimer[1:*] - Stimer[0:n_elements(Stimer)-2]
          EV_struct.tcycle = mean(time_diff)/EV_struct.clock
      endif else begin
          zero = max([Stimer[0]/EV_struct.clock, EV_struct.mimaxT[0]])
          EV_struct.tcycle = EV_struct.mimaxT[1] - zero
      endelse

  endif else EV_struct.tcycle = EV_struct.mimaxT[1] - EV_struct.mimaxT[0]

  ; Show times in milliseconds
  widget_control, EV_struct.bcycle, set_value = strtrim(EV_struct.tcycle*1000., 2)
  
  update_slice_time
  
  ; Show info about how many detector events are in selected range
  tmin = (long64(EV_struct.mimaxt[0]*EV_struct.clock) - EV_struct.clocktol) > 0
  tmax = long64(EV_struct.mimaxt[1]*EV_struct.clock) + EV_struct.clocktol
  selected = where(Dtimer ge tmin and Dtimer le tmax, nbsel)
  mess = '  ' + strtrim(nbsel, 2) + ' detector events inside chosen time range'
  event_info, mess
  
end


pro update_slice_time
;+
;** ****************
;** Updates value in time/slice box
;-

  common EV_common1, EV_struct
  
  EV_struct.tslice = EV_struct.tcycle/EV_struct.nslice ;time per slice
  widget_control, EV_struct.btslice, set_value = strtrim(EV_struct.tslice*1000., 2)

end


pro update_frame_slider
  ;+
  ;** ****************
  ;** Update number of frames in frame slider
  ;-

  common EV_common1, EV_struct

  widget_control, EV_struct.bslidS, set_slider_max=EV_struct.nslice

end


pro remove_extreme_events
;+
;** ****************
;** Remove detector events having too low or too large times
;** Min and max times are given by the times of the 1st and last events,
;** and any event having a time < min or > max is removed
;-

  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger

  bad_times = where((Dtimer gt Dtimer[-1]) or (Dtimer lt Dtimer[0]), number_bad_events, complement=good_times)
  if number_bad_events gt 0 then begin
      Dtimer = Dtimer[good_times]
      Xdet = Xdet[good_times]
      Ydet = Ydet[good_times]
      mess = '  ' + strtrim(number_bad_events, 2) + ' events removed because time < t_1st or > t_last!'
      update_info_array_contents
      event_info, mess
  endif else begin
      mess = '  No bad events found.'
      event_info, mess
  endelse
  
end


pro remove_outliers_win_event, ev
;+
;** ****************
;** Handle events of outliers interface
;-

  widget_control, ev.id, get_uvalue=uv
  case uv of
     
      'outliers_apply': begin
          bd = widget_info(ev.top, find_by_uname='bpol')
          bs = widget_info(ev.top, find_by_uname='bsig')
          bp = widget_info(ev.top, find_by_uname='bper')
          bl = widget_info(ev.top, find_by_uname='bplot')
          widget_control, bd, get_value = poly_deg
          widget_control, bs, get_value = N_sigmas
          widget_control, bp, get_value = percent
          widget_control, bl, get_value = to_plot
          remove_outliers, fix(poly_deg[0]), float(N_sigmas[0]), float(percent[0]), to_plot
          widget_control, ev.top, /destroy
      end  
      
      'outliers_cancel': widget_control, ev.top, /destroy
      
      else:

  endcase

end

pro remove_outliers_win
;+
;** ****************
;** Interface to get values for removing outliers using a polynomial fit
;-

  RemOutlier = widget_base(Map=0, /align_center)
  widget_control, RemOutlier, /realize
  outBox = widget_base(title='Remove outlier events', Group_Leader=RemOutlier, /modal, /column, xsize=380)

  bid = widget_label(outBox, value='Removes detector events based on a polynomial fit')
  bid = widget_label(outBox, value='Give degree of polynomial and number of accepted sigmas, N')
  bid = widget_label(outBox, value='Any event with |time - fit_time| > N*sigma will be removed')
  bid = widget_label(outBox, value=' ')
  bid = widget_label(outBox, value='If the number of bad events found is larger than the given %')
  bid = widget_label(outBox, value='none of the events will be removed (as likely the fit was too bad)')
  bid = widget_label(outBox, value=' ')

  brow_pol = widget_base(outBox, /row)
  bid      = widget_label(brow_pol, value='Degree of polynomial =')
  bid      = widget_label(brow_pol, value='  ')
  bpol     = widget_text(brow_pol, value='1', uname='bpol', uv='poldeg', xsize=3, /edit)

  brow_sig = widget_base(outBox, /row)
  bid      = widget_label(brow_sig, value='Accepted deviation in number of sigmas =')
  bid      = widget_label(brow_sig, value='  ')
  bsig     = widget_text(brow_sig, value='10', uname='bsig', uv='sigmas', xsize=3, /edit)

  brow_per = widget_base(outBox, /row)
  bid      = widget_label(brow_per, value='Maximum % of points to remove =')
  bid      = widget_label(brow_per, value='  ')
  bper     = widget_text(brow_per, value='10', uname='bper', uv='percent', xsize=3, /edit)

  brow_pl  = widget_base(outBox, /row)
  bplot    = cw_bgroup(brow_pl, ['No', 'Yes'], /row, /exclusive, $
                       label_left = 'Show fit using Lamp workspaces?  ', $
                       set_value=0, uname='bplot', uv='showfit')

  bid      = widget_label(outBox, value=' ')
  brow_fin = widget_base(outBox, /row)
  bid      = widget_label(brow_fin, value=' ', xsize=120)
  bcancel  = widget_button(brow_fin, Value='Cancel', uv='outliers_cancel')
  bid      = widget_label(brow_fin, value=' ')
  baccept  = widget_button(brow_fin, Value='Apply', uv='outliers_apply')

  widget_control, outBox, /realize

  xmanager, 'remove_outliers_win', outBox, event_handler='remove_outliers_win_event'

end


pro remove_outliers, poly_deg, N_sigmas, percent_max, to_plot
;+
;** ****************
;** Fit polynomial to time history of event data and remove
;** any event who has a time too far away from the fitted curve
;-

  @lamp.cbk
  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger
  
  ;set a max number of points in Lamp workspaces, as number of events can be huge!
  npoints_plot = 10000
  
  ;Lamp workspaces to use
  wks = [56, 57, 58, 59, 60]

  nevent = ulindgen(n_elements(Dtimer))
  result = poly_fit(nevent, Dtimer, poly_deg, chisq=chisq, yfit=yfit)
  sigma = sqrt(chisq)/n_elements(Dtimer)
  residuals = abs(Dtimer - yfit)
  bad_times = where(residuals gt N_sigmas*sigma, complement=good_times, number_bad_events)
  percent_removed = 100. * number_bad_events / n_elements(Dtimer)
  
  if percent_removed gt percent_max then begin
      mess = '  Too many bad events found (' + strtrim(percent_removed, 2) + '%). Nothing done!'
      event_info, mess
  endif else begin
      mess = '  ' + strtrim(number_bad_events, 2) + ' events removed as outliers from fitted curve'
      event_info, mess
  endelse
  
  if to_plot then begin ; Send different set of data to Lamp workspaces w56-w60
    
      nskip = n_elements(Dtimer)/npoints_plot
      
      xval = nevent[0:-1:nskip]
      data = Dtimer[0:-1:nskip] / EV_struct.clock
      send_to_lamp, wks[0], data, XVAL = xval, XLABEL = 'Event number', YLABEL = 'Time', $
                    TITLE = 'Event times read from list mode', SUBTITLE = EV_struct.curFile
      
      data = yfit[0:-1:nskip] / EV_struct.clock
      send_to_lamp, wks[1], data, XVAL = xval, XLABEL = 'Event number', YLABEL = 'Time', $
                    TITLE = 'Fitted times', SUBTITLE = EV_struct.curFile

      data = (yfit[0:-1:nskip] + N_sigmas*sigma) / EV_struct.clock
      send_to_lamp, wks[2], data, XVAL = xval, XLABEL = 'Event number', YLABEL = 'Time', $
                    TITLE = 'Upper limit of valid region', SUBTITLE = EV_struct.curFile

      data = (yfit[0:-1:nskip] - N_sigmas*sigma) / EV_struct.clock
      send_to_lamp, wks[3], data, XVAL = xval, XLABEL = 'Event number', YLABEL = 'Time', $
                    TITLE = 'Lower limit of valid region', SUBTITLE = EV_struct.curFile
      
      if number_bad_events gt 0 and percent_removed lt percent_max then begin
          xval = nevent[bad_times]
          data = Dtimer[bad_times] / EV_struct.clock
          send_to_lamp, wks[4], data, XVAL = xval, XLABEL = 'Event number', YLABEL = 'Time', $
                        TITLE = 'Events removed', SUBTITLE = EV_struct.curFile
      endif
      
      info_message = ['A sample (' + strtrim(npoints_plot, 2) + ' points) of the event data ', $
                      'and the fit has been sent to the following Lamp workspaces:', $
                      ' ', $
                      'W56: Times corresponding to the events read. ', $
                      'W57: Fitted time dependence as a function of event number.', $
                      'W58: Upper limit of the acceptance region.', $
                      'W59: Lower limit of the acceptance region.']
                     
      if number_bad_events gt 0 and percent_removed lt percent_max then $ 
        info_message = [info_message, 'W60: Events removed.']   
                 
      info = dialog_message(info_message, /information)

  endif
  
  ; Remove events
  if number_bad_events gt 0 and percent_removed lt percent_max then begin
      Dtimer = Dtimer[good_times]
      Xdet = Xdet[good_times]
      Ydet = Ydet[good_times]
  endif

end


pro remove_fast_triggers
;+
;** ****************
;** Remove triggers ocurring before a minimum delay from last trigger
;** has elapsed.
;-

  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger

  if EV_struct.UseCtrigg then RefTimer = Ctimer $
    else if EV_struct.UseStrigg then RefTimer = Stimer $
    else begin
        event_info, '  No trigger selected. Nothing done.'
        return
    endelse
  
  if n_elements(RefTimer) le 1 then begin
      event_info, '  Number of trigger events <= 1. Nothing done.'
      return
  endif
  
  ; Get value of minimum time between valid triggers
  widget_control, EV_struct.bdelay, get_value = tmp
  tmp = strtrim(tmp[0], 2)
  if tmp gt '' then begin
      if valid_num(tmp) then begin
          tmp = float(tmp[0])*1.e-3 ;pass from ms to s
          min_delay = long64(tmp * EV_struct.clock)
      endif else begin
          event_info, '  Could not read a valid minimum delay in corresponding box. Nothing done.'
          return
      endelse
  endif else begin
      event_info, '  No value given in minimum delay box. Nothing done.'
      return
  endelse  

  actual_time = RefTimer[0] & keepVector = RefTimer & nLast = 0
  for i = 1, n_elements(RefTimer)-1 do begin
      if (RefTimer[i] - actual_time) gt min_delay then begin
          nLast += 1
          actual_time = RefTimer[i]
          keepVector[nLast] = RefTimer[i]
      endif
  endfor
  totalRemoved = n_elements(RefTimer) - nLast - 1
  RefTimer = keepVector[0:nLast]
  
  if EV_struct.UseCtrigg then Ctimer = RefTimer else Stimer = RefTimer  

  if totalRemoved gt 0 then begin
      times_from_trigger ;Recalculate times using new 'good' triggers
      widget_control, EV_struct.bmess4, get_value=MessEvents
      mess = '  ' + strtrim(totalRemoved, 2) + ' triggers with time < minimum delay removed! ' 
      mess += '(' + strtrim(MessEvents, 2) + ')'
      update_info_array_contents
  endif else mess = '  No triggers found with time < minimum delay'
  
  update_cycle_time
  event_info, mess

end


pro apply_trigger_pattern
;+
;** ****************
;** Remove triggers applying the given pattern
;-

  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger

  if EV_struct.UseCtrigg then RefTimer = Ctimer $
  else if EV_struct.UseStrigg then RefTimer = Stimer $
  else begin
    event_info, '  No trigger selected. Nothing done.'
    return
  endelse

  if n_elements(RefTimer) le 1 then begin
    event_info, '  Number of trigger events <= 1. Nothing done.'
    return
  endif

  ; Get pattern given by the user
  widget_control, EV_struct.bdelay, get_value = tmp
  pattern = strtrim(tmp, 2)
  if pattern gt ' ' then begin
      bpattern = boolarr(strlen(pattern))
      for i = 0, n_elements(bpattern)-1 do begin
          if strmid(pattern, i, 1) eq 'o' or $
             strmid(pattern, i, 1) eq 'O' or $
             strmid(pattern, i, 1) eq '0' then bpattern[i] = 'True'
      endfor
      check = where(bpattern eq 1, ntrue)
      if ntrue eq 0 then begin
          event_info, '  No triggers to remove in given pattern. Nothing done.'
          return
      endif
  endif else begin
      event_info, '  No trigger pattern given in corresponding box. Nothing done.'
      return
  endelse
      
  nrepeat = n_elements(RefTimer)/n_elements(bpattern)+1
  bpattern = reform(rebin(bpattern, n_elements(bpattern), nrepeat), nrepeat*n_elements(bpattern))
  keepVector = RefTimer & nLast = 0
  for i = 0, n_elements(RefTimer)-1 do begin
      if bpattern[i] then begin
          keepVector[nLast] = RefTimer[i]
          nLast += 1
      endif
  endfor
  totalRemoved = n_elements(RefTimer) - nLast
  RefTimer = keepVector[0:nLast-1]
  
  if EV_struct.UseCtrigg then Ctimer = RefTimer else Stimer = RefTimer

  if totalRemoved gt 0 then begin
    times_from_trigger ;Recalculate times using new 'good' triggers
    widget_control, EV_struct.bmess4, get_value=MessEvents
    mess = '  ' + strtrim(totalRemoved, 2) + ' triggers removed according to given pattern! '
    mess += '(' + strtrim(MessEvents, 2) + ')'
    update_info_array_contents
  endif else mess = '  No triggers to remove in given pattern.'
  
  update_cycle_time
  event_info, mess

end


pro do_bin
;+
;** ****************
;** Perform binning
;-

  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger
  common EV_common3, out_array, mon_array, time_array, nbrepaints, total_counting_time, ticks, ticks_end

  ; Define output arrays  
  s = size(counts)
  out_array = lonarr(s[1], s[2], EV_struct.nslice)
  mon_array = lonarr(EV_struct.nslice)
  time_array = fltarr(EV_struct.nslice)
  
  ; Define time range (tmin, tmax) for data to bin
  tmin = (long64(EV_struct.mimaxt[0]*EV_struct.clock) - EV_struct.clocktol) > 0
  tmax = long64(EV_struct.mimaxt[1]*EV_struct.clock) + EV_struct.clocktol
  total_counting_time = (tmax - tmin) / EV_struct.clock
  select_trange = where(Dtimer ge tmin and Dtimer le tmax, number_events)
  if EV_struct.Monitor gt 0 then $
    select_monitor = where(Mtimer ge tmin and Mtimer le tmax, number_monitors)

  ; Select times from trigger or not
  if EV_struct.USECTRIGG or EV_struct.USESTRIGG then begin
    
      ; Get times with respect to previous trigger --> Dtimer_trigger
      if n_elements(Dtimer_trigger) le 1 then times_from_trigger
      
      ; Select events inside given time range 
      ; (for this we need to use Dtimer (which contains absolute times) and do 
      ; the selection after the call to times_from_trigger, as some events
      ; happening before 1st trigger could have been removed there)
      select_trange = where(Dtimer ge tmin and Dtimer le tmax, number_events)
      select_monitor = where(Mtimer ge tmin and Mtimer le tmax, number_monitors)
      
      ; Times to use for the binning (event time lapse from previous trigger)
      times = Dtimer_trigger[select_trange]
      if EV_struct.Monitor gt 0 then times_monitor = Mtimer_trigger[select_monitor]
      
      ;Determine number of repaints 
      if EV_struct.USECTRIGG then begin
        select_repaints = where(Ctimer ge tmin and Ctimer le tmax, number_repetitions)
        nbrepaints = number_repetitions
      endif else begin
        select_repaints = where(Stimer ge tmin and Stimer le tmax, number_repetitions)
        nbrepaints = number_repetitions
      endelse 

      ; Min and max times for binning are given by cycle time
      tmin = 0
      tmax = long64(EV_struct.tcycle * EV_struct.clock)
    
  endif else begin
    
      ; Times to use for the binning (absolute times)
      times = Dtimer[select_trange]
      if EV_struct.Monitor gt 0 then times_monitor = Mtimer[select_monitor]
      nbrepaints = 1
    
  endelse

  ;Determine time bins to use for each possible binning case
  case EV_struct.BINMOD of
    
      1: begin
          delta_t = EV_struct.tcycle * EV_struct.clock / EV_struct.NSLICE
          ticks = tmin + long64(indgen(EV_struct.NSLICE)*delta_t)
          ticks_end = ticks + long64(EV_struct.tslice * EV_struct.clock)
      end
      
      2: begin
           delta_t = alog10(EV_struct.tcycle) / EV_struct.NSLICE
           ticks = 10^(findgen(EV_struct.NSLICE)*delta_t) 
           ticks = tmin + long64(ticks * EV_struct.clock) 
           ticks_end = lon64arr(n_elements(ticks))
           ticks_end[0:-2] = ticks[1:-1]
           ticks[0] = tmin
           ticks_end[-1] = tmax
      end
      
      3: begin
           delta_t = alog10(EV_struct.tcycle) / EV_struct.NSLICE
           ticks = 10^(findgen(EV_struct.NSLICE)*delta_t)
           ticks = tmin + long64(ticks * EV_struct.clock)
           ticks_end = lon64arr(n_elements(ticks))
           ticks_end[0:-2] = ticks[1:-1]
           ticks[0] = tmin
           ticks_end[-1] = tmax
           lapses = ticks_end - ticks
           ticks[0] = tmin 
           for i = 1, n_elements(ticks)-1 do ticks[i] = ticks[i-1] + lapses[n_elements(lapses)-i]
           ticks_end[0:-2] = ticks[1:-1]
           ticks_end[-1] = tmax
      end
     
      4: begin
            counts_per_slice = long64(number_events / EV_struct.NSLICE / nbrepaints)
            ticks = times[lindgen(EV_struct.NSLICE)*counts_per_slice] 
            ticks_end = lon64arr(n_elements(ticks))
            ticks_end[0:-2] = ticks[1:-1]
            ticks_end[-1] = tmax
            ticks[0] = 0
      end
    
  endcase
  
  ;Assign events to corresponding bin
  event_info, '  ' + strtrim(number_events, 2) + ' events to bin.'
  stime = systime(1)

  for islice = 0, EV_struct.NSLICE -1 do begin

      select_bin = where (times ge ticks[islice] and times lt ticks_end[islice])
    
      if select_bin[0] ne -1 then begin
          ;Assignment using VALUE_LOCATE
          XYdet = Xdet[select_trange[select_bin]] + EV_struct.Xsize*Ydet[select_trange[select_bin]]
          XYdet = XYdet[sort(XYdet)]
          XYunique = XYdet[UNIQ(XYdet)]
          if n_elements(XYdet) gt 1 then begin
              tmp = value_locate(XYdet, XYunique)
              counter = lonarr(n_elements(XYunique))
              counter[1:*] = tmp[1:*] - tmp[0:-2]
              counter[0] = n_elements(where (XYdet eq XYunique[0]))
              out_array[XYunique+islice*EV_struct.Xsize*EV_struct.Ysize] += counter
          endif else out_array[XYunique+islice*EV_struct.Xsize*EV_struct.Ysize] += 1
      endif

      ; Monitor
      if EV_struct.Monitor gt 0 then begin
           select_bin = where (times_monitor ge ticks[islice] and times_monitor lt ticks_end[islice])
           if select_bin[0] ne -1 then mon_array[islice] = n_elements(select_bin)
      endif else mon_array[islice] = 1

      ; Counting times per slice
      if ticks_end[islice] lt tmax then $
           time_array[islice] = (ticks_end[islice] - ticks[islice]) / EV_struct.clock $
      else time_array[islice] = (tmax - ticks[islice]) / EV_struct.clock

  endfor

  ; Clear arrays
  tempvar = size(temporary(tmp))
  event_info, '   Binning done: ' + strtrim(systime(1)-stime, 2) + ' seconds for assignment'
  
end


pro clear_times_from_trigger
;+
;** ****************
;** Clear trigger arrays
;-

  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger
  
  tempvar = size(temporary(Dtimer_trigger)) 
  tempvar = size(temporary(Mtimer_trigger))
  
end


pro times_from_trigger
;+
;** ****************
;** Calculates detector and monitor times from last trigger event
;-

  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger

  clear_times_from_trigger
  
  if n_elements(Dtimer) gt 1 then begin

      if EV_struct.UseCtrigg then RefTimer = Ctimer else RefTimer = Stimer
        
      ; Remove detector times < first trigger
      bad_times = where (Dtimer lt RefTimer[0], number_bad_events, complement=good_times)
      if number_bad_events gt 0 then begin
          Dtimer = Dtimer[good_times] & Xdet = Xdet[good_times] & Ydet = Ydet[good_times]
          mess = '  ' + strtrim(number_bad_events, 2) + ' events removed because time < time_1st_trigger!'
          update_info_array_contents
          event_info, mess
      endif
      Dtimer_trigger = Dtimer
      if n_elements(RefTimer) gt 1 then loc = value_locate(RefTimer, Dtimer) else loc = 0
      Dtimer_trigger = Dtimer - RefTimer[loc]
      ; Monitor
      if n_elements(Mtimer) gt 1 then begin
          bad_times = where (Mtimer lt RefTimer[0], number_bad_events, complement=good_times)
          if number_bad_events gt 0 then Mtimer = Mtimer[good_times]
          Mtimer_trigger = Mtimer
          if n_elements(RefTimer) gt 1 then loc = value_locate(RefTimer, Mtimer) else loc = 0
          Mtimer_trigger = Mtimer - RefTimer[loc]
      endif

  endif

end


pro show, W=wks
;+
;** ****************
;** Show results
;-

  common EV_common1, EV_struct
  
  case EV_struct.ShowMod of
    
      1: show_bin_slices, W=wks
      2: show_total_sum, W=wks
      3: show_event_times, W=wks
      4: show_event_distribution, W=wks
      5: show_trigger_periods, W=wks
      6: show_trigger_distribution, W=wks
      7: show_triggers, W=wks
      else: event_info, '  Requested ShowMod option not available.'
    
  endcase

end


pro show_bin_slices, W=wks
;+
;** ****************
;** Show slices
;-

  @lamp.cbk
  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger
  common EV_common3, out_array, mon_array, time_array, nbrepaints, total_counting_time, ticks, ticks_end
  
  ; Do binning if binned data not available
  if n_elements(out_array) le 1 then begin
      event_info, '  Empty data array. Calling binning routine with current settings.'
      do_bin
  endif

  ;Plot out_array
  midtimes = (ticks + 0.5*(ticks_end-ticks)) / EV_struct.clock
  if EV_struct.cframe eq 0 then plot_array = out_array $
    else plot_array = out_array[*,*,EV_struct.cframe-1]
  send_to_lamp, wks, plot_array, XLABEL = 'X detector', YLABEL = 'Y detector', $
                ZVAL = midtimes, ZLABEL = 'Time (seconds)', $
                TITLE = 'Binned slices', SUBTITLE = EV_struct.curFile
  
end


pro show_total_sum, W=wks
;+
;** ****************
;** Show sum of all slices
;-

  @lamp.cbk
  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger
  common EV_common3, out_array, mon_array, time_array, nbrepaints, total_counting_time, ticks, ticks_end

  if n_elements(out_array) le 1 then begin
      event_info, '  Empty data array. Calling binning routine with current settings.'
      do_bin
  endif
  
  if (size(out_array))[0] eq 3 then data = total(out_array, 3) else data = out_array
  
  send_to_lamp, wks, data, XLABEL = 'X detector', YLABEL = 'Y detector', $
                           TITLE = 'Histogram', SUBTITLE = EV_struct.curFile
    
end


pro show_event_times, W=wks
;+
;** ****************
;** Show time of events
;-

  @lamp.cbk
  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger
  
  ;Selected range
  tmin = (long64(EV_struct.mimaxt[0]*EV_struct.clock) - EV_struct.clocktol) > 0
  tmax = long64(EV_struct.mimaxt[1]*EV_struct.clock) + EV_struct.clocktol
  select_trange = where (Dtimer ge tmin and Dtimer le tmax, number_events)
  event_info, '  ' + strtrim(number_events, 2) + ' event times to be plotted!'
  if EV_struct.UseCtrigg or EV_struct.UseStrigg then begin
      if n_elements(Dtimer_trigger) eq 0 then begin
          times_from_trigger
          ;need to redo select because some events may have been chopped out by bin_trigger
          select_trange = where(Dtimer ge tmin and Dtimer le tmax, number_events)
      endif
      plot_data = Dtimer_trigger[select_trange]/EV_struct.clock
  endif else plot_data = Dtimer[select_trange]/EV_struct.clock
  send_to_lamp, wks, plot_data, XLABEL = 'Event number', YLABEL = 'Time (seconds)', $
                TITLE = 'Event time as a function of event number', SUBTITLE = EV_struct.curFile

end


pro show_event_distribution, W=wks
  ;+
  ;** ****************
  ;** Show distribution of events
  ;-

  @lamp.cbk
  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger

  ;Selected range
  tmin = (long64(EV_struct.mimaxt[0]*EV_struct.clock) - EV_struct.clocktol) > 0
  tmax = long64(EV_struct.mimaxt[1]*EV_struct.clock) + EV_struct.clocktol
  select_trange = where (Dtimer ge tmin and Dtimer le tmax, number_events)
  if EV_struct.UseCtrigg or EV_struct.UseStrigg then begin
    if n_elements(Dtimer_trigger) eq 0 then begin
      times_from_trigger
      ;need to redo select because some events may have been chopped out by bin_trigger
      select_trange = where(Dtimer ge tmin and Dtimer le tmax, number_events)
    endif
  endif
  nbins = 100
  if EV_struct.UseCtrigg or EV_struct.UseStrigg then begin
    bin_step = (max(Dtimer_trigger[select_trange])-min(Dtimer_trigger[select_trange])+2*EV_struct.clocktol)/nbins
    tzero = long64(min(Dtimer_trigger[select_trange])-EV_struct.clocktol) > 0
  endif else begin
    bin_step = (max(Dtimer[select_trange])-min(Dtimer[select_trange])+2*EV_struct.clocktol)/nbins
    tzero = long64(min(Dtimer[select_trange])-EV_struct.clocktol) > 0
  endelse
  histo_events = fltarr(nbins)
  histo_times = fltarr(nbins)
  for i = 0, nbins-1 do begin
    tini = tzero + i*bin_step
    tend = tzero + (i+1)*bin_step
    if EV_struct.UseCtrigg or EV_struct.UseStrigg then $
      tmp = where(Dtimer_trigger[select_trange] gt tini and Dtimer_trigger[select_trange] le tend, count) else $
      tmp = where(Dtimer[select_trange] gt tini and Dtimer[select_trange] le tend, count)
    histo_events[i] = count
    histo_times[i] = (tini + 0.5*bin_step) / EV_struct.clock
  endfor
  send_to_lamp, wks, histo_events, XVAL = histo_times, XLABEL = 'Time (s)', $
    YLABEL = 'Number of events', TITLE = 'Time distribution of detector events', $
    SUBTITLE = EV_struct.curFile

end


pro show_trigger_periods, W=wks
;+
;** ****************
;** Show lapse between consecutive triggers
;-

  @lamp.cbk
  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger

  if (not EV_struct.USECTRIGG) and (not EV_struct.USESTRIGG) then begin
    event_info, '  No trigger (chopper/sample) selected.'
    return
  endif

  ;Selected range
  tmin = (long64(EV_struct.mimaxt[0]*EV_struct.clock) - EV_struct.clocktol) > 0
  tmax = long64(EV_struct.mimaxt[1]*EV_struct.clock) + EV_struct.clocktol
  
  if EV_struct.UseCtrigg then RefTimer = Ctimer else RefTimer = Stimer

  if n_elements(RefTimer) ge 2 then begin
      select_trange = where (RefTimer ge tmin and RefTimer le tmax, number_events)
      event_info, '  ' + strtrim(number_events, 2) + ' event times to be plotted!'
      if n_elements(RefTimer[select_trange]) ge 2 then begin
          time_diff = RefTimer[select_trange[1:*]] - $
                      RefTimer[select_trange[0:n_elements(RefTimer[select_trange])-2]]
          time_diff /= EV_struct.clock
          send_to_lamp, wks, time_diff, XLABEL = 'Trigger number', YLABEL = 'Time (seconds)', $
                        TITLE = 'Time lapse between consecutive chopper triggers', $
                        SUBTITLE = EV_struct.curFile
      endif else event_info, '  Number of triggers in selected range < 2'
  endif else event_info, '  Number of triggers < 2'

end


pro show_triggers, W=wks
  ;+
  ;** ****************
  ;** Show trigger times
  ;-

  @lamp.cbk
  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger

  if (not EV_struct.USECTRIGG) and (not EV_struct.USESTRIGG) then begin
      event_info, '  No trigger (chopper/sample) selected.'
      return
  endif

  ;Selected range
  tmin = (long64(EV_struct.mimaxt[0]*EV_struct.clock) - EV_struct.clocktol) > 0
  tmax = long64(EV_struct.mimaxt[1]*EV_struct.clock) + EV_struct.clocktol

  if EV_struct.UseCtrigg then RefTimer = Ctimer else RefTimer = Stimer

  if n_elements(RefTimer) ge 1 then begin
      select_trange = where (RefTimer ge tmin and RefTimer le tmax, number_events)
      event_info, '  ' + strtrim(number_events, 2) + ' event times to be plotted!'
      if n_elements(RefTimer[select_trange]) ge 1 then begin
          data = fltarr(n_elements(RefTimer[select_trange])) + 1.0
          trigger_times = RefTimer[select_trange] / EV_struct.clock
          send_to_lamp, wks, data, XVAL = trigger_times, XLABEL = 'Time (seconds)', $
                        YLABEL = 'Trigger event', TITLE = 'Trigger time history', $
                        SUBTITLE = EV_struct.curFile
      endif else event_info, '  Number of triggers in selected range < 2'
  endif else event_info, '  Number of triggers < 2'

end


pro show_trigger_distribution, W=wks
;+
;** ****************
;** Show distribution of triggers
;-

  @lamp.cbk
  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger
  
  if (not EV_struct.USECTRIGG) and (not EV_struct.USESTRIGG) then begin
      event_info, '  No trigger (chopper/sample) selected.'
      return
  endif

  ;Selected range
  tmin = (long64(EV_struct.mimaxt[0]*EV_struct.clock) - EV_struct.clocktol) > 0
  tmax = long64(EV_struct.mimaxt[1]*EV_struct.clock) + EV_struct.clocktol
  
  if (EV_struct.UseCtrigg) then begin
      select_trange = where (Ctimer ge tmin and Ctimer le tmax, number_events)
      time_diff = Ctimer[select_trange[1:*]] - $
                  Ctimer[select_trange[0:n_elements(Ctimer[select_trange])-2]]
  endif else begin
      select_trange = where (Stimer ge tmin and Stimer le tmax, number_events)
      time_diff = Stimer[select_trange[1:*]] - $
                  Stimer[select_trange[0:n_elements(Stimer[select_trange])-2]]
  endelse
  if n_elements(time_diff) ge 2 then begin
      nbins = 100
      bin_step = (max(time_diff)-min(time_diff)+2*EV_struct.clocktol)/nbins
      tzero = long64(min(time_diff) - EV_struct.clocktol) > 0
      histo_events = fltarr(nbins)
      histo_times = fltarr(nbins)
      for i = 0, nbins-1 do begin
          tini = tzero + i*bin_step
          tend = tzero + (i+1)*bin_step
          tmp = where(time_diff gt tini and time_diff le tend, count)
          histo_events[i] = count
          histo_times[i] = (tini + 0.5*bin_step) / EV_struct.clock
      endfor
      send_to_lamp, wks, histo_events, XVAL = histo_times, XLABEL = 'Time (s)', $
                    YLABEL = 'Number of trigger events', $
                    TITLE = 'Time distribution of trigger periods', $
                    SUBTITLE = EV_struct.curFile
  endif else event_info, '  Not enough trigger events to calculate distribution.'

end


pro export_nxs_list
;+
;** ****************
;** Treat and export a series of files with current parameters
;-

  common EV_common1, EV_struct

  files = dialog_pickfile(filter='*.lst', path=EV_struct.Cpath, get_path=pthO, file=tmp, $
                          /read, /multiple_files, resource='lamptouch')
                          
  if n_elements(files) ge 1 then begin
      tmp = EV_struct.curfile
      ; store actual time range, as it is initialized on reading
      trmin = EV_struct.mimaxt[0]
      trmax = EV_struct.mimaxt[1]
      for i = 0, n_elements(files)-1 do begin
          widget_control, EV_struct.bfile, set_value=file_basename(files[i])
          read_eventfile, 'file'
          EV_struct.mimaxt[0] = trmin
          EV_struct.mimaxt[1] = trmax
          do_bin
          export_nxs_file
      endfor
      EV_struct.curfile = tmp
    endif

end


pro export_nxs_file
;+
;** ****************
;** Export binned slices as NeXus kinetic file
;-

  forward_function modify_nxs_d22

  common EV_common1, EV_struct
  common EV_common3, out_array, mon_array, time_array, nbrepaints, total_counting_time, ticks, ticks_end

  if n_elements(out_array) le 1 then begin
      event_info, '  No binned data (out_array) to export!'
      return
  endif

  filename = EV_struct.curFile
  
  if file_dirname(filename) eq '.' then $
    fileNX = EV_struct.cpath + path_sep() + strmid(filename,0,strpos(filename,'.lst')) + '.nxs' $
    else fileNX = strmid(filename,0,strpos(filename,'.lst')) + '.nxs'
  
  ok = file_test(fileNX)

  if ok then nexus = h5_parse(fileNX, /read_data) else begin
      event_info, '  No input Nexus file found! Cannot export'
      return
  endelse
  
  case EV_struct.inst of 
    
      'd22': begin
          event_info, '  Modifying structure of NeXus file ...'
          newNXS = modify_nxs_d22(nexus)
          event_info, '  Done. Writing file ...'
      end    
      else: begin
          event_info, '  No NeXus structure for instrument ' + EV_struct.inst
          return
      end    
    
  endcase
  
  ; Write NeXus from event mode data
  outfile = strmid(fileNX, 0, strpos(fileNX,'.nxs')) + '_lst.nxs'
  h5_create, outfile, newNXS
  event_info, '  Binned data exported to file ' + outfile

end


function modify_nxs_d22, nexus
;+
;** ****************
;** Modify structure of NeXus file for D22
;** (Based on NeXus files produced during cycle 183)
;-

  common EV_common3, out_array, mon_array, time_array, nbrepaints, total_counting_time, ticks, ticks_end

  tags_entry0 = tag_names(nexus.entry0)
  
  ;Change counting time as it could be different if Min/Max Time Ranges were used
  nexus.entry0.time._data = total_counting_time
  
  ;Set file mode = 3 (Kinetic)
  nexus.entry0.acquisition_mode._data = 3
  nexus.entry0.mode._data = 3
  
  s = size(out_array)
  if s[0] gt 2 then slices = s[3] else slices = 1
  
  ;Modify or create NBREPAINT structure
  check = where('NBREPAINT' eq tags_entry0, nbrepaint_found)
  if nbrepaint_found eq 0 then begin
      struc_nbrepaint = {_name:'nbrepaint', _icontype:'binary', _type:'dataset', $
                         _file:'', _path:'/entry0', _data:[1.], _ndimensions:1L, $
                         _dimensions:[1ULL], _nelements:1ULL, _datatype:'H5T_FLOAT',$
                         _storagesize:4UL, _precision:32L, _sign:''}
      struc_nbrepaint._file = nexus._file
      struc_nbrepaint._data = nbrepaints
  endif else begin
      nexus.entry0.nbrepaint._file = nexus._file
      nexus.entry0.nbrepaint._data = nbrepaints
  endelse
  
  ;Create SLICES structure
  check = where('SLICES' eq tags_entry0, slices_found)
  if slices_found eq 0 then begin
      struc_slices = {_name:'slices', _icontype:'binary', _type:'dataset', $
                      _file:'', _path:'/entry0', _data:[1.], _ndimensions:1L, $
                      _dimensions:[1ULL], _nelements:1ULL, _datatype:'H5T_FLOAT',$
                      _storagesize:4UL, _precision:32L, _sign:''}
      struc_slices._file = nexus._file
  endif    
  
  ;Create TOTAL_SLICES_TIME structure
  check = where('TOTAL_SLICES_TIME' eq tags_entry0, total_slices_time_found)
  if total_slices_time_found eq 0 then begin
      struc_total_slices_time = {_name:'total_slices_time', _icontype:'binary', _type:'dataset', $
                                 _file:'', _path:'/entry0', _data:[1.], _ndimensions:1L, $
                                 _dimensions:[1ULL], _nelements:1ULL, _datatype:'H5T_FLOAT', $
                                 _storagesize:4UL, _precision:32L, _sign:''}
      struc_total_slices_time._file = nexus._file
  endif    

  ;Check if additional tags need to be added to nexus.entry0
  NewEntry0 = create_struct(nexus.entry0)
  if nbrepaint_found eq 0 then NewEntry0 = create_struct(NewEntry0, 'NBREPAINT', struc_nbrepaint)
  if slices_found eq 0 then NewEntry0 = create_struct(NewEntry0, 'SLICES', struc_nbrepaint)
  if total_slices_time_found eq 0 then NewEntry0 = create_struct(NewEntry0, 'TOTAL_SLICES_TIME', struc_nbrepaint)
  
  newHeader = {_name:'', _icontype:'', _type:'', _file:'', _path:'', _comment:''}
  newHeader._name = nexus._name
  newHeader._icontype = nexus._icontype
  newHeader._type = nexus._type
  newHeader._file = nexus._file
  newHeader._path = nexus._path
  newHeader._comment = nexus._comment
  
  finalNX = create_struct(newHeader, $
            ['entry0', 'file_name', 'hdf5_version', 'file_time', 'nexus_version'], $
            NewEntry0, nexus.file_name, nexus.hdf5_version, nexus.file_time, nexus.nexus_version)

  ;measuring time for each slice
  finalNX.entry0.slices._dimensions = slices
  finalNX.entry0.slices._nelements = slices
  modify_struc, finalNX, ['entry0', 'slices', '_data'], 0, time_array
  finalNX.entry0.total_slices_time._dimensions = slices
  finalNX.entry0.total_slices_time._nelements = slices
  modify_struc, finalNX, ['entry0', 'total_slices_time', '_data'], 0, time_array * nbrepaints
  
  ;monitor counts for each slice
  finalNX.entry0.monitor1.inhibit_time = nexus.entry0.monitor1.inhibit_time
  finalNX.entry0.monitor1.integral = nexus.entry0.monitor1.integral
  finalNX.entry0.monitor1.mode = nexus.entry0.monitor1.mode
  finalNX.entry0.monitor1.mode._data = nexus.entry0.mode._data
  finalNX.entry0.monitor1.monrate = nexus.entry0.monitor1.monrate
  finalNX.entry0.monitor1.monsum = nexus.entry0.monitor1.monsum
  finalNX.entry0.monitor1.nbpickup = nexus.entry0.monitor1.nbpickup
  finalNX.entry0.monitor1.preset = nexus.entry0.monitor1.preset
  finalNX.entry0.monitor1.time_of_flight = nexus.entry0.monitor1.time_of_flight
  finalNX.entry0.monitor1.nx_class = nexus.entry0.monitor1.nx_class
  finalNX.entry0.monitor1.data._dimensions[0] = slices
  finalNX.entry0.monitor1.data._nelements = slices
  modify_struc, finalNX, ['entry0', 'monitor1', 'data', '_data'], 0, mon_array
  
  finalNX.entry0.monitor2.inhibit_time = nexus.entry0.monitor2.inhibit_time
  finalNX.entry0.monitor2.integral = nexus.entry0.monitor2.integral
  finalNX.entry0.monitor2.mode = nexus.entry0.monitor2.mode
  finalNX.entry0.monitor2.mode._data = nexus.entry0.mode._data
  finalNX.entry0.monitor2.monrate = nexus.entry0.monitor2.monrate
  finalNX.entry0.monitor2.monsum = nexus.entry0.monitor2.monsum
  finalNX.entry0.monitor2.preset = nexus.entry0.monitor2.preset
  finalNX.entry0.monitor2.nx_class = nexus.entry0.monitor2.nx_class
  finalNX.entry0.monitor2.data._dimensions[0] = slices
  finalNX.entry0.monitor2.data._nelements = slices
  modify_struc, finalNX, ['entry0', 'monitor2', 'data', '_data'], 0, mon_array*0+1

  ;change data
  finalNX.entry0.d22.detector.data._dimensions = [slices, s[2], s[1]]
  finalNX.entry0.d22.detector.data._nelements = n_elements(out_array)
  if slices gt 1 then write_array = transpose(out_array, [2,1,0]) else write_array = transpose(out_array)
  modify_struc, finalNX, ['entry0', 'd22', 'detector', 'data', '_data'], 0, write_array

  return, finalNX
  
end


pro read_lst_d22, filename, nEvents, nBoards
;+
;    Read D22 event data:
;    Electronic Cards: 4 (boards = 0-3) x 1740 QDIV (detector) + a GGPickup (board = 4) (monitor + triggers)
;
;        Word 1 --> Crate                ( 4 bits: 31-28)
;                   Board                ( 6 bits: 27-22)
;                   Channel              ( 6 bits: 21-16)
;                   Timestamp bits 47-32 (16 bits: 15-0)
;
;        Word 2 --> Timestamp bits 31-0  (32 bits: 31-0)
;
;        Word 3 (only for 1740 QDIV) --> X position (5 bits: 20-16)
;                                        Y position (8 bits: 7-0)
;
;        Word 4 (only for 1740 QDIV) --> A+B (16 bits: 31-16)
;                                        A   (16 bits: 15-0)
;                                        
;    Board 4 - channel 1 --> Stimer
;    Board 4 - channel 2 --> Mtimer
;    Board 4 - channel 4 --> Ctimer
;    Boards 0, 1, 2, 3   --> Dtimer, Xdet, Ydet
;                                        
;-
  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger
                       
  ; Opens the file for reading in binary mode.
  openr, u, filename, /get_lun
  
  ; Skip the header
  for i = 0, 3+nBoards do readu, u, tmp

  events = ulon64arr(2*nEvents) ;each event stored in 128 bits

  readu, u, events  &  byteorder, events, /swap_if_big_endian
  free_lun, u
    
  eventsA = events[0:-1:2]
  eventsB = events[1:-1:2]
  events = 0 ;free memory
    
  boards = ishft(eventsA, -22) and (2ULL^6-1)
    
  ; get times (in ns) for monitor, top & pickup events from board 0 
  Mtimer = [-1] ;channel 2 for monitor
  Stimer = [-1] ;channel 1 for top
  Ctimer = [-1] ;channel 4 for chopper pickup
  idx = where(boards eq 4)
    
  if idx[0] gt -1 then begin
    
    ;first get channel to discriminate different types of event in GGPICKUP
    channel = ishft(eventsA[idx], -16) and (2ULL^6-1)
    
    ;channel 1 --> Pickup sample
    idx_channel = where(channel eq 1)
    if idx_channel[0] gt -1 then begin
      time1 = ishft(eventsA[idx[idx_channel]], -32) and (2ULL^32-1)
      time2 = ishft(eventsA[idx[idx_channel]], 0) and (2ULL^16-1)
      Stimer = time2 * 2ULL^32 + time1
      time1 = 0
      time2 = 0
    endif

    ;channel 2 --> Monitor
    idx_channel = where(channel eq 2)
    if idx_channel[0] gt -1 then begin      
      time1 = ishft(eventsA[idx[idx_channel]], -32) and (2ULL^32-1)
      time2 = ishft(eventsA[idx[idx_channel]], 0) and (2ULL^16-1)
      Mtimer = time2 * 2ULL^32 + time1
      time1 = 0
      time2 = 0
    endif

    ;channel 4 --> Pickup chopper
    idx_channel = where(channel eq 4)
    if idx_channel[0] gt -1 then begin
      time1 = ishft(eventsA[idx[idx_channel]], -32) and (2ULL^32-1)
      time2 = ishft(eventsA[idx[idx_channel]], 0) and (2ULL^16-1)
      Ctimer = time2 * 2ULL^32 + time1
      time1 = 0
      time2 = 0
    endif
    
  endif
  
  ; get times and positions from boards 1-4 (detector)
  Dtimer = []
  Xdet = []
  Ydet = []
  order_boards = [0, 1, 2, 3]  ;good order of the boards to reconstruct detector image
  ipos = 0
  
  for iboard=0,3 do begin
    
    idx = where(boards eq order_boards[iboard])
    
    if idx[0] gt -1 then begin
 
      time1 = ishft(eventsA[idx], -32) and (2ULL^32-1)
      time2 = ishft(eventsA[idx], 0) and (2ULL^16-1)
      Dtimer = [Dtimer, time2 * 2ULL^32 + time1]
      time1 = 0
      time2 = 0
      Ydet = [Ydet, ishft(eventsB[idx], 0) and (2ULL^8-1)]
      detTube = (ishft(eventsB[idx], -16) and (2ULL^5-1)) +  iboard*32 ;each card corresponds to 32 detector tubes
      Xdet = [Xdet, detTube]
      detTube = 0
    
    endif
    
    ipos += 1
    
  endfor
                       
end


function read_lst, filename
;+
;                           
; Returns detector total histogram and fills the following arrays
; 
;  X_DETECTOR = Xdet   = X address of event in detector
;  Y_DETECTOR = Ydet   = Y address of event in detector
;  T_DETECTOR = Dtimer = time of event in detector
;  T_MONITOR  = Mtimer = time of event in monitor
;  T_SAMPLE   = Stimer = time of event connected to sample equipment (e.g. shear machine)
;  T_CHOPPER  = Ctimer = time of event connected to chopper
;
; Supported instruments:
; 
;   'd22'    (May 2018 - )
;
;-

  forward_function rdid
  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger
  
  ; If the file to read does not exist, return.
  if not file_test(filename) then return, 0

  eventBlockSize =  100000000UL                      ;To adjust depending on resources
  maxEvents =      4294967295UL
  stime = systime(1)

  ; Creates an error handler in case where an error occurred when parsing the file.
  on_ioerror, READ_ERROR
  u = -1 ; The file unit.

  ; Opens the file for reading in binary mode.
  openr, u, filename, /get_lun

  ; Header
  header = ulonarr(4)
  readu, u, header & byteorder, header, /swap_if_big_endian
  lst_version = header[0]
  lst_timeBase = header[1]
  nEvents = header[2]
  nBoards = header[3]

  free_lun, u
  
  ; Check if event mode version is the right one
  if lst_version ne 1 then begin
    error_message = ['Version of the event mode file not compatible with reading routine!']
    result = dialog_message(error_message, /error)
    return, 0    
  endif
  
  if nEvents gt maxEvents then begin
    error_message = ['Number of events > 4,294,967,296', $
      'ULONG arrays used in the code cannot handle such a large number of events!']
    result = dialog_message(error_message, /error)
    return, 0
  endif

  case EV_struct.inst of
    
    'd22': begin
              ;detector dimensions
              EV_struct.xsize = 128
              EV_struct.ysize = 256
              read_lst_d22, filename, nEvents, nBoards
    end
    
    else: begin
              line = 'Case EV_struct.inst =  ' + EV_struct.inst
              error_message = [line, ' not found. Exiting!']
              result = dialog_message(error_message, /error)
              return, 0
    end
                 
  endcase
  
  ; Fill histogram corresponding 2D image (sum over all events)
  counts = lonarr(EV_struct.xsize, EV_struct.ysize) 
  for i = 0, n_elements(Dtimer)-1 do counts[Xdet[i], Ydet[i]] += 1L 

  ; Sort monitor and detector events by time
  Mtimer = Mtimer[sort(Mtimer)]
  idx = sort(Dtimer)
  Dtimer = Dtimer[idx]
  Xdet = Xdet[idx]
  Ydet = Ydet[idx]

  ; Get number of events of each type
  if Dtimer[0] gt 0 then Dhits = n_elements(Dtimer) else Dhits = 0
  if Mtimer[0] gt 0 then Mhits = n_elements(Mtimer) else Mhits = 0
  if Stimer[0] gt 0 then Shits = n_elements(Stimer) else Shits = 0
  if Ctimer[0] gt 0 then Chits = n_elements(Ctimer) else Chits = 0
  
  ; Reading finished. Write summary and return histogram.
  event_log, strtrim(round((systime(1)-stime)*10.)/10., 2) + ' seconds to read ' + $
    strtrim(nEvents, 2) + ' events.'     
  event_log, 'Number of detector events = ' + strtrim(Dhits, 2)
  event_log, 'Number of monitor events  = ' + strtrim(Mhits, 2)
  event_log, 'Number of sample events   = ' + strtrim(Shits, 2)
  event_log, 'Number of chopper events  = ' + strtrim(Chits, 2)
  event_log, 'Counts in total histogram = ' + strtrim(total(counts), 2) & wait, 001
    
  return, counts
  
  ; Case where an error occured when parsing the file.
  READ_ERROR:
  if u gt 0 then free_lun, u
  lamp_error
  return, 0
  
end


pro test_function

  @lamp.cbk
  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger
  
  print, 'In test function'
  ;stop
  
;  diff = dtimer[1:*] - dtimer[0:-2]
;  time_intervals = (diff / EV_struct.clock) * 1e6 ;time-step in microseconds
;  bin_size = 0.2 ;
;  time_distrib = histogram(time_intervals, binsize = bin_size, min=0.0)
;  time_axis = findgen(n_elements(time_distrib))* bin_size + 0.5 * bin_size
  
  ;select a single tube
  ;ntube = 64
  ;tube = where(xdet eq (ntube-1))
  ;diff = (dtimer[tube])[1:*] - (dtimer[tube])[0:-2]
  ;time_intervals = (diff / EV_struct.clock) * 1e6 ;time-step in microseconds
  ;bin_size = 100
  ;time_distrib = histogram(time_intervals, binsize = bin_size, min=0.0, max=1e5)
  ;time_axis = findgen(n_elements(time_distrib))* bin_size + 0.5 * bin_size
  
  ;A = [600.0, 2.e-3]
  ;yfit = CURVEFIT(time_axis, time_distrib, weights, A, function_name='FIT_EXP')
  
end


pro event_file_dying, ev
;+
;** ****************
;** Free memory
;-

  common EV_common1, EV_struct
  common EV_common2, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger

  Counts=0 & Xdet=0 & Ydet=0 & Dtimer=0 & Ctimer=0 & Stimer=0
  Mtimer=0 & Dtimer_trigger=0 & Mtimer_trigger=0
    
end


pro event_file_event, ev
;+
;** ****************
;** event handler for event_file GUI
;-

  CATCH,stat & if stat ne 0 then begin & lamp_error & return & endif

  @lamp.cbk
  common EV_common1, EV_struct

  widget_control, ev.id, get_uvalue=uv
  
  ; Call the treatment macro corresponding to the case event
  ; Cases: 'help', 'file', 'browseW', 'browseD', 'raw', 'usetrigger', 'slides', 
  ;        'nslice', 'mintime', 'maxtime',
  ;        'setmint', 'setmaxt', 'bin_lin', 'bin_log', 'bin_equ', 'bin', 'show',
  ;        'show_histo', 'show_event', 'show_bin', 'show_trigger', 
  ;        'show_dist_events', 'show_dist_triggers', 'expcur', 'explst',
  ;        'quit', 'timecycle'
  ; ********************************************************
    
  case uv of
    
    'help'   : help_eventfile

    'raw'    : begin & EV_struct.removeBad = ev.select & end
    
    'file'   : read_eventfile, uv 
    
    'browseW' : read_eventfile, uv 

    'browseD' : read_eventfile, uv
    
    'remove_extreme_events': begin
      widget_control, ev.id,          get_value=pmod
      widget_control, EV_struct.braw, set_value=pmod
      remove_extreme_events
    end
    
    'remove_outliers': begin
      widget_control, ev.id,          get_value=pmod
      widget_control, EV_struct.braw, set_value=pmod
      remove_outliers_win
    end

    'remove_fast_triggers': begin
      widget_control, ev.id,          get_value=pmod
      widget_control, EV_struct.braw, set_value=pmod
      remove_fast_triggers
    end

    'apply_trigger_pattern': begin
      widget_control, ev.id,          get_value=pmod
      widget_control, EV_struct.braw, set_value=pmod
      apply_trigger_pattern
    end

    'mintime': begin ; Min Timer changed
      widget_control, EV_struct.bminT, get_value=tmp
      EV_struct.mimaxT[0] = tmp
      update_cycle_time
    end

    'maxtime': begin ; Max Timer changed
      widget_control, EV_struct.bmaxT, get_value=tmp
      EV_struct.mimaxT[1] = tmp
      update_cycle_time
    end

    'setmint': begin ; Min Timer reset
      widget_control, EV_struct.bidmn, get_value=mn
      widget_control, EV_struct.bminT, set_value=mn
      EV_struct.mimaxT[0] = Mn
      update_cycle_time
    end

    'setmaxt': begin ; Max Timer reset
      widget_control, EV_struct.bidmx, get_value=mx
      widget_control, EV_struct.bmaxT, set_value=mx
      EV_struct.mimaxT[1] = Mx
      update_cycle_time
    end

    'usetrigger': begin 
       EV_struct.UseCtrigg = 0
       EV_struct.UseStrigg = 0  
       clear_times_from_trigger 
       widget_control, EV_struct.btrigg, get_value = trigger_choice
       case trigger_choice of
         0: begin & EV_struct.UseCtrigg = 0 & EV_struct.UseStrigg = 0 & end
         1: begin & EV_struct.UseCtrigg = 1 & EV_struct.UseStrigg = 0 & end
         2: begin & EV_struct.UseCtrigg = 0 & EV_struct.UseStrigg = 1 & end
         else: event_info, '  Unhandled case in trigger_choice'
       endcase
       update_cycle_time
    end

    'timecycle': begin ;Set manually cycle time
      widget_control, EV_struct.bcycle, get_value=str
      EV_struct.tcycle = 1e-3 * double(str[0])
      update_slice_time
    end

    'nslice':  begin ; number of slices changed
      widget_control, EV_struct.bslice, get_value=tmp
      EV_struct.nslice = long(tmp)
      update_slice_time
      update_frame_slider
    end

    'tslice': begin ; time per slice given manually
      widget_control, EV_struct.btslice, get_value=str
      EV_struct.tslice = 1e-3 * double(str[0]) 
    end

    'bin_lin': begin ; Bin Spacing mode linear
        widget_control, ev.id,          get_value=bmod
        widget_control, EV_struct.bBin, set_value=bmod
        EV_struct.BinMod = 1
    end
             
    'bin_log': begin ; Bin Spacing mode log
        widget_control, ev.id,          get_value=bmod
        widget_control, EV_struct.bBin, set_value=bmod
        EV_struct.BinMod = 2
    end

    'bin_invlog': begin ; Bin Spacing mode inverse log
      widget_control, ev.id,          get_value=bmod
      widget_control, EV_struct.bBin, set_value=bmod
      EV_struct.BinMod = 3
    end
             
    'bin_equ': begin ; Bin Spacing mode equal events
        widget_control, ev.id,          get_value=bmod
        widget_control, EV_struct.bBin, set_value=bmod
        EV_struct.BinMod = 4
    end
  
    'bin': do_bin
             
    'show': begin ; Choose current show mode
        Wcur = ([0,1,2,3,4,5,6,7])[EV_struct.ShowMod]
        show, W=Wcur
        widget_control, EV_struct.bmessData, set_value=':   W'+strtrim(Wcur, 2)
    end

    'show_bin': begin ; Bin Details
      widget_control, ev.id,           get_value=pmod
      widget_control, EV_struct.bPlot, set_value=pmod
      EV_struct.ShowMod = 1
      Wcur = ([0,1,2,3,4,5,6,7])[EV_struct.ShowMod]
      show, W=Wcur
      widget_control, EV_struct.bmessData, set_value=':   W'+strtrim(Wcur, 2)
    end

    'show_histo': begin ; Histogram sum
        widget_control, ev.id,           get_value=pmod
        widget_control, EV_struct.bPlot, set_value=pmod
        EV_struct.ShowMod = 2
        Wcur = ([0,1,2,3,4,5,6,7])[EV_struct.ShowMod]
        show, W=Wcur
        widget_control, EV_struct.bmessData, set_value=':   W'+strtrim(Wcur, 2)
    end
  
    'show_event': begin ; Event Data
        widget_control, ev.id,           get_value=pmod
        widget_control, EV_struct.bPlot, set_value=pmod
        EV_struct.ShowMod = 3
        Wcur = ([0,1,2,3,4,5,6,7])[EV_struct.ShowMod]
        show, W=Wcur
        widget_control, EV_struct.bmessData, set_value=':   W'+strtrim(Wcur, 2)
    end

    'show_dist_events': begin ; Time distribution of detector events
      widget_control, ev.id,           get_value=pmod
      widget_control, EV_struct.bPlot, set_value=pmod
      EV_struct.ShowMod = 4
      Wcur = ([0,1,2,3,4,5,6,7])[EV_struct.ShowMod]
      show, W=Wcur
      widget_control, EV_struct.bmessData, set_value=':   W'+strtrim(Wcur, 2)
    end
   
    'show_trigger': begin ; Trigger
        widget_control, ev.id,           get_value=pmod
        widget_control, EV_struct.bPlot, set_value=pmod
        EV_struct.ShowMod = 5
        Wcur = ([0,1,2,3,4,5,6,7])[EV_struct.ShowMod]
        show, W=Wcur
        widget_control, EV_struct.bmessData, set_value=':   W'+strtrim(Wcur, 2)
    end
             
    'show_dist_triggers': begin ; Time distribution of trigger events
        widget_control, ev.id,           get_value=pmod
        widget_control, EV_struct.bPlot, set_value=pmod
        EV_struct.ShowMod = 6
        Wcur = ([0,1,2,3,4,5,6,7])[EV_struct.ShowMod]
        show, W=Wcur
        widget_control, EV_struct.bmessData, set_value=':   W'+strtrim(Wcur, 2)
    end

    'show_trigger_history': begin ; Time history of triggers
      widget_control, ev.id,           get_value=pmod
      widget_control, EV_struct.bPlot, set_value=pmod
      EV_struct.ShowMod = 7
      Wcur = ([0,1,2,3,4,5,6,7])[EV_struct.ShowMod]
      show, W=Wcur
      widget_control, EV_struct.bmessData, set_value=':   W'+strtrim(Wcur, 2)
    end

    'slides':  begin
      EV_struct.Cframe= ev.value  ; Current frame (slider)
      if ev.value eq 0 then widget_control, EV_struct.bslidL, set_value='All'
      if ev.value gt 0 then widget_control, EV_struct.bslidL, set_value='Current'
    end
  
    'expcur': export_nxs_file ;Export current binned file 
  
    'explst': export_nxs_list ;Export a list of files 
    
    'test': test_function
    
    'quit': begin & widget_control, ev.top,/destroy & return & end
           
    else:

  endcase

end


pro event_file, instrument=instrument
;+
;** **********
;** User callable
;** Event_file analysis GUI
;** keyword instrument to select a particular instrument 
;** (supported instruments: d22)
;-

; Possible events in event_file (uv) and corresponding widget_control (needs to be updated):
; 
; ‘file’ : File name -> EV_struct.bfile -> read_eventfile (initialize_info_messages, initialize_event_arrays, 
;          check_file, read_lst, update_after_reading)
; ‘browseW’ : Choose file from working path -> read_eventfile
; ‘browseD’ : Choose file from data path -> read_eventfile
; ‘raw’ : Remove bad events
; ‘setmint’ : Min label, contains also the minimum time -> EV_struct.bidmn
; ‘mintime’ : Minimum time to use (editable by user) -> EV_struct.bmint
; ‘setmaxt’ : Max label, contains also the maximum time -> EV_struct.bidmx
; ‘maxtime’ : Maximum time to use (editable by user) -> EV_struct.bmaxt
; ‘usetrigger’ : Selection (exclusive) of trigger (none, chopper, sample) -> EV_struct.btrigg
; ‘delay’ : Forbidden (min) time before accepting a new trigger after the last one -> EV_struct.bdelay
; ‘timecycle’ : Show (set?) time of a cycle -> EV_struct.bcycle
; ‘nslice’ : Number of slices -> EV_struct.bslice
; ‘tslice’ : Time per slice (in ms) -> EV_struct.btslice
; ‘bin_lin’ : Linear binning -> EV_struct.bbin, EV_struct.binmod
; ‘bin_log’ : Log binning -> EV_struct.bbin, EV_struct.binmod
; ‘bin_equal’ : Variable time binning with equal number of events in each bin -> EV_struct.bbin, EV_struct.binmod
; ‘bin’ : Do binning
; ‘show_bin’ :
; ‘show_histo’ :
; ‘show_event’ :
; ‘show_trigger’ :
; ‘show_dist_events’ :
; ‘show_dist_triggers’ :
; ‘show’ :
; ‘slides’ : Slider to select a frame -> EV_struct.bslids, EV_struct.bslidl
; ‘expcur’ : Export current file
; ‘explst’ : Apply same rebinning and export a list of files
; ‘help’ :
; 'quit' :


;Contents of EV_struct defined below:
;
; bmess1    = label widget message
; bmess2    = label widget message 2nd line
; bmess3    = label widget message 3rd line
; bmess4    = label widget message 4th line: events removed
; bmessData = label widget message bottom: workspace containing the output
; bfile     = text widget input file
; curFile   = filename
; Wpath     = working path
; Dpath     = data path
; Cpath     = current path
; braw      = button to remove events or triggers
; btrigg    = use trigger choice
; bdelay    = minimum delay for interval between triggers or pattern of triggers to use
; bslice    = slider for slices
; btslice   = time per slice
; Nslice    = number of slices
; bslids    = slider widget
; bslidL    = slider label
; Cframe    = current chosen frame
; mxTime    = [min,max] data timer original
; bidmn     = button mini data timer
; bidmx     = button maxi data timer
; bminT     = text  widget input min Timer
; bmaxT     = text  widget input max Timer
; mimaxT    = Min & Max chosen timer
; Ndata     = number of event data
; Monitor   = number of monitor events
; Strigg    = number of sample trigger events
; Ctrigg    = number of chopper trigger events
; UseCtrigg = use chopper trigger
; UseStrigg = use sample trigger
; BinMod    = 1:Linear , 2:Log , 3:Equal events
; bBin      = button menu for Binmod
; ShowMod   = 1:Histogram , 2:Event Data , 3:Bin Details , etc
; bPlot     = button menu for ShowMod
; vers      = code version date
; removeBad = 0/1: 1 = remove bad events
; bcycle    = time per cycle
; clock     = time (in ns) per clock unit
; clocktol  = tolerance in clock units (used when converting from times in s to clock units)
; tcycle    = time of a cycle
; tslice    = time of each slice (needed for sliding average)
; inst      = instrument
; xsize     = number of pixels along horizontal
; vsize     = numer of pixels along vertical


  @lamp.cbk
  common EV_common1, EV_struct

  ;determine instrument
  valid_instruments = ['d22']

  if keyword_set(instrument) then begin
      instrument = strlowcase(strtrim(instrument, 2))
  endif else begin
      ;Try to get instrument name from Lamp interface
      if keyword_set(inst_value) then instrument = strlowcase(strtrim(inst_value, 2))
  endelse

  if ~keyword_set(instrument) then begin
      error_message = ['No instrument given or found.', 'Use call event_file, instrument="instrument name"', 'Exiting!']
      result = dialog_message(error_message, /error)
      return
  endif

  ;check if instrument is in list of valid instruments
  result = where(strmatch(valid_instruments, instrument))
  if result lt 0 then begin
      line = 'Instrument ' + instrument
      error_message = [line, ' is not yet supported.', 'Exiting!']
      result = dialog_message(error_message, /error)
      return
  endif

  vers = 'Beta v. (1/12/2018) - ' + instrument 

  if (xregistered('event_file_win') eq 0) or (n_elements(EV_struct) eq 0) then begin

      base0  = widget_base(title='Event file analysis (new electronics) - ' + vers, /column, $
                       resource_name='lamptouch')
                       
      Msiz   = 600 ; Message Xsize in pixels
    
      ; Input file
      brow1 = widget_base(  base0, /row)
      bid   = widget_label( brow1, value='    Input File   ', font=ft_b_normal)
      bfile = widget_text(  brow1, value='.lst',    xsize=20, font=ft_propor,   uv='file', /edit)
      bidm  = widget_button(brow1, value='Browse',    /menu , font=ft_b_normal)
      bid   = widget_button(bidm,  value='From Working Path', font=ft_b_normal, uv='browseW')
      bid   = widget_button(bidm,  value='From Data    Path', font=ft_b_normal, uv='browseD')
      bid   = widget_label( brow1, value='   '              , font=ft_propor)
  
      ; Remove events or triggers  
      bid   = widget_label( brow1, value=' Remove: '                     , font=ft_b_normal)
      braw  = widget_button(brow1, value=' Choose ...                   ', font=ft_propor, /menu)
      bid   = widget_button(braw,  value='Events with times < #1 or > #N', font=ft_propor, $
                            uv='remove_extreme_events')
      bid   = widget_button(braw,  value='Outlier events (using poly fit)', font=ft_propor, $
                            uv='remove_outliers')
      bid   = widget_button(braw,  value='Triggers with lapse < min delay', font=ft_propor, $
                            uv='remove_fast_triggers')
      bid   = widget_button(braw,  value='Triggers according to pattern' , font=ft_propor, $
                            uv='apply_trigger_pattern')

      ;Label messages
      bmess1 = widget_label(base0, value=' ', xsize=Msiz, font=ft_b_normal, /align_left)
      bmess2 = widget_label(base0, value=' ', xsize=Msiz, font=ft_b_normal, /align_left)
      bmess3 = widget_label(base0, value=' ', xsize=Msiz, font=ft_b_normal, /align_left)
      bid    = widget_label(base0, value=' ', ysize=5)
      bmess4 = widget_label(base0, value=' ', xsize=Msiz, font=ft_normal,   /align_left)
      bid    = widget_label(base0, value=' ', ysize=5)

      ;Time range box
      brow2 = widget_base(  base0, /row, /frame)
      bid   = widget_label( brow2, value='   '            , font=ft_propor)
      bid   = widget_label( brow2, value='Time Range (s) ', font=ft_b_normal)
      bid   = widget_label( brow2, value='   '            , font=ft_propor)
      bidm  = widget_button(brow2, value='Min'            , font=ft_b_normal, /menu)
      bidmn = widget_button(bidm,  value='0.00'           , font=ft_propor,   uv='setmint')
      bminT = widget_text(  brow2, value='0. ', xsize=20  , font=ft_propor,   uv='mintime', /edit)
      bid   = widget_label( brow2, value='   '            , font=ft_propor)
      bidm  = widget_button(brow2, value='Max'            , font=ft_b_normal, /menu)
      bidmx = widget_button(bidm,  value='0.00'           , font=ft_propor,   uv='setmaxt')
      bmaxT = widget_text(  brow2, value='0. ', xsize=20  , font=ft_propor,   uv='maxtime', /edit)
    
      ;Box for triggers and binning options
      boxBinning = widget_base(base0, /column, /frame)

      ;Triggers and slices
      brow3  = widget_base(boxBinning, /column)
      brow30 = widget_base(brow3 , /row)
      values = [' No ', ' Chopper ', ' Sample ']
      btrigg = cw_bgroup(brow30, values, /row, /exclusive, /frame, font=ft_b_normal, $
                         label_left = 'Use Trigger?  ', set_value=0, uv='usetrigger')
      bid    = widget_label(brow30, value=' ', xsize=3)
      
      ;bdelay used to give min time between triggers
      brow300 = widget_base( brow30, /col, /align_center)
      bdelay  = widget_text (brow300, value='  ', xsize=15, font=ft_propor, uv='delay', /edit)
      ;bid     = widget_label(brow300, value=' ')
      bid     = widget_label(brow30, value='  ', xsize=3)
      brow301 = widget_base( brow30, /col, /align_center)
      bid     = widget_label(brow301, value='(min delay in ms or', xsize=130)
      bid     = widget_label(brow301, value='trigger pattern, e.g. xox)', xsize=130)
                       
      brow31 = widget_base( brow3 , /row)
      bid    = widget_label(brow31, value='Time/cycle (ms) = ', font=ft_b_normal, /align_left)
      bcycle = widget_text (brow31, value=' ', xsize=15,        font=ft_propor, uv='timecycle', /edit)

      brow32 = widget_base( brow3 , /row)
      bid4   = widget_label(brow32, value='Number of slices = ', font=ft_b_normal)
      bid    = widget_label(brow32, value=' ', xsize=3)
      bslice = widget_text (brow32, value=' 1 ', xsize=6,        font=ft_propor, uv='nslice', /edit)
      bid    = widget_label(brow32, value=' ', xsize=20)

      bid6    = widget_label(brow32, value='Time/slice (ms) = ', font=ft_b_normal)
      bid     = widget_label(brow32, value=' ', xsize=10)
      btslice = widget_text (brow32, value=' ',  Xsize=15,       font=ft_propor, uv='tslice', /edit)
      bid     = widget_label(brow32, value=' (used for linear binning only)', xsize=200)
    
      ; Binning  
      brow4  = widget_base(boxBinning, /row)
      bid5   = widget_label( brow4, value=' Binning mode: '      , font=ft_b_normal)
      bbin   = widget_button(brow4, value='Linear bin           ', font=ft_propor, /menu)
      bid    = widget_button(bbin,  value='Linear bin           ', font=ft_propor, uv='bin_lin')
      bid    = widget_button(bbin,  value='Log bin              ', font=ft_propor, uv='bin_log')
      bid    = widget_button(bbin,  value='Inverse log bin      ', font=ft_propor, uv='bin_invlog')
      bid    = widget_button(bbin,  value='Equal #events per bin', font=ft_propor, uv='bin_equ')
      bid8   = widget_label(brow4,  value=' ', xsize=3)
      bdobin = widget_button(brow4, value='BIN ',                  font=ft_b_normal, uv='bin')

      ; View box
      brow5  = widget_base(base0, /row, /frame)
      bid9   = widget_label( brow5, value='  View: '                            , font=ft_b_normal)
      bplot  = widget_button(brow5, value=' Choose ...                         ', font=ft_propor, /menu)
      bid    = widget_button(bplot, value='Slices (x,y,frame)'                  , font=ft_propor, uv='show_bin')
      bid    = widget_button(bplot, value='Total 2D (x,y)'                      , font=ft_propor, uv='show_histo')
      bid    = widget_button(bplot, value='Time vs event number'                , font=ft_propor, uv='show_event')
      bid    = widget_button(bplot, value='Time distribution of events'         , font=ft_propor, uv='show_dist_events')
      bid    = widget_button(bplot, value='Trigger period'                      , font=ft_propor, uv='show_trigger')
      bid    = widget_button(bplot, value='Time distribution of trigger periods', font=ft_propor, uv='show_dist_triggers')
      bid    = widget_button(bplot, value='Trigger history'                     , font=ft_propor, uv='show_trigger_history')
      bid10  = widget_label( brow5, value=' ', xsize=3)
      bshow  = widget_button(brow5, value='SHOW ', font=ft_b_normal, uv='show')
      bid11  = widget_label( brow5, value=' ', xsize=10)
      brow53 = widget_base(  brow5, /row)
      bslidS = widget_slider(brow53, min=0, max=1, xsize=200, font=ft_b_normal, uv='slides', title='')
      bslidL = widget_label( brow53, value=' '   , xsize=1,   font=ft_b_normal, /align_center)
   
      ;Export/Quit box
      brow6     = widget_base(  base0 , /row, /frame)
      brow6b    = widget_base(  brow6, /col)
      bid12     = widget_label( brow6, value=' Data in workspace', font=ft_b_normal)
      bmessData = widget_label( brow6, value=' ', xsize=50,        font=ft_b_normal, /align_center)
      bid       = widget_label( brow6, value='', xsize=20,         font=ft_propor)
      bexpo     = widget_button(brow6, value='Export binned data', font=ft_b_normal, /menu)
      bid       = widget_button(bexpo, value='Current File',       font=ft_propor,   uv='expcur')
      bid       = widget_button(bexpo, value='Apply settings to a list of files and export', $
                                font=ft_propor,   uv='explst')
      bid       = widget_label( brow6, value='', xsize=120,        font=ft_propor)
      bidtest   = widget_button(brow6, value='Test',               font=ft_b_normal, uv='test')
      bid       = widget_label( brow6, value='', xsize=10,         font=ft_propor)
      bidh      = widget_button(brow6, value=' ? ',                font=ft_b_normal, uv='help')
      bid       = widget_label( brow6, value='', xsize=10,         font=ft_propor)
      bquit     = widget_button(brow6, value='Quit',               font=ft_b_normal, uv='quit')

      put_logo, widget_base(brow6b, event_pro='lamp_event_parser')
      widget_control, base0, group_leader=lamp_b1, /realize
      put_logo
      xmanager, 'event_file_win', base0, event_handler='event_file_event', $
                CLEANUP='event_file_dying', /just_reg, /no_block

      EV_struct={bmess1:bmess1, bmess2:bmess2, bmess3:bmess3, bmess4:bmess4, $
                 bmessData:bmessData, bfile:bfile, curFile:'.lst', Wpath:'', $
                 Dpath:path_for_online, Cpath:'', braw: braw, btrigg:btrigg, bdelay:bdelay, $
                 bslice:bslice, btslice:btslice, Nslice:1L, bslidS:bslidS, $
                 Cframe:0L, bslidL:bslidL, mxTime:[0D,0D], bidmn:bidmn, bidmx:bidmx, $
                 bminT:bminT, bmaxT:bmaxT, mimaxT:[0D,0D], Ndata:0L, Monitor:0L, $
                 Strigg:0L, Ctrigg:0L, UseCtrigg:0, UseStrigg:0, BinMod:1, bBin:bBin, $
                 ShowMod:1, bPlot:bPlot, vers:vers, removeBad:0, bcycle:bcycle, $
                 clock:1.e9, clocktol:100000L, tcycle:0D, tslice:0D, inst:instrument, $
                 xsize:256L, ysize:256L}
 
  endif
end


