;+
;
; :Description:
;    Application to read and bin an event mode file.
;    
;    The data format read is as follows :
;
;        Header = 4 * 32 bit words --> Last word contains the number of recorded events
;                                      
;        Data   = 2 * 32 bit words with data stored as:
;
;        01100010 | 01100010 | 01100010 | 01100010 | 01100010 | 01100010 | 01100010 | 01011010
;        0          8         16         24         32         40         48         56
;        YYYY... | XXXX.... | ..MMMMTT | TTTTTTTT | TTTTTTTT | TTTTTTTT | TTTTTTTT | TTTTTTTI
;
;        where:
;            X: X coordinates (variable number of bits)
;            Y: Y coordinates (variable number of bits)
;            M: mfcc (0 for a detector event, 1-15 for other events)
;            T: absolute time in units clock (1/10MHz)
;            I: internal use
;            
;        The number of bits dedicated to the X,Y coordinates of the detector can change 
;        depending on the instrument. Values used are defined at the beginning of rd_tisane
;
;    It seems that at the moment (20 March 2015), even if there 4 bits for the mfcc,
;    the card can only write 0 or 1. Therefore if mfcc = 1, different types of events
;    need to be distinguished as before (lemo in Format=1), using the first 2 bits.
;
;
; :History:
;    E. Pellegrini, Sep 2011    --> first rd_tisane routine 
;    D.R, Jan 2014              --> interface 
;    M. A. Gonzalez, April 2015 --> binning options
;
;-
 
PRO FIT_EXP, X, A, F, pder
    ; F(x) = a * exp(-b*x) 
    F = A[0] * exp(-A[1]*X)
    ; partial derivatives
    pder = [[exp(-A[1]*X)], [-A[0]*X*exp(-A[1]*X)]]
END    

FUNCTION  ROBUST_SIGMA,Y, ZERO=REF, GOODVEC = Q
  ;
  ;+
  ; NAME:
  ; ROBUST_SIGMA
  ;
  ; PURPOSE:
  ; Calculate a resistant estimate of the dispersion of a distribution.
  ; EXPLANATION:
  ; For an uncontaminated distribution, this is identical to the standard
  ; deviation.
  ;
  ; CALLING SEQUENCE:
  ; result = ROBUST_SIGMA( Y, [ /ZERO, GOODVEC = ] )
  ;
  ; INPUT:
  ; Y = Vector of quantity for which the dispersion is to be calculated
  ;
  ; OPTIONAL INPUT KEYWORD:
  ; /ZERO - if set, the dispersion is calculated w.r.t. 0.0 rather than the
  ;   central value of the vector. If Y is a vector of residuals, this
  ;   should be set.
  ;
  ; OPTIONAL OUPTUT KEYWORD:
  ;       GOODVEC = Vector of non-trimmed indices of the input vector
  ; OUTPUT:
  ; ROBUST_SIGMA returns the dispersion. In case of failure, returns
  ; value of -1.0
  ;
  ; PROCEDURE:
  ; Use the median absolute deviation as the initial estimate, then weight
  ; points using Tukey's Biweight. See, for example, "Understanding Robust
  ; and Exploratory Data Analysis," by Hoaglin, Mosteller and Tukey, John
  ; Wiley & Sons, 1983, or equation 9 in Beers et al. (1990, AJ, 100, 32)
  ;
  ; REVSION HISTORY:
  ; H. Freudenreich, STX, 8/90
  ;       Replace MED() call with MEDIAN(/EVEN)  W. Landsman   December 2001
  ;       Don't count NaN values  W.Landsman  June 2010
  ;
  ;-
  On_error,2
  compile_opt idl2

  EPS = 1.0E-20
  IF KEYWORD_SET(REF) THEN Y0=0. ELSE Y0  = MEDIAN(Y,/EVEN)

  ; First, the median absolute deviation MAD about the median:

  MAD = MEDIAN( ABS(Y-Y0), /EVEN )/0.6745

  ; If the MAD=0, try the MEAN absolute deviation:
  IF MAD LT EPS THEN MAD = MEAN( ABS(Y-Y0) )/.80
  IF MAD LT EPS THEN RETURN, 0.0

  ; Now the biweighted value:
  U   = (Y-Y0)/(6.*MAD)
  UU  = U*U
  Q   = WHERE(UU LE 1.0, COUNT)
  IF COUNT LT 3 THEN BEGIN
    PRINT,'ROBUST_SIGMA: This distribution is TOO WEIRD! Returning -1'
    SIGGMA = -1.
    RETURN,SIGGMA
  ENDIF

  N = TOTAL(FINITE(Y),/INT)      ;In case Y has NaN values          ;
  NUMERATOR = TOTAL( (Y[Q]-Y0)^2 * (1-UU[Q])^4 )
  DEN1  = TOTAL( (1.-UU[Q])*(1.-5.*UU[Q]) )
  SIGGMA = N*NUMERATOR/(DEN1*(DEN1-1.))

  IF SIGGMA GT 0. THEN RETURN, SQRT(SIGGMA) ELSE RETURN, 0.

END


FUNCTION ROB_CHECKFIT,Y, YFIT, EPS, DEL, SIG, FRACDEV, NGOOD,W,B,$
  BISQUARE_LIMIT=BLIM
  ;+
  ; NAME:
  ; ROB_CHECKFIT
  ; PURPOSE:
  ; Used by ROBUST_... routines to determine the quality of a fit and to
  ; return biweights.
  ; CALLING SEQUENCE:
  ; status = ROB_CHECKFIT( Y, YFIT, EPS, DEL, SIG, FRACDEV, NGOOD, W, B
  ;       BISQUARE_LIMIT = )
  ; INPUT:
  ; Y     = the data
  ; YFIT  = the fit to the data
  ; EPS   = the "too small" limit
  ; DEL   = the "close enough" for the fractional median abs. deviations
  ; RETURNS:
  ; Integer status. if =1, the fit is considered to have converged
  ;
  ; OUTPUTS:
  ; SIG   = robust standard deviation analog
  ; FRACDEV = the fractional median absolute deviation of the residuals
  ; NGOOD   = the number of input point given non-zero weight in the
  ;   calculation
  ; W     = the bisquare weights of Y
  ; B     = residuals scaled by sigma
  ;
  ; OPTIONAL INPUT KEYWORD:
  ; BISQUARE_LIMIT = allows changing the bisquare weight limit from
  ;     default 6.0
  ; PROCEDURES USED:
  ;       ROBUST_SIGMA()
  ; REVISION HISTORY:
  ; Written, H.T. Freudenreich, HSTX, 1/94
  ;-

  ISTAT = 0

  IF KEYWORD_SET(BLIM) THEN BFAC=BLIM ELSE BFAC=6.

  DEV = Y-YFIT

  SIG=ROBUST_SIGMA(DEV,/ZERO)
  ; If the standard deviation = 0 then we're done:
  IF SIG LT EPS THEN GOTO,DONE

  IF DEL GT 0. THEN BEGIN
    ; If the fraction std. deviation ~ machine precision, we're done:
    Q=WHERE( ABS(YFIT) GT EPS, COUNT )
    IF COUNT LT 3 THEN FRACDEV = 0. ELSE $
      FRACDEV = MEDIAN(ABS( DEV[Q]/YFIT[Q] ),/EVEN )
    IF FRACDEV LT DEL THEN GOTO,DONE
  ENDIF

  ISTAT = 1

  ; Calculate the (bi)weights:
  B = ABS(DEV)/(BFAC*SIG)
  S = WHERE( B GT 1.0,COUNT )  &  IF COUNT GT 0 THEN B[S] = 1.
  NGOOD = N_ELEMENTS(Y)-COUNT

  W=(1.-B^2)
  W=W/TOTAL(W)
  DONE:
  RETURN, ISTAT
END


FUNCTION  ROBUST_LINEFIT,XIN,YIN,YFIT,SIG,SS, NUMIT=THIS_MANY, BISECT=TYPE, $
  Bisquare_Limit=Bisquare_Limit, $
  Close_Factor=Close_Factor
  ;+
  ; NAME:
  ;       ROBUST_LINEFIT
  ;
  ; PURPOSE:
  ;       An outlier-resistant two-variable linear regression.
  ; EXPLANATION:
  ;       Either Y on X or, for the case in which there is no true independent
  ;       variable, the bisecting line of Y vs X and X vs Y is calculated. No
  ;       knowledge of the errors of the input points is assumed.
  ;
  ; CALLING SEQUENCE:
  ;       COEFF = ROBUST_LINEFIT( X, Y, YFIT, SIG, COEF_SIG, [ /BISECT,
  ;                       BiSquare_Limit = , Close_factor = , NumIT = ] )
  ;
  ; INPUTS:
  ;       X = Independent variable vector, floating-point or double-precision
  ;       Y = Dependent variable vector
  ;
  ; OUTPUTS:
  ;       Function result = coefficient vector.
  ;       If = 0.0 (scalar), no fit was possible.
  ;       If vector has more than 2 elements (the last=0) then the fit is dubious.
  ;
  ; OPTIONAL OUTPUT PARAMETERS:
  ;       YFIT = Vector of calculated y's
  ;       SIG  = The "standard deviation" of the fit's residuals. If BISECTOR
  ;               is set, this will be smaller by ~ sqrt(2).
  ;       COEF_SIG  = The estimated standard deviations of the coefficients. If
  ;               BISECTOR is set, however, this becomes the vector of fit
  ;               residuals measured orthogonal to the line.
  ;
  ; OPTIONAL INPUT KEYWORDS:
  ;       NUMIT = the number of iterations allowed. Default = 25
  ;       BISECT  if set, the bisector of the "Y vs X" and "X vs Y" fits is
  ;               determined.  The distance PERPENDICULAR to this line is used
  ;               in calculating weights. This is better when the uncertainties
  ;               in X and Y are comparable, so there is no true independent
  ;               variable.  Bisquare_Limit  Limit used for calculation of
  ;               bisquare weights. In units of outlier-resistant standard
  ;               deviations. Default: 6.
  ;               Smaller limit ==>more resistant, less efficient
  ; Close_Factor  - Factor used to determine when the calculation has converged.
  ;               Convergence if the computed standard deviation changes by less
  ;               than Close_Factor * ( uncertainty of the std dev of a normal
  ;               distribution ). Default: 0.03.
  ; SUBROUTINE CALLS:
  ;       ROB_CHECKFIT
  ;       ROBUST_SIGMA, to calculate a robust analog to the std. deviation
  ;
  ; PROCEDURE:
  ;       For the initial estimate, the data is sorted by X and broken into 2
  ;       groups. A line is fitted to the x and y medians of each group.
  ;       Bisquare ("Tukey's Biweight") weights are then calculated, using the
  ;       a limit of 6 outlier-resistant standard deviations.
  ;       This is done iteratively until the standard deviation changes by less
  ;       than CLOSE_ENOUGH = CLOSE_FACTOR * {uncertainty of the standard
  ;               deviation of a normal distribution}
  ;
  ; REVISION HISTORY:
  ;       Written, H. Freudenreich, STX, 4/91.
  ;       4/13/93 to return more realistic SS's HF
  ;       2/94 --more error-checking, changed convergence criterion HF
  ;       5/94 --added BISECT option. HF.
  ;       8/94 --added Close_Factor and Bisquare_Limit options  Jack Saba.
  ;       4/02 --V5.0 version, use MEDIAN(/EVEN)  W. Landsman
  ;-

  ON_ERROR,2

  IF N_ELEMENTS(THIS_MANY) GT 0 THEN ITMAX = THIS_MANY ELSE ITMAX=25

  IF  N_elements(Close_Factor) EQ 0  THEN Close_Factor = 0.03

  DEL = 5.0E-07
  EPS = 1.0E-20

  N = N_ELEMENTS(XIN)

  ; First, shift X and Y to their centers of gravity:
  X0 = TOTAL(XIN)/N  &  Y0=TOTAL(YIN)/N
  X = XIN-X0       &  Y = YIN-Y0

  CC=FLTARR(2)
  SS=FLTARR(2)
  SIG=0.
  YFIT=YIN
  BADFIT=0
  NGOOD=N

  ; Make sure the independent variables are not all the same.
  XRANGE=MAX(X)-MIN(X)
  AVEX= (TOTAL(ABS(X))/N) > EPS
  IF (XRANGE LT EPS) OR (XRANGE/AVEX LT DEL) THEN BEGIN
    message,'Independent variables the same. No fit possible.',/CON
    RETURN,0.
  ENDIF

  ; First guess:
  LSQ=0
  YP=Y
  IF N GT 5 THEN BEGIN
    ;  We divide the data into 2 groups and fit a line to their X and Y medians.
    S=SORT(X) &  U=X[S]  &  V=Y[S]
    NHALF=N/2-1
    X1=MEDIAN(U[0:NHALF],/EVEN) & X2=MEDIAN(U[NHALF+1:N-1],/EVEN)
    Y1=MEDIAN(V[0:NHALF],/EVEN) & Y2=MEDIAN(V[NHALF+1:N-1],/EVEN)
    IF ABS(X2-X1) LT EPS THEN BEGIN
      ;     The X medians are too close. Select the end-points instead.
      X1=U[0]  &  X2=U[N-1]
      Y1=V[0]  &  Y2=V[N-1]
    ENDIF
    CC[1]=(Y2-Y1)/(X2-X1)  & CC[0]=Y1-CC[1]*X1
    YFIT = CC[0]+CC[1]*X
    ISTAT = ROB_CHECKFIT(YP,YFIT,EPS,DEL,  SIG,FRACDEV,NGOOD,W,S)
    IF NGOOD LT 2 THEN LSQ=1
  ENDIF
  IF (LSQ EQ 1) OR (N LT 6) THEN BEGIN  ; Try a least-squares fit
    SX=TOTAL(X) & SY=TOTAL(Y) & SXY=TOTAL(X*Y) & SXX=TOTAL(X*X)
    D=SXX-SX*SX
    IF ABS(D) LT EPS THEN BEGIN
      PRINT,'ROBUST_LINEFIT: No fit possible.'
      RETURN,0.
    ENDIF
    YSLOP=(SXY-SX*SY)/D      &   YYINT=(SXX*SY-SX*SXY)/D

    IF KEYWORD_SET(TYPE) THEN BEGIN
      ;     Get the X vs Y line.
      SYY=TOTAL(Y*Y)
      D=SYY-SY*SY
      IF ABS(D) LT EPS THEN BEGIN
        PRINT,'ROBUST_LINEFIT: No fit possible.'
        RETURN,0.
      ENDIF
      TSLOP=(SXY-SY*SX)/D   &   TYINT=(SYY*SX-SY*SXY)/D
      ;     Now invert it to get the form Y=a+bX:
      IF ABS(TSLOP) LT EPS THEN BEGIN
        message,'No fit possible.',/CON
        RETURN,0.
      ENDIF
      XSLOP = 1./TSLOP       &   XYINT=-TYINT/TSLOP
      ;     Now calculate the equation of the bisector of the 2 lines:
      IF YSLOP GT XSLOP THEN BEGIN
        A1=YYINT  &  B1=YSLOP  &  R1=SQRT(1.+YSLOP^2)
        A2=XYINT  &  B2=XSLOP  &  R2=SQRT(1.+XSLOP^2)
      ENDIF ELSE BEGIN
        A2=YYINT  &  B2=YSLOP  &  R2=SQRT(1.+YSLOP^2)
        A1=XYINT  &  B1=XSLOP  &  R1=SQRT(1.+XSLOP^2)
      ENDELSE
      YINT = (R1*A2+R2*A1)/(R1+R2)
      SLOP = (R1*B2+R2*B1)/(R1+R2)
      ;     Now find the orthogonal distance to the line. Convert to normal
      ;     coordinates.
      R = SQRT(1.+SLOP^2)  & IF YINT GT 0. THEN R=-R
      U1 = SLOP/R  & U2=-1./R  &  U3=YINT/R
      YP = U1*X+U2*Y+U3  ; = orthog. distance to line
      YFIT = FLTARR(N)   ; to fool ROB_CHECKFIT
      SS=YP
    ENDIF ELSE BEGIN
      SLOP=YSLOP               &   YINT=YYINT
      YFIT = YINT+SLOP*X
    ENDELSE
    CC = [YINT,SLOP]
    ISTAT = ROB_CHECKFIT(YP,YFIT,EPS,DEL,  SIG,FRACDEV,NGOOD,W,S)
  ENDIF

  IF ISTAT EQ 0 THEN GOTO,AFTERFIT

  IF NGOOD LT 2 THEN BEGIN
    message,'Data Dangerously Weird. Fit Questionable.',/CON
    BADFIT=1
    GOTO,AFTERFIT
  ENDIF

  ; Now iterate until the solution converges:
  SIG_1= (100.*SIG) < 1.0E20
  CLOSE_ENOUGH = Close_Factor * SQRT(.5/(N-1)) > DEL
  DIFF= 1.0E20
  NIT = 0
  WHILE( (DIFF GT CLOSE_ENOUGH) AND (NIT LT ITMAX) ) DO BEGIN
    NIT=NIT+1
    SIG_2=SIG_1
    SIG_1=SIG
    SX=TOTAL(W*X) & SY=TOTAL(W*Y) & SXY=TOTAL(W*X*Y) & SXX=TOTAL(W*X*X)
    D=SXX-SX*SX
    IF ABS(D) LT EPS THEN BEGIN
      message,'No fit possible.',/CON
      RETURN,0.
    ENDIF
    YSLOP = (SXY-SX*SY)/D      &   YYINT = (SXX*SY-SX*SXY)/D
    SLOP = YSLOP               &   YINT = YYINT
    IF KEYWORD_SET(TYPE) THEN BEGIN
      ;    Get the X vs Y line.
      SYY=TOTAL(W*Y*Y)
      D=SYY-SY*SY
      IF ABS(D) LT EPS THEN BEGIN
        PRINT,'ROBUST_LINEFIT: No fit possible.'
        RETURN,0.
      ENDIF
      TSLOP=(SXY-SY*SX)/D   &   TYINT=(SYY*SX-SY*SXY)/D
      ;    Now invert it to get the form Y=a+bX:
      IF ABS(TSLOP) LT EPS THEN BEGIN
        PRINT,'ROBUST_LINEFIT: No fit possible.'
        RETURN,0.
      ENDIF
      XSLOP=1./TSLOP       &   XYINT=-TYINT/TSLOP
      ;    Now calculate the equation of the bisector of the 2 lines:
      IF YSLOP GT XSLOP THEN BEGIN
        A1=YYINT  &  B1=YSLOP  &  R1=SQRT(1.+YSLOP^2)
        A2=XYINT  &  B2=XSLOP  &  R2=SQRT(1.+XSLOP^2)
      ENDIF ELSE BEGIN
        A2=YYINT  &  B2=YSLOP  &  R2=SQRT(1.+YSLOP^2)
        A1=XYINT  &  B1=XSLOP  &  R1=SQRT(1.+XSLOP^2)
      ENDELSE
      YINT=(R1*A2+R2*A1)/(R1+R2)
      SLOP=(R1*B2+R2*B1)/(R1+R2)
      R=SQRT(1.+SLOP^2)  & IF YINT GT 0. THEN R=-R
      U1=SLOP/R  & U2=-1./R  &  U3=YINT/R
      YP=U1*X+U2*Y+U3  ; = orthog distance to line
      YFIT=FLTARR(N) & YFIT[*]=0.
      SS=YP
    ENDIF ELSE BEGIN
      YFIT = YINT+SLOP*X
    ENDELSE
    CC=[YINT,SLOP]
    ISTAT=ROB_CHECKFIT(YP,YFIT,EPS,DEL,  SIG,FRACDEV,NGOOD,W,S, $
      Bisquare_Limit=Bisquare_Limit )

    IF ISTAT EQ 0 THEN GOTO,AFTERFIT
    IF NGOOD LT 2 THEN BEGIN
      PRINT,'ROBUST_LINEFIT: Data Dangerously Weird. Fit Questionable.'
      BADFIT=1
      GOTO,AFTERFIT
    ENDIF
    DIFF = (ABS(SIG_1-SIG)/SIG) < (ABS(SIG_2-SIG)/SIG)
  ENDWHILE

  AFTERFIT:
  ; Untranslate the coefficients
  CC[0] = CC[0]+Y0-CC[1]*X0

  IF N_PARAMS(0) GT 2 THEN YFIT = CC[0] + CC[1]*XIN
  IF KEYWORD_SET(BISECT) THEN RETURN,CC

  IF (N_PARAMS(0) GT 3) AND (SIG GT EPS) AND (NGOOD GT 2) THEN BEGIN
    ; Here we use an empirical formula to approximate the standard deviations
    ; of the coefficients. They are usually accurate to ~ 25%.
    SX2 = TOTAL(W*X*X)
    UU = S*S
    DEV = YIN-YFIT
    Y0 = TOTAL( W*DEV )
    Q = WHERE(UU LE 1.0,COUNT)
    DEN1 = ABS(TOTAL( (1.-UU[Q])*(1.-5.*UU[Q]) ))
    SIG = ROBUST_SIGMA(DEV,/ZERO)
    ; Now empirically derived estimates of the uncertainties:
    SS[0] = SIG/SQRT(DEN1)/1.105
    SS[1] = SS[0]/SQRT(SX2)
    ; Take the X shift into account:
    SS[0] = SQRT(SS[0]^2+X0*SS[1]^2)
  ENDIF

  IF BADFIT EQ 1 THEN CC=[CC,0.]

  RETURN,CC
END


FUNCTION ROBUST_POLY_FIT,X,Y,NDEG,YFIT,SIG, NUMIT=THIS_MANY, DOUBLE=DOUBLE
  ;+
  ; NAME:
  ; ROBUST_POLY_FIT
  ;
  ; PURPOSE:
  ; An outlier-resistant polynomial fit.
  ;
  ; CALLING SEQUENCE:
  ; COEFF = ROBUST_POLY_FIT(X,Y,NDEGREE, [ YFIT,SIG, /DOUBLE, NUMIT=] )
  ;
  ; INPUTS:
  ; X = Independent variable vector, floating-point or double-precision
  ; Y = Dependent variable vector
  ;       NDEGREE - integer giving degree of polynomial to fit, maximum = 6
  ; OUTPUTS:
  ; Function result = coefficient vector, length NDEGREE+1.
  ; IF COEFF=0.0, NO FIT! If N_ELEMENTS(COEFF) > degree+1, the fit is poor
  ; (in this case the last element of COEFF=0.)
  ; Either floating point or double precision.
  ;
  ; OPTIONAL OUTPUT PARAMETERS:
  ; YFIT = Vector of calculated y's
  ; SIG  = the "standard deviation" of the residuals
  ;
  ; OPTIONAL INPUT KEYWORD:
  ;       /DOUBLE - If set, then force all computations to double precision.
  ;       NUMIT - Maximum number of iterations to perform, default = 25
  ; RESTRICTIONS:
  ; Large values of NDEGREE should be avoided. This routine works best
  ; when the number of points >> NDEGREE.
  ;
  ; PROCEDURE:
  ; For the initial estimate, the data is sorted by X and broken into
  ; NDEGREE+2 sets. The X,Y medians of each set are fitted to a polynomial
  ;  via POLY_FIT.   Bisquare ("Tukey's Biweight") weights are then
  ; calculated, using a limit  of 6 outlier-resistant standard deviations.
  ; The fit is repeated iteratively until the robust standard deviation of
  ; the residuals changes by less than .03xSQRT(.5/(N-1)).
  ;
  ; PROCEDURES CALLED:
  ;        POLY(), POLY_FIT()
  ;       ROB_CHECKFIT()
  ; REVISION HISTORY
  ; Written, H. Freudenreich, STX, 8/90. Revised 4/91.
  ; 2/94 -- changed convergence criterion
  ;        Added /DOUBLE keyword, remove POLYFITW call  W. Landsman  Jan 2009
  ;-

  ON_ERROR,2
  COMPILE_OPT IDL2

  EPS   = 1.0E-20
  DEL   = 5.0E-07
  DEGMAX= 6

  IF N_ELEMENTS(THIS_MANY) GT 0 THEN ITMAX=THIS_MANY ELSE ITMAX=25

  BADFIT=0

  NPTS = N_ELEMENTS(X)
  MINPTS=NDEG+1
  IF (NPTS/4*4) EQ NPTS THEN NEED2 = 1 ELSE NEED2 = 0
  N3 = 3*NPTS/4  &  N1 = NPTS/4

  ; If convenient, move X and Y to their centers of gravity:
  IF NDEG LT DEGMAX THEN BEGIN
    X0=TOTAL(X)/NPTS  &  Y0=TOTAL(Y)/NPTS
    U=X-X0            &  V=Y-Y0
  ENDIF ELSE BEGIN
    U=X               &  V=Y
  ENDELSE

  ; The initial estimate.

  ; Choose an odd number of segments:
  NUM_SEG = NDEG+2
  IF (NUM_SEG/2*2) EQ NUM_SEG THEN NUM_SEG =NUM_SEG+1
  MIN_PTS = NUM_SEG*3
  IF NPTS LT 10000 THEN BEGIN ;MIN_PTS THEN BEGIN
    ;  Settle for least-squares:
    LSQFIT = 1
    CC = POLY_FIT( U, V, NDEG, YFIT , DOUBLE=DOUBLE)
  ENDIF ELSE BEGIN
    ;  Break up the data into segments:
    LSQFIT = 0
    Q = SORT(U)
    U = U[Q]  &  V = V[Q]
    N_PER_SEG = REPLICATE( NPTS/NUM_SEG, NUM_SEG)

    ;  Put the leftover points in the middle segment:
    N_LEFT = NPTS - N_PER_SEG[0]*NUM_SEG
    N_PER_SEG[NUM_SEG/2] = N_PER_SEG[NUM_SEG/2] + N_LEFT
    R = DBLARR(NUM_SEG)  &  S = DBLARR(NUM_SEG)
    R[0]=MEDIAN( U[0:N_PER_SEG[0]-1],/EVEN )
    S[0]=MEDIAN( V[0:N_PER_SEG[0]-1],/EVEN )
    I2 = N_PER_SEG[0]-1
    FOR I=1,NUM_SEG-1 DO BEGIN
      I1 = I2 + 1
      I2 = I1 + N_PER_SEG[I] - 1
      R[I] = MEDIAN( U[I1:I2], /EVEN)      &  S[I] = MEDIAN( V[I1:I2],/EVEN )
    ENDFOR
    ;  Now fit:
    CC = POLY_FIT( R,S, NDEG, DOUBLE=DOUBLE )
    YFIT = POLY(U,CC)
  ENDELSE

  ISTAT = ROB_CHECKFIT(V,YFIT,EPS,DEL,  SIG,FRACDEV,NGOOD,W,S)

  IF ISTAT EQ 0 THEN GOTO,AFTERFIT

  IF NGOOD LT MINPTS THEN BEGIN
    IF LSQFIT EQ 0 THEN BEGIN
      ;  Try a least-squares:
      CC = POLY_FIT( U, V, NDEG, YFIT, DOUBLE=DOUBLE )
      ISTAT = ROB_CHECKFIT(V,YFIT,EPS,DEL,  SIG,FRACDEV,NGOOD,W,S)
      IF ISTAT EQ 0 THEN GOTO,AFTERFIT
      NGOOD = NPTS-COUNT
    ENDIF
    IF NGOOD LT MINPTS THEN BEGIN
      PRINT,'ROBUST_POLY_FIT: No Fit Possible!'
      RETURN,0.
    ENDIF
  ENDIF

  ; Now iterate until the solution converges:
  CLOSE_ENOUGH = .03*SQRT(.5/(NPTS-1)) > DEL
  DIFF= 1.0E10
  SIG_1= (100.*SIG) < 1.0E20
  NIT = 0
  WHILE( (DIFF GT CLOSE_ENOUGH) AND (NIT LT ITMAX) ) DO BEGIN
    NIT=NIT+1
    SIG_2=SIG_1
    SIG_1=SIG
    ; We use the "obsolete" POLYFITW routine because it allows us to input weights
    ; rather than measure errors
    g = where(W gt 0, Ng)
    if Ng LT N_elements(w) then begin    ;Throw out points with zero weight
      u = u[g]
      v = v[g]
      w = w[g]
    endif
    CC = POLY_FIT( U, V, NDEG, YFIT, MEASURE_ERRORS = 1/W^2, DOUBLE=DOUBLE )
    ISTAT = ROB_CHECKFIT(V,YFIT,EPS,DEL,  SIG,FRACDEV,NGOOD,W,S)
    IF ISTAT EQ 0 THEN GOTO,AFTERFIT
    IF NGOOD LT MINPTS THEN BEGIN
      PRINT,'ROBUST_POLY_FIT: Questionable Fit!'
      BADFIT=1
      GOTO,AFTERFIT
    ENDIF
    DIFF = (ABS(SIG_1-SIG)/SIG) < (ABS(SIG_2-SIG)/SIG)
  ENDWHILE

  ;IF NIT GE ITMAX THEN PRINT,'ROBUST_POLY_FIT: Did not converge in',ITMAX,$
  ;' iterations!'

  AFTERFIT:
  CC=REFORM(CC)

  IF NDEG LT DEGMAX THEN BEGIN
    CASE NDEG OF
      1: CC[0] = CC[0]-CC[1]*X0 + Y0
      2: BEGIN
        CC[0] = CC[0]-CC[1]*X0+CC[2]*X0^2 + Y0
        CC[1] = CC[1]-2.*CC[2]*X0
      END
      3: BEGIN
        CC[0] = CC[0]-CC[1]*X0+CC[2]*X0^2-CC[3]*X0^3 + Y0
        CC[1] = CC[1]-2.*CC[2]*X0+3.*CC[3]*X0^2
        CC[2] = CC[2]-3.*CC[3]*X0
      END
      4: BEGIN
        CC[0] = CC[0]-   CC[1]*X0+CC[2]*X0^2-CC[3]*X0^3+CC[4]*X0^4+ Y0
        CC[1] = CC[1]-2.*CC[2]*X0+3.*CC[3]*X0^2-4.*CC[4]*X0^3
        CC[2] = CC[2]-3.*CC[3]*X0+6.*CC[4]*X0^2
        CC[3] = CC[3]-4.*CC[4]*X0
      END
      5: BEGIN
        CC[0] = CC[0]-  CC[1]*X0+CC[2]*X0^2-CC[3]*X0^3+CC[4]*X0^4-CC[5]*X0^5+ Y0
        CC[1] = CC[1]-2.*CC[2]*X0+ 3.*CC[3]*X0^2- 4.*CC[4]*X0^3+5.*CC[5]*X0^4
        CC[2] = CC[2]-3.*CC[3]*X0+ 6.*CC[4]*X0^2-10.*CC[5]*X0^3
        CC[3] = CC[3]-4.*CC[4]*X0+10.*CC[5]*X0^2
        CC[4] = CC[4]-5.*CC[5]*X0
      END
    ENDCASE
  ENDIF

  ; Calculate the fit at points X:
  IF( N_PARAMS(0) GT 3 )THEN YFIT=POLY(X,CC)

  IF BADFIT EQ 1 THEN CC=[CC,0.]

  RETURN,CC
END


function graphselect, x, y, n, NOFILL=nofill, RESTORE=restore
  ;+
  ; NAME:
  ;       GRAPHSELECT
  ;
  ; PURPOSE:
  ; Select points within an area of a graph defined by the cursor.
  ;       Return the indices of these points.
  ;
  ; CALLING SEQUENCE:
  ;       Result = GRAPHSELECT(X, Y [,N][,/NOFILL][,/RESTORE])
  ;
  ; INPUTS:
  ;       X: array of x values on the plot.
  ;       Y: array of y values on the plot.
  ;
  ; KEYWORD PARAMETERS:
  ;       /NOFILL - Set this keyword to inhibit filling of the
  ;                 defined region on completion.
  ;       /RESTORE - Set this keyword to restore the display to its
  ;                  original state upon completion.  N.B., this
  ;                  doesn't do such a great job, but that's DEFROI's
  ;                  fault!
  ;
  ; OUTPUTS:
  ;       Returns the indices of the points that are within the
  ;       selected area.
  ;
  ; OPTIONAL OUTPUTS:
  ;       N: the number of points inside the selected area.
  ;
  ; COMMON BLOCKS:
  ;       None.
  ;
  ; SIDE EFFECTS:
  ;       The area selected is drawn on the plot window and, unless the
  ;       /NOFILL keyword is set, filled in.
  ;
  ; RESTRICTIONS:
  ;       Only works for interactive, pixel oriented devices with a cursor.
  ;       Selected region must have less than 1001 vertices.
  ;       Can only be used in most recently created window.
  ;
  ; PROCEDURES CALLED:
  ;       DEFROI()
  ;
  ; EXAMPLE:
  ;       Create some x and y values...
  ;       IDL> x = indgen(20)
  ;       IDL> y = randomn(seed,20)
  ;
  ;       Plot the points...
  ;       IDL> plot, x, y, psym=4
  ;
  ;       Select a region of the plot and return the indices
  ;       of these points...
  ;       IDL> indx = graphselect(x,y,n_indx)
  ;
  ; NOTES:
  ;       If you are trying to obtain the indices of an image, you need to
  ;       pass in arrays of x and y values, e.g., they should be arrays of
  ;       size N_elements(x) by N_elements(y) in which the x array is
  ;       repeated in N_elements(y) rows and the y array is repeated in
  ;       N_elements(x) columns.  The routine will return the indices of
  ;       points within the selected polygon.
  ;
  ; MODIFICATION HISTORY:
  ; Written by Carl Heiles. 12 Sep 1998.
  ;       Souped up by Tim Robishaw 19 Aug 2003.
  ;-

  ; IS A WINDOW EVEN OPEN...
  if (!d.window eq -1L) then begin
    message, 'No window is open!', /INFO
    return, -1
  endif

  ; DEFROI USES THE !D.TABLE_SIZE-1th COLOR TO OVERPLOT...
  ; SO, MAKE SURE WE SHUT OFF COLOR DECOMPOSITION...
  device, get_decomposed=decomposed
  device, decomposed=0
  inside = defroi(!d.x_size, !d.y_size, $
    NOFILL=keyword_set(NOFILL), $
    RESTORE=keyword_set(RESTORE))
  device, decomposed=decomposed

  ; MAKE A MASK WHICH IS ZERO OUTSIDE THE SELECTED REGION, ONE INSIDE...
  mask = bytarr(!d.x_size,!d.y_size)
  mask[inside] = 1B

  ; CONVERT DATA POINTS FROM DATA COORDINATES TO DEVICE COORDINATES...
  data2dev = convert_coord(x,y,/DATA,/TO_DEVICE)
  xdev = reform(data2dev[0,*])
  ydev = reform(data2dev[1,*])


  ; RETURN THE INDICES INSIDE THE SELECTED REGION...
  return, where(mask[xdev,ydev] eq 1B, n)

end; graphselect


;+
; Returns the complement of an index array.
;
; :Examples:
;    For example, try::
;
;       IDL> print, mg_complement([0, 9, 3, 6, 7], 10)
;                  1           2           4           5           8
;
; :Returns:
;    `lonarr` or `-1L` if complement is empty
;
; :Params:
;    indices : in, required, type=lonarr
;       indices to complement
;    n : in, required, type=integer type
;       number of elements in full array
;
; :Keywords:
;    count : out, optional, type=long
;       set to a named variable to return the number of elements in the
;       complement
;-
function mg_complement, indices, n, count=ncomplement
  compile_opt strictarr

  all = bytarr(n)
  if (indices[0] ne -1L) then all[indices] = 1B
  return, where(all eq 0B, ncomplement)
end


function rd_tisane, filename, X_DETECTOR=Xdet, Y_DETECTOR=Ydet, T_DETECTOR=Dtimer, $
                              T_MONITOR=Mtimer, T_SAMPLE=Stimer, T_CHOPPER=Ctimer, W=Wi
;+
;******* *********
;** User callable
;** W1 = rd_tisane(filename, X_DETECTOR=Xdet, Y_DETECTOR=Ydet, T_DETECTOR=Dtimer, 
;                            T_CHOPPER=Ctimer, T_SAMPLE=Stimer, T_MONITOR=Mtimer)
;** return detector total histogram
;**  X_DETECTOR = Xdet   = X address of event in detector
;**  Y_DETECTOR = Ydet   = Y address of event in detector
;**  T_DETECTOR = Dtimer = time of event in detector
;**  T_MONITOR  = Mtimer = time of event registered in channel 1 (equivalent lemo=), usually connected to monitor 1
;**  T_SAMPLE   = Stimer = time of event registered in channel 2 (equivalent lemo=1), usually connected to sample equipment (e.g. shear machine)
;**  T_CHOPPER  = Ctimer = time of event registered in channel 3 (equivalent lemo=2), usually connected to chopper
;-

  forward_function rdid
  common c_event_file, EV_struct
  
  ; Supported instruments
  ; 'd22'    (Dec 2014 - )
  ; 'salsa'  (Sep 2016 - )
  ; 'd11'    (Feb 2017 - )

  case ev_struct.inst of
    'd22': begin
             ; Detector size = 128 x 256
             xbits = 7 & Xsize = 128L
             ybits = 8 & Ysize = 256L
           end
    'salsa': begin
             ; Detector size = 256 x 256
             xbits = 8 & Xsize = 256L
             ybits = 8 & Ysize = 256L
           end
    'd11': begin
             ; Detector size = 128 x 128
             xbits = 8 & Xsize = 256L
             ybits = 8 & Ysize = 256L
           end
    else: begin
            line = 'Case EV_struct.inst =  ' + ev_struct.inst
            error_message = [line, ' not found. Exiting!']
            result = dialog_message(error_message, /error)
            return, 0
          end       
  endcase
  
  ;store detector dimensions in structure
  ev_struct.xsize = Xsize
  ev_struct.ysize = Ysize
  
  ;number of bits and shifts needed to read mfcc and time
  mbits = 4  & mshift = -18
  tbits = 41 & tshift = -22 
  
  ; If the file to read does not exist, return.
  if not file_test(filename) then return, 0
  
  ;--> MAG 16/9/2016: Commented block. Decided to use only event_mode file!
  ; Do we have an histogram file? 
  ;fileGZ = strmid(filename,0,strpos(filename,'.lst')) + '.gz'
  ;fileNX = strmid(filename,0,strpos(filename,'.lst')) + '.nxs'
  ;if file_test(fileNX) then okHist = fileNX else $
  ;  if file_test(fileGZ) then okHist = fileGZ else okHist = ''

  ; Read associated Nexus/ASCII file and get 2D histogram
  ;if okHist gt ' ' then begin
  ;  counts = rdid(['d22','lss','1','0'], '', okHist, STATUS, DATP)
  ;  if STATUS ne 0 then okHist='' else give_datp, DATP, W=Wi
  ;endif

  ; If no associated Nexus/ASCII file read, then create 2D histogram from detector events
  ;if okHist eq '' then counts = lonarr(Xsize,Ysize)
  
  counts = lonarr(Xsize,Ysize)  ;To store total 2D image (sum over all events)
  eventBlockSize = 1000000L     ;To adjust depending on resources
  maxEvents = 4294967295UL 
  stime = systime(1)
    
  ; Creates an error handler in case where an error occurred when parsing the file.
  on_ioerror, READ_ERROR
  u = -1 ; The file unit.
  
  ; Opens the file for reading in binary mode.
  openr, u, filename, /get_lun
  
  ; The header of the file will be read as 4 words of 32bits.
  header = ulonarr(4)
  readu, u, header & byteorder, header, /swap_if_big_endian
  
  ; The last element of the header gives the number of registered events.
  nEvents = header[3]
  if nEvents gt maxEvents then begin
    error_message = ['Number of events > 4,294,967,296', $
                     'ULONG arrays used in the code cannot handle such a large number of events!']
    result = dialog_message(error_message, /error)
    return, 0
  endif
  
  ; Predefine arrays to speed_up allocations
  Xdet   = uintarr(nEvents)                               ; X detector address
  Ydet   = uintarr(nEvents)                               ; Y detector address
  Dtimer = ulon64arr(nEvents) & idx0 = -1 & Dhits = 0ULL  ; time of detector event
  Mtimer = ulon64arr(nEvents) & idxM = -1 & Mhits = 0ULL  ; time of monitor event
  Stimer = ulon64arr(nEvents) & idxS = -1 & Shits = 0ULL  ; time of sample event
  Ctimer = ulon64arr(nEvents) & idxC = -1 & Chits = 0ULL  ; time of chopper event
    
  ; To not trigger memory overhead when reading, the file will be read by blocks of eventBlockSize
  nBlocks = nEvents/eventBlockSize
  if (nBlocks*eventBlockSize lt nEvents) then nBlocks +=1
  
  events = ulon64arr(nEvents < eventBlockSize) ; Input buffer 8 words of 8 bits events
  remind = nEvents
  
  for i = 1, nBlocks do begin
  
    if remind lt eventBlockSize then events  = ulon64arr(remind) $ ; Last loop
    else remind -= eventBlockSize
  
    readu, u, events  &  byteorder, events, /swap_if_big_endian
  
    mfcc = ishft(events, mshift) and 2ULL^mbits-1
  
    ;MFCC values
    ;mfcc_unique = mfcc[UNIQ(mfcc, SORT(mfcc))] & print, mfcc_unique
  
    ; Detector events --> MFCC = 0 / Other (monitor/chopper/sample) --> MFCC > 0
    ; **************************************************************************
  
    idx0 = where(mfcc eq 0 , complement=idx1) & ztmp1 = 0 
  
    if idx0[0] gt -1 then begin
      
      ztmp1 = n_elements(idx0)
      
      Devent        = events[idx0] & idx0=0
      Xdet[Dhits]   = fix( ishft(Devent, -ybits) and (2ULL^xbits-1) )
      Ydet[Dhits]   = fix( Devent                and (2ULL^ybits-1) )
      Dtimer[Dhits] = ishft(Devent, tshift)       and (2ULL^tbits-1)
  
      ;Remove here events where Dtimer=0 TO DO?
      ;*********************************
  
      ; Fill histogram
      for n = Dhits, Dhits+n_elements(Devent)-1 do counts[Xdet[n], Ydet[n]] += 1L
  
      Dhits += n_elements(Devent)
      Devent = 0  ; Free memory
      
    endif ;endif detector events --> Xdet, Ydet, Dtimer
  
    ; Other events (MFCC>0) --> monitor, sample trigger, chopper trigger, etc.
    ; ************************************************************************
    if idx1[0] gt -1 then begin
  
          ;Check MFCC value and put event in corresponding array
          idx1 = where(mfcc eq 1) ;At the moment (April 2015), only MFCC = 0/1 allowed. Any event with MFCC between 2 and 15 will be ignored.
          idx2 = where(mfcc gt 1) ;Check number of events with MFCC > 1
          if idx2[0] gt -1 then ztmp2 = n_elements(idx2) else ztmp2 = 0 
          
          Levent = events[idx1] & idx1 = 0 & idx2 = 0
          lemo   = Levent and (2ULL^2-1) ;(2ULL^18-1) In theory the first 18 bits are used to store the type of event, but as only 3 types are considered for the moment we read only 2 bits
  
          ; Monitor --> lemo = 0
          idxM = where(lemo eq 0) & ztmp3 = 0 
          if idxM[0] gt -1 then begin
            ztmp3 = n_elements(idxM)
            Mevent = Levent[idxM] & idxM = 0
            Mtimer[Mhits] = ishft(Mevent, tshift) and (2ULL^tbits-1)
            Mhits += n_elements(Mevent)
            Mevent = 0  ; Free memory
          endif
  
          ; Sample trigger --> lemo = 1
          idxS = where(lemo eq 1) & ztmp4 = 0
          if idxS[0] gt -1 then begin
            ztmp4 = n_elements(idxS)
            Sevent = Levent[idxS] & idxS = 0
            Stimer[Shits] = ishft(Sevent, tshift) and (2ULL^tbits-1)
            Shits += n_elements(Sevent)
            Sevent = 0  ; Free memory
          endif

          ; Chopper trigger --> lemo = 2
          idxC = where(lemo eq 2) & ztmp5 = 0
          if idxC[0] gt -1 then begin
            ztmp5 = n_elements(idxC)
            Cevent = Levent[idxC] & idxC = 0
            Ctimer[Chits] = ishft(Cevent, tshift) and (2ULL^tbits-1)
            Chits += n_elements(Cevent)
            Cevent = 0  ; Free memory
          endif
  
          ; Lemo > 2
          idx1 = where(lemo gt 2) & ztmp6 = 0
          if idx1[0] gt -1 then ztmp6 = n_elements(idx1)
          idx1 = 0
          
          ; Check events
          if i eq 1 then print, 'Block - Events read - Detector e - Monitor e - Sample e - Chopper e - MFCC>1 - Lemo>2 - Sum'
          print, ' ', strtrim(i,2), '   ', n_elements(mfcc), ' ', ztmp1, ' ', ztmp3, ' ', ztmp4, ' ', ztmp5, ' ', ztmp2, ' ', ztmp6, ztmp1+ztmp2+ztmp3+ztmp4+ztmp5+ztmp6
   
    endif ; endif MFCC = 1
  
  endfor
    
  if Dhits gt 0 then begin
    Dtimer = Dtimer[0:Dhits-1] 
    Xdet = Xdet[0:Dhits-1]
    Ydet = Ydet[0:Dhits-1]
  endif else begin
    Dtimer = -1
    Xdet = -1
    Ydet = -1
  endelse
  if Mhits gt 0 then Mtimer = Mtimer[0:Mhits-1] else Mtimer = -1
  if Shits gt 0 then Stimer = Stimer[0:Shits-1] else Stimer = -1
  if Chits gt 0 then Ctimer = Ctimer[0:Chits-1] else Ctimer = -1
  
  ;Ongoing work to explore detector dead times for D11 
  if (ev_struct.inst eq 'd11') then begin
     ;Remove monitor events stored in position 0,0     
     monitor_events = where(Xdet eq 0 and Ydet eq 0, complement = detector_events)
     Xtmp = Xdet[detector_events]
     Ytmp = Ydet[detector_events]
     Dtmp = Dtimer[detector_events]
     ;Select detector region. Default = All = [0, 255]
     xlimits = [60, 61]
     ylimits = [0, 255] 
     select = where(Xtmp ge xlimits[0] and Xtmp le xlimits[1] and Ytmp ge ylimits[0] and Ytmp le ylimits[1])
     if select[0] gt -1 then begin
       Xsel = Xtmp[select]
       Ysel = Ytmp[select]
       Dsel = Dtmp[select]
     endif
     ;Free memory
     Xtmp = 0 & Ytmp = 0 & Dtmp = 0 & monitor_events = 0 & detector_events = 0 & select = 0
     time_intervals = (Dsel[1:*] - Dsel[0:-2]) / 10. ;in microseconds
     bin_size = 1.
     time_distrib = histogram(time_intervals, binsize = bin_size)
     time_axis = findgen(n_elements(time_distrib))* bin_size + 0.5 * bin_size
     ;Fit to A[0]*exp(-A[1]*t)
     sigmas = sqrt(time_distrib)
     zeros = where(time_distrib lt 1)
     sigmas[zeros] = 1.
     weights = 1.0 / sigmas^2
     A = [600.0, 2.e-3]
     yfit = CURVEFIT(time_axis, time_distrib, weights, A, function_name='FIT_EXP') 
     plot = plot(time_axis, time_distrib, /xlog, linestyle=' ', symbol='dot')
     plot = plot(time_axis, yfit, /overplot, color='red')
     ;plot = errorplot(time_axis, time_distrib, sigmas, /xlog, linestyle=' ', symbol='o')
     ;print output file
     outputfile = 'output_' + file_basename(EV_struct.curfile, '.lst') + '.txt'
     openw, u, outputfile, /get_lun
     printf, u, '# Fit parameters = ', A
     printf, u, '# Time, histogram, fit'
     for i = 0, n_elements(time_axis)-1 do begin
        printf, u, time_axis[i], time_distrib[i], yfit[i]
     endfor
     close, u
     free_lun, u
  endif
  
  print, ' '
  print, strtrim(round((systime(1)-stime)*10.)/10.,1),' seconds to read ', $
    strtrim(nEvents,1),' Events.'     
  print, 'Number of detector events = ', strtrim(Dhits,1)
  print, 'Number of monitor events  = ', strtrim(Mhits,1)
  print, 'Number of sample events   = ', strtrim(Shits,1)
  print, 'Number of chopper events  = ', strtrim(Chits,1)
  print, 'Counts in total histogram = ', strtrim(total(counts),1) & wait,001
  
  ; The file was parsed properly. Deallocate the file unit.
  free_lun,u
  
  return, counts
  
  ; Case where an error occured when parsing the file.
  READ_ERROR:
  if u gt 0 then free_lun, u
  lamp_error
  return, 0
  
end



;+
; :Description:
;    Modifies a structure changing the type, dimensionality, and contents of an 
;    existing structure field. It can go inside nested structures.
;    
;    The call is 
;       > modify_struc, struct, tagkeys, 0, data
;
; :Parameters (all required):
;    struct : in/out, type="structure" - structure to modify.
;    tagkeys: in, type="string" or "string array"  - case insensitive tag name describing 
;             the structure field to modify. To modify a field in a structure inside a structure
;             give the ordered list of structure names and the field name at the end, e.g.
;             ['struct1', 'struct2', 'struct3', 'field'].
;    0      : in - needed for recursive calls.
;    data   : in, any type - data that will replace the current contents of the structure field,
;             modifying the type and dimensions of the field.
;
;  :Example:
;    To modify the data in a 'nexus' structure obtained from reading a NeXus file with h5_parse, use
;       > modify_struc, nexus, ['entry0', 'data', 'multidetector_linear_data', '_data'], 0, data
;-
pro modify_struc, struct, tagkeys, ilevel, data

  ;Check that input is a structure.
  if size(struct, /tname) ne 'STRUCT' then begin
    message, 'First argument is not a structure'
  endif

  ;Get list of structure tags.
  tags = tag_names(struct)
  ntags = n_elements(tags)

  ;Check that requested field exists in input structure.
  ctag = strupcase(strtrim(tagkeys, 2))   ;canoncial form of tag
  itag = where(tags eq ctag[ilevel], nmatch)
  if nmatch eq 0 then begin
    message, 'structure does not contain ' + ctag + ' field'
    return
  endif
  itag = itag[0]        ;convert to scalar

  ;Choose tag name for the output structure.
  ;if keyword_set(newtag) then otag = newtag else otag = ctag
  otag = ctag[ilevel]

  ;Copy any fields that precede target field. Then add target field.
  if itag eq 0 then begin

    tmpstruct = struct.(itag)
    if size(tmpstruct, /tname) ne 'STRUCT' then new = create_struct(otag, data) $
    else begin
      modify_struc, tmpstruct, tagkeys, ilevel+1, data
      new = create_struct(otag, tmpstruct)
    endelse

  endif else begin

    new = create_struct(tags[0], struct.(0))  ;initialize structure
    for i=1, itag-1 do begin                  ;insert leading unchange
      new = create_struct(new, tags[i], struct.(i))
    endfor

    ;insert new data
    tmpstruct = struct.(itag)
    if size(tmpstruct, /tname) ne 'STRUCT' then new = create_struct(new, otag, data) $
    else begin
      modify_struc, tmpstruct, tagkeys, ilevel+1, data
      new = create_struct(new, otag, tmpstruct)
    endelse

  endelse
  ;Replicate remainder of structure after desired tag.
  for i=itag+1, ntags-1 do begin
    new = create_struct(new, tags[i], struct.(i))
  endfor

  ;Replace input structure with new structure.
  struct = new

end


;+
; :Description:
;    Removes bad events, i.e. events having a time smaller than the time of the
;    previous event (with a given tolerance if tol is given).
;
;    The call is
;       > remove_bad_events, ...
;
; :Parameters (all required):
;
;  :Example:
;-
pro remove_bad_events, timeVector, totalRemoved, xVector=xVector, yVector=yVector, tolerance=tolerance

  if n_elements(tolerance) eq 0 then tolerance = 0
  
  totalRemoved = 0

  ;Remove events that have a time > last event or time < first event
  ;bad_times = where((timeVector gt timeVector[-1]) or (timeVector lt timeVector[0]), number_bad_events, complement=good_times)
  ;if number_bad_events gt 0 then begin
  ;  timeVector = timeVector[good_times] 
  ;  if n_elements(xVector) gt 0 then xVector = xVector[good_times]
  ;  if n_elements(yVector) gt 0 then yVector = yVector[good_times]
  ;  lamp_journal, strtrim(number_bad_events)+ ' events removed because time is out of range!'
  ;  print,        strtrim(number_bad_events)+ ' events removed because time is out of range!'
  ;  totalRemoved += number_bad_events
  ;endif
  

;  ;NEED TO INVESTIGATE ALGORITHMS TO DETECT OUTLIERS
;  ;OR DECIDE IF BAD EVENTS MUST BE REMOVED MANUALLY !!!!!!!
;
;  xxx = ulindgen(n_elements(timeVector))
;  
;  ;Testing manual choice
;  info_message = ['Check event times in EVENT SELECTION WINDOW ', $
;                  'and select a zone containing events to be discarded.', $
;                  ' ', $
;                  'To select a zone, use the mouse: ', $
;                  '  Left button to set a point, ', $
;                  '  Middle button to erase the last point, ', $
;                  '  Right button to close the region.', $
;                  ' ', $
;                  'At least 3 points need to be set before closing the region.', $
;                  ' ', $
;                  'The plot will be updated after removing the selected points ', $
;                  'and an additional selection can be done.', $
;                  'To finish, select a zone in the plot that it does not contain any point.']
;  instruct = dialog_message(info_message, /information)
;  
;  window, 1, xsize=500, ysize=500, title='EVENT SELECTION WINDOW'
;  finish_selection = 0
;  bad_times = [] & good_times = ulindgen(n_elements(timeVector))  
;  while ~finish_selection do begin
;    plot, xxx[good_times], timeVector[good_times]
;    indx = graphselect(xxx, timeVector, n_selected, /nofill)
;    if n_selected eq 0 then finish_selection = 1 else bad_times = [bad_times, indx]
;    if n_elements(bad_times) gt 0 then good_times = mg_complement(bad_times, n_elements(timeVector))
;  endwhile  
;  wdelete, 1
;  if n_elements(bad_times) gt 0 then begin
;    timeVector = timeVector[good_times]
;    if n_elements(xVector) gt 0 then xVector = xVector[good_times]
;    if n_elements(yVector) gt 0 then yVector = yVector[good_times]
;    totalRemoved += n_elements(bad_times)
;  endif
  
  ;Remove based on a linear fit
  ;result = linfit(xxx, timeVector, chisqr=chisqr, sigma=sigma, yfit=yfit)
  ;sigma = sqrt(chisqr/n_elements(timeVector))
  ;residuals = abs(timeVector - yfit)
  ;bad_times = where(residuals gt 10.0*sigma, complement=good_times, number_bad_events)
  ;plot = plot(xxx, timeVector)
  ;plot = plot(xxx, yfit, /overplot, color='red')
  ;stop
  ;if number_bad_events gt 0 and number_bad_events lt n_elements(residuals)/10 then begin
    ;plot = plot(xxx[bad_times[0]-100:bad_times[0]+100], timeVector[bad_times[0]-100:bad_times[0]+100], symbol='o', line='none')
    ;stop
    ;timeVector = timeVector[good_times]
    ;if n_elements(xVector) gt 0 then xVector = xVector[good_times]
    ;if n_elements(yVector) gt 0 then yVector = yVector[good_times]
    ;totalRemoved += number_bad_events
  ;endif

;  ;Bad events (outliers) searched using a method close to that proposed by
;  ;HJ Motulsky and RE Brown in BMC Bioinformatics 7, 123 (2006)
;  q = 0.01 ;Q choice sujective, but recommended 1% in paper compared to 0.1 or 10%
;  ;Robust fit to event times using a 6th degree polynomial 
;  n_degree = 6
;  xxx = findgen(n_elements(timeVector))
;  ;result = robust_poly_fit(xxx, timeVector, n_degree, yfit, sigma)
;  result = robust_linefit(xxx, timeVector, yfit, sigma, result_sigma)
;  ;plot = plot(xxx, timeVector, symbol='o', line='none')
;  plot = plot(xxx, timeVector)
;  plot = plot(xxx, yfit, /overplot, color='red')
;  stop
;  residuals = abs(timeVector - yfit)
;  index_sort_residuals = sort(residuals) & sorted_residuals = residuals[index_sort_residuals]
;  t = sorted_residuals / sigma
;  alpha = q * (n_elements(residuals)-l64indgen(n_elements(residuals))) / n_elements(residuals)
;  p = 2.0 * (1.0 - t_pdf(t, n_elements(residuals)-n_degree))
;  tmp = p - alpha
;  outliers = where(tmp lt 0, complement=valid_indexes, number_bad_events)
;  ;if more than 10% of bad events, then likely this approach failed
;  if number_bad_events gt 0 and number_bad_events lt n_elements(residuals)/10 then begin
;    tmp = sort(index_sort_residuals[valid_indexes]) & good_times = index_sort_residuals[tmp] 
;    timeVector = timeVector[good_times]
;    if n_elements(xVector) gt 0 then xVector = xVector[good_times]
;    if n_elements(yVector) gt 0 then yVector = yVector[good_times]
;    totalRemoved += number_bad_events
;  endif
;  ;undefine all temporary variables 
;  tempvar = SIZE(TEMPORARY(xxx))
;  tempvar = SIZE(TEMPORARY(yfit))
;  tempvar = SIZE(TEMPORARY(residuals))
;  tempvar = SIZE(TEMPORARY(index_sort_residuals))
;  tempvar = SIZE(TEMPORARY(t))
;  tempvar = SIZE(TEMPORARY(alpha))
;  tempvar = SIZE(TEMPORARY(p))
;  tempvar = SIZE(TEMPORARY(tmp))
;  tempvar = SIZE(TEMPORARY(outliers))
;  tempvar = SIZE(TEMPORARY(good_times))

;  errors = 1 ;CAN BE VERY VERY LONG !!!
;  while errors do begin
;    time_diff = timeVector[1:*] - timeVector[0:n_elements(timeVector)-2]
;    bad_times = where (time_diff lt -tolerance, number_bad_events, complement=good_times)
;    if number_bad_events gt 0 then begin
;      good_times += 1
;      timeVector = [timeVector[0], timeVector[good_times]]
;      if n_elements(xVector) gt 0 then xVector = [xVector[0], xVector[good_times]]
;      if n_elements(yVector) gt 0 then yVector = [yVector[0], yVector[good_times]]
;      totalRemoved += number_bad_events
;    endif else errors = 0
;  endwhile

end

pro remove_forbidden_events, timeVector, totalRemoved, xVector=xVector, yVector=yVector, tolerance=tolerance

  actual_time = timeVector[0] & keepVector = timeVector & nLast = 0
  for i = 1, n_elements(timeVector)-1 do begin
     if (timeVector[i] - actual_time) gt tolerance then begin
       nLast += 1
       actual_time = timeVector[i]
       keepVector[nLast] = timeVector[i]
     endif
  endfor
  totalRemoved = n_elements(timeVector) - nLast - 1
  timeVector = keepVector[0:nLast]
  
end


pro event_file_treat, flag, struc, file=file, Message=Mess, Mess2=Mess2, Mess3=Mess3, MessRem=MessRem, W=Wi
;+
;** ****************
;** Called from the event handler of event_file GUI or web client
;** flag: 'read'  read a new file
;**       'free'  Free memory
;**       'show'  plot what is demanded (ex:histogram sum)
;**       'etc...'
;-

;struc tags:
;**********
;bmess   = label widget message
;trigg   = 1:use triggers else 0
;slice   = 1:use slices   else 0
;Nslice  = number of slices
;Cframe  = current choosen frame
;mxTime  = [min,max] data timer original
;mimaxT  = [Min,Max] choosen data timer
;Strigg  = number of sample triggers
;Ctrigg  = number of chopper triggers
;Ndata   = number of event data
;BinMod  = 1:Linear , 2:Log , Equal events
;ShowMod = 1:Bin Details , 2:Histogram , 3:Event Data , etc
;noWind  = 1:invisible Gui (or web client)
;vers    = code version date
;removeBad   = 0/1: 1 = remove bad events

  common EV_common, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger, out_array, mon_array, time_array, nbrepaints, total_counting_time
  common EV_common_2, ticks, temp1, temp2, temp3, temp4, temp5

  case flag of
    
  'free': begin 
            Counts=0 & Xdet=0 & Ydet=0 & Dtimer=0 & Ctimer=0 & Stimer=0 
            Mtimer=0 & Dtimer_trigger=0 & Mtimer_trigger=0
          end
  
  'read': begin 
          Counts=0 & Xdet=0 & Ydet=0 & Dtimer=0 
          Ctimer=0 & Stimer=0 & Mtimer=0 
          Dtimer_trigger=0 & Mtimer_trigger=0 
  
          time_ini = systime(1)
           
          ;read event mode file
          counts = rd_tisane(file, X_DETECTOR=Xdet,  Y_DETECTOR=Ydet, T_DETECTOR=Dtimer, $
                                     T_CHOPPER=Ctimer, T_SAMPLE=Stimer, T_MONITOR=Mtimer, W=Wi)
          time_end = systime(1)
            
          if n_elements(counts) eq 1 then Mess = 'Bad input file!' else begin 
             
            if n_elements(Xdet) gt 1 then tmp1 = n_elements(Xdet) else $
              if Xdet[0] eq -1 then tmp1 = 0 else tmp1 = 1
            if n_elements(Mtimer) gt 1 then tmp2 = n_elements(Mtimer) else $
              if Mtimer[0] eq -1 then tmp2 = 0 else tmp2 = 1
            if n_elements(Stimer) gt 1 then tmp3 = n_elements(Stimer) else $
              if Stimer[0] eq -1 then tmp3 = 0 else tmp3 = 1
            if n_elements(Ctimer) gt 1 then tmp4 = n_elements(Ctimer) else $
              if Ctimer[0] eq -1 then tmp4 = 0 else tmp4 = 1
                
            Mess = 'Detector events = '               + strtrim(tmp1,2)  + $
                   ', Monitor events = '              + strtrim(tmp2,2)
                    
            Mess2 = 'Trigger 1 (sample) events = '   + strtrim(tmp3,2) + $
                    ', Trigger 2 (chopper) events = '  + strtrim(tmp4,2)
                     
            Mess3 = 'XY counts = '      + strtrim(long(total(Counts)),1)
                    
            mini = min([Dtimer],max=maxi) & struc.mxTime = [mini,maxi] / struc.clock
            if Stimer[0] ne -1 then struc.Strigg  = n_elements(Stimer) else struc.Strigg  = 0
            if Ctimer[0] ne -1 then struc.Ctrigg  = n_elements(Ctimer) else struc.Ctrigg  = 0
            if Dtimer[0] ne -1 then struc.Ndata   = n_elements(Dtimer) else struc.Ndata   = 0
            if Mtimer[0] ne -1 then struc.Monitor = n_elements(Mtimer) else struc.Monitor = 0
               
            tempvar = size(temporary(Dtimer_trigger)) ;Undefine Dtimer_trigger to force recalculation later on
            tempvar = size(temporary(mtimer_trigger)) 
            tempvar = size(temporary(out_array))
            tempvar = size(temporary(mon_array))

            ;trigger (get minimum time between valid triggers)
            widget_control, struc.bdelay, get_value = tmp
            if strtrim(tmp[0],2) gt '' then tmp = float(tmp[0])*1.e-3 else tmp = 0 ;forbidden time in seconds
            forbidden_trigger = long64(tmp * struc.clock) 

            ;Remove bad events in detector 
            total_removed = 0
            remove_bad = struc.removeBad  ;remove "bad" events (1) or not (0)
            tolerance  = 0  ;tolerance in time units: = 0 --> all events that have a time smaller than the previous event are discarded
            ;                         = N --> all events that have a time < t_previous - N are discarded

            if (remove_bad) then begin
                
               stime = systime(1)
               MessRem = 'Events removed: '
  
               ;Remove events "out" of detector (Max values of detector not to be hardcoded!!!)
               bad_pos = where((Xdet lt 0) or (Xdet ge struc.Xsize) or (Ydet lt 0) or (Ydet ge struc.Ysize), number_bad_events, complement=good_pos)
               if number_bad_events gt 0 then begin
                  Dtimer = Dtimer[good_pos] & Xdet = Xdet[good_pos] & Ydet = Ydet[good_pos]
                  lamp_journal, strtrim(number_bad_events)+ ' events removed because XY position is wrong!'
                  print,        strtrim(number_bad_events)+ ' events removed because XY position is wrong!'
                  total_removed += number_bad_events
               endif
               
               ;Remove events with bad times (either events with time > last_event or < first_event or
               ;events with a time smaller than the time of a previous event)
               times_removed = 0
               remove_bad_events, Dtimer, times_removed, xVector=Xdet, yVector=Ydet, tolerance=tolerance
               total_removed += times_removed

               ;Get new min and max times
               if total_removed gt 0 then begin
                  struc.mxTime = [min([Dtimer],max=maxi), maxi] / struc.clock
                  struc.Ndata  = n_elements(Dtimer)
               endif
               
               print,        strtrim(times_removed)+ ' events removed because of bad times!'
               lamp_journal, strtrim(times_removed)+ ' events removed because of bad times!'
               ;print, ''
               ;print, round((systime(1)-stime)*10.)/10., ' sec. for removing bad detector events.'
               ;print, ''
               MessRem = MessRem + strtrim(total_removed,2) + ' det'
  
               ;Check sample trigger events
               total_removed = 0
               forbidden_removed = 0
               if n_elements(Stimer) gt 1 then begin
                 stime = systime(1)
                 remove_bad_events, Stimer, total_removed, tolerance=tolerance
                 if forbidden_trigger gt 0 then remove_forbidden_events, Stimer, forbidden_removed, tolerance=forbidden_trigger
               endif
               struc.Strigg = n_elements(Stimer)
               lamp_journal, strtrim(total_removed)+ ' sample_trigger events removed because of suspicious times!'
               print,        strtrim(total_removed)+ ' sample_trigger events removed because of suspicious times!'
               lamp_journal, strtrim(forbidden_removed)+ ' sample_trigger events removed because inside delay time!'
               print,        strtrim(forbidden_removed)+ ' sample_trigger events removed because inside delay time!'
               ;print, round((systime(1)-stime)*10.)/10., ' sec. for removing bad sample_trigger events.'
               MessRem = MessRem + ', ' + strtrim(total_removed+forbidden_removed,2) + ' sample'

               ;Check chopper trigger events
               total_removed = 0
               forbidden_removed = 0
               if n_elements(Ctimer) gt 1 then begin
                 stime = systime(1)
                 remove_bad_events, Ctimer, total_removed, tolerance=tolerance
                 if forbidden_trigger gt 0 then remove_forbidden_events, Ctimer, forbidden_removed, tolerance=forbidden_trigger
               endif
               struc.Ctrigg = n_elements(Ctimer)
               lamp_journal, strtrim(total_removed)+ ' chopper_trigger events removed because of suspicious times!'
               print,        strtrim(total_removed)+ ' chopper_trigger events removed because of suspicious times!'
               lamp_journal, strtrim(forbidden_removed)+ ' chopper_trigger events removed because inside delay time!'
               print,        strtrim(forbidden_removed)+ ' chopper_trigger events removed because inside delay time!'
               print, round((systime(1)-stime)*10.)/10., ' sec. for removing bad chopper_trigger events.'
               MessRem = MessRem + ', ' + strtrim(total_removed+forbidden_removed,2) + ' chopper'

               ;Check monitor events
               total_removed = 0
               if n_elements(Mtimer) gt 1 then begin
                 stime = systime(1)
                 remove_bad_events, Mtimer, total_removed, tolerance=tolerance
               endif
               struc.Monitor = n_elements(Mtimer)
               lamp_journal, strtrim(total_removed)+ ' monitor events removed because of suspicious times!'
               print,        strtrim(total_removed)+ ' monitor events removed because of suspicious times!'
               print, round((systime(1)-stime)*10.)/10., ' sec. for removing bad monitor events.'
               MessRem = MessRem + ', ' + strtrim(total_removed,2) + ' monitor'

            endif
          endelse
                                                         
          end ;end read

  'clear_trigger': begin
    tempvar = size(temporary(Dtimer_trigger)) ;Clear array dtimer_trigger whenever trigger option is changed
    tempvar = size(temporary(Mtimer_trigger)) 
  end
  'slices':  ;Choosen slices   mode event
  'nslice':  ;Choosen number of slices event
  'slides':  ;Choosen current frame event
  'mintime': ;Choosen min data Timer event
  'maxtime': ;Choosen max data Timer event
  'setmint': ;Choosen original min data Timer event
  'setmaxt': ;Choosen original max data Timer event
  
  'bin_trigger': if n_elements(Dtimer) gt 1 then begin
               ; Calculate times with respect to chopper/sample trigger (to do it only once)
               if struc.UseCtrigg and n_elements(Dtimer_trigger) eq 0 then begin
                   ; Remove detector times < first trigger
                   bad_times = where (Dtimer lt Ctimer[0], number_bad_events, complement=good_times)
                   if number_bad_events gt 0 then begin
                      Dtimer = Dtimer[good_times] & Xdet = Xdet[good_times] & Ydet = Ydet[good_times] 
                      lamp_journal, strtrim(number_bad_events)+ ' events removed because time < time_first_chopper_trigger!'
                      print,        strtrim(number_bad_events)+ ' events removed because time < time_first_chopper_trigger!'
                   endif
                   Dtimer_trigger = Dtimer 
                   if n_elements(Ctimer) gt 1 then loc = value_locate(Ctimer, Dtimer) else loc = 0
                   Dtimer_trigger = Dtimer - Ctimer[loc]
                   print, 'Minimum and maximum times between chopper trigger = ', min(Dtimer_trigger), max(Dtimer_trigger)
                   ;monitor
                   if n_elements(Mtimer) gt 1 then begin
                     bad_times = where (Mtimer lt Ctimer[0], number_bad_events, complement=good_times)
                     if number_bad_events gt 0 then Mtimer = Mtimer[good_times] 
                     Mtimer_trigger = Mtimer
                     if n_elements(Ctimer) gt 1 then loc = value_locate(Ctimer, Mtimer) else loc = 0
                     Mtimer_trigger = Mtimer - Ctimer[loc]
                   endif
               endif  
                 
               if struc.UseStrigg and n_elements(Dtimer_trigger) eq 0 then begin
                 ; Remove detector times < first trigger
                 bad_times = where (Dtimer lt Stimer[0], number_bad_events, complement=good_times)
                 if number_bad_events gt 0 then begin
                   Dtimer = Dtimer[good_times] & Xdet = Xdet[good_times] & Ydet = Ydet[good_times]
                   lamp_journal, strtrim(number_bad_events)+ ' events removed because time < time_first_sample_trigger!'
                   print,        strtrim(number_bad_events)+ ' events removed because time < time_first_sample_trigger!'
                 endif
                 Dtimer_trigger = Dtimer
                 if n_elements(Stimer) gt 1 then loc = value_locate(Stimer, Dtimer) else loc = 0
                 Dtimer_trigger = Dtimer - Stimer[loc]
                 print, 'Minimum and maximum times between sample trigger = ', min(Dtimer_trigger), max(Dtimer_trigger)
                 ;monitor
                 if n_elements(Mtimer) gt 1 then begin
                   bad_times = where (Mtimer lt Stimer[0], number_bad_events, complement=good_times)
                   if number_bad_events gt 0 then Mtimer = Mtimer[good_times]
                   Mtimer_trigger = Mtimer
                   if n_elements(Stimer) gt 1 then loc = value_locate(Stimer, Mtimer) else loc = 0
                   Mtimer_trigger = Mtimer - Stimer[loc]
                 endif
               endif

             endif

     'bin': begin
            s = size(counts)
            out_array = lonarr(s[1], s[2], struc.nslice)
            mon_array = lonarr(struc.nslice)
            time_array = fltarr(struc.nslice)            
            ;Selected range
            tmin = long64(struc.mimaxt[0]*struc.clock)
            tmax = long64(struc.mimaxt[1]*struc.clock)
            select_trange = where(Dtimer ge tmin and Dtimer le tmax, number_events)
            if struc.Monitor gt 0 then select_monitor = where(Mtimer ge tmin and Mtimer le tmax, number_monitors)
            
            ;Select times from trigger or not
            if struc.USECTRIGG or struc.USESTRIGG then begin
              if n_elements(Dtimer_trigger) eq 0 then begin
                event_file_treat, 'bin_trigger', struc
                ;need to redo select because some events may have been chopped out by bin_trigger
                select_trange = where(Dtimer ge tmin and Dtimer le tmax, number_events) 
                select_monitor = where(Mtimer ge tmin and Mtimer le tmax, number_monitors)
              endif  
              times = Dtimer_trigger[select_trange]
              if struc.Monitor gt 0 then times_monitor = Mtimer_trigger[select_monitor]
              tmin = 0
              widget_control, struc.bcycle, get_value = tmp
              tmax = (long64(double(tmp) * 1.e-3 * struc.clock))[0]
            endif else begin
              times = Dtimer[select_trange]
              if struc.Monitor gt 0 then times_monitor = Mtimer[select_monitor]
            endelse
            
            ;Avoid having negative times (this should not happen) or zero --> otherwise the log binning will fail
;            if tmin eq 0 then timeShift = 1 else $
;              if tmin lt 0 then timeShift = -tmin + 1 else timeShift = 0
;            times +=  timeShift
;            tmin  += timeShift
;            tmax  += timeShift
;            if struc.Monitor gt 0 then times_monitor += timeShift
;            if struc.Monitor gt 0 then times_monitor += timeShift
            
            ;Compute times for different types of binning
            if struc.BINMOD eq 1 then begin ;linear binning
              delta_t = (tmax - tmin) / struc.NSLICE
              ticks = long64(tmin + indgen(struc.NSLICE)*delta_t)
              ticks_end = ticks + long64(struc.tslice * 1e-3 * struc.clock)
            endif else if struc.BINMOD eq 2 then begin ;log binning
              delta_t = (alog10(tmax) - alog10(tmin>1)) / struc.NSLICE
              ticks = alog10(tmin>1) + indgen(struc.NSLICE)*delta_t
              ticks = long64(10^ticks)
              ticks_end = lon64arr(n_elements(ticks))
              ticks_end[0:-2] = ticks[1:-1]
              ticks_end[-1] = tmax
            endif else if struc.BINMOD eq 3 then begin ;equal count binning
              if struc.USECTRIGG or struc.USESTRIGG then begin
                print, 'Warning: This type of binning does not use the trigger!'
                tmax = long64(struc.mimaxt[1]*struc.clock)
                times = Dtimer[select_trange]
                if struc.Monitor gt 0 then times_monitor = Mtimer[select_monitor]
              endif            
              counts_per_slice = long64(number_events / struc.NSLICE)
              ticks = Dtimer[lindgen(struc.NSLICE)*counts_per_slice]
              ticks_end = lon64arr(n_elements(ticks))
              ticks_end[0:-2] = ticks[1:-1]
              ticks_end[-1] = tmax
            endif
            
            ;Assign events
            print, number_events, ' events to bin.'
;            if struc.BINMOD eq 3 and (struc.USECTRIGG or struc.USESTRIGG) then loc = value_locate(ticks, Dtimer[select]) $
;            else loc = value_locate(ticks, times)
            stime = systime(1)
            
            for islice = 0, struc.NSLICE -1 do begin
            
              select_bin = where (times ge ticks[islice] and times lt ticks_end[islice])
              if select_bin[0] ne -1 then begin
                ;Assignment using VALUE_LOCATE
                XYdet = Xdet[select_trange[select_bin]]+struc.Xsize*Ydet[select_trange[select_bin]]
                XYdet = XYdet[sort(XYdet)]
                XYunique = XYdet[UNIQ(XYdet)]
                if n_elements(XYdet) gt 1 then begin
                  tmp = value_locate(XYdet, XYunique)
                  counter = lonarr(n_elements(XYunique))
                  counter[1:*] = tmp[1:*] - tmp[0:-2]
                  counter[0] = n_elements(where (XYdet eq XYunique[0]))
                  out_array[XYunique+islice*struc.Xsize*struc.Ysize] += counter
                endif else out_array[XYunique+islice*struc.Xsize*struc.Ysize] += 1  
              endif
              
              ;Monitor
              if struc.Monitor gt 0 then begin
                select_bin = where (times_monitor ge ticks[islice] and times_monitor lt ticks_end[islice])
                if select_bin[0] ne -1 then mon_array[islice] = n_elements(select_bin) 
              endif else mon_array[islice] = 1
              
              ;Counting times per slice
              if ticks_end[islice] lt tmax then time_array[islice] = (ticks_end[islice] - ticks[islice]) / struc.clock $
                                           else time_array[islice] = (tmax - ticks[islice]) / struc.clock
            
            endfor

            ;Determine number of repaints and total counting time
            tmin = long64(struc.mimaxt[0]*struc.clock)
            tmax = long64(struc.mimaxt[1]*struc.clock)
            total_counting_time = (tmax - tmin) / struc.clock
            if struc.USECTRIGG then begin
              select_repaints = where(Ctimer ge tmin and Ctimer le tmax, number_events)
              nbrepaints = number_events
            endif else if struc.USESTRIGG then begin
              select_repaints = where(Stimer ge tmin and Stimer le tmax, number_events)
              nbrepaints = number_events
            endif else nbrepaints = 1
           
            tempvar = size(temporary(tmp))
            print, systime(1)-stime, ' sec. for assignment'
            ;print, 'Times = ', ticks
    
            end
   
     'show': begin
      
                case struc.ShowMod of
                
                  1: begin                         ; Slice binning --> pass to bin

                    if n_elements(out_array) eq 0 then $
                      event_file_treat, 'bin', struc

                    ;Plot out_array
                    take_datp, dp, W=Wi
                    mod_datp, dp, 'Z', ticks/struc.clock ;Pass slice times in seconds
                    dp.X_tit = 'X detector'
                    dp.Y_tit = 'Y detector'
                    dp.Z_tit = 'Time (seconds)'
                    dp.w_tit = struc.curFile
                    dp.Other_tit = 'Binned slices'
                    give_datp, dp, W=Wi
                    if struc.cframe eq 0 then plot_array = out_array else plot_array = out_array[*,*,struc.cframe-1]
                    give_w, plot_array, W=Wi
                    see, W=Wi

                  end

                 2: begin & give_w, Counts, W=Wi ; Histogram sum
                            take_datp,  dp, W=Wi
                            dp.w_tit = struc.curFile
                            dp.X_tit = 'X detector'
                            dp.Y_tit = 'Y detector'
                            dp.Other_tit = 'Histogram'
                            give_datp,  dp, W=Wi
                            see, W=Wi
                    end

                 3: begin                        ; Event Data
                        ;Selected range
                        tmin = long64(struc.mimaxt[0]*struc.clock)
                        tmax = long64(struc.mimaxt[1]*struc.clock)
                        select_trange = where (Dtimer ge tmin and Dtimer le tmax, number_events)
                        print, number_events, ' event times to be plotted!' 
                        messRem = strtrim(number_events,2) + ' event times to be plotted!'
                        widget_control, struc.bmessRem, set_value=MessRem
                        if struc.UseCtrigg or struc.UseStrigg then begin
                          if n_elements(Dtimer_trigger) eq 0 then begin
                            event_file_treat, 'bin_trigger', struc
                            ;need to redo select because some events may have been chopped out by bin_trigger
                            select_trange = where(Dtimer ge tmin and Dtimer le tmax, number_events)
                          endif
                          give_w, Dtimer_trigger[select_trange]/struc.clock, W=Wi
                        endif else give_w, Dtimer[select_trange]/struc.clock, W=Wi
                        take_datp,  dp, W=Wi
                        dp.w_tit = struc.curFile
                        dp.X_tit = 'Event number'
                        dp.Y_tit = 'Time (seconds)'
                        dp.Other_tit = 'Event times'
                        give_datp,  dp, W=Wi
                        see, W=Wi
                    end

                 4: begin                        ; Trigger Data
                      ;Selected range
                      tmin = long64(struc.mimaxt[0]*struc.clock)
                      tmax = long64(struc.mimaxt[1]*struc.clock)
                      if struc.USECTRIGG then begin
                        if n_elements(Ctimer) ge 2 then begin
                          select_trange = where (Ctimer ge tmin and Ctimer le tmax, number_events)
                          print, number_events, ' event times to be plotted!'
                          if n_elements(Ctimer[select_trange]) ge 2 then begin
                            time_diff = Ctimer[select_trange[1:*]] - Ctimer[select_trange[0:n_elements(Ctimer[select_trange])-2]]
                            time_diff /= struc.clock
                            give_w, time_diff, W=Wi
                            take_datp,  dp, W=Wi
                            dp.w_tit = struc.curFile
                            dp.X_tit = 'Time lapse between consecutive chopper triggers'
                            dp.Y_tit = 'Time (seconds)'
                            dp.Other_tit = 'Chopper trigger period'
                            give_datp,  dp, W=Wi
                            see, W=Wi
                          endif else print, 'Number of chopper triggers in selected range < 2'
                        endif else print, 'Number of chopper triggers < 2'
                      endif else if struc.USESTRIGG then begin
                        if n_elements(Stimer) ge 2 then begin
                          select_trange = where (Stimer ge tmin and Stimer le tmax, number_events)
                          print, number_events, ' event times to be plotted!'
                          if n_elements(Stimer[select_trange]) ge 2 then begin
                            time_diff = Stimer[select_trange[1:*]] - Stimer[select_trange[0:n_elements(Stimer[select_trange])-2]]
                            time_diff /= struc.clock
                            give_w, time_diff, W=Wi
                            take_datp,  dp, W=Wi
                            dp.w_tit = struc.curFile
                            dp.X_tit = 'Time lapse between consecutive sample triggers'
                            dp.Y_tit = 'Time (clock units)'
                            dp.Other_tit = 'Sample trigger period'
                            give_datp,  dp, W=Wi
                            see, W=Wi
                          endif else print, 'Number of sample triggers in selected range < 2'
                        endif else print, 'Number of sample triggers < 2'
                      endif else begin
                        print, 'No trigger (chopper/sample) selected.'
                      endelse  
                    end

                 5: begin                        ; Distribution of Event Data
                      ;Selected range
                      tmin = long64(struc.mimaxt[0]*struc.clock)
                      tmax = long64(struc.mimaxt[1]*struc.clock)
                      select_trange = where (Dtimer ge tmin and Dtimer le tmax, number_events)
                      if struc.UseCtrigg or struc.UseStrigg then begin
                        if n_elements(Dtimer_trigger) eq 0 then begin
                          event_file_treat, 'bin_trigger', struc
                          ;need to redo select because some events may have been chopped out by bin_trigger
                          select_trange = where(Dtimer ge tmin and Dtimer le tmax, number_events)
                        endif
                      endif  
                      nbins = 100
                      if struc.UseCtrigg or struc.UseStrigg then begin
                        bin_step = (max(Dtimer_trigger)-min(Dtimer_trigger))/nbins 
                        tzero = min(Dtimer_trigger)
                      endif else begin
                        bin_step = (max(Dtimer)-min(Dtimer))/nbins
                        tzero = min(Dtimer)
                      endelse  
                      histo_events = fltarr(nbins)
                      histo_times = fltarr(nbins)
                      for i = 0, nbins-1 do begin
                          tini = tzero + i*bin_step
                          tend = tzero + (i+1)*bin_step
                          if struc.UseCtrigg or struc.UseStrigg then $
                             tmp = where(Dtimer_trigger gt tini and Dtimer_trigger le tend, count) else $
                             tmp = where(Dtimer gt tini and Dtimer le tend, count)
                          histo_events[i] = count
                          histo_times[i] = tini / struc.clock
                      endfor
                      give_w, histo_events, W=Wi
                      take_datp,  dp, W=Wi
                      mod_datp, dp, 'X', histo_times
                      dp.w_tit = struc.curFile
                      dp.X_tit = 'Time (s)'
                      dp.Y_tit = 'Number of detector events'
                      dp.Other_tit = 'Time distribution of detector events'
                      give_datp,  dp, W=Wi
                      see, W=Wi
                    end

                 6: begin                        ; Distribution of triggers
                      ;Selected range
                      tmin = long64(struc.mimaxt[0]*struc.clock)
                      tmax = long64(struc.mimaxt[1]*struc.clock)
                      if struc.UseCtrigg or struc.UseStrigg then begin
                        if (struc.UseCtrigg) then begin
                          select_trange = where (Ctimer ge tmin and Ctimer le tmax, number_events)
                          time_diff = Ctimer[select_trange[1:*]] - Ctimer[select_trange[0:n_elements(Ctimer[select_trange])-2]]
                        endif else begin
                          select_trange = where (Stimer ge tmin and Stimer le tmax, number_events)
                          time_diff = Stimer[select_trange[1:*]] - Stimer[select_trange[0:n_elements(Stimer[select_trange])-2]]
                        endelse
                        if n_elements(time_diff) ge 2 then begin
                          nbins = 100
                          bin_step = (max(time_diff)-min(time_diff))/nbins
                          tzero = min(time_diff)
                          histo_events = fltarr(nbins)
                          histo_times = fltarr(nbins)
                          for i = 0, nbins-1 do begin
                            tini = tzero + i*bin_step
                            tend = tzero + (i+1)*bin_step
                            tmp = where(time_diff gt tini and time_diff le tend, count) 
                            histo_events[i] = count
                            histo_times[i] = tini / struc.clock
                          endfor
                          give_w, histo_events, W=Wi
                          take_datp,  dp, W=Wi
                          mod_datp, dp, 'X', histo_times
                          ;dp.x = histo_times
                          dp.w_tit = struc.curFile
                          dp.X_tit = 'Time (s)'
                          dp.Y_tit = 'Number of trigger events'
                          dp.Other_tit = 'Time distribution of trigger periods'
                          give_datp,  dp, W=Wi
                          see, W=Wi
                        endif else print, 'No enough trigger events.'
                      endif else begin
                        print, 'No trigger (chopper/sample) selected.'
                      endelse
                    end
                    
                else:
               endcase
             end

  'expcur': begin ;export binned data as Nexus
               
               if n_elements(out_array) eq 0 then event_file_treat, 'bin', struc    

               filename = struc.curFile
               fileNX = struc.cpath + strmid(filename,0,strpos(filename,'.lst')) + '.nxs'
               okNX = file_test(fileNX)

               if okNX then begin

                 nexus = h5_parse(fileNX, /read_data)
                 
                 ;counting time (different from value read if Min/Max Time Ranges used)
                 nexus.entry0.time._data = total_counting_time

                 ;Kinetic acquisition mode = 3
                 nexus.entry0.acquisition_mode._data = 3
                 nexus.entry0.mode._data = 3
                 
                 s = size(out_array)
                 if s[0] gt 2 then slices = s[3] else slices = 1
                 
                 ;NBREPAINT structure
                 struc_nbrepaint = {_name:'nbrepaint', _icontype:'binary', _type:'dataset', $
                  _file:'', _path:'/entry0', _data:[1.], _ndimensions:1L, _dimensions:[1ULL], _nelements:1ULL, $
                  _datatype:'H5T_FLOAT', _storagesize:4UL, _precision:32L, _sign:''}
                  struc_nbrepaint._file = nexus._file
                  struc_nbrepaint._data = nbrepaints

                  ;SLICES structure
                  struc_slices = {_name:'slices', _icontype:'binary', _type:'dataset', $
                    _file:'', _path:'/entry0', _data:[1.], _ndimensions:1L, _dimensions:[1ULL], _nelements:1ULL, $
                    _datatype:'H5T_FLOAT', _storagesize:4UL, _precision:32L, _sign:''}
                  struc_slices._file = nexus._file

                  ;TOTAL_SLICES_TIME structure
                  struc_total_slices_time = {_name:'total_slices_time', _icontype:'binary', _type:'dataset', $
                    _file:'', _path:'/entry0', _data:[1.], _ndimensions:1L, _dimensions:[1ULL], _nelements:1ULL, $
                    _datatype:'H5T_FLOAT', _storagesize:4UL, _precision:32L, _sign:''}
                  struc_total_slices_time._file = nexus._file
                 
newEntry0 = create_struct(nexus.entry0, ['nbrepaint', 'slices', 'total_slices_time'], $
  struc_nbrepaint, struc_slices, struc_total_slices_time)

newHeader = {_name:'', _icontype:'', _type:'', _file:'', _path:'', _comment:''}
newHeader._name = nexus._name
newHeader._icontype = nexus._icontype
newHeader._type = nexus._type
newHeader._file = nexus._file
newHeader._path = nexus._path
newHeader._comment = nexus._comment

finalNX = create_struct(newHeader, ['entry0', 'file_name', 'hdf5_version', 'file_time', 'nexus_version'], $
  newEntry0, nexus.file_name, nexus.hdf5_version, nexus.file_time, nexus.nexus_version)


                 ;measuring time for each slice
                 finalNX.entry0.slices._dimensions = slices
                 finalNX.entry0.slices._nelements = slices
                 modify_struc, finalNX, ['entry0', 'slices', '_data'], 0, time_array
                 finalNX.entry0.total_slices_time._dimensions = slices
                 finalNX.entry0.total_slices_time._nelements = slices
                 modify_struc, finalNX, ['entry0', 'total_slices_time', '_data'], 0, time_array * nbrepaints
                 
                 ;monitor counts for each slice
                 finalNX.entry0.monitor1.inhibit_time = nexus.entry0.monitor1.inhibit_time
                 finalNX.entry0.monitor1.integral = nexus.entry0.monitor1.integral
                 finalNX.entry0.monitor1.mode = nexus.entry0.monitor1.mode
                 finalNX.entry0.monitor1.monrate = nexus.entry0.monitor1.monrate
                 finalNX.entry0.monitor1.monsum = nexus.entry0.monitor1.monsum
                 ;finalNX.entry0.monitor1.nbpickup = nexus.entry0.monitor1.nbpickup
                 finalNX.entry0.monitor1.preset = nexus.entry0.monitor1.preset
                 ;finalNX.entry0.monitor1.time_of_flight = nexus.entry0.monitor1.time_of_flight
                 finalNX.entry0.monitor1.nx_class = nexus.entry0.monitor1.nx_class
                 finalNX.entry0.monitor1.data._dimensions[0] = slices
                 finalNX.entry0.monitor1.data._nelements = slices
                 modify_struc, finalNX, ['entry0', 'monitor1', 'data', '_data'], 0, mon_array

                 finalNX.entry0.monitor2.inhibit_time = nexus.entry0.monitor2.inhibit_time
                 finalNX.entry0.monitor2.integral = nexus.entry0.monitor2.integral
                 finalNX.entry0.monitor2.mode = nexus.entry0.monitor2.mode
                 finalNX.entry0.monitor2.monrate = nexus.entry0.monitor2.monrate
                 finalNX.entry0.monitor2.monsum = nexus.entry0.monitor2.monsum
                 finalNX.entry0.monitor2.preset = nexus.entry0.monitor2.preset
                 finalNX.entry0.monitor2.nx_class = nexus.entry0.monitor2.nx_class
                 finalNX.entry0.monitor2.data._dimensions[0] = slices
                 finalNX.entry0.monitor2.data._nelements = slices
                 modify_struc, finalNX, ['entry0', 'monitor2', 'data', '_data'], 0, mon_array*0+1

                 
                 ;change raw data
                 finalNX.entry0.d22.detector.data_raw._dimensions = [slices, s[2], s[1]]
                 finalNX.entry0.d22.detector.data_raw._nelements = n_elements(out_array)
                 if slices gt 1 then write_array = transpose(out_array, [2,1,0]) else write_array = transpose(out_array)
                 modify_struc, finalNX, ['entry0', 'd22', 'detector', 'data_raw', '_data'], 0, write_array
                 

;apply shift
;using code from Grasp
;get calibration file for vertical correction of tubes (need to think how to do this)

if s[1] eq 128 and s[2] eq 256 then begin 
  t0 = systime(1)
  calfile = 'red_d22_tube_peak_fit_det.dat'
  openr, u, calfile , /get_lun
  line = ''
  readf, u, line ;read first line containing aperture positions in mm
  aperture_positions = float(strsplit(line, /extract))
  fit_positions = fltarr(n_elements(aperture_positions), s[1])
  readf, u, fit_positions 
  ;fit_positions -= 1 ;subtract 1 because fit positions correspond to pixels 1-256 and we work 0-255
  free_lun, u
  pixels = findgen(s[2])
  pixel_size = 4.0 ;mm
  offset = 0.0     ;mm
  new_data = write_array * 0.0
  for islice = 0, slices-1 do begin
    for itube = 0, s[1]-1 do begin
      ;if itube eq 10 or itube eq 14 or itube eq 97 then stop
      ;interpolate to get the physical position corresponding to each pixel
      bottom = interpol(aperture_positions, fit_positions[*,itube], pixels+0.5, /spline)
      top    = interpol(aperture_positions, fit_positions[*,itube], pixels+1.5, /spline)
      ;reassign intensities
      for ipixel = 0, s[2]-1 do begin
        ;if itube eq 10 and ipixel eq 255 then stop
        nbottom = ipixel*pixel_size + offset
        ntop    = (ipixel+1)*pixel_size + offset
        ;case1: old pixels that fully fall within the new pixel
        temp = where(bottom ge nbottom and top le ntop, count)
        if count gt 0 then if slices gt 1 then new_data[islice, ipixel, itube] += total(write_array[islice, temp, itube]) $
                                          else new_data[ipixel, itube]         += total(write_array[temp, itube]) 
        ;case2: old pixels that span the new pixel top boundary
        temp = where(top ge ntop and bottom lt ntop and bottom ge nbottom, count)
        if count gt 0 then begin
          fraction = (ntop-bottom[temp])/(top[temp]-bottom[temp])
          if slices gt 1 then new_data[islice, ipixel, itube] += write_array[islice, temp, itube] * fraction $
                         else new_data[ipixel, itube]         += write_array[temp, itube] * fraction 
        endif  
        ;case3: old pixels that span the new pixel bottom boundary
        temp = where(bottom lt nbottom and top ge nbottom and top lt ntop, count)
        if count gt 0 then begin
          fraction = (top[temp]-nbottom)/(top[temp]-bottom[temp])
          if slices gt 1 then new_data[islice, ipixel, itube] += write_array[islice, temp, itube] * fraction $
                         else new_data[ipixel, itube]         += write_array[temp, itube] * fraction
        endif
        ;case4: old pixel that completely spans new pixel
        temp = where(top ge ntop and bottom lt nbottom, count)
        if count gt 0 then begin
          fraction = (ntop-nbottom)/(top[temp]-bottom[temp])
          if slices gt 1 then new_data[islice, ipixel, itube] += write_array[islice, temp, itube] * fraction $
                         else new_data[ipixel, itube]         += write_array[temp, itube] * fraction
        endif
      endfor
      ;if itube eq 10 or itube eq 14 or itube eq 97 then stop
    endfor
  endfor
  print, 'time to correct rawdata = ', systime(1) - t0
  new_data = round(new_data)
endif else print, 'correction rawdata --> data only possible for detector with 128 tubes x 256 pixels/tube'

;for checking
;nxraw = nexus.entry0.d22.detector.data_raw._data ;raw data in nexus file written by nomad
;nxcor = nexus.entry0.d22.detector.data._data     ;corrected data in nexus file written by nomad
;export = list mode data binned here
;new_data = list mode data binned here and corrected above

                ;change corrected data
                finalNX.entry0.d22.detector.data._dimensions = [slices, s[2], s[1]]
                finalNX.entry0.d22.detector.data._nelements = n_elements(new_data)
                modify_struc, finalNX, ['entry0', 'd22', 'detector', 'data', '_data'], 0, new_data


;                 finalNX.entry0.data._file = nexus.entry0._file      ;links
;                 finalNX.entry0.data_raw._file = nexus.entry0._file  ;links

                 ;new filename
                 outfile = strmid(filename,0,strpos(filename,'.lst')) + '_lst.nxs'

                 h5_create, outfile, finalNX

               endif ELSE print, 'No input Nexus file found! Cannot export'

            end
            
 'explst': begin ;bin and export a list of files as Nexus
             fifi=dialog_pickfile(filter='*.lst', path=struc.Cpath, get_path=pthO, file=tmp, /read, /multiple_files, resource='lamptouch')
             if n_elements(fifi) ge 1 then begin
               tmp = struc.curfile
               for i = 0, n_elements(fifi)-1 do begin
                  struc.curfile = file_basename(fifi[i])
                  event_file_treat, 'read', struc, file=fifi[i], $
                                            Message=Mess, Mess2=Mess2, Mess3=Mess3, W=2
                  event_file_treat, 'bin', struc
                  event_file_treat, 'expcur', struc
               endfor
               struc.curfile = tmp
             endif
           end

  else:
  endcase
end

pro event_file_dying, ev
;** ****************
;** Used to free memory
    event_file_treat, 'free', noStruct
end

pro event_file_web, flag, file
;+
;** **************
;** event handler for event_file Web Client
;-
@lamp.cbk
common c_event_file, EV_struct

if n_tags(EV_struct) eq 0 then event_file, /nowind

case flag of
  'file': begin & EV_struct.Dpath = path_for_online ;User may have changed data path
                  EV_struct.Cpath = EV_struct.Dpath
                  new_file = file
                  event_file_treat, 'read', EV_struct, file=EV_struct.Cpath+new_file, $
                                            Message=Mess, Mess2=Mess2, Mess3=Mess3, W=2
                  EV_struct.curFile = new_file
                  print,"Min,Max Timer:", EV_struct.mxTime

                  if EV_struct.Ndata gt 1 then begin
                    event_file_treat, 'show', EV_struct, W=2
                  endif
          end

  'quit': begin & event_file_treat, 'free', Ev_struct ;free variables
                  EV_struct = 0
          end
   else:
endcase

end

pro event_file_event, ev
;+
;** ****************
;** event handler for event_file GUI
;-

CATCH,stat & if stat ne 0 then begin & lamp_error & return & endif

@lamp.cbk
common c_event_file, EV_struct
common EV_common, Counts, Xdet, Ydet, Dtimer, Ctimer, Stimer, Mtimer, Dtimer_trigger, Mtimer_trigger, out_array, mon_array, time_array, nbrepaints, total_counting_time
common EV_common_2, ticks, temp1, temp2, temp3, temp4, temp5

  widget_control,ev.id,  get_uvalue=uv
  ;Mess = '' & Mess2='' & Mess3='' 
  MessRem=''
  widget_control, EV_struct.bmess, set_value=Mess
  widget_control, EV_struct.bmess2, set_value=Mess2
  widget_control, EV_struct.bmess3, set_value=Mess3
  widget_control, EV_struct.bmessRem, set_value=MessRem
  widget_control, EV_struct.bfile, get_value=txt_file
  new_file = strtrim(txt_file[0],2)
  old_file = EV_struct.curFile
 
  ; Test if we have a file to read
  ; ******************************
  okR = 0
  if uv eq 'file'         then okR = 1 else $          ; <cr> entered from [Input file]
  if uv ne 'expcur' and uv ne 'explst' and old_file ne new_file then okR = 2                 ; [Input file] modified
  if (uv eq 'browseW') or (uv eq 'browseD') then begin ; browse asked
     if (uv eq 'browseW') then pthI = EV_struct.Wpath else pthI = path_for_online
     fifi=dialog_pickfile(filter='*.lst',path=pthI,get_path=pthO, /read, resource='lamptouch')
     if fifi gt ' ' then begin
        okR = 3 & new_file = file_basename(fifi)
        if (uv eq 'browseW') then EV_struct.Wpath = pthO else EV_struct.Dpath = pthO
        EV_struct.Cpath = pthO
        widget_control,EV_struct.bfile, set_value=new_file
     endif
  endif
  widget_control, /hourglass

  if okR gt 0 then begin widget_control,EV_struct.bmess,  set_value='Reading the file'
                         event_file_treat, 'read', EV_struct, file=EV_struct.Cpath+new_file, $
                                            Message=Mess, Mess2=Mess2, Mess3=Mess3, MessRem=MessRem, W=Wi
                         widget_control,EV_struct.bmess, set_value = Mess
                         widget_control,EV_struct.bmess2, set_value = Mess2
                         widget_control,EV_struct.bmess3, set_value = Mess3
                         widget_control,EV_struct.bmessRem, set_value = MessRem
                         EV_struct.curFile = new_file
                         if EV_struct.Ndata eq 1 then begin
                                  mini='0' & maxi='0'                   ;BAD FILE!
                         endif else begin
                                  mini = strtrim(EV_struct.mxTime[0],2) ;Set min max Timer
                                  maxi = strtrim(EV_struct.mxTime[1],2)
                         endelse
                         widget_control, EV_struct.bidmn, set_value=mini ;Update widget values
                         widget_control, EV_struct.bidmx, set_value=maxi
                         widget_control, EV_struct.bminT, set_value=mini
                         widget_control, EV_struct.bmaxT, set_value=maxi
                         if EV_struct.Ndata gt 1 then uv = 'show' ;Force a plot
  endif

  ; Input text widgets when changed without event
  ; *********************************************
  on_ioerror, misNslice ;Number of slices
  widget_control,EV_struct.bslice, get_value=str & Ns = long(str[0]) > 1
  if (EV_struct.Nslice ne Ns) then begin
     widget_control, EV_struct.bslidS, set_slider_max=Ns ;Update current frame slider
     EV_struct.Nslice = Ns  
  endif
  misNslice:
  
  on_ioerror, misMinT   ;Min Timer
  widget_control,EV_struct.bminT,  get_value=str & Mn = double(str[0]) > 0D
  EV_struct.mimaxT[0] = Mn
  misMinT:
  
  on_ioerror, misMaxT   ;Max Timer
  widget_control,EV_struct.bmaxT,  get_value=str & Mx = double(str[0]) > 0D
  EV_struct.mimaxT[1] = Mx
  misMaxT:

    
  ; Call the treatment macro corresponding to the case event
  ; ********************************************************
  case uv of
  'help'   : begin
             end
  'raw'    : begin & EV_struct.removeBad = ev.select               
             end
  'usetrigger': begin 
                 EV_struct.UseCtrigg = 0
                 EV_struct.UseStrigg = 0   
                 event_file_treat, 'clear_trigger'                  
                 widget_control, EV_struct.btrigg, get_value = trigger_choice
                 if trigger_choice eq 1 then EV_struct.UseCtrigg = 1
                 if trigger_choice eq 2 then EV_struct.UseStrigg = 1
               end
;  'slices':  begin & EV_struct.slice = ev.select                ; Slices mode
;                     widget_control, EV_struct.bslice, sens= ev.select
;                     widget_control, EV_struct.bslidS, sens= ev.select
;                     event_file_treat, uv, EV_struct
;             end
  'slides':  begin & EV_struct.Cframe= ev.value                ; Current frame (slider)
                     if ev.value eq 0 then widget_control,EV_struct.bslidL, set_value='All'
                     if ev.value gt 0 then widget_control,EV_struct.bslidL, set_value='Current'
                     event_file_treat, uv, EV_struct
             end
  'nslice':  begin & event_file_treat, uv, EV_struct           ; # slices changed
             end ; See Input text widgets
  'mintime': begin & event_file_treat, uv, EV_struct           ; Min Timer changed
             end ; See Input text widgets
  'maxtime': begin & event_file_treat, uv, EV_struct           ; Max Timer changed
             end ; See Input text widgets
  'setmint': begin ; Original min Timer                        ; Min Timer reseted
                   widget_control, EV_struct.bidmn, get_value=mn
                   widget_control, EV_struct.bminT, set_value=mn
                   EV_struct.mimaxT[0] = Mn
                   event_file_treat, uv, EV_struct
             end
  'setmaxt': begin ; Original max Timer                        ; Max Timer reseted
                   widget_control, EV_struct.bidmx, get_value=mx
                   widget_control, EV_struct.bmaxT, set_value=mx
                   EV_struct.mimaxT[1] = Mx
                   event_file_treat, uv, EV_struct
             end
;  'tslice': begin ; Set time per slice
;               widget_control, EV_struct.btslice, set_value=strtrim(TPS,2)
;               ;event_file_treat, uv, EV_struct
;             end
  'bin_lin': begin ; Bin Spacing mode linear
                   widget_control, ev.id,           get_value=bmod
                   widget_control, EV_struct.bBin , set_value=bmod
                   EV_struct.BinMod = 1
             end
  'bin_log': begin ; Bin Spacing mode log
                   widget_control, ev.id,           get_value=bmod
                   widget_control, EV_struct.bBin,  set_value=bmod
                   EV_struct.BinMod = 2
             end
  'bin_equ': begin ; Bin Spacing mode equal events
                   widget_control, ev.id,           get_value=bmod
                   widget_control, EV_struct.bBin,  set_value=bmod
                   EV_struct.BinMod = 3
             end
  'bin':     begin ; Bin Spacing mode equal events
                widget_control, EV_struct.bBin,  get_value=bmod
                case bmod of 
                  'Linear bin  ': begin
                                   EV_struct.BinMod = 1
                                   event_file_treat, 'bin', EV_struct
                                 end
                  'Log    bin  ': begin
                                EV_struct.BinMod = 2
                                event_file_treat, 'bin', EV_struct
                              end
                  'Equal events': begin
                                 EV_struct.BinMod = 3
                                 event_file_treat, 'bin', EV_struct
                               end                           
                endcase  
             end
  'show':    begin ; Choose current show mode
                   Wcur = ([0,1,2,3,4,5,6])[EV_struct.ShowMod]
                   event_file_treat, 'show', EV_struct, W=Wcur
                   widget_control, EV_struct.bmess4, set_value='-> W'+strtrim(Wcur,2)
             end
  'show_histo':   begin ; Histogram sum
                   widget_control, ev.id,           get_value=pmod
                   widget_control, EV_struct.bPlot, set_value=pmod
                   EV_struct.ShowMod = 2
                   Wcur = ([0,1,2,3,4,5,6])[EV_struct.ShowMod]
                   event_file_treat, 'show', EV_struct, W=Wcur
                   widget_control, EV_struct.bmess4, set_value='-> W'+strtrim(Wcur,2)
             end
  'show_event':   begin ; Event Data
                   widget_control, ev.id,           get_value=pmod
                   widget_control, EV_struct.bPlot, set_value=pmod
                   EV_struct.ShowMod = 3
                   Wcur = ([0,1,2,3,4,5,6])[EV_struct.ShowMod]
                   event_file_treat, 'show', EV_struct, W=Wcur
                   widget_control, EV_struct.bmess4, set_value='-> W'+strtrim(Wcur,2)
             end
  'show_bin':   begin ; Bin Details
                   widget_control, ev.id,           get_value=pmod
                   widget_control, EV_struct.bPlot, set_value=pmod
                   EV_struct.ShowMod = 1
                   Wcur = ([0,1,2,3,4,5,6])[EV_struct.ShowMod]
                   event_file_treat, 'show', EV_struct, W=Wcur
                   widget_control, EV_struct.bmess4, set_value='-> W'+strtrim(Wcur,2)
             end
   'show_trigger':   begin ; Bin Details
                   widget_control, ev.id,           get_value=pmod
                   widget_control, EV_struct.bPlot, set_value=pmod
                   EV_struct.ShowMod = 4
                   Wcur = ([0,1,2,3,4,5,6])[EV_struct.ShowMod]
                   event_file_treat, 'show', EV_struct, W=Wcur
                   widget_control, EV_struct.bmess4, set_value='-> W'+strtrim(Wcur,2)
             end
   'show_dist_events':   begin ; Time distribution of detector events
               widget_control, ev.id,           get_value=pmod
               widget_control, EV_struct.bPlot, set_value=pmod
               EV_struct.ShowMod = 5
               Wcur = ([0,1,2,3,4,5,6])[EV_struct.ShowMod]
               event_file_treat, 'show', EV_struct, W=Wcur
               widget_control, EV_struct.bmess4, set_value='-> W'+strtrim(Wcur,2)
             end
   'show_dist_triggers':   begin ; Time distribution of trigger events
               widget_control, ev.id,           get_value=pmod
               widget_control, EV_struct.bPlot, set_value=pmod
               EV_struct.ShowMod = 6
               Wcur = ([0,1,2,3,4,5,6])[EV_struct.ShowMod]
               event_file_treat, 'show', EV_struct, W=Wcur
               widget_control, EV_struct.bmess4, set_value='-> W'+strtrim(Wcur,2)
             end
  'expcur':  begin ;Export current binned file 
                   widget_control, ev.id,           get_value=pmod
                   event_file_treat, 'expcur', EV_struct
             end
  'explst': begin ;Export a list of files 
                   widget_control, ev.id,           get_value=pmod
                   event_file_treat, 'explst', EV_struct
             end
  
  'quit':    begin & widget_control, ev.top,/destroy
                   return
             end
             
  'valuetimecycle': begin ;Set manually cycle time 
                      widget_control, EV_struct.bcycle, get_value=str
                      tcycle = double(str[0])
                      EV_struct.tslice = tcycle/EV_struct.nslice
                      widget_control, EV_struct.btslice, set_value = strtrim(EV_struct.tslice,2)
                    end           
  else:
  endcase

  ;time/cycle
  if uv ne 'valuetimecycle' then begin
    tmess = ''
    if EV_struct.USECTRIGG then begin
      if n_elements(Ctimer) ge 2 then begin
        time_diff = Ctimer[1:*] - Ctimer[0:n_elements(Ctimer)-2]
        tcycle = mean(time_diff)/EV_struct.clock 
        tmin = min(time_diff)/EV_struct.clock * 1000 ;*1000 -> time in milliseconds
        tmax = max(time_diff)/EV_struct.clock * 1000 ;*1000 -> time in milliseconds
        tmess = ' (Min = ' + strtrim(tmin,2) + ',  Max = ' + strtrim(tmax,2) + ')'
      endif else tcycle = EV_struct.mimaxT[1] - Ctimer[0]/EV_struct.clock
    endif else if EV_struct.USESTRIGG then begin
      if n_elements(Stimer) ge 2 then begin
        time_diff = Stimer[1:*] - Stimer[0:n_elements(Stimer)-2]
        tcycle = mean(time_diff)/EV_struct.clock 
        tmin = min(time_diff)/EV_struct.clock * 1000 ;*1000 -> time in milliseconds
        tmax = max(time_diff)/EV_struct.clock * 1000 ;*1000 -> time in milliseconds
        tmess = ' (Min = ' + strtrim(tmin,2) + ',  Max = ' + strtrim(tmax,2) + ')'
      endif else tcycle = EV_struct.mimaxT[1] - Stimer[0]/EV_struct.clock
    endif else tcycle = EV_struct.mimaxT[1] - EV_struct.mimaxT[0]
    tcycle *= 1000.0 ;time in milliseconds
    
    widget_control, EV_struct.bcycle, set_value = strtrim(tcycle,2)
    ;widget_control, EV_struct.bcycleMess, set_value = tmess
  endif
  
  ;update time/slice when needed (value modified manually or trigger changed or number of slices changed)
  if uv eq 'tslice' then begin
    widget_control, EV_struct.btslice, get_value = str
    EV_struct.tslice = double(str[0])
  endif else if uv eq 'nslice' or uv eq 'usetrigger' then begin
    EV_struct.tslice = tcycle/EV_struct.nslice
    widget_control, EV_struct.btslice, set_value = strtrim(EV_struct.tslice,2)
  endif

  widget_control, EV_struct.bmess, set_value=Mess

end

pro event_file_old, instrument=instrument, nowind=nowind
;+
;** **********
;** User callable
;** Event_file analysis GUI
;** keyword /nowind allows access for a web client.
;** keyword instrument to select a particular instrument 
;** (supported instruments: d22, salsa, d11)
;-
@lamp.cbk

common c_event_file, EV_struct

;determine instrument
valid_instruments = ['d22', 'salsa', 'd11']

if keyword_set(instrument) then begin
  instrument = strlowcase(strtrim(instrument,2))
endif else begin
  ;Try to get instrument name from Lamp interface
  if keyword_set(inst_value) then instrument = strlowcase(strtrim(inst_value,2))
endelse

if ~keyword_set(instrument) then begin
  error_message = ['No instrument given or found.', 'Use call event_file, instrument="instrument name"', 'Exiting!']
  result = dialog_message(error_message, /error)
  return
endif

;check if instrument is in list of valid instruments
result = where(strmatch(valid_instruments, instrument))
if result lt 0 then begin
  line = 'Instrument ' + instrument
  error_message = [line, ' is not yet supported.', 'Exiting!']
  result = dialog_message(error_message, /error)
  return
endif

vers = 'v0.91 - Instrument: ' + instrument
if keyword_set(nowind) then mapi=0 else mapi=1

if (xregistered('event_file_win') eq 0) or (n_elements(EV_struct) eq 0) then begin

  if (mapi) then begin

    base0  = widget_base(title='Event file analysis (old electronics) - ' + vers, /column, resource_name='lamptouch')
    Msiz   = 530 ; Message Xsize in pixels
    
    ;Input file
    brow1 = widget_base(  base0 , /row)
    bid   = widget_label( brow1 , value='Input File '      , font=ft_b_normal)
    bfile = widget_text(  brow1 , value='.lst', xsize=15   , font=ft_propor  , uv='file',/edit)
    bidm  = widget_button(brow1 , value='Browse', /menu    , font=ft_b_normal)
    bid   = widget_button(bidm  , value='From Working Path', font=ft_b_normal, uv='browseW')
    bid   = widget_button(bidm  , value='From Data    Path', font=ft_b_normal, uv='browseD')
    braw  = widget_button(widget_base(brow1,/nonexclusive) , font=ft_b_normal, uv='raw', value='Remove bad events')
    widget_control, braw, set_button=0
    bid   = widget_label( brow1 , value=' '                , font=ft_propor)

    ;Label messages
    bmess  = widget_label( base0 , value=' ', xsize=Msiz    , font=ft_b_normal, /align_left)
    bmess2 = widget_label( base0 , value=' ', xsize=Msiz    , font=ft_b_normal, /align_left)
    bmess3 = widget_label( base0 , value=' ', xsize=Msiz    , font=ft_b_normal, /align_left)
    bmessRem = widget_label( base0 , value=' ', xsize=Msiz   , font=ft_b_normal, /align_left)

    ;Time range box
    brow2 = widget_base(  base0 , /row, /frame)
    bid   = widget_label( brow2 , value='Time Range (s) '   , font=ft_b_normal)
    bidm  = widget_button(brow2 , value='Min'               , font=ft_b_normal, /menu)
    bid   = widget_button(bidm  , value='  '                , font=ft_propor)
    bidmn = widget_button(bidm  , value='0.00'              , font=ft_propor,   uv='setmint')
    bminT = widget_text(  brow2 , value='0. ', xsize=14     , font=ft_propor,   uv='mintime', /edit)
    bidm  = widget_button(brow2 , value='Max'               , font=ft_b_normal, /menu)
    bid   = widget_button(bidm  , value='  '                , font=ft_propor)
    bidmx = widget_button(bidm  , value='0.00'              , font=ft_propor,   uv='setmaxt')
    bmaxT = widget_text(  brow2 , value='0. ', xsize=14     , font=ft_propor,   uv='maxtime', /edit)

    ;Box for binning options
    boxBinning  = widget_base(base0, /column, /frame)
    
    ;Triggers and slices
    brow3  = widget_base(boxBinning, /column)
    
;    brow31 = widget_base(brow3 , /column, /exclusive)
;    bid0   = widget_button(brow31, value=' No trigger',          font=ft_b_normal, uv='Notrigger')
;    bid1   = widget_button(brow31, value=' Use Chopper trigger', font=ft_b_normal, uv='Ctrigger')
;    bid2   = widget_button(brow31, value=' Use Sample trigger' , font=ft_b_normal, uv='Strigger')
    brow30 = widget_base(brow3 , /row)
    values = [' No ', ' Chopper ', ' Sample ']
    btrigg = cw_bgroup(brow30, values, /row, /exclusive, /frame, font=ft_b_normal, $
                       label_left = 'Use Trigger?  ', set_value=0, uv='usetrigger')
    bid = widget_label(brow30, value=' ', xsize=3)
    ;bdelay used to give min time between triggers
    bdelay  = widget_text (brow30, value='  ',  Xsize=10 , font=ft_propor  , uv='nslice', /edit)
    bid = widget_label(brow30, value=' (min delay in ms)', xsize=105)
                       
    brow31 = widget_base(brow3 , /row)
    bid = widget_label(brow31, value='Time/cycle (ms) = ', font=ft_b_normal, uv='timecycle', /align_left)
    bcycle  = widget_text (brow31, value=' ',  Xsize=10 , font=ft_propor  , uv='valuetimecycle', /edit)
    ;bcycle = widget_label( brow31 , value=' ', xsize=70   , font=ft_b_normal, /align_left)                 
    ;bcycleMess = widget_label( brow31 , value=' ', xsize=200   , font=ft_b_normal, /align_left)

    brow32 = widget_base(brow3 , /row)
    bid4   = widget_label(brow32, value='Number of slices = ', font=ft_b_normal, uv='slices')
    bid = widget_label(brow32, value=' ', xsize=3)
    bslice  = widget_text (brow32, value=' 1 ',  Xsize=3 , font=ft_propor  , uv='nslice', /edit)
    bid = widget_label(brow32, value=' ', xsize=20)

    ;brow33  = widget_base(brow3 , /row)
    bid6   = widget_label(brow32, value='Time/slice (ms) = ', font=ft_b_normal, uv='timeslices')
    bid = widget_label(brow32, value=' ', xsize=10)
    btslice = widget_text (brow32, value=' ',  Xsize=10, font=ft_propor  , uv='tslice', /edit)
    bid = widget_label(brow32, value=' (linear bin)', xsize=105)
    
    ;Binning
    brow4  = widget_base(boxBinning , /row)
    bid5 = widget_label( brow4 , value='Binning mode: '    , font=ft_propor)
    bbin = widget_button(brow4 , value='Linear bin  '     , font=ft_propor,  /menu)
    bid  = widget_button(bbin  , value='Linear bin  '     , font=ft_propor,   uv='bin_lin')
    bid  = widget_button(bbin  , value='Log    bin  '     , font=ft_propor,   uv='bin_log')
    bid  = widget_button(bbin  , value='Equal events'     , font=ft_propor,   uv='bin_equ')
    bid8 = widget_label(brow4, value=' ', xsize=3)
    bdobin = widget_button(brow4 , value='BIN '             , font=ft_b_normal, uv='bin')

    ;View box
    brow5  = widget_base(base0 , /row, /frame)
    bid9 = widget_label( brow5 , value='View: ', font=ft_propor)
    bplot= widget_button(brow5 , value='Bin Details ', font=ft_propor,  /menu)
    bid = widget_button(bplot , value='Bin Details' , font=ft_propor,   uv='show_bin')
    bid = widget_button(bplot , value='Total 2D' , font=ft_propor,   uv='show_histo')
    bid = widget_button(bplot , value='Det events'  , font=ft_propor,   uv='show_event')
    bid = widget_button(bplot , value='Trigger step'  , font=ft_propor,   uv='show_trigger')
    bid = widget_button(bplot , value='Time dist events'  , font=ft_propor,   uv='show_dist_events')
    bid = widget_button(bplot , value='Time dist triggers'  , font=ft_propor,   uv='show_dist_triggers')
    bid10 = widget_label(brow5, value=' ', xsize=3)
    bshow = widget_button(brow5 , value='SHOW '      , font=ft_b_normal, uv='show')
    bid11 = widget_label(brow5, value=' ', xsize=10)
    brow53 = widget_base(  brow5, /row)
    bslidS = widget_slider(brow53, min=0, max=1, Xsize=200, font=ft_b_normal, uv='slides', title='')
    bslidL = widget_label( brow53, value=' '   , Xsize=1 , font=ft_b_normal,/align_center)
    
    ;Export/Quit box
    brow6  = widget_base(base0 , /row, /frame)
    bid12 = widget_label(brow6, value='Data in workspace ' , font=ft_b_normal)
    bmess4  = widget_label( brow6 , value=' ', xsize=50    , font=ft_b_normal, /align_center)
    bid   = widget_label( brow6 , value='', xsize=75       , font=ft_propor)
    bexpo = widget_button(brow6, value='Export binned data', font=ft_b_normal, /menu)
    bid = widget_button(bexpo , value='Current File'       , font=ft_propor,   uv='expcur')
    bid = widget_button(bexpo , value='From a FileList'    , font=ft_propor,   uv='explst')
    bid   = widget_label( brow6 , value='', xsize=10       , font=ft_propor)
    bidh  = widget_button(brow6 , value=' ? '              , font=ft_b_normal, uv='help')
    bid   = widget_label( brow6 , value='', xsize=10       , font=ft_propor)
    bquit = widget_button(brow6 , value='Quit'             , font=ft_b_normal, uv='quit')

    put_logo, widget_base(brow1, event_pro='lamp_event_parser')
    widget_control,base0,group_leader=lamp_b1,/realize
    put_logo
    xmanager, 'event_file_win', base0,event_handler='event_file_event',CLEANUP='event_file_dying',/just_reg,/no_block

  endif else begin ; Web client, so widget_Ids are unavailable
    bmess=0L & bfile=0L & bslice=0L & bslidS=0L & bidmn=0L & bidmx=0L
    bminT=0L  & bmaxT=0L & bmess2=0L & bmess3=0L & bmess4=0L & bmessRem=0L & bBin=0L  & bPlot=0L
  endelse

  EV_struct={bmess:bmess, bmess2:bmess2, bmess3:bmess3, bmessRem:bmessRem, bmess4:bmess4, $
    bfile:bfile, curFile:'.lst', Wpath:'', Dpath:path_for_online, Cpath:'', $
    btrigg:btrigg, bdelay:bdelay, $
    bslice:bslice, btslice:btslice, Nslice:1L, Noverlap:0., bslidS:bslidS, Cframe:0L, bslidL:bslidL, $
    mxTime:[0D,0D], bidmn:bidmn, bidmx:bidmx, bminT:bminT, bmaxT:bmaxT,   $
    mimaxT:[0D,0D], Strigg:0L, Ctrigg:0L, Ndata:0L, Monitor:0L, UseCtrigg:0, UseStrigg:0,   $
    BinMod:1, bBin:bBin, bdoBin:bDoBin, ShowMod:1, bPlot:bPlot, bShow:bShow, $
    noWind:1-mapi, vers:vers, removeBad:0, bcycle:bcycle, clock:1.e7, tslice:0D, $
    inst:instrument, xsize:256L, ysize:256L}

  ;bmess   = label widget message
  ;bmess2  = label widget message 2nd line
  ;bmess3  = label widget message 3rd line
  ;bmessRem = label widget message 4th line: events removed
  ;bfile   = text  widget input file
  ;curFile = filename
  ;Wpath   = working path
  ;Dpath   = data path
  ;Cpath   = current path
  ;trigg   = 1:use triggers else 0
  ;slice   = 1:use slices   else 0
  ;btrigL  = trigger label
  ;bslice  = text  widget input number of slices
  ;Nslice  = number of slices
  ;bslids  = slider widget
  ;bslidL  = slider label
  ;Cframe  = current choosen frame
  ;mxTime  = [min,max] data timer original
  ;bidmn   = button mini data timer
  ;bidmx   = button maxi data timer
  ;bminT   = text  widget input min Timer
  ;bmaxT   = text  widget input max Timer
  ;mimaxT  = Min & Max choosen timer
  ;Strigg  = number of sample triggers
  ;Ctrigg  = number of chopper triggers
  ;Ndata   = number of event data
  ;BinMod  = 1:Linear , 2:Log , 3:Equal events
  ;bBin    = button menu for Binmod
  ;ShowMod = 1:Histogram , 2:Event Data , 3:Bin Details , etc
  ;bPlot   = button menu for ShowMod
  ;noWind  = 1:invisible Gui (or web client)
  ;vers    = code version date
  ;removeBad   = 0/1: 1 = remove bad events
  ;btslice  = time of each slice (needed for sliding average)
  ;bdelay
 
endif
end
