;+
; This program reads ascii dcs file derived
; from the dcs_binary2ascii routines written by J. Copley
; N.B: all the experimental parameter were saved ... will read only a few of them
;
;     Standard call for a wout-read function interfacing LAMP.
;
;     Return of the function
;      wout     is an array of any dimensions and type containing the wout values (spectra).
;
;     Input  parameters:
;      PATH     is the full path where to find the wout (string defined in customize tables).
;      FILENAME is the name of the wout file.
;               if FILENAME(1) exists,  this is the requested image number in the file.
;
;     Output parameters:
;      STATUS   is the returned error code you can choose from the following list:
;               0 =' Successfull read'
;               1 =' Client/server on local node not established'
;               2 =' Client/server on router node not established'
;               3 =' The local  node cannot access the server node'
;               4 =' The router node cannot access the server node'
;               5 =' VME memory read error'
;               7 =' Sequence error in wout transfer'
;               9 =' Parameter error'
;               10=' Router is busy with other transfer'
;               11=' Cant open the file or file not found'
;               13=' wout file incomplete'
;               14=' Bad instrument wout definition'
;               24=' Cant read the file'.
;
;      DATP     is a structure defined as follow: (all tags are OPTIONAL)
;               DATP.X        = vector of x coordinates.
;               DATP.Y        = vector of y coordinates.
;               DATP.Z        = vector of z coordinates.
;               DATP.W_TIT    =   main title
;               DATP.X_TIT    = x axis title
;               DATP.Y_TIT    = y axis title
;               DATP.Z_TIT    = z axis title
;               DATP.OTHER_TIT=    sub title
;               DATP.N        = monitors
;               DATP.P        = vector of float parameter values up to 41
;               DATP.PAR_TXT  = string array of text associated to DATP.P (same size)
;               DATP.PV       = an array of any dimensions containing other parameter values
;               DATP.E        = the errors associated to wout (same size)
;               DATP.TIME     = string date of the experiment.
;
;
;
; S. Rols August 2001 (rols@ill.fr)
;
; JO, Thu Mar 22 09:14:33 CET 2007 Reworked to read new DCS formats (from ascii files)
;                                  Binary data needs first to be translated into ascii files
;                                  using dcs_binary2ascii.pro (several routines):
;                                    retall
;                                    .r read_dcsasc.pro
;                                    .r dcs_binary2ascii.pro
;                                    .r dcs_rawdata_variables_choose.pro
;                                    .r dcs_rawdata_ptr.pro
;                                    .r dcs_read_binaryoctavefile_ptr.pro
;                                    .r dcs_write_summary_ptr.pro
;                                    .r dcs_write_asciioctavefile_ptr.pro
;
;                                    At the IDL prompt:
;                                    >> dcs_binary2ascii
;                                    Then save data with all parameters:
;                                    "Check All" 
;                                    "Make individual Files"
;                                    "Done" (ask for the name of the .dcs file then -- one by one)
;        
;
;                                 Scattering angles are hard-coded in this routine as they do 
;                                 not appear in the raw data files.
;
;                                 DCS data are then integrable in LAMP as if it would be IN5 data 
; 
;  JO, Mon Jun 16 12:05:20 CEST 2008  Corrected an error on the name/runnumber in paratmeters
;-

function read_dcsasc, FILENAME, verbose = verbose
;+
;******* ***********
;** This program reads ascii dcs file derived
;** from the dcs_binary2ascii routines written by J. Copley
;** N.B: all the experimental parameter were saved ... will read only a few of them
;-

;; set the path to the working directory
 cd, current = PATH
; this routine only for DCS
 INST = 'dcs'
 
 wout  = 0
 STATUS= 11    ;Handle non-IO errors
 CATCH,stat 

 
 if stat ne 0 then begin 
    print,!error_state.msg 
    RETURN, wout
endif

;** ***********************************
;** newpath and newfile construction **
;** ***********************************
    NEWPATH=PATH
    NEWFILE=FILENAME[0]
    if strlen(NEWFILE) eq 6 then begin
;    Input PATH     is of the form "/........../2001/"
;    Input FILENAME is of the form "MMDDnb"
;    --------------------------------------
    ii=strlen(NEWPATH)
    yy=strmid(NEWPATH,ii-5,4)
    mm=strmid(NEWFILE,0,2)
    dd=strmid(NEWFILE,2,2)
    nb=strmid(NEWFILE,4,2)
    if (mm gt 12) or (dd gt 31) then return,wout
    NEWPATH=strmid(NEWPATH,0,ii-1)+mm+strmid(NEWPATH,ii-1,1)
    NEWFILE=yy+mm+dd+'_'+nb+'.dcs'
        strrunn=yy+mm+dd+nb
    run_number=float(strrunn)
    rr=file_search(NEWPATH+NEWFILE+'.gz.',count=n)
    if n gt 0 then NEWFILE=NEWFILE+'.gz.'
    endif else begin
        ii=strlen(NEWFILE)
        yy=strmid(NEWFILE,0,4)
    mm=strmid(NEWFILE,4,2)
    dd=strmid(NEWFILE,6,2)
    nb=strmid(NEWFILE,9,3)
    strrunn=yy+mm+dd+nb
        ; a slash is missing in the filepath-filename
        NEWFILE = '/'+NEWFILE
    endelse

;   Open  the wout file, use EXECUTE for compatibility with Idl5.2
;   --------------------------------------------------------------
;   ON_IOERROR, no_file
    if strpos(NEWFILE,'.gz') gt 0 then cmps=',/compress' else cmps=''
    unit = -1
;;    if keyword_set(verbose) then $
        print, 'READ_DCSASC: Loading ',NEWPATH+NEWFILE,' ...'
    rr=EXECUTE('OPENR,unit, NEWPATH+NEWFILE,/get_lun'+cmps)

    if unit le 0 then begin
        print, 'Last error: ',!error_state.msg
        return,wout
    endif
;** **********************
;** End of construction ******************************************
;** **********************

STATUS=13
ON_IOERROR, read_err
iunit=unit

;** **************************************
;** Reading in the .asc files and dealing
;** with wout and parameter
;** **************************************
twolines=strarr(2)
while (not eof(iunit)) do begin
    readf, iunit, twolines
    ;print, twolines
    strname=strcompress(twolines[0],/remove_all) & len0=strlen(strname)-6
    strtype=strcompress(twolines[1],/remove_all) & len1=strlen(strtype)-6
   ; print, strname
   ; print, strtype
    strname=strmid(strname,6,len0)
    strtype=strmid(strtype,6,len1)
    ;print, strname
    ;print, strtype
    ; On a lu le nom de la variable dans strname
    ; On a lu le type de cette variable et on va creer une variable tampon du meme type: vartamp
    case strtype of
        "scalar": begin
                    vartamp=0.
                  end
        "matrix": begin
                    readf,iunit, twolines
                    twolines=strcompress(twolines,/remove_all)
                    strrow=twolines[0] & len0=strlen(strrow)-6
                    strcolumn=twolines[1] & len1=strlen(strcolumn)-9
                    strrow=strmid(strrow,6,len0)
                    strcolumn=strmid(strcolumn,9,len1)
                    nrow=float(strrow) & ncolumn=float(strcolumn)
                    vartamp=fltarr(nrow,ncolumn)
                  end
   "stringarray": begin
                    strnumel=strarr(1)
                    readf, iunit, strnumel
                    strnumel=strcompress(strnumel,/remove_all) & len0=strlen(strnumel[0])-10
                    strnumel=strmid(strnumel,10,len0)
                    nelement=float(strnumel[0])
                    ;print, strnumel
                    ;print, nelement
                    if (nelement gt 1.) then begin
                        vartamp=strarr(2*nelement)
                    endif else begin
                        strlength=strarr(1)
                        readf,iunit, strlength
                        vartamp=strarr(1)
                    endelse
                  end
    endcase
    case strname of
        "CH_DELAY": begin ; ToFDelay / chopper
                        readf, iunit,vartamp
                        chopper1_delay = vartamp[0]
                        chopper2_delay = vartamp[1]
                        chopper3_delay = vartamp[2]
                        chopper4_delay = vartamp[3]
                        chopper5_delay = vartamp[4]
                        chopper6_delay = vartamp[5]
                        chopper7_delay = vartamp[6]
                    end
        "CH_DIS": begin ; sample- choppers distances
                        readf, iunit,vartamp
                        sample_chopper1 = vartamp[0]
                        sample_chopper2 = vartamp[1]
                        sample_chopper3 = vartamp[2]
                        sample_chopper4 = vartamp[3]
                        sample_chopper5 = vartamp[4]
                        sample_chopper6 = vartamp[5]
                        sample_chopper7 = vartamp[6]
                    end
        "CH_INPUT": begin ; des frequences de rotation des choppers
                        readf, iunit,vartamp
                        w_chop = vartamp[1] 
                        ratio1 = vartamp[2]
                        ratio2 = vartamp[3] 
                        ratio3 = vartamp[4]
                        period_at_sample = vartamp[5]
                    end
        "CH_PHASE": begin ; sample- choppers distances
                        readf, iunit,vartamp
                        chopper1_phase = vartamp[0]
                        chopper2_phase = vartamp[1]
                        chopper3_phase = vartamp[2]
                        chopper4_phase = vartamp[3]
                        chopper5_phase = vartamp[4]
                        chopper6_phase = vartamp[5]
                        chopper7_phase = vartamp[6]
                    end
        "COMMENTS": begin ; lecture du titre du run
                        title=''
                        readf, iunit, vartamp
                        title=vartamp[0]
                    end
        "DET_DIS" : begin ; lecture de la distance Sample-detector
                        readf, iunit, vartamp
                        dist_sd=vartamp
                    end
        "DURATION": begin ; lecture de la duree du run
                        readf, iunit, vartamp
                        duration=vartamp
                    end
       "HISTOHIGH": begin ; lecture du moniteur
                        readf, iunit, vartamp
                        spec_mon=vartamp[*,3]
                    end
       "HISTODATA": begin ;lecture des donnees
                        readf, iunit, vartamp
                        wout = vartamp
                    end
      "START_DATE": begin ; lecture de la date START
                        startd=''
                        readf, iunit, vartamp
                        startd=vartamp
                    end
       "STOP_DATE": begin ; lecture de la date STOP
                        stopd=''
                        readf, iunit, vartamp
                        stopd=vartamp
                    end
           "CH_WL": begin ; lecture de la long. d'onde incidente
                        readf, iunit, vartamp
                        lambda=vartamp
                    end
           "CH_MS": begin ; chopper master speed
                        readf, iunit, vartamp
                        chopper_master_speed = vartamp
                    end
      "CH_SRDENOM": begin ; chopper master ratio
                        readf, iunit, vartamp
                        chopper_ratio = vartamp
                    end
          "TSDMIN": begin ; lecture du minimal flight time
                        readf, iunit, vartamp
                        tof_minimum = vartamp
                    end
         "woutMAX": begin ; Nb angle max ?
                        readf, iunit, vartamp
                        woutmax = vartamp
                    end
         "DETSUM": begin ; lecture des moniteurs
                        readf, iunit, vartamp
                        mon1=vartamp[915,0]
                        mon2=vartamp[916,0]
                    end
         "TEMP_SETPOINT": begin ; Temperauture setpoint
                        readf, iunit, vartamp
                        temp_setpoint = vartamp
                    end
         "TEMP_CONTROL": begin ; Temperauture setpoint
                        readf, iunit, vartamp
                        ind = where(vartamp ge 0.0)
                        temp_control = mean(vartamp[ind])
                    end
         "TEMP_SAMPLE": begin ; Temperauture setpoint
                        readf, iunit, vartamp
                        ind = where(vartamp ge 0.0)
                        temp_sample = mean(vartamp[ind])
                    end
         "TIMSUM": begin ; Total count
                        readf, iunit, vartamp
                        total_count = total(vartamp)
                    end
        else: readf,iunit, vartamp ; on lit pour rien ATTENTION AUX STRING ARRAY !!!
    endcase
endwhile
;;Result=dialog_message('Lecture successful !!',/information)
close, iunit

if keyword_set(verbose) then begin
     print, 'READ_DCSASC: Chopper matser speed = ', w_chop,' rpm'
     print, 'READ_DCSASC: Incident wavelength = ',lambda,' A'
     print, 'READ_DCSASC: Chopper 1 phase ', chopper1_phase,' deg'
     print, 'READ_DCSASC: Chopper 2 phase ', chopper2_phase,' deg'
     print, 'READ_DCSASC: Chopper 3 phase ', chopper3_phase,' deg'
     print, 'READ_DCSASC: Chopper 4 phase ', chopper4_phase,' deg'
     print, 'READ_DCSASC: Chopper 5 phase ', chopper5_phase,' deg'
     print, 'READ_DCSASC: Chopper 6 phase ', chopper6_phase,' deg'
     print, 'READ_DCSASC: Chopper 7 phase ', chopper7_phase,' deg'
     print, 'READ_DCSASC: Sample chopper 1 distance', sample_chopper1, ' m'
     print, 'READ_DCSASC: Sample chopper 2 distance', sample_chopper2, ' m'
     print, 'READ_DCSASC: Sample chopper 3 distance', sample_chopper3, ' m'
     print, 'READ_DCSASC: Sample chopper 4 distance', sample_chopper4, ' m'
     print, 'READ_DCSASC: Sample chopper 5 distance', sample_chopper5, ' m'
     print, 'READ_DCSASC: Sample chopper 6 distance', sample_chopper6, ' m'
     print, 'READ_DCSASC: Sample chopper 7 distance', sample_chopper7, ' m'
     print, 'READ_DCSASC: Chopper 1 delay ', chopper1_delay,' mu-sec'
     print, 'READ_DCSASC: Chopper 2 delay ', chopper2_delay,' mu-sec'
     print, 'READ_DCSASC: Chopper 3 delay ', chopper3_delay,' mu-sec'
     print, 'READ_DCSASC: Chopper 4 delay ', chopper4_delay,' mu-sec'
     print, 'READ_DCSASC: Chopper 5 delay ', chopper5_delay,' mu-sec'
     print, 'READ_DCSASC: Chopper 6 delay ', chopper6_delay,' mu-sec'
     print, 'READ_DCSASC: Chopper 7 delay ', chopper7_delay,' mu-sec'
     print, 'READ_DCSASC: Ratio 1 = ', ratio1
     print, 'READ_DCSASC: Ratio 2 = ', ratio2
     print, 'READ_DCSASC: Ratio 3 = ', ratio3
     print, 'READ_DCSASC: Anti-overlap ratio', chopper_ratio
     print, 'READ_DCSASC: Period at sample = ', period_at_sample,' mu-sec'
     print, 'READ_DCSASC: Run Title = ',title
     print, 'READ_DCSASC: Detetctor distance = ',dist_sd,' mm'
     print, 'READ_DCSASC: Run duration = ',duration,' sec'
     print, 'READ_DCSASC: Start date: = ',startd
     print, 'READ_DCSASC: Stop date: = ',stopd
     print, 'READ_DCSASC: Chopper master speed = ',chopper_master_speed,' rpm'
     print, 'READ_DCSASC: Chopper ratio = ',chopper_ratio
     print, 'READ_DCSASC: Tof_delay',vartamp,chopper4_delay,' mu-sec'
     print, 'READ_DCSASC: ToF minimum',vartamp,tof_minimum,' mu-sec'
     print, 'READ_DCSASC: Temperature setpoint = ', temp_setpoint, ' K'
     print, 'READ_DCSASC: Temperature control  = ', temp_control, ' K'
     print, 'READ_DCSASC: Temperature sample   = ', temp_sample, ' K'
     print, 'READ_DCSASC: Total count   = ', total_count
endif

; --------------------------------------------------------------------------------------
;
;       Scattering 2*theta for each of the 913 detectors
;       From data file taken: 27-May-2006 10:41:45
;
; --------------------------------------------------------------------------------------

det_angles =  [ -5.02, 5.02, -5.51,  5.51, -6.00,  6.00, -6.49,  6.49, -6.98, $
                6.98, -7.47,  7.47, -7.96,  7.96, -8.45,  8.45, -8.94,  8.94, $
                -9.43,  9.43, -9.92,  9.92,-10.01, 10.01,-10.01, 10.01,-10.41,$
                10.41,-10.81, 10.81,-10.81, 10.81,-10.90, 10.90,-11.39, 11.39,$
                -11.53, 11.53,-11.53, 11.53,-11.88, 11.88,-12.20, 12.20,-12.20,$
                 12.20,-12.37, 12.37,-12.84, 12.84,-12.84, 12.84,-12.86, 12.86,$
                 -13.35, 13.35,-13.46, 13.46,-13.46, 13.46,-13.84, 13.84,-14.07, $
                 14.07,-14.07, 14.07,-14.33, 14.33,-14.66, 14.66,-14.66, 14.66,$
                 -14.82, 14.82,-15.24, 15.24,-15.24, 15.24,-15.31, 15.31,-15.80, $
                 15.80,-15.81, 15.81,-15.81, 15.81,-16.29, 16.29,-16.38, 16.38,$
                 -16.38, 16.38,-16.78, 16.78,-16.94, 16.94,-16.94, 16.94,-17.27, $
                 17.27,-17.49, 17.49,-17.49, 17.49,-17.76, 17.76,-18.04, 18.04,$
                 -18.04, 18.04,-18.25, 18.25,-18.58, 18.58,-18.58, 18.58,-18.74, $
                 18.74,-19.12, 19.12,-19.12, 19.12,-19.23, 19.23,-19.66, 19.66,$
                 -19.66, 19.66,-19.72, 19.72,-20.19, 20.19,-20.19, 20.19,-20.21, $
                 20.21,-20.70, 20.70,-20.72, 20.72,-20.72, 20.72,-21.19, 21.19,$
                 -21.25, 21.25,-21.25, 21.25,-21.68, 21.68,-21.78, 21.78,-21.78, $
                 21.78,-22.17, 22.17,-22.31, 22.31,-22.31, 22.31,-22.66, 22.66,$
                 -22.83, 22.83,-22.83, 22.83,-23.15, 23.15,-23.35, 23.35,-23.35, $
                 23.35,-23.64, 23.64,-23.87, 23.87,-23.87, 23.87,-24.13, 24.13,$
                 -24.39, 24.39,-24.39, 24.39,-24.62, 24.62,-24.91, 24.91,-24.91,$
                 24.91,-25.11, 25.11,-25.43, 25.43,-25.43, 25.43,-25.60, 25.60,$
                 -25.95, 25.95,-25.95, 25.95,-26.09, 26.09,-26.46, 26.46,-26.46,$
                  26.46,-26.58, 26.58,-26.97, 26.97,-26.97, 26.97,-27.07, 27.07,$
                  -27.48, 27.48,-27.48, 27.48,-27.56, 27.56,-27.99, 27.99,-27.99, $
                  27.99,-28.05, 28.05,-28.50, 28.50,-28.50, 28.50,-28.54, 28.54,$
                  -29.01, 29.01,-29.01, 29.01,-29.03, 29.03,-29.52, 29.52,-29.52,$
                 29.52,-29.52, 29.52, 30.03, 30.03, 30.03, 30.54, 30.54, 30.54, $
                 31.05, 31.05, 31.05, 31.56, 31.56, 31.56, 32.07, 32.07, 32.07, $
                 32.58, 32.58, 32.58, 33.09, 33.09, 33.09, 33.59, 33.59, 33.59, $
                 34.09, 34.09, 34.09, 34.59, 34.59, 34.59, 35.09, 35.09, 35.09,$
                 35.59, 35.59, 35.59, 36.09, 36.09, 36.09, 36.59, 36.59, 36.59, $
                 37.09, 37.09, 37.09, 37.59, 37.59, 37.59, 38.09, 38.09, 38.09,$
                 38.59, 38.59, 38.59, 39.09, 39.09, 39.09, 39.59, 39.59, 39.59,$
                 40.09, 40.09, 40.09, 40.59, 40.59, 40.59, 41.09, 41.09, 41.09,$
                 41.59, 41.59, 41.59, 42.09, 42.09, 42.09, 42.59, 42.59, 42.59,$
                 43.09, 43.09, 43.09, 43.59, 43.59, 43.59, 44.09, 44.09, 44.09,$
                 44.59, 44.59, 44.59, 45.09, 45.09, 45.09, 45.59, 45.59, 45.59,$
                 46.09, 46.09, 46.09, 46.59, 46.59, 46.59, 47.09, 47.09, 47.09, $
                 47.59, 47.59, 47.59, 48.09, 48.09, 48.09, 48.59, 48.59, 48.59, $
                 49.09, 49.09, 49.09, 49.59, 49.59, 49.59, 50.09, 50.09, 50.09, $
                 50.59, 50.59, 50.59, 51.09, 51.09, 51.09, 51.59, 51.59, 51.59, $
                 52.09, 52.09, 52.09, 52.59, 52.59, 52.59, 53.09, 53.09, 53.09, $
                 53.59, 53.59, 53.59, 54.09, 54.09, 54.09, 54.59, 54.59, 54.59, $
                 55.09, 55.09, 55.09, 55.59, 55.59, 55.59, 56.09, 56.09, 56.09, $
                 56.59, 56.59, 56.59, 57.09, 57.09, 57.09, 57.59, 57.59, 57.59, $
                 58.09, 58.09, 58.09, 58.59, 58.59, 58.59, 59.09, 59.09, 59.09, $
                 59.59, 59.59, 59.59, 60.09, 60.09, 60.09, 60.59, 60.59, 60.59, $
                 61.09, 61.09, 61.09, 61.58, 61.58, 61.58, 62.07, 62.07, 62.07, $
                 62.56, 62.56, 62.56, 63.05, 63.05, 63.05, 63.54, 63.54, 63.54, $
                 64.03, 64.03, 64.03, 64.52, 64.52, 64.52, 65.01, 65.01, 65.01, $
                 65.50, 65.50, 65.50, 65.99, 65.99, 65.99, 66.48, 66.48, 66.48, $
                 66.97, 66.97, 66.97, 67.46, 67.46, 67.46, 67.95, 67.95, 67.95, $
                 68.44, 68.44, 68.44, 68.93, 68.93, 68.93, 69.42, 69.42, 69.42, $
                 69.91, 69.91, 69.91, 70.40, 70.40, 70.40, 70.89, 70.89, 70.89, $
                 71.38, 71.38, 71.38, 71.87, 71.87, 71.87, 72.36, 72.36, 72.36, $
                 72.85, 72.85, 72.85, 73.34, 73.34, 73.34, 73.83, 73.83, 73.83, $
                 74.32, 74.32, 74.32, 74.81, 74.81, 74.81, 75.30, 75.30, 75.30, $
                 75.79, 75.79, 75.79, 76.28, 76.28, 76.28, 76.77, 76.77, 76.77, $
                 77.26, 77.26, 77.26, 77.75, 77.75, 77.75, 78.24, 78.24, 78.24, $
                 78.73, 78.73, 78.73, 79.22, 79.22, 79.22, 79.71, 79.71, 79.71, $
                 80.20, 80.20, 80.20, 80.69, 80.69, 80.69, 81.18, 81.18, 81.18, $
                 81.67, 81.67, 81.67, 82.16, 82.16, 82.16, 82.65, 82.65, 82.65, $
                 83.14, 83.14, 83.14, 83.63, 83.63, 83.63, 84.12, 84.12, 84.12, $
                 84.61, 84.61, 84.61, 85.10, 85.10, 85.10, 85.59, 85.59, 85.59, $
                 86.08, 86.08, 86.08, 86.57, 86.57, 86.57, 87.06, 87.06, 87.06, $
                 87.55, 87.55, 87.55, 88.04, 88.04, 88.04, 88.53, 88.53, 88.53, $
                 89.02, 89.02, 89.02, 89.51, 89.51, 89.51, 90.00, 90.00, 90.00, $
                 90.49, 90.49, 90.49, 90.98, 90.98, 90.98, 91.47, 91.47, 91.47, $
                 91.96, 91.96, 91.96, 92.45, 92.45, 92.45, 92.94, 92.94, 92.94, $
                 93.43, 93.43, 93.43, 93.92, 93.92, 93.92, 94.41, 94.41, 94.41, $
                 94.90, 94.90, 94.90, 95.39, 95.39, 95.39, 95.88, 95.88, 95.88, $
                 96.37, 96.37, 96.37, 96.86, 96.86, 96.86, 97.35, 97.35, 97.35, $
                 97.84, 97.84, 97.84, 98.33, 98.33, 98.33, 98.82, 98.82, 98.82, $
                 99.31, 99.31, 99.31, 99.80, 99.80, 99.80,100.29,100.29,100.29,$
                 100.78,100.78,100.78,101.27,101.27,101.27,101.76,101.76,101.76,$
                 102.25,102.25,102.25,102.74,102.74,102.74,103.23,103.23,103.23,$
                 103.72,103.72,103.72,104.21,104.21,104.21,104.70,104.70,104.70,$
                 105.19,105.19,105.19,105.68,105.68,105.68,106.17,106.17,106.17,$
                 106.66,106.66,106.66,107.15,107.15,107.15,107.64,107.64,107.64,$
                 108.13,108.13,108.13,108.62,108.62,108.62,109.11,109.11,109.11,$
                 109.60,109.60,109.60,110.09,110.09,110.09,110.58,110.58,110.58,$
                 111.07,111.07,111.07,111.56,111.56,111.56,112.05,112.05,112.05,$
                 112.54,112.54,112.54,113.03,113.03,113.03,113.52,113.52,113.52,$
                 114.01,114.01,114.01,114.50,114.50,114.50,114.99,114.99,114.99,$
                 115.48,115.48,115.48,115.97,115.97,115.97,116.46,116.46,116.46,$
                 116.95,116.95,116.95,117.44,117.44,117.44,117.93,117.93,117.93,$
                 118.42,118.42,118.42,118.91,118.91,118.91,119.41,119.41,119.41,$
                 119.91,119.91,119.91,120.41,120.41,120.41,120.91,120.91,120.91,$
                 121.41,121.41,121.41,121.91,121.91,121.91,122.41,122.41,122.41,$
                 122.91,122.91,122.91,123.41,123.41,123.41,123.91,123.91,123.91,$
                 124.41,124.41,124.41,124.91,124.91,124.91,125.41,125.41,125.41,$
                 125.91,125.91,125.91,126.41,126.41,126.41,126.91,126.91,126.91,$
                 127.41,127.41,127.41,127.91,127.91,127.91,128.41,128.41,128.41,$
                 128.91,128.91,128.91,129.41,129.41,129.41,129.91,129.91,129.91,$
                 130.41,130.41,130.41,130.91,130.91,130.91,131.41,131.41,131.41,$
                 131.91,131.91,131.91,132.41,132.41,132.41,132.91,132.91,132.91,$
                 133.41,133.41,133.41,133.91,133.91,133.91,134.41,134.41,134.41,$
                 134.91,134.91,134.91,135.41,135.41,135.41,135.91,135.91,135.91,$
                 136.41,136.41,136.41,136.91,136.91,136.91,137.41,137.41,137.41,$
                 137.91,137.91,137.91,138.41,138.41,138.41,138.91,138.91,138.91,$
                 139.41,139.41,139.41,139.91]
detector_number = n_elements(det_angles)
if keyword_set(verbose) then $
     print, 'READ_DCSASC: Detector number   = ', detector_number
; --------------------------------------------------------------------------------------


nb_channels_reserved = ( size(wout) )[1]
nb_channels = 1000.0

Y_wk = det_angles 
X_wk = findgen(nb_channels_reserved) + 1.

ch_width =  period_at_sample/nb_channels    
nrji     =  81.805/(lambda^2)
tel      =  sqrt(5.2267*(1.e-6*dist_sd)^2*lambda^2/81.805) ;temps de vol S-D pour un neutron elastique en s
nch_el   =  (1.e6*tel - tof_minimum)/ch_width
inch_el  =  fix(nch_el)



; ** Dealing with parameters
; ** ************************
PP = fltarr(31)
PT = strarr(31)

;  Classification of the parameter  to be found in LAMP:
;  ------------------------------------------------------
;  
;0  Run duration (seconds)             =      60.0000
;1  Time in 0.1 seconds                = 
;2  Number of reserved channels        =      1024.00
;3  Counts in M1                       = 
;4  Counts in M2                       = 
;5  Counts in M3                       = 
;6  Total counts                       = 
;7  Not used                           = 
;8  Sample angle (deg.)                = 
;9  Elastic peak position (channel)    =      685.000
;10 Numor                              =      29246.0
;11 Sample temperature (K)             =      300.199
;12 Digital Voltmeter Reading          = 
;13 Repetition period (microsec.)      = 
;14 Multiplier for repetition period   = 
;15 Not used                           = 
;16 Not used                           = 
;17 Not used                           = 
;18 Channel width (microsec.)          =      11.6352
;19 Number of channels used            =      1023.00
;20 TOF delay (microsec.)              =      7135.56
;21 Wavelength (angstroms)             =      8.50000
;22 Distance CH4 - M1     (meter)      =      9.23590
;23 Distance CH4 - Sample (meter)      =      9.08610
;24 Not used                           = 
;25 Distance M1  - M2     (meter)      =      2.05345
;26 Distance M1  - M3     (meter)      =      1.28340
;27 Distance Det - Sample (meter)      =      3.97700
;28 Contents scaler 1                  =      3600.00
;29 Contents scaler 2                  =      254605.
;30 Number of angles                   =      88.0000
;  
;  

PT[0]='Run duration [s]'                   &    PP[0] = duration
PT[1]='Run duration [mn]'                  &    PP[1] = duration/60.0
PT[2]='Number reserved channels'           &    PP[2] = nb_channels_reserved
PT[3]='Ratio 1'                            &    PP[3] = ratio1
PT[4]='Ratio 2'                            &    PP[4] = ratio2
PT[5]='Ratio 3'                            &    PP[5]= ratio3
PT[6]='Total count detectors'              &    PP[6]= total_count
PT[7]='Not used'                           &    PP[7]= 0
PT[8]='Sample angle'                       &    PP[8]= 0
PT[9]='Elastic peak position'              &    PP[9]= inch_el
PT[10]='Run number'                        &    PP[10] = long( stregex(strrunn,'[0-9]+',/extract) )
PT[11]='Temperature sample    [K]'         &    PP[11]= temp_sample
PT[12]='Not used'                          &    PP[12]= 0
PT[13]='Period at sample [us]'             &    PP[13]=  period_at_sample
PT[14]='Anti-overlap ratio'                &    PP[14] = chopper_ratio
PT[15]='Temperature set point [K]'         &    PP[15]= temp_setpoint
PT[16]='Temperature control   [K]'         &    PP[16]= temp_control
PT[17]='Temperature sample    [K]'         &    PP[17]= temp_sample
PT[18]='Channel Width [us]'                &    PP[18]= ch_width
PT[19]='Number used channels'              &    PP[19] = nb_channels
PT[20]='TOF delay [us]'                    &    PP[20]= chopper4_delay
PT[21]='Incident wavelength [A]'           &    PP[21] = lambda
PT[22]='Chopper 1 - sample dist. [m]'      &    PP[22] = -sample_chopper1
PT[23]='Chopper 2 - sample dist. [m]'      &    PP[23] = -sample_chopper2
PT[24]='Chopper 4 - sample dist. [m]'      &    PP[24] = -sample_chopper4
PT[25]='Chopper 6 - sample dist. [m]'      &    PP[25] = -sample_chopper6
PT[26]='Chopper 7 - sample dist. [m]'      &    PP[26] = -sample_chopper7
PT[27]='Sample detectors dist.   [m]'      &    PP[27] = dist_sd/1000.0
PT[28]='Sum Monitor 0 (counts)'            &    PP[28]= mon1
PT[29]='Sum Monitor 1 (counts)'            &    PP[29]= mon2
PT[30]='Deetector number'                  &    PP[30]= detector_number


;  
;  PT(1)='Run number'                         &    PP(1) = FILENAME
;  PT(2)='Incident wavelength [A]'            &    PP(2) = lambda
;  PT(3)='Incident energy [meV]'              &    PP(3) = nrji
;  PT(4)='Number reserved channels'           &    PP(4) = nb_channels_reserved
;  PT(5)='Number used channels'               &    PP(5) = nb_channels
;  PT(6)='Chopper speed [rpm]'                &    PP(6) = w_chop
;  PT(7)='Anti-overlap ratio'                 &    PP(7) = chopper_ratio
;  PT(8)='Ratio 1'                            &    PP(8) = ratio1
;  PT(9)='Ratio 2'                            &    PP(9) = ratio2
;  PT(10)='Ratio 3'                           &    PP(10)= ratio3
;  PT(11)='Period at sample [us]'             &    PP(11)=  period_at_sample
;  PT(12)='Channel Width [us]'                &    PP(12)= ch_width
;  PT(13)='Elastic peak position'             &    PP(13)= inch_el
;  PT(14)='TOF delay [us]'                    &    PP(14)= chopper4_delay
;  PT(15)='Sum Monitor 0 (counts)'            &    PP(15)= mon1
;  PT(16)='Sum Monitor 1 (counts)'            &    PP(16)= mon2
;  PT(17)='Temperature set point [K]'         &    PP(17)= temp_setpoint
;  PT(18)='Temperature control   [K]'         &    PP(18)= temp_control
;  PT(19)='Temperature sample    [K]'         &    PP(19)= temp_sample
;  
;
  
STATUS=0                            ;Status is ok
;********

; Reorder on 2*theta before storing
ind = sort(det_angles)
det_angles = det_angles[ind]
wout       = wout[*,ind]

  DATP={X:      X_wk,                $
        Y:      det_angles,          $
        W_TIT:  title,               $
        X_TIT:  'Channel #',         $
        Y_TIT:  'Detector angle',    $
        P:      PP,                  $
        PAR_TXT:PT,                  $
        N:      spec_mon,            $
        TIME:   startd+' to '+stopd}

give_datp,DATP


;     **********************
 FREE_LUN,unit                    ;Free the unit number




 RETURN, wout            

 read_err:  
 FREE_LUN,unit                    ;Free the unit number
 print, !error_state.msg
 
 no_file:   FREE_LUN,unit
 RETURN, wout            

 END
