;+
; :description:
;      Routine to visualize data in a radar chart, or Radar Plot (also known as Spider Plot). See Wikipedia:
;      
;      "A radar chart is a graphical method of displaying multivariate data in the form of a two-dimensional chart of three 
;       or more quantitative variables represented on axes starting from the same point. The relative position and angle of 
;       the axes is typically uninformative.
;       The radar chart is also known as web chart, spider chart, star chart, star plot, cobweb chart, irregular polygon,
;       polar chart, or kiviat diagram."
;       
;       Reference:
;       http://en.wikipedia.org/wiki/Radar_chart
;
; :params:
;       max_radious : Maximum lenght of the "axes" or "spiderweb stems"
;       data_in     : This is an array containing the data to be plotted
;       n_data      : Number of data sets to be plotted
;       n_axis      : Number of axes, defined by the dimension of the input array
;
;
; :keywords:
;       There are no keywords parameters
;
;
; :Examples:
;       Example:
;       ----------
;       See "Main Test Program" at the end of this routine.
;       Please, remove the main program test before you use the routine in your own code.
;
; :requires:
; IDL 8.x
;
; :History:
;       Fernando Santoro - Exelis VIS, May 29 15:23 MST 2013
;       D.R. added /test keyword -ILL,              Feb 2015
;            make a function & return W (curent).
;
; :author:
;       Fernando Santoro, Exelis VIS, 2013
;
; :DISCLAIMER:
;       EXELIS VISUAL INFORMATION SOLUTIONS ("EXELIS VIS") IS PROVIDING YOU WITH SOFTWARE CODE ONCE AVAILABLE  ON ITS CODE LIBRARY SITE.
;       THIS SOFTWARE CODE IS BEING PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY AND FREE OF CHARGE.  EXELIS VIS DOES NOT PROVIDE
;       SUPPORT FOR ANY CODE PROVIDED IN THIS FILE.

;-
function spider_plot, max_radious, data_in, n_data, n_axis, test=test

  ;       max_radious : Maximum lenght of the "axes" or "spiderweb stems"
  ;       data_in     : This is an array containing the data to be plotted
  ;       n_data      : number of data sets to be plotted
  ;       n_axis      : number of axes, defined by the dimension of the input arrays

if keyword_set(test) then begin

  ;#######################################  Test Program  #############################################

  ;The following example main program is only used as an example for input data. The user can change it so that the
  ;the number data sets can be greater or smaller, including the dimensions of the spiderweb, i.e. the number of axis.

  ;Example: 4 sets of data
  n_data   =4 ;Number of data sets

  ;Data Sets (an example)
  adata_in=[ 2.2,  4.3,  1.1,  6.0,  9.5, 14.4, 22.7,  8.3]
  bdata_in=[10.2, 27.3, 33.4, 40.3, 20.3, 10.0,  7.3, 13.0]
  cdata_in=[20.2, 14.3, 10.1, 16.0, 19.5, 34.4,  2.7, 18.3]
  ddata_in=[25.2, 44.3, 15.1, 56.0, 29.5,  4.4, 12.7, 28.3]

  ;Embed the data sets into one array:
  data_in =transpose([[adata_in], [bdata_in], [cdata_in], [ddata_in]])

  ; This will be the total number of "axis" of the radar plot (i.e. number of spiderweb stems)
  n_axis=n_elements(adata_in)

  ;Maximum lenght of each axis (or stem)
  max_radious= max(data_in) + 10.

endif

radious=max_radious
    
w= window(dimension=[600,600], /widget)
plot_cart=plot([-Radious,Radious],[-Radious,Radious], /nodata, /current, TITLE='Radar Plot', $
                AXIS_STYLE=0, xrange=[-Radious, Radious], yrange=[-Radious, Radious])


bottom =0.0
top    =2*!pi
angle  =0.0

x_endaxis   = fltarr(n_axis)
y_endaxis   = fltarr(n_axis)
tick_number = strarr(n_axis)
text_ang    = fltarr(n_axis)
datax_cart  = fltarr(n_data, n_axis)
datay_cart  = fltarr(n_data, n_axis)

; color to choose from when ploting each data set. This can be increase of the number of data set is greater than 8, see:
; http://www.exelisvis.com/docs/formattingsymsandlines.html#Color
symcolors=['Blue','Green','Red','Cyan','Magenta','Yellow','Black','Orange']

;PART I: Drawing of the n_axis axes (or stems)

; Here, for each axis, we start by using POLAR coordinates, radious and angle, and convert them into carthesian coords. so I can use POLYLINE
; to draw the axes. 
for i=0, n_axis-1 do begin

  angle=top/n_axis + angle

  x= radious * cos(angle)
  y= radious * sin(angle)
  
  ;Drawing of the spiderweb stem axes:
  line     = POLYLINE([0,x], [0,y], /data, /current, THICK=1, color='green')
    
  x_endaxis[i] =[x]
  y_endaxis[i] =[y]
  tick_number[i] = string(i)
  text_ang[i]    = angle * 180/!PI

endfor

; We add the name of each axis here:
for i=0,n_axis-1 do begin
  axis_name= TEXT(x_endaxis[i], y_endaxis[i], ' $\it X_'+ STRTRIM(tick_number[i], 1) + '$', FONT_SIZE=13, $
    ORIENTATION=text_ang[i], /data, clip=0, color='black')
endfor


; Part II: Drawing of the connected ticks between each axis

;The following draw the EXTERNAL connected lines between all the axes
naxisarray=findgen(n_axis)
conn = [n_axis+1, naxisarray, naxisarray[0]]
close_axis=POLYLINE(x_endaxis, y_endaxis, CONNECTIVITY=conn, /data, /current, THICK=1, color='black')

; What follow draw internal "ticks" between the axes.
m = 4    ; number of "ticks" for each axis
j = m/2. ; position in relation with the lenght of each axis

for n=2, m, j do begin
  
    x_onaxis=x_endaxis/n
    y_onaxis=y_endaxis/n
    ticks= POLYLINE(x_onaxis, y_onaxis, CONNECTIVITY=conn, /data, /current, THICK=2, color='black', LINESTYLE='dot')
    
    ; what follows add another connected tick closer to the external connected line for the last "tick" n=m
    if (n eq m) then begin
      x_onaxis=x_endaxis - x_endaxis/n
      y_onaxis=y_endaxis - y_endaxis/n 
      ticks= POLYLINE(x_onaxis, y_onaxis, CONNECTIVITY=conn, /data, /current, THICK=2, color='black', LINESTYLE='dot')
    endif

endfor

; PART III: Plotting the Input Data Points
; 
for i=0, n_data-1 do begin
  for k=0, n_axis-1 do begin

    angle=top/n_axis + angle

    datax= data_in[i, k] * cos(angle)
    datay= data_in[i, k] * sin(angle)

    data=plot([datax], [datay], SYMBOL='Circle',  /CURRENT, /data, /overplot)
    data.SYM_FILLED=1
    data.SYM_COLOR=symcolors[i]
    
    ; these two arrays contain the data points located on each axis, plus one more point 
    ; at the end of the array which will be use to close the polygons
    datax_cart[i,k]=[datax]
    datay_cart[i,k]=[datay]

  endfor
endfor


;Drawing the Data Sets polygons:

for idata=0, n_data-1 do begin
	adatax_connlines = reform(datax_cart[idata,*])
	adatay_connlines = reform(datay_cart[idata,*])
	adatalines = POLYGON(adatax_connlines, adatay_connlines, CONNECTIVITY=conn, /data, /current, THICK=2, color=symcolors[idata])
	adatalines.FILL_COLOR=symcolors[idata]
	adatalines.FILL_TRANSPARENCY=70
endfor

return, w
end

