; This function writes a PDB file.
;
; Eric PELLEGRINI (Mar 2010)

Function PCell_To_Mat, a0, b0, c0, alpha0, beta0, gamma0
;+
;******* ************
;**
;** This function performs the conversion from the six cell parameters a, b, c, alpha, beta and gamma
;** to the direct cell matrix assuming that:
;**    - a //x
;**    - b in (x,y) plane
;** a, b, c in Angstrom
;** Alpha, beta and gamma in degree
;** The inverted cell matrix allows to convert from direct coordinates to fractional coordinates.
;-


    ; Local copy of the input parameters.
    a     = a0
    b     = b0
    c     = c0
    alpha = alpha0
    beta  = beta0
    gamma = gamma0

    ; The cell matrix is initialized.
    Mat = fltarr(3,3)
    
    ; Alpha, Beta and Gamma are converted in radian.
    alpha  = alpha * !pi/180.0
    beta   = beta  * !pi/180.0
    gamma  = gamma * !pi/180.0
    
    ; Some useful intermediates.
    cA  = cos(alpha)
    cB  = cos(beta)
    cG  = cos(gamma)
    sG  = sin(gamma)
    tG  = tan(gamma)
    phi = Sqrt(1.0 - cA^2 - cB^2 - cG^2 + 2.0*cA*cB*cG)
    
    ; The cell matrix taken from Int. Table of Crys. Vol B p.345
    Mat[0,0] = a 
    Mat[0,1] = b*cG 
    Mat[0,2] = c*cB
    Mat[1,0] = 0.0
    Mat[1,1] = b*sG
    Mat[1,2] = c*(cA - cB*cG)/sG
    Mat[2,0] = 0.0
    Mat[2,1] = 0.0
    Mat[2,2] = c*phi/sG

    Return, Mat
    
End

Pro Write_PDB, PDBFile, Num_At, W=n
;+
;** *********
;** User callable
;** This procedure writes a PDB file out of a workspace. To do so the workspace must be
;** of liv_objet type.
;** Eric PELLEGRINI (Mar 2010)
;-

    Forward_Function d_livTypetoSymb
    
    liv_objet
                    
    ; Checks that a pdb file name has been provided as first argument.
    If N_Elements(PDBFile) ne 1 Then Begin
        Lamp_Message, 'No PDB output file name provided !'
        Return
    EndIf
    
    ; Checks that a workspace has been provided as an argument or as a keyword.
    ; As a keyword first (e.g. w = 19).
    If N_Elements(n) eq 1 Then Begin    
        Take_DatP, DatP  , W=n
        Take_W   , Num_At, W=n
        
    ; Then as an argument (e.g. w19)            
    EndIf Else If Arg_Present(Num_At) Then Begin    
        Take_DatP, DatP

    ; Otherwise stops there.       
    EndIf Else Begin
        Lamp_Message, 'No input data provided !'
        Return                  
    EndElse
        
    ; Checks that the workspace is of liv_objet type.
    If (size(Num_At))[0] ne 1 Then Begin
        Lamp_Message, 'The workspace may not contain a liv_objet.'
        Return
        
    ; If so, checks that the X, Y, Z and W attributes have the same number of elements.
    EndIf Else Begin
        ; If not, stops here.
        If ((N_Elements(DatP.X) ne N_Elements(DatP.Y)) or $
            (N_Elements(DatP.X) ne N_Elements(DatP.Z)) or $
            (N_Elements(DatP.X) ne N_Elements(Num_At))) Then Begin
            
            Lamp_Message, 'The workspace may not contain a liv_objet.'
            Return
            
        EndIf
                
    EndElse
    
    

    ; Some formats definitions.
    ; The 'TITLE' format
    title_fmt  = '(A6,2X,A-60)'
    ; The 'REMARK' format.
    remark_fmt = '(A6,1X,I2,1X,A-58)'    
    ; The 'ATOM'/'HETATM' format without the last element not useful for us.
    atom_long_fmt  = '(A-6,I5,1X,A2,2X,A1,A3,1X,A1,I4,A1,3X,3F8.3,2F6.2,10X,A2)'
    ; The 'CRYST1' format.
    cryst1_fmt = '(A6,3F9.3,3F7.2,1X,A-11,I4)'        
        
    ; The PDB is opened for writing.
    On_ioerror,MISpdb & u=-1
    OpenW, u, PDBFile, /get_lun
    
    ; The title for the PDB file.
    title = strmid(strtrim(strcompress(datp.w_tit),2),0,60)
    ; Write a 'TITLE' PDB field if it is not an empty string.    
    If strlen(title) gt 0 Then Begin
        printf, u, 'TITLE', title, format = title_fmt
    EndIf Else Begin
        printf, u, 'TITLE', pdbFile, format = title_fmt    
    EndElse
    
    Printf, u, 'REMARK', 1, 'File generated with LAMP'       , format = remark_fmt
    Printf, u, 'REMARK', 2, 'Created by: '+GetEnv('USERNAME'), format = remark_fmt
    Printf, u, 'REMARK', 3, 'Date: '+systime()               , format = remark_fmt
    Printf, u, 'REMARK', 4, 'Direct coordinates'             , format = remark_fmt
    
    ; The symbol are extracted from the atomic number array.
    At_Symbol = d_livTypetoSymb([Num_At])
    
    N_Atoms = N_Elements(At_Symbol)
    
    ; The X, Y and Z coordinates are extracted from datp structure.
    X = DatP.X
    Y = DatP.Y
    Z = DatP.Z
    
    Coord = FltArr(3, N_Atoms)                
        
    ; The a, b, c, alpha, beta and gamma cell parameters.
    a     = DatP.P[2]
    b     = DatP.P[4]
    c     = DatP.P[6]
    alpha = DatP.P[7]
    beta  = DatP.P[8]
    gamma = DatP.P[9]
    
    ; The integer coding for the space group.
    spgr  = DatP.P[16]
    
    ; The space group integer is converted to its corresponding letter.
    Case spgr Of
        1: space_group = "P"
        2: space_group = "C"
        3: space_group = "A"
        4: space_group = "B"
        5: space_group = "I"
        6: space_group = "F"
        7: space_group = "R"
        9: space_group = "H"
        Else: space_group = "P"
    EndCase

    ; The 'CRYST1' line is written.
    Printf, u, 'CRYST1', a, b, c, alpha, beta, gamma, space_group, 1, format = cryst1_fmt
    
    ; The cell matrix to convert from fractional to direct coordinates.
    Mat = PCell_To_Mat(a, b, c, alpha, beta, gamma)
        
    Coord[0,*] = X
    Coord[1,*] = Y
    Coord[2,*] = Z
        
    ; The fractional coordinates are converted to direct coordinates.
    Coord = Matrix_Multiply(Mat, Coord)
                             
    ; This structure stores the contents of one 'ATOM' PDB line.   
    struct = {atom        : "ATOM",$
              serial_num  : 0     ,$
              at_name     : ""    ,$
              alt_loc     : ""    ,$
              res_name    : "UNK" ,$
              ch_name     : "A"   ,$
              res_seq_num : 1     ,$
              res_ins     : ""    ,$
              x_coord     : 0.0   ,$
              y_coord     : 0.0   ,$
              z_coord     : 0.0   ,$
              occupancy   : 0.0   ,$
              temp_fact   : 0.0   ,$
              at_symbol   : ""}

    ; It is replicated to the number of atoms to write out.
    atoms = Replicate(struct, N_Atoms)

    ; The non_constant fields are defined:
    ; The serial number.    
    atoms[*].serial_num = Lindgen(N_Atoms) + 1L
    ; The atom name.    
    atoms[*].at_name = At_Symbol
    ; The x, y and z atomic coordinates.
    atoms[*].x_coord = Reform(Coord[0,*])
    atoms[*].y_coord = Reform(Coord[1,*])
    atoms[*].z_coord = Reform(Coord[2,*])
    ; The atomic symbols.
    atoms[*].at_symbol = At_Symbol
    
    ; The replicated structure is written in the output file.
    Printf, u, atoms, format = atom_long_fmt
      
    ; The final PDB keyword is written.
    Printf, u, 'END'
    
    ; Close the unit associated to the PDB file.
    MISpdb: if u gt 0 then Free_Lun, u else Lamp_Message,!error_state.msg
        
End
