/******************************************************************************
 *                                IDL2XREF Project
 *-----------------------------------------------------------------------------
 *   ILL (Institut Laue Langevin)
 *
 *   38000 GRENOBLE Cedex
 *-----------------------------------------------------------------------------
 *   Module              :       Base de connaissance
 *   Auteurs             :       Yannick Raoul
 *   Date creation       :       10 / 05 / 2013
 *
 *****************************************************************************/


#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <libgen.h>
#include <ctype.h>
#include <unistd.h>
#include <time.h>   /* pour le calcul de temps de traduction */

#include <getopt.h>

#include "hashtable.h"
#include "table_symb.h"
#include "type.h"
#include "tree.h"
#include "exploreForDoc.h"

#ifdef BUILD_WITH_XCODE
#include "idl-parser.tab.h" /* on inclut les constantes definies dans le resultat de yacc */
#else
#include "idl-parser.h" /* pareil, mais avec autoconf/automake */
#endif

extern int yyparse(void);
extern int yydebug;
extern FILE *yyin;
extern size_t larger_string_size;
extern char * lexError;
extern void my_flush_buffer();// defined in idl-lexer.l

/* variables externes */

/* The name of this program */
const char* program_name;


char output_filename[FILENAME_MAX];

char sourceFileName[FILENAME_MAX];       /*+ chemin et nom du fichier source +*/
char baseFileName[FILENAME_MAX];         /*+ nom du fichier cible +*/
char sourceDirName[PATH_MAX];        /*+ repertoire source +*/
char* stringTranslationResult;  /*+ chaine contenant la traduction +*/



/*******************************/
/* fonctions locales au module */
/*******************************/



/* ouvre ou creer un fichier en ecriture et ecrit la chaine de caractere dedans */
int openFile(char *pfileName, char *pstrLine)
{
    /* la fonction renvoie 0 si tout se passe bien et -1 sinon */
    FILE *file;
    file = fopen(pfileName, "a+"); /* open or create a file for appending */
    
    if(file==NULL) {
        return -1;
    }
    else {
        fputs(pstrLine, file);
    }
    fclose(file);
    return 0;
}

/* affiche un message d'erreur en cas de mauvaise utilisation */
static void usage () {
    printf ("Usage : idl2xref [-oqwhv] <.pro idl files...> \n") ;
    printf ("  idl2xref %c version %s\n", PATHSEP, I2X_VERSION_2);
    printf ("Action: aid to document IDL source code\n");
    printf ("Options :\n") ;
    printf ("-o target_file  give the name of destination file for xref\n") ;
    printf ("-q be quiet, no output on console\n") ;
    printf ("-w print some warning\n") ;
    printf ("-t print abstract tree\n") ;
    printf ("-v print version number\n") ;
    printf ("\n");
    printf ("Examples :\n") ;
    printf ("  idl2xref -o outFile.txt test1.pro test2.pro ...\n") ;
    exit (0) ;
}



/* met la chaine en minuscule */
void strToLower(char* str) {
    int i;
    
    i = 0;
    while (str[i] != '\0') {
        str[i] = tolower(str[i]);
        i++;
    }
}

/* Cherche un caractere dans une chaine */
/* renvoie 1 si s contient c / 0 sinon */
int containsChar(char* s, char c) {
    int result;
    
    result= 0;
    while ((*s != '\0') && (result == 0)) {
        /* on parcours la chaine s */
        if (*s == c) {  /* on trouve le caratere c dans s */
            result = 1;
        }
        s++;
    }
    return result;
}


void version() {
    fprintf(stderr,"#####################################\n");
    fprintf(stderr,"## idl2xref %c version %s ##\n", PATHSEP, I2X_VERSION_2);
    fprintf(stderr,"#####################################\n");
}





/***********************/
/* fonctions exportees */
/***********************/

/* affiche un message d'erreur de syntaxe */
void yyerror(char *s) {
    /* cette fonction est appelle par yacc en cas d'erreur de syntaxe */
    
    if (displayMessage)
        fprintf(stderr, "%s at %s:%d\n", s, sourceFileName, yylloc.first_line);
    translationError = 1;
}


/* A string listing the valid short options letters */
const char *short_options = "o:pqwythv";

/* An array listing valid long options */
static const struct option long_options[] = {
    { "output",     required_argument,  NULL,       'o' },
    { "progress",   no_argument,        NULL,       'p' },
    { "quiet",      no_argument,        NULL,       'q' },
    { "warning",    no_argument,        NULL,       'w' },
    { "yydebug",    no_argument,        NULL,       'y' },
    { "ast",        no_argument,        NULL,       't' },
    { "usage",      no_argument,        NULL,       'h' },
    { "version",    no_argument,        NULL,       'v' },
    { NULL,         0,                  NULL,       0 }
    /* End of array need by getopt_long do not delete it */
};


/* programme principal */
int main (int argc, char *argv[]) {
    time_t start_time;	/* variable de temps */
    time_t end_time;
    char* c_time_string;
    char tmpFileName[FILENAME_MAX];
    
    time(&start_time);	/* on prend l'heure courante */
    
    /* initialisation des parametres au valeurs par defauts */
    output_filename[0]='\0';

    displayProgress = 0;
    displayMessage = 1;    	            	/* affichage des messages si = 1 sinon 0 */
    writeWarning = 0 ;     	            	/* ecrit les warning dans le fichier log si = 1 */
    writeAbstractTree = 0;	            	/* affiche l'arbre abstrait si = 1 */
    tabVal = 2; 	      	      	      	/* indentation par defaut */
    commentaire="%";
    yydebug = 0;
    larger_string_size = 0;
    
    /* Remember the program name to incorporate it in messages. */
	program_name = argv[0];
    
    
    int ch;
    while ((ch = getopt_long(argc, argv, short_options, long_options, NULL)) != -1)
        switch (ch) {
            case 'o':
                strcpy(output_filename,optarg);
                break;
            case 'p':
                displayProgress = 1;
                break;
            case 'q':
                displayMessage = 0;    	      	/* affichage des messages si = 1 sinon 0 */
                break;
            case 'w':
                writeWarning = 1;
                break;
            case 't':
                writeAbstractTree = 1;
                break;
            case 'y':
                yydebug = 5;
                break;
            case 'h':
                usage();
                return(0);
                break;
            case 'v':
                version();
                return(0);
                break;
            default:
                usage();
                return(0);
        }
    
    /* Handle any remaining command line arguments, should be lot of .pro file */
    int file_index = optind;
    int files_in_error = 0;// count "not parsed" files
    
    if (file_index == argc)
    {
        fprintf (stderr, "Need some idl \".pro\" file on command line\n");
        return(0);
    } else {
        
        // First pre flight test on output_filename
        // No need to parse xxx files if we can't write the report
        if (output_filename[0]!='\0') {
            
            FILE* open_write_test = fopen(output_filename, "w");
            
            if (open_write_test!=NULL) {
                fclose(open_write_test);
                // OK, we can continue
            } else {
                fprintf (stderr, "Unable to open and write to %s. Aborting\n", output_filename);
                return(1);
            }
        }
        
        
        if (displayMessage) { /* on affiche l'heure de debut */
            c_time_string = ctime(&start_time);
            if (c_time_string) fprintf (stderr,"\nStarted at : %s\n", c_time_string);
        }
        
        yyin = NULL;
        startIDLXRef(argc-file_index);
        while (file_index < argc) {

            /* Extract from man 3 dirname on linux :
             *	Both  dirname()  and  basename()  may modify the contents of path,
             *	so it may be desirable to pass a copy  when calling one of these functions.
             */
            
            strcpy(sourceFileName,argv[file_index]);
            strcpy(sourceDirName,dirname(strcpy(tmpFileName, sourceFileName)));
            strcpy(baseFileName,basename(strcpy(tmpFileName, sourceFileName)));       
            
            if (displayMessage && displayProgress) {
                fprintf (stderr,"\n[%i/%i] Work on : %s\n", file_index-optind+1, argc-optind, sourceFileName);
            }
            /* initialisation du detecteur d'erreur a 0 */
            translationError = 0;
            yylloc.first_line=1 ; 
            if (lexError!=NULL) {
                free(lexError);
                lexError = NULL;
            }
            
           
            /* On donne le source a Yacc */
            
            yyin = fopen (sourceFileName, "r") ; /* ouverture du fichier passe en argument */
            if (yyin == NULL) {
                fprintf (stderr, "Cannot open %s\n", sourceFileName) ;
                files_in_error++;
                file_index++;
                continue;// next file
            }
            
            
            /*************  on appelle le parseur de Yacc ***********/
            int parseResult = yyparse();
            if (parseResult == 0) {
                if (displayMessage && displayProgress) {  /* syntaxe OK */
                    fprintf (stderr,"Syntax analysis OK ...\n") ;
                }
            }
            fclose (yyin) ;
            my_flush_buffer();
            yyin = NULL;
            
            /*************  Affichage de l'arbre abstrait ***********/
            if (writeAbstractTree==1) {
                printf("Arbre abstrait : \n");
                printTree(root);
                printf("Fin Arbre abstrait\n");
            }
            /********************************************************/

            /* Traitement si l'analyse lexicale et syntaxique est reussie */
            if (parseResult==0) {
                
                /*************  s'occupe de la table des symb ***********/
                const int hashTableSize = 256;
                hashTable = (nodeType **) calloc(hashTableSize, sizeof(nodeType *));
                genererTableSymb();
                /********************************************************/
                
                
                /********************* Extraction d'information ******************/
                collectIDL_Def_and_Ref(baseFileName, sourceDirName );
                
            } else {
                /* sinon affichage de messages */
                if (displayMessage) {
                    
//                    fprintf (stderr,"SYNTAX ERROR, %d:%s\n", yylloc.first_line, sourceFileName) ;
//                    if (lexError!=NULL) {
//                        fprintf (stderr,"Message %s\n", lexError) ;
//                    }
                    
                }
                
                free(lexError);
                lexError = NULL;
                
                files_in_error++;
            }
            
                    
           
            file_index++;
        }// loop on .pro file

    }// args are ok
    
    
    if (file_index > optind) { // Means some files were processed
    
        outputFormattedXref(output_filename);
        // Note: if output_filename is an empty string, output goes to the console

        //  if (displayMessage) {
        //    fprintf (stderr,"%d files generated - %d lines - %d Warnings\n", nbLinesTotal, nbWarning) ;
        //  }
        
        time(&end_time); /* on prend le temps a la fin */
        if (displayMessage) { /* on affiche l'heure d'arret */
            c_time_string = ctime(&end_time);
            if (c_time_string) fprintf (stderr,"Finished at : %s\n", c_time_string);
            fprintf (stderr,"Larger string size : %lu\n", larger_string_size);
            if (files_in_error>0)
                fprintf (stderr,"Unable to parse %d files on %d.\n", files_in_error, argc-optind);

        }
    }
    
    if (files_in_error==0) {
        return 0;
    } else {
        return 1;
    }
}
