
;+
; NAME:
;	build_lamp_loops
;
; PURPOSE:
;	Return the name of the sharable library containing the external
;	C code used by LAMP CALL_EXTERNAL.
;	If necessary, builds the library.
;
; CATEGORY:
;	Dynamic Linking
;
; CALLING SEQUENCE
;	LibName = build_lamp_loops
;
; INPUTS:
;	None
;
; OUTPUTS:
;	Scalar string containing path to IDL CALL_EXTERNAL lamp_loops.so
;
; KEYWORDS:
;	VERBOSE
;	If set, cause the underlying MAKE_DLL to show the commands it
;	executes to build the sharable library, and all of the output
;	produced by those commands. If not set, this routine does its
;	work silently.
;
; SIDE EFFECTS
;	On the first call in an IDL session, the sharable library is
;	build using the MAKE_DLL procedure.
;
; RESTRICTIONS
;	For MAKE_DLL to be successful, a C compiler compatible with the
;	one described in the !MAKE_DLL system variable must be installed
;	on the system.
;
; PROCEDURE:
;	Caches the library path in a private COMMON block. On the first
;	call, builds the library and sets its name in the COMMON block for
;	following calls.
;
; Modification History:
;	YR, 16 Mai 2013
;-




function build_lamp_loops, VERBOSE=verbose
  common GET_CALLEXT_LOOPLIB_BLK, shlib

  ; Build sharable library if this is the first call or lib doesn't exist
  build_lib = n_elements(shlib) eq 0
  if (not build_lib) then build_lib = not FILE_TEST(shlib)
  if (build_lib) then begin
    ; Location of the CALL_EXTERNAL files from LAMP
    
    
    ;;;;;;;;;;;;;;;; DEV ONLY ; TO BE FIXED
    lamp_loop_dir = FILEPATH('',ROOT_DIR=sys_dep('GETENV','LAMP_DIR'), SUBDIRECTORY=['plug_in', 'LAMP-Loops', 'src'])
    ;;;;;;;;;;;;;;;;

    ; Use MAKE_DLL to build the widget_call_ex sharable library in the
    ; !MAKE_DLL.COMPILE_DIRECTORY directory.
    ;
    ; Normally, you wouldn't use VERBOSE, or SHOW_ALL_OUTPUT once your
    ; work is debugged, but as a learning exercize it can be useful to
    ; see all the underlying work that gets done. If the user specified
    ; VERBOSE, then use those keywords to show what MAKE_DLL is doing.
    source = [ 'LL_GetVersion', 'D33_panels_loop', 'D33_tof1_loop', 'D33_tof2_loop', 'D19_XYcor', 'HKL_vol', 'HKL_calc', 'IN5_dist', 'STR_proj', 'STR_textu']
    export_rtns = [ 'LL_GetVersion', $
                    'D33_panels_loop', $
                    'D33_tof1', $
                    'D33_tof2', $
                    'HKL_vol', $
                    'D19_Xcor', $
                    'D19_Ycor', $
                    'HKL_vol', $
                    'HKL_calc', $
                    'IN5_dist', $
                    'STR_proj', $
                    'STR_textu' $
                  ]
                   
;Note from IDL documentation: It is possible to use C compilers other than the one assumed in !MAKE_DLL to build sharable libraries. To do so, you can alter the contents of !MAKE_DLL or use the CC and/or LD keyword to MAKE_DLL and CALL_EXTERNAL. Please understand that a list of all possible compilers and the necessary compiler options is not maintained. This information is available in your compiler and system documentation. It is the programmer’s responsibility to understand the rules for the chosen compiler.

;SPAWN, 'C:\Program Files (x86)\Microsoft Visual Studio 10.0\Common7\Tools\vsvars32.bat'

; We need the include directory for the IDL export.h header
; file. One way to get this is to extract it from the
; !MAKE_DLL system variable using the STREGEX function
;    INCLUDE=STREGEX(!MAKE_DLL.CC, '[-/]I[^ ]+', /EXTRACT)
;hum...  

;INCLUDE = '"C:\Program Files\ITT\IDL\IDL81\external\include"'


  os =!VERSION.OS

  CASE os of
    'Win32': lamp_loops_cflags = '/TP /O2'    ; with windows sdk compiler
    ELSE: lamp_loops_cflags = '-std=c99 -O2'  ; with gcc
  ENDCASE

;YR    lamp_loops_cflags = '-g -std=c99'

    MAKE_DLL, source, 'lamp_loops', export_rtns, $
      INPUT_DIR=lamp_loop_dir, $
      DLL_PATH=shlib, $
      EXTRA_CFLAGS = lamp_loops_cflags, $ ; system dependent
;YR       /NOCLEANUP, $ ; debug feature (acces to object files...), to be removed
      VERBOSE=verbose, SHOW_ALL_OUTPUT=verbose
	  
  endif

   return, shlib
end
